<?php

/**
 * Some HelperClasses used be the generate.php script
 *
 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @TODO insert directly?
 */

require_once 'Smarty.class.php';

use ch\monitoring\Application;

/**
 * Returns the path to the directory in which this file is located
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 */
function basePath($file = '') {
	return dirname(__FILE__) . '/' .$file;
}




/**
 * Get the table data
 *
 * First, parse the json file containing the base data
 * second, get some information from the database. The information is
 * * field names
 * * constraints that cascade
 * * constraints that set NULL
 *
 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
 */
class TableData {

	public $tableData;


	/**
	 * Parse the json
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @SuppressWarnings(unused)
	 */
	public function __construct() {
		$this->position = 0;
		
		$yamlData = file_get_contents(basePath('data.yaml'));
		$this->tableData = yaml_parse($yamlData);

		$this->db = Application::getDbInstance();

		foreach ($this->tableData as $entity => $value) {
			$this->getInformation($entity);
		}
	}

	/**
	 * Retrieve information from the database
	 *
	 * @param         string $enty The name of the table
	 */
	protected function getInformation($entity) {
		$this->getFields($entity);
		$this->getConstraints($entity);
	}

	/**
	 * Retrieve the fields of a table from the database
	 *
	 * Ignore the id field, since it makes no sense to record that
	 * (it won't change, and the ID is recorded in the auditLog table
	 * anyway)
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $entity The name of the table
	 */
	protected function getFields($entity) {
		$query = 'DESCRIBE ' . $entity;

		$stmt = $this->db->query($query);
		$fields = $stmt->fetchAll();

		$this->tableData[$entity]['fields'] = array();
		$this->tableData[$entity]['hasID'] = false;

		foreach ($fields as $field) {
			if ($field['Field'] == 'id') {
				$this->tableData[$entity]['hasID'] = true;
				continue;
			}
			$this->tableData[$entity]['fields'][] = $field['Field'];
		}
	}

	/**
	 * Parse the constraints
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $entity The name of the table
	 */
	protected function getConstraints($entity) {
		$query = '	SELECT
						TABLE_NAME,
						DELETE_RULE,
						CONSTRAINT_NAME
					FROM
						information_schema.REFERENTIAL_CONSTRAINTS
					WHERE
						REFERENCED_TABLE_NAME = "' . $entity. '"
				';

		$stmt = $this->db->query($query);
		$references = $stmt->fetchAll();

		foreach ($references as $ref) {
			if (array_key_exists($ref['TABLE_NAME'], $this->tableData)) {
				// we only need to consider these references if they are to be
				// logged in the audit log table

				if ($ref['DELETE_RULE'] == 'CASCADE') {
					$this->tableData[$entity]['referenceDelete'][] = array(
						'table'             => $ref['TABLE_NAME'],
						'referencingColumn' => $this->getReferencingColumn(
							$ref['CONSTRAINT_NAME']
						)
					);
				}
				else if ($ref['DELETE_RULE'] == 'SET NULL') {
					$this->tableData[$entity]['referenceUpdate'][] = array(
						'table'             => $ref['TABLE_NAME'],
						'referencingColumn' => $this->getReferencingColumn(
							$ref['CONSTRAINT_NAME']
						),
						'nameField'      => (
							isset($this->tableData[$ref['TABLE_NAME']]['namingColumn'])
							)
							? $this->tableData[$ref['TABLE_NAME']]['namingColumn']
							: NULL
					);
				}
			}
		}
	}

	/**
	 * Get the column name of the constraint, that references the target table
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $constraintName The name of the constraint to
	 *                                       retrieve information from
	 */
	protected function getReferencingColumn($constraintName) {
		$query = '	SELECT
						COLUMN_NAME
					FROM
						information_schema.KEY_COLUMN_USAGE
					WHERE
						CONSTRAINT_NAME = "' . $constraintName .'"';

		$stmt = $this->db->query($query);
		$col = $stmt->fetchColumn(0);

		return $col;
	}
}


/**
 * Generate the triggers for a given table
 */
class TriggerGenerator {

	public function __construct($tableName, $table) {
		$this->tpl = new Smarty();
		$this->tpl->setCompileDir('/tmp');

		$this->tableName = $tableName;
		$this->table     = $table;
	}

	/**
	 * Return the parsed triggers
	 */
	public function get() {
		$result  = $this->updateBeforeTrigger();
		$result .= $this->updateAfterTrigger();

		$result .= $this->deleteBeforeTrigger();
		$result .= $this->deleteAfterTrigger();

		$result .= $this->insertAfterTrigger();
		$result .= $this->insertBeforeTrigger();


		return $result;
	}

	public function getDrops() {
		$drops = "";
		$this->tpl->clearAllAssign();
		$this->tpl->assign('entity', $this->tableName);

		foreach (array('insert', 'update', 'delete') as $operation) {
			foreach (array('before', 'after') as $time) {
				$this->tpl->assign('operation', $operation);
				$this->tpl->assign('time', $time);
				$drops .= $this->tpl->fetch(basePath('drop.tpl'));
			}
		}

		return $drops;
	}

	private function assignEssentials($namingSource = 'old') {
		$this->tpl->assign('entity', $this->tableName);
		$this->tpl->assign('fields', $this->table['fields']);
		$this->tpl->assign('no_id', !$this->table['hasID']);

		if (isset($this->table['namingColumn'])) {
			$this->tpl->assign('nameField', sprintf('%s.%s', $namingSource, $this->table['namingColumn']));
		}
		else {
			$this->tpl->assign('nameField', "''");
		}
	}

	/**
	 * Generate update triggers
	 */
	private function updateBeforeTrigger() {
		$this->tpl->clearAllAssign();
		$this->assignEssentials('old');

		if (isset($this->table['triggers']['update']['before'])) {

			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['update']['before']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['update']['before']['body']);
		}
		
		return $this->tpl->fetch(basePath('updateBefore.tpl'));
	}

	/**
	 *
	 */
	private function updateAfterTrigger() {
		$this->tpl->clearAllAssign();

		if (isset($this->table['triggers']['update']['after'])) {

			$this->tpl->assign('entity', $this->tableName);
			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['update']['after']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['update']['after']['body']);

			return $this->tpl->fetch(basePath('updateAfter.tpl'));
		}
	}

	/**
	 * Generate deletion after triggers
	 *
	 * The deletion trigger are somewhat special in regard to
	 * cascading foreign keys.
	 *
	 * The Problem with cascading foreign keys is, the triggers are not
	 * executed when cascading.
	 * Therefore the related databases have to be included in the data
	 * structure. The referencing entries are then deleted explicitly in the
	 * trigger, which triggers their triggers.
	 */
	private function deleteBeforeTrigger() {
		$this->tpl->clearAllAssign();
		$this->assignEssentials('old');


		if (isset($this->table['referenceDelete'])) {
			$this->tpl->assign('referenceDelete', $this->table['referenceDelete']);
		}

		if (isset($this->table['referenceUpdate'])) {
			$this->tpl->assign('referenceUpdate', $this->table['referenceUpdate']);
		}

		if (isset($this->table['triggers']['delete']['before'])) {

			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['delete']['before']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['delete']['before']['body']);
		}

		return $this->tpl->fetch(basePath('deleteBefore.tpl'));
	}

	/**
	 *
	 */
	private function deleteAfterTrigger() {
		$this->tpl->clearAllAssign();

		if (isset($this->table['triggers']['delete']['after'])) {

			$this->tpl->assign('entity', $this->tableName);
			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['delete']['after']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['delete']['after']['body']);

			return $this->tpl->fetch(basePath('deleteAfter.tpl'));
		}
	}

	/**
	 * Generate insert triggers
	 */
	private function insertAfterTrigger() {
		$this->tpl->clearAllAssign();
		$this->assignEssentials('new');

		if (isset($this->table['triggers']['insert']['after'])) {

			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['insert']['after']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['insert']['after']['body']);
		}

		return $this->tpl->fetch(basePath('insertAfter.tpl'));
	}

	/**
	 * Genereate the insert before triggers if necessary
	 */
	private function insertBeforeTrigger() {
		$this->tpl->clearAllAssign();

		if (isset($this->table['triggers']['insert']['before'])) {

			$this->tpl->assign('entity', $this->tableName);
			$this->tpl->assign('additionalTriggerDecl', $this->table['triggers']['insert']['after']['declarations']);
			$this->tpl->assign('additionalTriggerBody', $this->table['triggers']['insert']['before']['body']);

			return $this->tpl->fetch(basePath('insertAfter.tpl'));
		}
	}
}

