<?php

/**
 * Cronjob helper for creating maintenance tickets
 * for groups which are due.
 *
 * @category  None
 * @package   None
 * @author    Adfinis SyGroup AG <info@adfinis-sygroup.ch>
 * @copyright 2014-2015 Adfinis SyGroup AG
 * @license   LGPLv3 http://www.gnu.org/licenses/lgpl.html
 * @link      None
 *
 * @SuppressWarnings(PHPMD.ExcessiveClassLength)
 * @SuppressWarnings(PHPMD.TooManyMethods)
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 **/

use ch\monitoring\Application;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use ch\monitoring\util;
use org\adfinis\framework\utils;
use org\adfinis\framework\config;

use org\adfinis\framework\datamapper\IdentityMap;
$idmContext = IdentityMap::getContext();

/**
 * Notifies maintenance masters by e-mail.
 *
 * @return None
 */
function notifyMastersByMail($subject, $text)
{
	$masters = datamapper\UserMapper::getInstance()->getAllByPermission(
		model\Permission::MAINTENANCE_MASTER,
		null,
		true
	);

	$mastersArray = array();
	foreach ($masters as $master) {
		$mastersArray[] = array(
			'firstName' => '',
			'lastName'  => $master->fullName,
			'address'   => $master->email
		);
	}

	if (count($mastersArray) == 0) {
		error_log("Got 0 maintenance master to inform, skipping..");
		return;
	}
	else {
		error_log(sprintf("Got %d maintenance master to inform: %s", count($mastersArray), var_export($mastersArray, true)));
	}

	utils\MailHelper::sendMail(
		$mastersArray,
		$subject,
		$text,
		'',
		true,
		75,
		array(),
		array(),
		config\ConfManager::getVar('af', 'mail', 'fromSystem', 'fromMaintenanceMail'),
		config\ConfManager::getVar('af', 'mail', 'fromSystem', 'fromMaintenanceName')
	);
}

/**
 * Converts seconts to human-readable format (days, hours, minutes and seconds)
 * http://stackoverflow.com/questions/8273804/convert-seconds-into-days-hours-minutes-and-seconds
 *
 * @param         int      $seconds
 *
 * @return        string   human-readable formatted seconds (days, hours,
 *                         minutes and seconds)
 */
function secondsToHumanReadable($seconds)
{
    $dateTimeFormat = new DateTime("@0");
    $dateTimeTime   = new DateTime("@$seconds");

    return $dateTimeFormat->diff($dateTimeTime)->format(
        "%a days, %h hours, %i minutes and %s seconds"
    );
}

/**
 * Creates maintenance tickets for groups which are due
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 *
 * @return        void
 */
function createTickets()
{
	$today   = new \Zend_Date();
	$tickets = array();
	$groups  = datamapper\MaintenanceGroupMapper::getInstance()->getAll();

	foreach ($groups as $group) {
		// If the group is not configured for maintenance
		// we don't open a ticket for this group
		if ($group->maintenanceCycleId === null) {
			error_log(sprintf("Maintenance group id=%d has no cycle defined, skipping..", $group->id));
			continue;
		}

		try {
			$nextMaintenanceDate = $group->getNextMaintenanceDate();
		}
		catch (Exception $e) {
			// No next maintenance date means no ticket for this group
			error_log(sprintf("Maintenance group id=%d has no next maintenance date, skipping..", $group->id));
			continue;
		}

		$difference = clone $nextMaintenanceDate;
		$difference->sub(new Zend_Date());

		$measure = new Zend_Measure_Time(
			$difference->toValue(),
			Zend_Measure_Time::SECOND
		);
		$seconds = $measure->getValue();

		// The next maintenance date is in the future
		if ($seconds > 0) {
			error_log(sprintf(
				"Maintenance date (%s) for group id=%d is in future, skipping..",
				secondsToHumanReadable($seconds),
				$group->id
			));
			continue;
		}

		Application::getDbInstance()->beginTransaction();

		$ticket = new model\MaintenanceTicket();
		$ticket->allowRead();

		$ticket->groupId     = $group->id;
		$ticket->customerId  = $group->getCustomer()->id;
		$ticket->userId      = null;
		$ticket->dateDue     = null;
		$ticket->dateCreated = $today;

		$ticket->getMapper()->save($ticket); // prevent policy check
		$tickets[] = $ticket;

		foreach ($group->getMembers() as $member) {
			$host = $member->getHost();

			$ticketHost = new model\MaintenanceTicketHost();
			$ticketHost->allowRead();

			$ticketHost->ticketId = $ticket->id;
			$ticketHost->hostId   = $host->id;
			$ticketHost->stateId  = model\MaintenanceState::STATE_NEW;
			$ticketHost->getMapper()->save($ticketHost); //prevent policy check

			// Set initial comment about creation
			$masters = datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_MASTER,
				null,
				true
			);
			$masters->next();
			$comment = new model\MaintenanceComment();
			$comment->allowRead();
			$comment->timestamp    = util\TimeUtility::dateOrNull(new \Zend_Date());
			$comment->comment      = "Ticket automatically created by cronjob.";
			$comment->ticketHostId = $ticketHost->id;
			$comment->userId       = null;

			$mapping = array(
				'id'           => $comment->id,
				'timestamp'    => $comment->timestamp,
				'comment'      => $comment->comment,
				'ticketHostId' => $comment->ticketHostId,
				'userId'       => $comment->userId
			);

			Application::getDbInstance()->insert(
				'MaintenanceComment',
				$mapping
			);
		}

		Application::getDbInstance()->commit();
	}

	if (count($tickets) > 0) {
		error_log(sprintf("Created %d maintenance tickets for group id=%d", count($tickets), $group->id));

		$subject = 'New maintenance tickets';
		$text    = "New maintenance tickets are available:\n\n";

		foreach ($tickets as $ticket) {
			$text .= sprintf(
				"#% 4s: %s\n",
				$ticket->id,
				$ticket->getGroup()->name
			);
			$text .= "Hosts:\n";
			foreach ($ticket->getAllRelatedTicketHosts() as $ticketHost) {
				$host = $ticketHost->getHost();
				$text .= sprintf(
					"       %s [%s]\n",
					$host->hostname,
					$host->getCustomer()->name
				);
			}
			$text .= "\n";
		}

		error_log("Notifying maintenance masters by mail..");
		notifyMastersByMail($subject, $text);
	}
	else {
		error_log(sprintf("No maintenance tickets for group id=%d, skipping..", $group->id));
	}
}

createTickets();
