<?php

use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\utils;
use org\adfinis\framework\config\ConfManager;

use org\adfinis\framework\datamapper\IdentityMap;
$idmContext = IdentityMap::getContext();

/**
 * Returns the users which need to be notified
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @returns       array
 */
function getUsers() {
	return array(
		array(
			'name' => 'Deniss',
			'method' => model\User::AUTH_METHOD_LDAP
		),
		array(
			'name' => 'Daniel',
			'method' => model\User::AUTH_METHOD_LDAP
		)
	);
}

/**
 * Returns all notifications which are older than given minutes
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @param         int $age Minimum age in minutes
 * @returns       DBIterator
 */
function getNotifications($age) {
	$dateTo = new \Zend_Date();
	$dateTo->sub($age, \Zend_Date::MINUTE);

	return datamapper\NagiosLogMapper::getInstance()->getAll(
		null,
		$dateTo,
		null,
		null,
		null,
		null,
		null,
		array(
			model\NagiosLog::STATE_OPEN
		)
	);
}

/**
 * Are there any notifications older than a certain amount of minutes?
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @param         int $age Minimum age in minutes
 * @returns       bool
 */
function hasOldNotifications($age = 5) {
	$notifications = getNotifications($age);
	return count($notifications) > 0;
}

/**
 * Send an email and an sms to the users which are to be notified
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @returns       void
 */
function alarmUsers() {
	$topic = "SyMonitoring: Alerting is BROKEN";
	$text  = "The nagios notification log was not processed twice for more than 5 minutes. The ALERTING most probably is BROKEN, please fix asap and watch for incidents in the Icinga web interface.";

	$users = getUsers();
	foreach ($users as $user) {
		$user = datamapper\UserMapper::getInstance()->getByName(
			$user['name']
		);
		$user->allowRead();

		if ($user->email !== null) {
			utils\MailHelper::sendMail(
				array(
					array(
						'address'   => $user->email,
						'firstName' => '',
						'lastName'  => ''
					)
				),
				tr($topic),
				$text,
				'',
				true
			);
		}

		if ($user->phone !== null) {
			$sms = new util\phpASPSMS();
			$sms->Recipients  = $user->phone;
			$sms->MessageText = $text;
			$sms->SendTextSMS();
		}
	}
}

/**
 * Do we have to alarm the users?
 *
 * If this was the second smngalert failure within the last 15 minutes alarm the defined users
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @returns       bool
 */
function notificationNeeded() {
	try {
		$timestamp   = util\Setting::get('lastSmngalertRestart');
		$lastRestart = new \Zend_Date($timestamp, \Zend_Date::TIMESTAMP);
		$now         = new \Zend_Date();
		$difference  = $now->sub($lastRestart)->get(\Zend_Date::TIMESTAMP);
		return $difference < 900;
	}
	catch (\OutOfBoundsException $e) {
		return false;
	}
}

/**
 * Restarts smngalert
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @returns       void
 */
function restartSmngalert() {
	$smngalertCommand = sprintf(
		"/bin/bash smngalert/restart.sh 2>&1 > /dev/null",
		ConfManager::getVar('af', 'webRoot', 'uri')
	);

	exec($smngalertCommand);

	$now = new \Zend_Date();
	util\Setting::set('lastSmngalertRestart', $now->get(\Zend_Date::TIMESTAMP));
}

// Only do anything if there are old notifications
if (hasOldNotifications()) {
	// This needs to be done like this because restartSmngAlert() sets a new timestamp
	if (notificationNeeded()) {
		restartSmngAlert();
		alarmUsers();
	}
	else {
		restartSmngAlert();
	}
}
