<?php

use org\adfinis\framework as af;
use ch\monitoring\Application;
use org\adfinis\framework\datamapper\IdentityMap;
$idmContext = IdentityMap::getContext();

if (!$argv[1]) {
	die("Specify xml file location!\n");
}

if (!file_exists($argv[1])) {
	die("{$argv[1]} not found!\n");
}

$hosts = array();
$templates = array();
parseTemplates($argv[1], 'collectTemplates');
processHosts();

/**
 * Parses the zabbix export and executes the callback
 * function for each event
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @throws        XMLParserException
 * @param         string   $logfile
 * @param         callable $fun
 * @returns       void
 */
function parseTemplates($log, $fun) {
	$parser  = af\xml\Parser::create();
	$template = null;

	$parser->caseFolding = false;

	$parser->on('start element', function($self, $name, $attr) use (&$template) {
		if ($self->level === 1 && $name == "template") {
			$template = new StdClass;
			$template->name = $attr['name'];
			$template->groups = array();
		}
	});

	$parser->on('end element', function($self, $name) use (&$template) {
		if ($self->level === 1 && $name == "template") {
			$self->emit('element done', $template);
			$template = null;
		}
	});

	$parser->on('character data', function($self, $data) use (&$template) {
		if ($self->level === 3 && $self->currentElement == "group") {
			$template->groups[] = $data;
		}
	});

	$parser->on('element done', function($self, $template) use (&$fun) {
		call_user_func($fun, $template);
	});

	$buffsize = 8192;

	$parser->parseFile($log, $buffsize);
}

/**
 * Adds a template to the templates list
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @param         StdClass $event
 * @returns       void
 */
function collectTemplates($template) {
	global $templates;
	$templates[] = $template;
}

/**
 * Process the hosts
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @returns       void
 */
function processHosts() {
	global $templates;

	$groupStatement = Application::getDbInstance()->prepare("
		SELECT
			`id`
		FROM
			`smng_services_groups`
		WHERE
			`name` = :name
	");

	$hostStatement = Application::getDbInstance()->prepare("
		SELECT
			`id`
		FROM
			`smng_hosts`
		WHERE
			`hostname` = :hostname
	");

	foreach ($templates as $template) {
		if (count($template->groups) == 0) {
			echo sprintf(
				"Template %s: No groups defined!",
				$template->name
			);
			continue;
		}

		$hosts = getHostsByTemplate($template->name);
		if (count($hosts) == 0) {
			echo sprintf(
				"Template %s: No hosts found!\n",
				$template->name
			);
			continue;
		}

		foreach ($hosts as $host) {
			$hostStatement->execute(array('hostname' => $host));
			$hostId = $hostStatement->fetch(\PDO::FETCH_COLUMN, 0);

			if (empty($hostId)) {
				echo sprintf(
					"Template %s: Couldn't find host %s!\n",
					$template->name,
					$host
				);
				continue;
			}

			foreach ($template->groups as $group) {
				$groupStatement->execute(array('name' => $group));
				$groupId = $groupStatement->fetch(\PDO::FETCH_COLUMN, 0);

				if (empty($groupId)) {
					echo sprintf(
						"Template %s: Couldn't find group %s for host %s!\n",
						$template->name,
						$group,
						$host
					);
					continue;
				}


				host\Host::addGroup(array(
					'hostId' => $hostId,
					'groupId' => $groupId
				));
				echo sprintf(
					"Template %s: Added host %s to group %s\n",
					$template->name,
					$host,
					$group
				);
			}
		}
	}
}

/**
 * Returns all hosts which are assigned to a template (zabbix)
 *
 * Recursion needed because there are transitive dependencies.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @param         string $templateName
 * @returns       array
 */
function getHostsByTemplate($templateName) {
	$statement = Application::getDbInstance('zabbix')->prepare("
		SELECT
			`hosts`.`host`,
			`hosts`.`available`
		FROM
			`hosts`
		JOIN `hosts_templates` ON (
			`hosts_templates`.`hostid` = `hosts`.`hostid`
		)
		JOIN `hosts` AS `templates` ON (
			`templates`.`hostid` = `hosts_templates`.`templateid`
		)
		WHERE
			`templates`.`host` = :templateName
	");

	$statement->execute(array('templateName' => $templateName));
	$hosts = $statement->fetchAll();

	$returnHosts = array();
	if (count($hosts) > 0) {
		foreach ($hosts as $host) {
			// Don't add templates to the hosts list
			if ($host['available'] == '1') {
				$returnHosts[] = $host['host'];
			}

			$linkedHosts = getHostsByTemplate($host['host']);
			if (count($linkedHosts) > 0) {
				$returnHosts = array_merge($returnHosts, $linkedHosts);
			}
		}
	}

	return $returnHosts;
}
