<?php

namespace ch\monitoring;

use ch\monitoring\model;

use org\adfinis\framework\db\DbManager;
use org\adfinis\framework\utils\Registry;
use org\adfinis\framework\session\SessionManager;
use org\adfinis\framework\config\ConfManager;

/**
 * Handles global needed application data and static function calls of frequently
 * used classes.
 *
 * @copyright     Adfinis-SyGroup AG 2011-2012
 * @license       BSD 3-Clause
 */
class Application {

	/**
	 * Initialise all needed objects and data
	 *
	 * @returns       void
	 */
	public static function init() {
		$loggingLevel = ConfManager::getVar('sm', 'logging', 'level');
		if ($loggingLevel === \Zend_Log::DEBUG) {
			ini_set('display_errors', 1);
			ini_set('display_startup_errors', 1);
			error_reporting(E_ALL - E_DEPRECATED);
		}
		self::addUserDataToRegistry();
		self::refreshCache();
		self::initLogger();
	}

	/**
	 * Save registry variables to the session
	 *
	 * @returns       void
	 */
	public static function save() {
		$session = SessionManager::getInstance();

		$session->hostChanged = self::getRegistry()->get('hostChanged');
		$session->satelliteChanged = self::getRegistry()->get('satelliteChanged');
		$session->allSatellitesChanged = self::getRegistry()->get('allSatellitesChanged');
	}

	/**
	 * Adds the User data to the registry if they are available
	 *
	 * @returns       void
	 */
	private static function addUserDataToRegistry() {
		$session = SessionManager::getInstance();

		if (!isset($session->user) || !$session->user->hasIdentifier()) {
			$session->user = new model\User();
			$session->user->allowRead();
		}

		self::getRegistry()->set('user', $session->user);

		if (!isset($session->hostChanged)) {
			$session->hostChanged = 0;
		}
		self::getRegistry()->set('hostChanged', $session->hostChanged);
		
		if (!isset($session->satelliteChanged)) {
			$session->satelliteChanged = array();
		}
		self::getRegistry()->set('satelliteChanged', $session->satelliteChanged);

		if (!isset($session->allSatellitesChanged)) {
			$session->allSatellitesChanged = 0;
		}
		self::getRegistry()->set('allSatellitesChanged', $session->allSatellitesChanged);
	}

	/**
	 * Returns a DbWrapper instance.
	 *
	 * @returns       org\adfinis\framework\db\DbManager
	 */
	public static function getDbInstance($instanceName = 'default') {
		return DbManager::getInstance($instanceName);
	}

	/**
	 * Returns a SyRPC client instance.
	 *
	 * @returns       SyRPC\Client
	 */
	public static function getRPCClientInstance($appName, $queues = 64) {
		$settings = array(
			'encoding'        => ConfManager::getVar('sm',  'RPC',  'encoding'),
			'app_name'        => $appName,
			'amq_virtualhost' => ConfManager::getVar('sm',  'RPC',  'amq',        'virtualhost'),
			'amq_host'        => ConfManager::getVar('sm',  'RPC',  'amq',        'host'),
			'amq_user'        => ConfManager::getVar('sm',  'RPC',  'amq',        'user'),
			'amq_password'    => ConfManager::getVar('sm',  'RPC',  'amq',        'password'),
			'amq_num_queues'  => $queues,
			'timeout'         => ConfManager::getVar('sm',  'RPC',  'timeout')
		);
		\SyRPC\Common::$lg = \SyRPC\Common::setupLogger(
				'symonitoring-frontend',
				(int) ConfManager::getVar('sm', 'logging', 'level')
		);
		return \SyRPC\Client::getInstance($settings);
	}

	/**
	 * Returns a Registry instance
	 *
	 * @returns       org\adfinis\framework\utils\Registry
	 */
	public static function getRegistry() {
		return Registry::getInstance();
	}

	/**
	 * Returns the current user
	 *
	 * @returns       model\User
	 */
	public static function getUser() {
		return self::getRegistry()->get('user');
	}

	/**
	 * Returns the application logger
	 *
	 * @returns      Zend_Log
	 */
	public static function getLogInstance() {
		try {
			$log = self::getRegistry()->get('log');
		}
		catch (\Exception $e) {
			Application::initLogger();
			$log = self::getRegistry()->get('log');
		}
		return $log;
	}

	/**
	 * Refreshs the user cache
	 *
	 * @returns       void
	 */
	public static function refreshCache() {
		self::getUser()->getACL()->refreshCache();
	}

	/**
	 * Creates a logger and sets it in the Registry
	 *
	 * @returns      void
	 */
	public static function initLogger() {
		$loggingLevel = ConfManager::getVar('sm', 'logging', 'level');
		$writer = new \Zend_Log_Writer_Syslog(
			array(
				'application' => 'symonitoring-frontend',
			)
		);
		$writer->addFilter($loggingLevel);
		$logger = new \Zend_Log($writer);
		self::getRegistry()->set('log', $logger);
	}


	/**
	 * Updates the cache to current date tiem
	 *
	 * The cache will be rebuilded on next page refresh
	 *
	 * @return        void
	 */
	public static function updateCache() {
		self::getUser()->getACL()->updateCache();
	}

	/**
	 * Sets the 'host changed' flag
	 *
	 * @returns       void
	 */
	public static function setHostChanged() {
		self::getRegistry()->set('hostChanged', 1);
	}
	
	/**
	 * Unsets the 'host changed' flag
	 *
	 * @returns       void
	 */
	public static function unsetHostChanged() {
		self::getRegistry()->set('hostChanged', 0);
	}
	
	/**
	 * Sets the 'satellite changed' array
	 *
	 * @param         int $hostId
	 * @param         string $hostname
	 * @returns       void
	 */
	public static function setSatelliteChanged($hostId, $hostname) {
		$satelliteChanged = self::getRegistry()->get('satelliteChanged');
		$satelliteChanged[$hostId] = $hostname;
		self::getRegistry()->set('satelliteChanged', $satelliteChanged);
	}
	
	/**
	 * Unsets an 'satellite changed' array flag
	 *
	 * @param         int $hostId
	 * @returns       void
	 */
	public static function unsetSatelliteChanged($hostId) {
		$satelliteChanged = self::getRegistry()->get('satelliteChanged');
		unset($satelliteChanged[$hostId]);
		self::getRegistry()->set('satelliteChanged', $satelliteChanged);
	}

	/**
	 * Sets the 'all satellites changed' flag
	 *
	 * @returns       void
	 */
	public static function setAllSatellitesChanged() {
		self::getRegistry()->set('allSatellitesChanged', 1);
	}
	
	/**
	 * Unsets the 'all satellites changed' flag
	 *
	 * @returns       void
	 */
	public static function unsetAllSatellitesChanged() {
		self::getRegistry()->set('allSatellitesChanged', 0);
	}
}
