<?php

namespace ch\monitoring\acl;

use ch\monitoring\datamapper;
use ch\monitoring\util;
use ch\monitoring\Application;
use org\adfinis\framework\session\SessionManager;
use org\adfinis\framework\exceptions\ForbiddenException;

/**
 * Implements Acl for cached ACL in the session
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SessionCachedACL implements ACL {
	/**
	 * The cache which holds a lookup table for permissions
	 *
	 * @var           \ch\monitoring\model\Customer $cacheRootCustomer
	 */
	private $cacheRootCustomer;

	/**
	 * The cache which holds a lookup table for permissions
	 *
	 * @var           array $cache
	 */
	private $cache = array();

	/**
	 * The cache which holds a lookup table for customers
	 *
	 * @var           array $cacheCustomer
	 */
	private $cacheCustomer;

	/**
	 * The date when the cache was last updated
	 *
	 * @var           Zend_Date $date
	 */
	private $date;

	/**
	 * Build permission cache for a user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function __construct($userId = null) {
		if ($userId === null) {
			$session = SessionManager::getInstance();
			$user = $session->user;
		}
		else {
			$user = datamapper\UserMapper::getInstance()->getById($userId);
		}
		$this->buildCache($user);
	}

	/**
	 * Build the permission cache for logged in user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function buildCache($user) {
		$customers            = $user->getCustomersWithRoles();
		$customersQuery       = "SELECT `id` FROM `Customer` ORDER BY `name` ASC";
		$allCustomers         = Application::getDbInstance()->fetchCol($customersQuery);
		$this->date           = new \Zend_Date();
		$this->cacheCustomers = $allCustomers;

		$this->cache          = array();
		foreach ($customers as $customerId => $roles) {
			$subCustomers = $customerId != null
							? datamapper\CustomerMapper::getInstance()->getSubCustomers($customerId)
							: array();

			foreach ($roles as $role) {
				$permissions = $role->getPermissionsAsArray();

				foreach ($permissions as $permission) {
					if ($customerId == null) {
						$this->cache[$permission] = $allCustomers;
						continue;
					}

					$this->cache[$permission][] = $customerId;

					foreach ($subCustomers as $subCustomerId) {
						$this->cache[$permission][] = $subCustomerId;
					}

					$this->cache[$permission] = array_unique($this->cache[$permission]);
				}
			}
		}
	}

	/**
	 * Refreshs the cache if it expired
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function refreshCache() {
		$this->cacheRootCustomer = null;

		$updateCache = util\Setting::get('updateCache');
		$updateCacheDate = new \Zend_Date($updateCache, \Zend_Date::ISO_8601);
		if ($this->date->isEarlier($updateCacheDate)) {
			$session = SessionManager::getInstance();
			$user = $session->user;
			$this->buildCache($user);
		}
	}

	/**
	 * Updates the cache to current date tiem
	 *
	 * The cache will be rebuilded on next page refresh
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        void
	 */
	public function updateCache() {
		$this->cacheRootCustomer = null;

		$date = new \Zend_Date();
		util\Setting::set('updateCache', $date->get(\Zend_Date::ISO_8601));
	}

	/**
	 * Returns a list of customers a user got a certain permission on
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $permissionName
	 * @returns       array
	 */
	public function getCustomersWithPermission($permissionName) {
		if(!isset($this->cache[$permissionName])) {
			throw new ForbiddenException(tr("Access denied"));
		}

		return $this->cache[$permissionName];
	}

	/**
	 * Verifies whether a user has a certain permission for any customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function hasPermission($permissionName) {
		return (
			isset($this->cache[$permissionName])
			&&
			is_array($this->cache[$permissionName])
			&&
			count($this->cache[$permissionName]) > 0
		);
	}

	/**
	 * Enforces a permission
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId
	 * @throws        ForbiddenException
	 * @returns       void
	 */
	public function enforcePermission($permissionName) {
		if (!$this->hasPermission($permissionName)) {
			throw new ForbiddenException(
				tr("Access denied")
			);
		}
	}

	/**
	 * Verifies whether a user has a certain permission for a certain customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId
	 * @returns       bool
	 */
	public function hasPermissionForCustomer($permissionName, $customerId) {
		return (
			isset($this->cache[$permissionName])
			&&
			is_array($this->cache[$permissionName])
			&&
			in_array($customerId, $this->cache[$permissionName])
		);
	}
	
	/**
	 * Enforces a permission for a customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId
	 * @throws        ForbiddenException
	 * @returns       void
	 */
	public function enforcePermissionForCustomer($permissionName, $customerId) {
		if (!$this->hasPermissionForCustomer($permissionName, $customerId)) {
			throw new ForbiddenException(
				tr("Access denied")
			);
		}
	}

	/**
	 * Verifies whether a user has a certain permission for the whole system (global customer)
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function hasPermissionForWholeSystem($permissionName) {
		return $this->hasPermissionForCustomer($permissionName, $this->getRootCustomerIdentifier());
	}

	/**
	 * Enforces a permission for the global customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @throws        ForbiddenException
	 * @returns       void
	 */
	public function enforcePermissionForWholeSystem($permissionName) {
		if(!$this->hasPermissionForWholeSystem($permissionName)) {
			throw new ForbiddenException(
				tr("No Systemwide rights")
			);
		}
	}

	/**
	 * Returns a cached version of the root customer
	 *
	 * Read rights are always set. DONT USE THIS OBJECT FOR WRITING!
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       \ch\monitoring\model\Customer
	 */
	public function getRootCustomer() {
		if(!$this->cacheRootCustomer) {
			$this->cacheRootCustomer = datamapper\CustomerMapper::getInstance()->getRootCustomer();
			$this->cacheRootCustomer->allowRead();
		}

		return $this->cacheRootCustomer;
	}

	/**
	 * Returns a cached version of the PK from the root customer
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getRootCustomerIdentifier() {
		return $this->getRootCustomer()->id;
	}

	public function getAllCustomers() {
		return $this->cacheCustomers;
	}
}
