<?php

namespace ch\monitoring\controller;

use ch\monitoring\Application;
use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle account page requests
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AccountController extends UserBaseController {
	/**
	 * Handles GET requests
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');
		$this->setHeaderTitle(tr("Account"));
		$user = $this->getWritableUserById($this->session->user->id, false);
		if (
			($this->action === 'ldapSync' || $this->action == 'saveLDAPSync') &&
			$user->authenticationMethod === model\User::AUTH_METHOD_LDAP_SYNC
		) {
			$ldap = new util\Ldap();
			$ldapAttributes = $ldap->getChangedAttributes($user);
			$this->parameters['user'] = $user;
			$this->parameters['ldapAttributes'] = $ldapAttributes;
			$this->tpl->assign('template', 'users/usersLDAPSync.tpl');
		}
		else {
			$this->session->user->enforcePermission(
				model\Permission::USER_SETTINGS
			);
			$this->assignFormVariables();
			$this->tpl->assign('template', 'users/usersAccount.tpl');
		}
		$this->outputPage('index.tpl');
	}

	/**
	 * Handles POST Requests
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handlePOST() {
		$user = $this->session->user;
		if ($this->action !== 'saveLDAPSync') {
			$user->enforcePermission(
				model\Permission::USER_SETTINGS
			);
		}
	}

	/**
	 * Handles AJAX requests
	 *
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');
		$this->callActionHandler();

		$this->tpl->assign('template', 'users/usersAccountAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Synchronizes the given attributes for the given user
	 *
	 * @throws        UserException if no user was given
	 * @throws        UserException if the user has the wrong authentication
	 *                              method
	 * @returns       void
	 */
	public function saveLDAPSyncAction() {
		$userId = $this->request->getInt('id');
		$user = $this->getWritableUserById($userId, false);
		$ldapAttributes = $this->request->getRaw('ldap');

		if ($user->authenticationMethod !== model\User::AUTH_METHOD_LDAP_SYNC) {
			throw new UserException(
				sprintf(
					tr('The user "%s" has LDAP synchronization not enabled, '
					.  ' please contact an administrator'),
					$user->name
				)
			);
		}
		if (is_null($ldapAttributes)) {
			throw new UserException(
				tr(
					'No attributes for LDAP synchronization selected. '
				.   'Please select at least one attribute to synchronize'
				)
			);
		}
		$ldap = new util\Ldap();
		$ldapUser = $ldap->getUserByName(
			$user->name,
			array_keys(util\Ldap::$syncedAttributes)
		);
		$userData = $ldap->getUserDataByObject($ldapUser);
		$ldap->synchronize($userData, $user, $ldapAttributes);
		$this->parameters['user'] = $user;
		$this->parameters['ldapAttributes'] = $ldap->getChangedAttributes($user);
	}

	/**
	 * Save user contact information
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveContactAction() {
		$userId = $this->request->getInt('id');
		$user = $this->getWritableUserById($userId);
		$this->assignUserContact($user);
		$user->save();

		$this->parameters['user'] = $user;
	}

	/**
	 * Save user alert settings
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAlertSettingsAction() {
		$userId = $this->request->getInt('id');
		$user = $this->getWritableUserById($userId);

		$this->assignUserAlertSettings($user);
		$user->save();

		$this->assignFormVariables();
	}

	/**
	 * Save user alert template settings
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAlertTemplateSettingsAction() {
		$userId = $this->request->getInt('id');

		$user = $this->getWritableUserById($userId);
		$this->assignUserAlertTemplateSettings($user);
		$user->save();

		$this->assignFormVariables();
	}

	/**
	 * Prepare user settings
	 *
	 * @throws        UserException
	 * @returns       model\User
	 */
	private function getWritableUserById($userId, $enforce = true) {
		if (empty($userId)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		$user = datamapper\UserMapper::getInstance()->getByID($userId);
		if ($enforce) {
			$user->enforceWritePermission(
				model\Permission::USER_SETTINGS
			);
		}
		else {
			$user->allowRead();
		}

		return $user;
	}

	/**
	 * Assigns variables needed for populating
	 * the forms
	 *
	 * @returns       void
	 */
	private function assignFormVariables() {
		$user              = $this->getWritableUserById($this->session->user->id);
		$customers         = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::USER_SETTINGS
		);
		$templates         = util\IteratorUtil::getKeyValueArray(
			datamapper\AlertTemplateMapper::getInstance()->getAll()
		);
		$slaActions        = datamapper\SLAActionMapper::getInstance()->getAll();

		$this->parameters['customers'] = $customers;
		$this->parameters['templates'] = $templates;
		$this->select['slaActions']    = $slaActions;
		$this->parameters['user']      = $user;
		$this->parameters['activeTab'] = 'contact';
	}
}
