<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\utils;
use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\util;
use ch\monitoring\model;

/**
 * Handle alert page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertController extends LogBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::ALERTS_READ
		);

		$this->setHeaderTitle(tr("Alerts"));

		$limit = 20;
		$this->parameters['limit'] = $limit;

		if ($this->page == 'alertsHistory') {
			$alerts = datamapper\AlertMapper::getInstance()->getAll(
				false,
				false,
				null,
				null,
				null,
				null,
				null,
				false,
				model\Permission::ALERTS_READ,
				true
			);
			$alerts->setOffsetLimit(0, $limit);

			$this->parameters['allAlerts'] = $alerts;
			$this->parameters['dates']     = array('dateFrom' => null, 'dateTo' => null);
		}
		else if ($this->page == 'alertsOpen') {
			// show all open alerts
			$alerts = datamapper\AlertMapper::getInstance()->getAll(
				false,
				true,
				null,
				null,
				null,
				null,
				null,
				false,
				model\Permission::ALERTS_READ,
				true
			);
			$this->parameters['allAlerts'] = $alerts;
		}
		else {
			// show all new alerts
			$alerts = datamapper\AlertMapper::getInstance()->getAll(
				true,
				null,
				null,
				null,
				null,
				null,
				null,
				false,
				model\Permission::ALERTS_READ,
				true
			);
			$this->parameters['allAlerts'] = $alerts;
		}

		$this->tpl->assign('template', 'logs/alerts.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		if ($this->action == "save" ||
			$this->action == "deleteComment") {
			$alertId = $this->request->getInt('alertId');
			$alert = datamapper\AlertMapper::getInstance()->getByID($alertId);
			$this->tpl->assign('alert', $alert);
			$this->outputAjaxPage(
				array(
					'#box_comments' => 'logs/alertsSave.tpl',
					'#item'.$alertId => 'logs/alertsAcknowledge.tpl',
				)
			);
			return;
		}

		$this->tpl->assign('template', 'logs/alertsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Shows all new alerts
	 *
	 * @returns       void
	 */
	protected function newAlertsAction() {
		// show all new alerts
		$alerts = datamapper\AlertMapper::getInstance()->getAll(true);
		$this->parameters['allAlerts'] = $alerts;
	}

	/**
	 * Shows all open alerts
	 *
	 * @returns       void
	 */
	protected function openAlertsAction() {
		// show all open alerts
		$alerts = datamapper\AlertMapper::getInstance()->getAll(false, true);
		$this->parameters['allAlerts'] = $alerts;
	}

	/**
	 * Show the alert log
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function logAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Alert id is missing")
			);
		}

		$this->parameters['alert'] = datamapper\AlertMapper::getInstance()->getByID($id);
		$this->parameters['alert']->enforceReadPermission(
			model\Permission::ALERTS_READ
		);
	}

	/**
	 * Show the form to edit an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Alert id is missing")
			);
		}

		$this->parameters['alert'] = datamapper\AlertMapper::getInstance()->getByID($id);
		$this->parameters['alert']->enforceReadPermission(
			model\Permission::ALERTS_READ
		);

		$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
			datamapper\AlertTagMapper::getInstance()->getAll(),
			'id',
			'name'
		);
	}

	/**
	 * Add a comment to an alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function saveAction() {
		$userId  = $this->session->user->id;
		$alertId = $this->request->getInt('alertId');
		$comment = $this->request->getString('comment');

		$tags = $this->request->getRaw('commentTags');
		if (empty($tags)) {
			$tags = array();
		}

		$alert = datamapper\AlertMapper::getInstance()->getByID($alertId);
		$alert->enforceWritePermission(
			model\Permission::ALERTS_WRITE
		);

		$comment = $alert->addComment($comment, $tags, $userId);

		$this->parameters['comment'] = $comment;
		$this->parameters['alert'] = $alert;
	}

	/**
	 * Delete a comment from an alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteCommentAction() {
		$commentId = $this->request->getInt('commentId');

		if (empty($commentId)) {
			throw new UserException(tr("Comment id is missing"));
		}

		$alertComment = datamapper\AlertCommentMapper::getInstance()->getByID(
			$commentId
		);
		$alertComment->enforceWritePermission(
			model\Permission::ALERTS_WRITE
		);
		$alertComment->delete();

		$this->parameters['alert'] = $alertComment->getAlert();
	}

	/**
	 * Bulk acknowledge alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function bulkSaveAction() {
		$alerts = $this->request->getRaw('alerts');
		if (is_array($alerts)) {
			foreach ($alerts as $alertId) {
				if (!empty($alertId)) {
					$alert = datamapper\AlertMapper::getInstance()->getByID($alertId);
					$alert->enforceWritePermission(
						model\Permission::ALERTS_WRITE
					);

					$alert->acknowledge($this->session->user->id);
				}
			}
		}
		else {
			throw new UserException(tr("No alerts selected for bulk saving"));
		}

		$this->parameters['allAlerts'] = datamapper\AlertMapper::getInstance()->getAll(true);
	}

	/**
	 * Acknowledge an alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function acknowledgeAction() {
		$alertId = $this->request->getInt('alertId');

		if (empty($alertId)) {
			throw new UserException(tr("Alert id is missing"));
		}

		$alert = datamapper\AlertMapper::getInstance()->getByID($alertId);
		$alert->enforceWritePermission(
			model\Permission::ALERTS_WRITE
		);

		if ($alert->acknowledged) {
			throw new UserException(
				tr('This alert already is acknowledged')
			);
		}

		$alert = $alert->acknowledge($this->session->user->id);
		$host = $alert->getHost();

		$this->parameters['alert'] = $alert;
		$this->tpl->assign('alert', $alert);

		if ($alert->serviceInstanceId == null) {
			$command = array(
				"ACKNOWLEDGE_HOST_PROBLEM",
				$host->hostname,               // host_name
				2,                             // sticky
				1,                             // notify
				0,                             // persistent
				"root",                        // author
				"Acknowledged in SyMonitoring" // comment
			);
			util\Monitoring::sendCommand($command);
		}
		else {
			$command = array(
				"ACKNOWLEDGE_SVC_PROBLEM",
				$host->hostname,               // host_name
				$alert->getHostServiceInstance()->getObjectName(), // host_name
				2,                             // sticky
				1,                             // notify
				0,                             // persistent
				"root",                        // author
				"Acknowledged in SyMonitoring" // comment
			);
			util\Monitoring::sendCommand($command);
		}
	}

	/**
	 * Show an item
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function getItemAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(tr("Alert id is missing"));
		}

		$alert = datamapper\AlertMapper::getInstance()->getByID($id);
		$alert->enforceReadPermission(
			model\Permission::ALERTS_READ
		);
		$this->parameters['alert'] = $alert;
	}

	/**
	 * Delete an alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr("Alert id is missing"));
		}

		$alert = datamapper\AlertMapper::getInstance()->getByID($id);
		$alert->enforceWritePermission(
			model\Permission::ALERTS_WRITE
		);
		$alert->delete();
	}

	/**
	 * Show all alerts in a given time frame
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function showAlertsAction() {
		$this->session->user->enforcePermission(
			model\Permission::ALERTS_READ
		);

		$dateTo = null;
		$dateFrom = null;

		if ($this->request->getString('dateFrom') !== "") {
			try {
				$dateFrom = new \Zend_Date(
					$this->request->getString('dateFrom'),
					\Zend_Date::ISO_8601
				);
			} catch (\Exception $e) {
				throw new UserException(
					tr("Start date is missing or has an incorrect format")
				);
			}
		}

		if ($this->request->getString('dateTo') !== "") {
			try {
				$dateTo = new \Zend_Date(
					$this->request->getString('dateTo'),
					\Zend_Date::ISO_8601
				);
				$dateTo->add(1, \Zend_Date::DAY);
			} catch (\Exception $e) {
				throw new UserException(
					tr("End date ist missing or has an incorrect format")
				);
			}
		}

		if ($dateFrom && $dateTo) {
			if (!$dateFrom->isEarlier($dateTo)) {
				throw new UserException(
					tr("Start date is more recent than end date")
				);
			}
		}

		$limit = $this->request->getInt('limit');
+		$this->parameters['limit'] = $limit;

		$this->parameters['dates'] = array(
			'dateTo'   => $dateTo,
			'dateFrom' => $dateFrom
		);

		$alerts = datamapper\AlertMapper::getInstance()->getAll(
			false,
			false,
			null,
			$dateFrom,
			$dateTo
		);

		$alerts->setOffsetLimit(0, $limit);

		$this->parameters['allAlerts'] = $alerts;
	}
}
