<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Handle hostOS page requests
 *
 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class ArchitectureController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::HOSTS_MASTER
		);

		$this->setHeaderTitle(tr("Architectures"));
		$this->parameters['architectures'] = datamapper\HostOSArchitectureMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'hosts/architectures.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();
		$this->tpl->assign('template', 'hosts/architectureAjax.tpl');

		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Opens an edit dialogue form
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function editFormAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$architecture = datamapper\HostOSArchitectureMapper::getInstance()->getByID($id);
		$architecture->enforceReadPermission(model\Permission::HOSTS_MASTER);

		$this->parameters['architecture'] = $architecture;
	}

	/**
	 * Opens a form for adding an architecture
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addFormAction() {
		$architecture = new model\HostOSArchitecture();
		$architecture->allowRead();
		$this->parameters['architecture'] = $architecture;
	}

	/**
	 * Adds a new architecture to the database
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$architecture = new model\HostOSArchitecture();
		$architecture->allowWrite(model\Permission::HOSTS_MASTER);
		$this->saveArchitecture($architecture);
		$this->parameters['architecture'] = $architecture;
	}

	/**
	 * Saves an existing architecture to the database
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id', request\REQUEST_POST);
		$architecture = datamapper\HostOSArchitectureMapper::getInstance()->getByID($id);
		$architecture->enforceWritePermission(model\Permission::HOSTS_MASTER);

		$this->saveArchitecture($architecture);
	}

	/**
	 * Opens an edit dialogue form
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$architecture = datamapper\HostOSArchitectureMapper::getInstance()->getByID($id);
		$architecture->enforceReadPermission(
			model\Permission::HOSTS_MASTER
		);

		$this->parameters['architecture'] = $architecture;
	}

	/**
	 * Deletes an existing architecture from the database
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(
				tr("Architecture is missing")
			);
		}

		$architecture = datamapper\HostOSArchitectureMapper::getInstance()->getByID($id);
		$architecture->enforceWritePermission(model\Permission::HOSTS_MASTER);

		try {
			$architecture->delete();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			$hostOSVersions = datamapper\HostOSVersionMapper::getInstance()->getByArchitectureID($architecture->id)->toArray();
			if ($hostOSVersions) {
				$entries = array();
				foreach ($hostOSVersions as $hostOSVersion) {
					$entries[] = sprintf(
						'Host OS: "%s" Version: "%s %s"',
						$hostOSVersion->getHostOS()->name,
						$hostOSVersion->shortName,
						$hostOSVersion->name
					);
					if (count($entries) > 5) {
						$entries[] = tr('and some more...');
						break;
					}
				}
				throw new UserException(sprintf(
					"%s\n%s",
					tr("The architecture cannot be deleted. The architecture is being referenced by following host OS version(s):"),
					join("\n", $entries)
				));
			}
			$hostsOS = datamapper\HostOSMapper::getInstance()->getByArchitectureID($architecture->id)->toArray();
			if ($hostsOS) {
				foreach ($hostsOS as $hostOS) {
					$entries[] = sprintf(
						'%s',
						$hostOS->name
					);
					if (count($entries) > 5) {
						$entries[] = tr('and some more...');
						break;
					}
				}
				throw new UserException(sprintf(
					"%s\n%s",
					tr("The architecture cannot be deleted. The architecture is being referenced by following host OS:"),
					join("\n", $entries)
				));
			}
			throw new UserException(
				tr("The architecture cannot be deleted. Please verify that no host OS
				version is using this architecture any longer.")
			);
		}
	}

	/**
	 * Private helper method to persist an architecture in the database
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function saveArchitecture($architecture) {
		$architecture->name = $this->request->getString('name');
		$architecture->save();

		$this->parameters['architecture'] = $architecture;
	}
}
