<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Handle application page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class BREApplicationController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::BREAPPLICATION_READ
		);

		$this->setHeaderTitle(tr("Applications"));
		$this->addLinkWithSourcePath('vis/dist/vis.min.css');

		$this->select['breapplications'] = datamapper\BREApplicationMapper::getInstance()->getApplications();

		$this->tpl->assign('template', 'hosts/applications.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		$this->callActionHandler();
		// TODO: if the action is getApplication, then return the application
	}

	/**
	 * Generic addForm Action to reduce double code
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function genericAddFormAction($modelName, $paramName) {
		$breobj = new $modelName();
		$breobj->allowRead();
		$this->parameters[$paramName] = $breobj;
		$this->assignFormVariables();
		$this->outputAjaxPage(
			sprintf(
				"applications/%s.tpl",
				$this->action
			)
		);
	}

	/**
	 * Display the form to add an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addFormAction() {
		$this->genericAddFormAction(
			'\ch\monitoring\model\BREApplication',
			'breapplication'
		);
	}

	/**
	 * Display the form to delete an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteFormAction() {
		$this->parameters['applicationId'] = $this->request->getInt('application', request\REQUEST_GET);
		$this->outputAjaxPage('applications/deleteForm.tpl');
	}

	/**
	 * Display the form to edit an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function editFormAction() {

		$breappId = $this->request->getInt('application', request\REQUEST_GET);
		$breapp = datamapper\BREApplicationMapper::getInstance()->getByID($breappId);

		$this->parameters['breapplication'] = $breapp;
		$this->addFormVariables();
		$this->outputAjaxPage('applications/editForm.tpl');
	}

	/**
	 * Display the form to add a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addComponentFormAction() {
		$this->genericAddFormAction(
			'\ch\monitoring\model\BREComponent',
			'brecomponent'
		);
	}

	/**
	 * Display the form to edit a component
	 *
	 * @return        void
	 */
	protected function editComponentFormAction() {
		$componentId = $this->request->getInt('component', request\REQUEST_GET);
		if (empty($componentId)) {
			throw new UserException(
				tr("Component ID is missing")
			);
		}

		$component   = datamapper\BREComponentMapper::getInstance()->getByID($componentId);
		$this->parameters['brecomponent'] = $component;
		$this->editComponentFormVariables();
		$this->outputAjaxPage('applications/editComponentForm.tpl');
	}

	/**
	 * Display the form to delete a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteComponentFormAction() {
		$this->parameters['componentId'] = $this->request->getInt('component', request\REQUEST_GET);
		$this->outputAjaxPage('applications/deleteComponentForm.tpl');
	}

	/**
	 * Display the form to add a service
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addServiceFormAction() {
		$this->genericAddFormAction(
			'\ch\monitoring\model\BREService',
			'breservice'
		);
	}

	/**
	 * Display the form to delete a service
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteServiceFormAction() {
		$this->parameters['componentId'] = $this->request->getInt('component', request\REQUEST_GET);
		$this->parameters['serviceId'] = $this->request->getInt('service', request\REQUEST_GET);
		$this->outputAjaxPage('applications/deleteServiceForm.tpl');
	}

	/**
	 * Display the form to add a host
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addHostFormAction() {
		$this->genericAddFormAction(
			'\ch\monitoring\model\BREHost',
			'brehost'
		);
	}

	/**
	 * Display the form to delete a host
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteHostFormAction() {
		$this->parameters['componentId'] = $this->request->getInt('component', request\REQUEST_GET);
		$this->parameters['hostId'] = $this->request->getInt('host', request\REQUEST_GET);
		$this->outputAjaxPage('applications/deleteHostForm.tpl');
	}

	/**
	 * Display the form to add a host
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addApplicationFormAction() {
		$this->genericAddFormAction(
			'\ch\monitoring\model\BREComponentApplication',
			'brecomponentapplication'
		);
	}

	/**
	 * Display the form to delete a component application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteApplicationFormAction() {
		$this->parameters['componentId'] = $this->request->getInt('component', request\REQUEST_GET);
		$this->parameters['applicationId'] = $this->request->getInt('application', request\REQUEST_GET);
		$this->outputAjaxPage('applications/deleteApplicationForm.tpl');
	}

	/**
	 * Generic add action
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        instance of type $modelName
	 */
	protected function genericAddAction($modelName) {
		try {
			$breobj = new $modelName();
			$breobj->allowWrite(model\Permission::BREAPPLICATION_WRITE);
			$breobj->fillFromRequestData($this->request);

			$mapper = $breobj->getMapper();
			$mapper->save($breobj);
		}
		catch (UserException $exception) {
			if ($exception->getCode() === UserException::INTEGRITY_ERROR) {
				throw new UserException(tr(sprintf(
					"The %s with the name '%s' is already present.\n" .
					"Please choose an unique name per BRE application.",
					substr($modelName, strrpos($modelName, '\\') + 1),
					$breobj->name
				)));
			}
			else {
				throw $exception;
			}
		}

		return $breobj;
	}

	/**
	 * Add an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addAction() {
		$breobj = $this->genericAddAction('\ch\monitoring\model\BREApplication');
		$this->parameters['application'] = $breobj;
		$this->outputAjaxPage(null);
	}

	/**
	 * Delete an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteAction() {
		$breappId = $this->request->getInt('applicationId');
		datamapper\BREApplicationMapper::getInstance()->delete($breappId);
		$this->outputAjaxPage(null);
	}

	/**
	 * Edit an application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function editAction() {
		$breappId = $this->request->getInt('applicationId');
		if (empty($breappId)) {
			throw new UserException(
				tr("Application ID is missing")
			);
		}

		$mapper = datamapper\BREApplicationMapper::getInstance();
		$breapp = $mapper->getByID($breappId);
		$breapp->allowWrite(model\Permission::BREAPPLICATION_WRITE);
		$breapp->fillFromRequestData($this->request);
		$mapper->save($breapp);

		$this->parameters['application'] = $breapp;
		$this->outputAjaxPage(null);
	}

	/**
	 * Add a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addComponentAction() {
		$brecomp = $this->genericAddAction('\ch\monitoring\model\BREComponent');
		$mapper = datamapper\BREApplicationMapper::getInstance();
		$this->parameters['application'] = $mapper->getById($brecomp->applicationId);
		$this->parameters['component'] = $brecomp;
		$this->outputAjaxPage(null);
	}

	/**
	 * Edit a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function editComponentAction() {
		$brecomponentId = $this->request->getInt('componentId');
		if (empty($brecomponentId)) {
			throw new UserException(
				tr("Component ID is missing")
			);
		}

		$mapper = datamapper\BREComponentMapper::getInstance();
		$brecomp = $mapper->getByID($brecomponentId);
		$brecomp->allowWrite(model\Permission::BREAPPLICATION_WRITE);
		$brecomp->fillFromRequestData($this->request);
		$mapper->save($brecomp);

		$mapper = datamapper\BREApplicationMapper::getInstance();
		$breapp = $mapper->getByID($brecomp->applicationId);

		$this->parameters['application'] = $breapp;
		$this->parameters['component'] = $brecomp;
		$this->outputAjaxPage(null);
	}

	/**
	 * Delete a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteComponentAction() {
		$brecomponentId = $this->request->getInt('componentId');
		datamapper\BREComponentMapper::getInstance()->delete($brecomponentId);
		$this->outputAjaxPage(null);
	}

	/**
	 * Add a service
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addServiceAction() {
		$breobj = new model\BREService();
		$breobj->allowWrite(model\Permission::BREAPPLICATION_WRITE);
		$breobj->fillFromRequestData($this->request);
		$mapper = $breobj->getMapper();

		foreach ($this->request->getRaw('serviceId') as $serviceId) {
			$breobj->serviceId = (int)$serviceId;
			$mapper->save($breobj);
		}

		$this->outputAjaxPage(null);
	}

	/**
	 * Delete a service
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteServiceAction() {
		$brecomponentId = $this->request->getInt('componentId');
		$breserviceId = $this->request->getInt('serviceId');
		datamapper\BREServiceMapper::getInstance()->deleteService($brecomponentId, $breserviceId);
		$this->outputAjaxPage(null);
	}

	/**
	 * Add a host
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addHostAction() {
		$breobj = new model\BREHost();
		$breobj->allowWrite(model\Permission::BREAPPLICATION_WRITE);
		$breobj->fillFromRequestData($this->request);
		$mapper = $breobj->getMapper();

		foreach ($this->request->getRaw('hostId') as $hostId) {
			$breobj->hostId = (int)$hostId;
			$mapper->save($breobj);
		}

		$this->outputAjaxPage(null);
	}

	/**
	 * Delete a host
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteHostAction() {
		$brecomponentId = $this->request->getInt('componentId');
		$brehostId = $this->request->getInt('hostId');
		datamapper\BREHostMapper::getInstance()->deleteHost($brecomponentId, $brehostId);
		$this->outputAjaxPage(null);
	}

	/**
	 * Add an application to a component
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function addApplicationAction() {
		$breobj = new model\BREComponentApplication();
		$breobj->allowWrite(model\Permission::BREAPPLICATION_WRITE);
		$breobj->fillFromRequestData($this->request);
		$mapper = $breobj->getMapper();

		foreach ($this->request->getRaw('componentapplicationId') as $appId) {
			$breobj->componentapplicationId = (int)$appId;
			$mapper->save($breobj);
		}

		$this->outputAjaxPage(null);
	}

	/**
	 * Delete a component application
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function deleteApplicationAction() {
		$brecomponentId = $this->request->getInt('componentId');
		$breappId = $this->request->getInt('applicationId');
		datamapper\BREComponentApplicationMapper::getInstance()->deleteComponentApplication($brecomponentId, $breappId);
		$this->outputAjaxPage(null);
	}

	/**
	 * Used to get rid of duplicated code
	 *
	 * @author        Pablo Vergés <pablo.verges@adfini-sygroup.ch>
	 */

	private function genericAssignFormVariables($objName, $paramName) {
		$objId = $this->request->getInt($objName, request\REQUEST_GET);

		if ($objId > 0) {
			$this->parameters[$paramName] = $objId;

			return $objId;
		}

		return null;
	}

	/**
	 * Assigns variables for the add/save form
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	private function assignFormVariables() {
		$method = array(
			$this,
			sprintf(
				"%sVariables",
				$this->action
			)
		);

		if (is_callable($method)) {
			call_user_func($method);
		}
	}

	private function addComponentFormVariables() {
		if ($this->genericAssignFormVariables('application', 'applicationId') !== null) {
			$this->genericAssignFormVariables('parent', 'parentId');
		}
		else {

			$this->select['applications'] = datamapper\BREApplicationMapper::getInstance()->getApplications(
				null,
				model\Permission::BREAPPLICATION_WRITE
			);

			$this->select['parents'] = datamapper\BREComponentMapper::getInstance()->getComponents(
				null,
				model\Permission::BREAPPLICATION_WRITE
			);
		}
	}

	private function editComponentFormVariables() {

		$this->select['applications'] = datamapper\BREApplicationMapper::getInstance()->getApplications(
			null,
			model\Permission::BREAPPLICATION_WRITE
		);

		$this->select['parents'] = datamapper\BREComponentMapper::getInstance()->getComponents(
			null,
			model\Permission::BREAPPLICATION_WRITE
		);
	}

	private function addServiceFormVariables() {
		$currentApplicationId = $this->request->getInt('application', request\REQUEST_GET);

		if ($this->genericAssignFormVariables('component', 'componentId') == null) {
			$this->select['components'] = datamapper\BREComponentMapper::getInstance()->getComponents(
				null,
				model\Permission::BREAPPLICATION_WRITE
			);
		}

		$services             = array();
		$hostServiceInstances = datamapper\HostServiceInstanceMapper::getInstance()->getAll(
			null,
			null,
			null,
			null,
			null,
			null,
			model\Permission::HOSTS_READ,
			null,
			$currentApplicationId
		);

		$optgroups = array();

		foreach ($hostServiceInstances as $hostServiceInstance) {
			$host    = $hostServiceInstance->getHost();
			$service = $hostServiceInstance->getService();

			$services[$host->hostname][$hostServiceInstance->id] = sprintf(
				"%s.%s",
				$service->name,
				$hostServiceInstance->getName()
			);

			$optgroups[] = $host->hostname;
		}

		$optgroups = array_unique($optgroups);
		sort($optgroups, SORT_STRING + SORT_FLAG_CASE | SORT_NATURAL);
		$this->select['optgroups'] = $optgroups;
		$this->select['services'] = $services;
	}

	private function addHostFormVariables() {
		$currentApplicationId = $this->request->getInt('application', request\REQUEST_GET);

		if ($this->genericAssignFormVariables('component', 'componentId') == null) {
			$this->select['components'] = datamapper\BREComponentMapper::getInstance()->getComponents(
				null,
				model\Permission::BREAPPLICATION_WRITE
			);
		}

		$hosts = array();
		$hostInstances = datamapper\HostMapper::getInstance()->getAll(
			null,
			false,
			false,
			null,
			null,
			model\Permission::HOSTS_READ,
			null,
			null,
			null,
			null,
			null,
			$currentApplicationId
		);

		$optgroups = array();

		foreach ($hostInstances as $hostInstance) {
			$optgroups[] = $hostInstance->getCustomer()->name;
			$hosts[$hostInstance->getCustomer()->name][$hostInstance->id] = $hostInstance->hostname;
		}

		$optgroups = array_unique($optgroups);
		sort($optgroups, SORT_STRING + SORT_FLAG_CASE | SORT_NATURAL);

		$this->select['optgroups'] = $optgroups;
		$this->select['hosts'] = $hosts;
	}

	private function addApplicationFormVariables() {

		// Get the component for the form
		if ($this->genericAssignFormVariables('component', 'componentId') == null) {
			$this->select['components'] = datamapper\BREComponentMapper::getInstance()->getComponents(
				null,
				model\Permission::BREAPPLICATION_WRITE
			);
		}

		// Get the applications for the form
		$currentApplicationId = $this->request->getInt('application', request\REQUEST_GET);
		$this->select['componentapplications'] = datamapper\BREApplicationMapper::getInstance()->getApplications(
			null,
			model\Permission::BREAPPLICATION_READ,
			$currentApplicationId
		);
		$optgroups = array();
		foreach ($this->select['componentapplications'] as $customer => $application) {
			$optgroups[] = $customer;
		}
		$optgroups = array_unique($optgroups, SORT_STRING | SORT_FLAG_CASE | SORT_NATURAL);
		sort($optgroups, SORT_STRING + SORT_FLAG_CASE | SORT_NATURAL);
		$this->select['optgroups'] = $optgroups;
	}

	private function addFormVariables() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::BREAPPLICATION_WRITE
			)
		);
	}

	/**
	 * Get details for an application
	 */
	protected function getApplicationAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);
		$data = array(
			'application_id' => $id
		);
		$resultId = $rpcClient->putRequest('symonitoring_bpe.get_application', $data);
		$result = $rpcClient->getResult($resultId);

		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}

		$this->parameters['application'] = $result;
		$this->outputAjaxPage(null);
	}

	/**
	 * Get a select button of the available applications
	 */
	protected function getApplicationListAction() {

		$this->select['breapplications'] = datamapper\BREApplicationMapper::getInstance()->getApplications();
		$this->parameters['selected'] = $this->request->getInt('selected', request\REQUEST_GET);
		$this->outputAjaxPage('applications/snippets/applicationSelectButton.tpl');
	}
}
