<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use ch\monitoring\Application;
use ch\monitoring\util;
use org\adfinis\framework\config\ConfManager;

/**
 * Abstract class that you can use as a base to implement object-oriented controller classes.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       BSD 3-Clause
 **/
abstract class BaseController extends \org\adfinis\framework\controller\BaseController {

	/**
	 * The page name
	 *
	 * @var           string $page
	 */
	protected $page;

	/**
	 * The action
	 *
	 * @var           string $action
	 */
	protected $action;

	/**
	 * Select menu information for smarty
	 *
	 * @var           array $select
	 */
	protected $select = array();

	/**
	 * Parameters for smarty
	 *
	 * @var           array $parameters
	 */
	protected $parameters = array();

	/**
	 * Error message storage
	 *
	 * @var           array<string> $notifyMessages
	 */
	protected $errorMessages = array();

	/**
	 * Notification message storage
	 *
	 * @var           array<string> $notifyMessages
	 */
	protected $notifyMessages = array();

	/**
	 * Constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         org\adfinis\framework\request\RequestInterface $request
	 * @param         array $urlMatch
	 */
	public function __construct($request, $urlMatch) {
		parent::__construct($request, $urlMatch);

		$this->loadMessagesFromSession();

		$this->page = $request->getString('page', request\REQUEST_GET);
		if (empty($this->page)) {
			$this->page = $request->getString('page', request\REQUEST_POST);
		}

		$this->action = $request->getString('action', request\REQUEST_GET);
		if (empty($this->action)) {
			$this->action = $request->getString('action', request\REQUEST_POST);
		}

		$this->parameters['action'] = $this->action;

		$this->parameters['avoidAlerting'] = util\Setting::monitoringIsDisabled();

		$this->setDbVars();
	}

	/**
	 * Handles the request
	 *
	 * Depending on request type, calls
	 * - handleGET(),
	 * - handlePOST(),
	 * - handleAJAX(),
	 * - handleExternal()
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handle() {
		try {
			$ajaxTabs = $this->request->getString('ajaxTabs', request\REQUEST_GET);
			if ($ajaxTabs === "true") {
				return $this->handleGET();
			}

			$external = (bool)$this->request->getString('external', request\REQUEST_GET);
			if ($external === "true") {
				return $this->handleExternal();
			}

			return parent::handle();
		}
		catch (\org\adfinis\framework\exceptions\InvalidInputException $e) {
			$this->handleException($e);
		}
		catch (\org\adfinis\framework\exceptions\UserException $e) {
			$this->handleException($e, false);
		}
		catch (\org\adfinis\framework\exceptions\ForbiddenException $e) {
			$this->handleException($e);
		}
		catch (\RuntimeException $e) {
			$this->handleException($e);
		}
		catch (\Exception $e) {
			$this->handleException($e);
		}
	}

	/**
	 * Handles an exception
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Exception $e
	 * @param         bool $logError Optional, default true
	 * @param         bool $showError Optional, default true
	 * @returns       void
	 */
	private function handleException($e, $logError = true, $showError = true) {

		if ($showError) {
			if (in_array(ConfManager::getVar('af', 'env'), ['dev', 'vagrant'])) {
				$errorMessage = sprintf(
					"%s<br><br>Stacktrace:<br>%s",
					$e->getMessage(),
					$e->getTraceAsString()
				);
			}
			else {
				$errorMessage = $e->getMessage();
			}
			$this->addErrorMessage($errorMessage);
		}

		if ($logError) {
			$errorMessage = sprintf(
				"%s\n%s",
				$e->getMessage(),
				$e->getTraceAsString()
			);
			Application::getLogInstance()->err($errorMessage);
			error_log($errorMessage);
		}

		$ajaxTabs = $this->request->keyExists('ajaxTabs', request\REQUEST_GET);
		if ($ajaxTabs) {
			$this->outputPage('error.tpl');
		}
		elseif (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
			$this->outputAjaxPage();
		}
		else {
			if ($this->page == 'login') {
				$this->tpl->assign('template', 'main/login.tpl');
				$this->outputPage('indexSimple.tpl');
			}
			else {
				$this->outputPage('index.tpl');
			}
		}
	}

	/**
	 * Does nothing
	 *
	 * Dummy implementation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handlePOST() {
	}

	/**
	 * Does nothing
	 *
	 * Dummy implementation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
	}

	/**
	 * Automatically calls a method according to the action
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @return        mixed
	 */
	public function callActionHandler() {
		$method = array(
			$this,
			sprintf(
				"%sAction",
				$this->action
			)
		);

		if (is_callable($method)) {
			return call_user_func($method);
		}
		else {
			throw new \RuntimeException(sprintf(
				"Action '%s' is not defined",
				$this->action
			));
		}
	}

	/**
	 * Does nothing
	 *
	 * Dummy implementation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleExternal() {
	}

	/**
	 * Returns the page name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getPage() {
		return $this->page;
	}

	/**
	 * Returns the action
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getAction() {
		return $this->action;
	}

	/**
	 * Assigns smarty variables
	 *
	 * These are the same for every page request
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignVariables() {
		$this->tpl->assign('page'   , $this->getPage());
		$this->tpl->assign('action' , $this->getAction());

		$this->tpl->assign('headerTitle'       , $this->headerTitle);
		$this->tpl->assign('metaFields'        , $this->getMetaTags());
		$this->tpl->assign('linkFields'        , $this->getLinkTags());
		$this->tpl->assign('topScripts'        , $this->getScriptTags(self::SCRIPT_POSITION_TOP));
		$this->tpl->assign('bottomScripts'     , $this->getScriptTags(self::SCRIPT_POSITION_BOTTOM));
		$this->tpl->assign('defaultURL'        , ConfManager::getVar('af', 'URLSSL', 'default'));
		$this->tpl->assign('monitoringURL'     , ConfManager::getVar('af', 'URLSSL', 'monitoring'));
		$this->tpl->assign('statusURL'         , ConfManager::getVar('af', 'URLSSL', 'status'));
		$this->tpl->assign('cactiURL'          , ConfManager::getVar('af', 'URLSSL', 'cacti'));
		$this->tpl->assign('nfsenURL'          , ConfManager::getVar('af', 'URLSSL', 'nfsen'));
		$this->tpl->assign('smokepingURL'      , ConfManager::getVar('af', 'URLSSL', 'smokeping'));
		$this->tpl->assign('weathermapURL'     , ConfManager::getVar('af', 'URLSSL', 'weathermap'));
		$this->tpl->assign('env'               , ConfManager::getVar('af', 'env'));
		$this->tpl->assign('hasErrorMessages'  , $this->hasErrorMessages());
		$this->tpl->assign('errorMessages'     , $this->errorMessages);
		$this->tpl->assign('hasNotifyMessages' , $this->hasNotifyMessages());
		$this->tpl->assign('notifyMessages'    , $this->notifyMessages);

		$this->tpl->assign('user', model\User::getOrCreateActive());

		$this->tpl->assign('rootCustomer', datamapper\CustomerMapper::getInstance()->getRootCustomer()->allowRead());

		$this->tpl->assign('hostChanged'          , Application::getRegistry()->get('hostChanged'));
		$this->tpl->assign('satelliteChanged'     , Application::getRegistry()->get('satelliteChanged'));
		$this->tpl->assign('allSatellitesChanged' , Application::getRegistry()->get('allSatellitesChanged'));
		$this->tpl->assign('parameters'           , $this->parameters);
		$this->tpl->assign('select'               , $this->select);

		// For access to the model constants
		$this->tpl->registerClass("Permission", "\ch\monitoring\model\Permission");
		$this->tpl->registerClass("User", "\ch\monitoring\model\User");

		// To access Zend_Date formats
		$this->tpl->registerClass("Zend_Date", "\Zend_Date");
	}

	/**
	 * Output page
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $tpl
	 * @returns       void
	 */
	public function outputPage($tpl = '') {
		$this->assignVariables();

		$this->tpl->display($tpl);
	}

	/**
	 * Output page in json object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         mixed $tpl
	 * @returns       void
	 */
	public function outputAjaxPage($tpl=null) {
		$this->assignVariables();
		if (empty($tpl)) {
			$html = '';
		}
		else if (is_array($tpl)) {
			$html = array();
			foreach ($tpl as $selector => $tplItem) {
				if (empty($selector) || empty($tplItem)) {
					throw new \RuntimeException(
						tr("Selector and template must be defined")
					);
				}

				$element = new \stdClass();
				$element->selector = $selector;
				$element->html     = $this->tpl->fetch($tplItem);
				$html[] = $element;
			}
		}
		else {
			$html = $this->tpl->fetch($tpl);
		}
		$parameters = $this->getParametersForJSON($html);

		header('Content-Type: application/json; charset=utf-8');
		echo json_encode($parameters);
	}

	/**
	 * Adds the given message to the error messages
	 *
	 * The given message is added to a list, which is then passed to the template when outputPage() is called.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $message
	 * @returns       void
	 */
	public function addErrorMessage($message) {
		$this->errorMessages[] = $message;
	}

	/**
	 * Adds the given message to the notification messages
	 *
	 * The given message is added to a list, which is then passed to the template when outputPage() is called.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $message
	 * @returns       void
	 */
	public function addNotifyMessage($message) {
		$this->notifyMessages[] = $message;
	}

	/**
	 * Returns whether there are error messages
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function hasErrorMessages() {
		return (bool)sizeof($this->errorMessages);
	}

	/**
	 * Returns whether there are notification messages
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function hasNotifyMessages() {
		return (bool)sizeof($this->notifyMessages);
	}

	/**
	 * Redirect to another url
	 *
	 * Stores error and notify messages in the session
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $url The target URL
	 * @param         bool $permanent Optional. Defaults to false.
	 * @returns       void
	 */
	public function redirectToURL($url, $permanent = false) {
		$this->saveMessagesToSession();
		parent::redirectToURL($url, $permanent);
	}

	/**
	 * Saves messages to the session
	 *
	 * This is used before a redirect, so messages can be displayed on the next page.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function saveMessagesToSession() {
		$this->session->baseControllerStoredMessages = serialize(array(
			'errorMessages'  => $this->errorMessages,
			'notifyMessages' => $this->notifyMessages
		));
	}

	/**
	 * Save the username and userID to the databse for use in triggers
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function setDbVars() {
		$username = $this->session->user->name;
		$userID   = $this->session->user->id;

		Application::getDbInstance()->query(sprintf("SET @username  = '%s'", $username));
		Application::getDbInstance()->query(sprintf("SET @userID    = '%s'", $userID));
		Application::getDbInstance()->query("SET @auditLogActionID  = 0");
		Application::getDbInstance()->query("SET @auditLogOrder     = 0");
		Application::getDbInstance()->query("SET @auditLogActivated = 1");
	}

	/**
	 * Returns parameters to be JSON encoded as an array
	 *
	 * @param       string $html Optional. Additional HTML string for the
	 *                           response
	 * @return      array        the controller parameters to be JSON encoded
	 */
	protected function getParametersForJSON($html=null) {
		$parameters = array(
			'error' => array(
				'hasErrorMessages' => $this->hasErrorMessages(),
				'errorMessages'    => $this->errorMessages
			),
			'notify' => array(
				'hasNotifyMessages' => $this->hasNotifyMessages(),
				'notifyMessages'    => $this->notifyMessages
			),
			'parameters' => $this->parameters,
		);
		if ($html) {
			$parameters['html'] = $html;
		}
		return $parameters;
	}

	/**
	 * Loads messages from session
	 *
	 * This is called in the constructor and loads messages from the session
	 * that have been put there before a redirect.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	private function loadMessagesFromSession() {
		if (isset($this->session->baseControllerStoredMessages)) {
			$data = unserialize($this->session->baseControllerStoredMessages);
			$this->errorMessages  = $data['errorMessages'];
			$this->notifyMessages = $data['notifyMessages'];

			unset($this->session->baseControllerStoredMessages);
		}
	}

	/**
	 * Override SSL required from config.
	 *
	 * This makes using SyMonitoring in a dev vagrant box much easier since the
	 * app doesn't need to redirect to a secure endpoint in all cases. On
	 * production we can still let the app to do redirecting even though there
	 * isn't a way to actual trigger the redirect (apache does the redirecting
	 * looong before our actual PHP code gets hit).
	 *
	 * @returns boolean
	 */
	protected function sslRequired() {
		$required = true;
		if (
			ConfManager::keyExists('af', 'disableTLSRedirect') &&
			ConfManager::keyExists('af', 'disableTLSRedirect')
		) {
			$required = false;
		}
		return $required;
	}
}
