<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;

/**
 * Handle emergency calendar page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CalendarController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::EMERGENCY_READ
		);

		$this->setHeaderTitle(tr("Emergency calendar"));

		$this->getCalendarAction();
		$this->getUsersAction();
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::EMERGENCY_READ
			)
		);

		$this->tpl->assign('template', 'emergency/calendar.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		header('Content-Type: text/html; charset=utf-8');
		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'emergency/calendarAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Get calendar entries for a year and month
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getCalendarAction() {
		// If no month or year is given, use current month and year
		$year = $this->request->keyExists('year')
			? $this->request->getInt('year')
			: date('Y');

		$month = $this->request->keyExists('month')
			? $this->request->getInt('month')
			: date('m');

		$customerId = $this->request->keyExists('customerId')
			? $this->request->getInt('customerId')
			: $this->session->user->customerId;

		$this->session->user->enforcePermissionForCustomer(
			model\Permission::EMERGENCY_READ,
			$customerId
		);

		$this->parameters['calendar'] = datamapper\CustomerTeamMapper::getInstance()->getCalendarMonthArray(
			$year,
			$month,
			$customerId
		);
	}

	/**
	 * Get emergency enabled users
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getUsersAction() {
		$customerId = $this->request->keyExists('customerId')
			? $this->request->getInt('customerId')
			: $this->session->user->customerId;

		$this->session->user->enforcePermissionForCustomer(
			model\Permission::EMERGENCY_READ,
			$customerId
		);

		$users = datamapper\UserMapper::getInstance()->getAllByPermission(
			model\Permission::EMERGENCY_ALERT,
			$customerId,
			true
		);

		if (count($users) == 0) {
			$users = array();
		}

		$usersArray = array();
		foreach ($users as $user) {
			$usersArray[$user->id] = sprintf(
				"%s - %s",
				$user->authenticationMethod,
				$user->fullName
			);
		}

		$this->select['users'] = $usersArray;
	}

	/**
	 * Save calendar entries
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAction() {
		$customerId = $this->request->getInt('customerId');

		if (empty($customerId)) {
			throw new UserException(
				tr("Customer id is mandatory")
			);
		}

		$this->session->user->enforcePermissionForCustomer(
			model\Permission::EMERGENCY_WRITE,
			$customerId
		);

		$users       = $this->request->getRaw('users');
		if (empty($users)) {
			throw new UserException(
				tr("No users for assignment chosen")
			);
		}

		if (!$this->request->getString('dateFrom')) {
			throw new UserException(
				tr("No starting date for assignment chosen")
			);
		}
		$dateFrom    = new \Zend_Date(
			$this->request->getString('dateFrom'),
			\Zend_Date::ISO_8601
		);

		if (!$this->request->getString('dateTo')) {
			throw new UserException(
				tr("No end date for assignment chosen")
			);
		}
		$dateTo      = new \Zend_Date(
			$this->request->getString('dateTo'),
			\Zend_Date::ISO_8601
		);
		$dateCurrent = clone $dateFrom;

		// Assign the given users to each date in day range
		while (!$dateCurrent->isLater($dateTo)) {
			datamapper\CustomerTeamMapper::getInstance()->deleteByDate(
				$dateCurrent,
				$customerId
			);

			// The order of the team members is the order how they are notified
			$priority = 0;
			foreach ($users as $userId) {
				$customerTeam = new model\CustomerTeam();
				$customerTeam->allowWrite(model\Permission::EMERGENCY_WRITE);

				$customerTeam->customerId = $customerId;
				$customerTeam->userId     = $userId;
				$customerTeam->date       = $dateCurrent;
				$customerTeam->priority   = $priority++;
				$customerTeam->save();
			}

			$dateCurrent->add(1, \Zend_Date::DAY);
		}

		$this->parameters['calendar'] = datamapper\CustomerTeamMapper::getInstance()->getCalendarMonthArray(
			$dateFrom->get(\Zend_Date::YEAR),
			$dateFrom->get(\Zend_Date::MONTH),
			$customerId
		);
	}

	/**
	 * Delete a calendar entry
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteAction() {
		$customerId = $this->request->getInt('customerId');
		$date       = new \Zend_Date($this->request->getString('date'));

		if (empty($customerId)) {
			throw new UserException(tr("Customer id is missing"));
		}

		if (empty($date)) {
			throw new UserException(tr("Date is missing"));
		}

		$this->session->user->enforcePermissionForCustomer(
			model\Permission::EMERGENCY_WRITE,
			$customerId
		);

		datamapper\CustomerTeamMapper::getInstance()->deleteByDate($date, $customerId);

		$this->parameters['calendar'] = datamapper\CustomerTeamMapper::getInstance()->getCalendarMonthArray(
			$date->get(\Zend_Date::YEAR),
			$date->get(\Zend_Date::MONTH),
			$customerId
		);
	}
}
