<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Handle command page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CommandController extends ProtectedController {

	private $dirManager;

	public function __construct($request, $urlMatch) {
		parent::__construct($request, $urlMatch);
		$dirPath = ConfManager::getVar('sm', 'command', 'path', 'fs');
		$this->dirManager = new util\DirectoryManager(
			$dirPath,
			[$this, "addNotifyMessage"]
		);
	}

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::COMMANDS_READ
		);

		$this->setHeaderTitle(tr("Commands"));
		$this->parameters['allCommands'] = datamapper\CommandMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'services/commands.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'services/commandsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Create a new command and load the form
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$command = new model\Command();
		$command->allowRead();
		$this->parameters['command'] = $command;

		$this->assignFormVariables();
	}

	/**
	 * Load the command that is to edit and load the form
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$command = datamapper\CommandMapper::getInstance()->getByID($id);
		$command->enforceReadPermission(
			model\Permission::COMMANDS_WRITE
		);

		$this->parameters['command'] = $command;
		$this->assignFormVariables($id);
	}

	/**
	 * Assigns variables for the add/save form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $id   The id of the currently set command
	 * @returns       void
	 */
	private function assignFormVariables($id = null) {
		$this->select['commands'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CommandMapper::getInstance()->getAll()
		);

		$this->parameters['assignedGuidelines'] = datamapper\GuidelineMapper::getInstance()->getAllByCommandID($id);

		$this->select['oss'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostOSMapper::getInstance()->getAll()
		);

		$this->select['versions'] = util\IteratorUtil::getKeyValueArray(
			array()
		);
	}

	/**
	 * Add a new command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$command = new model\Command();
		$command->allowWrite(model\Permission::COMMANDS_WRITE);

		$this->saveCommand($command);
	}

	/**
	 * After editing, save the command
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id', request\REQUEST_POST);
		$command = datamapper\CommandMapper::getInstance()->getByID($id);
		$command->enforceWritePermission(model\Permission::COMMANDS_WRITE);

		$oldCommandName = $command->name;
		$this->saveCommand($command);
		$newCommandName = $command->name;

		//move directories if command name has changed
		if($oldCommandName != $newCommandName) {
			$guidelinesList = datamapper\GuidelineMapper::getInstance()->getAllByCommandID($command->id);
			foreach($guidelinesList as $guideline) {
				$oldDirPath = $this->dirManager->createDirectoryPath(
					$guideline->getHostOS->name,
					$guideline->getHostOSVersion->shortName,
					$oldCommandName
				);
				$newDirPath = $this->dirManager->createDirectoryPath(
					$guideline->getHostOS->name,
					$guideline->getHostOSVersion->shortName,
					$newCommandName
				);
				$this->dirManager->moveDir($oldDirPath, $newDirPath);
			}
		}
	}

	/**
	 * Save a command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Command $command
	 * @returns       void
	 */
	private function saveCommand($command) {
		$command->name        = $this->request->getString('name');
		$command->commandline = $this->request->getString('commandline');
		$command->description = $this->request->getString('description');
		$command->v6Capable   = $this->request->keyExists('v6Capable');
		$command->save();

		$this->parameters['command'] = $command;
		$this->parameters['allCommands'][] = $command;

		Application::setHostChanged();
		Application::setAllSatellitesChanged();
	}

	/**
	 * Delete a command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Command id is missing")
			);
		}

		try {
			$command = datamapper\CommandMapper::getInstance()->getByID($id);
			$command->enforceWritePermission(model\Permission::COMMANDS_WRITE);

			//delete guideline folders
			$guidelinesList =
				datamapper\GuidelineMapper::getInstance()->getAllByCommandID($id);
			foreach($guidelinesList as $guideline) {
				$dirPath = $this->dirManager->createDirectoryPath(
					$guideline->getHostOS()->name,
					$guideline->getHostOSVersion()->shortName,
					$command->name
				);
				$this->dirManager->removeDir($dirPath);
				$guideline->allowWrite(model\Permission::COMMANDS_WRITE);
				$guideline->delete();
			}

			foreach ($command->getServices() as $service) {
				$service->enforceWritePermission(model\Permission::COMMANDS_WRITE);
				$serviceAlerts = $service->getAlerts();
				foreach ($serviceAlerts as $alert) {
					datamapper\AlertMapper::getInstance()->delete(
						array('id' => $alert->id)
					);
				}
				$hostServices = $service->getHostServices();
				foreach ($hostServices as $hostService) {
					$hostService->enforceWritePermission(model\Permission::COMMANDS_WRITE);
					$hostServiceInstances = $hostService->getInstances();
					foreach ($hostServiceInstances as $hsi) {
						$downtimes = $hsi->getDowntimes();
						foreach ($downtimes as $downtime) {
							$downtime->enforceWritePermission(model\Permission::COMMANDS_WRITE);
							datamapper\DowntimeMapper::getInstance()->delete(
								array('id' => $downtime->id)
							);
						}

						datamapper\HostServiceInstanceMapper::getInstance()->delete(
							array('id' => $hsi->id)
						);
					}
					$hostService->delete();
				}
				$service->delete();
			}

			foreach ($command->getHostAlerts() as $hostAlert) {
				$hostAlert->enforceWritePermission(model\Permission::COMMANDS_WRITE);
				$hostAlert->delete();
			}

			$command->delete();

		} catch (\Exception $e) {
			throw new UserException(sprintf(
				"%s: %s",
				tr("Could not delete Command: There are depenencies"),
				$e->getMessage()
			));
		}
		Application::setHostChanged();
		Application::setAllSatellitesChanged();
	}

	/**
	 * Shows all dependencies of a command
	 *
	 * @returns void
	 */
	protected function showDependenciesAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(tr('Host id is missing'));
		}
		$command = datamapper\CommandMapper::getInstance()->getByID($id);
		$command->enforceWritePermission(model\Permission::COMMANDS_WRITE);
		$this->parameters['command'] = $command;
	}

	/**
	 * Returns all OS versions for a given OS
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getVersionsAction() {
		$id = $this->request->getInt('osId', request\REQUEST_GET);
		$commandId = $this->request->getInt('commandId', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("OS id is missing")
			);
		}
		$versionsList =
			datamapper\HostOSVersionMapper::getInstance()->getAllExcludedByCommandID($commandId, $id);

		$this->select['versions'] = $versionsList;
	}

	/**
	 * Saves and adds a guideline to a
	 * command.
	 *
	 * @author        Sven Osterwalder <sven.osterwalder@adfinis-sygroup.ch>
	 *
	 * @returns       void
	 */
	protected function addGuidelineAction() {
		$id          = $this->request->getInt('commandId',              request\REQUEST_POST);
		$osId        = $this->request->getInt('osId',                   request\REQUEST_POST);
		$osVersionId = $this->request->getInt('osVersionId',            request\REQUEST_POST);
		$content     = $this->request->getString('guidelineContent',    request\REQUEST_POST);

		if (empty($content)) {
			throw new UserException(
				tr("Guideline content must not be empty")
			);
		}

		$exists = datamapper\GuidelineMapper::getInstance()->doesGuidelineExist($osVersionId, $id);
		if ($exists) {
			throw new UserException(
				tr("There is already a guideline for this command and host os version")
			);
		}

		try {
			$command     = datamapper\CommandMapper::getInstance()->getByID($id);
			$command->allowRead(model\Permission::COMMANDS_READ);
		}
		catch (\Exception $e) {
			throw new UserException(
				tr("Command is missing or was not found")
			);
		}

		try {
			$os = datamapper\HostOSMapper::getInstance()->getByID($osId);
			$os->allowRead(model\Permission::COMMANDS_READ);
		}
		catch (\Exception $e) {
			throw new UserException(
				tr("Operating system is missing or was not found")
			);
		}

		try {
			$osVersion   = datamapper\HostOSVersionMapper::getInstance()->getByID($osVersionId);
			$osVersion->allowRead(model\Permission::COMMANDS_READ);
		}
		catch (\Exception $e) {
			throw new UserException(
				tr("You must choose an OS Version. If none is shown, please
					add one in 'Operating Systems'.")
			);
		}
		$file     = $_FILES['file'];
		$isManual = (
			file_exists($file['tmp_name'])
			|| is_uploaded_file($file['tmp_name'])
		);

		$guideline                          = new model\Guideline();
		$guideline->content                 = $content;
		$guideline->userId                  = $this->session->user->id;
		$guideline->updated                 = new \Zend_Date();
		$guideline->needsManualInstallation = $isManual;
		$guideline->versionId               = $osVersion->id;
		$guideline->commandId               = $command->id;
		$guideline->allowWrite(model\Permission::COMMANDS_WRITE);
		$guideline->filename                = $file['name'];
		$guideline->save();

		$dirPath = $this->dirManager->createDirectoryPath($os->name, $osVersion->shortName, $command->name);
		$this->dirManager->makeDir($dirPath);

		if($isManual && !move_uploaded_file($file['tmp_name'], $dirPath .basename($file['name']))) {
			$this->addNotifyMessage(
				tr('Could not upload file. Please manually upload it at: ' .$dirPath .'.')
			);
		}

		$this->parameters['command'] = $command;
		$this->assignFormVariables($command->id);
	}

	/**
	 * Deletes a guidelines and removes it from
	 * all commands.
	 *
	 * @author        Sven Osterwalder <sven.osterwalder@adfinis-sygroup.ch>
	 *
	 * @returns       void
	 */
	protected function deleteGuidelineAction() {
		$guidelineId   = $this->request->getInt('guidelineId', request\REQUEST_POST);

		try {
			$mapper = datamapper\GuidelineMapper::getInstance();
			$guideline = $mapper->getByID($guidelineId);
			$guideline->allowWrite(model\Permission::COMMANDS_READ);
		}
		catch (\Exception $e) {
			throw new UserException(
				tr("Given guideline was not found")
			);
		}

		$command = datamapper\CommandMapper::getInstance()->getByID($guideline->commandId)->allowRead();
		$dirPath = $this->dirManager->createDirectoryPath(
			$guideline->getHostOS()->name,
			$guideline->getHostOSVersion()->shortName,
			$command->name
		);

		$guideline->delete();
		$this->dirManager->removeDir($dirPath);
	}
}
