<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\request;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;


/**
 * Handle customer page requests
 *
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @return        void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::CUSTOMERS_READ
		);

		$this->setHeaderTitle(tr("Customers"));
		$this->parameters['allCustomers'] = datamapper\CustomerMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'customers/customers.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @return        void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'customers/customersAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Edit a customer
	 *
	 * @throws        UserException
	 * @return        void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$customer = datamapper\CustomerMapper::getInstance()->getByID($id);
		$customer->enforceReadPermission(
			model\Permission::CUSTOMERS_WRITE
		);

		$this->parameters['customer'] = $customer;
		$this->parameters['alertTypes'] = datamapper\SLAActionMapper::getInstance()->getAll();

		$this->select['slas']  = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::CUSTOMERS_READ
			)
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);

		$users = datamapper\UserMapper::getInstance()->getAllByPermission(
			model\Permission::EMERGENCY_ALERT,
			$id,
			true
		);
		if (count($users) == 0) {
			$users = array();
		}
		$usersArray = array();
		foreach ($users as $user) {
			$usersArray[$user->id] = sprintf(
				"%s - %s",
				$user->authenticationMethod,
				$user->fullName
			);
		}
		$this->select['users'] = $usersArray;
		$this->select['ldapGroups'] = $this->getLDAPCustomerGroups();

		$roles = array_filter(
			datamapper\RoleMapper::getInstance()->getAll()->toArray(),
			function($r) { return !$r->isGlobal(); }
		);

		$this->select['roles'] = util\IteratorUtil::getKeyValueArray($roles);
	}

	/**
	 * Add a customer
	 *
	 * @return        void
	 */
	protected function addFormAction() {
		$customer = new model\Customer(true);
        $customer->defaultRoleId = util\Setting::get('defaultRoleId', 0);
		$customer->allowRead();
		$customer->isEnabled = true;

		$this->parameters['customer'] = $customer;

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::CUSTOMERS_READ
			)
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
        
		$this->parameters['alertTypes'] = datamapper\SLAActionMapper::getInstance()->getAll();
		$this->select['ldapGroups'] = $this->getLDAPCustomerGroups();

		$roles = array_filter(
			datamapper\RoleMapper::getInstance()->getAll()->toArray(),
			function($r) { return !$r->isGlobal(); }
		);

		$this->select['roles'] = util\IteratorUtil::getKeyValueArray($roles);
	}

	/**
	 * Edit customer information
	 *
	 * @throws        UserException
	 * @return        void
	 */
	protected function editInformationAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$this->parameters['customer'] = datamapper\CustomerMapper::getInstance()->getByID($id);
		$this->parameters['customer']->enforceReadPermission(
			model\Permission::CUSTOMERS_WRITE
		);
	}



	/**
	 * Edit userpermissions
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @throws        UserException
	 * @return        void
	 */
	protected function editRightAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);
		$this->assignDefaultValuesForRightForm($id);
	}

	/**
	 * Assigns the default variables to the template for the rightform
	 * 
	 * @throws        UserException
	 * 
	 */
	private function assignDefaultValuesForRightForm($id = null) {
		if (is_null($id) || empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$this->parameters['customer'] = datamapper\CustomerMapper::getInstance()->getByID($id);
		$this->parameters['customer']->enforceReadPermission(
			model\Permission::CUSTOMERS_READ
		);

		if(!$this->session->user->hasPermissionForCustomer(
					model\Permission::USERS_WRITE,
					$id
				)
			) {
			throw new UserException(tr("Not allowed to edit userrights inside this customer"));
		}

		//get all assigned roles/users
		$this->parameters['rights'] = util\IteratorUtil::getKeyObjectArray(
			datamapper\RightMapper::getInstance()->getAllUpperRights($id)
		);
		
		//get all user which we're allowed to assign to this customer
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByUser($this->session->user, null)
		);

		//get all rights which we're allowed to assign to this customer
		$this->select['roles'] = util\IteratorUtil::getKeyValueArray(
			datamapper\RoleMapper::getInstance()->getAllByCustomer(
				$id,
				Application::getUser()->getACL()->getRootCustomerIdentifier() != $id
			)
		);
	}

	/**
	 * Delete a right
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @throws        UserException
	 * @return        void
	 */
	protected function deleteRightAction() {
		$id = $this->request->getInt('rightId', request\REQUEST_POST);

		if (empty($id)) {
			throw new UserException(
				tr("Right id is missing")
			);
		}

		$right = datamapper\RightMapper::getInstance()->getById($id);
		$right->enforceWritePermission(
			model\Permission::USERS_WRITE
		);

		$right->delete();

		// Cache needs to be updated
		Application::updateCache();
	}

	/**
	 * Add a right
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @throws        UserException
	 * @return        void
	 */
	protected function addRightAction() {
		$customerId = $this->request->getInt('customerId', request\REQUEST_POST);
		$userId = $this->request->getInt('userId', request\REQUEST_POST);
		$roleId = $this->request->getInt('roleId', request\REQUEST_POST);

		if (!$userId) {
			throw new UserException(tr("No User was selected, please specify a user to add a new right"));
		}

		$customer = datamapper\CustomerMapper::getInstance()->getById($customerId);
		$customer->allowRead();
		$role = datamapper\RoleMapper::getInstance()->getById($roleId);
		$role->allowRead();

		try {
			$user = datamapper\UserMapper::getInstance()->getById($userId);
		}
		catch (\OutOfBoundsException $e) {
			throw new UserException(tr("No user was found with given identification"));
		}

		if(Application::getUser()->getACL()->getRootCustomerIdentifier() != $customerId) {
			if($role->isGlobal()) {
				throw new UserException(tr("You cannot assign a global role to a customer which is not the root customer"));
			}
		}

		$right = new model\Right();

		$right->customerId = $customer->id;
		$right->roleId = $role->id;
		$right->userId = $user->id;

		$right->enforceWritePermission(
			model\Permission::USERS_WRITE
		);

		try {
			$right->save();
		}
		catch(\Zend_Db_Statement_Exception $e) {
			if($e->getCode() == 23000) {
				throw new UserException(tr('The user has already the selected role assigned for the selected customer'));
			}
			else {
				throw $e;
			}
		}

		$this->assignDefaultValuesForRightForm($customerId);

		// Cache needs to be updated
		Application::updateCache();
	}

	/**
	 * Save a customer
	 *
	 * @throws        UserException
	 * @return        void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$customer = datamapper\CustomerMapper::getInstance()->getByID($id);
		$customer->enforceWritePermission(
			model\Permission::CUSTOMERS_WRITE
		);

		$this->saveCustomer($customer);
	}

	/**
	 * Add a customer
	 *
	 * @return        void
	 */
	protected function addAction() {
		$customer = new model\Customer();
		$customer->allowWrite(model\Permission::CUSTOMERS_WRITE);
		$this->saveCustomer($customer);
	}

	/**
	 * Save a customer
	 *
	 * @param         model\Customer $customer The customer model to be saved
	 * @return        void
	 */
	private function saveCustomer($customer) {
		$oldCustomerName = $customer->name;

		$rootCustomerId = Application::getUser()->getACL()->getRootCustomerIdentifier();
		$parentId = $this->request->getInt('parentId');

		if (empty($parentId)) {
			$parentId = $rootCustomerId;
		}

		if($customer->id != $rootCustomerId) {
			if($parentId <= 0) {
				throw new UserException(tr("You must set a parent customer"));
			}

			$customer->parentId = $parentId;
		}

		$customer->name       = $this->request->getString('name');
		$customer->wiki       = $this->request->getString('wiki') ?: null;
		$customer->email      = $this->request->getString('email') ?: null;
		$customer->slaId      = $this->request->getInt('slaId');
		$customer->comment    = $this->request->getString('comment') ?: null;
		if($customer->id == $rootCustomerId) {
			$customer->isEnabled = true;
		}
		else {
			$customer->isEnabled  = $this->request->keyExists('isEnabled');
		}
		$customer->warnHoster = $this->request->keyExists('warnHoster');
		$customer->ldapGroup = $this->request->getString('userLDAPGroup') ?: null;
		$customer->defaultRoleId = $this->request->getInt('defaultRole') ?: null;
		if ($customer->hasIdentifier()) {
			$customer->save();
		}
		else {
			$customer->getMapper()->save($customer);
		}

		$alertTypes = datamapper\SLAActionMapper::getInstance()->getAll();
		foreach($alertTypes as $alertType) {
			$customer->setAlertSetting($alertType->id, $this->request->getInt('alerttype_' . $alertType->id . '_enabled'));
		}
		

		if (!empty($oldCustomerName) && $customer->name != $oldCustomerName) {
			$cacti = new util\Cacti(null, null);
			$cacti->updateCustomer(
				$customer->name,
				$oldCustomerName
			);

			$icingaWeb = new util\IcingaWeb(null, null);
			$icingaWeb->updateCustomer(
				$customer->name,
				$oldCustomerName
			);
		}

		$this->parameters['customer'] = $customer;

		$users = $this->request->getRaw('users');
		if (is_array($users) && count($users) > 0) {
			datamapper\CustomerTeamMapper::getInstance()->deleteByDate(
				null,
				$customer->id
			);

			$priority = 0;
			foreach ($users as $userId) {
				$customerTeam = new model\CustomerTeam();
				$customerTeam->allowWrite(model\Permission::CUSTOMERS_WRITE);

				$customerTeam->customerId = $customer->id;
				$customerTeam->userId     = $userId;
				$customerTeam->date       = null;
				$customerTeam->priority   = $priority++;
				$customerTeam->save();
			}
		}

		$this->parameters['allCustomers'][] = $customer;

		$date = new \Zend_Date();
		util\Setting::set('updateCache', $date->get(\Zend_Date::ISO_8601));
		Application::refreshCache();
	}

	/**
	 * Delete a customer
	 *
	 * @throws        UserException
	 * @return        void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$customer = datamapper\CustomerMapper::getInstance()->getByID($id);
		if(Application::getUser()->getACL()->getRootCustomerIdentifier() == $customer->id) {
			throw new UserException(tr("Cannot not delete root customer"));
		}

		$customer->enforceWritePermission(
			model\Permission::CUSTOMERS_WRITE
		);
		$customer->delete();
	}

	/**
	 * Save additional customer information
	 *
	 * @return        void
	 */
	protected function saveInformationAction() {
		$customerInformation = new model\CustomerInformation();
		$customerInformation->allowWrite(model\Permission::CUSTOMERS_WRITE);

		$customerInformation->id                = $this->request->getInt('id');
		$customerInformation->name              = $this->request->getString('name') ?: null;
		$customerInformation->address           = $this->request->getString('address') ?: null;
		$customerInformation->contactTechnical  = $this->request->getString('contactTechnical') ?: null;
		$customerInformation->contactCommercial = $this->request->getString('contactCommercial') ?: null;
		$customerInformation->phoneRegular      = $this->request->getString('phoneRegular') ?: null;
		$customerInformation->phoneEmergency    = $this->request->getString('phoneEmergency') ?: null;
		$customerInformation->comment           = $this->request->getString('comment') ?: null;
		$customerInformation->mayContact        = $this->request->keyExists('mayContact');

		$customerInformation->save();
	}


	/**
	 * Removes an emergency member from a customer
	 *
	 * @return        void
	 */
	protected function deleteMemberAction() {
		$memberId   = $this->request->getInt('id');

		if (empty($memberId)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$member = datamapper\CustomerTeamMapper::getInstance()->getByID($memberId);
		$member->enforceWritePermission(
			model\Permission::EMERGENCY_WRITE
		);
		$member->delete();
	}

	/**
	 * Returns all customer groups from LDAP.
	 *
	 * @return        array sorted customer groups from LDAP
	 */
	private function getLDAPCustomerGroups() {
		$ldapGroupsArray = array();
		$allowLdap  = ConfManager::getVar('af', 'auth', 'ldap')  == 'yes';
		if ($allowLdap) {
			$ldap = new util\Ldap();
			try {
				$ldapGroups = $ldap->getCustomerGroups(null, array('cn'));
				foreach ($ldapGroups as $ldapGroup) {
					$ldapGroupsArray[$ldapGroup['dn']] = $ldapGroup['cn'][0];
				}
				natcasesort($ldapGroupsArray);
			}
			catch (\UnexpectedValueException $exception) {
				Application::getLogInstance()->warn(sprintf(
					'%s: No LDAP customer groups were found',
					__METHOD__
				));
			}
		}
		return $ldapGroupsArray;
	}
}
