<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\exceptions\ForbiddenException;
use ch\monitoring\util\recurr\RecurrenceRule;
use ch\monitoring\util\recurr\RecurrenceRuleTransformer;


/**
 * Handle downtime API requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2015
 * @license       LGPLv3
 **/
class DowntimeAPIController extends ProtectedController {

	private $host     = null;
	private $instance = null;
	private $duration = 0;
	private $password = '';

	public function handle() {
		try {
			return parent::handle();
		}
		catch (\OutOfBoundsException $e) {
			$this->addErrorMessage($e->getMessage());
			$this->outputPage();
		}
		catch (ForbiddenException $e) {
			$this->addErrorMessage($e->getMessage());
			$this->outputPage();
		}
	}

	/**
	 * Verify whether the user has sent the correct password
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        bool
	 */
	public function hasPermission() {
		$this->setRequestVariables();
		if (is_null($this->host) || empty($this->password)) {
			return false;
		}
		return $this->host->password === $this->password;
	}

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->addNotifyMessage('Please use POST to send commands');
		$this->outputPage();
	}

	/**
	 * Handles POST requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	public function handlePOST() {
		try {
			$this->callActionHandler();
		}
		catch (\RuntimeException $e) {
			$this->addErrorMessage($e->getMessage());
		}
		$this->outputPage();
	}

	/**
	 * Output page
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @return        void
	 */
	public function outputPage($tpl='') {

		if ($tpl) {
			parent::outputPage($tpl);
		}

		try {
			$style = strtolower(
				$this->request->getString('style', request\REQUEST_GET)
			);
		}
		catch (\OutOfRangeException $e) { // no style get parameter?
			$style = 'text';
		}

		switch ($style) {
			case 'json':
				header('Content-Type: application/json; charset=utf-8');
				echo json_encode($this->getParametersForJSON());
				break;

			case 'xml':
				header('Content-Type: text/xml; charset=utf-8');
				parent::outputPage('maintenance/downtimesAPIxml.tpl');
				break;

			case 'text':
			case '':
			default:
				header('Content-Type: text/plain; charset=utf-8');
				parent::outputPage('maintenance/downtimesAPI.tpl');
				break;
		}
	}

	/**
	 * Assign request variables
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @return        void
	 */
	private function setRequestVariables() {
		$hostId     = $this->request->getInt('hostId');
		$instanceId = $this->request->getInt('instanceId');
		$duration   = $this->request->getInt('duration');
		$password   = $this->request->getString('password');

		if (empty($hostId)) {
			$hostId = $this->request->getInt('hostId', request\REQUEST_GET);
			if (empty($hostId)) {
				$this->addErrorMessage('Please provide a hostId');
				return;
			}
		}

		if (empty($password)) {
			$password = $this->request->getString('password', request\REQUEST_GET);
			if (empty($password)) {
				$this->addErrorMessage('Please provide a password');
				return;
			}
		}

		try {
			$host = datamapper\HostMapper::getInstance()->getByID($hostId);
		}
		catch (\OutofBoundsException $e) {
			$this->addErrorMessage('Could not find host with the given parameters');
			return;
		}
		$host->allowRead();
		$this->host = $host;

		if ($instanceId > 0) {
			$instance = datamapper\HostServiceInstanceMapper::getInstance()->getByID($instanceId);
			$instance->allowRead();
			$this->instance = $instance;
		}

		$this->duration = $duration;
		$this->password = $password;
	}

	/**
	 * Add a downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @return        void
	 */
	protected function addAction() {
		if (empty($this->duration)) {
			$this->addErrorMessage(tr('Please provide a duration'));
			return;
		}

		if ($this->duration < 1) {
			$this->addErrorMessage(tr('Please provide a valid duration'));
			return;
		}
		$now = new \Zend_Date();
		$downtime = new model\Downtime();
		$downtime->allowRead();
		$downtime->datetimeFrom = $now;

		$datetimeTo = clone $now;
		$datetimeTo->add($this->duration, \Zend_Date::MINUTE);
		$downtime->datetimeTo = $datetimeTo;
		$downtime->hostId = $this->host->id;

		if (!is_null($this->instance)) {
			$downtime->serviceInstanceId = $this->instance->id;
		}

		datamapper\DowntimeMapper::getInstance()->save($downtime);
		$this->addNotifyMessage(tr('Downtime was added'));
	}

	/**
	 * Stop all active downtimes of a given host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function stopAction() {
		datamapper\DowntimeMapper::getInstance()->unscheduleHost(
			$this->host,
			$this->instance
		);
		$this->addNotifyMessage(tr(
			'All active downtimes of this host/service are stopped'
		));
	}
}
