<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\exceptions\UserException;


/**
 * Handle downtime page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class DowntimeController extends MaintenanceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::DOWNTIMES_READ
		);

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
							false,
							false,
							false,
							null,
							null,
							model\Permission::DOWNTIMES_READ
			),
			"id",
			"hostname"
		);

		$this->setHeaderTitle(tr("Downtimes"));

		$this->tpl->assign('template', 'maintenance/downtimes.tpl');
		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'maintenance/downtimesAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Show the form to add a downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::DOWNTIMES_WRITE
			)
		);

		$this->select['hostgroups'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostGroupMapper::getInstance()->getAll()
		);
	}

	/**
	 * Add a downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function addAction() {
		$hosts            = $this->request->getRaw('host');
		$services         = $this->request->getRaw('services');
		$dateFrom         = $this->request->getString('dateFrom');
		$dateTo           = $this->request->getString('dateTo');
		$rrule            = $this->request->getString('rrule');
		$durationHours    = $this->request->getString('durationHours');

		if (count($hosts) == 0) {
			throw new UserException(
				tr("No hosts provided")
			);
		}

		if (empty($dateFrom)) {
			throw new UserException(
				tr("From date is missing")
			);
		}

		$dateFrom = new \Zend_Date(
			$dateFrom,
			\Zend_Date::ISO_8601
		);


		foreach ($hosts as $hostId => $value) {
			$host = datamapper\HostMapper::getInstance()->getByID($hostId);
			$host->enforceWritePermission(model\Permission::DOWNTIMES_WRITE);

			$downtime = new model\Downtime();
			$downtime->allowWrite(model\Permission::DOWNTIMES_WRITE);

			$downtime->datetimeFrom = $dateFrom;
			$downtime->hostId       = $hostId;

			if(!empty($rrule)) {
				if (empty($durationHours)) {
					throw new UserException(
						tr("The duration is missing")
					);
				}

				$tks = explode(':', $durationHours);
				$parsedDurationHours = (float)$tks[0] + ((float)$tks[1] / 60);

				$downtime->durationHours = $parsedDurationHours;
				$downtime->rrule         = $rrule;

				$rruleObject = new \Recurr\Rule($rrule, $dateFrom);
				$lastDate = $rruleObject->getUntil();
				if($lastDate) {
					$downtime->datetimeTo = new \Zend_Date($lastDate->getTimestamp(), \Zend_Date::TIMESTAMP);
				}
				else {
					//if no "end date" is set by the scheduler then COUNT is probably set

					$transformer = new \Recurr\Transformer\ArrayTransformer();
					$endDates    = $transformer->transform($rruleObject);
					if(!empty($endDates)) {
						$endDate = $endDates->last();
						$downtime->datetimeTo = new \Zend_Date($endDate->getEnd()->getTimestamp(), \Zend_Date::TIMESTAMP);
					}
					else {
						//it's an endless repeat
						$downtime->datetimeTo = NULL;
					}
				}
			}
			else {
				if (empty($dateTo)) {
					throw new UserException(
						tr("To date is missing")
					);
				}

				$dateTo = new \Zend_Date(
					$dateTo,
					\Zend_Date::ISO_8601
				);

				if ($dateTo->isEarlier($dateFrom)) {
					throw new UserException(
						tr("Start time cannot be after end time")
					);
				}

				$downtime->datetimeTo = $dateTo;
			}

			if (is_array($services) && array_key_exists($hostId, $services) && count($services[$hostId]) > 0) {
				foreach ($services[$hostId] as $instanceId) {
					$downtime->id = null;

					$instance = datamapper\HostServiceInstanceMapper::getInstance()->getByID($instanceId);
					$instance->enforceWritePermission(model\Permission::DOWNTIMES_WRITE);
					$downtime->serviceInstanceId = $instanceId;
					$downtime->save();
				}
			}
			else {
				$downtime->serviceInstanceId = null;
				$downtime->save();
			}

			$this->parameters['allDowntimes'][] = $downtime;
		}
	}

	/**
	 * Stop a downtime
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function stopAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Downtime id is missing")
			);
		}

		$mapper = datamapper\DowntimeMapper::getInstance();
		$downtime = $mapper->getByID($id);
		$downtime->enforceWritePermission(
			model\Permission::DOWNTIMES_WRITE
		);
		if(empty($downtime->rrule)) {
			$mapper->unschedule($downtime);
		}
		$dateTo = new \Zend_Date(
			null,
			\Zend_Date::ISO_8601
		);
		// Subtract a small offset of 10 seconds, as otherwise
		// the downtime may not appear archived
		$dateTo->sub('00:00:10', \Zend_Date::TIMES);
		// DatetimeFrom is more recent than current stop date
		if ($downtime->datetimeFrom->compare($dateTo) === 1) {
			$downtime->datetimeTo = $downtime->datetimeFrom;
		}
		else {
			// Current stop date lies within range, so set it
			$downtime->datetimeTo = $dateTo;
		}
		$downtime->save();
		if(empty($downtime->rrule)) {
			$mapper->schedule($downtime);
		}
	}

	/**
	 * Delete a downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Downtime id is missing")
			);
		}

		$downtime = datamapper\DowntimeMapper::getInstance()->getByID($id);
		$downtime->enforceWritePermission(
			model\Permission::DOWNTIMES_WRITE
		);
		$downtime->delete();
	}

	/**
	 * Get hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function getHostsAction() {
		$customerId = $this->request->getInt('customerId', request\REQUEST_GET);

		if (empty($customerId)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$this->parameters['hosts'] = datamapper\HostMapper::getInstance()->getAll(
			null,
			false,
			false,
			null,
			$customerId
		);
	}

	/**
	 * Get hosts by a given hostgroup
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function getHostsForHostGroupAction() {
		$id = $this->request->getInt('hostgroupId', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$this->parameters['hosts'] = datamapper\HostMapper::getInstance()->getAll(
			null,
			false,
			false,
			null,
			null,
			model\Permission::HOSTS_READ,
			null,
			null,
			null,
			$id
		);
	}

	/**
	 * Get downtimes by filter
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getDowntimesAction() {
		$this->session->user->enforcePermission(
			model\Permission::DOWNTIMES_READ
		);

		$hostId              = $this->request->getInt('hostId') ?: null;
		$limit               = $this->request->getInt('limit');
		$isRecurringDowntime = $this->request->getInt('recurring');

		$downtimes = datamapper\DowntimeMapper::getInstance()->getAll(
			null,
			$hostId,
			null,
			null,
			null,
			model\Permission::DOWNTIMES_READ,
			$isRecurringDowntime
		);

		$downtimes->setOffsetLimit(0, $limit);

		$this->parameters['allDowntimes'] = $downtimes;
	}

	/**
	 * Handles DataTable requests
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        string json
	 */
	public function dataTableAction() {
		$this->session->user->enforcePermission(
			model\Permission::DOWNTIMES_READ
		);
		$settings = $_GET;
		unset($settings['action']);

		$downtimes = datamapper\DowntimeMapper::getInstance()->getAllForDataTable($settings);

		header('Content-Type: application/json; charset=utf-8');
		print json_encode($downtimes->toDataTable());
		exit;
	}
}
