<?php

namespace ch\monitoring\controller;

use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;

/**
 * Handle export requests
 *
 * @author		  Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright	  Adfinis SyGroup AG 2012
 * @license		  LGPLv3
 **/
class ExportController extends ProtectedController {
	/**
	 * Holds the persistent database setting name of the last reload date
	 *
	 * @const string RELOAD_CONFIG_SETTING
	 */
	const RELOAD_CONFIG_SETTING = 'lastIcingaConfigReload';

	/**
	 * Holds the last found timeout after writing a config
	 *
	 * @var         int $lastTimeout
	 */
	private $lastTimeout;

	/**
	 * Handles GET requests
	 *
	 * @author		  Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns		  void
	 */
	public function handleGET() {
		// Not implemented
	}

	/**
	 * Handles AJAX requests
	 *
	 * @return		  void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');
		print json_encode($this->callActionHandler());
	}

	/**
	 * Write the given icinga config kind
	 *
	 * @param       string $kind
	 * @param       array  $IDs
	 * @return      object
	 */
	private function writeIcingaConfig($kind, $data = null) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'wqAppName'),
			ConfManager::getVar('sm','RPC', 'wqQueues')
		);
		$timeout   = ConfManager::getVar('sm', 'RPC', 'icinga_timeout');
		$type      = 'writeIcingaConfig';
		$params    = array(
			'kind' => $kind
		);
		if (is_array($data)) {
			$params += $data;
		}
		$resultID = $rpcClient->putRequest($type, $params);
		$result   = $rpcClient->getResult($resultID, $timeout);
		if (isset($result->exception)) {
			Application::getLogInstance()->warn(sprintf(
				'There was an exception writing Icinga configuration: %s',
				$result->exception->traceback
			));
			throw new UserException($result->exception->message, $result->exception->code);
		}
		$this->lastTimeout = $result->timeout;
		return $result;
	}

	protected function getTimeUntilReloadAction() {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'wqAppName'),
			ConfManager::getVar('sm','RPC', 'wqQueues')
		);
		$timeout   = ConfManager::getVar('sm', 'RPC', 'icinga_timeout');
		$type     = 'getTimeout';
		$resultID = $rpcClient->putRequest($type);
		$result   = $rpcClient->getResult($resultID);
		return [ 'data' => (int)$result->timeout ];
	}

	protected function getLastReloadDateAction() {
		$lastReloadDate = util\Setting::get(static::RELOAD_CONFIG_SETTING, false);
		return [ 'data' => $lastReloadDate ];
	}

	/**
	 * Export hosts to monitoring configuration
	 *
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws      UserException
	 * @return      void
	 */
	protected function buildAction() {
		Application::getLogInstance()->debug(sprintf(
			"%s: Trying to build monitoring configuration",
			__METHOD__
		));
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_WRITE
		);
		$result = $this->writeIcingaConfig('master');
		return $this->handleExportResult($result, 'The monitoring configuration');
	}

	/**
	 * Export nrpe configuration
	 *
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws      UserException
	 * @return      void
	 */
	protected function buildNrpeAction() {
		$nrpeHostIDs = $this->request->getInt('nrpeHostId');
		// nrpeHostIDs can be either an array or a scalar
		if ($nrpeHostIDs > 0) {
			$nrpeHostIDs = array($nrpeHostIDs);
		}
		else {
			$nrpeHostIDs = $this->request->getRaw('nrpeHostId');

			// Error if nrpeHostIDs is neither a positive int nor an array
			if (!is_array($nrpeHostIDs)) {
				throw new UserException(tr(
					"No NRPE hosts selected. Please select at least one host"
				));
			}
		}
		$result = $this->writeIcingaConfig('NRPE', array(
			'IDs' => $nrpeHostIDs
		));
		return $this->handleExportResult($result, 'The selected NRPE host(s)');
	}

	/**
	 * Export all nrpe configuration
	 *
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns     void
	 */
	protected function buildAllNrpeAction() {
		$customerID = $this->request->getInt('customerId') ?: null;
		$hosts      = datamapper\HostMapper::getInstance()->getAll(
			false,
			true,
			false,
			null,
			$customerID,
			model\Permission::HOSTS_READ,
			true
		);

		foreach ($hosts as $host) {
			$host->enforceWritePermission(
				model\Permission::HOSTS_WRITE
			);
		}

		$result = $this->writeIcingaConfig('NRPE', array(
			'customerID' => $customerID
		));
		return $this->handleExportResult($result, 'All NRPE host(s)');
	}

	/**
	 * Export a satellite to monitoring configuration
	 *
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws      UserException
	 * @return      void
	 */
	protected function buildSatelliteAction() {
		$satelliteIDs = $this->request->getRaw('satelliteId');

		// satelliteId can be either an array or a scalar
		if (is_numeric($satelliteIDs)) {
			$satelliteIDs = array($satelliteIDs);
		}
		else if (!$satelliteIDs) {
			$satelliteIDs = $this->request->getRaw('id');

			// Error if satelliteIDs is neither a positive int nor an array
			if (!is_array($satelliteIDs)) {
				throw new UserException(tr(
					"No satellites selected. Please select at least one satellite"
				));
			}
		}
		foreach ($satelliteIDs as $currentID) {
			if (empty($currentID)) {
				continue;
			}

			try {
				$satellite = datamapper\HostMapper::getInstance()->getByID(
					$currentID
				);
				$satellite->enforceWritePermission(
					model\Permission::HOSTS_WRITE
				);
				Application::unsetSatelliteChanged($satellite->id);
			}
			catch (\OutOfBoundsException $e) {
				throw new UserException(
					tr('Satellite not found')
				);
			}
		}
		$result = $this->writeIcingaConfig('satellite', array(
			'IDs' => $satelliteIDs
		));
		return $this->handleExportResult($result, 'The selected satellite(s)');
	}

	/**
	 * Builds all satellites
	 *
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return      void
	 */
	protected function buildAllSatellitesAction() {
		$customerID = $this->request->getInt('customerId');
		$satellites = datamapper\HostMapper::getInstance()->getAll(
			false,
			false,
			true,
			null,
			$customerID ?: null,
			model\Permission::HOSTS_READ,
			true
		);
		// Check the permissions (called in the destructor of a model)
		foreach ($satellites as $satellite) {
			$satellite->enforceWritePermission(
				model\Permission::HOSTS_WRITE
			);
		}
		$result = $this->writeIcingaConfig('satellite', array(
			'customerID' => $customerID
		));
		Application::unsetAllSatellitesChanged();
		return $this->handleExportResult($result, 'All satellites');
	}

	/**
	 * Hide master configuration export reminder
	 *
	 * @author		  Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns		  void
	 */
	protected function hideMessageAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_WRITE
		);
		Application::unsetHostChanged();
		return $this->getParametersForJSON();
	}

	/**
	 * Hide master configuration export reminder
	 *
	 * @author		  Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns		  void
	 */
	protected function hideSatelliteAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_WRITE
		);
		$id = $this->request->getInt('id');
		Application::unsetSatelliteChanged($id);
		return $this->getParametersForJSON();
	}

	/**
	 * Hide master configuration export reminder
	 *
	 * @author		  Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns		  void
	 */
	protected function hideAllSatellitesAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_WRITE
		);
		Application::unsetAllSatellitesChanged();
		return $this->getParametersForJSON();
	}

	/**
	 * Handles given result of an export
	 * 
	 * @param         stdClass $result  Result of an export
	 * @return        void
	 */
	protected function handleExportResult($result, $subject) {
		Application::getLogInstance()->debug(sprintf(
			'%s: Got result for export: %s',
			__METHOD__, var_export($result, true)
		));
		if ($result->hasInvalidConfiguration) {
			$this->addErrorMessage(sprintf(
				"%s %s:",
				tr("There was an error exporting"),
				strtolower($subject)
			));
			foreach ($result->verifyStdout as $message) {
				if (strpos(strtolower($message), 'error') === 0) {
					$this->addErrorMessage($message);
				}
			}
		}
		else {
			$this->addNotifyMessage(tr(sprintf(
				'%s has been successfully written',
				$subject
			)));
			if ($result->timeout) {
				$this->addNotifyMessage(tr(sprintf(
					'%s will be reloaded in %d seconds',
					$subject,
					$result->timeout
				)));
			}
			else {
				$this->addNotifyMessage(tr(sprintf(
					'%s has been reloaded',
					$subject
				)));
			}
		}
		return $this->getParametersForJSON();
	}

}
