<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\utils\PasswordGenerator;
use org\adfinis\framework\config\ConfManager;
use ch\monitoring\Application;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\view;

/**
 * Handle host page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostController extends HostBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_READ
		);

		$this->setHeaderTitle(tr("Hosts"));

		$this->tpl->assign('template', 'hosts/hosts.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		// No output is needed after saveServices (dialogue closes on save)
		if ($this->action == 'saveServices') {
			$this->outputAjaxPage(null);
			return;
		}

		$this->tpl->assign('template', 'hosts/hostsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Load data needed to display the form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['satellites'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(false, false, true),
			'id',
			'hostname'
		);

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(),
			'id',
			'hostname'
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		$this->select['commands'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CommandMapper::getInstance()->getAll()
		);


		$hostOSs = datamapper\HostOSMapper::getInstance()->getAll();

		$this->select['OSs'] = util\IteratorUtil::getKeyValueArray(
			$hostOSs
		);

		$this->select['versions'] = util\IteratorUtil::getKeyValueArray(
			$hostOSs->current()->getVersions()
		);

		$this->select['types'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostTypeMapper::getInstance()->getAll()
		);

		$this->parameters['domain'] = ConfManager::getVar('af', 'domain', 'default');
	}

	/**
	 * Edit a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Id is missing")
			);
		}

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);
		$this->parameters['host'] = $host;

		$this->assignFormVariables();
		$this->select['versions'] = util\IteratorUtil::getKeyValueArray(
			$host->getHostOS()->getVersions()
		);
	}

	/**
	 * Add a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addFormAction() {
		$host = new model\Host();
		$host->allowRead();

		$host->isEnabled      = true;
		$host->checkHostAlive = true;

		$this->parameters['host'] = $host;
		$this->assignFormVariables();
	}

	/**
	 * Show assigned services of a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function hostServicesAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(tr("Id is missing"));
		}

		// For access to the argument type constants
		$this->tpl->registerClass(
			"ServiceArgument",
			"\ch\monitoring\model\ServiceArgument"
		);

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceReadPermission(
			model\Permission::HOSTS_READ
		);

		$this->parameters['host'] = $host;

		$this->select['groups'] = util\IteratorUtil::getKeyValueArray(
			datamapper\ServiceGroupMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		$this->select['services'] = util\IteratorUtil::getKeyValueArray(
			datamapper\ServiceMapper::getInstance()->getAll(
				null,
				null,
				null,
				null,
				null,
				model\Permission::HOSTS_READ
			)
		);
	}

	/**
	 * Show a list of SLAs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function showSLAsAction() {
		$slas = datamapper\SLAMapper::getInstance()->getAll(model\Permission::HOSTS_READ);

		foreach ($slas as $sla) {
			$this->parameters['slas'][$sla->id]['sla'] = $sla;

			$timeframes = datamapper\TimeframeMapper::getInstance()->getAll($sla->id);

			foreach ($timeframes as $timeframe) {
				$this->parameters['slas'][$sla->id]['timeframes'][$timeframe->id]['timeframe'] = $timeframe;
				$timeframeRules = datamapper\TimeframeRuleMapper::getInstance()->getAll($timeframe->id);
				$this->parameters['slas'][$sla->id]['timeframes'][$timeframe->id]['rules'] = $timeframeRules;
			}
		}
	}

	/**
	 * Show the SLAs dropdown
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getSLAsAction() {
		$id = $this->request->getInt('customerId', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Customer id is missing")
			);
		}

		$customer = datamapper\CustomerMapper::getInstance()->getByID($id);
		$customer->enforceReadPermission(
			model\Permission::HOSTS_READ
		);

		$host = new model\Host();
		$host->allowRead();
		$host->customerId = $customer->id;

		$this->parameters['host'] = $host;

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);
	}

	/**
	 * Save a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Host id is missing")
			);
		}

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);

		$this->saveHost($host);
	}

	/**
	 * Add a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$host = new model\Host();
		$host->allowWrite(model\Permission::HOSTS_WRITE);

		$this->saveHost($host);
	}

	/**
	 * Add or save a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Host $host The host model to be saved
	 * @returns       void
	 */
	private function saveHost($host) {
		$oldSatelliteId = $host->satelliteId;
		$oldIsSatellite = $host->isSatellite;

		$host->customerId           = $this->request->getInt('customerId');
		$host->slaId                = $this->request->getInt('slaId') ?: null;
		$host->dependencyId         = $this->request->getInt('dependencyId') ?: null;
		$host->satelliteId          = $this->request->getInt('satelliteId') ?: null;
		$host->osId                 = $this->request->getInt('osId');
		$host->osVersionId          = $this->request->getInt('osVersionId') ?: null;
		$host->typeId               = $this->request->getInt('typeId') ?: null;
		$host->hostname             = $this->request->getString('hostname');
		$host->description          = $this->request->getString('description');
		$host->IPv4                 = $this->request->getString('IPv4') ?: null;
		$host->IPv6                 = $this->request->getString('IPv6') ?: null;
		$host->comment              = $this->request->getString('comment') ?: null;
		$host->checkHostAlive       = $this->request->keyExists('checkHostAlive');
		$host->isSatellite          = $this->request->keyExists('isSatellite');
		$host->isEnabled            = $this->request->keyExists('isEnabled');
		$host->inheritSla           = $this->request->getInt('slaId') == 0;
		$host->wiki                 = $this->request->getString('wiki') ?: null;

		if (isset($host->slaId)) {
			$sla = datamapper\SLAMapper::getInstance()->getByID(
				$host->slaId
			);
			$sla->enforceWritePermission(
				model\Permission::HOSTS_WRITE
			);
		}

		// A host depends on its satellite unless explicitly overridden
		if ($host->dependencyId === null && $host->satelliteId !== null) {
			$host->dependencyId = $host->satelliteId;
		}

		$isNew = !$host->hasIdentifier();

		$host->save();

		if ($isNew) {
			$this->setRandomPassword($host);
		}
		// If the host becomes passive, all services need to become passive. And vice versa.
		elseif ($oldSatelliteId != $host->satelliteId || $oldIsSatellite != $host->isSatellite) {
			$isPassive = $host->satelliteId != 0 || $host->isSatellite;

			$instances = $host->getHostServiceInstances();
			foreach ($instances as $instance) {
				$instance->allowWrite(model\Permission::HOSTS_WRITE);
				$instance->isPassive = $isPassive;
				$instance->save();
			}
		}

		Application::setHostChanged();
		$this->parameters['hosts'][] = $host;

		if ($host->isSatellite) {
			Application::setSatelliteChanged($host->id, $host->hostname);
		}

		if ($host->satelliteId !== null) {
			$satellite = $host->getSatellite();
			Application::setSatelliteChanged($satellite->id, $satellite->hostname);
		}
		// If the host was removed from a satellite, re-create
		// the satellites configuration
		else if($oldSatelliteId !== null) {
			$satellite = datamapper\HostMapper::getInstance()->getByID($oldSatelliteId);
			$satellite->allowWrite(model\Permission::HOSTS_WRITE);
			$this->writeIcingaConfig('satellite', array(
				'IDs' => array($satellite->id)
			));
		}
	}

	/**
	 * Delete a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(tr('Host id is missing'));
		}

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);
		$host->delete();
		Application::setHostChanged();
	}

	/**
	 * Shows all dependencies of a host
	 *
	 * @returns void
	 */
	protected function showDependenciesAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(tr('Host id is missing'));
		}

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);
		$this->parameters['host'] = $host;
	}

	/**
	 * Assigns the members of a service group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getGroupMembersAction() {
		$groupId = $this->request->getInt('groupId', request\REQUEST_GET);

		if (empty($groupId)) {
			throw new UserException(tr("Group id is missing"));
		}

		$group = datamapper\ServiceGroupMapper::getInstance()->getByID(
			$groupId
		);
		$group->enforceReadPermission(model\Permission::HOSTS_READ);

		$this->parameters['objectIds'] = $group->getMemberIds();
	}

	/**
	 * Assigns the host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getServicesListAction() {
		$hostId = $this->request->getInt('hostId', request\REQUEST_GET);

		if (empty($hostId)) {
			throw new UserException(tr("Host id is missing"));
		}

		$host = datamapper\HostMapper::getInstance()->getByID(
			$hostId
		);
		$host->enforceReadPermission(model\Permission::HOSTS_READ);

		$this->parameters['host'] = $host;
	}

	/**
	 * Batch save assigned services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveServicesAction() {
		$hostId          = $this->request->getInt('hostId');
		$serviceIds      = $this->request->getRaw('serviceId') ?: array();
		$instanceIds     = $this->request->getRaw('instanceId') ?: array();
		$names           = $this->request->getRaw('name') ?: array();
		$slaIds          = $this->request->getRaw('slaId') ?: array();
		$arguments       = $this->request->getRaw('argument') ?: array();
		$isEnabled       = $this->request->getRaw('isEnabled') ?: array();
		$isPassive       = $this->request->getRaw('isPassive') ?: array();
		$forAvailability = $this->request->getRaw('forAvailability') ?: array();

		if (empty($hostId)) {
			throw new UserException(tr("Host id is missing"));
		}

		$host = datamapper\HostMapper::getInstance()->getByID(
			$hostId
		);
		$host->enforceWritePermission(model\Permission::HOSTS_WRITE);

		Application::getDBInstance()->beginTransaction();

		$assignedInstances = $host->getHostServiceInstancesAsArray();
		foreach ($serviceIds as $serviceId) {
			// Either the service already is assigned to the host or it was newly assigned
			try {
				$hostService = datamapper\HostServiceMapper::getInstance()->getByHostAndService(
					$hostId,
					$serviceId
				);
			}
			catch(\OutOfBoundsException $e) {
				$hostService  = new model\HostService();

				$hostService->hostId    = $hostId;
				$hostService->serviceId = $serviceId;

				$hostService->getMapper()->save($hostService);
			}

			$hostService->enforceWritePermission(model\Permission::HOSTS_WRITE);

			$moreThanTwo = count($instanceIds[$serviceId]) >= 2;
			foreach ($instanceIds[$serviceId] as $instanceId => $new) {

				// When the instance was newly created, it must be ... created
				if ($new) {
					$instance                = new model\HostServiceInstance();
					$instance->hasTrouble    = false;
					$instance->hostServiceId = $hostService->id;

					$instanceId = sprintf(
						"new-%d",
						$instanceId
					);
				}
				// Otherwise, insist that it already exists
				else {
					try {
						$instance = datamapper\HostServiceInstanceMapper::getInstance()->getByID(
							$instanceId
						);

						// This instance shall not be deleted
						unset($assignedInstances[$instanceId]);

						$instanceId = sprintf(
							"old-%d",
							$instanceId
						);
					}
					catch (\OutOfBoundsException $e) {
						throw new UserException(
							tr("Instance no longer exists. Please reload the dialog.")
						);
					}
				}

				// If an exception is thrown, read permission must be granted
				$instance->allowRead();

				// An instance name may only be empty if there is no other instance for the same service
				if ($moreThanTwo && empty($names[$instanceId])) {
					throw new UserException(sprintf(
						tr("Instance names are mandatory if there is more than one instance (service '%s')"),
						$hostService->getService()->name
					));
				}

				// Save the new values (this is the same for existing and new instances)
				$instance->enforceWritePermission(model\Permission::HOSTS_WRITE);

				$instance->name            = $names[$instanceId] ?: null;
				$instance->slaId           = $slaIds[$instanceId] ?: null;
				$instance->isEnabled       = array_key_exists($instanceId, $isEnabled);
				$instance->isPassive       = array_key_exists($instanceId, $isPassive);
				$instance->forAvailability = array_key_exists($instanceId, $forAvailability);
				$instance->save();

				if (isset($instance->slaId)) {
					$sla = datamapper\SLAMapper::getInstance()->getByID(
						$instance->slaId
					);
					$sla->enforceWritePermission(
						model\Permission::HOSTS_WRITE
					);
				}

				// If argument values are supplied, apply them
				if (array_key_exists($instanceId, $arguments)) {
					datamapper\HostServiceArgumentMapper::getInstance()->deleteByInstanceId(
						$instance->id
					);

					// Get all arguments which can be overwritten
					$variableArguments = $hostService->getService()->getArgumentsAsArray(
						model\ServiceArgument::TYPE_VARIABLE
					);

					foreach ($arguments[$instanceId] as $argumentId => $argumentValue) {
						if (!array_key_exists($argumentId, $variableArguments)) {
							throw new UserException(
								tr("Cannot set a value for arguments of fixed type")
							);
						}

						if (!is_numeric($argumentValue) && strval($argumentValue) === '') {
							continue;
						}

						$hostServiceArgument = new model\HostServiceArgument();
						$hostServiceArgument->allowWrite(model\Permission::HOSTS_WRITE);

						$hostServiceArgument->serviceArgumentId = $argumentId;
						$hostServiceArgument->instanceId        = $instance->id;
						$hostServiceArgument->assignedValue     = $argumentValue;

						$hostServiceArgument->save();
					}
				}
			}
		}

		// Every instance which no longer is assigned must be deleted
		foreach ($assignedInstances as $instanceId => $name) {
			$instance = datamapper\HostServiceInstanceMapper::getInstance()->getByID(
				$instanceId
			);
			$instance->enforceReadPermission(
				model\Permission::HOSTS_WRITE
			);
			datamapper\HostServiceInstanceMapper::getInstance()->delete(
				array('id' => $instanceId)
			);
		}

		if ($this->hasErrorMessages()) {
			Application::getDBInstance()->rollBack();
		}
		else {
			Application::getDBInstance()->commit();
			Application::setHostChanged();
			if ($host->isSatellite) {
				Application::setSatelliteChanged($host->id, $host->hostname);
			}
			if ($host->satelliteId !== null) {
				$satellite = $host->getSatellite();
				Application::setSatelliteChanged($satellite->id, $satellite->hostname);
            }

            // Cleanup hostservices - every service with 0 instances must be unassigned
            $rpcClient = Application::getRPCClientInstance(
                ConfManager::getVar('sm','RPC', 'lcAppName'),
                ConfManager::getVar('sm','RPC', 'lcQueues')
            );

            $resultId = $rpcClient->putRequest(
                'symonitoring_rpcserver.cleanup_hostservices',
                array('host_id' => $host->id)
            );
            $result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

            if (isset($result->exception)) {
                throw new UserException($result->exception);
            }
		}
		// This way the user doesn't have to manually export the nrpe configuration
		$this->writeIcingaConfig('NRPE', array('IDs' => array($host->id)));
	}

	/**
	 * Write the given icinga config kind
	 *
	 * @param       string $kind
	 * @param       array  $IDs
	 * @return      object
	 */
	private function writeIcingaConfig($kind, $data = null) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm', 'RPC', 'wqAppName'),
			ConfManager::getVar('sm', 'RPC', 'wqQueues')
		);
		$timeout   = ConfManager::getVar('sm', 'RPC', 'icinga_timeout');
		$type      = 'writeIcingaConfig';
		$params    = array(
			'kind' => $kind
		);
		if (is_array($data)) {
			$params += $data;
		}
		$resultID = $rpcClient->putRequest($type, $params);
		$result   = $rpcClient->getResult($resultID, $timeout);
        if (isset($result->exception)) {
			Application::getLogInstance()->warn(sprintf(
				'There was an exception writing Icinga configuration: %s',
				$result->exception
            ));
			throw new UserException($result->exception, $result->state);
		}

		if ($result->hasInvalidConfiguration) {
			$this->addErrorMessage(
				tr("There was an error exporting the NRPE configuration:")
			);
			foreach ($result->verifyStdout as $message) {
				if (strpos(strtolower($message), 'error') === 0) {
					$this->addErrorMessage($message);
				}
			}
		}
	}

	/**
	 * Sets a new host password
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function setPasswordAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr("Host id is missing"));
		}

		$host = datamapper\HostMapper::getInstance()->getByID($id);
		$host->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);

		$this->setRandomPassword($host);
		$this->parameters['host'] = $host;
	}

	/**
	 * Set a new random password and export new config
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Host $host
	 * @returns       void
	 */
	private function setRandomPassword($host) {
		$host->password = PasswordGenerator::generatePassword(32);
		$host->save();

		$this->writeIcingaConfig('NRPE', array('IDs' => array($host->id)));

		if ($host->isSatellite) {
			$this->writeIcingaConfig('satellite', array('IDs' => array($host->id)));
		}
	}

	/**
	 * Get a new service panel row
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getRowAction() {
		$hostId = $this->request->getInt('hostId', request\REQUEST_GET);
		$host   = datamapper\HostMapper::getInstance()->getByID($hostId);
		$host->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$serviceId = $this->request->getInt('serviceId', request\REQUEST_GET);
		$service   = datamapper\ServiceMapper::getInstance()->getByID($serviceId);
		$service->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$tempId = $this->request->getInt('tempId', request\REQUEST_GET);
		$tempId += 1;

		$this->parameters['host'] = $host;
		$this->parameters['service'] = $service;
		$this->parameters['tempId'] = $tempId;

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		// For access to the argument type constants
		$this->tpl->registerClass(
			"ServiceArgument",
			"\ch\monitoring\model\ServiceArgument"
		);
	}

	/**
	 * Get a new service panel
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getPanelAction() {
		$hostId = $this->request->getInt('hostId', request\REQUEST_GET);
		$host   = datamapper\HostMapper::getInstance()->getByID($hostId);
		$host->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$serviceId = $this->request->getInt('serviceId', request\REQUEST_GET);
		$service   = datamapper\ServiceMapper::getInstance()->getByID($serviceId);
		$service->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$tempId = $this->request->getInt('tempId', request\REQUEST_GET);
		$tempId += 1;

		$this->parameters['host'] = $host;
		$this->parameters['service'] = $service;
		$this->parameters['tempId'] = $tempId;

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);

		// For access to the argument type constants
		$this->tpl->registerClass(
			"ServiceArgument",
			"\ch\monitoring\model\ServiceArgument"
		);
	}

	/**
	 * Get downtime form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function downtimeFormAction() {
		$hostId = $this->request->getInt('hostId', request\REQUEST_GET);
		$host = datamapper\HostMapper::getInstance()->getByID($hostId);
		$host->enforceReadPermission(model\Permission::DOWNTIMES_WRITE);

		$this->parameters['host'] = $host;
	}

	/**
	 * Get the versions of an operating system
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getVersionsAction() {
		$osId = $this->request->getInt('osId', request\REQUEST_GET);
		$hostId = $this->request->getInt('hostId', request\REQUEST_GET);

		if (empty($osId)) {
			throw new UserException(tr("OS id is missing"));
		}


		$system = datamapper\HostOSMapper::getInstance()->getByID($osId);
		$system->allowRead();

		if (empty($hostId)) {
			$this->parameters['host'] = new model\Host();
			$this->parameters['host']->allowRead();
		}
		else {
			$host = datamapper\HostMapper::getInstance()->getByID($hostId);
			$host->enforceReadPermission(model\Permission::HOSTS_WRITE);
			$this->parameters['host'] = $host;
		}

		$this->select['versions'] = util\IteratorUtil::getKeyValueArray(
			$system->getVersions()
		);
	}

	/**
	 * Handles DataTable requests
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        string json
	 */
	public function dataTableAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_READ
		);
		$settings = $_GET;
		unset($settings['action']);

		$hosts = datamapper\HostMapper::getInstance()->getAllForDataTable($settings);

		header('Content-Type: application/json; charset=utf-8');
		print json_encode($hosts->toDataTable());
		exit;
	}
}
