<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\exceptions\UserException;

/**
 * Handle host export page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostExportController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::HOSTS_WRITE
		);

		$this->setHeaderTitle(tr("Export hosts"));

		$this->select['satellites'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(false, false, true),
			'id',
			'hostname'
		);

		$this->select['nrpeHosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(false, true),
			'id',
			'hostname'
		);

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(),
			'id',
			'hostname'
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::HOSTS_WRITE
			)
		);

		$this->tpl->assign('template', 'hosts/export.tpl');

		//$this->addScriptFile('js/ajaxHosts.js', self::SCRIPT_POSITION_TOP);
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'hosts/exportAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Assigns the satellites
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getSatellitesAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_READ
		);

		$customerId = $this->request->getInt('customerId', request\REQUEST_GET);

		$this->select['satellites'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				true,
				null,
				$customerId ?: null
			),
			'id',
			'hostname'
		);
	}

	/**
	 * Assigns the nrpe hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getNrpeHostsAction() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_READ
		);

		$customerId = $this->request->getInt('customerId', request\REQUEST_GET);

		$this->select['nrpeHosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				true,
				false,
				null,
				$customerId ?: null
			),
			'id',
			'hostname'
		);
	}

	/**
	 * Show hosts behind a satellite
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function showSatelliteHostsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (!empty($id)) {
			$this->parameters['host'] = datamapper\HostMapper::getInstance()->getByID($id);
			$this->parameters['host']->enforceReadPermission(
				model\Permission::HOSTS_READ
			);
			$this->parameters['domain'] = ConfManager::getVar('af', 'domain', 'default');
		}
	}

	/**
	 * Provides information about a satellite
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function satelliteInformationAction() {
		$id = $this->request->getInt('satelliteId', request\REQUEST_GET);

		if (!empty($id)) {
			try {
				$this->parameters['host'] = datamapper\HostMapper::getInstance()->getByID($id);
				$this->parameters['host']->enforceReadPermission(
					model\Permission::HOSTS_READ
				);
				$this->parameters['domain'] = ConfManager::getVar('af', 'domain', 'default');
			}
			catch (\OutOfBoundsException $e) {
				throw new UserException(
					tr('Satellite not found')
				);
			}
		}
	}

	/**
	 * Provides information about an nrpe host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function nrpeHostInformationAction() {
		$id = $this->request->getInt('nrpeHostId', request\REQUEST_GET);

		if (!empty($id)) {
			try {
				$this->parameters['host'] = datamapper\HostMapper::getInstance()->getByID($id);
				$this->parameters['host']->enforceReadPermission(
					model\Permission::HOSTS_READ
				);
				$this->parameters['domain'] = ConfManager::getVar('af', 'domain', 'default');
			}
			catch (\OutOfBoundsException $e) {
				throw new UserException(
					tr('Host not found')
				);
			}
		}
	}
}
