<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;

/**
 * Handle host group page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostGroupController extends HostBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::HOSTS_READ
		);

		$this->setHeaderTitle(tr("Host groups"));
		$this->parameters['allGroups'] = datamapper\HostGroupMapper::getInstance()->getAll(model\Permission::HOSTS_READ);
		$this->tpl->assign('template', 'hosts/groups.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'hosts/groupsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Show the form to add a host group
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$this->parameters['group'] = new model\HostGroup();
		$this->parameters['group']->allowRead();
		$this->assignFormVariables();
	}

	/**
	 * Show the form to edit a host group
	 *
	 * @throws        UserException
	 * @returns       void
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Host group id is missing")
			);
		}

		$this->parameters['group'] = datamapper\HostGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$this->assignFormVariables();
	}

	/**
	 * Assigns variables for the host group form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::HOSTS_READ
			)
		);
	}

	/**
	 * Create a new host group and invoke the save
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addAction() {
		$hostGroup = new model\HostGroup();
		$hostGroup->allowWrite(model\Permission::HOSTS_WRITE);

		$this->saveHostGroup($hostGroup);
	}

	/**
	 * Save an existing host group
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		$hostGroup = datamapper\HostGroupMapper::getInstance()->getByID($id);
		$hostGroup->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);

		$this->saveHostGroup($hostGroup);
	}

	/**
	 * Save the data of a created or edited group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         model\HostGroup $hostGroup
	 * @returns       void
	 */
	private function saveHostGroup($hostGroup) {
		$hostGroup->name        = $this->request->getString('name');
		$hostGroup->description = $this->request->getString('description');
		$hostGroup->customerId  = $this->request->getInt('customerId');
		$hostGroup->save();

		$this->parameters['group'] = $hostGroup;
		$this->parameters['groups'][] = $hostGroup;
		Application::setHostChanged();
	}

	/**
	 * Delete a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Group id is missing")
			);
		}

		$hostGroup = datamapper\HostGroupMapper::getInstance()->getByID($id);
		$hostGroup->enforceWritePermission(
			model\Permission::HOSTS_WRITE
		);
		$hostGroup->delete();
		Application::setHostChanged();
	}

	/**
	 * Add a host to a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addHostAction() {
		$hostGroupId = $this->request->getInt('hostGroupId');
		$hostId = $this->request->getInt('groupHostId');

		$hostGroup = datamapper\HostGroupMapper::getInstance()->getByID($hostGroupId);
		$hostGroup->enforceWritePermission(model\Permission::HOSTS_WRITE);

		$host = datamapper\HostMapper::getInstance()->getByID($hostId);
		$host->enforceReadPermission(model\Permission::HOSTS_WRITE);

		$customerIds = datamapper\CustomerMapper::getInstance()->getSubCustomers($hostGroup->customerId);
		$customerIds[] = $hostGroup->customerId;

		if (!in_array($host->customerId,  $customerIds)) {
			throw new UserException(
				tr('Host must belong to the same customer as the host group.')
			);
		}

		try {
			$hostGroup->assign($host->id);
		}
		catch(\Zend_Db_Statement_Exception $e) {
			if($e->getCode() == 23000) {
				throw new UserException(
					tr('This host already is assigned')
				);
			}
			else {
				throw $e;
			}
		}

		$this->parameters['group'] = $hostGroup;
		Application::setHostChanged();
	}

	/**
	 * Show the group hosts form 
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function groupHostsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$this->parameters['group'] = datamapper\HostGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::HOSTS_WRITE
		);

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				null,
				false,
				false,
				null,
				$this->parameters['group']->customerId
			),
			'id',
			'hostname'
		);

	}

	/**
	 * Remove a group member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteHostAction() {
		$hostGroupId = $this->request->getInt('hostGroupId');
		$hostId = $this->request->getInt('hostId');

		$hostGroup = datamapper\HostGroupMapper::getInstance()->getByID($hostGroupId);
		$hostGroup->enforceWritePermission(model\Permission::HOSTS_WRITE);
		$hostGroup->unassign($hostId);

		$this->parameters['group'] = $hostGroup;
		Application::setHostChanged();
	}
}
