<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Handle hostOS page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostOSController extends ProtectedController {

	private $dirManager;

	public function __construct($request, $urlMatch) {
		parent::__construct($request, $urlMatch);
		$dirPath =  ConfManager::getVar('sm', 'command', 'path', 'fs');
		$this->dirManager = new util\DirectoryManager(
			$dirPath,
			[$this, "addNotifyMessage"]
		);
	}

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::HOSTS_MASTER
		);

		$this->setHeaderTitle(tr("Operating systems"));
		$this->parameters['systems'] = datamapper\HostOSMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'hosts/hostOS.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();
		$this->tpl->assign('template', 'hosts/hostOSAjax.tpl');

		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Create a new hostOS and load the form
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$hostOS = new model\HostOS();
		$hostOS->allowRead();
		$hostOS->image = 'base/default.png';
		$this->parameters['system'] = $hostOS;
	}

	/**
	 * Load the hostOS that is to edit and load the form
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$hostOS = datamapper\HostOSMapper::getInstance()->getByID($id);
		$hostOS->enforceReadPermission(
			model\Permission::HOSTS_MASTER
		);

		$this->parameters['system'] = $hostOS;
	}

	/**
	 * Add a new HostOS
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$hostOS = new model\HostOS();
		$hostOS->allowWrite(model\Permission::HOSTS_MASTER);

		$this->saveHostOS($hostOS);
		$dirPath = $this->dirManager->createDirectoryPath($hostOS->name);
		$this->dirManager->makeDir($dirPath);
	}

	/**
	 * After editing, save the HostOS
	 *
	 * @return        void
	 */
	protected function saveAction()
	{
		$id = $this->request->getInt('id', request\REQUEST_POST);
		$hostOS = datamapper\HostOSMapper::getInstance()->getByID($id);
		$hostOS->enforceWritePermission(model\Permission::HOSTS_MASTER);

		$oldName = $hostOS->name;
		$this->saveHostOS($hostOS);
		if ($hostOS->name != $oldName) {
			$newDirPath = $this->dirManager->createDirectoryPath(
				$this->request->getString('name')
			);

			if (isset($oldName)) {
				$oldDirPath = $this->dirManager->createDirectoryPath($oldName);

				if (file_exists($oldDirPath)) {
					$this->dirManager->moveDir($oldDirPath, $newDirPath);
				}
				else {
					$this->dirManager->makeDir($newDirPath);
				}
			}
			else {
				$this->dirManager->makeDir($newDirPath);
			}
		}
	}

	/**
	 * Save a HostOS
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\HostOS $hostOS
	 * @returns       void
	 */
	private function saveHostOS($hostOS) {
		$hostOS->name         = $this->request->getString('name');
		$hostOS->image        = $this->request->getString('image');

		$hostOS->save();
		$this->parameters['system'] = $hostOS;
	}

	/**
	 * Delete a HostOS
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("HostOS id is missing")
			);
		}

		$hostOS = datamapper\HostOSMapper::getInstance()->getByID($id);
		$hostOS->enforceWritePermission(model\Permission::HOSTS_MASTER);

		try {
			$hostOS->delete();
			$dirPath = $this->dirManager->createDirectoryPath($hostOS->name);
			$this->dirManager->removeDir($dirPath);
		}catch(\Zend_Db_Statement_Exception $e) {
			throw new UserException(
				tr("The HostOS cannot be deleted. Please verify that no host is using this HostOS any longer.")
			);
		}
	}

	/**
	 * Show the versions form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function versionsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("HostOS id is missing")
			);
		}

		$system = datamapper\HostOSMapper::getInstance()->getByID($id);
		$system->enforceReadPermission(model\Permission::HOSTS_MASTER);

		$this->parameters['architectures'] = datamapper\HostOSArchitectureMapper::getInstance()->getAll();
		$this->parameters['system']        = $system;
	}

	/**
	 * Add a version
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addVersionAction() {
		$osId                 = $this->request->getInt('osId');
		$name                 = $this->request->getString('name');
		$shortName            = $this->request->getString('shortName');
		$architectureId       = $this->request->getString('architecture');

		if (empty($osId)) {
			throw new UserException(
				tr("HostOS id is missing")
			);
		}

		if (empty($name)) {
			throw new UserException(
				tr("The name may not be empty")
			);
		}

		$osVersion = new model\HostOSVersion();
		$osVersion->osId                 = $osId;
		$osVersion->name                 = $name;
		$osVersion->shortName            = $shortName;
		$osVersion->architectureId       = $architectureId;

		$osVersion->enforceWritePermission(model\Permission::HOSTS_MASTER);
		$osVersion->save();

		$dirPath = $this->dirManager->createDirectoryPath($osVersion->getHostOS()->name, $shortName);
		$this->dirManager->makeDir($dirPath);

		$this->parameters['system'] = $osVersion->getHostOS();
		$this->parameters['architectures'] = datamapper\HostOSArchitectureMapper::getInstance()->getAll();
	}

	/**
	 * Delete a version
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteVersionAction() {
		$versionId = $this->request->getInt('versionId');

		if (empty($versionId)) {
			throw new UserException(
				tr("Version id is missing")
			);
		}

		$osVersion = datamapper\HostOSVersionMapper::getInstance()->getByID($versionId);
		$osVersion->enforceWritePermission(model\Permission::HOSTS_MASTER);

		$system = $osVersion->getHostOS();
		try {
			$osVersion->delete();
		}
		catch(\Exception $e) {
			throw new UserException(sprintf(
				"%s: %s",
				tr("There may be command guidelines for this OS version
					or hosts using this OS version.
					Delete/unassign those first to delete this OS version."),
				$e->getMessage()
			));
		}
		$dirPath = $this->dirManager->createDirectoryPath($osVersion->getHostOS()->name, $osVersion->shortName);
		$this->dirManager->removeDir($dirPath);
		$this->parameters['system'] = $system;
	}

	/**
	 * Save a version
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function saveVersionAction() {
		$versionIds               = $this->request->getRaw('versionId');
		$names                    = $this->request->getRaw('name');
		$shortNames               = $this->request->getRaw('shortName');
		$architectures            = $this->request->getRaw('architecture');

		Application::getDbInstance()->beginTransaction();
		for ($i = 0; $i < count($versionIds); $i++) {
			$versionId            = $versionIds[$i];
			$name                 = $names[$i];
			$shortName            = $shortNames[$i];
			$architecture         = $architectures[$i];

			if (empty($versionId)) {
				throw new UserException(
					tr("Version id is missing")
				);
			}

			if (empty($name)) {
				throw new UserException(
					tr("Name may not be empty")
				);
			}

			$osVersion = datamapper\HostOSVersionMapper::getInstance()->getByID($versionId);
			$osVersion->enforceWritePermission(model\Permission::HOSTS_MASTER);

			$oldShortName                    = $osVersion->shortName;
			$osVersion->name                 = $name;
			$osVersion->shortName            = $shortName;
			$osVersion->architectureId       = $architecture;
			$osVersion->save();

			//move directory if shortname has changed
			if ($oldShortName != $shortName) {
				$hostOSName = $osVersion->getHostOS()->name;

				$oldDirPath = $this->dirManager->createDirectoryPath($hostOSName, $oldShortName);
				$newDirPath = $this->dirManager->createDirectoryPath($hostOSName, $shortName);

				if (isset($oldShortName) && file_exists($oldDirPath)) {
					$this->dirManager->moveDir($oldDirPath, $newDirPath);
				}
				else {
					$this->dirManager->makeDir($newDirPath);
				}
			}
		}
		Application::getDbInstance()->commit();
	}
}
