<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\exceptions\ForbiddenException;
use ch\monitoring\util;
use ch\monitoring\Application;
use ch\monitoring\datamapper;
use ch\monitoring\model;

/**
 * Handle login page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportController extends PublicController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(model\Permission::IMPORT);

		header('Content-Type: text/html; charset=utf-8');

		$this->setHeaderTitle(tr("Import"));

		$this->tpl->assign('template', 'import/import.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->parameters['files'] = datamapper\ImportFileMapper::getInstance()->getAll();
		$this->parameters['defaultCustomerId'] = datamapper\CustomerMapper::getInstance()->getDefaultCustomer(
			model\Permission::IMPORT
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::IMPORT
			)
		);

		$this->select['operatingSystems'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostOSMapper::getInstance()->getAll()
		);

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		$this->session->user->enforcePermission(model\Permission::IMPORT);

		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Process upload requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	protected function uploadAction() {
		$parser = new util\NagiosParser($_FILES['file']['tmp_name']);

		$this->parameters['file'] = $parser->save();
	}

	/**
	 * Import overview
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	protected function getOverviewAction() {
		$this->parameters['hosts'] = datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			null,
			null,
			true
		);

		$this->parameters['services'] = datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			true,
			null,
			true,
			null,
			false
		);

		$this->parameters['commands'] = datamapper\ImportCommandMapper::getInstance()->getAll();
		$this->parameters['hostgroups'] = datamapper\ImportHostgroupMapper::getInstance()->getAll();

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::IMPORT
			)
		);

		$this->select['operatingSystems'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostOSMapper::getInstance()->getAll()
		);

		$this->tpl->assign('template', 'import/overview.tpl');
	}

	/**
	 * Save import settings
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	protected function saveSettingsAction() {
		$customerId = $this->request->getInt('defaultCustomerId');
		$osId       = $this->request->getInt('defaultOsId');

		datamapper\ImportHostMapper::getInstance()->setDefaultCustomer(
			$customerId
		);

		datamapper\ImportHostMapper::getInstance()->setDefaultOS(
			$osId
		);

		// Resolve all dependencies
		util\NagiosParser::resolve();
	}

	/**
	 * Conflict overview
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	protected function getConflictsAction() {
		$this->parameters['hosts'] = datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			null,
			true
		);

		$this->parameters['services'] = datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			true,
			true
		);

		$this->parameters['commands'] = datamapper\ImportCommandMapper::getInstance()->getAll(
			null,
			true
		);

		$this->parameters['hostgroups'] = datamapper\ImportHostgroupMapper::getInstance()->getAll(
			null,
			true
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::IMPORT
			)
		);

		$this->select['operatingSystems'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostOSMapper::getInstance()->getAll()
		);

		$this->tpl->assign('template', 'import/conflicts.tpl');
	}

	/**
	 * Save new names for conflicting objects
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function saveConflictsAction() {
		$hostNames          = $this->request->getRaw('hostName')          ?: array();
		$serviceNames       = $this->request->getRaw('serviceName')       ?: array();
		$existingServices   = $this->request->getRaw('existingService')   ?: array();
		$commandNames       = $this->request->getRaw('commandName')       ?: array();
		$existingCommands   = $this->request->getRaw('existingCommand')   ?: array();
		$hostgroupNames     = $this->request->getRaw('hostgroupName')     ?: array();
		$existingHostgroups = $this->request->getRaw('existingHostgroup') ?: array();

		foreach ($hostNames as $hostId => $hostName) {
			$host = datamapper\ImportHostMapper::getInstance()
					->getByID($hostId)->allowWrite(model\Permission::IMPORT);
			$host->rename(trim($hostName));
		}

		foreach ($serviceNames as $serviceOldName => $serviceNewName) {
			datamapper\ImportServiceMapper::getInstance()->rename(
				$serviceOldName,
				trim($serviceNewName)
			);
		}

		foreach ($existingServices as $name => $serviceId) {
			datamapper\ImportServiceMapper::getInstance()->updateServiceId(
				$name,
				$serviceId
			);
		}

		foreach ($commandNames as $commandId => $commandName) {
			$command = datamapper\ImportCommandMapper::getInstance()
					->getByID($commandId)->allowWrite(model\Permission::IMPORT);
			$command->rename(trim($commandName));
		}

		foreach ($existingCommands as $importCommandId => $commandId) {
			$command = datamapper\ImportCommandMapper::getInstance()
					->getByID($importCommandId)->allowWrite(model\Permission::IMPORT);
			$command->commandId = $commandId;
			$command->save();
		}

		foreach ($hostgroupNames as $hostgroupId => $hostgroupName) {
			$hostgroup = datamapper\ImportHostgroupMapper::getInstance()
					->getByID($hostgroupId)->allowWrite(model\Permission::IMPORT);
			$hostgroup->hostgroup_name = trim($hostgroupName);
			$hostgroup->save();
		}

		foreach ($existingHostgroups as $importHostgroupId => $hostgroupId) {
			$hostgroup = datamapper\ImportHostgroupMapper::getInstance()
					->getByID($importHostgroupId)->allowWrite(model\Permission::IMPORT);
			$hostgroup->hostgroupId = $hostgroupId;
			$hostgroup->save();
		}

		$this->parameters['hosts'] = datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			null,
			true
		)->toArray();

		$this->parameters['services'] = datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			true,
			true,
			false
		)->toArray();

		$this->parameters['commands'] = datamapper\ImportCommandMapper::getInstance()->getAll(
			null,
			true
		)->toArray();

		$this->parameters['hostgroups'] = datamapper\ImportHostgroupMapper::getInstance()->getAll(
			null,
			true
		)->toArray();
	}

	/**
	 * Delete an uploaded file
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteFileAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr('File id is missing'));
		}

		$file = datamapper\ImportFileMapper::getInstance()->getByID($id);
		$file->enforceWritePermission(
			model\Permission::IMPORT
		);
		$file->delete();
	}

	/**
	 * Delete a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteHostAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr('Host id is missing'));
		}

		$host = datamapper\ImportHostMapper::getInstance()->getByID($id);
		$host->enforceWritePermission(
			model\Permission::IMPORT
		);
		$host->delete();

		datamapper\ImportServiceMapper::getInstance()->deleteByHost(
			$host->host_name
		);
	}

	/**
	 * Delete a service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteServiceAction() {
		$name = $this->request->getString('id');

		if (empty($name)) {
			throw new UserException(tr('Service name is missing'));
		}

		datamapper\ImportServiceMapper::getInstance()->deleteByName($name);
	}

	/**
	 * Delete a command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteCommandAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr('Command id is missing'));
		}

		$command = datamapper\ImportCommandMapper::getInstance()->getByID($id);
		$command->enforceWritePermission(
			model\Permission::IMPORT
		);
		$command->delete();

		datamapper\ImportServiceMapper::getInstance()->deleteByCommand(
			$command->command_name
		);
	}

	/**
	 * Delete a host group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function deleteHostgroupAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr('Host group id is missing'));
		}

		$hostgroup = datamapper\ImportHostgroupMapper::getInstance()->getByID($id);
		$hostgroup->enforceWritePermission(
			model\Permission::IMPORT
		);
		$hostgroup->delete();
	}

	/**
	 * Cancel the whole import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function cancelAction() {
		datamapper\ImportFileMapper::getInstance()->cancel();
	}

	/**
	 * Finish the import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function finishAction() {
		$hosts    = $this->request->getRaw('hosts');
		$services = $this->request->getRaw('services');
		$commands = $this->request->getRaw('commands');

		// We need to save the values now because the actual import might fail
		foreach ($hosts as $hostId => $values) {
			$host = datamapper\ImportHostMapper::getInstance()->getByID(
				$hostId
			);
			$host->allowWrite(model\Permission::IMPORT);
			$host->customerId = $values['customerId'];
			$host->osId       = $values['osId'];
			$host->save();
		}

		foreach ($services as $serviceName => $values) {
			foreach ($values as $key => $value) {
				datamapper\ImportServiceMapper::getInstance()->updateByName(
					$serviceName,
					$key,
					$value
				);
			}
		}

		foreach ($commands as $commandId => $values) {
			$command = datamapper\ImportCommandMapper::getInstance()->getByID(
				$commandId
			);
			$command->allowWrite(model\Permission::IMPORT);
			$command->command_line = $values['command_line'];
			$command->save();
		}

		Application::getDbInstance()->beginTransaction();

		$this->finishCommands();
		$this->finishHosts();
		$this->finishServices();
		$this->finishHostgroups();

		// If everything went well, the import can be wiped.
		datamapper\ImportFileMapper::getInstance()->cancel();

		Application::getDbInstance()->commit();
	}

	/**
	 * Finish commands import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function finishCommands() {
		$commands = datamapper\ImportCommandMapper::getInstance()->getAll(
			null,
			null,
			true
		);

		foreach ($commands as $rawCommand) {
			if ($rawCommand->commandId !== null) {
				continue;
			}

			$command = new model\Command();
			$command->allowWrite(model\Permission::IMPORT);
			$command->name        = $rawCommand->command_name;
			$command->description = tr('Imported command');
			$command->commandline = $rawCommand->command_line;
			$command->v6Capable   = false;
			$command->save();
		}
	}

	/**
	 * Finish hosts import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by denis:   Better handling of addresses with hostname instead of IP address
	 * @returns       void
	 */
	private function finishHosts() {
		$hosts = datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			null,
			null,
			true
		);

		// Matches IPv4 addresses
		$regex = '/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/';

		foreach ($hosts as $rawHost) {
			$host = new model\Host();
			$host->allowWrite(model\Permission::IMPORT);
			$host->customerId     = $rawHost->customerId;
			$host->slaId          = null;
			$host->dependencyId   = null;
			$host->satelliteId    = null;
			$host->osId           = $rawHost->osId;
			$host->osVersionId    = null;
			$host->typeId         = $rawHost->typeId;
			$host->inheritSla     = true;
			$host->checkHostAlive = true;
			$host->hostname       = $rawHost->host_name;
			$host->description    = tr('Imported host');

			if (!preg_match($regex, $rawHost->address)) {
				$host->IPv4 = "127.0.0.1";
			}
			else {
				$host->IPv4 = $rawHost->address;
			}

			$host->IPv6           = null;
			$host->isSatellite    = false;
			$host->isEnabled      = true;
			$host->save();
		}
	}

	/**
	 * Finish services import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function finishServices() {
		$services = datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			false,
			null,
			true,
			null,
			false
		);

		foreach ($services as $rawService) {
			try {
				$service = datamapper\ServiceMapper::getInstance()->getByName(
					$rawService->service_description
				)->allowRead();
			}
			catch (\OutOfBoundsException $e) {
				$service = $this->createService($rawService);
			}

			try {
				$host = datamapper\HostMapper::getInstance()->getByName(
					$rawService->host_name
				)->allowRead();
			}
			catch (\OutOfBoundsException $e) {
				throw new \RuntimeException(sprintf(
					tr('Host %s for service %s not found.'),
					$rawService->host_name,
					$rawService->service_description
				));
			}

			$hostService = new model\HostService();
			$hostService->allowWrite(model\Permission::IMPORT);
			$hostService->hostId    = $host->id;
			$hostService->serviceId = $service->id;
			$hostService->save();

			$hostServiceInstance = new model\HostServiceInstance();
			$hostServiceInstance->allowWrite(model\Permission::IMPORT);
			$hostServiceInstance->hostServiceId   = $hostService->id;
			$hostServiceInstance->slaId           = null;
			$hostServiceInstance->name            = null;
			$hostServiceInstance->isPassive       = false;
			$hostServiceInstance->forAvailability = false;
			$hostServiceInstance->hasTrouble      = false;
			$hostServiceInstance->isEnabled       = true;
			$hostServiceInstance->save();

			$rawArguments      = $rawService->getArguments();
			$existingArguments = $service->getArguments();

			// Add instance arguments if they differ from the default ones
			foreach ($existingArguments as $argument) {
				if ($rawArguments[$argument->position - 1] != $argument->value) {
					$hostServiceArgument = new model\HostServiceArgument();
					$hostServiceArgument->allowWrite(model\Permission::IMPORT);
					$hostServiceArgument->instanceId = $hostServiceInstance->id;

					foreach ($existingArguments as $argument) {
						$value = $rawArguments[$argument->position - 1];
						$value = preg_replace("/'/", '', $value);
						if (!is_numeric($value) && strval($value) === '') {
							continue;
						}

						$hostServiceArgument->id = null;
						$hostServiceArgument->serviceArgumentId = $argument->id;
						$hostServiceArgument->assignedValue = $value;
						$hostServiceArgument->save();
					}

					break;
				}
			}
		}
	}

	/**
	 * Create a service from raw data
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\IncludeService $rawService The raw service object
	 * @returns       model\Service
	 */
	private function createService($rawService) {
		try {
			$command = datamapper\CommandMapper::getInstance()->getByName(
				$rawService->getCommandName()
			)->allowRead();
		}
		catch (\OutOfBoundsException $e) {
			throw new UserException(sprintf(
				tr('Did not find command %s for service %s.'),
				$rawService->getCommandName(),
				$rawService->service_description
			));
		}

		$service = new model\Service();
		$service->allowWrite(model\Permission::IMPORT);
		$service->commandId        = $command->id;
		$service->customerId       = null;
		$service->name             = $rawService->service_description;
		$service->description      = tr('Imported service');
		$service->maxCheckAttempts = $rawService->max_check_attempts;
		$service->checkInterval    = $rawService->check_interval;
		$service->retryInterval    = $rawService->retry_interval;
		$service->freshness        = 0;
		$service->isNrpe           = false;
		$service->save();

		$serviceArgument = new model\ServiceArgument();
		$serviceArgument->allowWrite(model\Permission::IMPORT);
		$serviceArgument->serviceId = $service->id;

		$i = 1;
		foreach ($rawService->getArguments() as $argument) {
			$serviceArgument->id = null;
			$serviceArgument->position = $i++;
			$serviceArgument->option = null;
			$serviceArgument->value = preg_replace("/'/", '', $argument);
			$serviceArgument->description = null;
			$serviceArgument->type = model\ServiceArgument::TYPE_VARIABLE;
			$serviceArgument->save();
		}

		return $service;
	}

	/**
	 * Finish host groups import
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function finishHostgroups() {
		$hostgroups = datamapper\ImportHostgroupMapper::getInstance()->getAll(
			null,
			null,
			true
		);

		foreach ($hostgroups as $rawHostgroup) {
			if ($rawHostgroup->hostgroupId !== null) {
				$notFound = false;
				try {
					$hostgroup = datamapper\HostGroupMapper::getInstance()->getByID(
						$rawHostgroup->hostgroupId
					);
					$hostgroup->allowWrite(model\Permission::IMPORT);

					foreach ($rawHostgroup->getMembersAsArray() as $hostname) {
						try {
							$host = datamapper\HostMapper::getInstance()->getByName(
								$hostname
							);
						}
						catch (\OutOfBoundsException $e) {
							continue;
						}

						$hostgroup->assign($host->id);
					}
				}
				catch (\OutOfBoundsException $e) {
					$notFound = true;
				}

				if (!$notFound) {
					continue;
				}
			}

			$hostgroup = new model\HostGroup();
			$hostgroup->allowWrite(model\Permission::IMPORT);
			$hostgroup->customerId = $this->session->user->getACL()->getRootCustomerIdentifier();
			$hostgroup->name = $rawHostgroup->hostgroup_name;
			$hostgroup->description = tr('Imported hostgroup');
			$hostgroup->save();

			foreach ($rawHostgroup->getMembersAsArray() as $hostname) {
				$host = datamapper\HostMapper::getInstance()->getByName($hostname)->allowRead();
				$hostgroup->assign($host->id);
			}
		}
	}
}
