<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use ch\monitoring\Application;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\exceptions\UserException;

/**
 * Handle host page requests
 *
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class InfofeedController extends ProtectedController{

	/**
	 * Handles GET requests
	 *
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::INFOFEEDS
		);

		$infofeeds = datamapper\InfofeedMapper::getInstance()->getAll(Application::getUser()->id);
		$this->setHeaderTitle(tr("Infofeeds"));
		$this->parameters['infofeeds'] = $infofeeds;
		$this->checkUserSettings();

		$this->tpl->assign('template', 'infofeeds/infofeeds.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();
		$this->checkUserSettings();

		$this->tpl->assign('template', 'infofeeds/infofeedsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Get all the defined notification types of the given user
	 *
	 * @param         User
	 * @return        array
	 */
	private function getUserNotificationTypes($user) {
		$types = [];

		if (isset($user->email)) {
			$types[] = model\SLAAction::EMAIL;
		}
		if (isset($user->sms)) {
			$types[] = model\SLAAction::SMS;
		}
		if (isset($user->jabber)) {
			$types[] = model\SLAAction::JABBER;
		}
		if (isset($user->mattermost)) {
			$types[] = model\SLAAction::MATTERMOST;
		}
		if (isset($user->phone)) {
			$types[] = model\SLAAction::PHONE;
		}

		return $types;
	}

	/**
	 * Checks if there are notification values defined in the user settings
	 *
	 * @param         User $user
	 * @return        void
	 */
	private function checkUserSettings() {
		$user = datamapper\UserMapper::getInstance()->getByID($this->session->user->id);
		$user->allowRead();

		$notificationTypes = model\SLAAction::getMapper()->getByInfofeedUserId(
			$user->id
		)->toArray();
		$userNotificationTypes = $this->getUserNotificationTypes($user);

		$unusedTypes = array_filter(
			$notificationTypes,
			function($type) use ($userNotificationTypes) {
				return !in_array($type->id, $userNotificationTypes);
			}
		);

		$notificationTypes = array_map(
			function($type) {
				return $type->name;
			},
			$unusedTypes
		);

		if (count($notificationTypes) > 0) {
			$text = sprintf(
				"There are no notification targets set for the following settings: %s",
				join(', ', $notificationTypes)
			);
			$text .= "<br>\nPlease add these settings in the <strong>User Settings</strong>";
			$this->addNotifyMessage($text);
		}
	}

	/**
	 * Load data needed to display the add form
	 *
	 * @returns       void
	 */
	private function assignAddFormVariables() {
		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(),
			'id',
			'hostname'
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::INFOFEEDS
			)
		);

		$this->select['hostgroups'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostGroupMapper::getInstance()->getAll()
		);

		$this->select['notificationtypes'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);
	}

	/**
	 * Load data needed to display the edit form
	 *
	 * @returns       void
	 */
	private function assignEditFormVariables() {
		$this->select['notificationtypes'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);
	}

	/**
	 * Edit an infofeed
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	public function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Infofeed id is missing")
			);
		}

		$infofeed = datamapper\InfofeedMapper::getInstance()->getByID($id);
		$notificationtypes = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getByInfofeedId($id),
			'id'
		);
		$this->parameters['infofeed'] = $infofeed;
		$this->parameters['notificationtypes'] = array_keys($notificationtypes);

		$abstractType = $infofeed->getObjectAndType();
		if($abstractType['type'] == 'Customer') {
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				datamapper\HostMapper::getInstance()->getAll(
					null,
					false,
					false,
					null,
					$abstractType['object']->id
				),
				'id',
				'hostname'
			);
		}
		
		if($abstractType['type'] == 'Host Group') {
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				$abstractType['object']->getMembers(),
				'id',
				'hostname'
			);
		}

		$this->assignEditFormVariables();
	}

	/**
	 * Delete an infofeed
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(tr('Infofeed id is missing'));
		}

		$infofeed = datamapper\InfofeedMapper::getInstance()->getByID($id);
		datamapper\InfofeedMapper::getInstance()->removeAllExcludedHosts($infofeed->id);
		$infofeed->delete();
	}

	/**
	 * Show the form to add an infofeed
	 *
	 */
	protected function addFormAction() {
		$infofeed = new model\Infofeed();

		$this->parameters['infofeed'] = $infofeed;
		$this->assignAddFormVariables();
	}

	/**
	 * Add an infofeed
	 *
	 * @returns       void
	 */
	protected function addAction() {
		$infofeed = new model\Infofeed();
		$infofeed->userId  = Application::getUser()->id;

		$this->saveInfofeed($infofeed, true);
	}

	/**
	 * Get objects of a type
	 *
	 * @returns       void
	 */
	protected function getTypeSelectAction() {
		$type = $this->request->getInt('typeId', request\REQUEST_GET) ?: null;

		if ($type === 1) {
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				datamapper\HostMapper::getInstance()->getAll(),
				'id',
				'hostname'
			);
		}
		elseif ($type === 2) {
			$this->select['hostgroups'] = util\IteratorUtil::getKeyValueArray(
				datamapper\HostGroupMapper::getInstance()->getAll(),
				'id',
				'name'
			);
		}
		elseif ($type === 3) {
			$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
				datamapper\CustomerMapper::getInstance()->getAll(),
				'id',
				'name'
			);
		}
	}

	/**
	 * Get hosts for a customer
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function customerHostsAction() {
		$customerId = $this->request->getInt('customerId');

		if (empty($customerId)) {
			throw new UserException(tr('Customer id is missing'));
		}
		
		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				null,
				false,
				false,
				null,
				$customerId
			),
			'id',
			'hostname'
		);
	}

	/**
	 * Returns all hosts of a hostgroup
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function hostgroupHostsAction() {
		$id = $this->request->getInt('hostGroupId');
		
		if (empty($id)) {
			throw new UserException(tr('Hostgroup id is missing'));
		}
		
		$hostgroup = datamapper\HostGroupMapper::getInstance()->getByID($id);
		$hostgroup->allowRead();

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			$hostgroup->getMembers(),
			'id',
			'hostname'
		);
	}

	/**
	 * Shows Infofeeds according to filter criteria
	 *
	 * @throws        UserException
	 * @returns       void
	 */
	protected function showAction() {
		$typeId      = $this->request->getInt('typeId', request\REQUEST_GET) ?: null;
		$hostId      = $this->request->getInt('hostId', request\REQUEST_GET) ?: null;
		$hostGroupId = $this->request->getInt('hostGroupId', request\REQUEST_GET) ?: null;
		$customerId  = $this->request->getInt('customerId', request\REQUEST_GET) ?: null;

		$infofeeds = datamapper\InfofeedMapper::getInstance()->getAll(
			Application::getUser()->id,
			$typeId,
			$hostId,
			$hostGroupId,
			$customerId
		);

		$this->parameters['infofeeds'] = $infofeeds;
	}

	/**
	 * Save an infofeed
	 *
	 */
	protected function saveInfofeed($infofeed, $add = false) {
		$infofeed->onAlert       = $this->request->keyExists('onAlert');
		$infofeed->onWarning     = $this->request->keyExists('onWarning');
		$infofeed->onAcknowledge = $this->request->keyExists('onAcknowledge');
		
		$notificationTypes       = $this->request->getRaw('notifyTypes') ?: array();
		$hostId                  = $this->request->getInt('hostId') ?: null;
		$hostGroupId             = $this->request->getInt('hostGroupId') ?: null;
		$customerId              = $this->request->getInt('customerId') ?: null;

		$infofeed->save();

		if(!($hostId || $hostGroupId || $customerId)) {
			throw new UserException(
				tr('You have to select a customer, host or a hostgroup')
			);
		}
		if (
			!$infofeed->onAlert &&
			!$infofeed->onWarning &&
			!$infofeed->onAcknowledge
		) {
			throw new UserException(
				tr('You have to select at least one notification event')
			);
		}
		if (!$notificationTypes) {
			throw new UserException(
				tr('You have to select at least one notification type')
			);
		}

		if ($add) {
			if ($hostId) {
				datamapper\InfofeedMapper::getInstance()->assignHost(
					$infofeed->id,
					$hostId
				);
			}

			if ($hostGroupId) {
				datamapper\InfofeedMapper::getInstance()->assignHostGroup(
					$infofeed->id,
					$hostGroupId
				);


			}

			if ($customerId) {
				datamapper\InfofeedMapper::getInstance()->assignCustomer(
					$infofeed->id,
					$customerId
				);
			}
		}

		// assign notifications to infofeed, if any were selected
		datamapper\InfofeedMapper::getInstance()->changeNotificationTypes(
			$infofeed->id,
			$notificationTypes
		);

		//destroy old excluded hosts and add the new ones
		datamapper\InfofeedMapper::getInstance()->removeAllExcludedHosts($infofeed->id);
		
		if($this->request->keyExists('excludedHostGroupHosts')) {
			$exludedHosts = (array)$this->request->getRaw('excludedHostGroupHosts');
			datamapper\InfofeedMapper::getInstance()->excludeHostForHostGroup($infofeed->id, $exludedHosts);
		}

		if($this->request->keyExists('excludedCustomerHosts')) {
			$exludedHosts = (array)$this->request->getRaw('excludedCustomerHosts');
			datamapper\InfofeedMapper::getInstance()->excludeHostForCustomer($infofeed->id, $exludedHosts);
		}

		$this->parameters['infofeed'] = $infofeed;
	}

	/**
	 * Save an existing infofeed
	 *
	 */
	protected function saveEditAction() {
		$id = $this->request->getInt('id');
		$infofeed = datamapper\InfofeedMapper::getInstance()->getByID($id);

		$this->saveInfofeed($infofeed);

	}
}
