<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\exceptions\ForbiddenException;
use ch\monitoring\util;
use ch\monitoring\Application;
use ch\monitoring\datamapper;
use ch\monitoring\model;

/**
 * Handle login page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class LoginController extends PublicController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');
		$this->setHeaderTitle(tr("Login"));
		$this->tpl->assign('return', $this->request->getString('return', request\REQUEST_GET));
		$this->tpl->assign('template', 'main/login.tpl');
		$this->outputPage('indexSimple.tpl');
	}

	/**
	 * Handles POST requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handlePOST() {
		header('Content-Type: text/html; charset=utf-8');

		$this->setHeaderTitle(tr("Login"));
		$this->tpl->assign('return', $this->request->getString('return'));
		$this->tpl->assign('template', 'main/login.tpl');

		$this->callActionHandler();

		$this->outputPage('indexSimple.tpl');
	}

	/**
	 * Imports a user from ldap
	 *
	 * @param         string $username  The username to import
	 * @param         string $password  The password to authenticate with
	 * @throws        \OutOfBoundsException
	 * @throws        UserException  when authentication was not possible
	 * @returns       model\User  the imported user
	 */
	private function importUserFromLDAP($username, $password) {
		$user = new model\User();
		$user->allowWrite(model\Permission::USERS_WRITE);
		$user->name = $username;
		$user->authenticationMethod = model\User::AUTH_METHOD_LDAP;
		if (!$user->authenticate($password)) {
			// Still no user found
			throw new UserException(
				tr("Authentication failed. Probably there is a local vs. LDAP"
				.  "user conflict. Please contact an administrator.")
			);
		}
		return $user;
	}

	/**
	 * Process login requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @throws        \RuntimeException
	 * @throws        UserException
	 * @returns       void
	 */
	protected function processLoginAction() {
		$username = $this->request->getString('username');
		$password = $this->request->getString('password');
		$allowLocal = ConfManager::getVar('af', 'auth', 'local') == 'yes';
		$allowLdap  = ConfManager::getVar('af', 'auth', 'ldap')  == 'yes';

		if (empty($username) || empty($password)) {
			throw new UserException(
				tr("Username and password are required")
			);
		}

		if (!$allowLocal && !$allowLdap) {
			throw new \RuntimeException(
				tr("Login impossible (no login methods allowed)")
			);
		}

		try {
			$user = datamapper\UserMapper::getInstance()->getByName(
				$username
			)->allowRead();
			if (!$user->isEnabled) {
				throw new UserException(
					tr("User is disabled")
				);
			}
			if (!$user->authenticate($password)) {
				throw new UserException(
					tr("Authentication failed. Probably there is a local vs. LDAP"
					.  "user conflict. Please contact an administrator.")
				);
			}
		}
		catch (\OutOfBoundsException $e) {
			// No such local user found, so try to import from LDAP
			$user = $this->importUserFromLDAP($username, $password);
		}

		// Must be set for authentication (ACL factory needs this)
		Application::getRegistry()->set('user', $user);
		$this->session->user = $user;
		// Synchronize external systems
		$cacti = new util\Cacti($user, $password);
		$cacti->synchronize();
		$icingaWeb = new util\IcingaWeb($user, $password);
		try {
			$icingaWeb->synchronize();
		}
		catch (\UnexpectedValueException $e) {
			Application::getLogInstance()->warn(sprintf(
				"There was an error while synchronizing " .
				"current user with IcingaWeb:\n " .
				"%s",
				$e->getMessage()
			));
		}
		$userLogin = new model\UserLogin();
		$userLogin->allowRead();
		$userLogin->userId    = $user->id;
		$userLogin->timestamp = new \Zend_Date();
		$userLogin->IP        = $_SERVER['REMOTE_ADDR'];
		// Must go over mapper to avoid permission verification!
		$userLogin->getMapper()->save($userLogin);

		$redirect = $this->request->getString('return');
		if ($redirect === null) {
			$redirect = '?page=home';
		}
		else {
			$urlTokens = parse_url($redirect);
			$redirect  = '';
			$redirect .= $urlTokens['path'] ?: '';
			$redirect .= $urlTokens['query'] ? '?' . $urlTokens['query'] : '';
			$redirect .= $urlTokens['fragment'] ?  '#' . $urlTokens['fragment'] : '';
		}
		if ($user->authenticationMethod === model\User::AUTH_METHOD_LDAP_SYNC) {
			$ldap = new util\Ldap();
			$ldapAttributes = $ldap->getChangedAttributes($user);
			if (!empty($ldapAttributes)) {
				$redirect = '?page=account&action=ldapSync';
			}
		}
		$this->redirectToURL($redirect);
	}
}
