<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\utils;
use org\adfinis\framework\exceptions\UserException;

/**
 * Handle maintenancechecklist page requests
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklistController extends MaintenanceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCE_ADMIN
		);

		$this->setHeaderTitle(tr("Checklists"));
		$this->parameters['checklists'] = datamapper\MaintenanceChecklistMapper::getInstance()->getAll(null, null, true);

		$this->tpl->assign('template', 'maintenance/checklists.tpl');
		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'maintenance/checklistsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}


	/**
	 * Show the form to add a maintenance checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$this->parameters['checklist'] = new model\MaintenanceChecklist();
		$this->parameters['checklist']->allowRead();
		$this->assignFormVariables();
	}

	/**
	 * Create a new maintenance checklist and invoke the save
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addAction() {
		$checklist = new model\MaintenanceChecklist();
		$checklist->allowWrite(model\Permission::MAINTENANCE_WRITE);

		$this->saveMaintenanceChecklist($checklist);
	}

	/**
	 * Show the form to edit a maintenance checklist
	 *
	 * @throws        UserException
	 * @returns       void
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$checklistId = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($checklistId)) {
			throw new UserException(
				tr("Maintenance checklist id is missing")
			);
		}

		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($checklistId);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		$this->assignFormVariables();
	}

	/**
	 * Delete a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Checklist id is missing")
			);
		}
		$checklist = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$checklist->enforceWritePermission(model\Permission::MAINTENANCE_WRITE);
		$checklist->delete();
	}

	/**
	 * Save an existing maintenance checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		$checklist = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$checklist->enforceWritePermission(model\Permission::MAINTENANCE_WRITE);

		$this->saveMaintenanceChecklist($checklist);
	}

	/**
	 * Assignes a template to a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function assignTemplateAction() {
		$id         = $this->request->getInt('id');
		$templateId = $this->request->getInt('templateId');
		
		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);

		$checklist = datamapper\MaintenanceChecklistMapper::getInstance()->assignTemplate($id, $templateId);
		$this->assignFormVariables();
	}


	/**
	 * Unssignes a template from a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function unassignTemplateAction() {
		$id         = $this->request->getInt('id');
		$templateId = $this->request->getInt('templateId');

		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		$checklist = datamapper\MaintenanceChecklistMapper::getInstance()->unassignTemplate($id, $templateId);
		$this->assignFormVariables();
	}

	/**
	 * Displays all available questions of a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function questionsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);
		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);

		$this->parameters['questions'] = datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getAll($id);
	}

	/**
	 * Shows the create form
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addQuestionAction() {
		$id = $this->request->getInt('id');
		
		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);

		$question = new model\MaintenanceChecklistQuestion();

		$question->name    = $this->request->getString('name');
		$question->comment = $this->request->getString('description');
		$question->save();

		datamapper\MaintenanceChecklistMapper::getInstance()->addQuestion($question->id, $id);
		
		$this->parameters['questions'][] = $question;
	}


	/**
	 * Removes the assignement from question to checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function deleteQuestionAction() {
		$id         = $this->request->getInt('id');
		$questionId = $this->request->getInt('questionId');

		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($id);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		$this->parameters['questions'] = datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getAll($id);

		datamapper\MaintenanceChecklistMapper::getInstance()->removeQuestion($questionId, $id);
	}

	/**
	 * Excludes a question from a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function excludeQuestionAction() {
		$id         = $this->request->getInt('id');
		$questionId = $this->request->getInt('questionId');
		datamapper\MaintenanceChecklistQuestionMapper::getInstance()->excludeQuestion($id, $questionId, true);
	}

	/**
	 * Includes a question from a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function includeQuestionAction() {
		$id         = $this->request->getInt('id');
		$questionId = $this->request->getInt('questionId');
		datamapper\MaintenanceChecklistQuestionMapper::getInstance()->excludeQuestion($id, $questionId, false);
	}


	/**
	 * Save the data of a created or edited checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         model\MaintenanceChecklist $checklist
	 * @returns       void
	 */
	private function saveMaintenanceChecklist($checklist) {
		$checklist->name        = $this->request->getString('name');
		$checklist->comment     = $this->request->getString('description');
		$checklist->isTemplate  = true;
		$checklist->save();

		$this->parameters['checklist'] = $checklist;
		$this->parameters['checklists'][] = $checklist;
		$this->assignFormVariables();
	}

	/**
	 * Assigns variables for the maintenance checklist form
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['templates']  = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
				null,
				null,
				true
			),
			'id',
			'name'
		);
	}
}
