<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\request;
use org\adfinis\framework\utils;
use ch\monitoring\Application;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle maintenance page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 *
 * @SuppressWarnings(PHPMD.ExcessiveClassLength)
 * @SuppressWarnings(PHPMD.TooManyMethods)
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 **/
class MaintenanceController extends MaintenanceBaseController {

	/**
	 * Constructor.
	 *
	 * @param         org\adfinis\framework\request\RequestInterface $request
	 * @param         array $urlMatch
	 */
	public function __construct($request, $urlMatch) {
		parent::__construct($request, $urlMatch);
		$this->tpl->assign('isArchive', $this->page == 'maintenanceHistory');
	}

	/**
	 * Handles GET requests
	 *
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCE_READ
		);
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		if ($this->page == "maintenanceOverview") {
			$this->setHeaderTitle(tr("Maintenance"));
			$tickets = datamapper\MaintenanceTicketMapper::getInstance()->getAll(
				null,
				null,
				null,
				null,
				null,
				null,
				false,
				null,
				null,
				model\Permission::MAINTENANCE_READ,
				util\IteratorUtil::getKeyArray($customers)
			);
			$this->tpl->assign('template', 'maintenance/maintenance.tpl');
		}
		else {
			$this->setHeaderTitle(tr("Maintenance history"));
			$tickets = datamapper\MaintenanceTicketMapper::getInstance()->getAll(
				null,
				null,
				null,
				null,
				null,
				null,
				true,
				null,
				null,
				model\Permission::MAINTENANCE_READ,
				util\IteratorUtil::getKeyArray($customers)
			);
			$limit = 50;
			$tickets->setOrderBy('dateArchived.DESC');
			$tickets->setOffsetLimit(0, $limit);
			$hosts = array();
			$this->parameters['limit'] = $limit;
			$this->tpl->assign('template', 'maintenance/history.tpl');
			$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
				$customers
			);
			$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
				datamapper\MaintenanceTagMapper::getInstance()->getAll()
			);
			if ($tickets->getCount()) {
				$ticketKeys = util\IteratorUtil::getKeyArray($tickets);
				$ticketHosts = datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
					empty($ticketKeys) ? null : $ticketKeys
				);
				foreach ($ticketHosts as $ticketHost) {
					array_push($hosts, $ticketHost->getHost());
				}
			}
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				$hosts,
				'id', 'hostname'
			);
		}

		$this->parameters['allTickets'] = $tickets;
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_READ,
				util\IteratorUtil::getKeyArray($customers),
				true
			),
			'id',
			'fullName'
		);
		$this->select['groups'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceGroupMapper::getInstance()->getAll(
				null,
				util\IteratorUtil::getKeyArray($customers)
			)
		);
		$this->tpl->registerClass("MaintenanceState", "\ch\monitoring\model\MaintenanceState");
		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'maintenance/maintenanceAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Update a ticket host and add a comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @param[in]     MaintenanceTicketHost $ticketHost The maintenance ticket host to be changed the state of
	 * @param         string $comment
	 * @param         int $newStateId The new state ID to be set
	 * @param         string $newOsVersionId The new os version to be set
	 * @param         bool $notifyUsers Whether to notify the users by eMail
	 * @param         array $tags Tags to be added to the comment
	 *
	 * @returns       MaintenanceComment
	 *
	 * @SuppressWarnings(PHPMD.CyclomaticComplexity)
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	private function updateTicket(
		model\MaintenanceTicketHost $ticketHost,
		$commentText,
		$newStateId        = null,
		$newOsVersionId    = null,
		$notifyUsers       = true,
		Array $commentTags = array()
	) {

		$comment = new model\MaintenanceComment();
		$comment->allowWrite(model\Permission::MAINTENANCE_WRITE);

		$comment->timestamp     = new \Zend_Date();
		$comment->userId        = $this->session->user->id;
		$comment->comment       = $commentText;

		if ($newStateId !== null && $newStateId !== $ticketHost->stateId) {
			$comment->oldStateId = $ticketHost->stateId;
			$comment->newStateId = $newStateId;
			$ticketHost->stateId = $newStateId;
		}

		if ($newOsVersionId != $ticketHost->getHost()->osVersionId) {
			$comment->oldOsVersionId = $ticketHost->getHost()->osVersionId;
			$comment->newOsVersionId = $newOsVersionId;

			$host = $ticketHost->getHost();
			$host->allowWrite(model\Permission::MAINTENANCE_WRITE);
			$host->osVersionId = $newOsVersionId;
			$host->save();
		}

		$ticketHost->save();

		$comment->ticketHostId = $ticketHost->id;
		$comment->save();

		foreach ($commentTags as $tagId) {
			$comment->addMaintenanceTag($tagId);
		}

		$hosts = $ticketHost->getTicket()->getTicketHosts();
		$hostCount = count($hosts);
		foreach ($hosts as $host) {
			if ($host->stateId == model\MaintenanceState::STATE_DONE) {
				$hostCount--;
			}
		}

		$ticket = $ticketHost->getTicket();
		if ($hostCount == 0) {
			if ($ticket->dateArchived === null) {
				$ticket->dateArchived = new \Zend_Date();
				$ticket->getMapper()->save($ticket);
			}
		}
		else {
			if ($ticket->dateArchived !== null) {
				$ticket->dateArchived = null;
				$ticket->getMapper()->save($ticket);
			}
		}

		if ($notifyUsers === true) {
			$this->notifyUsers($comment);
		}
	}

	/**
	 * Returns all maintenance masters for a customer
	 *
	 * The returned array contains structures which can be used directly with MailHelper
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @returns       array
	 */
	private function getMaintenanceMasters($customerId) {
		$masters = datamapper\UserMapper::getInstance()->getAllByPermission(
			model\Permission::MAINTENANCE_MASTER,
			$customerId,
			true
		);

		$mastersArray = array();
		foreach ($masters as $master) {
			// You probably don't want to get your own interactions
			if ($master->id == $this->session->user->id || empty($master->email)) {
				continue;
			}

			$mastersArray[] = array(
				'firstName' => '',
				'lastName'  => $master->fullName,
				'address'   => $master->email
			);
		}

		return $mastersArray;
	}

	/**
	 * Send an eMail to notify the involved persons about the changes
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\MaintenanceComment $comment
	 * @returns       void
	 */
	private function notifyUsers($comment) {
		$ticketHost = $comment->getTicketHost();
		$host       = $ticketHost->getHost();

		$mastersArray = $this->getMaintenanceMasters($host->customerId);

		if (
			count($mastersArray) > 0
			&&
			$comment->newStateId == model\MaintenanceState::STATE_DONE
			&&
			$comment->newStateId != $comment->oldStateId
		) {
			$subject = sprintf(
				'State for host %s in maintenance ticket #%d changed to done',
				$host->hostname,
				$ticketHost->getTicket()->id
			);

			$text = sprintf(
				"State for maintenance ticket #%d for host\n\n\t%s\n\nwas set to\n\n\tdone\n\nby\n\n\t%s.\n\n",
				$ticketHost->getTicket()->id,
				$host->hostname,
				$comment->getUser()->fullName
			);
			$text .= sprintf(
				"You receive this e-mail as you are maintenance master for customer %s.\n",
				$host->getCustomer()->name
			);
			$text .= "Which means you have the MAINTENANCE_MASTER permission set for the mentioned customer.";

			util\MailHelper::sendMail(
				$mastersArray,
				$subject,
				$text,
				'',
				true
			);
		}
	}

	/**
	 * Sends an email to the assigned person and to all maintenance users
	 *
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\MaintenanceTicket $ticket
	 * @returns       void
	 *
	 */
	private function notifyAssignedUser($ticket) {
		try {
			$user = $ticket->getUser();
		}
		catch (\OutOfBoundsException $e) {
			$user = null;
		}
		$creator = $this->session->user;
		$ticketHostsString = $this->getTicketHostsString($ticket);

		if($user && $user->email != null) {
			$subject = sprintf(
				'Maintenance ticket #%d for host(s) %s was assigned to you.',
				$ticket->id,
				$ticketHostsString
			);

			$text = sprintf(
				"The maintenance ticket #%d for the following host(s) has been assigned to you by %s:\n\n",
				$ticket->id,
				$creator->fullName
			);
			foreach (explode(', ', $ticketHostsString) as $ticketHost) {
				$text .= sprintf("   * %s\n", $ticketHost);
			}

			util\MailHelper::sendMail(
				array(
					array(
						'firstName' => '',
						'lastName'  => $user->fullName,
						'address'   => $user->email
					)
				),
				$subject,
				$text,
				'',
				true
			);
		}

		$mastersArray = $this->getMaintenanceMasters(null);
		if (count($mastersArray) > 0) {
			// And now notify the maintenance masters
			$subject = sprintf(
				'Maintenance ticket #%d was assigned to another user.',
				$ticket->id
			);

			$text = '';
			if (!is_null($user)) {
				$text = sprintf(
					"Maintenance ticket #%d has been assigned to %s by %s.\n\n",
					$ticket->id,
					$user->fullName,
					$creator->fullName
				);
			}
			else {
				$text = sprintf(
					"Maintenance ticket #%d has been assigned to nobody by %s.\n\n",
					$ticket->id,
					$creator->fullName
				);
			}
			$text .= "You receive this e-mail as you are maintenance master for all customers.\n";
			$text .= "Which means you have the MAINTENANCE_MASTER permission set for the mentioned customer.";

			$recipient = $this->getDefaultRecipient($mastersArray);

			// Send mail to every maintenance master for this customer
			util\MailHelper::sendMail(
				$recipient,
				$subject,
				$text,
				'',
				true,
				75,
				array(),
				$mastersArray
			);
		}
	}


	/**
	 * Add a ticket by host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addHostFormAction() {
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_WRITE,
				util\IteratorUtil::getKeyArray($customers),
				true
			),
			'id',
			'fullName'
		);
		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				null,
				model\Permission::MAINTENANCE_WRITE
			),
			'id',
			'hostname'
		);
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			$customers
		);
	}

	/**
	 * Add a ticket by maintenance group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addGroupFormAction() {
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_WRITE,
				util\IteratorUtil::getKeyArray($customers),
				true
			),
			'id',
			'fullName'
		);
		$this->select['groups'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceGroupMapper::getInstance()->getAll(
				null,
				util\IteratorUtil::getKeyArray($customers)
			)
		);
	}

	/**
	 * Update a ticket host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function updateAction() {
		$ticketHostId = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($ticketHostId)) {
			throw new UserException(
				tr("Ticket host id is missing")
			);
		}

		$ticketHost = datamapper\MaintenanceTicketHostMapper::getInstance()->getByID($ticketHostId);
		$isStateDone = $ticketHost->stateId === model\MaintenanceState::STATE_DONE;

		$this->parameters['ticketHost'] = $ticketHost;
		$this->parameters['ticketHost']->enforceReadPermission(
			model\Permission::MAINTENANCE_WRITE
		);
		$this->parameters['isStateDone'] = $isStateDone;
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_WRITE,
				util\IteratorUtil::getKeyArray($customers)
			),
			'id',
			'fullName'
		);
		$this->select['states'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceStateMapper::getInstance()->getAll()
		);

		$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceTagMapper::getInstance()->getAll()
		);

		$this->tpl->registerClass(
			"MaintenanceState",
			"\ch\monitoring\model\MaintenanceState"
		);
	}

	/**
	 * Add a ticket for a single host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	protected function addHostAction() {
		$customerId = $this->request->getInt('customerId');
		$userId     = $this->request->getInt('userId') ?: null;
		$dateDue    = $this->request->getString('dateDue') ?: null;
		$hostId     = $this->request->getInt('hostId');

		$host = datamapper\HostMapper::getInstance()->getByID($hostId);
		$host->enforceReadPermission(model\Permission::MAINTENANCE_READ);

		/* A ticket can either be directly assigned to a user or free to take.
		 * Dependent on this the initial status is different. */
		$stateId = $userId
			? model\MaintenanceState::STATE_IN_PROGRESS
			: model\MaintenanceState::STATE_NEW;

		$ticket = new model\MaintenanceTicket();
		$ticket->allowWrite(model\Permission::MAINTENANCE_WRITE);
		$ticket->customerId   = $customerId;
		$ticket->userId       = $userId;
		$ticket->dateCreated  = new \Zend_Date();
		$ticket->dateArchived = null;
		if ($dateDue !== null) {
			$ticket->dateDue = new \Zend_Date($dateDue);
		}
		$ticket->save();

		$ticketHost = new model\MaintenanceTicketHost();
		$ticketHost->allowWrite(model\Permission::MAINTENANCE_WRITE);
		$ticketHost->ticketId = $ticket->id;
		$ticketHost->hostId   = $hostId;
		$this->updateTicket(
			$ticketHost,
			sprintf(
				"Ticket created by %s.",
				$this->session->user->fullName
			),
			$stateId
		);
		$this->tpl->assign('ticket', $ticket);
		$this->notifyAssignedUser($ticket);
	}

	/**
	 * Add a ticket for a maintenance group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	protected function addGroupAction() {
		$userId  = $this->request->getInt('userId') ?: null;
		$dateDue = $this->request->getString('dateDue') ?: null;
		$groupId = $this->request->getInt('groupId');

		$group = datamapper\MaintenanceGroupMapper::getInstance()->getByID($groupId);
		$group->enforceReadPermission(model\Permission::MAINTENANCE_READ);

		/* A ticket can either be directly assigned to a user or free to take.
		 * Dependent on this the initial status and substatus are different. */
		$stateId = $userId
			? model\MaintenanceState::STATE_IN_PROGRESS
			: model\MaintenanceState::STATE_NEW;

		$ticket = new model\MaintenanceTicket();
		$ticket->allowWrite(model\Permission::MAINTENANCE_WRITE);

		$ticket->customerId   = $group->getCustomer()->id;
		$ticket->groupId      = $groupId;
		$ticket->userId       = $userId;
		$ticket->dateCreated  = new \Zend_Date();
		$ticket->dateArchived = null;

		if ($dateDue !== null) {
			$ticket->dateDue = new \Zend_Date($dateDue);
		}

		$ticket->save();

		foreach ($group->getMembers() as $member) {
			$ticketHost = new model\MaintenanceTicketHost();
			$ticketHost->allowWrite(model\Permission::MAINTENANCE_WRITE);

			$ticketHost->ticketId = $ticket->id;
			$ticketHost->hostId   = $member->hostId;

			$this->updateTicket(
				$ticketHost,
				sprintf(
					"Ticket created by %s.",
					$this->session->user->fullName
				),
				$stateId
			);
		}

		$this->tpl->assign('ticket', $ticket);

		$this->notifyAssignedUser($ticket);
	}

	/**
	 * Delete a ticket comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteCommentAction() {
		$commentId = $this->request->getInt('id');

		if (empty($commentId)) {
			throw new UserException(
				tr("Id is missing")
			);
		}

		$comment = datamapper\MaintenanceCommentMapper::getInstance()->getByID($commentId);
		$comment->enforceWritePermission(
			model\Permission::MAINTENANCE_WRITE
		);
		$comment->delete();

		$this->parameters['ticketHost'] = $comment->getTicketHost();
	}

	/**
	 * Save a ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @throws        UserException
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.CyclomaticComplexity)
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	protected function saveAction() {
		$customerId        = $this->request->getInt('customerId');
		$ticketHostId      = $this->request->getInt('id');
		$comment           = $this->request->getString('comment');
		$stateId           = $this->request->getInt('stateId') ?: null;
		$osVersionId       = $this->request->getInt('osVersionId') ?: null;
		$tags              = $this->request->getRaw('commentTags') ?: array();
		$checklistComments = $this->request->getRaw('checklistComment') ?: array();
		$checked           = $this->request->getRaw('checked') ?: array();

		if (empty($ticketHostId)) {
			throw new UserException(
				tr("Id is missing")
			);
		}

		$ticketHost = datamapper\MaintenanceTicketHostMapper::getInstance()->getByID($ticketHostId);
		$ticketHost->enforceWritePermission(
			model\Permission::MAINTENANCE_WRITE
		);

		switch ($stateId) {
			case model\MaintenanceState::STATE_NEW:
				throw new UserException(
					tr("Can't set new state")
				);
				break;
			case model\MaintenanceState::STATE_IN_PROGRESS:
			case model\MaintenanceState::STATE_DONE:
				// All right!
				break;
			default:
				throw new UserException(
					tr("Illegal ticket state")
				);
				break;
		}

		$answers = $ticketHost->getAnswers();
		$allComments = array();

		if ($ticketHost->stateId != model\MaintenanceState::STATE_DONE) {
			foreach ($answers as $answer) {
				$comments = array();
				$question = $answer->getQuestion();

				if ($answer->comment != $checklistComments[$answer->id]) {
					$comments[] = sprintf("%s", $answer->comment);
				}
				if ($answer->isChecked != array_key_exists($answer->id, $checked)) {
					$isChecked = array_key_exists($answer->id, $checked) ? tr('checked') : tr('unchecked');
					$comments[] = sprintf("%s: %s", tr('Checklist question has been'), $isChecked);
				}
				$answer->comment   = $checklistComments[$answer->id];
				$answer->isChecked = array_key_exists($answer->id, $checked);
				$answer->getMapper()->save($answer);

				if (count($comments) > 0) {
					$allComments[] = sprintf("Checklist '%s':\n%s\n", $question->name, join("\n", $comments));
				}
			}
		}

		if (count($checklistComments) > 0) {
			if ($comment) {
				$comment .= "\n\n";
			}
			$comment .= join("\n", $allComments);
		}

		$hasNewStateID   = $stateId !== null && $stateId !== $ticketHost->stateId;
		$hasNewVersionID = $osVersionId != $ticketHost->getHost()->osVersionId;
		if (trim($comment) || $hasNewStateID || $hasNewVersionID) {
			$this->updateTicket(
				$ticketHost,
				$comment,
				$stateId,
				$osVersionId,
				true,
				$tags
			);
		}

		$this->parameters['ticket'] = $ticketHost->getTicket();
		$this->tpl->assign('ticket', $ticketHost->getTicket());

		$this->tpl->registerClass(
			"MaintenanceState",
			"\ch\monitoring\model\MaintenanceState"
		);
	}

	/**
	 * Shows ticket history for given ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function historyAction() {
		$ticketId = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($ticketId)) {
			throw new UserException(
				tr("Ticket id is missing")
			);
		}

		$this->parameters['ticketHost'] = datamapper\MaintenanceTicketHostMapper::getInstance()->getByID($ticketId);
		$this->parameters['ticketHost']->enforceReadPermission(
			model\Permission::MAINTENANCE_READ
		);
	}

	/**
	 * Show a checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function checklistAction() {
		$ticketId = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($ticketId)) {
			throw new UserException(
				tr("Ticket id is missing")
			);
		}

		$this->parameters['ticketHost'] = datamapper\MaintenanceTicketHostMapper::getInstance()->getByID($ticketId);
		$this->parameters['ticketHost']->enforceReadPermission(
			model\Permission::MAINTENANCE_READ
		);
	}

	/**
	 * Tick check list points
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function setPointsAction() {
		$ticketId = $this->request->getInt('id');
		$ticket   = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
		$points   = $this->request->getRaw('points') ?: array();

		$ticket->enforceWritePermission(
			model\Permission::MAINTENANCE_WRITE
		);

		$checkedPoints = array();
		datamapper\MaintenanceChecklistMapper::getInstance()->deleteAssociations($ticket->id);
		foreach ($points as $pointId) {
			$point = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($pointId)->allowRead();
			$checkedPoints[] = $point->name;
			datamapper\MaintenanceChecklistMapper::getInstance()->createAssociation($point->id, $ticket->id);
		}

		$this->updateTicket(
			$ticket,
			sprintf(
				"Points checked: %s",
				implode(', ', $checkedPoints)
			)
		);
	}

	/**
	 * Take a ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function takeAction() {
		$ticketId = $this->request->getInt('id');

		if (empty($ticketId)) {
			throw new UserException(
				tr("Ticket id is missing")
			);
		}

		$this->tpl->registerClass(
			"MaintenanceState",
			"\ch\monitoring\model\MaintenanceState"
		);

		$ticket = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
		$ticket->enforceWritePermission(
			model\Permission::MAINTENANCE_WRITE
		);

		$ticket->userId = $this->session->user->id;
		$ticket->save();

		foreach ($ticket->getTicketHosts() as $ticketHost) {
			$ticketHost->enforceWritePermission(
				model\Permission::MAINTENANCE_WRITE
			);

			$this->updateTicket(
				$ticketHost,
				 sprintf(
					"%s changed the owner of this ticket.",
					$this->session->user->fullName
				),
				model\MaintenanceState::STATE_IN_PROGRESS
			);
		}

		$this->parameters['ticket'] = $ticket;
		$this->tpl->assign('ticket', $ticket);
	}

	/**
	 * Bulk set owner
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @throws        UserException
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.CyclomaticComplexity)
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
	 */
	protected function bulkOwnerAction() {
		$userId  = $this->request->getInt('userId');
		$tickets = $this->request->getRaw('tickets');

		if (empty($userId)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		if (!is_array($tickets) || count($tickets) == 0) {
			throw new UserException(
				tr("No tickets chosen")
			);
		}

		$mastersArray = $this->getMaintenanceMasters(null);
		$newUser = datamapper\UserMapper::getInstance()->getByID($userId);

		$affectedHosts = [];
		foreach ($tickets as $ticketId) {
			$ticket = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
			$ticket->enforceWritePermission(
				model\Permission::MAINTENANCE_WRITE
			);
			$isAssignmentValid = $newUser->hasPermissionForCustomer(
				model\Permission::MAINTENANCE_WRITE,
				$ticket->customerId
			);
			if (!$isAssignmentValid) {
				throw new UserException(
					sprintf(tr(
						'The user %s has no MAINTENANCE_WRITE right on customer %s'
					), $newUser->name, $ticket->getCustomer()->name)
				);
			}
			if ($userId == $ticket->userId) {
				continue;
			}
			$ticket->userId = $userId;
			$ticket->save();
			$ticketHostsString = $this->getTicketHostsString($ticket);
			array_push($affectedHosts, $ticketHostsString);

			// First notify the new owner
			if ($newUser->email != null) {
				$subject = sprintf(
					'Maintenance ticket #%d for host(s) %s was assigned to you.',
					$ticket->id,
					$ticketHostsString
				);

				$text = sprintf(
					"The maintenance ticket #%d for the following host(s) has been assigned to you by %s:\n\n",
					$ticket->id,
					$this->session->user->fullName
				);
				foreach (explode(', ', $ticketHostsString) as $ticketHost) {
					$text .= sprintf("   * %s\n", $ticketHost);
				}

				util\MailHelper::sendMail(
					array(
						array(
							'firstName' => '',
							'lastName'  => $newUser->fullName,
							'address'   => $newUser->email
						)
					),
					$subject,
					$text,
					'',
					true
				);
			}

			if (count($mastersArray) > 0) {
				// And now notify the maintenance masters
				$subject = sprintf(
					'Maintenance ticket #%d for host(s) %s was assigned to another user.',
					$ticket->id,
					$ticketHostsString
				);

				$text = sprintf(
					"The maintenance ticket #%d for the following host(s) has been assigned to %s by %s:\n\n",
					$ticket->id,
					$newUser->fullName,
					$this->session->user->fullName
				);
				foreach (explode(', ', $ticketHostsString) as $ticketHost) {
					$text .= sprintf("   * %s\n", $ticketHost);
				}

				$recipient = $this->getDefaultRecipient($mastersArray);

				// Send mail to every maintenance master
				util\MailHelper::sendMail(
					$recipient,
					$subject,
					$text,
					'',
					true,
					75,
					array(),
					$mastersArray
				);

			}
		}
		$affectedHosts = array_unique($affectedHosts);

		// Inform user via frontend
		if (count($affectedHosts) > 0) {
			$text = sprintf(
				"The maintenance ticket(s) for the following host(s) has/have " .
				"been assigned to %s:",
				$newUser->fullName
			);
			foreach ($affectedHosts as $ticketHostsString) {
				$text .= '<ul>';
				foreach (explode(', ', $ticketHostsString) as $ticketHost) {
					$text .= sprintf("<li>%s</li>", $ticketHost);
				}
				$text .= '</ul>';
			}
		}
		else {
			$text = "No host(s) has/have been affected by the action " .
				"(there was most probably no change in user)";
		}
		$this->addNotifyMessage(tr($text));

		// Reload all tickets (yeah, this sucks..)
		$this->reloadTickets();
	}

	/**
	 * Generic action function to bulk update the state of tickets
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         int $stateID
	 * @throws        UserException
	 */
	private function bulkChangeState($stateID) {
		$comment = $this->request->getString('comment');
		$tickets = $this->request->getRaw('tickets');
		$tags = $this->request->getRaw('commentTags') ?: array();

		if (empty($comment)) {
			throw new UserException(
				tr("Comment is missing")
			);
		}

		if (!is_array($tickets) || count($tickets) == 0) {
			throw new UserException(
				tr("No tickets chosen")
			);
		}

		if (!is_array($tags)) {
			$tags = array();
		}

		$state = model\MaintenanceState::getMapper()->getByID($stateID);
		$text  = _(sprintf(
			"The maintenance ticket(s) for the following host(s) has/have " .
			"been set to `%s`:",
			strtolower($state->name)
		));
		foreach ($tickets as $ticketId) {
			$ticket = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
			$ticket->enforceWritePermission(
				model\Permission::MAINTENANCE_WRITE
			);
			$ticketHostsString = $this->getTicketHostsString($ticket) . ", ";

			foreach ($ticket->getTicketHosts() as $ticketHost) {
				$ticketHost->enforceWritePermission(
					model\Permission::MAINTENANCE_WRITE
				);

				$this->updateTicket(
					$ticketHost,
					$comment,
					$stateID,
					$ticketHost->getHost()->osVersionId,
					true,
					$tags
				);
			}

			if ($stateID == model\MaintenanceState::STATE_DONE) {
				$ticket->dateArchived = new \Zend_Date();
			}
			else {
				$ticket->dateArchived = null;
			}

			$ticket->save();

			// Inform user via frontend
			$ticketHostsString = $this->getTicketHostsString($ticket);
			$text .= '<ul>';
			foreach (explode(', ', $ticketHostsString) as $ticketHost) {
				$text .= sprintf("   <li>%s</li>", $ticketHost);
			}
			$text .= '</ul>';
		}
		$this->addNotifyMessage($text);

		return $this->getTicketsAction();
	}

	/**
	 * Bulk open
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function bulkOpenAction() {
		$this->bulkChangeState(model\MaintenanceState::STATE_IN_PROGRESS);
	}

	/**
	 * Bulk close
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function bulkCloseAction() {
		$this->bulkChangeState(model\MaintenanceState::STATE_DONE);
	}

	/**
	 * Bulk comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function bulkCommentAction() {
		$comment = $this->request->getString('comment');
		$tickets = $this->request->getRaw('tickets');
		$tags    = $this->request->getRaw('commentTags') ?: array();

		if (empty($comment)) {
			throw new UserException(
				tr("Comment is missing")
			);
		}

		if (!is_array($tickets) || count($tickets) == 0) {
			throw new UserException(
				tr("No tickets chosen")
			);
		}

		if (!is_array($tags)) {
			$tags = array();
		}

		$ticketHostsString = '';
		foreach ($tickets as $ticketId) {
			$ticket = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
			$ticket->enforceWritePermission(
				model\Permission::MAINTENANCE_WRITE
			);

			foreach ($ticket->getTicketHosts() as $ticketHost) {
				$ticketHost->enforceWritePermission(
					model\Permission::MAINTENANCE_WRITE
				);

				$this->updateTicket(
					$ticketHost,
					$comment,
					null,
					$ticketHost->getHost()->osVersionId,
					true,
					$tags
				);
			}
			$ticketHostsString .= $this->getTicketHostsString($ticket) . ", ";
		}

		// Inform user via frontend
		$text = sprintf(
			"The maintenance ticket(s) for the following host(s) has/have " .
			"been commented with '%s':",
			$comment
		);
		$text .= '<ul>';
		foreach (explode(', ', $ticketHostsString) as $ticketHost) {
			$text .= sprintf("<li>%s</li>", $ticketHost);
		}
		$text .= '</ul>';
		$this->addNotifyMessage(tr($text));

		// Reload all tickets (yeah, this sucks..)
		$this->reloadTickets();
	}

	/**
	 * Bulk set date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * 
	 * @throws        UserException
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.CyclomaticComplexity)
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
	 */
	protected function bulkDateAction() {
		$date    = $this->request->getString('date');
		$comment = $this->request->getString('comment');
		$tickets = $this->request->getRaw('tickets');
		$tags    = $this->request->getRaw('commentTags') ?: array();

		if (empty($date)) {
			throw new UserException(
				tr("Date is missing")
			);
		}

		$date = new \Zend_Date($date, \Zend_Date::ISO_8601);

		if (empty($comment)) {
			throw new UserException(
				tr("Comment is missing")
			);
		}

		if (!is_array($tickets) || count($tickets) == 0) {
			throw new UserException(
				tr("No tickets chosen")
			);
		}

		if (!is_array($tags)) {
			$tags = array();
		}

		foreach ($tickets as $ticketId) {
			$ticket = datamapper\MaintenanceTicketMapper::getInstance()->getByID($ticketId);
			$ticket->enforceWritePermission(
				model\Permission::MAINTENANCE_WRITE
			);

			$ticket->dateDue = $date;
			$ticket->save();

			$ticketHostsString = $this->getTicketHostsString($ticket) . ", ";
			$mastersArray = $this->getMaintenanceMasters(null);

			// If maintenance masters are set for the root customer (?, why
			// not for specific customer of ticket(s)?)
			// inform them
			if (count($mastersArray) > 0) {
				$subject = sprintf(
					'Due date of maintenance ticket for host(s) %s was set to %s.',
					$ticketHostsString,
					$date->get(\Zend_Date::DATE_MEDIUM)
				);

				$text = sprintf(
					"Due date of maintenance ticket for the following host(s) has been set to %s by %s:\n\n",
					$date->get(\Zend_Date::DATE_MEDIUM),
					$this->session->user->fullName
				);
				foreach (explode(', ', $ticketHostsString) as $ticketHost) {
					$text .= sprintf("   * %s\n", $ticketHost);
				}
				$text .= "You receive this e-mail as you are maintenance master for all customers.\n";
				$text .= "Which means you have the MAINTENANCE_MASTER permission set for the mentioned customer.";

				$recipient = $this->getDefaultRecipient($mastersArray);

				// Send mail to every maintenance master
				util\MailHelper::sendMail(
					$recipient,
					$subject,
					$text,
					'',
					true,
					75,
					array(),
					$mastersArray
				);
			}

			$comment = sprintf(
				"%s %s:\n%s",
				tr("Set a new due date."),
				tr("Reason"),
				$comment
			);

			foreach ($ticket->getTicketHosts() as $ticketHost) {
				$ticketHost->enforceWritePermission(
					model\Permission::MAINTENANCE_WRITE
				);

				$this->updateTicket(
					$ticketHost,
					$comment,
					null,
					$ticketHost->getHost()->osVersionId,
					true,
					$tags
				);
			}
		}

		// Inform user via frontend
		$text = sprintf(
			"The maintenance ticket(s) for the following host(s) has/have " .
			"been set with the date '%s':",
			$date->get(\Zend_Date::DATE_MEDIUM)
		);
		$text .= '<ul>';
		foreach (explode(', ', $ticketHostsString) as $ticketHost) {
			$text .= sprintf("<li>%s</li>\n", $ticketHost);
		}
		$text .= '</ul>';
		$this->addNotifyMessage(tr($text));

		// Reload all tickets (yeah, this sucks..)
		$this->reloadTickets();
	}

	/**
	 * Get tickets by filter
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 *
	 * @returns       void
	 *
	 * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
	 * @SuppressWarnings(PHPMD.NPathComplexity)
	 */
	protected function getTicketsAction() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCE_READ
		);

		$groupId = $this->request->getInt('groupId') ?: null;
		$userId  = $this->request->getInt('userId') ?: null;
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);

		if ($this->page == "maintenanceHistory") {
			$customerId     = $this->request->getInt('customerIdHistory') ?: null;
			if ($customerId === null) {
					$customerId = util\IteratorUtil::getKeyArray($customers);
			}
			$hostId         = $this->request->getInt('hostId') ?: null;
			$limit          = $this->request->getInt('limit');
			$dateFromString = $this->request->getString('dateFrom');
			$dateToString   = $this->request->getString('dateTo');
			$dateFrom       = $dateFromString
				? new \Zend_Date($dateFromString, \Zend_Date::ISO_8601)
				: null;
			$dateTo         = $dateToString
				? new \Zend_Date($dateToString, \Zend_Date::ISO_8601)
				: null;

			$tickets = datamapper\MaintenanceTicketMapper::getInstance()->getAll(
				$groupId,
				$userId,
				null,
				null,
				null,
				null,
				true,
				$dateFrom,
				$dateTo,
				model\Permission::MAINTENANCE_READ,
				$customerId,
				$hostId
			);

			$tickets->setOrderBy('dateArchived.DESC');
			$tickets->setOffsetLimit(0, $limit);
			$this->parameters['limit'] = $limit;
			$this->select['users'] = util\IteratorUtil::getKeyValueArray(
				datamapper\UserMapper::getInstance()->getAllByPermission(
					model\Permission::MAINTENANCE_WRITE,
					$customerId,
					true
				),
				'id',
				'fullName'
			);
			$this->select['groups'] = util\IteratorUtil::getKeyValueArray(
				datamapper\MaintenanceGroupMapper::getInstance()->getAll(
					null,
					$customerId
				)
			);
		}
		else {
			$tickets = datamapper\MaintenanceTicketMapper::getInstance()->getAll(
				$groupId,
				$userId,
				null,
				null,
				null,
				null,
				false,
				null,
				null,
				model\Permission::MAINTENANCE_READ,
				util\IteratorUtil::getKeyArray($customers)
			);
			$this->select['users'] = util\IteratorUtil::getKeyValueArray(
				datamapper\UserMapper::getInstance()->getAllByPermission(
					model\Permission::MAINTENANCE_WRITE,
					util\IteratorUtil::getKeyArray($customers),
					true
				),
				'id',
				'fullName'
			);
			$this->select['groups'] = util\IteratorUtil::getKeyValueArray(
				datamapper\MaintenanceGroupMapper::getInstance()->getAll(
					null,
					util\IteratorUtil::getKeyArray($customers)
				)
			);
		}
		$this->parameters['allTickets'] = $tickets;
	}

	/**
	 * Shows the details field for bulk owner
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getBulkOwnerDetailsAction() {
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAllByPermission(
				model\Permission::MAINTENANCE_WRITE,
				util\IteratorUtil::getKeyArray($customers),
				true
			),
			'id',
			'fullName'
		);
	}

	/**
	 * Shows the details field for bulk close
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getBulkCloseDetailsAction() {
		$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceTagMapper::getInstance()->getAll()
		);
	}

	/**
	 * Shows the details field for bulk comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getBulkCommentDetailsAction() {
		$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceTagMapper::getInstance()->getAll()
		);
	}

	/**
	 * Shows the details field for bulk due date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getBulkDateDetailsAction() {
		$this->select['tags'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceTagMapper::getInstance()->getAll()
		);
	}

	/**
	 * Return hosts according to customerId
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function getHostsAction() {
		$customerId = $this->request->getInt('customerId', request\REQUEST_GET);

		if ($customerId == 0) {
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				datamapper\HostMapper::getInstance()->getAll(),
				'id',
				'hostname'
			);
		}
		else {
			$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
				datamapper\HostMapper::getInstance()->getAll(
					null,
					false,
					false,
					null,
					$customerId
				),
				'id',
				'hostname'
			);
		}
	}

	/**
	 * Helper function for getting all
	 * hosts and their customers for
	 * a given ticket in a single (line) string.
	 *
	 * @param         model\MaintenanceTicket $ticket
	 *
	 * @returns       string
	 */
	private function getTicketHostsString($ticket) {
		// Get hosts for ticket
		$ticketHosts = $ticket->getTicketHosts();
		$ticketHostsString = '';

		foreach ($ticketHosts as $ticketHost) {
			$customerName = $ticketHost->getHost()->getCustomer()->name;

			$ticketHostsString .= sprintf(
				"%s (%s), ",
				$ticketHost->getHost()->hostname,
				isset($customerName) ? $customerName : 'None'
			);
		}

		$ticketHostsString = substr(
			$ticketHostsString, 0, -2
		);

		return $ticketHostsString;
	}

	/**
	 * Helper function for getting the default
	 * recipient if possible or using a fallback
	 * as default recipient.
	 *
	 * @param         array  $fallback    list of associative arrays with receiver information
	 *                                    array(
	 *                                        array('firstName'=>'David', 'lastName'=>'Vogt', 'address'=>'dv@adfinis.ch'),
	 *                                        ...
	 *                                    )
	 *
	 * @returns       array $recipient
	 */
	private function getDefaultRecipient(array $fallback) {
		// Get default recipient if possible,
		// or fall back to given fallback
		try {
			$recipient = array(
				array(
					'firstName' => 'SyMonitoring',
					'lastName'  => 'Mailerdaemon',
					'address'   => ConfManager::getVar('af', 'mail', 'fromSystem', 'fromMail')
				)
			);
		}
		catch (\OutOfRangeException $e) {
			$recipient = $fallback;
		}

		return $recipient;
	}

	/**
	 * Gets all maintenance tickets and assigns them to parameters
	 *
	 * @return
	 * @returns       void
	 */
	private function reloadTickets() {
		$customers = datamapper\CustomerMapper::getInstance()->getAll(
			model\Permission::MAINTENANCE_READ
		);
		$tickets = datamapper\MaintenanceTicketMapper::getInstance()->getAll(
			null,
			null,
			null,
			null,
			null,
			null,
			false,
			null,
			null,
			model\Permission::MAINTENANCE_READ,
			util\IteratorUtil::getKeyArray($customers)
		);
		$this->parameters['allTickets'] = $tickets;
	}
}
