<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle maintenance group page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceGroupController extends MaintenanceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCEGROUPS_READ
		);

		$this->setHeaderTitle(tr("Maintenance groups"));
		$this->parameters['allGroups'] = datamapper\MaintenanceGroupMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'maintenance/groups.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'maintenance/groupsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Show the form to add a maintenance group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$this->parameters['group'] = new model\MaintenanceGroup();
		$this->parameters['group']->allowRead();

		$this->assignFormVariables();
	}

	/**
	 * Show the form to edit a maintenance group
	 *
	 * @throws        UserException
	 * @returns       void
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Maintenance group id is missing")
			);
		}

		$this->parameters['group'] = datamapper\MaintenanceGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::MAINTENANCEGROUPS_WRITE
		);

		$this->assignFormVariables();
	}

	/**
	 * Assigns variables for the maintenance group form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::MAINTENANCE_READ
			)
		);
		$this->select['hosts']  = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				null,
				model\Permission::MAINTENANCEGROUPS_READ
			),
			'id',
			'hostname'
		);

		$this->select['cycles'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceCycleMapper::getInstance()->getAll()
		);
	}

	/**
	 * Create a new maintenance group and invoke the save
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function addAction() {
		$maintenanceGroup = new model\MaintenanceGroup();
		$maintenanceGroup->allowWrite(model\Permission::MAINTENANCEGROUPS_WRITE);

		$this->saveMaintenanceGroup($maintenanceGroup);
	}

	/**
	 * Save an existing maintenance group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		$maintenanceGroup = datamapper\MaintenanceGroupMapper::getInstance()->getByID($id);
		$maintenanceGroup->enforceWritePermission(
			model\Permission::MAINTENANCEGROUPS_WRITE
		);

		$this->saveMaintenanceGroup($maintenanceGroup);
	}

	/**
	 * Save the data of a created or edited group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\MaintenanceGroup $maintenanceGroup
	 * @returns       void
	 */
	private function saveMaintenanceGroup($maintenanceGroup) {
		$dateStart = $this->request->getString('dateStart');

		$maintenanceGroup->customerId         = $this->request->getInt('customerId');
		$maintenanceGroup->name               = $this->request->getString('name');
		$maintenanceGroup->description        = $this->request->getString('description');
		$maintenanceGroup->dateStart          = empty($dateStart) ? null : new \Zend_Date($dateStart);
		$maintenanceGroup->maintenanceCycleId = $this->request->getInt('maintenanceCycleId') ?: null;

		$maintenanceGroup->save();

		$this->parameters['group'] = $maintenanceGroup;
		$this->parameters['allGroups'][] = $maintenanceGroup;
	}

	/**
	 * Delete a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Group id is missing")
			);
		}

		$maintenanceGroup = datamapper\MaintenanceGroupMapper::getInstance()->getByID($id);
		$maintenanceGroup->enforceWritePermission(
			model\Permission::MAINTENANCEGROUPS_WRITE
		);
		$maintenanceGroup->delete();
	}

	/**
	 * Add a host to a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addHostAction() {
		$groupMember = new model\MaintenanceGroupMember();
		$groupMember->allowWrite(model\Permission::MAINTENANCEGROUPS_WRITE);

		$groupMember->groupId = $this->request->getInt('groupId');
		$groupMember->hostId  = $this->request->getInt('hostId');

		if (datamapper\MaintenanceGroupMemberMapper::getInstance()->existsByGroupAndHost(
			$groupMember->groupId,
			$groupMember->hostId
		)) {
			throw new UserException(
				tr('This host is already assigned')
			);
		}

		$groupMember->save();

		$this->parameters['group'] = $groupMember->getMaintenanceGroup();
	}

	/**
	 * Show the group hosts form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function groupHostsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				null,
				model\Permission::MAINTENANCEGROUPS_READ
			),
			'id',
			'hostname'
		);

		$this->parameters['group'] = datamapper\MaintenanceGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::MAINTENANCEGROUPS_WRITE
		);
	}

	/**
	 * Remove a group member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteHostAction() {
		$memberId = $this->request->getInt('memberId');

		if (empty($memberId)) {
			throw new UserException(
				tr("Group member id is missing")
			);
		}

		$member = datamapper\MaintenanceGroupMemberMapper::getInstance()->getByID(
			$memberId
		);
		$member->enforceWritePermission(
			model\Permission::MAINTENANCEGROUPS_WRITE
		);
		$member->delete();

		$this->parameters['group'] = $member->getMaintenanceGroup();
	}

	/**
	 * Assignes variables to the template for the checklists
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignVariablesForChecklist($id) {
		$this->select['checklists'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceChecklistMapper::getInstance()->getAll(null, null, true)
		);

		$this->parameters['group'] = datamapper\MaintenanceGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::MAINTENANCE_READ
		);
		
		$this->parameters['checklists'] = $this->parameters['group']->getTemplateChecklists();
	}


	/**
	 * Shows all assigned templates and the questions of the main checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function checklistsAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);
		$this->assignVariablesForChecklist($id);
	}

	/**
	 * Add a checklist to a group
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addChecklistAction() {
		$groupId  = $this->request->getInt('id');
		$checklistId  = $this->request->getInt('checklistId');

		try {
			datamapper\MaintenanceChecklistMapper::getInstance()->assignGroup($groupId, $checklistId);
		}
		catch(\Zend_Db_Statement_Exception $e) {
			if($e->getCode() == 23000) {
				throw new UserException(tr('This checklist is already assigned to this group.'));
			}
			else {
				throw $e;
			}
		}

		$this->assignVariablesForChecklist($groupId);
	}

	/**
	 * Delete a checklist from a group
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteChecklistAction() {
		$id = $this->request->getInt('id');
		$checklistId = $this->request->getInt('checklistId');

		if (empty($id)) {
			throw new UserException(
				tr("Group id is missing")
			);
		}

		if (empty($checklistId)) {
			throw new UserException(
				tr("Checklist id is missing")
			);
		}

		//remove the assignment
		datamapper\MaintenanceChecklistMapper::getInstance()->unassignGroup($id, $checklistId);
		
		$this->assignVariablesForChecklist($id);
	}

	/**
	 * Creates a new question for the group
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addQuestionAction() {
		$groupId = $this->request->getInt('groupId');

		$checklist = datamapper\MaintenanceGroupMapper::getInstance()->getByID($groupId)->getPrimaryChecklist();
		$checklist->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		
		$question = new model\MaintenanceChecklistQuestion();
		$question->name    = $this->request->getString('name');
		$question->comment = $this->request->getString('description');
		$question->save();

		datamapper\MaintenanceChecklistMapper::getInstance()->addQuestion($question->id, $checklist->id);
		
		$this->assignVariablesForChecklist($groupId);
	}

	/**
	 * Removes the assignement from question to checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function deleteQuestionAction() {
		$groupId      = $this->request->getInt('groupId');
		$questionId   = $this->request->getInt('questionId');

		$checklist = datamapper\MaintenanceGroupMapper::getInstance()->getByID($groupId)->getPrimaryChecklist();
		$checklist->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);

		datamapper\MaintenanceChecklistMapper::getInstance()->removeQuestion($questionId, $checklist->id);

		$this->assignVariablesForChecklist($groupId);
	}

	/**
	 * Excludes/Includes a question from a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function toggleQuestionAction() {
		$id         = $this->request->getInt('id');
		$questionId = $this->request->getInt('questionId');
		$groupId     = $this->request->getInt('groupId');
		$question = datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getByID($questionId);
		datamapper\MaintenanceChecklistQuestionMapper::getInstance()->excludeQuestion(
			$id,
			$questionId,
			null,
			$groupId,
			!$question->isExcluded($id, null, $groupId)
		);
	}
}
