<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\utils;

/**
 * Handle maintenance host page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceHostController extends MaintenanceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCE_ADMIN
		);

		$this->setHeaderTitle(tr("Maintenance hosts"));

		$this->select['users'] = util\IteratorUtil::getKeyValueArray(
			datamapper\UserMapper::getInstance()->getAll(null, true)
		);

		$this->tpl->assign('template', 'maintenance/hosts.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'maintenance/hostsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Bulk add hosts for maintenance
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function bulkAddAction() {
		$allHosts = $this->request->getBoolean('allHosts');
		$hosts    = $this->request->getRaw('hosts');
		$userId   = $this->request->getInt('userId') ?: null;


		if (!$allHosts && (!is_array($hosts) || count($hosts) == 0)) {
			throw new UserException(
				tr("No hosts chosen")
			);
		}

		$maintenanceHost = new model\MaintenanceTicketHost();
		$maintenanceHost->allowWrite(model\Permission::MAINTENANCE_WRITE);

		/*
		 * A ticket can either be directly assigned to a user or free to take.
		 * Dependent on this the initial status and substatus are different.
		 */
		$maintenanceHost->stateId = $userId
		                           ? model\MaintenanceState::STATE_IN_PROGRESS
		                           : model\MaintenanceState::STATE_NEW;

		$maintenanceTicket = new model\MaintenanceTicket();
		$maintenanceTicket->allowWrite(model\Permission::MAINTENANCE_WRITE);
		$maintenanceTicket->userId  = $userId;
		$maintenanceTicket->dateDue = null;
		$maintenanceTicket->dateCreated  = new \Zend_Date();

		$maintenanceTicket->save();

		// send email to assigned user and to the maintenance master
		$user    = $maintenanceTicket->getUser();
		$creator = $this->session->user;
		
		if($user && $user->email != null) {
			$subject = sprintf(
				'Maintenance ticket #%d was assigned to you.',
				$maintenanceTicket->id
			);

			$text = sprintf(
				"Maintenance ticket #%d has been assigned to you by %s.",
				$maintenanceTicket->id,
				$creator->fullName
			);

			utils\MailHelper::sendMail(
				array(
					array(
						'firstName' => '',
						'lastName'  => $user->fullName,
						'address'   => $user->email
					)
				),
				$subject,
				$text,
				'',
				true
			);
		}
		

		$masters = datamapper\UserMapper::getInstance()->getAllByPermission(
			model\Permission::MAINTENANCE_MASTER,
			null,
			true
		);

		$mastersArray = array();
		foreach ($masters as $master) {
			// You probably don't want to get your own interactions
			if ($master->id == $this->session->user->id || empty($master->email)) {
				continue;
			}

			$mastersArray[] = array(
				'firstName' => '',
				'lastName'  => $master->fullName,
				'address'   => $master->email
			);
		}

		if (count($mastersArray) > 0) {
			// And now notify the maintenance masters
			$subject = sprintf(
				'Maintenance ticket #%d was assigned to another user.',
				$maintenanceTicket->id
			);

			$text = sprintf(
				"Maintenance ticket #%d has been assigned to %s by %s.",
				$maintenanceTicket->id,
				$user->fullName,
				$creator->fullName
			);

			utils\MailHelper::sendMail(
				$mastersArray,
				$subject,
				$text,
				'',
				true
			);
		}


		$maintenanceHost->ticketId = $maintenanceTicket->id;

		if ($allHosts) {
			$hosts = datamapper\HostMapper::getInstance()->getAll(
				true,
				false,
				false,
				null,
				null,
				model\Permission::MAINTENANCE_READ
			);
		}
		foreach ($hosts as $host) {
			if (is_numeric($host)) {
				$host = datamapper\HostMapper::getInstance()->getByID(
					$host
				);
			}
			$host->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);

			$maintenanceHost->id     = null;
			$maintenanceHost->hostId = $host->id;
			$maintenanceHost->save();
		}
	}

	/**
	 * Add a checklist to a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addChecklistAction() {
		$hostId      = $this->request->getInt('id');
		$checklistId = $this->request->getInt('checklistId');

		try {
			datamapper\MaintenanceChecklistMapper::getInstance()->assignHost($hostId, $checklistId);
		}
		catch(\Zend_Db_Statement_Exception $e) {
			if($e->getCode() == 23000) {
				throw new UserException(tr('This checklist is already assigned to this host.'));
			}
			else {
				throw $e;
			}
		}

		$this->assignVariablesForChecklist($hostId);
	}

	/**
	 * Delete a checklist from a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteChecklistAction() {
		$id          = $this->request->getInt('id');
		$checklistId = $this->request->getInt('checklistId');

		if (empty($id)) {
			throw new UserException(
				tr("Host id is missing")
			);
		}

		if (empty($checklistId)) {
			throw new UserException(
				tr("Checklist id is missing")
			);
		}

		//remove the assignment
		datamapper\MaintenanceChecklistMapper::getInstance()->unassignHost($id, $checklistId);

		$this->assignVariablesForChecklist($id);
	}

	/**
	 * Show checklist of a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function checklistsAction() {
		$hostId = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($hostId)) {
			throw new UserException(
				tr("Host id is missing")
			);
		}

		$this->assignVariablesForChecklist($hostId);
	}

	/**
	 * Assigns variables to the template for the checklists
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignVariablesForChecklist($hostId) {
		$this->parameters['host'] = datamapper\HostMapper::getInstance()->getByID($hostId);
		$this->parameters['host']->enforceReadPermission(
			model\Permission::MAINTENANCE_READ
		);

		$this->parameters['checklists'] = $this->parameters['host']->getTemplateChecklists();
		$this->select['checklists'] = util\IteratorUtil::getKeyValueArray(
			datamapper\MaintenanceChecklistMapper::getInstance()->getAll(null, null, true)
		);
	}

	/**
	 * Creates a new question for the host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function addQuestionAction() {
		$hostId      = $this->request->getInt('hostId');
		$checklistId = $this->request->getInt('primaryChecklistId');

		$checklist = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($checklistId);
		$checklist->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		
		$question = new model\MaintenanceChecklistQuestion();
		$question->name    = $this->request->getString('name');
		$question->comment = $this->request->getString('description');
		$question->save();

		datamapper\MaintenanceChecklistMapper::getInstance()->addQuestion($question->id, $checklistId);
		
		$this->assignVariablesForChecklist($hostId);
		$this->tpl->assign('question', $question);
	}


	/**
	 * Removes the assignement from question to checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function deleteQuestionAction() {
		$hostId      = $this->request->getInt('hostId');
		$checklistId = $this->request->getInt('checklistId');
		$questionId  = $this->request->getInt('questionId');

		$this->parameters['checklist'] = datamapper\MaintenanceChecklistMapper::getInstance()->getByID($checklistId);
		$this->parameters['checklist']->enforceReadPermission(model\Permission::MAINTENANCE_WRITE);
		$this->parameters['questions'] = datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getAll($checklistId);

		datamapper\MaintenanceChecklistMapper::getInstance()->removeQuestion($questionId, $checklistId);
		$this->assignVariablesForChecklist($hostId);
	}

	/**
	 * Excludes/Includes a question from a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	protected function toggleQuestionAction() {
		$id         = $this->request->getInt('id');
		$questionId = $this->request->getInt('questionId');
		$hostId     = $this->request->getInt('hostId');
		$question = datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getByID($questionId);
		datamapper\MaintenanceChecklistQuestionMapper::getInstance()->excludeQuestion(
			$id,
			$questionId,
			$hostId,
			null,
			!$question->isExcluded($id, $hostId, null)
		);
	}

	/**
	 * Handles DataTable requests
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        string json
	 */
	public function dataTableAction() {
		$this->session->user->enforcePermission(
			model\Permission::MAINTENANCE_READ
		);

		$settings = $_GET;
		unset($settings['action']);

		$hosts = datamapper\HostMapper::getInstance()->getAllForDataTable(
			$settings,
			model\Permission::MAINTENANCE_READ,
			'ch\monitoring\model\MaintenanceHost'
		);

		header('Content-Type: application/json; charset=utf-8');
		print json_encode($hosts->toDataTable());
		exit;
	}
}
