<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle nagios/icinga notification page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class NotificationController extends LogBaseController {

	/**
	 * Status codes signification
	 *
	 * @var           array $statusCodes
	 */
	private $statusCodes;

	/**
	 * Initializes the status codes
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function __construct($request, $urlMatch) {
		parent::__construct($request, $urlMatch);

		$this->statusCodes = static::getStatusCodes();
	}

	/**
	 * Get all the possible status codes
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        array
	 */
	public static function getStatusCodes() {
		return array(
			model\NagiosLog::STATE_OPEN             => tr("Open"),
			model\NagiosLog::STATE_ALERTED          => tr("An alert was created"),
			model\NagiosLog::STATE_NO_SLA           => tr("Not within SLA"),
			model\NagiosLog::STATE_MALFORMED_ROW    => tr("Malformed row"),
			model\NagiosLog::STATE_HOST_DISABLED    => tr("Host disabled or configuration error"),
			model\NagiosLog::STATE_ALERT_CLOSED     => tr("Alert closed"),
			model\NagiosLog::STATE_ALERT_REPETITION => tr("Alert repetition"),
			model\NagiosLog::STATE_NO_TEAM          => tr("No on-call team members defined"),
			model\NagiosLog::STATE_IN_MAINTENANCE   => tr("In maintenance mode"),
			model\NagiosLog::STATE_SOFT             => tr("SOFT state notification"),
			model\NagiosLog::HOST_DOWN              => tr("Whole host is down"),
			model\NagiosLog::NO_PREVIOUS_DOWN       => tr("OK message for a not existing alert"),
			model\NagiosLog::DEPENDENCY_DOWN        => tr("A host dependency is down"),
			model\NagiosLog::DOES_NOT_EXIST         => tr("The alert does not exist")
		);
	}

	/**
	 * Show notifications
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::NOTIFICATIONS_READ
		);

		$this->setHeaderTitle(tr("Notifications"));

		$limit = 50;

		$this->parameters['dateFrom']    = null;
		$this->parameters['dateTo']      = null;
		$this->parameters['statusCodes'] = $this->statusCodes;
		$this->parameters['limit']       = $limit;

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::NOTIFICATIONS_READ
			)
		);
		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				null,
				model\Permission::NOTIFICATIONS_READ
			),
			'id',
			'hostname'
		);

		$this->tpl->assign('template', 'logs/notifications.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::NOTIFICATIONS_READ
		);

		$this->callActionHandler();

		$this->tpl->assign('template', 'logs/notificationsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Shows log entries according to filter criteria
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function showNotificationsAction() {
		$dateFrom   = new \Zend_Date(
			$this->request->getString('dateFrom'),
			\Zend_Date::ISO_8601
		);
		$dateTo     = new \Zend_Date(
			$this->request->getString('dateTo'),
			\Zend_Date::ISO_8601
		);
		// add 24h hours, or else it'll be $dateTo 00:00
		$dateTo->add('24', \Zend_Date::HOUR);

		$timeFrom   = $this->getTime($this->request->getString('timeFrom'));
		$timeTo     = $this->getTime($this->request->getString('timeTo'));

		$customerId = $this->request->getInt('customerId') ?: null;
		$hostId     = $this->request->getInt('hostId') ?: null;
		$hostname   = $hostId
			? datamapper\HostMapper::getInstance()->getByID($hostId)->allowRead()->hostname
			: null;

		$alerted    = $this->request->keyExists('alerted');

		$limit      = $this->request->getInt('limit');

		$notifications = datamapper\NagiosLogMapper::GetInstance()->getAll(
			$dateFrom,
			$dateTo,
			$timeFrom,
			$timeTo,
			$customerId,
			$hostname,
			$alerted ?: null
		);
		$notifications->setOffsetLimit(0, $limit);
		$this->parameters['allNotifications'] = $notifications;

		$this->parameters['statusCodes'] = $this->statusCodes;
	}

	/**
	 * Check the time and return it
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $time The Time
	 * @returns       string
	 */
	private function getTime($time) {
		if (!preg_match('/^(\d{1,2}):(\d{2})$/', $time, $matches)) {
			return null;
		}

		$hours   = $matches[1] > 23 ? 23 : $matches[1];
		$minutes = $matches[2] > 59 ? 59 : $matches[2];

		return sprintf("%02d:%02d", $hours, $minutes);
	}

	/**
	 * Get hosts for a customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getCustomerHostsAction() {
		$id = $this->request->getInt('customerId', request\REQUEST_GET) ?: null;

		$this->select['hosts'] = util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				$id === null ? null : $id,
				model\Permission::NOTIFICATIONS_READ
			),
			'id',
			'hostname'
		);
	}

	/**
	 * Handles DataTable requests
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        string json
	 */
	public function dataTableAction() {
		$this->session->user->enforcePermission(
			model\Permission::NOTIFICATIONS_READ
		);
		$settings = $_GET;
		unset($settings['action']);

		$notifications = datamapper\NagiosLogMapper::getInstance()->getAllForDataTable($settings);

		header('Content-Type: application/json; charset=utf-8');
		print json_encode($notifications->toDataTable());
		exit;
	}
}
