<?php

namespace ch\monitoring\controller;

use DateTime;
use org\adfinis\framework\request;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;

/**
 * Handle reports page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ReportController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::REPORTS
		);
		$this->session->user->enforcePermission(
			model\Permission::BREAPPLICATION_READ
		);

		$this->setHeaderTitle(tr("Reports"));

		$this->tpl->assign('template', 'reports/reports.tpl');

		if (!empty($this->action)) {
			$this->callActionHandler();
		}
		else {
			// it's the first call
			$dateTo   = new \Zend_Date();
			$dateFrom = clone $dateTo;
			$dateFrom->subMonth(1);

			$this->parameters['alertsArray']         = null;
			$this->parameters['dateFrom']            = $dateFrom;
			$this->parameters['dateTo']              = $dateTo;
			$this->parameters['showMaintenanceInfo'] = false;

			//
			$customerId = datamapper\CustomerMapper::getInstance()->getDefaultCustomer(
				model\Permission::REPORTS
			);
			$this->parameters['customer'] = datamapper\CustomerMapper::getInstance()->getByID(
				$customerId
			)->allowRead();

			// sort the hosts by customer into option groups
			$hosts = array();
			$hostInstances = datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				false,
				null,
				$customerId,
				model\Permission::REPORTS
			);
			foreach ($hostInstances as $hostInstance) {
				$optgroups[] = $hostInstance->getCustomer()->name;
				$hosts[$hostInstance->getCustomer()->name][$hostInstance->id] = $hostInstance->hostname;
			}

			// sort the applications by customer into option groups
			$applications = datamapper\BREApplicationMapper::getInstance()->getApplications();
			foreach ($applications as $customer => $application) {
				$optgroups[] = $customer;
			}

			$this->select['optgroups'] = $optgroups;


			$optgroups = array_unique($optgroups);
			sort($optgroups, SORT_STRING + SORT_FLAG_CASE | SORT_NATURAL);

			$this->select['optgroups'] = $optgroups;
			$this->select['hosts'] = $hosts;
			$this->select['applications'] = $applications;
		}

		// TODO: is this obsolete?
		$this->tpl->registerClass("AlertTag", "\ch\monitoring\model\AlertTag");

		header('Content-Type: text/html; charset=utf-8');
		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');
		$this->callActionHandler();
	}

    private function makeRPCRequest($functionName, $data) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);
        $resultId = $rpcClient->putRequest($functionName, $data);

		return $rpcClient->getResult($resultId);
    }

	/**
	 * Show a report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function getReportAction() {
		// get the ids of the objects which need a report
		$hostIds = array();
		$applicationIds = array();
		if (count($this->request->getRaw('hostIds', request\REQUEST_GET))) {
			foreach ($this->request->getRaw('hostIds', request\REQUEST_GET) as $hostId) {
				array_push($hostIds, (int)$hostId);
			}
		}
		if (count($this->request->getRaw('applicationIds', request\REQUEST_GET))) {
			foreach ($this->request->getRaw('applicationIds', request\REQUEST_GET) as $appId) {
				array_push($applicationIds, (int)$appId);
			}
		}
		// get the start and end date to generate the report for
		$start   = $this->request->getString('start', request\REQUEST_GET);
        $end     = $this->request->getString('end', request\REQUEST_GET);

		# TODO: what to do with the maintenance info?
		$data = array(
			'host_ids'        => $hostIds,
			'application_ids' => $applicationIds,
			'start'           => $start,
			'end'             => $end,
        );

        $result = $this->makeRPCRequest(
            'symonitoring_reporting.get_aggregated_facts',
            $data
        );
		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}
		$this->parameters['facts'] = $result;
		$this->outputAjaxPage(null);
	}

	/**
	 * Show a report
	 *
	 * @author        Pablo Verges <pablo.verges@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getIncidentsAction() {
		// get the start and end date to generate the report for
		$id      = $this->request->getInt('id', request\REQUEST_GET);
		$type    = $this->request->getString('type', request\REQUEST_GET);
		$start   = $this->request->getString('start', request\REQUEST_GET);
        $end     = $this->request->getString('end', request\REQUEST_GET);

		// prepare the data structure with the parameters for the api request
		$data = array(
			'start' => $start,
            'end'   => $end,
            'id'    => $id
        );

        $rpcClientFunction = '';
		if ($type == 'host') {
			$rpcClientFunction = 'symonitoring_reporting.get_incidents_for_host';
		}
        elseif ($type == 'application') {
			$rpcClientFunction = 'symonitoring_reporting.get_incidents_for_application';
        }
        elseif ($type == 'service') {
            $rpcClientFunction = 'symonitoring_reporting.get_incidents_for_service';
        }

		$result = $this->makeRPCRequest($rpcClientFunction, $data);
		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}
		$this->parameters['incidents'] = $result;
		$this->outputAjaxPage(null);
	}

    protected function getPrintViewAction() {
		$hostIds = array();
        $applicationIds = array();

        if ($this->request->keyExists('hostIds', request\REQUEST_GET)) {
            foreach ($this->request->getRaw('hostIds', request\REQUEST_GET) as $hostId) {
                array_push($hostIds, (int)$hostId);
            }
        }
        if ($this->request->keyExists('applicationIds', request\REQUEST_GET)) {
            foreach ($this->request->getRaw('applicationIds', request\REQUEST_GET) as $appId) {
                array_push($applicationIds, (int)$appId);
            }
        }

		$start = $this->request->getString('start', request\REQUEST_GET);
        $end   = $this->request->getString('end', request\REQUEST_GET);

		$data = array(
			'host_ids'        => $hostIds,
			'application_ids' => $applicationIds,
			'start'           => $start,
			'end'             => $end,
        );

        $facts = $this->makeRPCRequest(
            'symonitoring_reporting.get_aggregated_facts',
            $data
        );

		if (isset($facts->exception)) {
			throw new UserException($facts->exception);
        }

        // Get alerts for each host and his services
        $hostData = array(
            'start'   => $start,
            'end'     => $end,
            'host_id' => null
        );
        foreach($facts->data->hosts as $host) {
            $hostData['host_id'] = $host->id;
            $issues = $this->makeRPCRequest(
                'symonitoring_reporting.get_alerts_for_host_inc_services',
                $hostData
            );
            if (isset($issues->exception)) {
                throw new UserException($issues->exception);
            }
            $host->issue_list = (object)array('alerts' => $issues->data);
        }

        // Get states for each application
        $appData = array(
            'start' => $start,
            'end'   => $end,
            'id'    => null
        );
        foreach($facts->data->applications as $app) {
            $appData['id'] = $app->id;
            $issues = $this->makeRPCRequest(
                'symonitoring_reporting.get_incidents_for_application',
                $appData
            );
            if (isset($issues->exception)) {
                throw new UserException($issues->exception);
            }
            $app->issue_list = $issues->data;
        }

        $this->parameters['facts'] = $facts;

        $this->tpl->registerPlugin('modifier', 'humantime', array($this, 'humantime'));
        $this->tpl->registerPlugin('modifier', 'parseDatetime', array($this, 'parseDatetime'));

        $start = new DateTime($start);
        $end   = new DateTime($end);

        $this->tpl->assign('facts', $facts);
        $this->tpl->assign('dateStart', $start->format('d.m.Y'));
        $this->tpl->assign('dateEnd', $end->format('d.m.Y'));
        $this->tpl->assign('totalDayRange', $facts->data->total_seconds / 3600 / 24);
        $this->outputAjaxPage('reports/print.tpl');
    }

    public function humantime($seconds) {
        $results = array();
        // days
        if ($seconds >= 24*60*60) {
          array_push($results, floor($seconds / (24*60*60)) . "d");
          $seconds = $seconds % (24*60*60);
        }
        // hours
        if ($seconds >= 60*60) {
          array_push($results, floor($seconds / (60*60)) . "h");
          $seconds = $seconds % 3600;
        }
        // minutes
        if ($seconds >= 60) {
          array_push($results, floor($seconds / 60) . "m");
          $seconds = $seconds % 60;
        }
        // seconds
        if ($seconds > 0) {
          array_push($results, $seconds . "s");
        }
        // total_seconds was 0, then results is going to be empty,
        // if it's empty send '0s', else sende the computed results
        return count($results) > 0 ? implode(' ', $results) : '0s';
    }

    public function parseDatetime($datetime) {
        $parsed = new DateTime();
        $parsed->setDate(
            $datetime->year,
            $datetime->month,
            $datetime->day
        );
        $parsed->setTime(
            $datetime->hour,
            $datetime->minute,
            $datetime->second
        );

        return $parsed->format('d.m.Y H:i');
    }
}
