<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;

/**
 * Handle role page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 */
class RoleController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		//$this->addScriptFile('js/ajaxForm.js', self::SCRIPT_POSITION_TOP);

		$this->session->user->enforcePermission(
			model\Permission::ROLES_READ
		);

		$this->setHeaderTitle(tr("Roles"));
		$this->parameters['allRoles'] = datamapper\RoleMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'users/roles.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'users/rolesAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Load data needed to display the form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['permissions'] = array();
		$permissions = datamapper\PermissionMapper::getInstance()->getAll();
		
		foreach($permissions as $permission) {
			$this->select['permissions'][$permission->id] = $permission->name;
			if($permission->isGlobal()) {
				$this->select['permissions'][$permission->id] .= " ".tr("(Global)");
			}
		}

		$this->parameters['roles'] = datamapper\RoleMapper::getInstance()->getAll();
	}

	/**
	 * Edit a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Role id is missing")
			);
		}

		$this->parameters['role'] = datamapper\RoleMapper::getInstance()->getByID($id);
		$this->parameters['role']->enforceReadPermission(
			model\Permission::ROLES_WRITE
		);

		$assignedRoles = datamapper\RoleMapper::
			getInstance()->
			getAssignedRoleIds($id);

		if (empty($assignedRoles)) {
			$assignedRoles = array();
		}

		$this->parameters['assignedRoles'] = $assignedRoles;

		$this->assignFormVariables();
	}

	/**
	 * Add a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addFormAction() {
		$role = new model\Role();
		$role->allowRead();

		$this->parameters['role']          = $role;
		$this->parameters['assignedRoles'] = array();

		$this->assignFormVariables();
	}

	/**
	 * Save a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Role id is missing")
			);
		}

		$role = datamapper\RoleMapper::getInstance()->getByID($id);
		$role->enforceWritePermission(
			model\Permission::ROLES_WRITE
		);

		$this->saveRole($role);
	}

	/**
	 * Add a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$role = new model\Role();
		$role->allowWrite(model\Permission::ROLES_WRITE);
		$this->saveRole($role);
	}

	/**
	 * Save a role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Role $role
	 * @returns       void
	 */
	private function saveRole($role) {
		$role->name        = $this->request->getString('name');
		$role->description = $this->request->getString('description');
		$role->save();

		datamapper\PermissionMapper::getInstance()->deleteAssociations($role->id);
		$rolePermissions = $this->request->getRaw('rolePermissions');
		foreach ($rolePermissions as $rolePermission) {
			$role->addPermission($rolePermission);
		}

		$assignedRoles = $this->request->getRaw('assignedRoles');
		datamapper\RoleMapper::getInstance()->removeAllRoles($role->id);

		if (!empty($assignedRoles)) {
			foreach ($assignedRoles as $assignedRole) {
				datamapper\RoleMapper::getInstance()->addRole($role->id, $assignedRole);
			}
		}

		$this->parameters['role'] = $role;
		$this->parameters['allRoles'][] = $role;
		
		// Cache needs to be updated
		Application::updateCache();
	}

	/**
	 * Delete a role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Role id is missing")
			);
		}

		$role = datamapper\RoleMapper::getInstance()->getByID(
			$id
		);
		$role->enforceWritePermission(
			model\Permission::ROLES_WRITE
		);
		$role->delete();
	}
}
