<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\config\ConfManager;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Handle service page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceController extends ServiceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::SERVICES_READ
		);

		$this->setHeaderTitle(tr("Services"));
		$this->parameters['allServices']  = datamapper\ServiceMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'services/services.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'services/servicesAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Show the form to add a service
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$service = new model\Service();
		$service->allowRead();

		$service->flapping           = 1;
		$service->max_check_attempts = 5;
		$service->check_interval     = 5;
		$service->retry_interval     = 1;
		$service->freshness          = 0;

		// TODO Probably handle in JavaScript?
		// Assigns the first possible command as default
		$commands = datamapper\CommandMapper::getInstance()->getAll();
		$commands->rewind();
		$service->commandId = $commands->current()->id;

		$this->parameters['service'] = $service;

		$serviceArgument = new model\ServiceArgument();
		$serviceArgument->allowRead();
		$this->parameters['arguments'] = array($serviceArgument);

		$this->parameters['tempId'] = 1;

		$this->assignFormVariables();
	}

	/**
	 * Edit a service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Service id is missing")
			);
		}

		$this->parameters['service'] = datamapper\ServiceMapper::getInstance()->getByID($id);
		$this->parameters['service']->enforceReadPermission(
			model\Permission::SERVICES_WRITE
		);

		$arguments = $this->parameters['service']->getArguments();
		$this->parameters['arguments'] = $arguments;
		$tempId = 0;

		foreach($arguments as $argument) {
			if($tempId < $argument->id) {
				$tempId = $argument->id;
			}
		}
		$this->parameters['tempId'] = $tempId;

		$this->assignFormVariables();
	}

	/**
	 * Assign variables to the forms that show up in edit and add
	 * @author       Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns      void
	 */
	private function assignFormVariables() {
		$this->select['commands']  = util\IteratorUtil::getKeyValueArray(
			datamapper\CommandMapper::getInstance()->getAll()
		);
		$this->select['services']  = util\IteratorUtil::getKeyValueArray(
			datamapper\ServiceMapper::getInstance()->getAll()
		);
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::SERVICES_READ
			)
		);
		$this->select['slas']      = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::SERVICES_READ
			)
		);

		// For access to the ServiceArgument types
		$this->tpl->registerClass(
			"ServiceArgument",
			"\ch\monitoring\model\ServiceArgument"
		);
	}

	/**
	 * Add a new service to the DB
	 * 
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addAction() {
		$service = new model\Service();
		$service->allowWrite(model\Permission::SERVICES_WRITE);

		$this->saveService($service);
	}

	/**
	 * Save/edit an existing service
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		$service = datamapper\ServiceMapper::getInstance()->getByID($id);
		$service->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$this->saveService($service);
	}
	
	/**
	 * Add or save a service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Service $service
	 * @throws        UserException
	 * @returns       void
	 */
	private function saveService($service) {
		// Transaction because service must not be saved when error in arguments
		Application::getDbInstance()->beginTransaction();

		$service->slaId            = $this->request->getInt('slaId') ?: null;
		$service->customerId       = $this->request->getInt('customerId') ?: null;
		$service->name             = $this->request->getString('name');
		$service->description      = $this->request->getString('description');
		$service->maxCheckAttempts = $this->request->getInt('maxCheckAttempts');
		$service->checkInterval    = $this->request->getInt('checkInterval');
		$service->retryInterval    = $this->request->getInt('retryInterval');
		$service->freshness        = $this->request->getInt('freshness');
		$service->nagiosGrapher    = $this->request->keyExists('nagiosGrapher');
		$service->isNrpe           = $this->request->keyExists('isNrpe');

		if ($service->isNrpe) {
			try {
				$commandId = (int)util\Setting::get('nrpeCommandId');
			}
			catch (\OutOfBoundsException $e) {
				throw new UserException(
					tr("No default nrpe command is defined")
				);

			}

			$service->commandId = $commandId;
		}
		else {
			$service->commandId = $this->request->getInt('commandId');
		}

		$service->save();

		$this->parameters['service'] = $service;
		$this->parameters['services'][] = $service;

		Application::setHostChanged();
		Application::setAllSatellitesChanged();

		// Now assign the arguments
		$argumentIds          = $this->request->getRaw('argumentsIds') ?: array();
		$argumentOptions      = $this->request->getRaw('argumentsOptions') ?: array();
		$argumentValues       = $this->request->getRaw('argumentsValues') ?: array();
		$argumentDescriptions = $this->request->getRaw('argumentsDescriptions') ?: array();
		$argumentTypes        = $this->request->getRaw('argumentsTypes') ?: array();

		$position = 1;
		$serviceId = $service->id;
		foreach ($argumentValues as $key => $value) {
			$id          = isset($argumentIds[$key]) ? ((int)$argumentIds[$key] ?: null) : null;
			$description = isset($argumentDescriptions[$key]) ? $argumentDescriptions[$key] : null;
			$option      = isset($argumentOptions[$key]) ? $argumentOptions[$key] : null;
			$type        = isset($argumentTypes[$key]) ? $argumentTypes[$key] : null;

			// We must respect that the number 0 is a non-empty value, too
			if (!is_numeric($description) && strval($description) == '') {
				$description = null;
			}

			if (!is_numeric($option) && strval($option) == '') {
				$option = null;
			}

			if (!is_numeric($value) && strval($value) == '') {
				$value = null;
			}

			// When type is variable, value und option may be empty given that there is a description
			$validArguments =
				(
					$type == model\ServiceArgument::TYPE_VARIABLE
					&& (
						is_numeric($description)
						||
						strval($description) !== ''
					)
				)
				||
				$option !== null
				||
				$value !== null;

			if (!$validArguments) {
				// If there is one empty argument it means that no arguments were assigned to this service
				if (count($argumentValues) == 1 && $id == null) {
					continue;
				}

				throw new UserException(
					tr("Argument description (for variable arguments) or option or value (for fixed arguments) are required")
				);
			}


			$serviceArgument = new model\ServiceArgument();
			$serviceArgument->allowWrite(model\Permission::SERVICES_WRITE);

			$serviceArgument->id          = $id;
			$serviceArgument->serviceId   = $serviceId;
			$serviceArgument->position    = $position++;
			$serviceArgument->option      = $option;
			$serviceArgument->value       = $value;
			$serviceArgument->description = $description;
			$serviceArgument->type        = $type;
			$serviceArgument->save();
		}

		// We assume that until this point everything was ok
		// Create NRPE config for each host, if the service
		// is NRPE
		if ($service->isNrpe) {
			$hostServices = datamapper\HostServiceMapper::getInstance()->getAll(
				null,
				$serviceId
			);

			foreach ($hostServices as $key => $hostService) {
				// This way the user doesn't have to manually export the nrpe configuration
				$host = $hostService->getHost();
				$host->allowWrite(model\Permission::HOSTS_WRITE);
				$this->writeIcingaConfig('NRPE', array('IDs' => array($host->id)));
			}
		}

		Application::getDbInstance()->commit();
	}

	/**
	 * Write the given icinga config kind
	 *
	 * @param       string $kind
	 * @param       array  $IDs
	 * @return      object
	 */
	private function writeIcingaConfig($kind, $data = null) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm', 'RPC', 'wqAppName'),
			ConfManager::getVar('sm', 'RPC', 'wqQueues')
		);
		$timeout   = ConfManager::getVar('sm', 'RPC', 'icinga_timeout');
		$type      = 'writeIcingaConfig';
		$params    = array(
			'kind' => $kind
		);
		if (is_array($data)) {
			$params += $data;
		}
		$resultID = $rpcClient->putRequest($type, $params);
		$result   = $rpcClient->getResult($resultID, $timeout);
		if (isset($result->exception)) {
			Application::getLogInstance()->warn(sprintf(
				'There was an exception writing Icinga configuration: %s',
				$result->exception->traceback
			));
			throw new UserException($result->exception->message, $result->exception->code);
		}

		if ($result->hasInvalidConfiguration) {
			$this->addErrorMessage(
				tr("There was an error exporting the NRPE configuration:")
			);
			foreach ($result->verifyStdout as $message) {
				if (strpos(strtolower($message), 'error') === 0) {
					$this->addErrorMessage($message);
				}
			}
		}
	}

	/**
	 * Delete a service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Service id is missing")
			);
		}

		$service = datamapper\ServiceMapper::getInstance()->getByID($id);
		$service->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$service->delete();

		Application::setHostChanged();
		Application::setAllSatellitesChanged();
	}

	/**
	 * Shows all dependencies of a service
	 *
	 * @returns void
	 */
	protected function showDependenciesAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(tr('Host id is missing'));
		}
		$service = datamapper\ServiceMapper::getInstance()->getByID($id);
		$service->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$this->parameters['service'] = $service;
	}

	/**
	 * Get the command line of a command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function getCommandLineAction() {
		$commandId = $this->request->getInt('commandId', request\REQUEST_GET);

		if (empty($commandId)) {
			throw new UserException(
				tr("Command id is missing")
			);
		}

		$command = datamapper\CommandMapper::getInstance()->getByID(
			$commandId
		);
		$command->enforceReadPermission(
			model\Permission::SERVICES_READ
		);
		$this->parameters['command'] = $command;
	}

	protected function addNewArgumentAction() {
		$argument = new model\ServiceArgument();
		$argument->allowRead();
		$this->tpl->assign('argument', $argument);
		
		$tempId = $this->request->getInt('tempId', request\REQUEST_GET);
		$tempId += 1;
		$this->parameters['tempId'] = $tempId;
		$this->parameters['rowIndex'] = $tempId;
		

		// For access to the ServiceArgument types
		$this->tpl->registerClass(
			"ServiceArgument",
			"\ch\monitoring\model\ServiceArgument"
		);
	}

	protected function deleteArgumentAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Argument id is missing")
			);
		}

		$serviceArgument = datamapper\ServiceArgumentMapper::getInstance()->getByID($id);
		$serviceArgument->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$serviceArgument->delete();
	}

	/**
	 * Get the list of hosts which use this service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function serviceAssignmentsAction() {
		$serviceId = $this->request->getInt('serviceId', request\REQUEST_GET);
		$this->parameters['hostServices'] = datamapper\HostServiceMapper::getInstance()->getAll(
			null,
			$serviceId
		);
	}
}
