<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle service group page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceGroupController extends ServiceBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		$this->session->user->enforcePermission(
			model\Permission::SERVICES_READ
		);

		$this->setHeaderTitle(tr("Services groups"));
		$this->parameters['allGroups'] = datamapper\ServiceGroupMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'services/groups.tpl');

		parent::handleGET();
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'services/groupsAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Show the form to add a service group
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addFormAction() {
		$this->parameters['group'] = new model\ServiceGroup();
		$this->parameters['group']->allowRead();

		$this->assignFormVariables();
	}

	/**
	 * Show the form to edit a service group
	 *
	 * @throws        UserException
	 * @returns       void
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Service group id is missing")
			);
		}

		$this->parameters['group'] = datamapper\ServiceGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::SERVICES_WRITE
		);

		$this->assignFormVariables();
	}

	/**
	 * Assigns variables for the service group form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::SERVICES_READ
			)
		);
		$this->select['services']  = util\IteratorUtil::getKeyValueArray(
			datamapper\ServiceMapper::getInstance()->getAll()
		);
		$this->select['slas']      = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll(
				model\Permission::SERVICES_READ
			)
		);
	}

	/**
	 * Create a new service group and invoke the save
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function addAction() {
		$serviceGroup = new model\ServiceGroup();
		$serviceGroup->allowWrite(model\Permission::SERVICES_WRITE);

		$this->saveServiceGroup($serviceGroup);
	}

	/**
	 * Save an existing service group
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		$serviceGroup = datamapper\ServiceGroupMapper::getInstance()->getByID($id);
		$serviceGroup->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);

		$this->saveServiceGroup($serviceGroup);
	}

	/**
	 * Save the data of a created or edited group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         model\ServiceGroup $serviceGroup
	 * @returns       void
	 */
	private function saveServiceGroup($serviceGroup) {
		$serviceGroup->customerId  = $this->request->getInt('customerId') ?: null;
		$serviceGroup->name        = $this->request->getString('name');
		$serviceGroup->description = $this->request->getString('description');
		$serviceGroup->save();

		$this->parameters['group'] = $serviceGroup;
		$this->parameters['allGroups'][] = $serviceGroup;
	}

	/**
	 * Delete a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Group id is missing")
			);
		}

		$serviceGroup = datamapper\ServiceGroupMapper::getInstance()->getByID($id);
		$serviceGroup->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$serviceGroup->delete();
	}

	/**
	 * Add a service to a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addServiceAction() {
		$serviceGroupMember = new model\ServiceGroupMember();
		$serviceGroupMember->allowWrite(model\Permission::SERVICES_WRITE);

		$serviceGroupMember->groupId   = $this->request->getInt('itemId');
		$serviceGroupMember->serviceId = $this->request->getInt('serviceId');

		if (datamapper\ServiceGroupMemberMapper::getInstance()->existsByGroupAndService(
			$serviceGroupMember->groupId,
			$serviceGroupMember->serviceId
		)) {
			throw new UserException(
				tr('This service is already assigned')
			);
		}

		$serviceGroupMember->save();

		$this->parameters['group'] = $serviceGroupMember->getServiceGroup();
	}

	/**
	 * Show the group services form 
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	protected function groupServicesAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		$this->select['services'] = util\IteratorUtil::getKeyValueArray(
			datamapper\ServiceMapper::getInstance()->getAll()
		);

		$this->parameters['group'] = datamapper\ServiceGroupMapper::getInstance()->getByID($id);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::SERVICES_WRITE
		);
	}

	/**
	 * Remove a group member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteServiceAction() {
		$memberId = $this->request->getInt('memberId');

		if (empty($memberId)) {
			throw new UserException(
				tr("Group member id is missing")
			);
		}

		$serviceGroupMember = datamapper\ServiceGroupMemberMapper::getInstance()->getByID(
			$memberId
		);
		$serviceGroupMember->enforceWritePermission(
			model\Permission::SERVICES_WRITE
		);
		$serviceGroupMember->delete();

		$this->parameters['group'] = $serviceGroupMember->getServiceGroup();
	}

	/**
	 * Save group members (services)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveServicesAction() {
		$groupId = $this->request->getInt('groupId');

		if (empty($groupId)) {
			throw new UserException(
				tr("Group id is missing")
			);
		}

		$this->parameters['group'] = datamapper\ServiceGroupMapper::getInstance()->getByID($groupId);
		$this->parameters['group']->enforceReadPermission(
			model\Permission::SERVICES_WRITE
		);

		$groupServiceIdArray = $this->request->getRaw('groupServiceId');

		foreach ($groupServiceIdArray as $groupServiceId) {
			$serviceGroupMember = datamapper\ServiceGroupMemberMapper::getInstance()->getByID($groupServiceId);
			$serviceGroupMember->enforceWritePermission(
				model\Permission::SERVICES_WRITE
			);

			$serviceGroupMember->id        = $groupServiceId;
			$serviceGroupMember->groupId   = $groupId;

			$serviceGroupMember->save();
		}
	}
}
