<?php

namespace ch\monitoring\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle sla page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SlaController extends ProtectedController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::SLAS_READ
		);

		$this->setHeaderTitle(tr("Alert Schedulers"));
		$this->parameters['allSLAs'] = datamapper\SLAMapper::getInstance()->getAll();
		$this->tpl->assign('template', 'customers/slas.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');
		$this->callActionHandler();
		$this->tpl->assign('template', 'customers/slasAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Load data needed to display the form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::SLAS_READ
			)
		);
	}

	/**
	 * Edit a sla
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		$sla = datamapper\SLAMapper::getInstance()->getByID(
			$id
		);

		$sla->enforceReadPermission(
			model\Permission::SLAS_WRITE
		);

		$this->parameters['sla'] = $sla;
		$this->assignFormVariables();
	}

	/**
	 * Add a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addFormAction() {
		$sla = new model\SLA();
		$sla->allowRead();
		$this->parameters['sla'] = $sla;
		$this->assignFormVariables();
	}

	/**
	 * Add a sla
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addAction() {
		$sla = new model\SLA();
		$sla->allowWrite(model\Permission::SLAS_WRITE);
		$this->saveSla($sla);
	}

	/**
	 * Manage assigned rules
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function expandAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		$sla = datamapper\SLAMapper::getInstance()->getByID($id);
		$sla->enforceReadPermission(
			model\Permission::SLAS_READ
		);
		$this->parameters['sla'] = $sla;

		$this->select['actions'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
	}

	/**
	 * Show form to assign objects to new SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function assignFormAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		$sla = datamapper\SLAMapper::getInstance()->getByID($id);
		$sla->enforceReadPermission(
			model\Permission::SLAS_READ
		);
		$this->parameters['sla'] = $sla;

		$this->select['slas'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAMapper::getInstance()->getAll()
		);
	}

	/**
	 * Assign all objects which use this SLA to another SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAssignmentAction() {
		$id       = $this->request->getInt('id');
		$newSlaId = $this->request->getInt('newSlaId');

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		if (empty($newSlaId)) {
			throw new UserException(
				tr("New sla id is missing")
			);
		}

		if ($id === $newSlaId) {
			throw new UserException(
				tr("Old and new SLA are the same")
			);
		}

		datamapper\SLAMapper::getInstance()->batchAssign($id, $newSlaId);
	}
	/**
	 * Save a sla
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		$sla = datamapper\SLAMapper::getInstance()->getByID($id);
		$sla->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);
		$this->saveSla($sla);
	}

	/**
	 * Save a sla
	 *
	 * @param         model\SLA $sla
	 * @returns       void
	 */
	private function saveSla($sla) {
		$previousCustomerId   = $sla->customerId;
		$sla->customerId      = $this->request->getInt('customerId') ?: null;
		$sla->name            = $this->request->getString('name');
		$sla->description     = $this->request->getString('description');
		$sla->delayCycle      = $this->request->getInt('delayCycle');
		$sla->delayRestart    = $this->request->getInt('delayRestart');
		$sla->timeout         = $this->request->getInt('timeout');
		$sla->minAvailability = $this->request->getFloat('minAvailability');
		$sla->save();

		// If a timeframe has set a customer explicitely and that customer
		// matches the customer of the SLA itself, the user most probably wants
		// to update those timeframes as well.
		// The other timeframes which have a customer explicitely set - but a
		// different one than the one from the SLA - are most likely set on
		// purpose and do not get changed therefore.
		if ($previousCustomerId !== null) {
			foreach ($sla->getTimeframes() as $timeframe) {
				if ($timeframe->customerId === $previousCustomerId) {
					$timeframe->enforceWritePermission(
						model\Permission::SLAS_READ
					);
					$timeframe->customerId = $sla->customerId;
					$timeframe->save();
				}
			}
		}

		$this->parameters['sla'] = $sla;
		$this->parameters['allSLAs'][] = $sla;
	}

	/**
	 * Delete a sla
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("Sla id is missing")
			);
		}

		$sla = datamapper\SLAMapper::getInstance()->getByID($id);
		$sla->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);
		$sla->delete();
	}



	/**
	 * Add a sla timeframe
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addTimeframeAction() {
		$timeframe = new model\Timeframe();
		
		$timeframe->slaId        = $this->request->getInt('slaId');
		$timeframe->customerId   = $this->request->getInt('customerId') ?: NULL;
		$timeframe->monday       = $this->request->keyExists('monday');
		$timeframe->tuesday      = $this->request->keyExists('tuesday');
		$timeframe->wednesday    = $this->request->keyExists('wednesday');
		$timeframe->thursday     = $this->request->keyExists('thursday');
		$timeframe->friday       = $this->request->keyExists('friday');
		$timeframe->saturday     = $this->request->keyExists('saturday');
		$timeframe->sunday       = $this->request->keyExists('sunday');
		$timeframe->timeFrom     = $this->request->getString('timeFrom');
		$timeframe->timeTo       = $this->request->getString('timeTo');
		$timeframe->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);
		$timeframe->save();

		$this->parameters['sla'] = $timeframe->getSLA();

		$this->select['actions'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
	}

	/**
	 * Delete a sla timeframe
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteTimeframeAction() {
		$timeframeId = $this->request->getInt('timeframeId');

		if (empty($timeframeId)) {
			throw new UserException(
				tr("Timeframe id is missing")
			);
		}

		$timeframe = datamapper\TimeframeMapper::getInstance()->getByID(
			$timeframeId
		);
		$timeframe->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);
		$timeframe->delete();

		$this->parameters['sla'] = $timeframe->getSLA();

		$this->select['actions'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
	}

	/**
	 * Add a timeframe rule
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addTimeframeRuleAction() {
		$rule = new model\TimeframeRule();
		
		$rule->timeframeId  = $this->request->getInt('timeframeId');
		$rule->actionId     = $this->request->getInt('actionId');
		$rule->delay        = $this->request->getInt('delay');
		$rule->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);
		$rule->save();

		$this->parameters['sla'] = $rule->getTimeframe()->getSLA();

		$this->select['actions'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
	}

	/**
	 * Delete a timeframe rule
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteTimeframeRuleAction() {
		$ruleId = $this->request->getInt('ruleId');

		if (empty($ruleId)) {
			throw new UserException(
				tr("Rule id is missing")
			);
		}

		$rule = datamapper\TimeframeRuleMapper::getInstance()->getByID(
			$ruleId
		);
		$rule->enforceWritePermission(
			model\Permission::SLAS_WRITE
		);

		$rule->delete();

		$this->parameters['sla'] = $rule->getTimeframe()->getSLA();

		$this->select['actions'] = util\IteratorUtil::getKeyValueArray(
			datamapper\SLAActionMapper::getInstance()->getAll()
		);

		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll()
		);
	}
}
