<?php

namespace ch\monitoring\controller;

use ch\monitoring\Application;
use org\adfinis\framework\request;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\exceptions\ForbiddenException;
use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;

/**
 * Handle user page requests
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class UserController extends UserBaseController {

	/**
	 * Handles GET requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleGET() {
		header('Content-Type: text/html; charset=utf-8');

		$this->session->user->enforcePermission(
			model\Permission::USERS_READ
		);
		$this->setHeaderTitle(tr("Users"));
		$this->parameters['allUsers'] = datamapper\UserMapper::getInstance()->getAllByUser(
			$this->session->user,
			null
		);
		
		$this->tpl->assign('template', 'users/users.tpl');
		$this->tpl->assign('templateSubmenu', 'submenu.tpl');

		$this->outputPage('index.tpl');
	}

	/**
	 * Handles POST Requests
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handlePOST() {
		$user = $this->session->user;
		
		$user->enforcePermission(
			model\Permission::USERS_READ
		);
	}
	

	/**
	 * Handles AJAX requests
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		header('Content-Type: application/json; charset=utf-8');

		$this->callActionHandler();

		$this->tpl->assign('template', 'users/usersAjax.tpl');
		$this->outputAjaxPage('ajax.tpl');
	}

	/**
	 * Load data needed to display the form
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function assignFormVariables() {
		$this->tpl->registerClass("User", "ch\monitoring\model\User");
		$this->select['roles'] = util\IteratorUtil::getKeyValueArray(
			datamapper\RoleMapper::getInstance()->getAll()
		);
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::USERS_WRITE
			)
		);
		$this->select['authMethods'] = array(
			model\User::AUTH_METHOD_LOCAL     => tr('Local'),
			model\User::AUTH_METHOD_LDAP      => tr('LDAP synced automatically'),
			model\User::AUTH_METHOD_LDAP_SYNC => tr('LDAP synced manually'),
		);
	}

	/**
	 * Edit a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function editAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);

		if (empty($id)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		$user = datamapper\UserMapper::getInstance()->getByID($id);
		$this->enforceUserWritePermission($user);
		$this->parameters['user'] = $user;
		$this->assignFormVariables();
	}

	/**
	 * Add a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function addFormAction() {
		$user = new model\User();
		$user->allowRead();
		
		$user->isEnabled = true;
		$user->emergency = true;

		$this->parameters['user']		= $user;
		$this->assignFormVariables();
	}

	/**
	 * Save a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function saveAction() {
		$id = $this->request->getInt('id');
		if (empty($id)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		$user = datamapper\UserMapper::getInstance()->getByID($id);
		$this->enforceUserWritePermission($user);
		if(!$this->request->keyExists('isEnabled')) {
			// Check if the user tries to disable himself
			if($user->id == Application::getUser()->id) {
				throw new UserException(tr('You cannot disable your own user'));
				return;
			}

			// Check if the user is still used in a ticket
			$ticketCount = datamapper\MaintenanceTicketMapper::getInstance()->countTickets($user->id, true);
			if($ticketCount > 0) {
				$this->addNotifyMessage(tr('The User has unclosed tickets!'));
			}

			// Check if the user is assigned to a CustomerTeam (pikett)
			$teamCount = datamapper\CustomerTeamMapper::getInstance()->countTeams($user->id, new \Zend_Date());
			if($teamCount > 0) {
				$this->addNotifyMessage(
					tr('The User is assigned to an on-call team or is set as default for the on-call teams!')
				);
			}
		}
		$this->saveUser($user);
	}

	/**
	 * Add a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function addAction() {
		$user = new model\User();
		$user->allowWrite(model\Permission::USERS_WRITE);
		$user->authenticationMethod = model\User::AUTH_METHOD_LOCAL;
		$this->saveUser($user);
	}

	/**
	 * Save a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\User $user
	 * @returns       void
	 */
	private function saveUser($user) {
		$user->name                 = $this->request->getString('name');
		$user->fullName             = $this->request->getString('fullName');
		$user->emergency            = $this->request->keyExists('emergency');
		$user->authenticationMethod = $this->request->getString('authMethod');
		$password                   = $this->request->getString('password')  ?: null;
		$password2                  = $this->request->getString('password2') ?: null;

		if ($password !== null && $password === $password2) {
			$user->setPassword($password);
		}
		else if ($password !== null) {
			throw new UserException(
				tr('Please enter the same password twice')
			);
		}
		
		$this->assignUserContact($user);
		$user->customerId = $this->request->getInt('customerId');
		$user->isEnabled  = $this->request->keyExists('isEnabled');
		$this->session->user->enforcePermissionForCustomer(
			model\Permission::USERS_WRITE,
			$user->customerId
		);
		$identifier = $user->getMapper()->save($user);

		// We need to add the assignment to the customer
		$user->getMapper()->addCustomer($identifier['id'], $user->customerId);

		// And tidy up User_Customer
		datamapper\RightMapper::getInstance()->fixUserCustomers();

		// Permission cache must be updated
		$date = new \Zend_Date();
		util\Setting::set('updateCache', $date->get(\Zend_Date::ISO_8601));

		$this->parameters['user'] = $user;
		$this->parameters['allUsers'][] = $user;
	}

	/**
	 * Delete a user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteAction() {
		$id = $this->request->getInt('id');

		if (empty($id)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		$user = datamapper\UserMapper::getInstance()->getByID($id);
		$this->enforceUserWritePermission($user);
		$user->getMapper()->delete($user->getIdentifier());
	}

	/**
	 * Edit customerpermissions
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * 
	 * @returns       void
	 */
	protected function editRightAction() {
		$id = $this->request->getInt('id', request\REQUEST_GET);
		$this->assignDefaultValuesForRightForm($id);
	}

	/**
	 * Assigns the default variables to the template for the rightform
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * 
	 */
	private function assignDefaultValuesForRightForm($id = null) {
		if (is_null($id) || empty($id)) {
			throw new UserException(
				tr("User id is missing")
			);
		}

		$user = datamapper\UserMapper::getInstance()->getByID($id);
		$this->enforceUserWritePermission($user);
		$this->parameters['user'] = $user;

		//get all assigned roles/users
		$this->parameters['rights'] = util\IteratorUtil::getKeyObjectArray(
			datamapper\RightMapper::getInstance()->getAll(
				null, $id, $user->authenticationMethod
			)
		);
		
		//get all roles which we're allowed to assign to this user
		$this->select['customers'] = util\IteratorUtil::getKeyValueArray(
			datamapper\CustomerMapper::getInstance()->getAll(model\Permission::USERS_WRITE)
		);
		$this->assignRolesByCustomerId(key($this->select['customers']));
	}

	/**
	 * Returns the available roles for given customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @todo          push this to RightController (reusedable code)
	 * 
	 */
	protected function getRolesForCustomerAction() {
		$id = $this->request->getInt('customerId', request\REQUEST_GET);
		$this->assignRolesByCustomerId($id);
	}

	/**
	 * Delete a right
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @author        Mathtieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function deleteRightAction() {
		$id = $this->request->getInt('rightId', request\REQUEST_POST);

		if (empty($id)) {
			throw new UserException(
				tr("Right id is missing")
			);
		}

		$right = datamapper\RightMapper::getInstance()->getById($id);
		$right->enforceWritePermission(
			model\Permission::USERS_WRITE
		);

		$right->delete();

		// Cache needs to be updated
		Application::updateCache();
	}

	/**
	 * Add a right
	 *
	 * @todo          push this to RightController (reusedable code)
	 * @author        Mathtieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       void
	 */
	protected function addRightAction() {
		$userId = $this->request->getInt('userId', request\REQUEST_POST);
		$roleId = $this->request->getInt('roleId', request\REQUEST_POST);
		$customerId = $this->request->getInt('customerId', request\REQUEST_POST);
		$user = datamapper\UserMapper::getInstance()->getById($userId);
		$role = datamapper\RoleMapper::getInstance()->getById($roleId);
		$user->allowRead();
		$role->allowRead();

		if(Application::getUser()->getACL()->getRootCustomerIdentifier() != $customerId) {
			if($role->isGlobal()) {
				throw new UserException(tr("You cannot assign a global role to a customer which is not the root customer"));
			}
		}

		$customer = datamapper\CustomerMapper::getInstance()->getById($customerId);
	
		$right = new model\Right();
		$right->customerId = $customer->id;
		$right->roleId = $role->id;
		$right->userId = $user->id;

		$right->enforceWritePermission(
			model\Permission::USERS_WRITE
		);

		try {
			$right->save();
		}
		catch(\Zend_Db_Statement_Exception $e) {
			if($e->getCode() == 23000) {
				throw new UserException(tr('The user has already the selected role assigned for the selected customer'));
			}
			else {
				throw $e;
			}
		}

		$this->assignDefaultValuesForRightForm($userId);

		// Cache needs to be updated
		Application::updateCache();
	}

	/**
	 * Fills $this->select['roles'] with an array holding
	 * all roles allowed for the given customer
	 *
	 * @returns      void
	 */
	protected function assignRolesByCustomerId($customerId) {
		$this->select['roles'] = util\IteratorUtil::getKeyValueArray(
			datamapper\RoleMapper::getInstance()->getAllByCustomer(
				$customerId,
				Application::getUser()->getACL()->getRootCustomerIdentifier() != $customerId
			)
		);
	}

	/**
	 * Enforces write permission for given user
	 *
	 * @param         model\User $user
	 * @returns       void
	 */
	private function enforceUserWritePermission($user) {
		try {
			$user->enforcePermission(
				model\Permission::USERS_WRITE,
				true
			);
		}
		catch (ForbiddenException $e) {
			$user->allowRead();

			$errorMessage = sprintf(
				"%s\n%s\n%s",
				tr(
					"This user is assigned to more than one customer, " .
					"but you do not have the permission to edit users " .
					"for all of those customers."
				),
				$e->getMessage(),
				$e->getTraceAsString()
			);
			Application::getLogInstance()->err($errorMessage);
			error_log($errorMessage);
			$this->addErrorMessage($errorMessage);
		}
	}
}
