<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the AlertComment model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertCommentMapper extends Mapper {

	/**
	 * AlertCommentMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertCommentMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\AlertComment');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertCommentMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a AlertComment by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\AlertComment
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`AlertComment`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$alertComment = $this->getByQuery($query, array((int)$id));
			$map->set($alertComment);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single AlertComment object identified by the given SQL query
	 *
	 * Runs the given query, initialises a AlertComment object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the AlertComment cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\AlertComment
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find alert comment with these parameters'),
					var_export($params, true)
				)
			);
		}

		$alertComment = new model\AlertComment();
		$alertComment->fillFromRowData($data);

		return $alertComment;
	}

	/**
	 * Load all AlertComments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $alertId Optional.
	 * @param         array $tags Optional.
	 * @param         string $permissionName Defaults to Permission::ALERTS_READ
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$alertId = null,
		$tags = null,
		$permissionName = model\Permission::ALERTS_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if (
			!Application::getUser()->hasPermission(model\Permission::ALERTS_WRITE)
		) {
			$tags = array(
				model\AlertTag::CUSTOMER_REPORT
			);
		}

		if ($alertId !== null) {
			$conditions->addStatement('Alert', new sql\WhereStatement(
				'alertId',
				'=',
				$alertId,
				'AlertComment'
			));
		}

		if ($tags !== null) {
			$conditions->addStatement('Tags', new sql\WhereStatement(
				'tagId',
				'IN',
				$tags,
				'AlertComment_AlertTag'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Alert',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`AlertComment`.`id`
				FROM
					`AlertComment`
				JOIN `Alert` ON (
					`Alert`.`id` = `AlertComment`.`alertId`
				)
				LEFT JOIN `AlertComment_AlertTag` ON (
					`AlertComment_AlertTag`.`commentId` = `AlertComment`.`id`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `AlertComment`";

		$alertCommentList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertComment',
			array()
		);

		$alertCommentList->addOrderVariant('timestamp.DESC', '`AlertComment`.`timestamp` DESC');
		$alertCommentList->addOrderVariant('timestamp.ASC', '`AlertComment`.`timestamp` ASC');

		return $alertCommentList;
	}

	/**
	 * Get comments by tag name and alert id
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $alertID The id of the alert
	 * @param         tag $tag Tagname
	 * @returns       oaf\db\DbIterator
	 */
	public function getByTag($alertID, $tag) {
		$searchQuery = '
			SELECT
				`ac`.`id`
			FROM
				`AlertComment` AS `ac`
			LEFT JOIN
				`AlertComment_AlertTag` AS `act` ON (
					`act`.`commentId` = `ac`.`id`
				)
			LEFT JOIN
				`AlertTag` AS `at` ON (
					`act`.`tagId` = `at`.`id`
				)
			WHERE
				`at`.`name`  = :tag
				AND
				`ac`.`alertId` = :alertID
		';

		$selectQuery = 'SELECT * FROM `AlertComment`';

		$alertComments = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Alert',
			array(
				'tag'     => $tag,
				'alertID' => $alertID
			)
		);

		return $alertComments;
	}

	/**
	 * Saves AlertComment information
	 *
	 * If the AlertComment object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $alertComment
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $alertComment) {
		$fieldDataMapping = array(
			'id'               => $alertComment->id,
			'alertId'          => $alertComment->alertId,
			'userId'           => $alertComment->userId,
			'timestamp'        => $alertComment->timestamp,
			'comment'          => $alertComment->comment
		);

		if ($alertComment->hasIdentifier()) {
			Application::getDBInstance()->update(
				'AlertComment',
				$fieldDataMapping,
				sprintf('id = %d', $alertComment->id)
			);
		}
		else {
			Application::getDBInstance()->insert(
				'AlertComment',
				$fieldDataMapping
			);
			$alertComment->id = Application::getDBInstance()->lastInsertId();
		}

		return $alertComment->getIdentifier();
	}

	/**
	 * Delete a AlertComment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDBInstance()->beginTransaction();

		AlertTagMapper::getInstance()->deleteAssociation($identifier['id']);

		Application::getDBInstance()->delete(
			'AlertComment',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Delete the comments of an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $alertID The alert ID
	 */
	public function deleteByAlertID($alertId) {
		Application::getDBInstance()->beginTransaction();

		$query = sprintf('
			SELECT
				`id`
			FROM
				`AlertComment`
			WHERE
				`AlertId` = %d
			',
			$alertId
		);

		$stmt = Application::getDBInstance()->query($query);

		foreach ($stmt->fetchAll() as $row) {
			// delete the associations of tags with the comments
			AlertTagMapper::getInstance()->deleteAssociation($row['id']);
		}

		Application::getDBInstance()->delete(
			'AlertComment',
			sprintf('`alertID` = %d', $alertId)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Add a comment to an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $comment The comment
	 * @param         int $userId The user that made the comment
	 * @param         int $alertId The alert ID
	 * @param         array $tags A list of tag IDs
	 * @return        model\Alertcomment
	 */
	public function create($comment, $userId, $alertId, $tags) {
		Application::getDBInstance()->beginTransaction();

		$alertComment = new model\AlertComment();
		$alertComment->allowWrite(model\Permission::ALERTS_WRITE);
		$alertComment->alertId   = $alertId;
		$alertComment->userId    = $userId;
		$alertComment->timestamp = null;
		$alertComment->comment   = $comment;
		$alertComment->save();

		foreach ($tags as $tag) {
			$alertComment->addAlertTag($tag);
		}

		Application::getDBInstance()->commit();
		return $alertComment;
	}

	/**
	 * Count the comments of an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $alertId The alert id
	 * @returns       int
	 */
	public function countByAlertId($alertId) {
		$query = sprintf('
			SELECT
				count(*) AS count
			FROM
				`AlertComment`
			WHERE
				`alertId` = %d
			',
			$alertId
		);

		return intval(Application::getDBInstance()->fetchOne($query));
	}
}
