<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the AlertLog model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertLogMapper extends Mapper {

	/**
	 * AlertLogMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertLogMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\AlertLog');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertLogMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a AlertLog by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\AlertLog
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`AlertLog`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$alertLog = $this->getByQuery($query, array((int)$id));
			$map->set($alertLog);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single AlertLog object identified by the given SQL query
	 *
	 * Runs the given query, initialises a AlertLog object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the AlertLog cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\AlertLog
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find alert log with these parameters'),
					var_export($params, true)
				)
			);
		}

		$alertLog = new model\AlertLog();
		$alertLog->fillFromRowData($data);

		return $alertLog;
	}

	/**
	 * Load all AlertLogs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $alertId Optional.
	 * @param         string $permissionName Defaults to Permission::ALERTS_READ
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$alertId        = null,
		$permissionName = model\Permission::ALERTS_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($alertId !== null) {
			$conditions->addStatement('Alert', new sql\WhereStatement(
				'alertId',
				'=',
				$alertId,
				'AlertLog'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Alert',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`AlertLog`.`id`
				FROM
					`AlertLog`
				JOIN `Alert` ON (
					`Alert`.`id` = `AlertLog`.`alertId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `AlertLog`";

		$alertLogList = SortingPolicyIterator::create(
            $searchQuery,
            $selectQuery,
            'ch\monitoring\model\AlertLog',
            array()
        );

        return $alertLogList;
	}

	/**
	 * Saves AlertLog information
	 *
	 * If the AlertLog object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $alertLog
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $alertLog) {
		$fieldDataMapping = array(
			'id'            => $alertLog->id,
			'typeId'        => $alertLog->typeId,
			'userId'        => $alertLog->userId,
			'alertId'       => $alertLog->alertId,
			'timestamp'     => $alertLog->timestamp,
			'hasFailed'     => $alertLog->hasFailed
		);

		if ($alertLog->hasIdentifier()) {
			Application::getDBInstance()->update(
				'AlertLog',
				$fieldDataMapping,
				sprintf('id = %d', $alertLog->id)
			);
		}
		else {
			Application::getDBInstance()->insert(
				'AlertLog',
				$fieldDataMapping
			);
			$alertLog->id = Application::getDBInstance()->lastInsertId();
		}

		return $alertLog->getIdentifier();
	}

	/**
	 * Delete a AlertLog
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'AlertLog',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Delete the alert logs associated to an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $alertID The alert ID
	 */
	public function deleteByAlertID($alertID) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'AlertLog',
			sprintf('`alertId` = %d', $alertID)
		);

		Application::getDBInstance()->commit();
	}
}
