<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\db\sql;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Alert model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertMapper extends Mapper {

	/**
	 * AlertMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertMapper $singleton
	 */
	private static $singleton;

	/**
	 * Show reaction time
	 *
	 * @const         int TYPE_REACTION
	 */
	const TYPE_REACTION = 1;

	/**
	 * Show down time
	 *
	 * @const         int TYPE_DOWN
	 */
	const TYPE_DOWN = 2;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Alert');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Alert by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Alert
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Alert`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$alert = $this->getByQuery($query, array($id));
			$map->set($alert);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single Alert object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Alert object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Alert cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Alert
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find alert with these parameters'),
					var_export($params, true)
				)
			);
		}

		$alert = new model\Alert();
		$alert->fillFromRowData($data);

		return $alert;
	}

	/**
	 * Load all Alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         bool      $new                Return only the not yet acknowledged alerts (optional)
	 * @param         bool      $open               Return only the not yet recovered alerts (optional)
	 * @param         int       $hostId             Limit to this host (optional)
	 * @param         Zend_Date $dateFrom           Minimum date (optional)
	 * @param         Zend_Date $dateTo             Maximum date (optional)
	 * @param         int       $serviceInstanceId  Limit to this service (optional)
	 * @param         int       $customerId         Limit to this customer (optional)
	 * @param         bool      $hostOnly           Whether to only select alerts caused by the host (optional)
	 * @param         string    $permissionName     Permission which is needed for getting alerts
	 * @param         bool      $hideDisabled       Do not return disabled alerts
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$new               = null,
		$open              = null,
		$hostId            = null,
		$dateFrom          = null,
		$dateTo            = null,
		$serviceInstanceId = null,
		$customerId        = null,
		$hostOnly          = false,
		$permissionName    = model\Permission::ALERTS_READ,
		$hideDisabled      = false,
		$commandId         = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
		$joins = array();

		if ($new !== null) {
			$conditions->addStatement(
				'acknowledged',
				new sql\WhereStatement(
					'acknowledged',
					'=',
					!$new,
					'Alert'
				)
			);
		}

		if ($open !== null) {
			$operator = $open ? "IS" : "IS NOT";
			$conditions->addStatement(
				'open',
				new sql\WhereStatement(
					'datetimeRecovery',
					$operator,
					null,
					'Alert',
					false
				)
			);
		}

		if ($hostId !== null) {
			$conditions->addStatement(
				'Host',
				new sql\WhereStatement(
					'hostId',
					'=',
					$hostId,
					'Alert',
					false
				)
			);
		}

		if ($dateFrom !== null) {
			$conditions->addStatement('Date from', new sql\WhereStatement(
				'datetimeOccured',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'Alert'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('Date to', new sql\WhereStatement(
				'datetimeOccured',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'Alert'
			));
		}

		if ($serviceInstanceId !== null) {
			$conditions->addStatement('service ID', new sql\WhereStatement(
				'serviceInstanceId',
				'=',
				$serviceInstanceId,
				'Alert'
			));
		}

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'=',
				$customerId,
				'Alert'
			));
		}

		if ($hostOnly) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'serviceInstanceId',
				'IS',
				null,
				'Alert',
				false
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Alert',
				false
			));
		}

		if ($hideDisabled) {
			$joins[] = '
				JOIN `Host` ON (
					`Host`.`id` = `Alert`.`hostId`
					AND
					`Host`.`isEnabled` = TRUE
				)
				JOIN `Customer` ON (
					`Customer`.`id` = `Host`.`customerId`
					AND
					`Customer`.`isEnabled` = TRUE
				)
			';
		}

		if ($commandId !== null) {
			$conditions->addStatement('Command', new sql\WhereStatement(
				'commandId',
				'=',
				$commandId,
				'Alert'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`Alert`.`id`
				FROM
					`Alert`
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);

		$selectQuery = "SELECT * FROM `Alert`";

		$alertList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Alert',
			array()
		);

		$alertList->addOrderVariant('id.DESC', '`Alert`.`id` DESC');
		$alertList->addOrderVariant('id.ASC',  '`Alert`.`id` ASC');

		return $alertList;
	}

	/**
	 * Acknowledge an alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $id Alert ID
	 * @param         int $userID The user ID that acknowledged
	 * @returns       ch\symonitoring\model\Alert
	 */
	public function acknowledge($id, $userID) {
		Application::getDBInstance()->beginTransaction();

		$now = new \Zend_Date();
		Application::getDBInstance()->update(
			'Alert',
			array(
				'acknowledged'         => 1,
				'acknowledgedBy'       => $userID,
				'datetimeAcknowledged' => $now->get(\Zend_Date::ISO_8601)
			),
			sprintf('`acknowledged` = 0 AND `id` = %d', $id)
		);

		$logEntry = new model\AlertLog();
		$logEntry->allowWrite(model\Permission::ALERTS_WRITE);

		$logEntry->typeId    = model\NotificationType::ACKNOWLEDGED;
		$logEntry->userId    = $userID;
		$logEntry->alertId   = $id;
		$logEntry->timestamp = null;

		$logEntry->save();

		Application::getDBInstance()->commit();

		IdentityMap::getInstance(get_called_class())->clear();
		return $this->getByID($id)->allowRead();
	}

	/**
	 * Saves Alert information
	 *
	 * If the Alert object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $alert
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $alert) {
		$fieldDataMapping = array(
			'id'                        => $alert->id,
			'customerId'                => $alert->customerId,
			'hostId'                    => $alert->hostId,
			'commandId'                 => $alert->commandId,
			'serviceInstanceId'         => $alert->serviceInstanceId,
			'slaId'                     => $alert->slaId,
			'slaCustomerId'             => $alert->slaCustomerId,
			'datetimeOccured'           => util\TimeUtility::dateOrNull($alert->datetimeOccured),
			'datetimeAcknowledged'      => util\TimeUtility::dateOrNull($alert->datetimeAcknowledged),
			'datetimeRecovery'          => util\TimeUtility::dateOrNull($alert->datetimeRecovery),
			'state'                     => $alert->state,
			'output'                    => $alert->output,
			'acknowledgedBy'            => $alert->acknowledgedBy,
			'acknowledged'              => $alert->acknowledged,
			'acknowledgedAutomatically' => $alert->acknowledgedAutomatically,
			'comment'                   => $alert->comment
		);

		if ($alert->hasIdentifier()) {
			Application::getDBInstance()->update(
				'Alert',
				$fieldDataMapping,
				sprintf('id = %d', $alert->id)
			);
		}
		else {
			Application::getDBInstance()->insert(
				'Alert',
				$fieldDataMapping
			);
			$alert->id = Application::getDBInstance()->lastInsertId();
		}

		return $alert->getIdentifier();
	}

	/**
	 * Delete an Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		$id = $identifier['id'];
		Application::getDBInstance()->beginTransaction();
		AlertCommentMapper::getInstance()->deleteByAlertID($id);
		AlertLogMapper::getInstance()->deleteByAlertID($id);
		$nagiosLogs = NagiosLogMapper::getInstance()->getByAlertID($id);
		foreach ($nagiosLogs as $nagiosLog) {
			$nagiosLogID = $nagiosLog->id;
			HoldMapper::getInstance()->deleteByNagiosLogID($nagiosLogID);
			HoldInformMapper::getInstance()->deleteByNagiosLogID($nagiosLogID);
		}
		Application::getDBInstance()->delete(
			'Alert',
			sprintf('`id` = %d', $id)
		);
		Application::getDBInstance()->commit();
	}

	/**
	 * Delete by Host Id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @returns       void
	 */
	public function deleteByHostId($hostId) {
		Application::getDBInstance()->beginTransaction();

		$alerts = $this->getAll(null, null, $hostId);
		foreach ($alerts as $alert) {
			$this->delete($alert->getIdentifier());
		}

		Application::getDBInstance()->commit();
	}

	/**
	 * Count the alerts
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $customerId Optional
	 * @param         int $hostId Optional
	 * @param         int $instanceId Optional
	 * @param         int $openOnly Optional Only count the open alerts
	 * @returns       int
	 */
	public function count(
		$customerId = null,
		$hostId     = null,
		$instanceId = null,
		$openOnly   = false
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				array_merge(
					array($customerId),
					CustomerMapper::getInstance()->getSubCustomers(
						$customerId
					)
				),
				'Alert'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'Alert'
			));
		}

		if ($instanceId !== null) {
			$conditions->addStatement('HostServiceInstance', new sql\WhereStatement(
				'serviceInstanceId',
				'=',
				$instanceId,
				'Alert'
			));
		}

		if ($openOnly) {
			$conditions->addStatement('Acknowledged', new sql\WhereStatement(
				'acknowledged',
				'=',
				false
			));
		}

		$query = sprintf('
				SELECT
					count(*) AS `count`
				FROM
					`Alert`
				%s
			',
			$conditions
		);

		return Application::getDBInstance()->fetchOne($query);
	}

	/**
	 * Returns the total reaction time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $type The type of time span to be verified
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @param         int $customerId Optional
	 * @param         int $hostId Optional
	 * @param         int $instanceId Optional
	 * @throws        \RuntimeException
	 * @returns       \Zend_Date The total time span
	 */
	public function getReactionTime(
		$type,
		$dateFrom   = null,
		$dateTo     = null,
		$customerId = null,
		$hostId     = null,
		$instanceId  = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($dateFrom !== null) {
			$conditions->addStatement('Date from', new sql\WhereStatement(
				'datetimeOccured',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'Alert'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('Date to', new sql\WhereStatement(
				'datetimeOccured',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'Alert'
			));
		}

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				array_merge(
					array($customerId),
					CustomerMapper::getInstance()->getSubCustomers(
						$customerId
					)
				),
				'Alert'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'Alert'
			));
		}

		if ($instanceId !== null) {
			$conditions->addStatement('Service', new sql\WhereStatement(
				'instanceId',
				'=',
				$serviceInstanceId,
				'Alert'
			));
		}

		switch ($type) {
			case self::TYPE_REACTION:
				$field1 = 'datetimeAcknowledged';
				$field2 = 'datetimeOccured';
				break;
			case self::TYPE_DOWN:
				$field1 = 'datetimeRecovery';
				$field2 = 'datetimeOccured';
				break;
			default:
				throw new \RuntimeException(
					tr('Invalid type')
				);
		}

		$conditions->addStatement('Not null', new sql\WhereStatement(
			$field1,
			'IS NOT',
			null,
			'Alert',
			false
		));

		$query = sprintf('
			SELECT
				SUM(TIME_TO_SEC(TIMEDIFF(`%s`, `%s`)))
			FROM
				`Alert`
			%s
			',
			$field1,
			$field2,
			$conditions
		);

		return Application::getDBInstance()->fetchOne($query);
	}
}
