<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the AlertTag model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertTagMapper extends Mapper {

	/**
	 * AlertTagMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertTagMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\AlertTag');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertTagMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a AlertTag by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\AlertTag
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`AlertTag`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$alertTag = $this->getByQuery($query, array((int)$id));
			$map->set($alertTag);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single AlertTag object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a AlertTag object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the AlertTag cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\AlertTag
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find alert tag with these parameters'),
					var_export($params, true)
				)
			);
		}

		$alertTag = new model\AlertTag();
		$alertTag->fillFromRowData($data);

		return $alertTag;
	}

	/**
	 * Load all AlertTags
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`AlertTag`.`id`
			FROM
				`AlertTag`
		";

		$selectQuery = "SELECT * FROM `AlertTag`";

		$alertTagList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertTag',
			array()
		);

		return $alertTagList;
	}

	/**
	 * Get the alert tags related to the comment 
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $commentID The comment id
	 * @returns       oaf\DbIterator
	 */
	public function getByAlertCommentID($commentID) {
		$searchQuery = '
			SELECT
				`at`.`id`
			FROM
				`AlertTag` AS `at`

			LEFT JOIN `AlertComment_AlertTag` AS `ac` ON (
					`at`.`id` = `ac`.`tagId`
				)

			WHERE
				`ac`.`commentId` = :commentID
			';

		$selectQuery = "SELECT * from `AlertTag`";

		$alertTagList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertTag',
			array('commentID' => $commentID)
		);

		return $alertTagList;
	}

	/**
	 * Saves AlertTag information
	 *
	 * If the AlertTag object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $alertTag
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $alertTag) {
		$fieldDataMapping = array(
			'id'                   => $alertTag->id,
			'name'                 => $alertTag->name,
			'description'          => $alertTag->description,
		);

		if ($alertTag->hasIdentifier()) {
			Application::getDBInstance()->update(
				'AlertTag',
				$fieldDataMapping,
				sprintf('id = %d', $alertTag->id)
			);
		}
		else {
			Application::getDBInstance()->insert(
				'AlertTag',
				$fieldDataMapping
			);
			$alertTag->id = Application::getDBInstance()->lastInsertId();
		}

		return $alertTag->getIdentifier();
	}

	/**
	 * Delete a AlertTag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'AlertComment_AlertTag',
			sprintf('`tagId` = %d', $identifier['id'])
		);

		Application::getDBInstance()->delete(
			'AlertTag',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Delete the associations of tags to comments
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $commentID The ID of the comment
	 */
	public function deleteAssociation($commentId) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'AlertComment_AlertTag',
			sprintf('`commentID` = %d', $commentId)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Create an association to a comment
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $tagId The Tag ID to associate
	 * @param         int $commentId The comment ID
	 */
	public function createAssociation($tagId, $alertCommentId) {
		Application::getDBInstance()->insert(
			'AlertComment_AlertTag',
			array(
				'tagID'     => $tagId,
				'commentId' => $alertCommentId
			)
		);
	}
}
