<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use org\adfinis\framework\db\SortingIterator;

/**
 * Datamapper for the AlertTemplate model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertTemplateMapper extends Mapper {

	/**
	 * AlertTemplateMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertTemplateMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\AlertTemplate');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertTemplateMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a AlertTemplate by name
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\AlertTemplate
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`AlertTemplate`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Load a AlertTemplate by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\AlertTemplate
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`AlertTemplate`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$template = $this->getByQuery($query, array((int)$id));
			$map->set($template);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single AlertTemplate object identified by the given SQL query
	 *
	 * Runs the given query, initialises a AlertTemplate object and returns it
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the AlertTemplate cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\AlertTemplate
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find the template with these parameters'),
					var_export($params, true)
				)
			);
		}

		$template = new model\AlertTemplate();
		$template->fillFromRowData($data);

		return $template;
	}

	/**
	 * Load all AlertTemplates
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`AlertTemplate`.`id`
			FROM
				`AlertTemplate`
		";

		$selectQuery = "SELECT * FROM `AlertTemplate`";
		$templateList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertTemplate',
			array()
		);

		$templateList->addOrderVariant('id.ASC',  '`AlertTemplate`.`id` ASC');
		$templateList->addOrderVariant('id.DESC', '`AlertTemplate`.`id` DESC');

		return $templateList;
	}

	/**
	 * Get all AlertTemplates for a certain SLA action
	 *
	 * @param         array  $slaActionId
	 *
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getBySlaAction($slaActionId) {
		$searchQuery = sprintf("
			SELECT
				`AlertTemplate`.`id`
			FROM
				`AlertTemplate`
			WHERE
				`AlertTemplate`.`slaActionId` = %d
		", $slaActionId);

		$selectQuery = "SELECT * FROM `AlertTemplate`";

		$templateList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertTemplate',
			array()
		);

		$templateList->addOrderVariant('id.ASC',  '`AlertTemplate`.`id` ASC');
		$templateList->addOrderVariant('id.DESC', '`AlertTemplate`.`id` DESC');

		return $templateList;
	}

	/**
	 * Saves AlertTemplate information
	 *
	 * If the AlertTemplate object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $data
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $template) {
		$fieldDataMapping = array(
			'id'                   => $template->id,
			'name'                 => $template->name,
			'description'          => $template->description,
			'content'              => $template->content,
			'slaActionId'          => $template->slaActionId
		);

		if ($template->hasIdentifier()) {
			Application::getDbInstance()->update(
				'AlertTemplate',
				$fieldDataMapping,
				sprintf('id = %d', $template->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'AlertTemplate',
				$fieldDataMapping
			);
			$template->id = Application::getDbInstance()->lastInsertId();
		}

		return $template->getIdentifier();
	}

	/**
	 * Delete a AlertTemplate
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		try {
			Application::getDbInstance()->delete(
				'AlertTemplate',
				sprintf('`id` = %d', $identifier['id'])
			);
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					tr("This template cannot be deleted because of dependencies")
				);
			}
			else {
				throw $e;
			}
		}

		Application::getDbInstance()->commit();
	}
}
