<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the AlertTemplateUser model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertTemplateUserMapper extends Mapper {

	/**
	 * AlertTemplateUserMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\AlertTemplateUserMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\AlertTemplateUser');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\AlertTemplateUserMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Returns a single AlertTemplateUser object identified by the given SQL query
	 *
	 * Runs the given query, initialises a AlertTemplateUser object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the AlertTemplateUser cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\AlertTemplateUser
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find AlertTemplateUser with these parameters'),
					var_export($params, true)
				)
			);
		}

		$user = new model\AlertTemplateUser();
		$user->fillFromRowData($data);

		return $user;
	}

	/**
	 * Returns a DBIterator of AlertTemplateUsers which can be found by a SQLCondition
	 *
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *
	 * @param         sql\WhereGroup $conditions optional
	 * @param         array $joins optional
	 * @param         array $paramaters optional
	 * @param         string $permissionName A permission to restrict users to
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	protected function getByCondition(
		sql\WhereGroup $conditions = null,
		Array $joins = array(),
		Array $parameters = array()
	) {
		if (is_null($conditions)) {
			$conditions = new sql\WhereGroup();
			$conditions->setOperand('AND');
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`AlertTemplate_User`.`id`
				FROM
					`AlertTemplate_User`
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);

		$selectQuery = "SELECT * FROM `AlertTemplate_User`";
		$list = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AlertTemplateUser',
			$parameters
		);

		$list->addOrderVariant('id.ASC',  '`User`.`id` ASC');
		$list->addOrderVariant('id.DESC', '`User`.`id` DESC');

		return $list;
	}

	/**
	 * Load a AlertTemplateUser by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\AlertTemplateUser
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`AlertTemplate_User`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$alerttemplateuser = $this->getByQuery($query, array((int)$id));
			$map->set($alerttemplateuser);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Delete all AlertTemplateUser by userId
	 *
	 * @param         int $userId
	 * @returns       ch\monitoring\model\AlertTemplateUser
	 */
	public function clearByUserId($userId) {
		Application::getDbInstance()->delete(
			'AlertTemplate_User',
			sprintf('`userId` = %d', $userId)
		);
	}

	/**
	 * Returns the AlertTemplateUser object that matches the param OR returns an unsaved preinitialized AlertTemplateUser
	 *
	 * @param         int $userId
	 * @param         int $slaAction
	 * @returns       ch\symonitoring\model\AlertTemplateUser
	 */
	public function getByNotificationType($userId, $slaActionId) {
		try {
			$selectQuery = '
				SELECT
					*
				FROM
					`AlertTemplate_User`
				WHERE
					`AlertTemplate_User`.`userId` = ?
				AND
					`AlertTemplate_User`.`slaActionId` = ?
			';
			return $this->getByQuery($selectQuery, array(
				$userId,
				$slaActionId
			));
		} catch(\Exception $e) {
			//$alertTemplateUser = new model\AlertTemplateUser();
			//$alertTemplateUser->userId      = $userId;
			//$alertTemplateUser->slaActionId = $slaActionId;

			return null;
		}
	}

	/**
	 * Saves AlertTemplateUser information
	 *
	 * If the AlertTemplateUser object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @param         \org\adfinis\framework\model\Entity $alertTemplateUser
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $alertTemplateUser) {
		$fieldDataMapping = array(
			'id'                    => $alertTemplateUser->id,
			'userId'                => $alertTemplateUser->userId,
			'slaActionId'           => $alertTemplateUser->slaActionId,
			'templateId'            => $alertTemplateUser->templateId,
		);

		if ($alertTemplateUser->hasIdentifier()) {
			Application::getDbInstance()->update(
				'AlertTemplate_User',
				$fieldDataMapping,
				sprintf('id = %d', $alertTemplateUser->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'AlertTemplate_User',
				$fieldDataMapping
			);
			$alertTemplateUser->id = Application::getDbInstance()->lastInsertId();
		}

		return $alertTemplateUser->getIdentifier();
	}

	/**
	 * Delete a AlertTemplateUser
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			Application::getDbInstance()->delete(
				'AlertTemplate_User',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					tr("This AlertTemplateUser cannot be deleted because of dependencies.")
				);
			}
			else {
				throw $e;
			}
		}

	}
}
