<?php


namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\db\sql;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;
use org\adfinis\framework\exceptions\UserException;


/**
 * Datamapper for the AlertLog model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AuditLogMapper extends Mapper {

	/**
	 * DowntimeMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\DowntimeMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Mapper');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\DownTimeMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Return all audit log entries
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         Zend_Date $dateFrom Optional
	 * @param         Zend_Date $dateTo Optional
	 * @param         Zend_Date $timeFrom Optional
	 * @param         Zend_Date $timeTo Optional
	 * @param         string $entity Optional
	 * @param         string $name Optional
	 * @param         string $user Optional
	 * @param         string $operation Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		\Zend_Date $dateFrom = null,
		\Zend_Date $dateTo   = null,
		$timeFrom  = null,
		$timeTo    = null,
		$entity    = 'all',
		$name      = 'all',
		$user      = 'all',
		$operation = 'all'
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($dateFrom == null) {
			$dateFrom = new \Zend_Date();
			$dateFrom->subDay(1);
		}

		if ($dateTo == null) {
			$dateTo = new \Zend_Date();
		}

		$conditions->addStatement('dateFrom', new sql\WhereStatement(
			'timestamp',
			'>=',
			$dateFrom->get(\Zend_Date::ISO_8601),
			'ala'
		));
		

		if ($dateTo !== null) {
			$conditions->addStatement('dateTo', new sql\WhereStatement(
				'timestamp',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'ala'
			));
		}

		if ($timeFrom !== null) {
			$conditions->addStatement('timeFromHour', new sql\WhereStatement(
				'timestamp',
				'>=',
				$timeFrom,
				'ala',
				false,
				'TIME'
			));
		}

		if ($timeTo !== null) {
			$conditions->addStatement('timeToMinute', new sql\WhereStatement(
				'timestamp',
				'<=',
				$timeTo,
				'ala',
				false,
				'TIME'
			));
		}

		if ($entity !== 'all') {
			$conditions->addStatement('entity', new sql\WhereStatement(
				'entity',
				'=',
				$entity,
				'al'
			));
		}

		if ($name !== 'all') {
			$conditions->addStatement('entityName', new sql\WhereStatement(
				'entityName',
				'=',
				$name,
				'al'
			));
		}

		if ($user !== 'all') {
			$conditions->addStatement('userName', new sql\WhereStatement(
				'userName',
				'=',
				$user,
				'ala'
			));
		}

		if ($operation !== 'all') {
			$conditions->addStatement('operation', new sql\WhereStatement(
				'operation',
				'=',
				$operation,
				'al'
			));
		}

		//make sure that only the first row of AuditLog will be fetch (order > 0 => subauditlogs)
		$conditions->addStatement('firstEntity', new sql\WhereStatement(
			'order',
			'=',
			0,
			'al'
		));

		$searchQuery = sprintf("
			SELECT
				`al`.`id`
			FROM
				`AuditLog` AS `al`
			JOIN `AuditLogAction` AS `ala` ON (
				`ala`.`id` = `al`.`auditLogActionID`
			)
				%s",
				$conditions
		);

		$selectQuery = sprintf("
			SELECT
				`al`.*,
				`ala`.`userID`,
				`ala`.`userName`,
				`ala`.`timestamp`
			FROM
				`AuditLog` AS `al`
			JOIN `AuditLogAction` AS `ala` ON (
				`ala`.`id` = `al`.`auditLogActionID`
			)
		");
		

		$auditLogList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AuditLog',
			array()
		);

		$auditLogList->addOrderVariant('timestamp.DESC',  '`ala`.`timestamp` DESC');
		$auditLogList->addOrderVariant('timestamp.ASC',   '`ala`.`timestamp` ASC');

		return $auditLogList;
	}

	/**
	 * Returns all audit entities
	 *
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getEntities() {
		$query = "
			SELECT DISTINCT
				`entity`
			FROM
				`AuditLog`
			WHERE
				`order` = 0
			ORDER BY
				`entity`
		";

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute();
		$entityList = $statement->fetchAll();

		return array_map(
			function($row) { return $row['entity'];},
			$entityList
		);
	}

	/**
	 * Returns all users
	 *
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $entity optional
	 * @returns       array
	 */
	public function getUsers() {
		$query = "
			SELECT DISTINCT
				`ala`.`userName`,
				`ala`.`userID`,
				`User`.`fullName`
			FROM
				`AuditLogAction` as `ala`

			LEFT JOIN `User` ON
				`User`.`id` = `ala`.`userID`
			
			ORDER BY
				`userName`
		";

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute();
		$userList = $statement->fetchAll();

		return $userList;
	}

	/**
	 * Returns entity names
	 *
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $entity optional
	 * @returns       array
	 */
	public function getEntityNames($entity = null) {
		$query = "
			SELECT DISTINCT
				`entityName`
			FROM
				`AuditLog`
			WHERE
				`order` = 0
		";

		if(!is_null($entity)) {
			$query .= sprintf("AND `entity` = '%s'" , $entity);
		}
		
		$query .= "
			ORDER BY
				`entityName`
		";

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute();
		$entityNamesList = $statement->fetchAll();

		return array_map(
			function($row) {
				return $row['entityName'];
			},
			$entityNamesList
		);
	}

	/**
	 * Audit Logs can not be saved. An exception will be thrown
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         ch\monitoring\model\AuditLog $auditLog
	 * @returns       void
	 */
	public function save(\org\adfinis\framework\model\Entity $auditLog) {
		throw new UserException("You cannot save an audit log");
	}

	/**
	 * Audit Logs can not be deleted. An exception will be thrown
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id identifier of an auditlog
	 * @returns       void
	 */
	public function delete($id) {
		throw new UserException("You cannot delete an audit log");
	}


	/**
	 * Load an AuditLog by id
	 *
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id identifier of an auditlog
	 * @returns       ch\monitoring\model\AuditLog
	 */
	public function getByID($id) {
		$query = "
			SELECT
				`al`.*,
				`ala`.`userID`,
				`ala`.`userName`,
				`ala`.`timestamp`
			FROM
				`AuditLog` as `al`
			
			JOIN `AuditLogAction` AS `ala` ON (
				`ala`.`id` = `al`.`auditLogActionID`
			)

			WHERE
				`al`.`id` = ?
		";

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute(array($id));
		$data = $statement->fetch();

		$audit = new model\AuditLog();
		$audit->fillFromRowData($data);

		return $audit;
	}

	/**
	 * Returns a list of changes that occured to an auditlog
	 *
	 * @author        Jonas Metzener <jonas.metzener@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id identifier of an auditlog
	 * @returns       array
	 */
	public function getLogChanges($id) {
		$query = "
			SELECT
				*
			FROM
				`AuditLogChange`
			WHERE
				`logEntryID` = ?
		";

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute(array($id));
		$data = $statement->fetchAll();
		
		return $data;
	}

	/**
	 * Returns a sorted list of the subauditlogs
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id identifier of an auditlogaction
	 * @returns       array
	 */
	public function getSubAuditLogs($id) {
		
		$searchQuery = "
			SELECT
				`al`.`id`
			FROM
				`AuditLogAction` as `ala`
			
			JOIN `AuditLog` AS `al` ON (
				`al`.`auditLogActionID` = `ala`.`id`
			)

			WHERE
				`ala`.`id` = ?
				AND
				`al`.`order` <> 0
		";


		$selectQuery = "
			SELECT
				`al`.*,
				`ala`.`userID`,
				`ala`.`userName`,
				`ala`.`timestamp`
			FROM
				`AuditLog` AS `al`

			JOIN `AuditLogAction` AS `ala` ON (
				`ala`.`id` = `al`.`auditLogActionID`
			)
		";
		

		$auditLogList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\AuditLog',
			array($id)
		);

		$auditLogList->addOrderVariant('order.ASC',   '`al`.`order` ASC');
		$auditLogList->addOrderVariant('order.DESC',  '`al`.`order` DESC');

		return $auditLogList;
	}
}
