<?php

namespace ch\monitoring\datamapper;

use ch\monitoring\Application;
use ch\monitoring\db\DataTableIterator;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\db\sql;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\view\datatable;

/**
 * Datamapper for the BREApplication model.
 *
 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2015
 * @license       LGPLv3
 **/
class BREApplicationMapper extends Mapper {

	/**
	 * Identity map for BREApplication (by database ID)
	 *
	 * @var           array $identityById
	 */
	private static $identityById = array();

	/**
	 * BREApplicationMapper object to implement singleton
	 *
	 * @var            BREApplicationMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\BREApplication');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\BREApplicationMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a BREApplication by id
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\BREApplication
	 */
	public function getByID($identifier) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_application',
			array("application_id" => $identifier)
		);

		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout
		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}

		if (isset($result->data)) {
			$breapp = new model\BREApplication();
			$breapp->fillFromAPIData($result->data);
			$breapp->allowRead();
			return $breapp;
		}

		return null;
	}

	/**
	 * Returns a single BREApplication object identified by the given SQL query
	 *
	 * Runs the given query, initialises a BREApplication object and returns it
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the BREApplication cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\BREApplication
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find business rule engine application with these parameters'),
					var_export($params, true)
				)
			);
		}

		$breapplication = new model\BREApplication();
		$breapplication->fillFromRowData($data);

		return $breapplication;
	}

	/**
	 * Load all BREApplications
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         int $customerId Optional. The Customer the BREApplication shall belong to.
	 * @param         string $permissionName Defaults to
	 * @return        org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$customerId = null,
		$permissionName = model\Permission::BREAPPLICATION_READ
	) {
		// TODO: use the API written by TobiasF
		return null;
	}

	/**
	 * Gets a list of BRE Applications according to the given Permission
	 *
	 * @author       Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 *
	 * @param        array $exclude list of identifiers of applications which shall be excluded
	 *
	 * @returns      array
	 */
	public function getApplications(
		$customers      = null,
		$permission     = model\Permission::BREAPPLICATION_READ,
		$for            = null
	) {

		if ($customers === null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permission
			);
		}

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);


		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_application_list',
			array(
				"customer_ids" => $customers,
				"application_id"  => $for
			)
		);

		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout
		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}

		$customerMapper = CustomerMapper::getInstance();
		$breapplications = array();

		if (isset($result->data)) {
			foreach ($result->data as $breapplication) {
				$customer = $customerMapper->getByID($breapplication->customer);
				$customer->allowRead();
				$breapplications[$customer->name][$breapplication->id] = $breapplication->name;
			}
		}

		return $breapplications;
	}

	public function update(\org\adfinis\framework\model\Entity $breapplication) {

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.create_application',
			array(
				"name"          => $breapplication->name,
				"customer_id"   => $breapplication->customerId,
				"min_available" => $breapplication->minAvailable
			)
		);

		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}

		$breapplication->id = $result->data->id;

		return $breapplication->getIdentifier();

	}

	/**
	 * Saves BREApplication information
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $breapplication
	 * @throws        UnexpectedValuException
	 * @return        array
	 */
	public function save(\org\adfinis\framework\model\Entity $breapplication) {

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		if ($breapplication->id !== null) {
			$resultId = $rpcClient->putRequest(
				'symonitoring_bpe.update_application',
				array(
					"application_id"  => $breapplication->id,
					"name"            => $breapplication->name,
					"customer_id"     => $breapplication->customerId,
					"min_available"   => $breapplication->minAvailable,
					"description"     => $breapplication->description,
				)
			);
		}

		else {
			$resultId = $rpcClient->putRequest(
				'symonitoring_bpe.create_application',
				array(
					"name"          => $breapplication->name,
					"customer_id"   => $breapplication->customerId,
					"min_available" => $breapplication->minAvailable,
					"description"   => $breapplication->description,
				)
			);
		}


		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}

		$breapplication->id = $result->data->id;

		return $breapplication->getIdentifier();
	}

	/**
	 * Delete a BREApplication
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @return        void
	 */
	public function delete($identifier) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.delete_application',
			array(
				"application_id" => $identifier
			)
		);

		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}
	}
}
