<?php

namespace ch\monitoring\datamapper;

use ch\monitoring\Application;
//use ch\monitoring\db\DataTableIterator;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\db\sql;
use org\adfinis\framework\exceptions\UserException;
use ch\monitoring\view\datatable;

/**
 * Datamapper for the BREComponent model.
 *
 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2015
 * @license       LGPLv3
 **/
class BREComponentMapper extends Mapper {

	/**
	 * Identity map for BREComponent (by database ID)
	 *
	 * @var           array $identityById
	 */
	private static $identityById = array();

	/**
	 * BREComponent object to implement singleton
	 *
	 * @var            BREComponentMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\BREComponent');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\BREComponentMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a BREComponent by ID
	 *
	 * @param         int $id the identifier of the component
	 * @return        ch\monitoring\model\BREComponent
	 */
	public function getByID($id) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_component',
			array("component_id" => $id)
		);

		$result = $rpcClient->getResult($resultId, 10); // 10 is the timeout
		if (isset($result->exception)) {
			throw new UserException($data->exception);
		}

		if (isset($result->data)) {
			$component = new model\BREComponent();
			$component->fillFromAPIData($result->data);
			$component->allowRead();

			return $component;
		}

		return null;
	}

	/**
	 * Returns a single BREComponent object identified by the given SQL query
	 *
	 * Runs the given query, initialises a BREComponent object and returns it
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the BREComponent cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\BREComponent
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find business rule engine components with these parameters'),
					var_export($params, true)
				)
			);
		}

		$brecomponent = new model\BREComponent();
		$brecomponent->fillFromRowData($data);

		return $brecomponent;
	}

	/**
	 * Load all BREComponents
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         int $customerId Optional. The Customer the BREComponent shall belong to.
	 * @param         string $permissionName Defaults to
	 * @return        org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$customerId = null,
		$permissionName = model\Permission::BREAPPLICATION_READ
	) {

		if ($customerId !== null) {
			$customers = [$customerId];
		}
		else {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);
		}

		/* TODO
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_components_list',
			array("customer_ids" => $customers)
		);

		$data = $rpcClient->getResult($resultId, 10); // 10 is the timeout
		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}
		 */

		$brecomponents = array();
		if (isset($data->components)) {
			foreach ($data->component as $brecomponent) {
				$breapplications[$brecomponent->id] = $brecomponent->name;
			}
		}

		return $brecomponents;
	}

	/**
	 * Gets a list of BRE Components according to the given Permission
	 *
	 * @author       Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns      array
	 */
	public function getComponents(
		$customers = null,
		$permission = model\Permission::BREAPPLICATION_READ
	) {

		if ($customers === null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permission
			);
		}

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_component_list',
			array("customer_ids" => $customers)
		);

		$data = $rpcClient->getResult($resultId, 10); // 10 is the timeout
		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}

		$brecomponents = array();
		if (isset($data->components)) {
			foreach ($data->components as $brecomponent) {
				$brecomponents[$brecomponent->id] = $brecomponent->name;
			}
		}

		return $brecomponents;
	}

	/**
	 * Returns all BREComponents belonging to given Host.
	 *
	 * @param         ch\monitoring\model\Host $host
	 *
	 * @return        array of all matching BREComponents
	 */
	public function getByHost($host) {
		$hostId = $host->id;

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_components_by_host',
			array(
				"host_id" => $hostId
			)
		);

		$data = $rpcClient->getResult($resultId, 10);

		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}

		$breComponents = array();
		if (isset($data->components)) {
			foreach ($data->components as $component) {
				$breComponent = new model\BREComponent();
				$breComponent->fillFromAPIData($component);
				$breComponent->allowRead();
				$breComponents[] = $breComponent;
			}
		}

		return $breComponents;
	}

	/**
	 * Returns all BREComponents belonging to given HostServiceInstance.
	 *
	 * @param         ch\monitoring\model\HostServiceInstance $service
	 *
	 * @return        array of all matching BREComponents
	 */
	public function getByService($service) {
		$serviceId = $service->id;

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.get_components_by_service',
			array(
				"service_id" => $serviceId
			)
		);

		$data = $rpcClient->getResult($resultId, 10);

		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}

		$breComponents = array();
		if (isset($data->components)) {
			foreach ($data->components as $component) {
				$breComponent = new model\BREComponent();
				$breComponent->fillFromAPIData($component);
				$breComponent->allowRead();
				$breComponents[] = $breComponent;
			}
		}

		return $breComponents;
	}

	/**
	 * Load all possible parent BRE components
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         int $customerId Optional. The Customer the BREComponent shall belong to.
	 * @param         string $permissionName Defaults to
	 * @return        org\adfinis\framework\db\DbIterator
	 */
	public function getParents(
		$applicationId = null,
		$permissionName = model\Permission::BREAPPLICATION_WRITE
	) {
		// TODO: use the API written by TobiasF
		// filter by component with applicationId == this applicationId
		return null;
	}

	/**
	 * Saves BREComponent information
	 *
	 * If the BREComponent object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $brecomponent
	 * @throws        UnexpectedValuException
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $brecomponent) {

		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		if ($brecomponent->id !== null) {
			$resultId = $rpcClient->putRequest(
				'symonitoring_bpe.update_component',
				array(
					"component_id"   => $brecomponent->id,
					"name"           => $brecomponent->name,
					"min_available"  => $brecomponent->minAvailable,
					"application_id" => $brecomponent->applicationId,
					"parent_id"      => $brecomponent->parentId,
					"description"    => $brecomponent->description
				)
			);
		}
		else {
			$resultId = $rpcClient->putRequest(
				'symonitoring_bpe.create_component',
				array(
					"name"           => $brecomponent->name,
					"min_available"  => $brecomponent->minAvailable,
					"application_id" => $brecomponent->applicationId,
					"parent_id"      => $brecomponent->parentId,
					"description"    => $brecomponent->description
				)
			);
		}

		$data = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}

		return $brecomponent->getIdentifier();
	}

	/**
	 * Delete a BREComponent
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		$rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
		);

		$resultId = $rpcClient->putRequest(
			'symonitoring_bpe.delete_component',
			array(
				"component_id" => $identifier,
			)
		);

		$data = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($data->exception)) {
			throw new UserException($data->exception->message, $data->exception->code);
		}
	}
}
