<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\db\sql;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\exceptions\UserException;

use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Command model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CommandMapper extends Mapper {

	/**
	 * CommandMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\CommandMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Command');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\CommandMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Command by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Command
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(__CLASS__);
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Command`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$command = $this->getByQuery($query, array((int)$id));
			$map->set($command);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Command by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\Command
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Command`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single Command object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Command object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Command cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Command
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find command with these parameters'),
					var_export($params, true)
				)
			);
		}

		$command = new model\Command();
		$command->fillFromRowData($data);

		return $command;
	}

	/**
	 * Load all Commands
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $satelliteId Optional.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($satelliteId = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join  = array();

		if ($satelliteId !== null) {
			$join[] = "
				JOIN `Service` ON (
					`Service`.`commandId` = `Command`.`id`
				)
				JOIN `HostService` ON (
					`HostService`.`serviceId` = `Service`.`id`
				)
				JOIN `Host` ON (
					`Host`.`id` = `HostService`.`hostId`
				)
			";

			$hostMatches = new sql\WhereStatement(
				'id',
				'=',
				$satelliteId,
				'Host'
			);

			$satelliteMatches = new sql\WhereStatement(
				'satelliteId',
				'=',
				$satelliteId,
				'Host'
			);

			$conditions->addStatement('Satellite', new sql\OrStatement(
				$hostMatches,
				$satelliteMatches
			));
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`Command`.`id`
				FROM
 					`Command`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `Command`";

		$commandList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Command',
			array()
		);

		$commandList->addOrderVariant('name.ASC',  '`Command`.`name` ASC');
		$commandList->addOrderVariant('name.DESC', '`Command`.`name` DESC');

		return $commandList;
	}

	/**
	 * Saves Command information
	 *
	 * If the Command object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $command
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $command) {
		$fieldDataMapping = array(
			'id'          => $command->id,
			'name'        => $command->name,
			'commandline' => $command->commandline,
			'description' => $command->description,
			'v6Capable'   => $command->v6Capable,
			'isDeleted'   => $command->isDeleted
		);

		if ($command->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Command',
				$fieldDataMapping,
				sprintf('id = %d', $command->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Command',
				$fieldDataMapping
			);
			$command->id = Application::getDbInstance()->lastInsertId();
		}

		return $command->getIdentifier();
	}

	/**
	 * Delete a Command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->delete(
				'Command',
				sprintf('`id` = %d', $identifier['id'])
			);
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(sprintf(
					"%s: %s",
					tr("This command cannot be deleted because of dependencies"),
					$e->getMessage()
				));
			}
			else {
				throw $e;
			}
		}
	}
}
