<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the CustomerInformation model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerInformationMapper extends Mapper {

	/**
	 * CustomerInformationMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\CustomerInformationMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\CustomerInformation');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\CustomerInformationMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a CustomerInformation by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\CustomerInformation
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`CustomerInformation`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$customerInformation = $this->getByQuery($query, array((int)$id));
			$map->set($customerInformation);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single CustomerInformation object identified by the given SQL query
	 *
	 * Runs the given query, initialises a CustomerInformation object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the CustomerInformation cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\CustomerInformation
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find customer information with these parameters'),
					var_export($params, true)
				)
			);
		}

		$customerInformation = new model\CustomerInformation();
		$customerInformation->fillFromRowData($data);

		return $customerInformation;
	}

	/**
	 * Load all CustomerInformations
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$customers = Application::getUser()->getCustomersWithPermission(
			model\Permission::CUSTOMERS_READ
		);

		$conditions->addStatement('Permission', new sql\WhereStatement(
			'id',
			$customers === null ? 'IS' : 'IN',
			$customers,
			'CustomerInformation',
			false
		));

		$searchQuery = sprintf("
				SELECT
					`CustomerInformation`.`id`
				FROM
					`CustomerInformation`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `CustomerInformation`";

		$customerList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\CustomerInformation',
			array()
		);

		return $customerList;
	}

	/**
	 * Saves CustomerInformation information
	 *
	 * If the CustomerInformation object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $customerInformation
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $customerInformation) {
		$fieldDataMapping = array(
			'id'                => $customerInformation->id,
			'name'              => $customerInformation->name,
			'address'           => $customerInformation->address,
			'contactTechnical'  => $customerInformation->contactTechnical,
			'contactCommercial' => $customerInformation->contactCommercial,
			'phoneRegular'      => $customerInformation->phoneRegular,
			'phoneEmergency'    => $customerInformation->phoneEmergency,
			'comment'           => $customerInformation->comment,
			'commentEmergency'  => $customerInformation->commentEmergency,
			'mayContact'        => $customerInformation->mayContact
		);

		// 1:1 relationship
		try {
			$this->getByID($customerInformation->id)->allowRead();
			Application::getDbInstance()->update(
				'CustomerInformation',
				$fieldDataMapping,
				sprintf('id = %d', $customerInformation->id)
			);
		}
		catch (\OutOfBoundsException $e) {
			Application::getDbInstance()->insert(
				'CustomerInformation',
				$fieldDataMapping
			);
		}

		return $customerInformation->getIdentifier();
	}

	/**
	 * Delete a CustomerInformation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'CustomerInformation',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
