<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Customer model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerMapper extends Mapper {

	/**
	 * CustomerMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\CustomerMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Customer');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\CustomerMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Customer by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Customer`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$customer = $this->getByQuery($query, array((int)$id));
			$map->set($customer);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Customer by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Customer`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Recursively get all sub customers of a customer
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $customerID The ID of the parent customer
	 * @returns       array
	 */
	public function getSubCustomers($customerID, $depth = 0) {
		if ($depth > 10) {
			throw new \RuntimeException(
				tr("Depth is higher than 10")
			);
		}

		$query = sprintf('
			SELECT
				`id`
			FROM
				`Customer`
			WHERE
				`parentId` = %d
			',
			$customerID
		);

		$ids = Application::getDbInstance()->fetchCol($query);

		$subIds = array();
		foreach ($ids as $id) {
			$subIds = array_merge($subIds, $this->getSubCustomers($id, $depth + 1));
		}

		return array_unique(array_merge($ids, $subIds));
	}

	/**
	 * Recursively get all sub customers of a customer
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerID The ID of the parent customer
	 * @param         bool $removeDisabledCustomers optional
	 * @returns       array
	 */
	public function getSubCustomersNames($customerID, $removeDisabledCustomers = false) {
		$whereRestriction = '';

		if($removeDisabledCustomers) {
			$whereRestriction = 'AND isEnabled <> 0';
		}

		$query = sprintf('
			SELECT
				`id`,
				`name`
			FROM
				`Customer`
			WHERE
				`parentId` = %d
				%s
			',
			$customerID,
			$whereRestriction
		);
		$names = Application::getDbInstance()->fetchPairs($query);

		$subNames = array();
		foreach ($names as $id => $name) {
			$subNames = array_merge($subNames, $this->getSubCustomersNames($id));
		}

		return array_unique(array_merge($names, $subNames));
	}

	/**
	 * Returns a single Customer object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a Customer object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Customer cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Customer
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find customer with these parameters'),
					var_export($params, true)
				)
			);
		}

		$customer = new model\Customer();
		$customer->fillFromRowData($data);

		return $customer;
	}

	/**
	 * Load all Customers
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName Defaults to customers read
	 * @param         int $slaId             The SLA id
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$permissionName = model\Permission::CUSTOMERS_READ,
		$slaId          = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');


		if ($slaId !== null) {
			$conditions->addStatement('SLA', new sql\WhereStatement(
				'slaId',
				'=',
				$slaId,
				'Customer'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'id',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Customer',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`Customer`.`id`
				FROM
					`Customer`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `Customer`";

		$customerList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Customer',
			array()
		);

		$customerList->addOrderVariant('name.ASC',  '`Customer`.`name` ASC');
		$customerList->addOrderVariant('name.DESC', '`Customer`.`name` DESC');

		return $customerList;
	}

	/**
	 * Returns a list of all CustomerId that are part of the parent-sequence
	 *
	 * The list will contain the given id
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *  
	 * @param         int customerId
	 * @throws        \RuntimeException if depth is too deep
	 *
	 * @returns       array(int)
	 **/
	public function getAllParentIDs($customerId) {
		$counter = 0;
		$list = array();
		$currentId = $customerId;

		$query = "
			SELECT
				`Customer`.`parentId`
			FROM
				`Customer`
			WHERE
				`Customer`.`id` = ?
		";

		$statement = Application::getDbInstance()->prepare($query);
		
		while($currentId) {
			$list[] = $currentId;

			if($counter++ > 10) {
				throw new \RuntimeException(tr('Depth is higher than 10'));
			}

			$statement->execute(array($currentId));
			$currentId = (int)$statement->fetchColumn();
		}
		
		return $list;
	}

	/**
	 * Returns the default customer
	 *
	 * That is the first customer the current user may access with a certain permission
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName Defaults to CUSTOMERS_READ
	 * @throws        \OutOfBoundsException
	 * @returns       model\Customer
	 */
	public function getDefaultCustomer(
		$permissionName = model\Permission::CUSTOMERS_READ
	) {
		$customers = Application::getUser()->getCustomersWithPermission(
			$permissionName
		);

		if (count($customers) == 0) {
			throw new \OutOfBoundsException(
				sprintf(
					tr('There is no customer the current user may access with the given permission (%s)'),
					$permissionName
				)
			);
		}

		return $customers[0];
	}

	/**
	 * Returns an array of Customer ids and Role objects.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId Return values for this user.
	 * @returns       array
	 */
	public function getCustomersWithRoles($userId) {
		$query = sprintf("
				SELECT
					`Customer`.`id` AS customerId,
					`Role`.`id` as roleId
				FROM
					`User_Role_Customer`

				JOIN `User_Customer` ON (
					`User_Customer`.`id` = `User_Role_Customer`.`userCustomerId`
				)

				LEFT JOIN `Customer` ON (
					`Customer`.`id` = `User_Customer`.`customerId`
				)

				JOIN `Role` ON (
					`Role`.`id` = `User_Role_Customer`.`roleId`
				)
				
				WHERE
					`User_Customer`.`userId` = %d
					AND
					`Customer`.`isEnabled` = 1
			",
			$userId
		);

		$customersRoles = array();
		$roles = Application::getDBInstance()->fetchAll($query);
		foreach ($roles as $role) {
			$customersRoles[$role['customerId']][] = RoleMapper::getInstance()->getByID(
				$role['roleId']
			)->allowRead();
		}

		return $customersRoles;
	}

	/**
	 * Saves Customer information
	 *
	 * If the Customer object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $customer
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $customer) {
		$fieldDataMapping = array(
			'id'                  => $customer->id,
			'parentId'            => $customer->parentId,
			'slaId'               => $customer->slaId,
			'name'                => $customer->name,
			'email'               => $customer->email,
			'warnHoster'          => $customer->warnHoster,
			'wiki'                => $customer->wiki,
			'comment'             => $customer->comment,
			'passwordInformation' => $customer->passwordInformation,
			'passwordMaintenance' => $customer->passwordMaintenance,
			'lastAccess'          => util\TimeUtility::dateOrNull($customer->lastAccess),
			'isEnabled'           => $customer->isEnabled,
			'isDeleted'           => $customer->isDeleted,
			'ldapGroup'           => $customer->ldapGroup,
			'defaultRoleId'       => $customer->defaultRoleId,
		);

		if ($customer->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Customer',
				$fieldDataMapping,
				sprintf('id = %d', $customer->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Customer',
				$fieldDataMapping
			);
			$customer->id = Application::getDbInstance()->lastInsertId();
		}

		return $customer->getIdentifier();
	}

	/**
	 * Delete a Customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			$query = '
				DELETE `Infofeed` FROM `Infofeed`
				JOIN `Infofeed_Customer` ON (
					`Infofeed_Customer`.`infofeedId` = `Infofeed`.`id`
				)
				WHERE
					`Infofeed_Customer`.`customerId` = ?
			';
			$statement = Application::getDbInstance()->prepare($query);
			$statement->execute(array(
				$identifier['id']
			));

			Application::getDbInstance()->delete(
				'Customer',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					sprintf(
						"%s: %s",
						tr("This customer cannot be deleted because of dependencies"),
						$e->getMessage()
					)
				);
			}
			else {
				throw $e;
			}
		}
	}


	/**
	 * Checks if the customer has a permission for the current user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @param         string $permissionName
	 * @returns       void
	 */
	public function hasPermission($customerId, $permissionName) {
		$query = "
			SELECT
				COUNT(*)
			FROM
				`Permission`

			JOIN `Role_Permission` ON (
				`Role_Permission`.`permissionId` = `Permission`.`id`
			)

			JOIN `User_Role_Customer` ON (
				`User_Role_Customer`.`roleId` = `Role_Permission`.`roleId`
			)
			
			JOIN `User_Customer` ON (
				`User_Customer`.`id` = `User_Role_Customer`.`userCustomerId`
			)

			WHERE
				`Permission`.`name` = ?
				AND
				`User_Customer`.`customerId` = ?
				AND
				`User_Customer`.`userId` = ?
		";

		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute(array($permissionName, Application::getUser()->id));
		return (bool)$statement->fetchColumn();
	}

	/**
	 * Returns the customer which expans the whole customer tree
	 * The root customer is like highlander ... it can only be one!
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       \org\adfinis\framework\model\Entity
	 *
	 */
	public function getRootCustomer() {
		$query = '
			SELECT
				*
			FROM
				`Customer`
			WHERE
				`parentId` IS NULL
			LIMIT
				1
		';

		return $this->getByQuery($query, array());
	}


	/**
	 * Sets the value (disabled, enabled, inherit) of an AlertType
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @param         int $alertTypeId
	 * @param         bool|null $value
	 * @returns       array
	 */
	public function setAlertSetting($customerId, $alertTypeId, $value = null) {
		$query = '
			INSERT INTO
				`NotificationSetting`
				(`customerId`, `slaActionId`, `enabled`)
			VALUES
				(?, ?, ?)
			ON DUPLICATE KEY UPDATE
				`enabled` = ?
		';

		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute(array(
			(int)$customerId,
			(int)$alertTypeId,
			(int)$value,
			(int)$value 
		));
	}
	
	/**
	 * Get the customer for an Infofeed
	 *
	 * @param         int $infofeedId
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getByInfofeedId($infofeedId) {
		$query = "
			SELECT
				`Customer`.*
			FROM
				`Customer`
			JOIN `Infofeed_Customer` ON (
				`Infofeed_Customer`.`customerId` = `Customer`.`id`
			)
			WHERE
				`Infofeed_Customer`.`infofeedId` = ?
			LIMIT 1
		";

		return $this->getByQuery($query, array((int)$infofeedId));

	}

	/**
	 * Returns a DB Iterator with all the AlertSettings of the customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @returns       array
	 */
	public function getAlertSettings($customerId) {
		$query = '
			SELECT
				`SLAAction`.`id`,
				IFNULL(`NotificationSetting`.`enabled`, 0) AS `enabled`
			FROM
				`SLAAction`

			LEFT JOIN `NotificationSetting` ON (
				`NotificationSetting`.`slaActionId` = `SLAAction`.`id`
				AND `NotificationSetting`.`customerId` = ?
			)
		';
		

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute(array($customerId));
		$list = array();

		foreach($statement->fetchAll() as $fields) {
			$list[$fields['id']] = $fields['enabled'];
		}

		return $list;
	}


	/**
	 * Returns the customer matching the given LDAP group name
	 *
	 * @param         string $ldapGroupName  The name of the LDAP group
	 * @throws        InvalidArgumentException  If no argument was given
	 * @returns       ch\monitoring\db\SortingPolicyIterator found customers
	 */
	public function getByLDAPGroupName($ldapGroupName) {
		if (strlen($ldapGroupName) === 0) {
			throw new InvalidArgumentException(tr('No LDAP group name given'));
		}
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
		$conditions->addStatement('LDAPGroupName', new sql\WhereStatement(
			'ldapGroup',
			'=',
			$ldapGroupName,
			'Customer',
			false
		));
		$searchQuery = sprintf("
			SELECT
				`Customer`.*
			FROM
				`Customer`
			%s
		", $conditions);
		$selectQuery = "SELECT * FROM `Customer`";
		$list = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Customer'
		);
		$list->addOrderVariant('id.ASC',        '`Customer`.`id` ASC');
		return $list;
	}
}
