<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\datamapper;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the CustomerTeam model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerTeamMapper extends Mapper {

	/**
	 * Cache for customer teams
	 *
	 * @var           array $teams
	 */
	private static $teams;

	/**
	 * CustomerTeamMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\CustomerTeamMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\CustomerTeam');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\CustomerTeamMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a CustomerTeam by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\CustomerTeam
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`CustomerTeam`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$customerTeam = $this->getByQuery($query, array((int)$id));
			$map->set($customerTeam);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single CustomerTeam object identified by the given SQL query
	 *
	 * Runs the given query, initialises a CustomerTeam object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the CustomerTeam cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\CustomerTeam
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find customer team with these parameters'),
					var_export($params, true)
				)
			);
		}

		$customerTeam = new model\CustomerTeam();
		$customerTeam->fillFromRowData($data);

		return $customerTeam;
	}

	/**
	 * Load all CustomerTeam members
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Zend_Date $data Null means default team.
	 * @param         int $customerId Return customer teams of this customer.
	 * @param         string $permissionName The permission which is required (null means none)
	 * @returns       array
	 */
	public function getAll(
		$date,
		$customerId,
		$permissionName = model\Permission::EMERGENCY_READ
	) {
		$dateIndex = $date instanceof \Zend_Date ? $date->get() : "null";
		if (isset(self::$teams[$customerId]) && isset(self::$teams[$customerId][$dateIndex])) {
			return self::$teams[$customerId][$dateIndex];
		}

		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'=',
				$customerId,
				'CustomerTeam'
			));
		}

		if ($date instanceof \Zend_Date) {
			$conditions->addStatement('Date', new sql\WhereStatement(
				'date',
				'=',
				$date->get(util\TimeUtility::MYSQL_DATE),
				'CustomerTeam'
			));
		}
		else {
			$conditions->addStatement('Date', new sql\WhereStatement(
				'date',
				'IS',
				null,
				'CustomerTeam',
				false
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'CustomerTeam',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`CustomerTeam`.`id`
				FROM
					`CustomerTeam`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `CustomerTeam`";

		$customerTeamList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\CustomerTeam',
			array()
		)->toArray();

		self::$teams[$customerId][$dateIndex] = $customerTeamList;

		return $customerTeamList;
	}

	/**
	 * Count all CustomerTeams
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $userId optional
	 * @param         \Zend_Date $fromDate optional
	 * @returns       int
	 */
	public function countTeams($userId = null, $fromDate = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if(!is_null($userId)) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'CustomerTeam',
				false
			));
		}

		if(!is_null($fromDate)) {
			$subconditions = new sql\WhereGroup();
			$subconditions->setOperand('OR');

			$subconditions->addStatement('DateNull', new sql\WhereStatement(
				'date',
				'IS',
				NULL,
				'CustomerTeam',
				false
			));

			$subconditions->addStatement('DateBigger', new sql\WhereStatement(
				'date',
				'>=',
				$fromDate->get(\Zend_Date::DATE_MEDIUM),
				'CustomerTeam',
				false
			));

			$conditions->addStatement('Date', $subconditions->getSQLConditions());
		}


		$query = sprintf('
			SELECT
				COUNT(*)
			FROM
				`CustomerTeam`
			%s
			',
			$conditions
		);
		
		return Application::getDbInstance()->fetchOne($query);
	}


	/**
	 * Saves CustomerTeam information
	 *
	 * If the CustomerTeam object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $customerTeam
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $customerTeam) {
		$fieldDataMapping = array(
			'id'         => $customerTeam->id,
			'userId'     => $customerTeam->userId,
			'customerId' => $customerTeam->customerId,
			'date'       => util\TimeUtility::dateOrNull($customerTeam->date),
			'priority'   => $customerTeam->priority
		);

		if ($customerTeam->hasIdentifier()) {
			Application::getDbInstance()->update(
				'CustomerTeam',
				$fieldDataMapping,
				sprintf('id = %d', $customerTeam->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'CustomerTeam',
				$fieldDataMapping
			);
			$customerTeam->id = Application::getDbInstance()->lastInsertId();
		}

		return $customerTeam->getIdentifier();
	}

	/**
	 * Returns a calendar view
	 *
	 * Returns an array of currently responsible CustomerTeam members of a given
	 * customer for each day of a certain month.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $year
	 * @param         int $month
	 * @param         int $customerId
	 * @returns       array
	 */
	public function getCalendarMonthArray($year, $month, $customerId) {
		// The first day of the given month
		$dateMonthFirstArray = array(
			'day' => 1,
			'month' => $month,
			'year' => $year
		);
		$dateMonthFirst = new \Zend_Date($dateMonthFirstArray);

		// The last day of the given month
		$dateMonthLastArray = array(
			'day' => $dateMonthFirst->get(\Zend_Date::MONTH_DAYS),
			'month' => $month,
			'year' => $year
		);
		$dateMonthLast = new \Zend_Date($dateMonthLastArray);

		// Calculate the first day to be shown on the calendar
		$offsetFirst = ($dateMonthFirst->get(\Zend_Date::WEEKDAY_DIGIT) - 1) % 6;
		$dateCalendarFirst = clone $dateMonthFirst;
		$dateCalendarFirst->sub($offsetFirst, \Zend_Date::DAY);


		// Calculate the last day to be shown on the calendar
		$offsetLast = 6 - (($dateMonthLast->get(\Zend_Date::WEEKDAY_DIGIT) - 1) % 6);
		$dateCalendarLast = clone $dateMonthLast;
		$dateCalendarLast->add($offsetLast, \Zend_Date::DAY);


		// Iterator for the calendar
		$dateCalendarCurrent = clone $dateCalendarFirst;

		$nextMonth = clone $dateMonthFirst;
		$nextMonth->add(1, \Zend_Date::MONTH);

		$prevMonth = clone $dateMonthFirst;
		$prevMonth->sub(1, \Zend_Date::MONTH);

		// Fill the calendar array
		$calendar = array(
			'dateCalendarFirst' => $dateCalendarFirst,
			'dateCalendarLast'  => $dateCalendarLast,
			'nextMonth'         => $nextMonth,
			'prevMonth'         => $prevMonth,
			'thisMonth'         => $dateMonthFirst,
			'customerId'        => $customerId,
			'defaultTeam'       => $this->getAll(null, $customerId),
			'currentDate'       => new \Zend_Date()
		);
		while (!$dateCalendarCurrent->isLater($dateCalendarLast)) {
			$calendar['dates'][] = array(
				'date'    => clone $dateCalendarCurrent,
				'members' => $this->getAll($dateCalendarCurrent, $customerId)
			);

			$dateCalendarCurrent->add(1, \Zend_Date::DAY);
		}

		return $calendar;
	}

	/**
	 * Delete a CustomerTeam member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'CustomerTeam',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Delete a CustomerTeam by date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Zend_Date $date
	 * @param         int $customerId
	 * @returns       void
	 */
	public function deleteByDate($date, $customerId) {
		Application::getDbInstance()->beginTransaction();

		$members = $this->getAll($date, $customerId);
		foreach ($members as $member) {
			$this->delete($member->getIdentifier());
		}
		self::$teams = null;

		Application::getDbInstance()->commit();
	}

	/**
	 * Delete customer teams by customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @returns       void
	 */
	public function deleteByCustomerId($customerId) {
		Application::getDbInstance()->delete(
			'CustomerTeam',
			sprintf('`customerId` = %d', $customerId)
		);
	}

	/**
	 * Delete customer teams by user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId
	 * @returns       void
	 */
	public function deleteByUserId($userId) {
		Application::getDbInstance()->delete(
			'CustomerTeam',
			sprintf('`userId` = %d', $userId)
		);
	}

	/**
	 * Returns the currently responsible team for a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getCurrentTeam($hostId) {
		$host = HostMapper::getInstance()->getByID(
			$hostId
		)->allowRead();

		if ($host->slaId != null) {
			$sla = $host->getSLA();
		}
		else {
			$sla = $host->getCustomer()->getSLA();
		}

		$now = new \Zend_Date();

		// We ASSUME here to get only one currently active timeframe
		// (which is probably wrong)
		$timeframes = datamapper\TimeframeMapper::getInstance()->getAll($sla->id, true);

		foreach ($timeframes as $timeframe) {
			$customerId = $timeframe->customerId;

			$team = $this->getAll(
				$now,
				$customerId,
				null
			);

			if (count($team) == 0) {
				$team = $this->getAll(
					null,
					$customerId,
					null
				);

				if (count($team) == 0) {
					$team = null;
				}
			}

			return $team;
		}

		return null;
	}
}
