<?php

namespace ch\monitoring\datamapper;

use ch\monitoring\Application;
use ch\monitoring\db\DataTableIterator;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\util;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\view\datatable;

/**
 * Datamapper for the Downtime model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class DowntimeMapper extends Mapper {

	/**
	 * DowntimeMapper object to implement singleton
	 *
	 * @var            DowntimeMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Downtime');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\DownTimeMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Downtime by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Downtime
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`Downtime`.*
				FROM
					`Downtime`
				WHERE
					`Downtime`.`id` = ?
				LIMIT
					1
			';

			$downtime = $this->getByQuery($query, array((int)$id));
			$map->set($downtime);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single Downtime object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Downtime object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Downtime cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Downtime
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find downtime with these parameters'),
					var_export($params, true)
				)
			);
		}

		$downtime = new model\Downtime();
		$downtime->fillFromRowData($data);

		return $downtime;
	}

	/**
	 * Load all Downtimes
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId Optional. The Customer the downtimes shall belong to.
	 * @param         int $hostId Optional. The Host the downtimes shall belong to.
	 * @param         int $serviceId Optional. The Service the downtimes shall belong to.
	 * @param         \Zend_Date $dateFrom Optional.
	 * @param         \Zend_Date $dateTo Optional.
	 * @param         string $permissionName Defaults to
	 * @param         null|bool $isRecurring
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$customerId = null,
		$hostId     = null,
		$serviceId  = null,
		$dateFrom   = null,
		$dateTo     = null,
		$permissionName = model\Permission::DOWNTIMES_READ,
		$isRecurring = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'=',
				$customerId,
				'Host'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'Downtime'
			));
		}

		if ($serviceId !== null) {
			$conditions->addStatement('Service', new sql\WhereStatement(
				'serviceInstanceId',
				'=',
				$serviceId,
				'Downtime'
			));
		}

		if ($dateFrom !== null) {
			$conditions->addStatement('Date from', new sql\WhereStatement(
				'datetimeFrom',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'Downtime'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('Date to', new sql\WhereStatement(
				'datetimeTo',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'Downtime'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		if ($isRecurring !== null) {
			$conditions->addStatement('Recurring', new sql\WhereStatement(
				'rrule',
				$isRecurring ? 'IS NOT' : 'IS',
				NULL,
				'Downtime',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`Downtime`.`id`
				FROM
					`Downtime`
				JOIN `Host` ON (
					`Host`.`id` = `Downtime`.`hostId`
				)
				%s
			",
			$conditions
		);


		$selectQuery = "SELECT * FROM `Downtime`";

		$downtimeList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Downtime',
			array()
		);

		$downtimeList->addOrderVariant('datetimeTo.DESC',  '`Downtime`.`datetimeTo` DESC');

		return $downtimeList;
	}

	/**
	 * Load all Downtimes for DataTables
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array  $settings
	 * @param         string $permissionName Defaults to
	 * @return        DataTableIterator
	 */
	public function getAllForDataTable(
		$settings,
		$permissionName = model\Permission::DOWNTIMES_READ
	) {
		$model = 'ch\monitoring\model\Downtime';
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($settings['hostId'] !== '0') {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$settings['hostId'],
				'Downtime'
			));
		}

		if($settings['recurring'] !== '') {
			$conditions->addStatement('Recurring', new sql\WhereStatement(
				'rrule',
				$settings['recurring'] ? 'IS NOT' : 'IS',
				NULL,
				'Downtime',
				false
			));
		}

		$fieldNameMapping = [
			datatable\Downtime::CUSTOMER  => ['Customer', 'name'],
			datatable\Downtime::HOST      => ['Host',     'hostname'],
			datatable\Downtime::SERVICES  => ['Service',  'name'],
			datatable\Downtime::RECURRING => ['Downtime', 'rrule', 'IS'],
		];
		$filter = DataTableIterator::getFilter($model, $settings, $fieldNameMapping);

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		if ((string)$conditions || $filter) {
			$filter = sprintf('WHERE %s',
				join(' AND ', array_filter([$conditions->getSQLConditions(), $filter]))
			);
		}

		$joins[] = '
			JOIN `Host` ON (
				`Host`.`id` = `Downtime`.`hostId`
			)
			JOIN `Customer` ON (
				`Customer`.`id` = `Host`.`customerId`
			)
			LEFT JOIN `Service` ON (
				`Service`.`id` = `Downtime`.`serviceInstanceId`
			)
		';

		$searchQuery = sprintf("
				SELECT
					`Downtime`.`id`
				FROM
					`Downtime`
				%s
				%s
			",
			implode("\n", $joins),
			$filter
		);

		$selectQuery = "SELECT * FROM `Downtime`";

		$downtimeList = DataTableIterator::create(
			$searchQuery,
			$selectQuery,
			$model
		);
		$downtimeList->setSettings($settings);

		$downtimeList->addOrderVariant('customer.ASC'  , '`Customer`.`name` ASC');
		$downtimeList->addOrderVariant('customer.DESC' , '`Customer`.`name` DESC');
		$downtimeList->addOrderVariant('hostname.ASC'  , '`Host`.`hostname` ASC');
		$downtimeList->addOrderVariant('hostname.DESC' , '`Host`.`hostname` DESC');
		$downtimeList->addOrderVariant('service.ASC'   , '`Service`.`name` ASC');
		$downtimeList->addOrderVariant('service.DESC'  , '`Service`.`name` DESC');
		$downtimeList->addOrderVariant('from.ASC'      , '`Downtime`.`datetimeFrom` ASC');
		$downtimeList->addOrderVariant('from.DESC'     , '`Downtime`.`datetimeFrom` DESC');
		$downtimeList->addOrderVariant('to.ASC'        , '`Downtime`.`datetimeTo` ASC');
		$downtimeList->addOrderVariant('to.DESC'       , '`Downtime`.`datetimeTo` DESC');
		$downtimeList->addOrderVariant('recurring.ASC' , '`Downtime`.`rrule` ASC');
		$downtimeList->addOrderVariant('recurring.DESC', '`Downtime`.`rrule` DESC');

		return $downtimeList;
	}

	/**
	 * Saves Downtime information
	 *
	 * If the Downtime object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $downtime
	 * @throws        UnexpectedValuException
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $downtime) {
		$fieldDataMapping = array(
			'id'                => $downtime->id,
			'hostId'            => $downtime->hostId,
			'serviceInstanceId' => $downtime->serviceInstanceId ?: null,
			'datetimeFrom'      => util\TimeUtility::dateOrNull($downtime->datetimeFrom),
			'datetimeTo'        => util\TimeUtility::dateOrNull($downtime->datetimeTo),
			'rrule'             => $downtime->rrule,
			'durationHours'     => $downtime->durationHours
		);

		if ($downtime->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Downtime',
				$fieldDataMapping,
				sprintf('id = %d', $downtime->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Downtime',
				$fieldDataMapping
			);
			$downtime->id = Application::getDbInstance()->lastInsertId();

			//recurring downtimes cannot not be scheduled to icinga/nagios
			if(empty($downtime->rrule)) {
				$this->schedule($downtime);
			}
		}

		return $downtime->getIdentifier();
	}

	/**
	 * Delete a Downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			$this->unschedule($this->getByID($identifier['id'])->allowRead());
		}
		catch (\RuntimeException $e) {
			// Ignore
		}

		Application::getDbInstance()->delete(
			'Downtime',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Schedule a downtime in Icinga/Nagios
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Downtime $downtime
	 * @returns       void
	 */
	public function schedule($downtime) {
		if ($downtime->serviceInstanceId === null) {
			$command = array(
				'SCHEDULE_HOST_DOWNTIME',
				$downtime->getHost()->hostname,                      // host_name
				$downtime->datetimeFrom->get(\Zend_Date::TIMESTAMP), // start_time
				$downtime->datetimeTo->get(\Zend_Date::TIMESTAMP),   // end_time
				'1',                                                 // fixed
				'0',                                                 // trigger_id
				'0',                                                 // duration (ignored when fixed)
				'root',                                              // author
				'Added by SyMonitoring'                              // comment
			);
			util\Monitoring::sendCommand($command);
		}
		else {
			$command = array(
				'SCHEDULE_SVC_DOWNTIME',
				$downtime->getHost()->hostname,                       // host_name
				$downtime->getHostServiceInstance()->getObjectName(), // service_description
				$downtime->datetimeFrom->get(\Zend_Date::TIMESTAMP),  // start_time
				$downtime->datetimeTo->get(\Zend_Date::TIMESTAMP),    // end_time
				'1',                                                  // fixed
				'0',                                                  // trigger_id
				'0',                                                  // duration (ignored when fixed)
				'root',                                               // author
				'Added by SyMonitoring'                               // comment
			);
			util\Monitoring::sendCommand($command);
		}
	}

	/**
	 * Unschedule a downtime in Icinga/Nagios
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Downtime $downtime
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	public function unschedule($downtime) {
		$downtimeStatement = Application::getDbInstance('icingaIdo')->prepare("
			SELECT
				`internal_downtime_id`
			FROM
				`icinga_scheduleddowntime`
			WHERE
				`downtime_type` = :downtimeType
				AND
				`object_id` = :objectId
				AND
				`scheduled_start_time` = :dateFrom
				AND
				`scheduled_end_time` = :dateTo
		");

		// We need to set timezone explicitely to UTC
		// for datetimeFrom and datetimeTo as otherwise
		// downtimes cannot be unscheduled as Icinga holds
		// them - given by the insertion as timestamp - as UTC.
		// If we use the default locale, which might not be UTC,
		// the downtime might not be found.
		$serverTimezone = date_default_timezone_get();
		$downtime->datetimeFrom->setTimezone($serverTimezone);
		$downtime->datetimeTo->setTimezone($serverTimezone);

		if ($downtime->serviceInstanceId === null) {
			$hostStatement = Application::getDbInstance('icingaIdo')->prepare("
				SELECT
					`host_object_id`
				FROM
					`icinga_hosts`
				WHERE
					`display_name` = :hostname
			");
			$hostStatement->execute(array(
				'hostname' => $downtime->getHost()->hostname
			));
			$icingaHostId = $hostStatement->fetch(\PDO::FETCH_COLUMN, 0);

			if (empty($icingaHostId)) {
				throw new \RuntimeException(sprintf(
					tr('Icinga host id (%s) not found when unscheduling host'),
					$downtime->getHost()->hostname
				));
			}

			$downtimeStatement->execute(array(
				'downtimeType' => 2,
				'objectId'     => $icingaHostId,
				'dateFrom'     => $downtime->datetimeFrom->get(\Zend_Date::ISO_8601),
				'dateTo'       => $downtime->datetimeTo->get(\Zend_Date::ISO_8601)
			));
			$icingaDowntimeId = $downtimeStatement->fetch(\PDO::FETCH_COLUMN, 0);

			if (empty($icingaDowntimeId)) {
				throw new \RuntimeException(sprintf(
					tr(
						'Icinga downtime id not found ' .
						'when unscheduling host ' .
						'(downtime type: 2, object id: %d, ' .
						'date from: %s, date to: %s)'
					),
					$icingaHostId,
					$downtime->datetimeFrom->get(\Zend_Date::ISO_8601),
					$downtime->datetimeTo->get(\Zend_Date::ISO_8601)
				));
				return;
			}

			$command = array(
				'DEL_HOST_DOWNTIME',
				$icingaDowntimeId
			);
			util\Monitoring::sendCommand($command);
		}
		else {
			$hostStatement = Application::getDbInstance('icingaIdo')->prepare("
				SELECT
					`icinga_services`.`service_object_id`
				FROM
					`icinga_services`,
					`icinga_hosts`
				WHERE
					`icinga_services`.`display_name` = :serviceName
					AND
					`icinga_services`.`host_object_id` = `icinga_hosts`.`host_object_id`
					AND
					`icinga_hosts`.`display_name` = :hostname
			");
			$hostStatement->execute(array(
				'serviceName' => $downtime->getHostServiceInstance()->getObjectName(),
				'hostname'    => $downtime->getHost()->hostname
			));
			$icingaServiceId = $hostStatement->fetch(\PDO::FETCH_COLUMN, 0);

			if (empty($icingaServiceId)) {
				throw new \RuntimeException(sprintf(
					tr('Icinga host id (%s) not found when unscheduling service'),
					$downtime->getHostServiceInstance()->getObjectName()
				));
			}

			$downtimeStatement->execute(array(
				'downtimeType' => 1,
				'objectId'     => $icingaServiceId,
				'dateFrom'     => $downtime->datetimeFrom->get(\Zend_Date::ISO_8601),
				'dateTo'       => $downtime->datetimeTo->get(\Zend_Date::ISO_8601)
			));
			$icingaDowntimeId = $downtimeStatement->fetch(\PDO::FETCH_COLUMN, 0);

			if (empty($icingaDowntimeId)) {
				throw new \RuntimeException(sprintf(
					tr(
						'Icinga downtime id not found ' .
						'when unscheduling service ' .
						'(downtime type: 1, object id: %d, ' .
						'date from: %s, date to: %s)'
					),
					$icingaServiceId,
					$downtime->datetimeFrom->get(\Zend_Date::ISO_8601),
					$downtime->datetimeTo->get(\Zend_Date::ISO_8601)
				));
				return;
			}

			$command = array(
				'DEL_SVC_DOWNTIME',
				$icingaDowntimeId
			);
			util\Monitoring::sendCommand($command);
		}
		// If no timezone is given, Zend_Date reverts back to the default
		// timezone
		$downtime->datetimeFrom->setTimezone();
		$downtime->datetimeTo->setTimezone();
	}

	/**
	 * Unschedule all active downtimes from host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Host $host
	 * @param         HostServiceInstance $instance
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	public function unscheduleHost($host, $instance) {
		$now = new \Zend_Date();

		if (is_null($instance)) {
			$instanceString = 'IS NULL';
		}
		else {
			$instanceString = sprintf('= %d', $instance->id);
		}
		$condition = sprintf('
				`hostId` = %d
				AND
				`serviceInstanceId` %s
				AND
				`datetimeFrom` < \'%s\'
				AND
				`datetimeTo` > \'%s\'
			',
			$host->id,
			$instanceString,
			$now->get(\Zend_Date::ISO_8601),
			$now->get(\Zend_Date::ISO_8601)
		);
		Application::getDbInstance()->update(
			'Downtime',
			array(
				'datetimeTo' => $now->get(\Zend_Date::ISO_8601)
			),
			$condition
		);
	}
}
