<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\db\sql;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;
use ch\monitoring\db\SortingPolicyIterator;
use org\adfinis\framework\exceptions\UserException;

/**
 * Datamapper for the Guideline model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class GuidelineMapper extends Mapper {

	/**
	 * GuidelineMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\GuidelineMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Guideline');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\GuidelineMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Guideline by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Guideline
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Guideline`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$guideline = $this->getByQuery($query, array($id));
			$map->set($guideline);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single Guideline object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Guideline object and returns it
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Guideline cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Guideline
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find guideline with these parameters'),
					var_export($params, true)
				)
			);
		}

		$guideline = new model\Guideline();
		$guideline->fillFromRowData($data);

		return $guideline;
	}

	/**
	 * Load all Guidelines
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$selectQuery = "SELECT * FROM `Guideline`";
		$searchQuery = "SELECT * FROM `Guideline`";

		$guidelineList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Guideline'
		);

		$guidelineList->addOrderVariant('id.DESC', '`Guideline`.`id` DESC');
		$guidelineList->addOrderVariant('id.ASC',  '`Guideline`.`id` ASC');

		return $guidelineList;
	}

	/**
	 * Load all Guidelines that are assigned to the command ID.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllByCommandID($commandId) {
		$searchQuery= sprintf("
			SELECT
				`id`
			FROM
				`Guideline`
			WHERE
				`commandId` = %d
		", $commandId);
		$selectQuery = "
			SELECT
				*
			FROM
				`Guideline`
		";
		$guidelineList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Guideline',
			array()
		);

		$guidelineList->addOrderVariant('id.DESC', '`Guideline`.`id` DESC');
		$guidelineList->addOrderVariant('id.ASC',  '`Guideline`.`id` ASC');

		return $guidelineList;
	}

	/**
	 * Load all Guidelines that are assigned to the command ID and hostOS Version ID.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getByVersionID($osVersionId, $commandId) {
		$searchQuery = sprintf("
			SELECT
				`id`
			FROM 
				`Guideline`
			WHERE 
				`commandId` = %d
				AND
				`versionId` = %d
			",
			$commandId,
			$osVersionId
		);
		$selectQuery = "
			SELECT
				*
			FROM
				`Guideline`
		";
		$guidelineList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Guideline'
		);
		return $guidelineList;
	}

	/**
	 * Return true if there is a Guideline for the command ID and Host OS Version ID.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       boolean
	 */
	public function doesGuidelineExist($osVersionId, $commandId) {
		$list = $this->getByVersionID($osVersionId, $commandId);
		return (bool) $list->count();
	}


	/**
	 * Saves Guideline information
	 *
	 * If the Guideline object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $guideline
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $guideline) {
		$fieldDataMapping = array(
			'id'                  => $guideline->id,
			'content'             => $guideline->content,
			'commandId'           => $guideline->commandId,
			'userId'              => $guideline->userId,
			'created'             => util\TimeUtility::dateOrNull($guideline->created),
			'updated'             => util\TimeUtility::dateOrNull($guideline->updated),
			'versionId'           => $guideline->versionId,
			'needsManualInstallation' => $guideline->needsManualInstallation,
			'filename'            => $guideline->filename
		);

		if ($guideline->hasIdentifier()) {
			Application::getDBInstance()->update(
				'Guideline',
				$fieldDataMapping,
				sprintf('id = %d', $guideline->id)
			);
		}
		else {
			Application::getDBInstance()->insert(
				'Guideline',
				$fieldDataMapping
			);
			$guideline->id = Application::getDBInstance()->lastInsertId();
		}

		return $guideline->getIdentifier();
	}

	/**
	 * Delete a guideline
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'Guideline',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
