<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the HostGroup model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostGroupMapper extends Mapper {

	/**
	 * HostGroupMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostGroupMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostGroup');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostGroupMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostGroup by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostGroup`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostGroup = $this->getByQuery($query, array((int)$id));
			$map->set($hostGroup);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostGroup by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`HostGroup`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		$hostGroup = $this->getByQuery($query, array($name));
		return $hostGroup;
	}

	/** 
	 * Returns a single HostGroup object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a HostGroup object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostGroup cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostGroup
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service group with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostGroup = new model\HostGroup();
		$hostGroup->fillFromRowData($data);

		return $hostGroup;
	}

	/**
	 * Load all HostGroups
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName Defaults to services read
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$permissionName = model\Permission::HOSTS_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$customerNull = new sql\WhereStatement(
			'customerId',
			'IS',
			null,
			'HostGroup',
			false
		);

		$customers = null;
		if ($permissionName) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);
		}

		$customerValid = new sql\WhereStatement(
			'customerId',
			$customers === null ? 'IS' : 'IN',
			$customers,
			'HostGroup',
			false
		);

		$conditions->addStatement('Permission', new sql\OrStatement(
			$customerNull,
			$customerValid
		));

		$searchQuery = sprintf("
				SELECT
					`HostGroup`.`id`
				FROM
					`HostGroup`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `HostGroup`";

		$hostGroupList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostGroup',
			array()
		);

		$hostGroupList->addOrderVariant('name.ASC',  '`HostGroup`.`name` ASC');
		$hostGroupList->addOrderVariant('name.DESC', '`HostGroup`.`name` DESC');

		return $hostGroupList;
	}

	/**
	 * Saves HostGroup information
	 *
	 * If the HostGroup object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostGroup
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostGroup) {
		$fieldDataMapping = array(
			'id'          => $hostGroup->id,
			'customerId'  => $hostGroup->customerId,
			'name'        => $hostGroup->name,
			'description' => $hostGroup->description
		);

		if ($hostGroup->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostGroup',
				$fieldDataMapping,
				sprintf('id = %d', $hostGroup->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostGroup',
				$fieldDataMapping
			);
			$hostGroup->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostGroup->getIdentifier();
	}

	/**
	 * Delete a HostGroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		$query = '
			DELETE `Infofeed` FROM `Infofeed`
			JOIN `Infofeed_HostGroup` ON (
				`Infofeed_HostGroup`.`infofeedId` = `Infofeed`.`id`
			)
			WHERE
				`Infofeed_HostGroup`.`hostGroupId` = ?
		';
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute(array(
			$identifier['id']
		));

		Application::getDbInstance()->delete(
			'HostGroup',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Assigns a host to a service group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostGroupId
	 * @param         int $hostId
	 * @returns       void
	 */
	public function assignHost($hostGroupId, $hostId) {
		Application::getDbInstance()->insert(
			'HostGroup_Host',
			array(
				'hostGroupId' => $hostGroupId,
				'hostId' => $hostId
			)
		);
	}

	/**
	 * Unassigns a host from a service group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostGroupId
	 * @param         int $hostId
	 * @returns       void
	 */
	public function unassignHost($hostGroupId, $hostId) {
		Application::getDbInstance()->delete(
			'HostGroup_Host',
			sprintf('`hostGroupId` = %d AND `hostId` = %d', $hostGroupId, $hostId)
		);
	}

	/**
	 * Get the hostgroup for an Infofeed
	 *
	 * @param         int $infofeedId
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getByInfofeedId($infofeedId) {
		$query = "
			SELECT
				`HostGroup`.*
			FROM
				`HostGroup`
			JOIN `Infofeed_HostGroup` ON (
				`Infofeed_HostGroup`.`hostGroupId` = `HostGroup`.`id`
			)
			WHERE
				`Infofeed_HostGroup`.`infofeedId` = ?
			LIMIT 1
		";

		return $this->getByQuery($query, array((int)$infofeedId));

	}
}
