<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use org\adfinis\framework\db;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\db\DataTableIterator;
use ch\monitoring\view\datatable;

/**
 * Datamapper for the Host model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostMapper extends Mapper {

	/**
	 * HostMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Host');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        HostMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Host by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Host
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Host`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$host = $this->getByQuery($query, array((int)$id));
			$map->set($host);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Host by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\Host
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Host`
			WHERE
				`hostname` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single Host object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Host object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Host cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Host
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host with these parameters'),
					var_export($params, true)
				)
			);
		}

		$host = new model\Host();
		$host->fillFromRowData($data);

		return $host;
	}

	/**
	 * Load all Hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $dummy            No longer needed
	 * @param         bool $onlyNrpe         Hosts with assigned services for Nrpe export
	 * @param         bool $onlySatellites   Satellites only
	 * @param         int  $satelliteId      Hosts behind a satellite
	 * @param         int  $customerId       Customer of a satellite
	 * @param         string $permissionName
	 * @param         bool $isEnabled
	 * @param         int $slaId             The SLA id
	 * @param         int $checklistId       Checklist assigned to this host
	 * @param         int $hostGroupId       A host group the host must be assigned
	 * @param         int $dependencyId      The dependency to check for
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$dummy                 = null,
		$onlyNrpe              = false,
		$onlySatellites        = false,
		$satelliteId           = null,
		$customerId            = null,
		$permissionName        = model\Permission::HOSTS_READ,
		$isEnabled             = null,
		$slaId                 = null,
		$checklistId           = null,
		$hostGroupId           = null,
		$dependencyId          = null,
		$excludedApplicationId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$joins = array();

		if ($onlyNrpe) {
			$conditions->addStatement('Nrpe', new sql\WhereStatement(
				'isNrpe',
				'=',
				true,
				'Service'
			));

			$joins[] = '
				JOIN `HostService` ON (
					`HostService`.`hostId` = `Host`.`id`
				)
				JOIN `Service` ON (
					`Service`.`id` = `HostService`.`serviceId`
				)
			';
		}

		if ($onlySatellites) {
			$conditions->addStatement('Only satellites', new sql\WhereStatement(
				'isSatellite',
				'=',
				true,
				'Host'
			));
		}

		if ($satelliteId !== null) {
			$conditions->addStatement('Satellite', new sql\WhereStatement(
				'satelliteId',
				'=',
				$satelliteId,
				'Host'
			));
		}

		if ($slaId !== null) {
			$conditions->addStatement('SLA', new sql\WhereStatement(
				'slaId',
				'=',
				$slaId,
				'Host'
			));
		}

		if ($customerId !== null) {
			$customerIds = CustomerMapper::getInstance()->getSubCustomers($customerId);
			$customerIds[] = $customerId;

			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerIds,
				'Host'
			));
		}

		if ($isEnabled !== null) {
			$conditions->addStatement('Enabled', new sql\WhereStatement(
				'isEnabled',
				'=',
				$isEnabled,
				'Host'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		if ($checklistId !== null) {
			$joins[] = "
				JOIN MaintenanceChecklist_Host ON (
					`Host`.`id` = `MaintenanceChecklist_Host`.`hostId`
				)
			";

			$conditions->addStatement('Checklist', new sql\WhereStatement(
				'checklistId',
				'=',
				$checklistId,
				'MaintenanceChecklist_Host'
			));
		}

		if ($hostGroupId !== null) {
			$joins[] = "
				JOIN HostGroup_Host ON (
					`Host`.`id` = `HostGroup_Host`.`hostId`
				)
			";

			$conditions->addStatement('HostGroup', new sql\WhereStatement(
				'hostGroupId',
				'=',
				$hostGroupId,
				'HostGroup_Host'
			));
		}

		if ($dependencyId !== null) {
			$conditions->addStatement('Dependency', new sql\WhereStatement(
				'dependencyId',
				'=',
				$dependencyId,
				'Host'
			));
		}

		if ($excludedApplicationId !== null) {
			$joins[] = "
				LEFT JOIN symonitoring_bpe_processhost as brehost ON (
					`Host`.`id` = `brehost`.`host_id`
				)
			";
			$joins[] = "
				LEFT JOIN symonitoring_bpe_processcomponent as brecomp ON (
					`brehost`.`component_id` = `brecomp`.`id`
				)
			";

			$applicationIdCond = new sql\OrStatement();

			$applicationIdCond->addStatement(new sql\WhereStatement(
				'application_id',
				'!=',
				$excludedApplicationId,
				'brecomp'
			));

			$applicationIdCond->addStatement(new sql\WhereStatement(
				'application_id',
				'IS',
				null,
				'brecomp',
				false
			));

			$conditions->addStatement('application_id', $applicationIdCond);

		}

		$searchQuery = sprintf("
				SELECT
					`Host`.`id`
				FROM
					`Host`
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);

		$selectQuery = "SELECT * FROM `Host`";

		$hostList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Host',
			array()
		);

		$hostList->addOrderVariant('hostname.ASC',  '`Host`.`hostname` ASC');
		$hostList->addOrderVariant('hostname.DESC', '`Host`.`hostname` DESC');

		$hostList->addOrderVariant('address.ASC',  '`Host`.`address` ASC');
		$hostList->addOrderVariant('address.DESC', '`Host`.`address` DESC');

		return $hostList;
	}

	/**
	 * Load all Hosts for DataTables
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         array  $settings
	 * @param         string $permissionName
	 * @param         string $model [optional]
	 * @return        \ch\monitoring\db\DataTableIterator
	 */
	public function getAllForDataTable(
		$settings,
		$permissionName = model\Permission::HOSTS_READ,
		$model = 'ch\monitoring\model\Host'
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$joins = array();

		$fieldNameMapping = [
			datatable\MaintenanceHost::IP            => ['Host',     'IPv4'],
			datatable\MaintenanceHost::CUSTOMER      => ['Customer', 'name'],
			datatable\MaintenanceHost::HOSTNAME_SORT => ['Host',     'hostname'],
			datatable\MaintenanceHost::DESCRIPTION   => ['Host',     'description'],
			datatable\MaintenanceHost::HOST_OS       => ['HostOS',   'name'],
			//datatable\MaintenanceHost::HOST_SERVICES => ['Service',  'name'],
		];
		$filter = DataTableIterator::getFilter($model, $settings, $fieldNameMapping);

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		if ((string)$conditions || $filter) {
			$filter = sprintf('WHERE %s',
				join(' AND ', array_filter([$conditions->getSQLConditions(), $filter]))
			);
		}

		$joins[] = '
			JOIN `Customer` ON (
				`Customer`.`id` = `Host`.`customerId`
			)
			LEFT JOIN `HostOS` ON (
				`HostOS`.`id` = `Host`.`osId`
			)
			LEFT JOIN `HostService` ON (
				`HostService`.`hostId` = `Host`.`id`
			)
			LEFT JOIN `Service` ON (
				`Service`.`id` = `HostService`.`serviceId`
			)
		';

		$searchQuery = sprintf("
				SELECT DISTINCT
					`Host`.`id`
				FROM
					`Host`
				%s
				%s
			",
			implode("\n", $joins),
			$filter
		);

		$selectQuery = sprintf(
			"SELECT * FROM `Host` WHERE %s",
			db\DBIterator::IDS_PLACEHOLDER
		);

		$hostList = DataTableIterator::create(
			$searchQuery,
			$selectQuery,
			$model
		);
		$hostList->usePlaceholder(true);
		$hostList->setSettings($settings);

		$hostList->addOrderVariant('customer.ASC',  '`Customer`.`name` ASC');
		$hostList->addOrderVariant('customer.DESC', '`Customer`.`name` DESC');

		$hostList->addOrderVariant('hostname_sort.ASC',  '`Host`.`hostname` ASC');
		$hostList->addOrderVariant('hostname_sort.DESC', '`Host`.`hostname` DESC');

		$hostList->addOrderVariant('description.ASC',  '`Host`.`description` ASC');
		$hostList->addOrderVariant('description.DESC', '`Host`.`description` DESC');

		//$hostList->addOrderVariant('host_services.ASC', '`Service`.`name` ASC');
		//$hostList->addOrderVariant('host_services.DESC', '`Service`.`name` DESC');

		$hostList->addOrderVariant('host_os.ASC', '`HostOS`.`name` ASC');
		$hostList->addOrderVariant('host_os.DESC', '`HostOS`.`name` DESC');

		return $hostList;
	}

	/**
	 * Verifies whether there is a dependency cycle
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Host $host The current host to be verified
	 * @param         int $verificationId The host id which is being saved
	 * @returns       bool
	 */
	private function verifyCycle($host, $verificationId) {
		if ($host->dependencyId === $verificationId) {
			return true;
		}

		// Recursion if there are dependencies
		if ($host->hasDependencies()) {
			foreach ($host->getDependencies() as $dependency) {
				// If false, the foreach loop must continue
				if ($this->verifyCycle($dependency, $verificationId)) {
					return true;
				}
			}
		}

		// No further dependencies means that there was no circular path detected
		return false;
	}

	/**
	 * Saves Host information
	 *
	 * If the Host object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @param         \org\adfinis\framework\model\Entity $data
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $host) {
		if ($host->id !== null && $host->hasDependencies()) {
			if ($this->verifyCycle($host->getDependency(), $host->id)) {
				throw new UserException(
					sprintf(
						tr('Dependency on %s would result in a circular parent/child chain.'),
						$dependency->hostname
					)
				);
			}
		}

		$fieldDataMapping = array(
			'id'                   => $host->id,
			'customerId'           => $host->customerId,
			'slaId'                => $host->slaId,
			'dependencyId'         => $host->dependencyId,
			'satelliteId'          => $host->satelliteId,
			'osId'                 => $host->osId,
			'osVersionId'          => $host->osVersionId,
			'typeId'               => $host->typeId,
			'hostname'             => $host->hostname,
			'description'          => $host->description,
			'IPv4'                 => $host->IPv4,
			'IPv6'                 => $host->IPv6,
			'exportDate'           => util\TimeUtility::dateOrNull($host->exportDate),
			'version'              => $host->version,
			'password'             => $host->password,
			'datetimeCreated'      => util\TimeUtility::dateOrNull($host->datetimeCreated),
			'comment'              => $host->comment,
			'checkHostAlive'       => $host->checkHostAlive,
			'isSatellite'          => $host->isSatellite,
			'isEnabled'            => $host->isEnabled,
			'inheritSla'           => $host->inheritSla,
			'hasTrouble'           => $host->hasTrouble,
			'isDeleted'            => $host->isDeleted,
			'wiki'                 => $host->wiki
		);

		if ($host->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Host',
				$fieldDataMapping,
				sprintf('id = %d', $host->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Host',
				$fieldDataMapping
			);
			$host->id = Application::getDbInstance()->lastInsertId();
		}

		return $host->getIdentifier();
	}

	/**
	 * Delete a Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
    public function delete($identifier) {
        $rpcClient = Application::getRPCClientInstance(
			ConfManager::getVar('sm','RPC', 'lcAppName'),
			ConfManager::getVar('sm','RPC', 'lcQueues')
        );

        $resultId = $rpcClient->putRequest(
            'symonitoring_rpcserver.delete_host',
            array('id' => $identifier['id'])
        );

        $result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

		if (isset($result->exception)) {
			throw new UserException($result->exception);
		}
	}

	/**
	 * Finds a node in a grouped Host array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $rootId
	 * @param         array &$host
	 * @returns       array null on miss
	 */
	private function findNode($rootId, &$host) {
		if ($host['id'] == $rootId) {
			return $host;
		}

		foreach ($host['children'] as $child) {
			$node = self::findNode($rootId, $child);

			if ($node != null) {
				return $node;
			}
		}

		return null;
	}

	/**
	 * Recursive helper function to build the Hosts hierarchy
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $node A reference to the current node to investigate
	 * @param         array $host The Host to be compared
	 * @returns       bool True if the node was found
	 */
	private function insertNode(&$node, $host) {
		if ($node->id == $host->dependencyId) {
			array_push($node->children, $host);
			return true;
		}

		foreach ($node->children as &$child) {
			if (self::insertNode($child, $host)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Get an array which represents the dependency hierarchy
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $rootId (Optional)
	 * @returns       array
	 */
	public function getAllGrouped($rootId = 0) {
		$hosts = self::getAll()->toArray();

		foreach ($hosts as $host) {
			$host->children = array();
		}

		foreach ($hosts as $id => $host) {
			if ($host->dependencyId == 0) {
					continue;
			}

			foreach ($hosts as $node) {
				if (($found = self::insertNode($node, $host))) {
					unset($hosts[$id]);
					break;
				}
			}
		}

		if ($rootId > 0) {
			$rootHost = array();
			foreach ($hosts as &$host) {
				if (($rootHost = self::findNode($rootId, $host)) != null) {
					return $rootHost;
				}
			}
			return $rootHost;
		}

		return $hosts;
	}

	/**
	 * Helper function to recursively build the printable hierarchy string
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $host The Host to be added to the string
	 * @param         string Current substring
	 * @returns       string
	 */
	private function insertNodePrint($host) {
		$ret = "";

		if (count($host->children) > 0) {
			$ret .= "<ul>\n";
			foreach ($host->children as $child) {
				$ret .= "<li>\n";
				$ret .= self::insertNodePrint($child);
				$ret .= "</li>\n";
			}
			$ret .= "</ul>\n";
		}

		return $host->hostname.$ret;
	}

	/**
	 * Get the Hosts hierarchy as a printable string
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getAllGroupedPrint() {
		$hosts = self::getAllGrouped();

		$ret = "";
		foreach ($hosts as $host) {
			$ret .= "<li>".self::insertNodePrint($host)."</li>\n";
		}

		return $ret;
	}

	/**
	 * Count the hosts
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerID Optional
	 * @param         bool $isSatellite Optional
	 * @param         bool $inTrouble Optional
	 * @returns       int
	 */
	public function count(
		$customerId = null,
		$isSatellite = null,
		$inTrouble = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				array_merge(
					array($customerId),
					CustomerMapper::getInstance()->getSubCustomers(
						$customerId
					)
				),
				'Host'
			));
		}

		if ($isSatellite !== null) {
			$conditions->addStatement('Satellite', new sql\WhereStatement(
				'isSatellite',
				'=',
				$isSatellite,
				'Host'
			));
		}

		if ($inTrouble !== null) {
			$conditions->addStatement('Trouble', new sql\WhereStatement(
				'hasTrouble',
				'=',
				$inTrouble,
				'Host'
			));
		}

		$query = sprintf('
				SELECT
					count(*) AS `count`
				FROM
					`Host`
				%s
			',
			$conditions
		);

		return Application::getDBInstance()->fetchOne($query);
	}

	/**
	 * Get the hosts which are assigned to a report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $reportId
	 * @returns       oaf\DbIterator
	 */
	public function getByReportId($reportId) {
		$searchQuery = '
			SELECT
				`Host`.`id`
			FROM
				`Host`
			JOIN `Report_Host` ON (
				`Report_Host`.`hostId` = `Host`.`id`
			)
			WHERE
				`Report_Host`.`reportId` = :reportId
		';

		$selectQuery = "SELECT * from `Host`";

		$hostList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Host',
			array('reportId' => $reportId)
		);

		$hostList->addOrderVariant('hostname.ASC',  '`Host`.`hostname` ASC');
		$hostList->addOrderVariant('hostname.DESC', '`Host`.`hostname` DESC');

		return $hostList;
	}

	/**
	 * Get the host for an Infofeed
	 *
	 * @param         int $infofeedId
	 * @returns       ch\monitoring\model\Host
	 */
	public function getByInfofeedId($infofeedId) {
		$query = "
			SELECT
				`Host`.*
			FROM
				`Host`
			JOIN `Infofeed_Host` ON (
				`Infofeed_Host`.`hostId` = `Host`.`id`
			)
			WHERE
				`Infofeed_Host`.`infofeedId` = ?
			LIMIT 1
		";

		return $this->getByQuery($query, array((int)$infofeedId));

	}
}
