<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the HostOSArchitecturemodel.
 *
 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
 * @copyright     Adfinis Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class HostOSArchitectureMapper extends Mapper {

	/**
	 * HostOSArchitectureMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostOSArchitectureMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostOSArchitecture');
	}

	/**
	 * Implements a singleton
	 * 
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostOSVersionMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostOSArchitecture by id
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostOSArchitecture
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Architecture`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostOSArchitecture= $this->getByQuery($query, array((int)$id));
			$map->set($hostOSArchitecture);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostOSArchitecture by name
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string $name
	 * @param         int $osId
	 * @returns       ch\monitoring\model\HostOSArchitecture
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Architecture`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single HostOSArchitecture object identified by the given SQL query
	 *
	 * Runs the given query, initialises a HostOSArchitecture object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostOSVersion cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostOSArchitecture
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host OS architecture with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostOSArchitecture = new model\HostOSArchitecture();
		$hostOSArchitecture->fillFromRowData($data);

		return $hostOSArchitecture;
	}

	/**
	 * Load all HostOSArchitectures
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         int $osId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($osId = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($osId !== null) {
			$conditions->addStatement('HostOS', new sql\WhereStatement(
				'osId',
				'=',
				$osId,
				'Architecture'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`Architecture`.`id`
				FROM
					`Architecture`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `Architecture`";

		$hostOSArchitectureList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOSArchitecture',
			array()
		);

		$hostOSArchitectureList->addOrderVariant('name.ASC',  '`Architecture`.`name` ASC');
		$hostOSArchitectureList->addOrderVariant('name.DESC', '`Architecture`.`name` DESC');

		return $hostOSArchitectureList;
	}

	/**
	 * Saves HostOSArchitecture information
	 *
	 * If the HostOSArchitecture object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostOSArchitecture
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostOSArchitecture) {
		$fieldDataMapping = array(
			'id'                   => $hostOSArchitecture->id,
			'name'                 => $hostOSArchitecture->name,
		);

		if ($hostOSArchitecture->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Architecture',
				$fieldDataMapping,
				sprintf('id = %d', $hostOSArchitecture->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Architecture',
				$fieldDataMapping
			);
			$hostOSArchitecture->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostOSArchitecture->getIdentifier();
	}

	/**
	 * Delete a HostOSVersion
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'Architecture',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
