<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the HostOS model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostOSMapper extends Mapper {

	/**
	 * HostOSMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostOSMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostOS');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostOSMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostOS by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostOS
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostOS`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostOS = $this->getByQuery($query, array((int)$id));
			$map->set($hostOS);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single HostOS object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a HostOS object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostOS cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostOS
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host OS with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostOS = new model\HostOS();
		$hostOS->fillFromRowData($data);

		return $hostOS;
	}

	/**
	 * Load all HostOSs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "SELECT `HostOS`.`id` FROM `HostOS`";

		$selectQuery = "SELECT * FROM `HostOS`";

		$hostOSList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOS',
			array()
		);

		$hostOSList->addOrderVariant('name.ASC',  '`HostOS`.`name` ASC');
		$hostOSList->addOrderVariant('name.DESC', '`HostOS`.`name` DESC');

		return $hostOSList;
	}

	/**
	 * Load all HostOSs by given architecture ID
	 *
	 * @param         int $architectureId
	 * @return        SortingPolicyIterator
	 */
	public function getByArchitectureID($architectureId) {
		$searchQuery = "SELECT `HostOS`.`id` FROM `HostOS`";

		$selectQuery = sprintf(
			"
				SELECT *
				FROM `HostOS` AS `h`
				JOIN `Architecture_HostOS` AS `ah` ON (
					`h`.`id` = `ah`.`hostOSId`
				)
				WHERE `ah`.`architectureId` = %d
			",
			$architectureId
		);

		$hostOSList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOS',
			array()
		);

		$hostOSList->addOrderVariant('name.ASC',  '`HostOS`.`name` ASC');
		$hostOSList->addOrderVariant('name.DESC', '`HostOS`.`name` DESC');

		return $hostOSList;
	}

	/**
	 * Saves HostOS information
	 *
	 * If the HostOS object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostOS
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostOS) {
		$fieldDataMapping = array(
			'id'           => $hostOS->id,
			'name'         => $hostOS->name,
			'image'        => $hostOS->image,
		);

		if ($hostOS->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostOS',
				$fieldDataMapping,
				sprintf('id = %d', $hostOS->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostOS',
				$fieldDataMapping
			);
			$hostOS->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostOS->getIdentifier();
	}

	/**
	 * Delete a HostOS
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'HostOS',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
