<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the HostOSVersion model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostOSVersionMapper extends Mapper {

	/**
	 * HostOSVersionMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostOSVersionMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostOSVersion');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostOSVersionMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostOSVersion by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostOSVersion`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostOSVersion = $this->getByQuery($query, array((int)$id));
			$map->set($hostOSVersion);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostOSVersion by name and os
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @param         int $osId
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getByName($name, $osId) {
		$query = '
			SELECT
				*
			FROM
				`HostOSVersion`
			WHERE
				`name` = ?
				AND
				`osId` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name, (int)$osId));
	}

	/**
	 * Load a HostOSVersion by short name and os
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string $name
	 * @param         int    $osId
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getByShortName($shortName, $osId) {
		$query = '
			SELECT
				*
			FROM
				`HostOSVersion`
			WHERE
				`shortName` = ?
				AND
				`osId` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($shortName, (int)$osId));
	}


	/**
	 * Load all HostOSVersions by given architecture ID
	 *
	 * @param         int $architectureId
	 * @return        org\adfinis\framework\db\DbIterator
	 */
	public function getByArchitectureID($architectureId) {
		$searchQuery = "SELECT `HostOSVersion`.`id` FROM `HostOSVersion`";

		$selectQuery = sprintf(
			"
				SELECT *
				FROM `HostOSVersion` AS `h`
				WHERE `h`.`architectureId` = %d
			",
			$architectureId
		);

		$hostOSVersionList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOSVersion',
			array()
		);

		$hostOSVersionList->addOrderVariant('name.ASC',  '`HostOSVersion`.`name` ASC');
		$hostOSVersionList->addOrderVariant('name.DESC', '`HostOSVersion`.`name` DESC');

		return $hostOSVersionList;
	}

	/**
	 * Returns a single HostOSVersion object identified by the given SQL query
	 *
	 * Runs the given query, initialises a HostOSVersion object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostOSVersion cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);
		
		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host OS version with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostOSVersion = new model\HostOSVersion();
		$hostOSVersion->fillFromRowData($data);

		return $hostOSVersion;
	}

	/**
	 * Load all HostOSVersions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $osId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$osId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($osId !== null) {
			$conditions->addStatement('HostOS', new sql\WhereStatement(
				'osId',
				'=',
				$osId,
				'HostOSVersion'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`HostOSVersion`.`id`
				FROM
					`HostOSVersion`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `HostOSVersion`";

		$hostOSVersionList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOSVersion',
			array()
		);

		$hostOSVersionList->addOrderVariant('name.ASC',  '`HostOSVersion`.`name` ASC');
		$hostOSVersionList->addOrderVariant('name.DESC', '`HostOSVersion`.`name` DESC');

		return $hostOSVersionList;
	}

	/**
	 * Load all HostOSVersions for a hostOS with no guideline assigned to the commandId.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         int $osId
	 * @param         int $commandId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllExcludedByCommandID($commandId, $osId) {
		$searchQuery = sprintf("
			SELECT
				`id`,
				`osId`,
				`architectureId`,
				`name`,
				`shortName`
			FROM
				`HostOSVersion`
			WHERE
				`HostOSVersion`.`osId` = %d
			AND
				`HostOSVersion`.`id` NOT IN (
					SELECT
						`versionId`
					FROM
						`Guideline`
					WHERE
						`commandId` = %d
						AND
						`osId` = %d
			)",
			$osId,
			$commandId,
			$osId
		);
		$selectQuery = "SELECT * FROM `HostOSVersion`";
		
		$hostOSVersionList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostOSVersion',
			array()
		);

		$hostOSVersionList->addOrderVariant('name.ASC',  '`HostOSVersion`.`name` ASC');
		$hostOSVersionList->addOrderVariant('name.DESC', '`HostOSVersion`.`name` DESC');

		return $hostOSVersionList;
	}

	/**
	 * Saves HostOSVersion information
	 *
	 * If the HostOSVersion object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostOSVersion
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostOSVersion) {
		$fieldDataMapping = array(
			'id'                     => $hostOSVersion->id,
			'osId'                   => $hostOSVersion->osId,
			'name'                   => $hostOSVersion->name,
			'shortName'              => $hostOSVersion->shortName,
			'architectureId'         => $hostOSVersion->architectureId
		);

		if ($hostOSVersion->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostOSVersion',
				$fieldDataMapping,
				sprintf('id = %d', $hostOSVersion->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostOSVersion',
				$fieldDataMapping
			);
			$hostOSVersion->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostOSVersion->getIdentifier();
	}

	/**
	 * Delete a HostOSVersion
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'HostOSVersion',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
