<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the HostServiceArgument model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostServiceArgumentMapper extends Mapper {

	/**
	 * HostServiceArgumentMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostServiceArgumentMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostServiceArgument');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostServiceArgumentMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostServiceArgument by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostServiceArgument
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostServiceArgument`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostServiceArgument = $this->getByQuery($query, array((int)$id));
			$map->set($hostServiceArgument);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostServiceArgument by foreign keys
	 *
	 * Needed when you got a HostServiceInstance with an Argument and want to know whether an optional value was set.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $instanceId
	 * @param         int $serviceArgumentId
	 * @returns       ch\monitoring\model\HostServiceArgument
	 */
	public function getByForeignKeys($instanceId, $serviceArgumentId) {
		$query = '
			SELECT
				*
			FROM
				`HostServiceArgument`
			WHERE
				`instanceId` = ?
				AND
				`serviceArgumentId` = ?
			LIMIT
				1
		';

		return $this->getByQuery(
			$query,
			array(
				$instanceId,
				$serviceArgumentId
			)
		)->allowRead();
	}

	/**
	 * Returns a single HostServiceArgument object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a HostServiceArgument object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostServiceArgument cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostServiceArgument
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host service argument with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostServiceArgument = new model\HostServiceArgument();
		$hostServiceArgument->fillFromRowData($data);

		return $hostServiceArgument;
	}

	/**
	 * Load all HostServiceArguments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $instanceId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$instanceId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($instanceId !== null) {
			$conditions->addStatement('HostServiceInstance', new sql\WhereStatement(
				'instanceId',
				'=',
				$instanceId,
				'HostServiceArgument'
			));
		}

		$customers = Application::getUser()->getCustomersWithPermission(
			model\Permission::HOSTS_READ
		);

		$conditions->addStatement('Permission', new sql\WhereStatement(
			'customerId',
			$customers === null ? 'IS' : 'IN',
			$customers,
			'Host',
			false
		));

		$searchQuery = sprintf("
				SELECT
					`HostServiceArgument`.`id`
				FROM
					`HostServiceArgument`
				JOIN `HostServiceInstance` ON (
					`HostServiceInstance`.`id` = `HostServiceArgument`.`instanceId`
				)
				JOIN `HostService` ON (
					`HostService`.`id` = `HostServiceInstance`.`hostServiceId`
				)
				JOIN `Host` ON (
					`Host`.`id` = `HostService`.`hostId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `HostServiceArgument`";

		$hostServiceArgumentList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostServiceArgument',
			array()
		);

		$hostServiceArgumentList->addOrderVariant('id.ASC',  '`HostServiceArgument`.`id` ASC');

		return $hostServiceArgumentList;
	}

	/**
	 * Saves HostServiceArgument information
	 *
	 * If the HostServiceArgument object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostServiceArgument
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostServiceArgument) {
		$fieldDataMapping = array(
			'instanceId'     => $hostServiceArgument->instanceId,
			'serviceArgumentId' => $hostServiceArgument->serviceArgumentId,
			'assignedValue'     => $hostServiceArgument->assignedValue
		);

		if ($hostServiceArgument->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostServiceArgument',
				$fieldDataMapping,
				sprintf('id = %d', $hostServiceArgument->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostServiceArgument',
				$fieldDataMapping
			);
			$hostServiceArgument->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostServiceArgument->getIdentifier();
	}

	/**
	 * Delete a HostServiceArgument
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'HostServiceArgument',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Delete all HostServiceArguments of a HostServiceInstance
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $instanceId
	 * @returns       void
	 */
	public function deleteByInstanceId($instanceId) {
		Application::getDbInstance()->beginTransaction();

		$hostServiceArguments = $this->getAll($instanceId);
		foreach ($hostServiceArguments as $hostServiceArgument) {
			$this->delete($hostServiceArgument->getIdentifier());
		}

		Application::getDbInstance()->commit();
	}
}
