<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the HostServiceInstance model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostServiceInstanceMapper extends Mapper {

	/**
	 * HostServiceInstanceMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostServiceInstanceMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostServiceInstance');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostServiceInstanceMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostServiceInstance by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostServiceInstance`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostServiceInstance = $this->getByQuery($query, array((int)$id));
			$map->set($hostServiceInstance);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostServiceInstance by name, host and service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @param         int $hostId
	 * @param         int $serviceId
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getByName($name, $hostId, $serviceId) {
		if ($name === null) {
			$query = '
				SELECT
					`HostServiceInstance`.*
				FROM
					`HostServiceInstance`
				JOIN `HostService` ON (
					`HostService`.`id` = `HostServiceInstance`.`hostServiceId`
				)
				WHERE
					`HostServiceInstance`.`name` IS NULL
					AND
					`HostService`.`hostId` = ?
					AND
					`HostService`.`serviceId` = ?
				LIMIT
					1
			';

			return $this->getByQuery($query, array((int)$hostId, (int)$serviceId));
		}
		else {
			$query = '
				SELECT
					`HostServiceInstance`.*
				FROM
					`HostServiceInstance`
				JOIN `HostService` ON (
					`HostService`.`id` = `HostServiceInstance`.`hostServiceId`
				)
				WHERE
					`HostServiceInstance`.`name` = ?
					AND
					`HostService`.`hostId` = ?
					AND
					`HostService`.`serviceId` = ?
				LIMIT
					1
			';

			return $this->getByQuery($query, array($name, (int)$hostId, (int)$serviceId));
		}
	}

	/**
	 * Returns a single HostServiceInstance object identified by the given SQL query
	 *
	 * Runs the given query, initialises a HostServiceInstance object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostServiceInstance cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	protected function getByQuery($query, $params) {
		Application::getDBInstance()->beginTransaction();
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);
		Application::getDBInstance()->commit();

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host service instance with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostServiceInstance = new model\HostServiceInstance();
		$hostServiceInstance->fillFromRowData($data);

		return $hostServiceInstance;
	}

	/**
	 * Load all HostServiceInstances
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId Optional
	 * @param         bool $hasTrouble Optional
	 * @param         bool $isNrpe Optional
	 * @param         int $serviceId Optional
	 * @param         bool $forAvailability
	 * @param         bool $isEnabled
	 * @param         string $permissionName
	 * @param         int $slaId The SLA id
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$hostServiceId   = null,
		$hostId          = null,
		$hasTrouble      = null,
		$isNrpe          = null,
		$forAvailability = null,
		$isEnabled       = null,
		$permissionName  = model\Permission::HOSTS_READ,
		$slaId           = null,
		$excludedApplicationId   = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$joins = array();

		if ($hostServiceId !== null) {
			$conditions->addStatement('HostService', new sql\WhereStatement(
				'hostServiceId',
				'=',
				$hostServiceId,
				'HostServiceInstance'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'id',
				'=',
				$hostId,
				'Host'
			));
		}

		if ($hasTrouble !== null) {
			$conditions->addStatement('Trouble', new sql\WhereStatement(
				'hasTrouble',
				'=',
				$hasTrouble,
				'HostServiceInstance'
			));
		}

		if ($isNrpe !== null) {
			$conditions->addStatement('Nrpe', new sql\WhereStatement(
				'isNrpe',
				'=',
				$isNrpe,
				'Service'
			));
		}

		if ($forAvailability !== null) {
			$conditions->addStatement('Availability', new sql\WhereStatement(
				'forAvailability',
				'=',
				$forAvailability,
				'HostServiceInstance'
			));
		}

		if ($isEnabled !== null) {
			$conditions->addStatement('Enabled', new sql\WhereStatement(
				'isEnabled',
				'=',
				$isEnabled,
				'HostServiceInstance'
			));
		}

		if ($slaId !== null) {
			$conditions->addStatement('SLA', new sql\WhereStatement(
				'slaId',
				'=',
				$slaId,
				'HostServiceInstance'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		if ($excludedApplicationId !== null) {
			$joins[] = "
				LEFT JOIN symonitoring_bpe_processservice as breserv ON (
					`HostServiceInstance`.`id` = `breserv`.`service_id`
				)
			";
			$joins[] = "
				LEFT JOIN symonitoring_bpe_processcomponent as brecomp ON (
					`breserv`.`component_id` = `brecomp`.`id`
				)
			";

			$applicationIdCond = new sql\OrStatement();

			$applicationIdCond->addStatement(new sql\WhereStatement(
				'application_id',
				'!=',
				$excludedApplicationId,
				'brecomp'
			));

			$applicationIdCond->addStatement(new sql\WhereStatement(
				'application_id',
				'IS',
				null,
				'brecomp',
				false
			));

			$conditions->addStatement('application_id', $applicationIdCond);

		}

		$searchQuery = sprintf("
				SELECT
					`HostServiceInstance`.`id`
				FROM
					`HostServiceInstance`
				JOIN `HostService` ON (
					`HostService`.`id` = `HostServiceInstance`.`hostServiceId`
				)
				JOIN `Service` ON (
					`HostService`.`serviceId` = `Service`.`id`
				)
				JOIN `Host` ON (
					`Host`.`id` = `HostService`.`hostId`
				)
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);

		$selectQuery = "SELECT * FROM `HostServiceInstance`";

		$hostServiceInstanceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostServiceInstance',
			array()
		);

		$hostServiceInstanceList->addOrderVariant('name.ASC',  '`Service`.`name` ASC');

		return $hostServiceInstanceList;
	}

	/**
	 * Saves HostServiceInstance information
	 *
	 * If the HostServiceInstance object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostService
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostServiceInstance) {
		$fieldDataMapping = array(
			'id'              => $hostServiceInstance->id,
			'hostServiceId'   => $hostServiceInstance->hostServiceId,
			'slaId'           => $hostServiceInstance->slaId ?: null,
			'name'            => $hostServiceInstance->name,
			'isPassive'       => $hostServiceInstance->isPassive,
			'isEnabled'       => $hostServiceInstance->isEnabled,
			'hasTrouble'      => $hostServiceInstance->hasTrouble,
			'forAvailability' => $hostServiceInstance->forAvailability
		);

		if ($hostServiceInstance->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostServiceInstance',
				$fieldDataMapping,
				sprintf('id = %d', $hostServiceInstance->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostServiceInstance',
				$fieldDataMapping
			);
			$hostServiceInstance->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostServiceInstance->getIdentifier();
	}

	/**
	 * Delete a HostServiceInstance
	 *
	 * @author        Adrian Wittwer <adrian.wittwer@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
    public function delete($identifier) {
        $rpcClient = Application::getRPCClientInstance(
            ConfManager::getVar('sm','RPC', 'lcAppName'),
            ConfManager::getVar('sm','RPC', 'lcQueues')
        );

        $resultId = $rpcClient->putRequest(
            'symonitoring_rpcserver.delete_hostserviceinstance',
            array('id' => $identifier['id'])
        );

        $result = $rpcClient->getResult($resultId, 10); // 10 is the timeout

        if (isset($result->exception)) {
            throw new UserException($result->exception);
        }
	}

	/**
	 * Count the service assignments
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId Optional
	 * @param         int $hostId Optional
	 * @param         bool $hasTrouble Optional
	 * @returns       int
	 */
	public function count(
		$customerId = null,
		$hostId = null,
		$hasTrouble = null
	) {
		$join = array('
			JOIN `HostService` ON (
				`HostService`.`id` = `HostServiceInstance`.`hostServiceId`
			)
		');
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$join[] = '
				JOIN `Host` ON (
					`Host`.`id` = `HostService`.`hostId`
				)
			';

			$subCustomerArray = CustomerMapper::getInstance()->getSubCustomers(
				$customerId
			);

			$customerArray = array_merge(
					array($customerId),
					$subCustomerArray
			);

			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerArray,
				'Host'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'HostService'
			));
		}

		if ($hasTrouble !== null) {
			$conditions->addStatement('Trouble', new sql\WhereStatement(
				'hasTrouble',
				'=',
				$hasTrouble,
				'HostServiceInstance'
			));
		}

		$query = sprintf('
				SELECT
					count(*) AS `count`
				FROM
					`HostServiceInstance`
				%s
				%s
			',
			implode("\n", $join),
			$conditions
		);

		return Application::getDBInstance()->fetchOne($query);
	}

	/**
	 * Get arguments of an assigned service as a string
	 *
	 * This is required to export Nagios/Icinga configuration files. A different
	 * syntax is needed depending on whether the service is of type Nrpe.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $instanceId
	 * @returns       string
	 */
	public function getArgumentsAsString($instanceId) {
		$hostServiceInstance = $this->getByID($instanceId);
		$hostServiceInstance->allowRead();
		$service = $hostServiceInstance->getHostService()->getService();
		$serviceArguments = $service->getArguments();
		$argumentsArray = array();

		foreach ($serviceArguments as $serviceArgument) {
			$assignedValue = $serviceArgument->getAssignedValue($hostServiceInstance->id);
			$value = $assignedValue === null ? $serviceArgument->value : $assignedValue;

			/**
			 * The syntax differs, depending on the service type.
			 *  - Nrpe: /path/to/binary -option1 value1 -option2 value2
			 *  - Otherwise: value1!value2!value3
			 */
			if ($service->isNrpe) {
				$option = $serviceArgument->option;
				$separator = $serviceArgument->option !== null && $value !== null ? ' ' : '';

				// If there is neither an option nor a value the argument would be empty.
				if ($option === null && $value === null) {
					continue;
				}
			}
			else {
				$option = '';
				$separator = '';
			}

			if (is_numeric($value) || strval($value) !== '') {
				$value = sprintf(
					"'%s'",
					$value
				);
			}

			$argumentsArray[] = sprintf(
				"%s%s%s",
				$option,
				$separator,
				$value
			);
		}

		// Remember: Nrpe means command line arguments (separated by space), normal service arguments are separated by !
		$separator = $service->isNrpe ? ' ' : '!';
		return implode($separator, $argumentsArray);
	}
}
