<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the HostService model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostServiceMapper extends Mapper {

	/**
	 * HostServiceMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostServiceMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostService');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostServiceMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostService by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostService
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostService`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostService = $this->getByQuery($query, array((int)$id));
			$map->set($hostService);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a HostService by host and service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $serviceId
	 * @returns       ch\monitoring\model\HostService
	 */
	public function getByHostAndService($hostId, $serviceId) {
		$query = '
			SELECT
				*
			FROM
				`HostService`
			WHERE
				`hostId` = ?
				AND
				`serviceId` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array((int)$hostId, (int)$serviceId));
	}

	/**
	 * Returns a single HostService object identified by the given SQL query
	 *
	 * Runs the given query, initialises a HostService object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostService cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostService
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host service with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostService = new model\HostService();
		$hostService->fillFromRowData($data);

		return $hostService;
	}

	/**
	 * Verify whether service is already assigned to a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $serviceId
	 * @returns       bool
	 */
	public function existsByHostAndService($hostId, $serviceId) {
		$query = '
			SELECT
				*
			FROM
				`HostService`
			WHERE
				`hostId` = ?
				AND
				`serviceId` = ?
			LIMIT
				1
		';

		try {
			$this->getByQuery($query, array($hostId, $serviceId))->allowRead();
			return true;
		}
		catch (\OutOfBoundsException $e) {
			return false;
		}
	}

	/**
	 * Load all HostServices
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId Optional
	 * @param         bool $hasTrouble Optional
	 * @param         bool $isNrpe Optional
	 * @param         int $serviceId Optional
	 * @param         bool $forAvailability
	 * @param         bool $isEnabled
	 * @param         string $permissionName
	 * @param         int $slaId The SLA id
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$hostId          = null,
		$serviceId       = null,
		$permissionName  = model\Permission::HOSTS_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'HostService'
			));
		}

		if ($serviceId !== null) {
			$conditions->addStatement('Service', new sql\WhereStatement(
				'serviceId',
				'=',
				$serviceId,
				'HostService'
			));
		}

		if ($permissionName !== null) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`HostService`.`id`
				FROM
					`HostService`
				JOIN `Service` ON (
					`HostService`.`serviceId` = `Service`.`id`
				)
				JOIN `Host` ON (
					`Host`.`id` = `HostService`.`hostId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `HostService`";

		$hostServiceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostService',
			array()
		);

		$hostServiceList->addOrderVariant('name.ASC',  '`Service`.`name` ASC');

		return $hostServiceList;
	}

	/**
	 * Saves HostService information
	 *
	 * If the HostService object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostService
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostService) {
		$fieldDataMapping = array(
			'id'              => $hostService->id,
			'hostId'          => $hostService->hostId,
			'serviceId'       => $hostService->serviceId
		);

		if ($hostService->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostService',
				$fieldDataMapping,
				sprintf('id = %d', $hostService->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostService',
				$fieldDataMapping
			);
			$hostService->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostService->getIdentifier();
	}

	/**
	 * Delete a HostService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();
		Application::getDbInstance()->delete(
			'HostService',
			sprintf('`id` = %d', $identifier['id'])
		);
		Application::getDbInstance()->commit();
	}

	/**
	 * Delete all HostServices by host id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $hostId
	 * @returns       void
	 */
	public function deleteByHostId($hostId) {
		Application::getDbInstance()->beginTransaction();

		$hosts = $this->getAll($hostId);
		foreach ($hosts as $host) {
			$this->delete($host->getIdentifier());
		}

		Application::getDbInstance()->commit();
	}
}
