<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the HostType model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostTypeMapper extends Mapper {

	/**
	 * HostTypeMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\HostTypeMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostType');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\HostTypeMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a HostType by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\HostType
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`HostType`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$hostType = $this->getByQuery($query, array((int)$id));
			$map->set($hostType);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single HostType object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a HostType object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the HostType cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\HostType
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find host type with these parameters'),
					var_export($params, true)
				)
			);
		}

		$hostType = new model\HostType();
		$hostType->fillFromRowData($data);

		return $hostType;
	}

	/**
	 * Load all HostTypes
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "SELECT `HostType`.`id` FROM `HostType`";

		$selectQuery = "SELECT * FROM `HostType`";

		$hostTypeList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\HostType',
			array()
		);

		$hostTypeList->addOrderVariant('name.ASC',  '`HostType`.`name` ASC');
		$hostTypeList->addOrderVariant('name.DESC', '`HostType`.`name` DESC');

		return $hostTypeList;
	}

	/**
	 * Saves HostType information
	 *
	 * If the HostType object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostType
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostType) {
		$fieldDataMapping = array(
			'id'    => $hostType->id,
			'name'  => $hostType->name,
			'short' => $hostType->short
		);

		if ($hostType->hasIdentifier()) {
			Application::getDbInstance()->update(
				'HostType',
				$fieldDataMapping,
				sprintf('id = %d', $hostType->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'HostType',
				$fieldDataMapping
			);
			$hostType->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostType->getIdentifier();
	}

	/**
	 * Delete a HostType
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'HostType',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
