<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ImportCommand model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportCommandMapper extends Mapper {

	/**
	 * ImportCommandMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ImportCommandMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ImportCommand');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportCommandMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ImportCommand by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportCommand
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`ImportCommand`.*,
					`Command`.`id` AS `conflictCommandId`
				FROM
					`ImportCommand`
				LEFT JOIN `Command` ON (
					`Command`.`name` = `ImportCommand`.`command_name`
				)
				WHERE
					`ImportCommand`.`id` = ?
				LIMIT
					1
			';

			$importCommand = $this->getByQuery($query, array((int)$id));
			$map->set($importCommand);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single ImportCommand object identified by the given SQL query
	 *
	 * Runs the given query, initialises a ImpoortCommand object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ImportCommand cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ImportCommand
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find import command with these parameters'),
					var_export($params, true)
				)
			);
		}

		$importCommand = new model\ImportCommand();
		$importCommand->fillFromRowData($data);

		return $importCommand;
	}

	/**
	 * Load all ImportCommands
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $fileId The file id the desired import commands must be assigned to (optional)
	 * @param         bool $conflict Whether to return commands with name conflict only
	 * @param         bool $register Whether to return objects to be registered
	 * @param         int $includedBy The command id which the command must be included by
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$fileId     = null,
		$conflict   = null,
		$register   = null,
		$includedBy = null
		
	) {
		$join = array();
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($fileId !== null) {
			$join[] = sprintf("
					JOIN `ImportFile` ON (
						`ImportFile`.`id` = `ImportCommand`.`fileId`
						AND
						`ImportFile`.`id` = %d
					)
				",
				$fileId
			);
		}

		if ($conflict !== null) {
			$join[] = "
				LEFT JOIN `Command` ON (
					`Command`.`name` = `ImportCommand`.`command_name`
				)
			";

			$conditions->addStatement('Conflict', new sql\WhereStatement(
				'id',
				$conflict ? 'IS NOT' : 'IS',
				null,
				'Command',
				false
			));

			if ($conflict) {
				$conditions->addStatement('Conflict2', new sql\WhereStatement(
					'commandId',
					'IS',
					null,
					'ImportCommand',
					false
				));
			}
		}

		if ($register !== null) {
			$conditions->addStatement('Register', new sql\WhereStatement(
				'register',
				'=',
				$register,
				'ImportCommand'
			));
		}

		if ($includedBy !== null) {
			$join[] = "
				JOIN `ImportCommandInclude` ON (
					`ImportCommandInclude`.`name` = `ImportCommand`.`name`
				)
			";

			$conditions->addStatement('IncludedBy', new sql\WhereStatement(
				'importCommandId',
				'=',
				$includedBy,
				'ImportCommandInclude'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ImportCommand`.`id`
				FROM
					`ImportCommand`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "
			SELECT
				`ImportCommand`.*,
				`Command`.`id` AS `conflictCommandId`
			FROM
				`ImportCommand`
			LEFT JOIN `Command` ON (
				`Command`.`name` = `ImportCommand`.`command_name`
			)
		";

		$commandList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ImportCommand',
			array()
		);

		$commandList->addOrderVariant('name.ASC',  '`ImportCommand`.`command_name` ASC');
		$commandList->addOrderVariant('name.DESC', '`ImportCommand`.`command_name` DESC');

		if ($includedBy !== null) {
			$commandList->addOrderVariant('includeId.ASC',  '`ImportCommandInclude`.`id` ASC');
			$commandList->addOrderVariant('includeId.DESC', '`ImportCommandInclude`.`id` DESC');
		}

		return $commandList;
	}

	/**
	 * Saves ImportCommand information
	 *
	 * If the ImportCommand object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $command
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $command) {
		$fieldDataMapping = array(
			'id'           => $command->id,
			'fileId'       => $command->fileId,
			'commandId'    => $command->commandId,
			'command_name' => $command->command_name,
			'command_line' => $command->command_line,
			'name'         => $command->name,
			'register'     => $command->register
		);

		if ($command->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ImportCommand',
				$fieldDataMapping,
				sprintf('id = %d', $command->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ImportCommand',
				$fieldDataMapping
			);
			$command->id = Application::getDbInstance()->lastInsertId();
		}

		return $command->getIdentifier();
	}

	/**
	 * Delete an ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		$command = $this->getById($identifier['id']);

		Application::getDbInstance()->delete(
			'ImportCommand',
			sprintf('`id` = %d', $identifier['id'])
		);

		ImportServiceMapper::getInstance()->deleteByCommand(
			$command->command_name
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Delete an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $includeID The ID of the include
	 */
	public function deleteInclude($includeId) {
		Application::getDBInstance()->delete(
			'ImportCommandInclude',
			sprintf('`id` = %d', $includeId)
		);
	}

	/**
	 * Add an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $importCommandId The ImportCommand to associate
	 * @param         string $name The command name to be included
	 */
	public function addInclude($importCommandId, $name) {
		Application::getDBInstance()->insert(
			'ImportCommandInclude',
			array(
				'importCommandId' => $importCommandId,
				'name'            => $name
			)
		);
	}

	/**
	 * Rename an ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $commandId The command to be renamed
	 * @param         string $newName The new command name
	 * @returns       void
	 */
	public function rename($commandId, $newName) {
		$command = $this->getByID($commandId)->allowRead();

		if ($command->command_name === $newName) {
			return;
		}

		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->update(
			'ImportCommand',
			array(
				'command_name' => $newName
			),
			sprintf('`id` = %d', $command->id)
		);

		Application::getDBInstance()->update(
			'ImportService',
			array(
				'check_command' => $newName
			),
			sprintf('`check_command` = %s', Application::getDBInstance()->quote(
				$command->command_name
			))
		);

		$replace = sprintf(
			'REPLACE(`check_command`, %s,%s)',
			sprintf('%s!', $command->command_name),
			sprintf('%s!', $newName)
		);

		Application::getDBInstance()->update(
			'ImportService',
			array(
				'check_command' => $replace
			),
			sprintf('`check_command` = %s', Application::getDBInstance()->quote(
				$command->command_name
			))
		);

		Application::getDBInstance()->commit();
	}
}
