<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ImportFile model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportFileMapper extends Mapper {

	/**
	 * ImportFileMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ImportFileMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ImportFile');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportFileMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ImportFile by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportFile
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`ImportFile`.*
				FROM
					`ImportFile`
				WHERE
					`ImportFile`.`id` = ?
				LIMIT
					1
			';

			$importFile = $this->getByQuery($query, array((int)$id));
			$map->set($importFile);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single ImportFile object identified by the given SQL query
	 *
	 * Runs the given query, initialises a ImpoortFile object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ImportFile cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ImportFile
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find import file with these parameters'),
					var_export($params, true)
				)
			);
		}

		$importFile = new model\ImportFile();
		$importFile->fillFromRowData($data);

		return $importFile;
	}

	/**
	 * Load all ImportFiles
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$searchQuery = sprintf("
				SELECT
					`ImportFile`.`id`
				FROM
					`ImportFile`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `ImportFile`";

		$fileList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ImportFile',
			array()
		);

		$fileList->addOrderVariant('name.ASC',  '`ImportFile`.`name` ASC');
		$fileList->addOrderVariant('name.DESC', '`ImportFile`.`name` DESC');

		return $fileList;
	}

	/**
	 * Saves ImportFile information
	 *
	 * If the ImportFile object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $file
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $file) {
		$fieldDataMapping = array(
			'id'         => $file->id,
			'name'       => $file->name,
			'hosts'      => $file->hosts,
			'services'   => $file->services,
			'commands'   => $file->commands,
			'hostgroups' => $file->hostgroups
		);

		if ($file->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ImportFile',
				$fieldDataMapping,
				sprintf('id = %d', $file->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ImportFile',
				$fieldDataMapping
			);
			$file->id = Application::getDbInstance()->lastInsertId();
		}

		return $file->getIdentifier();
	}

	/**
	 * Delete an ImportFile
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'ImportFile',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Cancel an import
	 *
	 * I.e. delete all ImportFile entries
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function cancel() {
		Application::getDbInstance()->delete(
			'ImportFile'
		);
	}
}
