<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ImportHost model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportHostMapper extends Mapper {

	/**
	 * ImportHostMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ImportHostMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ImportHost');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportHostMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ImportHost by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportHost
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`ImportHost`.*,
					`Host`.`id` AS `conflictHostId`
				FROM
					`ImportHost`
				LEFT JOIN `Host` ON (
					`Host`.`hostname` = `ImportHost`.`host_name`
				)
				WHERE
					`ImportHost`.`id` = ?
				LIMIT
					1
			';

			$importHost = $this->getByQuery($query, array((int)$id));
			$map->set($importHost);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a ImportHost by template name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportHost
	 */
	public function getByName($name) {
		$query = '
			SELECT
				`ImportHost`.*,
				`Host`.`id` AS `conflictHostId`
			FROM
				`ImportHost`
			LEFT JOIN `Host` ON (
				`Host`.`hostname` = `ImportHost`.`host_name`
			)
			WHERE
				`ImportHost`.`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single ImportHost object identified by the given SQL query
	 *
	 * Runs the given query, initialises a ImpoortHost object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ImportHost cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ImportHost
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find import host with these parameters'),
					var_export($params, true)
				)
			);
		}

		$importHost = new model\ImportHost();
		$importHost->fillFromRowData($data);

		return $importHost;
	}

	/**
	 * Load all ImportHosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $fileId The file id the desired import hosts must be assigned to (optional)
	 * @param         string $serviceName Limit to Hosts assigned to given service name
	 * @param         bool $conflict Whether to return Hosts with name conflict only
	 * @param         bool $register Whether to return register
	 * @param         int $includedBy The host id which the hosts must be included by
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$fileId      = null,
		$serviceName = null,
		$conflict    = null,
		$register    = null,
		$includedBy  = null
	) {
		$join = array();
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($fileId !== null) {
			$join[] = sprintf("
					JOIN `ImportFile` ON (
						`ImportFile`.`id` = `ImportHost`.`fileId`
						AND
						`ImportFile`.`id` = %d
					)
				",
				$fileId
			);
		}

		if ($serviceName !== null) {
			$join[] = sprintf("
					JOIN `ImportService` ON (
						`ImportService`.`host_name` = `ImportHost`.`host_name`
						AND
						`ImportService`.`service_description` = %s
					)
				",
				Application::getDbInstance()->quote($serviceName)
			);
		}

		if ($conflict !== null) {
			$join[] = "
				LEFT JOIN `Host` ON (
					`Host`.`hostname` = `ImportHost`.`host_name`
				)
			";

			$conditions->addStatement('Conflict', new sql\WhereStatement(
				'id',
				$conflict ? 'IS NOT' : 'IS',
				null,
				'Host',
				false
			));

			if ($conflict) {
				$conditions->addStatement('Conflict2', new sql\WhereStatement(
					'hostId',
					'IS',
					null,
					'ImportHost',
					false
				));
			}
		}

		if ($register !== null) {
			$conditions->addStatement('Register', new sql\WhereStatement(
				'register',
				'=',
				$register,
				'ImportHost'
			));
		}

		if ($includedBy !== null) {
			$join[] = "
				JOIN `ImportHostInclude` ON (
					`ImportHostInclude`.`name` = `ImportHost`.`name`
				)
			";

			$conditions->addStatement('IncludedBy', new sql\WhereStatement(
				'importHostId',
				'=',
				$includedBy,
				'ImportHostInclude'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ImportHost`.`id`
				FROM
					`ImportHost`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "
			SELECT
				`ImportHost`.*,
				`Host`.`id` AS `conflictHostId`
			FROM
				`ImportHost`
			LEFT JOIN `Host` ON (
				`Host`.`hostname` = `ImportHost`.`host_name`
			)
		";

		$hostList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ImportHost',
			array()
		);

		$hostList->addOrderVariant('name.ASC',  '`ImportHost`.`host_name` ASC');
		$hostList->addOrderVariant('name.DESC', '`ImportHost`.`host_name` DESC');

		if ($includedBy !== null) {
			$hostList->addOrderVariant('includeId.ASC',  '`ImportHostInclude`.`id` ASC');
			$hostList->addOrderVariant('includeId.DESC', '`ImportHostInclude`.`id` DESC');
		}

		return $hostList;
	}

	/**
	 * Saves ImportHost information
	 *
	 * If the ImportHost object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $host
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $host) {
		$fieldDataMapping = array(
			'id'         => $host->id,
			'fileId'     => $host->fileId,
			'hostId'     => $host->hostId,
			'osId'       => $host->osId,
			'customerId' => $host->customerId,
			'host_name'  => $host->host_name,
			'alias'      => $host->alias,
			'address'    => $host->address,
			'parents'    => $host->parents,
			'hostgroups' => $host->hostgroups,
			'name'       => $host->name,
			'register'   => $host->register
		);

		if ($host->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ImportHost',
				$fieldDataMapping,
				sprintf('id = %d', $host->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ImportHost',
				$fieldDataMapping
			);
			$host->id = Application::getDbInstance()->lastInsertId();
		}

		return $host->getIdentifier();
	}

	/**
	 * Delete an ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'ImportHost',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Delete an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $includeID The ID of the include
	 */
	public function deleteInclude($includeId) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'ImportHostInclude',
			sprintf('`id` = %d', $includeId)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Add an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $importHostId The ImportHost to associate
	 * @param         string $name The host name to be included
	 */
	public function addInclude($importHostId, $name) {
		Application::getDBInstance()->insert(
			'ImportHostInclude',
			array(
				'importHostId' => $importHostId,
				'name'         => $name
			)
		);
	}

	/**
	 * Rename an ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId The host to be renamed
	 * @param         string $newName The new host name
	 * @returns       void
	 */
	public function rename($hostId, $newName) {
		$host = $this->getByID($hostId)->allowRead();

		if ($host->host_name === $newName) {
			return;
		}

		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->update(
			'ImportHost',
			array(
				'host_name' => $newName
			),
			sprintf('`id` = %d', $host->id)
		);

		Application::getDBInstance()->update(
			'ImportService',
			array(
				'host_name' => $newName
			),
			sprintf('`host_name` = %s', Application::getDBInstance()->quote(
				$host->host_name)
			)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Set the same customer for all hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId The customer id to be set
	 * @returns       void
	 */
	public function setDefaultCustomer($customerId) {
		Application::getDBInstance()->update(
			'ImportHost',
			array(
				'customerId' => $customerId
			)
		);
	}

	/**
	 * Set the same operating system for all hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId The customer id to be set
	 * @returns       void
	 */
	public function setDefaultOS($osId) {
		Application::getDBInstance()->update(
			'ImportHost',
			array(
				'osId' => $osId
			)
		);
	}
}
