<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ImportHostgroup model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportHostgroupMapper extends Mapper {

	/**
	 * ImportHostgroupMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ImportHostgroupMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ImportHostgroup');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportHostgroupMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ImportHostgroup by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportHostgroup
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`ImportHostgroup`.*,
					`HostGroup`.`id` AS `conflictHostgroupId`
				FROM
					`ImportHostgroup`
				LEFT JOIN `HostGroup` ON (
					`HostGroup`.`name` = `ImportHostgroup`.`hostgroup_name`
				)
				WHERE
					`ImportHostgroup`.`id` = ?
				LIMIT
					1
			';

			$importHostgroup = $this->getByQuery($query, array((int)$id));
			$map->set($importHostgroup);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single ImportHostgroup object identified by the given SQL query
	 *
	 * Runs the given query, initialises a ImpoortHostgroup object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ImportHostgroup cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ImportHostgroup
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find import hostgroup with these parameters'),
					var_export($params, true)
				)
			);
		}

		$importHostgroup = new model\ImportHostgroup();
		$importHostgroup->fillFromRowData($data);

		return $importHostgroup;
	}

	/**
	 * Load all ImportHostgroups
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $fileId The file id the desired import hostgroups must be assigned to (optional)
	 * @param         bool $conflict Whether to return host groups with name conflicts only
	 * @param         bool $register Whether to return objects to be registered
	 * @param         int $includedBy The host group id which the host group must be included by
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$fileId   = null,
		$conflict = null,
		$register = null,
		$includedBy  = null
	) {
		$join = array();
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($fileId !== null) {
			$join[] = sprintf("
					JOIN `ImportFile` ON (
						`ImportFile`.`id` = `ImportHostgroup`.`fileId`
						AND
						`ImportFile`.`id` = %d
					)
				",
				$fileId
			);
		}

		if ($conflict !== null) {
			$join[] = "
				LEFT JOIN `HostGroup` ON (
					`HostGroup`.`name` = `ImportHostgroup`.`hostgroup_name`
				)
			";

			$conditions->addStatement('Conflict', new sql\WhereStatement(
				'id',
				$conflict ? 'IS NOT' : 'IS',
				null,
				'HostGroup',
				false
			));

			if ($conflict) {
				$conditions->addStatement('Conflict2', new sql\WhereStatement(
					'hostgroupId',
					'IS',
					null,
					'ImportHostgroup',
					false
				));
			}
		}

		if ($register !== null) {
			$conditions->addStatement('Register', new sql\WhereStatement(
				'register',
				'=',
				$register,
				'ImportHostgroup'
			));
		}

		if ($includedBy !== null) {
			$join[] = "
				JOIN `ImportHostgroupInclude` ON (
					`ImportHostgroupInclude`.`name` = `ImportHostgroup`.`name`
				)
			";

			$conditions->addStatement('IncludedBy', new sql\WhereStatement(
				'importHostgroupId',
				'=',
				$includedBy,
				'ImportHostgroupInclude'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ImportHostgroup`.`id`
				FROM
					`ImportHostgroup`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "
			SELECT
				`ImportHostgroup`.*,
				`HostGroup`.`id` AS `conflictHostgroupId`
			FROM
				`ImportHostgroup`
			LEFT JOIN `HostGroup` ON (
				`HostGroup`.`name` = `ImportHostgroup`.`hostgroup_name`
			)
		";

		$hostgroupList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ImportHostgroup',
			array()
		);

		$hostgroupList->addOrderVariant('name.ASC',  '`ImportHostgroup`.`hostgroup_name` ASC');
		$hostgroupList->addOrderVariant('name.DESC', '`ImportHostgroup`.`hostgroup_name` DESC');

		if ($includedBy !== null) {
			$hostgroupList->addOrderVariant('includeId.ASC',  '`ImportHostgroupInclude`.`id` ASC');
			$hostgroupList->addOrderVariant('includeId.DESC', '`ImportHostgroupInclude`.`id` DESC');
		}

		return $hostgroupList;
	}

	/**
	 * Saves ImportHostgroup information
	 *
	 * If the ImportHostgroup object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $hostgroup
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $hostgroup) {
		$fieldDataMapping = array(
			'id'                => $hostgroup->id,
			'fileId'            => $hostgroup->fileId,
			'hostgroupId'       => $hostgroup->hostgroupId,
			'hostgroup_name'    => $hostgroup->hostgroup_name,
			'alias'             => $hostgroup->alias,
			'members'           => $hostgroup->members,
			'hostgroup_members' => $hostgroup->hostgroup_members,
			'name'              => $hostgroup->name,
			'register'          => $hostgroup->register
		);

		if ($hostgroup->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ImportHostgroup',
				$fieldDataMapping,
				sprintf('id = %d', $hostgroup->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ImportHostgroup',
				$fieldDataMapping
			);
			$hostgroup->id = Application::getDbInstance()->lastInsertId();
		}

		return $hostgroup->getIdentifier();
	}

	/**
	 * Delete an ImportHostgroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'ImportHostgroup',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Delete an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $includeID The ID of the include
	 */
	public function deleteInclude($includeId) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'ImportHostgroupInclude',
			sprintf('`id` = %d', $includeId)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Add an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $importHostgroupId The ImportHostgroup to associate
	 * @param         string $name The hostgroup name to be included
	 */
	public function addInclude($importHostgroupId, $name) {
		Application::getDBInstance()->insert(
			'ImportHostgroupInclude',
			array(
				'importHostgroupId' => $importHostgroupId,
				'name'              => $name
			)
		);
	}
}
