<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ImportService model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportServiceMapper extends Mapper {

	/**
	 * ImportServiceMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ImportServiceMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ImportService');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportServiceMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ImportService by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ImportService
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`ImportService`.*,
					`Service`.`id` AS `conflictServiceId`
				FROM
					`ImportService`
				LEFT JOIN `Service` ON (
					`Service`.`name` = `ImportService`.`service_description`
				)
				WHERE
					`ImportService`.`id` = ?
				LIMIT
					1
			';

			$importService = $this->getByQuery($query, array((int)$id));
			$map->set($importService);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single ImportService object identified by the given SQL query
	 *
	 * Runs the given query, initialises a ImpoortService object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ImportService cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ImportService
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find import service with these parameters'),
					var_export($params, true)
				)
			);
		}

		$importService = new model\ImportService();
		$importService->fillFromRowData($data);

		return $importService;
	}

	/**
	 * Load all ImportServices
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $fileId The file id the desired import services must be assigned to (optional)
	 * @param         string $hostName Restrict to Services assigned to given host
	 * @param         string $commandName Return services with given command only
	 * @param         bool $distinct Return distinct services only (group by service name)
	 * @param         bool $conflict Whether to return services with name conflict only
	 * @param         bool $register Whether to return objects to be registered
	 * @param         int $includedBy The service id which the service must be included by
	 * @param         bool $emptyName Whether the servie name may be empty
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$fileId      = null,
		$hostName    = null,
		$commandName = null,
		$distinct    = false,
		$conflict    = null,
		$register    = null,
		$includedBy  = null,
		$emptyName   = null
	) {
		$join = array();
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($fileId !== null) {
			$join[] = sprintf("
					JOIN `ImportFile` ON (
						`ImportFile`.`id` = `ImportService`.`fileId`
						AND
						`ImportFile`.`id` = %d
					)
				",
				$fileId
			);
		}

		if ($hostName !== null) {
			$conditions->addStatement('Hostname', new sql\WhereStatement(
				'host_name',
				'=',
				$hostName,
				'ImportService'
			));
		}

		if ($commandName !== null) {
			/* Either the command is
			 * 1. used without arguments ('command') or
			 * 2. in the form 'command!arg1!arg2!...'
			 */
			$join[] = sprintf("
					JOIN `ImportCommand` ON (
						(
							`ImportCommand`.`command_name` = `ImportService`.`check_command`
							OR
							LOCATE(CONCAT(`ImportCommand`.`command_name`, '!'), `ImportService`.`check_command`) = 1
						)
						AND (
							`ImportService`.`check_command` = %s
							OR
							LOCATE(%s, `check_command`) = 1
						)
					)
				",
				Application::getDbInstance()->quote($commandName),
				Application::getDbInstance()->quote(
					sprintf(
						'%s!',
						$commandName
					)
				)
			);
		}

		if ($conflict !== null) {
			$join[] = "
				LEFT JOIN `Service` ON (
					`Service`.`name` = `ImportService`.`service_description`
				)
			";

			$conditions->addStatement('Conflict', new sql\WhereStatement(
				'id',
				$conflict ? 'IS NOT' : 'IS',
				null,
				'Service',
				false
			));

			if ($conflict) {
				$conditions->addStatement('Conflict2', new sql\WhereStatement(
					'serviceId',
					'IS',
					null,
					'ImportService',
					false
				));
			}
		}

		if ($register !== null) {
			$conditions->addStatement('Register', new sql\WhereStatement(
				'register',
				'=',
				$register,
				'ImportService'
			));
		}

		if ($includedBy !== null) {
			$join[] = "
				JOIN `ImportServiceInclude` ON (
					`ImportServiceInclude`.`name` = `ImportService`.`name`
				)
			";

			$conditions->addStatement('IncludedBy', new sql\WhereStatement(
				'importServiceId',
				'=',
				$includedBy,
				'ImportServiceInclude'
			));
		}

		if ($emptyName !== null) {
			$conditions->addStatement('Empty name', new sql\WhereStatement(
				'service_description',
				$emptyName ? '=' : '<>',
				'',
				'ImportService',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ImportService`.`id`
				FROM
					`ImportService`
				%s
				%s
				%s
			",
			implode("\n", $join),
			$conditions,
			$distinct ? 'GROUP BY `ImportService`.`service_description`' : ''
		);

		$selectQuery = "
			SELECT
				`ImportService`.*,
				`Service`.`id` AS `conflictServiceId`
			FROM
				`ImportService`
			LEFT JOIN `Service` ON (
				`Service`.`name` = `ImportService`.`service_description`
			)
		";

		$serviceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ImportService',
			array()
		);

		$serviceList->addOrderVariant('name.ASC',  '`ImportService`.`service_description` ASC');
		$serviceList->addOrderVariant('name.DESC', '`ImportService`.`service_description` DESC');

		if ($includedBy !== null) {
			$serviceList->addOrderVariant('includeId.ASC',  '`ImportServiceInclude`.`id` ASC');
			$serviceList->addOrderVariant('includeId.DESC', '`ImportServiceInclude`.`id` DESC');
		}

		return $serviceList;
	}

	/**
	 * Saves ImportService information
	 *
	 * If the ImportService object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $service
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $service) {
		$fieldDataMapping = array(
			'id'                  => $service->id,
			'fileId'              => $service->fileId,
			'serviceId'           => $service->serviceId,
			'host_name'           => $service->host_name,
			'hostgroup_name'      => $service->hostgroup_name,
			'service_description' => $service->service_description,
			'check_command'       => $service->check_command,
			'max_check_attempts'  => $service->max_check_attempts,
			'check_interval'      => $service->check_interval,
			'retry_interval'      => $service->retry_interval,
			'name'                => $service->name,
			'register'            => $service->register
		);

		if ($service->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ImportService',
				$fieldDataMapping,
				sprintf('id = %d', $service->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ImportService',
				$fieldDataMapping
			);
			$service->id = Application::getDbInstance()->lastInsertId();
		}

		return $service->getIdentifier();
	}

	/**
	 * Delete an ImportService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'ImportService',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Delete ImportServices by their name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       void
	 */
	public function deleteByName($name) {
		Application::getDbInstance()->delete(
			'ImportService',
			sprintf('`service_description` = %s', Application::getDbInstance()->quote($name))
		);
	}

	/**
	 * Delete ImportServices by host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $hostname
	 * @returns       void
	 */
	public function deleteByHost($hostname) {
		Application::getDbInstance()->delete(
			'ImportService',
			sprintf('`host_name` = %s', Application::getDbInstance()->quote($hostname))
		);
	}

	/**
	 * Delete ImportServices by their command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $commandName
	 * @returns       void
	 */
	public function deleteByCommand($commandName) {
		/* Either the command is
		 * 1. used without arguments ('command') or
		 * 2. in the form 'command!arg1!arg2!...'
		 */
		Application::getDbInstance()->delete(
			'ImportService',
			sprintf('
					`check_command` = %s
					OR
					LOCATE(%s, `check_command`) = 1
				',
				Application::getDbInstance()->quote($commandName),
				Application::getDbInstance()->quote(
					sprintf(
						'%s!',
						$commandName
					)
				)
			)
		);
	}

	/**
	 * Delete an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $includeID The ID of the include
	 */
	public function deleteInclude($includeId) {
		Application::getDBInstance()->beginTransaction();

		Application::getDBInstance()->delete(
			'ImportServiceInclude',
			sprintf('`id` = %d', $includeId)
		);

		Application::getDBInstance()->commit();
	}

	/**
	 * Rename services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $oldName
	 * @param         string $newName
	 */
	public function rename($oldName, $newName) {
		Application::getDBInstance()->update(
			'ImportService',
			array(
				'service_description' => $newName
			),
			sprintf('`service_description` = %s', Application::getDBInstance()->quote($oldName))
		);
	}

	/**
	 * Update service ids
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name The service name
	 * @param         string $serviceId The new serviceId to be set
	 */
	public function updateServiceId($name, $serviceId) {
		Application::getDBInstance()->update(
			'ImportService',
			array(
				'serviceId' => $serviceId
			),
			sprintf('`service_description` = %s', Application::getDBInstance()->quote($name))
		);
	}

	/**
	 * Update a service value
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name The service name
	 * @param         string $serviceId The new serviceId to be set
	 */
	public function updateByName($name, $key, $value) {
		Application::getDBInstance()->update(
			'ImportService',
			array(
				$key => $value
			),
			sprintf('`service_description` = %s', Application::getDBInstance()->quote($name))
		);
	}

	/**
	 * Add an include
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $importServiceId The ImportService to associate
	 * @param         string $name The service name to be included
	 */
	public function addInclude($importServiceId, $name) {
		Application::getDBInstance()->insert(
			'ImportServiceInclude',
			array(
				'importServiceId' => $importServiceId,
				'name'            => $name
			)
		);
	}
}
