<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the Infofeed model.
 *
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class InfofeedMapper extends Mapper {

	/**
	* Infofeed object to implement singleton
	*
	* @var            ch\monitoring\datamapper\InfofeedMapper $singleton
	*/
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\HostGroup');
	}

	/**
	 * Implements a singleton
	 *
	 * @returns       ch\monitoring\datamapper\InfofeedMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load an Infofeed by id
	 *
	 * @param         int $id
	 * @returns       ch\monitoring\model\infofeed
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Infofeed`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$infofeed = $this->getByQuery($query, array((int)$id));
			$map->set($infofeed);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single Infofeed object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Infofeed object and returns it
	 *   
	 * @throws        \OutOfBoundsException if the HostGroup cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Infofeed
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find infofeed with these parameters'),
					var_export($params, true)
				)
			);
		}

		$infofeed = new model\Infofeed();
		$infofeed->fillFromRowData($data);

		return $infofeed;
	}

	/**
	 * Load all Infofeeds
	 *
	 * @param         int $userId (per default null)
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$userId      = null,
		$typeId      = null,
		$hostId      = null,
		$hostGroupId = null,
		$customerId  = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();

		if ($userId) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'Infofeed'
			));
		}

		if ($typeId == 1 or $hostId) {
			$join[] = "
				JOIN `Infofeed_Host` ON (
					`Infofeed_Host`.`infofeedId` = `Infofeed`.`id`
				)
			";

			if ($hostId) {
				$conditions->addStatement('Host', new sql\WhereStatement(
					'hostId',
					'=',
					$hostId,
					'Infofeed_Host'
				));
			}
		}

		if ($typeId == 2 or $hostGroupId) {
			$join[] = "
				JOIN `Infofeed_HostGroup` ON (
					`Infofeed_HostGroup`.`infofeedId` = `Infofeed`.`id`
				)
			";

			if ($hostGroupId) {
				$conditions->addStatement('HostGroup', new sql\WhereStatement(
					'hostGroupId',
					'=',
					$hostGroupId,
					'Infofeed_HostGroup'
				));
			}
		}

		if ($typeId == 3 or $customerId) {
			$join[] = "
				JOIN `Infofeed_Customer` ON (
					`Infofeed_Customer`.`infofeedId` = `Infofeed`.`id`
				)
			";

			if ($customerId) {
				$conditions->addStatement('Customer', new sql\WhereStatement(
					'customerId',
					'=',
					$customerId,
					'Infofeed_Customer'
				));
			}
		}

		$searchQuery = sprintf("
				SELECT
					`Infofeed`.`id`
				FROM
					`Infofeed`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `Infofeed`";

		$infofeedList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Infofeed',
			array()
		);

		return $infofeedList;
	}

	/**
	 * Saves Infofeed information
	 *
	 * If the Infofeed object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @param         \org\adfinis\framework\model\Entity $infofeed
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $infofeed) {
		$fieldDataMapping = array(
			'id'            => $infofeed->id,
			'userId'        => $infofeed->userId,
			'onAlert'       => $infofeed->onAlert,
			'onWarning'     => $infofeed->onWarning,
			'onAcknowledge' => $infofeed->onAcknowledge
		);

		if ($infofeed->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Infofeed',
				$fieldDataMapping,
				sprintf('id = %d', $infofeed->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Infofeed',
				$fieldDataMapping
			);
			$infofeed->id = Application::getDbInstance()->lastInsertId();
		}

		return $infofeed->getIdentifier();
	}

	/**
	 * Delete an Infofeed
	 *
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'Infofeed',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Changes infofeeds notification types
	 *
	 * @param         array $notificationTypes
	 */
	public function changeNotificationTypes($infofeedId, $notificationTypes) {
		Application::getDbInstance()->beginTransaction();

		$this->unassignNotificationTypes($infofeedId);

		foreach ($notificationTypes as $notificationTypeId) {
			$this->assignNotificationType($infofeedId, $notificationTypeId);
		}

		Application::getDbInstance()->commit();
	}

	/**
	 * Get all infofeeds refered by given hostID
	 *
	 * @param         model\Host|int $host
	 * @return        SortingIterator
	 */
	public function findAllByHost($host) {
		$db = Application::getDbInstance();
		if (is_numeric($host)) {
			$host = model\Host::getMapper()->getByID($host);
		}

		$query = sprintf("SELECT `infofeedId` FROM `Infofeed_Host` WHERE `hostId` = %d", $host->id);
		$infofeedIDs = array_values($db->query($query)->fetchAll(\PDO::FETCH_COLUMN) ?: []);

		$query = sprintf(
			"
				SELECT `ic`.*
				FROM `Infofeed_Customer` AS `ic`
				JOIN `Host` AS `h` ON (`ic`.`customerId` = `h`.`customerId`)
				WHERE `h`.`id` = %d
			",
			$host->id
		);
		$infofeedsCustomer = $db->fetchAll($query);

		if ($infofeedsCustomer) {
			foreach ($infofeedsCustomer as $infofeedCustomer) {
				$excludedQuery = sprintf(
					"SELECT `hostId` FROM `Infofeed_Customer_Excluded_Hosts` WHERE `infofeedCustomerId` = %d",
					$infofeedCustomer['id']
				);
				$excludedHosts = $db->query($excludedQuery)->fetchAll(\PDO::FETCH_COLUMN);
				if (!in_array($host->id, $excludedHosts)) {
					$infofeedIDs[] = $infofeedCustomer['infofeedId'];
				}
			}
		}

		if (!$infofeedIDs) {
			return [];
		}

		$searchQuery = sprintf("
				SELECT
					`id`
				FROM
					`Infofeed`
				WHERE
					`id` IN (%s)
			",
			join(', ', array_map('intval', $infofeedIDs))
		);

		$selectQuery = "SELECT * FROM `Infofeed`";

		return SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Infofeed',
			array()
		);
	}

	/**
	 * Assigns a host to an infofeed
	 *
	 * @param         int $infofeedId
	 * @param         int $hostId
	 * @returns       void
	 */
	public function assignHost($infofeedId, $hostId) {
		Application::getDbInstance()->insert(
			'Infofeed_Host',
			array(
				'infofeedId' => $infofeedId,
				'hostId'     => $hostId
			)
		);
	}

	/**
	 * Assigns a hostgroup to an infofeed
	 *
	 * @param         int $infofeedId
	 * @param         int $hostGroupId
	 * @returns       void
	 */
	public function assignHostGroup($infofeedId, $hostGroupId) {
		Application::getDbInstance()->insert(
			'Infofeed_HostGroup',
			array(
				'infofeedId'  => $infofeedId,
				'hostGroupId' => $hostGroupId
			)
		);
	}

	/**
	 * Assigns a customer to an infofeed
	 *
	 * @param         int $infofeedId
	 * @param         int $customerId
	 * @returns       void
	 */
	public function assignCustomer($infofeedId, $customerId) {
		Application::getDbInstance()->insert(
			'Infofeed_Customer',
			array(
				'infofeedId' => $infofeedId,
				'customerId' => $customerId
			)
		);
	}

	/**
	 * Assigns a notification type to an infofeed
	 *
	 * @param         int $infofeedId
	 * @param         int $slaActionId
	 * @returns       void
	 */
	public function assignNotificationType($infofeedId, $slaActionId) {
		Application::getDbInstance()->insert(
			'Infofeed_NotificationType',
			array(
				'infofeedId'         => $infofeedId,
				'slaActionId'        => $slaActionId
			)
		);
	}

	/**
	 * Unassigns all notification types from an infofeed
	 *
	 * @param         int $infofeedId
	 * @returns       void
	 */
	public function unassignNotificationTypes($infofeedId) {
		Application::getDbInstance()->delete(
			'Infofeed_NotificationType',
			sprintf('`infofeedId` = %d', $infofeedId)
		);
	}

	/**
	 * Removes all hosts restrictions
	 *
	 * @param       int $infofeedId
	 * @returns     void
	 */
	public function removeAllExcludedHosts($infofeedId) {
		$query = sprintf("SELECT `id` FROM `Infofeed_Customer` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);

		Application::getDbInstance()->delete(
			'Infofeed_Customer_Excluded_Hosts',
			sprintf('`infofeedCustomerId` = %d', $id)
		);	
		

		$query = sprintf("SELECT `id` FROM `Infofeed_HostGroup` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);
		
		Application::getDbInstance()->delete(
			'Infofeed_HostGroup_Excluded_Hosts',
			sprintf('`infofeedHostGroupId` = %d', $id)
		);	
	}

	/**
	 * Exclude a host for a customer
	 *
	 * @param       int $infofeedId
	 * @param       array $hostIds
	 * @returns     void
	 */
	public function excludeHostForCustomer($infofeedId, $hostIds) {
		$query = sprintf("SELECT `id` FROM `Infofeed_Customer` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);

		foreach($hostIds as $hostId) {
			Application::getDbInstance()->insert(
				'Infofeed_Customer_Excluded_Hosts',
				array(
					'infofeedCustomerId' => $id,
					'hostId'             => $hostId
				)
			);
		}
	}


	/**
	 * Exclude a host for a hostgroup
	 *
	 * @param       int $infofeedId
	 * @param       array $hostIds
	 * @returns     void
	 */
	public function excludeHostForHostGroup($infofeedId, $hostIds) {
		$query = sprintf("SELECT `id` FROM `Infofeed_HostGroup` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);

		foreach($hostIds as $hostId) {
			Application::getDbInstance()->insert(
				'Infofeed_HostGroup_Excluded_Hosts',
				array(
					'infofeedHostGroupId' => $id,
					'hostId'             => $hostId
				)
			);
		}
	}

	/**
	 * Returns all IDS of the exclude host for a customer
	 *
	 * @param       int $infofeedId
	 * @returns     array
	 */
	public function excludedHostForCustomer($infofeedId) {
		$query = sprintf("SELECT `id` FROM `Infofeed_Customer` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);
		$query = sprintf("SELECT `hostId` FROM `Infofeed_Customer_Excluded_Hosts` WHERE `infofeedCustomerId` = %d", $id);
		return Application::getDbInstance()->fetchCol($query);
	}

	/**
	 * Returns all IDS of the exclude host for a hostgroup
	 *
	 * @param       int $infofeedId
	 * @returns     array
	 */
	public function excludedHostForHostGroup($infofeedId) {
		$query = sprintf("SELECT `id` FROM `Infofeed_HostGroup` WHERE `infofeedId` = %d", $infofeedId);
		$id = Application::getDbInstance()->fetchOne($query);
		$query = sprintf("SELECT `hostId` FROM `Infofeed_HostGroup_Excluded_Hosts` WHERE `infofeedHostGroupId` = %d", $id);
		return Application::getDbInstance()->fetchCol($query);
	}
}
