<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use org\adfinis\framework\db\SortingIterator;

/**
 * Datamapper for the MaintenanceChecklistAnswer model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklistAnswerMapper extends Mapper {

	/**
	 * MaintenanceChecklistMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceChecklistMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceChecklistAnswer');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceChecklistMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceChecklist by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceChecklistAnswer
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceHost_Question`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$answer = $this->getByQuery($query, array((int)$id));
			$map->set($answer);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single MaintenanceChecklistAnswer object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceChecklistAnswer object and returns it
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceChecklist cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceChecklistAnswer
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find checklist answer with these parameters'),
					var_export($params, true)
				)
			);
		}

		$answer = new model\MaintenanceChecklistAnswer();
		$answer->fillFromRowData($data);

		return $answer;
	}

	/**
	 * Load all MaintenanceChecklistAnswers
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId The Host the answer shall belong to
	 * @param         int $ticketId The ticket the answer shall belong to
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($hostId = null, $ticketId = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
		$join = array();
		
		if($hostId !== null || $ticketId !== null) {
			$join[] = "
				JOIN `MaintenanceTicketHost` ON (
					`MaintenanceTicketHost`.`id` = `MaintenanceHost_Question`.`ticketHostId`
				)
			";
		}

		if($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'MaintenanceTicketHost',
				false
			));
		}

		if($ticketId !== null) {
			$conditions->addStatement('Ticket', new sql\WhereStatement(
				'ticketId',
				'=',
				$ticketId,
				'MaintenanceTicketHost',
				false
			));
		}


		$searchQuery = sprintf("
				SELECT DISTINCT
					`MaintenanceHost_Question`.`id`
				FROM
					`MaintenanceHost_Question`
				JOIN `MaintenanceChecklistQuestion` ON (
					`MaintenanceChecklistQuestion`.`id` = `MaintenanceHost_Question`.`questionId`
				)
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `MaintenanceHost_Question`";

		$answers = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceChecklistAnswer',
			array()
		);

		$answers->addOrderVariant('name.ASC',  '`MaintenanceChecklistQuestion`.`name` ASC');
		$answers->addOrderVariant('name.DESC', '`MaintenanceChecklistQuestion`.`name` DESC');

		return $answers;
	}

	
	/**
	 * Saves Answer information
	 *
	 * If the MaintenanceChecklistAnswer object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $answer
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $answer) {
		$fieldDataMapping = array(
			'id'           => $answer->id,
			'comment'      => $answer->comment,
			'questionId'   => $answer->questionId,
			'ticketHostId' => $answer->ticketHostId,
			'isChecked'    => $answer->isChecked
		);

		if ($answer->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceHost_Question',
				$fieldDataMapping,
				sprintf('id = %d', $answer->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceHost_Question',
				$fieldDataMapping
			);
			$answer->id = Application::getDbInstance()->lastInsertId();
		}

		return $answer->getIdentifier();
	}

	/**
	 * Delete a MaintenanceChecklistAnswer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceHost_Question',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	
	/**
	 * Assignes a checklist to a host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 *
	public function assignHost($hostId, $checklistId) {
		Application::getDBInstance()->insert(
			'MaintenanceChecklist_Host',
			array(
				'hostId'  => $hostId,
				'checklistId' => $checklistId
			)
		);
	}

	/**
	 * Unassignes a checklist to a host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 *
	public function unassignHost($hostId, $checklistId) {
		Application::getDBInstance()->delete(
			'MaintenanceChecklist_Host',
			sprintf(
				"hostId = %d AND checklistId = %d",
				$hostId,
				$checklistId
			)
		);
	}
	*/
}
