<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the MaintenanceChecklist model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklistMapper extends Mapper {

	/**
	 * MaintenanceChecklistMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceChecklistMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceChecklist');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceChecklistMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceChecklist by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceChecklist
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceChecklist`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$maintenanceChecklist = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceChecklist);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a MaintenanceChecklist by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $name
	 * @returns       ch\monitoring\model\MaintenanceChecklist
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`MaintenanceChecklist`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single MaintenanceChecklist object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceChecklist object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceChecklist cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceChecklist
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance checklist with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceChecklist = new model\MaintenanceChecklist();
		$maintenanceChecklist->fillFromRowData($data);

		return $maintenanceChecklist;
	}

	/**
	 * Load all MaintenanceChecklists
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId The Host the checklists shall belong to
	 * @param         int $groupId The MaintenanceGroup the checklists shall belong to
	 * @param         bool|null $isTemplate make a restriction that only templates will be returned
	 * @param         bool $includePermission don't check the customer permission
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$hostId  = null,
		$groupId = null,
		$isTemplate = null,
		$includePermission = true
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();

		if ($hostId !== null) {
			$join[] = "
				JOIN `MaintenanceChecklist_Host` ON (
					`MaintenanceChecklist`.`id` = `MaintenanceChecklist_Host`.`checklistId`
				)
			";

			$join[] = "
				JOIN `Host` ON (
					`Host`.`id` = `MaintenanceChecklist_Host`.`hostId`
				)
			";

			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'MaintenanceChecklist_Host'
			));

			if ($includePermission) {
				$customers = Application::getUser()->getCustomersWithPermission(
					model\Permission::MAINTENANCE_READ
				);

				$conditions->addStatement('Permission', new sql\WhereStatement(
					'customerId',
					$customers === null ? 'IS' : 'IN',
					$customers,
					'Host',
					false
				));
			}
		}

		if ($groupId !== null) {
			$join[] = "
				JOIN `MaintenanceChecklist_MaintenanceGroup` ON (
					`MaintenanceChecklist`.`id` = `MaintenanceChecklist_MaintenanceGroup`.`checklistId`
				)
			";

			$conditions->addStatement('Group', new sql\WhereStatement(
				'groupId',
				'=',
				$groupId,
				'MaintenanceChecklist_MaintenanceGroup'
			));
		}

		if ($isTemplate !== null) {
			$conditions->addStatement('IsTemplate', new sql\WhereStatement(
				'isTemplate',
				'=',
				$isTemplate,
				'MaintenanceChecklist'
			));
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`MaintenanceChecklist`.`id`
				FROM
					`MaintenanceChecklist`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `MaintenanceChecklist`";

		$maintenanceChecklistList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceChecklist',
			array()
		);

		$maintenanceChecklistList->addOrderVariant('name.ASC',  '`MaintenanceChecklist`.`name` ASC');
		$maintenanceChecklistList->addOrderVariant('name.DESC', '`MaintenanceChecklist`.`name` DESC');

		return $maintenanceChecklistList;
	}

	/**
	 * Load all checklist parents (not parent parent!) of a MaintenanceChecklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceChecklist
	 * @returns       array
	 */
	public function getParentsByID($checklistId) {
		$searchQuery = sprintf("
				SELECT
					`MC_MC`.`parentId` as `id`
				FROM
					`MC_MC`
				WHERE
					`MC_MC`.`checklistId` = %d
			",
			$checklistId
		);
		
		$selectQuery = "SELECT * FROM `MaintenanceChecklist`";

		$maintenanceChecklistList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceChecklist',
			array()
		);

		$maintenanceChecklistList->addOrderVariant('name.ASC',  '`MaintenanceChecklist`.`name` ASC');
		$maintenanceChecklistList->addOrderVariant('name.DESC', '`MaintenanceChecklist`.`name` DESC');

		return $maintenanceChecklistList;
	}

	/**
	 * Saves MaintenanceChecklist information
	 *
	 * If the MaintenanceChecklist object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceChecklist
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceChecklist) {
		$fieldDataMapping = array(
			'id'         => $maintenanceChecklist->id,
			'name'       => $maintenanceChecklist->name,
			'comment'    => $maintenanceChecklist->comment,
			'isTemplate' => $maintenanceChecklist->isTemplate
		);

		if ($maintenanceChecklist->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceChecklist',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceChecklist->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceChecklist',
				$fieldDataMapping
			);
			$maintenanceChecklist->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceChecklist->getIdentifier();
	}

	/**
	 * Delete a MaintenanceChecklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		$questions = MaintenanceChecklistQuestionMapper::getInstance()->getAll($identifier['id'], null, false, false)->toArray();

		Application::getDbInstance()->delete(
			'MaintenanceChecklist',
			sprintf('`id` = %d', $identifier['id'])
		);

		foreach($questions as $question) {
			//if(!$question->isInUse()) {
				$question->delete();
			//}
		}
	}

	/**
	 * Add a question to this checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $questionId
	 * @param         int $checklistId
	 * @param         bool $exclude Optional, defaults to false
	 * @returns       void
	 */
	public function addQuestion($questionId, $checklistId) {
		Application::getDBInstance()->insert(
			'MaintenanceChecklist_Question',
			array(
				'questionId'  => $questionId,
				'checklistId' => $checklistId
			)
		);
	}

	/**
	 * Remove a question from this checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $questionId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function removeQuestion($questionId, $checklistId) {
		Application::getDBInstance()->delete(
			'MaintenanceChecklist_Question',
			sprintf(
				"questionId = %d AND checklistId = %d",
				$questionId,
				$checklistId
			)
		);

		//check if another checklist point to that question - if not, delete it
		if(!MaintenanceChecklistQuestionMapper::getInstance()->isInUse($questionId)) {
			MaintenanceChecklistQuestionMapper::getInstance()->delete(array('id' => $questionId));
		}
	}

	/**
	 * Assigns a checklist to a host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function assignHost($hostId, $checklistId) {
		Application::getDBInstance()->insert(
			'MaintenanceChecklist_Host',
			array(
				'hostId'  => $hostId,
				'checklistId' => $checklistId
			)
		);
	}

	/**
	 * Unassigns a checklist to a host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function unassignHost($hostId, $checklistId) {
		$statement = Application::getDBInstance()->prepare("
			DELETE FROM
				`MaintenanceChecklist_ExcludedQuestion`
			USING
				`MaintenanceChecklist_ExcludedQuestion`
			JOIN `MaintenanceChecklist_Question` ON (
				`MaintenanceChecklist_Question`.`id` = `MaintenanceChecklist_ExcludedQuestion`.`checklist_questionId`
				AND
				`MaintenanceChecklist_Question`.`checklistId` = :checklistId
			)
			WHERE
				`MaintenanceChecklist_ExcludedQuestion`.`hostId` = :hostId
		");
		$statement->execute(array(
			'hostId'      => $hostId,
			'checklistId' => $checklistId
		));

		Application::getDBInstance()->delete(
			'MaintenanceChecklist_Host',
			sprintf(
				"hostId = %d AND checklistId = %d",
				$hostId,
				$checklistId
			)
		);
	}

	/**
	 * Assigns a checklist to a maintenance group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function assignGroup($groupId, $checklistId) {
		Application::getDBInstance()->insert(
			'MaintenanceChecklist_MaintenanceGroup',
			array(
				'groupId'  => $groupId,
				'checklistId' => $checklistId
			)
		);
	}

	/**
	 * Unassigns a checklist from a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function unassignGroup($groupId, $checklistId) {
		$statement = Application::getDBInstance()->prepare("
			DELETE FROM
				`MaintenanceChecklist_ExcludedQuestion`
			USING
				`MaintenanceChecklist_ExcludedQuestion`
			JOIN `MaintenanceChecklist_Question` ON (
				`MaintenanceChecklist_Question`.`id` = `MaintenanceChecklist_ExcludedQuestion`.`checklist_questionId`
				AND
				`MaintenanceChecklist_Question`.`checklistId` = :checklistId
			)
			WHERE
				`MaintenanceChecklist_ExcludedQuestion`.`groupId` = :groupId
		");
		$statement->execute(array(
			'groupId' => $groupId,
			'checklistId' => $checklistId
		));

		Application::getDBInstance()->delete(
			'MaintenanceChecklist_MaintenanceGroup',
			sprintf(
				"groupId = %d AND checklistId = %d",
				$groupId,
				$checklistId
			)
		);
	}

	/**
	 * Assignes a template to a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function assignTemplate($checklistId, $templateId) {
		Application::getDBInstance()->insert(
			'MC_MC',
			array(
				'parentId'  => $templateId,
				'checklistId' => $checklistId
			)
		);
	}

	/**
	 * Unassignes a template from checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         int $checklistId
	 * @returns       void
	 */
	public function unassignTemplate($checklistId, $templateId) {
		Application::getDBInstance()->delete(
			'MC_MC',
			sprintf(
				"parentId = %d AND checklistId = %d",
				$templateId,
				$checklistId
			)
		);
	}
}
