<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;
use org\adfinis\framework\db\sql;

/**
 * Datamapper for the MaintenanceChecklistQuestion model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklistQuestionMapper extends Mapper {

	/**
	 * MaintenanceChecklistQuestionMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceChecklistQuestionMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceChecklistQuestion');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceChecklistQuestionMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceChecklistQuestion by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceChecklistQuestion
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceChecklistQuestion`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$question = $this->getByQuery($query, array((int)$id));
			$map->set($question);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single MaintenanceChecklistQuestion object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a MaintenanceChecklistQuestion object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceChecklistQuestion cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceChecklistQuestion
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance tag with these parameters'),
					var_export($params, true)
				)
			);
		}

		$question = new model\MaintenanceChecklistQuestion();
		$question->fillFromRowData($data);

		return $question;
	}

	/**
	 * Load all MaintenanceChecklistQuestions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $checklistId Optional.
	 * @param         null|bool $exclude
	 * @param         int $excludingHost Optional. the host that should be excluded
	 * @param         int $excludingGroup Optional. the group that should be excluded
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$checklistId = null,
		$exclude = null,
		$excludingHost= null,
		$excludingGroup=null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();
		
		if ($checklistId !== null || $exclude) {
			$join[] = "
				JOIN `MaintenanceChecklist_Question` ON (
					`MaintenanceChecklistQuestion`.`id` = `MaintenanceChecklist_Question`.`questionId`
				)
			";
		}

		if ($checklistId !== null) {
			$conditions->addStatement('Checklist', new sql\WhereStatement(
				'checklistId',
				'=',
				$checklistId,
				'MaintenanceChecklist_Question'
			));
		}

		if ($exclude !== null) {
			$join[] = sprintf("
				%s JOIN `MaintenanceChecklist_ExcludedQuestion` ON (
					`MaintenanceChecklist_ExcludedQuestion`.`checklist_questionId` = `MaintenanceChecklist_Question`.`id`
				)
				",

				$exclude == false ? "" : "LEFT OUTER"
			);

			$conditions->addStatement('Exclude', new sql\WhereStatement(
				!is_null($excludingHost) ? 'hostId' : 'groupId',
				'=',
				!is_null($excludingHost) ? $excludingHost : $excludingGroup,
				'MaintenanceChecklist_ExcludedQuestion'
			));
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`MaintenanceChecklistQuestion`.`id`
				FROM
					`MaintenanceChecklistQuestion`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);
		
		$selectQuery = "SELECT * FROM `MaintenanceChecklistQuestion`";

		$questionList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceChecklistQuestion',
			array()
		);

		$questionList->addOrderVariant('name.ASC',  '`MaintenanceChecklistQuestion`.`name` ASC');
		$questionList->addOrderVariant('name.DESC', '`MaintenanceChecklistQuestion`.`name` DESC');

		return $questionList;
	}

	/**
	 * Saves MaintenanceChecklistQuestion information
	 *
	 * If the MaintenanceChecklistQuestion object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $question
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $question) {
		$fieldDataMapping = array(
			'id'      => $question->id,
			'name'    => $question->name,
			'comment' => $question->comment
		);

		if ($question->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceChecklistQuestion',
				$fieldDataMapping,
				sprintf('id = %d', $question->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceChecklistQuestion',
				$fieldDataMapping
			);
			$question->id = Application::getDbInstance()->lastInsertId();
		}

		return $question->getIdentifier();
	}

	/**
	 * Delete a MaintenanceChecklistQuestion
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceChecklist_Question',
			sprintf('`questionId` = %d', $identifier['id'])
		);

		Application::getDbInstance()->delete(
			'MaintenanceChecklistQuestion',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Unassignes a template from checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $checklistId
	 * @param         int $questionId
	 * @param         bool $exclude
	 * @returns       void
	 */
	public function excludeQuestion($checklistId, $questionId, $hostId, $groupId, $exclude) {
		$query = sprintf("
			SELECT
				`id`
			FROM
				`MaintenanceChecklist_Question`
			WHERE
				`checklistId` = %d
				AND
				`questionId` = %d
			",
			$checklistId,
			$questionId
		);
		
		$checklist_questionId = Application::getDbInstance()->fetchOne($query);

		if ($exclude) {
			$statement = Application::getDbInstance()->prepare(sprintf("
				INSERT INTO
					`MaintenanceChecklist_ExcludedQuestion`
				(
					`checklist_questionId`,
					%s
				)
				VALUES(
					:checklist_questionId,
					:hostId
				)
				",
				!is_null($hostId) ? '`hostId`' : '`groupId`'
			));
		}
		else {
			$statement = Application::getDbInstance()->prepare(sprintf("
				DELETE FROM
					`MaintenanceChecklist_ExcludedQuestion`
				WHERE
					`checklist_questionId` = :checklist_questionId
					AND
					%s = :hostId
				",
				!is_null($hostId) ? '`hostId`' : '`groupId`'
			));
		}

		$statement->execute(array(
			'checklist_questionId' => $checklist_questionId,
			'hostId'               => !is_null($hostId) ? $hostId : $groupId,
		));
	}

	/**
	 * Indicate if a question is available or not inside the given checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $checklistId
	 * @param         int $questionId
	 * @param         int $hostId
	 * @param         int $groupId
	 * @returns       bool
	 */
	public function isExcluded($checklistId, $questionId, $hostId, $groupId) {
		$query = sprintf("
			SELECT
				`MaintenanceChecklist_ExcludedQuestion`.`id`
			FROM
				`MaintenanceChecklist_ExcludedQuestion`
			
			JOIN `MaintenanceChecklist_Question` ON (
				`MaintenanceChecklist_Question`.`id` = `MaintenanceChecklist_ExcludedQuestion`.`checklist_questionId`
			)

			WHERE
				`MaintenanceChecklist_Question`.`checklistId` = %d
				AND
				`MaintenanceChecklist_Question`.`questionId` = %d
				AND
				`MaintenanceChecklist_ExcludedQuestion`.%s = %d
			",
			$checklistId,
			$questionId,
			!is_null($hostId) ? '`hostId`' : '`groupId`',
			!is_null($hostId) ? $hostId : $groupId
		);

		return (bool)Application::getDbInstance()->fetchOne($query);
	}

	/**
	 * Indicates if a question is used by a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $questionId
	 * @returns       bool
	 */
	public function isInUse($questionId) {
		$query = sprintf("
			SELECT
				COUNT(*)
			FROM
				MaintenanceChecklist_Question
			WHERE
				questionId = %d
			",
			$questionId
		);

		return (bool)Application::getDbInstance()->fetchOne($query);
	}
}
