<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\util;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the MaintenanceComment model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceCommentMapper extends Mapper {

	/**
	 * MaintenanceCommentMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceCommentMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceComment');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceCommentMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceComment by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceComment
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`MaintenanceComment`.*
				FROM
					`MaintenanceComment`
				WHERE
					`MaintenanceComment`.`id` = ?
				LIMIT
					1
			';

			$maintenanceComment = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceComment);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single MaintenanceComment object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceComment object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceComment cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceComment
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance comment with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceComment = new model\MaintenanceComment();
		$maintenanceComment->fillFromRowData($data);

		return $maintenanceComment;
	}

	/**
	 * Load all MaintenanceComments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $ticketHostId Optional. Limit results to this maintenance ticket.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($ticketHostId = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
		$customers = Application::getUser()->getCustomersWithPermission(
			model\Permission::MAINTENANCE_READ
		);

		if ($ticketHostId !== null) {
			$conditions->addStatement('Maintenance', new sql\WhereStatement(
				'ticketHostId',
				'=',
				$ticketHostId,
				'MaintenanceComment'
			));
			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'MaintenanceTicket',
				false
			));
		}
		else {
			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
		}

		$searchQuery = sprintf("
				SELECT
					`MaintenanceComment`.`id`
				FROM
					`MaintenanceComment`
				JOIN `MaintenanceTicketHost` ON (
					`MaintenanceTicketHost`.`id` = `MaintenanceComment`.`ticketHostId`
				)
				JOIN `Host` ON (
					`Host`.`id` = `MaintenanceTicketHost`.`hostId`
				)
				JOIN `MaintenanceTicket` ON (
					`MaintenanceTicket`.`id` = `MaintenanceTicketHost`.`ticketId`
				)
				%s
			",
			$conditions
		);
		$selectQuery = "SELECT * FROM `MaintenanceComment`";

		$maintenanceCommentList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceComment',
			array()
		);

		$maintenanceCommentList->addOrderVariant('timestamp.DESC',  '`MaintenanceComment`.`timestamp` DESC');
		$maintenanceCommentList->addOrderVariant('timestamp.ASC',  '`MaintenanceComment`.`timestamp` ASC');

		return $maintenanceCommentList;
	}

	/**
	 * Saves MaintenanceComment information
	 *
	 * If the MaintenanceComment object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceComment
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceComment) {
		$fieldDataMapping = array(
			'id'             => $maintenanceComment->id,
			'ticketHostId'   => $maintenanceComment->ticketHostId,
			'userId'         => $maintenanceComment->userId,
			'oldStateId'     => $maintenanceComment->oldStateId,
			'newStateId'     => $maintenanceComment->newStateId,
			'oldOsVersionId' => $maintenanceComment->oldOsVersionId,
			'newOsVersionId' => $maintenanceComment->newOsVersionId,
			'timestamp'      => util\TimeUtility::dateOrNull($maintenanceComment->timestamp),
			'comment'        => $maintenanceComment->comment
		);

		if ($maintenanceComment->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceComment',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceComment->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceComment',
				$fieldDataMapping
			);
			$maintenanceComment->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceComment->getIdentifier();
	}

	/**
	 * Delete a MaintenanceComment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceComment',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Delete MaintenanceComments by maintenance id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $ticketHostId Delete all Comments from this ticket host.
	 * @returns       void
	 */
	public function deleteByTicketHostId($ticketHostId) {
		Application::getDbInstance()->beginTransaction();

		$comments = $this->getAll($ticketHostId);
		foreach ($comments as $comment) {
			$this->delete($comment->getIdentifier());
		}

		Application::getDbInstance()->commit();
	}
}
