<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the MaintenanceCycle model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceCycleMapper extends Mapper {

	/**
	 * MaintenanceCycleMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceCycleMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceCycle');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceCycleMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceCycle by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceCycle
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceCycle`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$maintenanceCycle = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceCycle);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single MaintenanceCycle object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a MaintenanceCycle object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceCycle cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceCycle
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance cycle with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceCycle = new model\MaintenanceCycle();
		$maintenanceCycle->fillFromRowData($data);

		return $maintenanceCycle;
	}

	/**
	 * Load all MaintenanceCycles
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`MaintenanceCycle`.`id`
			FROM
				`MaintenanceCycle`
		";

		$selectQuery = "SELECT * FROM `MaintenanceCycle`";

		$maintenanceCycleList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceCycle',
			array()
		);

		$maintenanceCycleList->addOrderVariant('interval.ASC',  '`MaintenanceCycle`.`interval` ASC');
		$maintenanceCycleList->addOrderVariant('interval.DESC', '`MaintenanceCycle`.`interval` DESC');

		return $maintenanceCycleList;
	}

	/**
	 * Saves MaintenanceCycle information
	 *
	 * If the MaintenanceCycle object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceCycle
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceCycle) {
		$fieldDataMapping = array(
			'id'       => $maintenanceCycle->id,
			'name'     => $maintenanceCycle->name,
			'interval' => $maintenanceCycle->interval
		);

		if ($maintenanceCycle->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceCycle',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceCycle->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceCycle',
				$fieldDataMapping
			);
			$maintenanceCycle->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceCycle->getIdentifier();
	}

	/**
	 * Delete a MaintenanceCycle
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceCycle',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
