<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\util;

/**
 * Datamapper for the MaintenanceGroup model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceGroupMapper extends Mapper {

	/**
	 * MaintenanceGroupMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceGroupMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceGroup');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceGroupMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceGroup by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceGroup`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$maintenanceGroup = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceGroup);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a MaintenanceGroup by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`MaintenanceGroup`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		$maintenanceGroup = $this->getByQuery($query, array($name));
		return $maintenanceGroup;
	}

	/**
	 * Returns a single MaintenanceGroup object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceGroup object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceGroup cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance group with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceGroup = new model\MaintenanceGroup();
		$maintenanceGroup->fillFromRowData($data);

		return $maintenanceGroup;
	}

	/**
	 * Load all MaintenanceGroups
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $checklistId A checklist the group shall be assigned to
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$checklistId = null,
		$customerId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$joins = array();

		if ($checklistId !== null) {
			$joins[] = "
				JOIN MaintenanceChecklist_MaintenanceGroup ON (
					`MaintenanceGroup`.`id` = `MaintenanceChecklist_MaintenanceGroup`.`groupId`
				)
			";

			$conditions->addStatement('Checklist', new sql\WhereStatement(
				'checklistId',
				'=',
				$checklistId,
				'MaintenanceChecklist_MaintenanceGroup'
			));
		}
		if ($customerId !== null) {
			if (!is_array($customerId)) {
				$customerId = array($customerId);
			}

			$subCustomers = array();
			foreach ($customerId as $id) {
				$subCustomers = array_merge(
					$subCustomers,
					CustomerMapper::getInstance()->getSubCustomers($id)
				);
			}
			$customerIds = array_unique(
				array_map(
					'intval',
					array_merge($customerId, $subCustomers)));

			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerIds,
				'MaintenanceGroup'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`MaintenanceGroup`.`id`
				FROM
					`MaintenanceGroup`
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);

		$selectQuery = "SELECT * FROM `MaintenanceGroup`";

		$maintenanceGroupList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceGroup',
			array()
		);

		$maintenanceGroupList->addOrderVariant('name.ASC',  '`MaintenanceGroup`.`name` ASC');
		$maintenanceGroupList->addOrderVariant('name.DESC', '`MaintenanceGroup`.`name` DESC');

		return $maintenanceGroupList;
	}


	/**
	 * Returns the maintenancegroup that belongs to a ticket
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         $hostId
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	public function getMaintenanceGroupByTicket($ticketId) {
		$query = 
			"
			SELECT
				*
			FROM
				`MaintenanceGroup`
			JOIN `MaintenanceGroup_Host` ON (
				`MaintenanceGroup_Host`.`groupId` = `MaintenanceGroup`.`id`
			)
			JOIN `MaintenanceTicketHost` ON (
				`MaintenanceTicketHost`.`hostId` = `MaintenanceGroup_Host`.`hostId`
			)

			WHERE
				`MaintenanceTicketHost`.`ticketId` = ?
			"
		;


		return $this->getByQuery($query, array($ticketId));
	}


	/**
	 * Returns a list of all maintenancegroups that are assigned to a specific host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         $hostId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */

	public function getAllByHost($hostId) {
		$selectQuery = "SELECT * FROM `MaintenanceGroup`";

		$searchQuery = sprintf("
				SELECT
					`MaintenanceGroup_Host`.`groupId` as `id`
				FROM
					`MaintenanceGroup_Host`
				
				LEFT JOIN
					`MaintenanceGroup`
				ON (
					`MaintenanceGroup`.`id` = `MaintenanceGroup_Host`.`groupId`
				)

				WHERE
					`MaintenanceGroup_Host`.`hostId` = %d
			",
			$hostId
		);

		$maintenanceGroupList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceGroup',
			array()
		);

		$maintenanceGroupList->addOrderVariant('name.ASC',  '`MaintenanceGroup`.`name` ASC');
		$maintenanceGroupList->addOrderVariant('name.DESC', '`MaintenanceGroup`.`name` DESC');
		
		return $maintenanceGroupList;
	}

	/**
	 * Saves MaintenanceGroup information
	 *
	 * If the MaintenanceGroup object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceGroup
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceGroup) {
		$fieldDataMapping = array(
			'id'                 => $maintenanceGroup->id,
			'customerId'         => $maintenanceGroup->customerId,
			'maintenanceCycleId' => $maintenanceGroup->maintenanceCycleId,
			'name'               => $maintenanceGroup->name,
			'description'        => $maintenanceGroup->description,
			'dateStart'          => util\TimeUtility::dateOrNull($maintenanceGroup->dateStart)
		);

		if ($maintenanceGroup->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceGroup',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceGroup->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceGroup',
				$fieldDataMapping
			);
			$maintenanceGroup->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceGroup->getIdentifier();
	}

	/**
	 * Delete a MaintenanceGroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceGroup',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
