<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the MaintenanceGroupMember model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceGroupMemberMapper extends Mapper {

	/**
	 * MaintenanceGroupMemberMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceGroupMemberMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceGroupMember');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceGroupMemberMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceGroupMember by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceGroupMember
	 */
	public function getByID($id) {
		$query = '
			SELECT
				*
			FROM
				`MaintenanceGroup_Host`
			WHERE
				`id` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array((int)$id));
	}

	/** 
	 * Returns a single MaintenanceGroupMember object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a MaintenanceGroupMember object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceGroupMember cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceGroupMember
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service group member with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceGroupMember = new model\MaintenanceGroupMember();
		$maintenanceGroupMember->fillFromRowData($data);

		return $maintenanceGroupMember;
	}

	/**
	 * Load all MaintenanceGroupMembers
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$groupId = null,
		$hostId  = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($groupId !== null) {
			$conditions->addStatement('MaintenanceGroup', new sql\WhereStatement(
				'groupId',
				'=',
				$groupId,
				'MaintenanceGroup_Host'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'MaintenanceGroup_Host'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`MaintenanceGroup_Host`.`id`
				FROM
					`MaintenanceGroup_Host`
				JOIN `Host` ON (
					`Host`.`id` = `MaintenanceGroup_Host`.`hostId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `MaintenanceGroup_Host`";

		$maintenanceGroupMemberList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceGroupMember',
			array()
		);

		$maintenanceGroupMemberList->addOrderVariant('hostname.ASC',  '`Host`.`hostname` ASC');

		return $maintenanceGroupMemberList;
	}

	/**
	 * Saves MaintenanceGroupMember information
	 *
	 * If the MaintenanceGroupMember object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $groupMember
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $groupMember) {
		$fieldDataMapping = array(
			'hostId'  => $groupMember->hostId,
			'groupId' => $groupMember->groupId
		);

		$statement = Application::getDbInstance()->prepare("
			INSERT INTO
				`MaintenanceGroup_Host`
			SET
				`hostId`  = :hostId,
				`groupId` = :groupId
		");

		$statement->execute(
			$fieldDataMapping
		);

		return $groupMember->getIdentifier();
	}

	/**
	 * Delete a MaintenanceGroupMember
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceGroup_Host',
			sprintf(
				'`id` = %d',
				$identifier['id']
			)
		);
	}

	/**
	 * Delete members by MaintenanceGroup Id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @returns       void
	 */
	public function deleteByMaintenanceGroupId($groupId) {
		Application::getDbInstance()->delete(
			'MaintenanceGroup_Host',
			sprintf(
				'`groupId` = %d',
				$groupId
			)
		);
	}

	/**
	 * Delete members by Host Id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @returns       void
	 */
	public function deleteByHostId($hostId) {
		Application::getDbInstance()->delete(
			'MaintenanceGroup_Host',
			sprintf(
				'`hostId` = %d',
				$hostId
			)
		);
	}

	/**
	 * Verify whether host is already assigned to a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @param         int $hostId
	 * @returns       bool
	 */
	public function existsByGroupAndHost($groupId, $hostId) {
		$query = '
			SELECT
				*
			FROM
				`MaintenanceGroup_Host`
			WHERE
				`groupId` = ?
				AND
				`hostId` = ?
			LIMIT
				1
		';

		try {
			$this->getByQuery($query, array($groupId, $hostId))->allowRead();
			return true;
		}
		catch (\OutOfBoundsException $e) {
			return false;
		}
	}
}
