<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the MaintenanceState model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceStateMapper extends Mapper {

	/**
	 * MaintenanceStateMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceStateMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceState');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceStateMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceState by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceState
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceState`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$maintenanceState = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceState);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single MaintenanceState object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a MaintenanceState object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceState cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceState
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance state with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceState = new model\MaintenanceState();
		$maintenanceState->fillFromRowData($data);

		return $maintenanceState;
	}

	/**
	 * Load all MaintenanceStates
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`MaintenanceState`.`id`
			FROM
				`MaintenanceState`
		";

		$selectQuery = "SELECT * FROM `MaintenanceState`";

		$maintenanceStateList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceState',
			array()
		);

		$maintenanceStateList->addOrderVariant('id.ASC',  '`MaintenanceState`.`id` ASC');
		$maintenanceStateList->addOrderVariant('id.DESC', '`MaintenanceState`.`id` DESC');

		return $maintenanceStateList;
	}

	/**
	 * Saves MaintenanceState information
	 *
	 * If the MaintenanceState object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceState
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceState) {
		$fieldDataMapping = array(
			'id'   => $maintenanceState->id,
			'name' => $maintenanceState->name
		);

		if ($maintenanceState->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceState',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceState->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceState',
				$fieldDataMapping
			);
			$maintenanceState->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceState->getIdentifier();
	}

	/**
	 * Delete a MaintenanceState
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceState',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
