<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the MaintenanceTag model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceTagMapper extends Mapper {

	/**
	 * MaintenanceTagMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceTagMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceTag');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceTagMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceTag by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceTag
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`MaintenanceTag`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$maintenanceTag = $this->getByQuery($query, array((int)$id));
			$map->set($maintenanceTag);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single MaintenanceTag object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a MaintenanceTag object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceTag cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceTag
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance tag with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenanceTag = new model\MaintenanceTag();
		$maintenanceTag->fillFromRowData($data);

		return $maintenanceTag;
	}

	/**
	 * Load all MaintenanceTags
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $maintenanceCommentId The maintenance comment the tags must belong to
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($maintenanceCommentId = null) {
		$join = array();
		if ($maintenanceCommentId !== null) {
			$join[] = sprintf('
					JOIN `MaintenanceComment_MaintenanceTag` ON (
						`MaintenanceComment_MaintenanceTag`.`tagId` = `MaintenanceTag`.`id`
					)
					JOIN `MaintenanceComment` ON (
						`MaintenanceComment`.`id` = `MaintenanceComment_MaintenanceTag`.`commentId`
						AND
						`MaintenanceComment`.`id` = %d
					)
				',
				$maintenanceCommentId
			);
		}

		$searchQuery = sprintf("
				SELECT
					`MaintenanceTag`.`id`
				FROM
					`MaintenanceTag`
				%s
			",
			implode("\n", $join)
		);

		$selectQuery = "SELECT * FROM `MaintenanceTag`";

		$maintenanceTagList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceTag',
			array()
		);

		return $maintenanceTagList;
	}

	/**
	 * Saves MaintenanceTag information
	 *
	 * If the MaintenanceTag object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $maintenanceTag
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $maintenanceTag) {
		$fieldDataMapping = array(
			'id'          => $maintenanceTag->id,
			'name'        => $maintenanceTag->name,
			'description' => $maintenanceTag->description
		);

		if ($maintenanceTag->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceTag',
				$fieldDataMapping,
				sprintf('id = %d', $maintenanceTag->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceTag',
				$fieldDataMapping
			);
			$maintenanceTag->id = Application::getDbInstance()->lastInsertId();
		}

		return $maintenanceTag->getIdentifier();
	}

	/**
	 * Delete a MaintenanceTag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceTag',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Create an association to a comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $tagId
	 * @param         int $maintenanceCommentId
	 * @returns       void
	 */
	public function createAssociation($tagId, $maintenanceCommentId) {
		Application::getDbInstance()->insert(
			'MaintenanceComment_MaintenanceTag',
			array(
				'commentId' => $maintenanceCommentId,
				'tagId'     => $tagId
			)
		);
	}
}
