<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\util;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the MaintenanceTicketHost model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceTicketHostMapper extends Mapper {

	/**
	 * MaintenanceTicketHostMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceTicketHostMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceTicketHost');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceTicketHostMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceTicketHost by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceTicketHost
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`MaintenanceTicketHost`.*
				FROM
					`MaintenanceTicketHost`
				WHERE
					`MaintenanceTicketHost`.`id` = ?
				LIMIT
					1
			';

			$maintenance = $this->getByQuery($query, array((int)$id));
			$map->set($maintenance);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single MaintenanceTicketHost object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceTicketHost object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceTicketHost cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceTicketHost
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance ticket with these parameters'),
					var_export($params, true)
				)
			);
		}

		$maintenance = new model\MaintenanceTicketHost();
		$maintenance->fillFromRowData($data);

		return $maintenance;
	}

	/**
	 * Load all MaintenanceTicketHosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Zend_Date $dateTo Optional. Minimum update date.
	 * @param         Zend_Date $dateFrom Optional. Maximum update date.
	 * @param         int $hostId Optional. Host to limit results to.
	 * @param         array $states Optional. A list of states the ticket must be in.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$ticketId       = null,
		$hostId         = null,
		$states         = array(),
		$dateFrom       = null,
		$dateTo         = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($ticketId !== null) {
			$conditions->addStatement('Ticket', new sql\WhereStatement(
				'ticketId',
				is_array($ticketId) ? 'IN' : '=',
				$ticketId,
				'MaintenanceTicketHost'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'hostId',
				'=',
				$hostId,
				'MaintenanceTicketHost'
			));
		}

		if (is_array($states) && count($states) > 0) {
			$conditions->addStatement('States', new sql\WhereStatement(
				'stateId',
				'IN',
				$states,
				'MaintenanceTicketHost'
			));
		}

		if ($dateFrom !== null) {
			if (!($dateFrom instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateFrom', new sql\WhereStatement(
				'datetimeChanged',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'MaintenanceTicketHost'
			));
		}

		if ($dateTo !== null) {
			if (!($dateTo instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateTo', new sql\WhereStatement(
				'datetimeChanged',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'MaintenanceTicketHost'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`MaintenanceTicketHost`.`id`
				FROM
					`MaintenanceTicketHost`
				JOIN `Host` ON (
					`Host`.`id` = `MaintenanceTicketHost`.`hostId`
				)

				JOIN `MaintenanceTicket` ON (
					`MaintenanceTicket`.`id` = `MaintenanceTicketHost`.`ticketId`
				)

				%s
			",
			$conditions
		);
		$selectQuery = "SELECT * FROM `MaintenanceTicketHost`";

		$maintenanceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceTicketHost',
			array()
		);

		$maintenanceList->addOrderVariant('hostname.ASC',  '`Host`.`hostname` ASC');
		$maintenanceList->addOrderVariant('hostname.DESC', '`Host`.`hostname` DESC');
		$maintenanceList->addOrderVariant('dateDue.ASC',  '`MaintenanceTicket`.`dateDue` ASC');
		$maintenanceList->addOrderVariant('dateDue.DESC', '`MaintenanceTicket`.`dateDue` DESC');

		return $maintenanceList;
	}

	/**
	 * Saves MaintenanceTicketHost information
	 *
	 * If the MaintenanceTicketHost object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $ticketHost
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $ticketHost) {
		$fieldDataMapping = array(
			'id'              => $ticketHost->id,
			'ticketId'        => $ticketHost->ticketId,
			'hostId'          => $ticketHost->hostId,
			'stateId'         => $ticketHost->stateId,
			'datetimeChanged' => util\TimeUtility::dateOrNull($ticketHost->datetimeChanged)
		);

		if ($ticketHost->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceTicketHost',
				$fieldDataMapping,
				sprintf('id = %d', $ticketHost->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceTicketHost',
				$fieldDataMapping
			);
			$ticketHost->id = Application::getDbInstance()->lastInsertId();
			$this->addQuestions($ticketHost);
		}

		return $ticketHost->getIdentifier();
	}

	/**
	 * Delete a MaintenanceTicketHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'MaintenanceTicketHost',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Assigns the checklist questions for this ticket host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	private function addQuestions($ticketHost) {
		$ticket = $ticketHost->getTicket();

		$questions = array();
		if ($ticket->groupId !== null) {
			$questions = $ticket->getGroup()->getQuestions();
		}

		$questions = array_unique(
			array_merge(
				$questions,
				$ticketHost->getHost()->getQuestions()
			),
			SORT_REGULAR
		);

		foreach ($questions as $question) {
			$answer = new model\MaintenanceChecklistAnswer();
			
			$answer->questionId   = $question->id;
			$answer->ticketHostId = $ticketHost->id;
			$answer->save();
		}
	}
}
