<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\util;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the MaintenanceTicket model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceTicketMapper extends Mapper {

	/**
	 * MaintenanceTicketMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\MaintenanceTicketMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\MaintenanceTicket');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\MaintenanceTicketMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a MaintenanceTicket by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\MaintenanceTicket
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					`MaintenanceTicket`.*
				FROM
					`MaintenanceTicket`
				WHERE
					`MaintenanceTicket`.`id` = ?
				LIMIT
					1
			';

			$ticket = $this->getByQuery($query, array((int)$id));
			$map->set($ticket);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single MaintenanceTicket object identified by the given SQL query
	 *
	 * Runs the given query, initialises a MaintenanceTicket object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the MaintenanceTicket cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\MaintenanceTicket
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find maintenance ticket with these parameters'),
					var_export($params, true)
				)
			);
		}

		$ticket = new model\MaintenanceTicket();
		$ticket->fillFromRowData($data);

		return $ticket;
	}

	/**
	 * Returns the latest ticket
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @returns       null or ch\monitoring\model\MaintenanceTicket
	 */
	public function getLatestGroupMaintenanceTicket($groupId) {
		$query = "
			SELECT
				*
			FROM
				`MaintenanceTicket`
			WHERE
				`groupId` = ?
			ORDER BY
				`dateCreated` DESC
			LIMIT 1
		";

		try {
			return $this->getByQuery($query, array($groupId));
		}
		catch(\OutOfBoundsException $e) {
			return NULL;
		}
	}

	/**
	 * Load all Maintenance tickets
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId Optional. MaintenanceGroup to limit results to.
	 * @param         int $userId Optional. User to limit results to.
	 * @param         Zend_Date $dateCreatedFrom Optional. Minimum creation date.
	 * @param         Zend_Date $dateCreatedTo Optional. Maximum creation date.
	 * @param         Zend_Date $dateDueFrom Optional. Minimum due date.
	 * @param         Zend_Date $dateDueTo Optional. Maximum due date.
	 * @param         bool $isArchived Optional. Whether the tickets shall be archived.
	 * @param         Zend_Date $dateArchivedFrom Optional. Minimum archived date.
	 * @param         Zend_Date $dateArchivedTo Optional. Maximum archived date.
	 * @param         string $permissionName Optional. Defaults to Permission::MAINTENANCE_READ.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$groupId          = null,
		$userId           = null,
		$dateCreatedFrom  = null,
		$dateCreatedTo    = null,
		$dateDueFrom      = null,
		$dateDueTo        = null,
		$isArchived       = null,
		$dateArchivedFrom = null,
		$dateArchivedTo   = null,
		$permissionName   = model\Permission::MAINTENANCE_READ,
		$customerId       = null,
		$hostId           = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();

		if ($groupId !== null) {
			$conditions->addStatement('Group', new sql\WhereStatement(
				'groupId',
				'=',
				$groupId,
				'MaintenanceTicket'
			));
		}

		if ($userId !== null) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'MaintenanceTicket'
			));
		}

		if ($customerId !== null || $hostId !== null) {
			$join[] = "
				JOIN `MaintenanceTicketHost` ON (
					`MaintenanceTicketHost`.`ticketId` = `MaintenanceTicket`.`id`
				)
				JOIN `Host` ON (
					`Host`.`id` = `MaintenanceTicketHost`.`hostId`
				)
			";
		}

		if ($customerId !== null) {
			if (!is_array($customerId)) {
				$customerId = array($customerId);
			}

			$subCustomers = array();
			foreach ($customerId as $id) {
				$subCustomers = array_merge(
					$subCustomers,
					CustomerMapper::getInstance()->getSubCustomers($id)
				);
			}
			$customerIds = array_unique(
				array_map(
					'intval',
					array_merge($customerId, $subCustomers)));
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerIds,
				'MaintenanceTicket'
			));
		}

		if ($hostId !== null) {
			$conditions->addStatement('Host', new sql\WhereStatement(
				'id',
				'=',
				$hostId,
				'Host'
			));
		}

		if ($dateCreatedFrom !== null) {
			if (!($dateCreatedFrom instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateCreatedFrom', new sql\WhereStatement(
				'dateCreated',
				'>=',
				$dateCreatedFrom->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($dateCreatedTo !== null) {
			if (!($dateCreatedTo instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateCreatedTo', new sql\WhereStatement(
				'dateCreated',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($dateDueFrom !== null) {
			if (!($dateDueFrom instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateDueFrom', new sql\WhereStatement(
				'dateDue',
				'>=',
				$dateDueFrom->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($dateDueTo !== null) {
			if (!($dateDueTo instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateDueTo', new sql\WhereStatement(
				'dateDue',
				'<=',
				$dateDueTo->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($isArchived !== null) {
			$conditions->addStatement('Archived', new sql\WhereStatement(
				'dateArchived',
				$isArchived ? 'IS NOT' : 'IS',
				null,
				'MaintenanceTicket',
				false
			));
		}

		if ($dateArchivedFrom !== null) {
			if (!($dateArchivedFrom instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateArchivedFrom', new sql\WhereStatement(
				'dateArchived',
				'>=',
				$dateArchivedFrom->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($dateArchivedTo !== null) {
			if (!($dateArchivedTo instanceof \Zend_Date)) {
				throw new \UnexpectedValueException('Zend_Date expected');
			}

			$conditions->addStatement('dateArchivedTo', new sql\WhereStatement(
				'dateArchived',
				'<=',
				$dateArchivedTo->get(\Zend_Date::ISO_8601),
				'MaintenanceTicket'
			));
		}

		if ($permissionName !== null) {
			// TODO how to filter?
			/*
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Host',
				false
			));
			 */
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`MaintenanceTicket`.`id`
				FROM
					`MaintenanceTicket`
				%s
				%s
			",
			join("\n", $join),
			$conditions
		);
		$selectQuery = "SELECT * FROM `MaintenanceTicket`";

		$maintenanceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\MaintenanceTicket',
			array()
		);

		$maintenanceList->addOrderVariant('dateCreated.ASC',  '`MaintenanceTicket`.`dateCreated` ASC');
		$maintenanceList->addOrderVariant('dateCreated.DESC', '`MaintenanceTicket`.`dateCreated` DESC');

		$maintenanceList->addOrderVariant('dateArchived.ASC',  '`MaintenanceTicket`.`dateArchived` ASC');
		$maintenanceList->addOrderVariant('dateArchived.DESC', '`MaintenanceTicket`.`dateArchived` DESC');

		return $maintenanceList;
	}

	/**
	 * This function returns the number of tickets that matches the condition
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $userId optional
	 * @param         int $isArchived optional
	 * @returns       int
	 * 
	 */
	public function countTickets($userId = null, $isArchived = null) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if(!is_null($userId)) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'MaintenanceTicket'
			));
		}

		if(!is_null($isArchived)) {
			$conditions->addStatement('Archived', new sql\WhereStatement(
				'dateArchived',
				$isArchived ? 'IS' : 'IS NOT',
				NULL,
				'MaintenanceTicket',
				false
			));
		}
		
		$query = sprintf('
			SELECT
				COUNT(*)
			FROM
				`MaintenanceTicket`
			%s
			',
			$conditions
		);
		
		return Application::getDbInstance()->fetchOne($query);
	}

	/**
	 * Saves MaintenanceTicket information
	 *
	 * If the MaintenanceTicket object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $ticket
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $ticket) {
		$fieldDataMapping = array(
			'id'           => $ticket->id,
			'customerId'   => $ticket->customerId,
			'groupId'      => $ticket->groupId,
			'userId'       => $ticket->userId,
			'dateCreated'  => util\TimeUtility::dateOrNull($ticket->dateCreated),
			'dateDue'      => util\TimeUtility::dateOrNull($ticket->dateDue),
			'dateArchived' => util\TimeUtility::dateOrNull($ticket->dateArchived)
		);

		if ($ticket->hasIdentifier()) {
			Application::getDbInstance()->update(
				'MaintenanceTicket',
				$fieldDataMapping,
				sprintf('id = %d', $ticket->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'MaintenanceTicket',
				$fieldDataMapping
			);
			$ticket->id = Application::getDbInstance()->lastInsertId();
		}

		return $ticket->getIdentifier();
	}

	/**
	 * Delete a MaintenanceTicket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'MaintenanceTicket',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}
}
