<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use org\adfinis\framework\db\sql;
use org\adfinis\framework\db;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use ch\monitoring\Application;
use ch\monitoring\util;
use ch\monitoring\db\DataTableIterator;
use ch\monitoring\view\datatable;

/**
 * Datamapper for the NagiosLog model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class NagiosLogMapper extends Mapper {

	/**
	 * NagiosLogMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\NagiosLogMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\NagiosLog');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\NagiosLogMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a NagiosLog by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\NagiosLog
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`NagiosLog`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$nagiosLog = $this->getByQuery($query, array((int)$id));
			$map->set($nagiosLog);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single NagiosLog object identified by the given SQL query
	 *
	 * Runs the given query, initialises a NagiosLog object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the NagiosLog cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\NagiosLog
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find nagios log with these parameters'),
					var_export($params, true)
				)
			);
		}

		$nagiosLog = new model\NagiosLog();
		$nagiosLog->fillFromRowData($data);

		return $nagiosLog;
	}

	/**
	 * Gets all NagiosLog objects corresponding the given
	 * Alert ID
	 *
	 * @throws        \OutOfBoundsException if the NagiosLog cannot be found
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getByAlertID($id) {
		$searchQuery = '
			SELECT
				*
			FROM
				`NagiosLog`
			WHERE
				`alertId` = :alertID
		';
		$selectQuery = sprintf(
			"SELECT * FROM `NagiosLog` WHERE %s",
			db\DBIterator::IDS_PLACEHOLDER
		);
		$nagiosLogList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\NagiosLog',
			array(
				'alertID' => $id
			)
		);
		$nagiosLogList->usePlaceholder(true);
		return $nagiosLogList;
	}

	/**
	 * Load all NagiosLogs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         Zend_Date $dateFrom Optional
	 * @param         Zend_Date $dateTo Optional
	 * @param         Zend_Date $timeFrom Optional
	 * @param         Zend_Date $timeTo Optional
	 * @param         int $customerId Optional
	 * @param         string $hostname Optional
	 * @param         bool $alerted Optional
	 * @param         array $status Optional
	 * @param         int $alertId Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		\Zend_Date $dateFrom = null,
		\Zend_Date $dateTo   = null,
		$timeFrom   = null,
		$timeTo     = null,
		$customerId = null,
		$hostname   = null,
		$alerted    = null,
		$status     = null,
		$alertId    = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();

		if ($dateFrom !== null) {
			$conditions->addStatement('dateFrom', new sql\WhereStatement(
				'timestamp',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'NagiosLog'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('dateTo', new sql\WhereStatement(
				'timestamp',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'NagiosLog'
			));
		}

		if ($timeFrom !== null) {
			$conditions->addStatement('timeFromHour', new sql\WhereStatement(
				'timestamp',
				'>=',
				$timeFrom,
				'NagiosLog',
				false,
				'TIME'
			));
		}

		if ($timeTo !== null) {
			$conditions->addStatement('timeToMinute', new sql\WhereStatement(
				'timestamp',
				'<=',
				$timeTo,
				'NagiosLog',
				false,
				'TIME'
			));
		}

		if ($customerId !== null) {
			$join[] = "
				JOIN `Host` ON (
					`Host`.`hostname` = `NagiosLog`.`hostname`
				)
			";

			$conditions->addStatement('customer', new sql\WhereStatement(
				'customerId',
				'=',
				$customerId,
				'Host'
			));
		}

		if ($hostname !== null) {
			$conditions->addStatement('host', new sql\WhereStatement(
				'hostname',
				'=',
				$hostname,
				'NagiosLog'
			));
		}

		if ($alerted !== null) {
			$conditions->addStatement('alerted', new sql\WhereStatement(
				'processed',
				'=',
				model\NagiosLog::STATE_ALERTED,
				'NagiosLog'
			));
		}

		if ($status !== null) {
			$conditions->addStatement('status', new sql\WhereStatement(
				'processed',
				'IN',
				$status,
				'NagiosLog'
			));
		}

		if ($alertId !== null) {
			$join[] = "
				JOIN `Alert` ON (
					`Alert`.`id` = `NagiosLog`.`alertId`
				)
			";

			$conditions->addStatement('alert', new sql\WhereStatement(
				'alertId',
				'=',
				$alertId,
				'Alert'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`NagiosLog`.`id`
				FROM
					`NagiosLog`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `NagiosLog`";

		$nagiosLogList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\NagiosLog',
			array()
		);

		$nagiosLogList->addOrderVariant('id.DESC',  '`NagiosLog`.`id` DESC');
		$nagiosLogList->addOrderVariant('id.ASC',  '`NagiosLog`.`id` ASC');

		return $nagiosLogList;
	}

	/**
	 * Saves NagiosLog information
	 *
	 * If the NagiosLog object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $nagiosLog
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $nagiosLog) {
		$fieldDataMapping = array(
			'id'           => $nagiosLog->id,
			'timestamp'    => util\TimeUtility::dateOrNull($nagiosLog->timestamp),
			'objectType'   => $nagiosLog->objectType,
			'stateType'    => $nagiosLog->stateType,
			'objectType'   => $nagiosLog->objectType,
			'name'         => $nagiosLog->name,
			'hostname'     => $nagiosLog->hostname,
			'checkCommand' => $nagiosLog->checkCommand,
			'address'      => $nagiosLog->address,
			'output'       => $nagiosLog->output,
			'perfdata'     => $nagiosLog->perfdata,
			'duration'     => $nagiosLog->duration,
			'datetime'     => util\TimeUtility::dateOrNull($nagiosLog->datetime),
			'attempt'      => $nagiosLog->attempt,
			'processed'    => $nagiosLog->processed
		);

		if ($nagiosLog->hasIdentifier()) {
			Application::getDbInstance()->update(
				'NagiosLog',
				$fieldDataMapping,
				sprintf('id = %d', $nagiosLog->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'NagiosLog',
				$fieldDataMapping
			);
			$nagiosLog->id = Application::getDbInstance()->lastInsertId();
		}

		return $nagiosLog->getIdentifier();
	}

	/**
	 * Delete a NagiosLog
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'NagiosLog',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Load all NagiosLog entries for DataTables
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         array  $settings
	 * @param         string $model [optional]
	 * @return        \ch\monitoring\db\DataTableIterator
	 */
	public function getAllForDataTable(
		$settings,
		$model = 'ch\monitoring\model\NagiosLog'
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$join = array();

		$fieldNameMapping = [
			datatable\NagiosLog::STATE             => ['NagiosLog', 'state'],
			datatable\NagiosLog::ID                => ['NagiosLog', 'id'],
			datatable\NagiosLog::TIMESTAMP         => ['NagiosLog', 'timestamp'],
			datatable\NagiosLog::STATE_DESCRIPTION => ['NagiosLog', 'state'],
			datatable\NagiosLog::TYPE              => ['NagiosLog', 'name'],
			datatable\NagiosLog::OUTPUT            => ['NagiosLog', 'output'],
			datatable\NagiosLog::STATUS            => ['NagiosLog', 'processed'],
		];
		$filter = DataTableIterator::getFilter($model, $settings, $fieldNameMapping);

		$dateFrom = null;
		$dateTo   = null;
		if (isset($settings['dateFrom']) && $settings['dateFrom']) {
			$dateFrom   = new \Zend_Date(
				$settings['dateFrom'],
				\Zend_Date::ISO_8601
			);
		}
		if (isset($settings['dateTo']) && $settings['dateTo']) {
			$dateTo     = new \Zend_Date(
				$settings['dateTo'],
				\Zend_Date::ISO_8601
			);

			// add 24h hours, or else it'll be $dateTo 00:00
			$dateTo->add('24', \Zend_Date::HOUR);
		}

		$timeFrom   = $this->getTime($settings['timeFrom']);
		$timeTo     = $this->getTime($settings['timeTo']);

		$customerId = $settings['customerId'] ?: null;
		$hostId     = $settings['hostId'] ?: null;
		$hostname   = $hostId
			? datamapper\HostMapper::getInstance()->getByID($hostId)->allowRead()->hostname
			: null;

		$alerted    = isset($settings['alerted']);
		if ($dateFrom !== null) {
			$conditions->addStatement('dateFrom', new sql\WhereStatement(
				'timestamp',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'NagiosLog'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('dateTo', new sql\WhereStatement(
				'timestamp',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'NagiosLog'
			));
		}

		if ($timeFrom !== null) {
			$conditions->addStatement('timeFromHour', new sql\WhereStatement(
				'timestamp',
				'>=',
				$timeFrom,
				'NagiosLog',
				false,
				'TIME'
			));
		}

		if ($timeTo !== null) {
			$conditions->addStatement('timeToMinute', new sql\WhereStatement(
				'timestamp',
				'<=',
				$timeTo,
				'NagiosLog',
				false,
				'TIME'
			));
		}

		if ($customerId !== null) {
			$customerIds = CustomerMapper::getInstance()->getSubCustomers($customerId);
			$customerIds[] = $customerId;
			$join[] = "
				JOIN `Host` ON (
					`Host`.`hostname` = `NagiosLog`.`hostname`
				)
			";

			$conditions->addStatement('customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerIds,
				'Host'
			));
		}

		if ($hostname !== null) {
			$conditions->addStatement('host', new sql\WhereStatement(
				'hostname',
				'=',
				$hostname,
				'NagiosLog'
			));
		}

		if ($alerted !== null && $alerted) {
			$conditions->addStatement('alerted', new sql\WhereStatement(
				'processed',
				'=',
				model\NagiosLog::STATE_ALERTED,
				'NagiosLog'
			));
		}

		if ((string)$conditions || $filter) {
			$filter = sprintf('WHERE %s',
				join(' AND ', array_filter([$conditions->getSQLConditions(), $filter]))
			);
		}

		$searchQuery = sprintf("
				SELECT
					`NagiosLog`.`id`
				FROM
					`NagiosLog`
				%s
				%s
			",
			implode("\n", $join),
			$filter
		);

		$selectQuery = sprintf(
			"SELECT * FROM `NagiosLog` WHERE %s",
			db\DBIterator::IDS_PLACEHOLDER
		);

		$notificationList = DataTableIterator::create(
			$searchQuery,
			$selectQuery,
			$model
		);
		$notificationList->usePlaceholder(true);
		$notificationList->setSettings($settings);

		$notificationList->addOrderVariant('id.ASC'               , '`NagiosLog`.`id` ASC');
		$notificationList->addOrderVariant('id.DESC'              , '`NagiosLog`.`id` DESC');

		$notificationList->addOrderVariant('timestamp.ASC'        , '`NagiosLog`.`timestamp` ASC');
		$notificationList->addOrderVariant('timestamp.DESC'       , '`NagiosLog`.`timestamp` DESC');

		$notificationList->addOrderVariant('stateDescription.ASC' , '`NagiosLog`.`state` ASC');
		$notificationList->addOrderVariant('stateDescription.DESC', '`NagiosLog`.`state` DESC');

		$notificationList->addOrderVariant('type.ASC'             , '`NagiosLog`.`objectType` ASC');
		$notificationList->addOrderVariant('type.DESC'            , '`NagiosLog`.`objectType` DESC');

		$notificationList->addOrderVariant('status.ASC'           , '`NagiosLog`.`processed` ASC');
		$notificationList->addOrderVariant('status.DESC'          , '`NagiosLog`.`processed` DESC');

		return $notificationList;
	}

	/**
	 * Check the time and return it
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         string $time The Time
	 * @returns       string
	 */
	private function getTime($time) {
		if (!preg_match('/^(\d{1,2}):(\d{2})$/', $time, $matches)) {
			return null;
		}

		$hours   = $matches[1] > 23 ? 23 : $matches[1];
		$minutes = $matches[2] > 59 ? 59 : $matches[2];

		return sprintf("%02d:%02d", $hours, $minutes);
	}
}
