<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the Permission model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class PermissionMapper extends Mapper {

	/**
	 * PermissionMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\PermissionMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Permission');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\PermissionMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Permission by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Permission
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Permission`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$permission = $this->getByQuery($query, array((int)$id));
			$map->set($permission);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Permission by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Permission
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Permission`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single Permission object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Permission object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Permission cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Permission
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find permission with these parameters'),
					var_export($params, true)
				)
			);
		}

		$permission = new model\Permission();
		$permission->fillFromRowData($data);

		return $permission;
	}

	/**
	 * Load all Permissions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($roleId = null) {
		$join = array();

		if ($roleId !== null) {
			$join[] = sprintf("
					JOIN `Role_Permission` ON (
						`Role_Permission`.`permissionId` = `Permission`.`id`
						AND
						`Role_Permission`.`roleId` = %d
					)
				",
				$roleId
			);
		}

		$searchQuery = sprintf("
				SELECT
					`Permission`.`id`
				FROM
					`Permission`
				%s
			",
			implode("\n", $join)
		);

		$selectQuery = "SELECT * FROM `Permission`";

		$permissionList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Permission',
			array()
		);

		$permissionList->addOrderVariant('name.ASC',  '`Permission`.`name` ASC');
		$permissionList->addOrderVariant('name.DESC', '`Permission`.`name` DESC');

		return $permissionList;
	}

	/**
	 * Saves Permission information
	 *
	 * If the Permission object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $permission
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $permission) {
		$fieldDataMapping = array(
			'id'          => $permission->id,
			'name'        => $permission->name,
			'description' => $permission->description
		);

		if ($permission->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Permission',
				$fieldDataMapping,
				sprintf('id = %d', $permission->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Permission',
				$fieldDataMapping
			);
			$permission->id = Application::getDbInstance()->lastInsertId();
		}

		return $permission->getIdentifier();
	}

	/**
	 * Delete a Permission
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'Permission',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Create an association to a role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $permissionId
	 * @param         int $roleId
	 * @returns       void
	 */
	public function createAssociation($permissionId, $roleId) {
		Application::getDBInstance()->insert(
			'Role_Permission',
			array(
				'permissionId' => $permissionId,
				'roleId'       => $roleId
			)
		);
	}

	/**
	 * Delete the associations of permissions to a role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $roleId
	 */
	public function deleteAssociations($roleId) {
		Application::getDBInstance()->delete(
			'Role_Permission',
			sprintf('`roleId` = %d', $roleId)
		);
	}
}
