<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\util\TimeUtility;

/**
 * Datamapper for the Report model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ReportMapper extends Mapper {

	/**
	 * ReportMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ReportMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Report');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\ReportMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Report by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Report
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Report`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$report = $this->getByQuery($query, array((int)$id));
			$map->set($report);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single Report object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Report object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Report cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Report
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find report with these parameters'),
					var_export($params, true)
				)
			);
		}

		$report = new model\Report();
		$report->fillFromRowData($data);

		return $report;
	}

	/**
	 * Load all Reports
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId Optional
	 * @param         int $customerId Optional
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @param         bool $isTemplate Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$userId = null,
		$customerId = null,
		$dateFrom = null,
		$dateTo = null,
		$isTemplate = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($userId !== null) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'Report'
			));
		}

		if ($customerId !== null) {
			$customerIds = CustomerMapper::getInstance()->getSubCustomers($customerId);
			$customerIds[] = $customerId;

			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				$customerIds,
				'Report'
			));
		}

		if ($dateFrom !== null) {
			$conditions->addStatement('Date from', new sql\WhereStatement(
				'timestamp',
				'>=',
				$dateFrom->get(\Zend_Date::ISO_8601),
				'Report'
			));
		}

		if ($dateTo !== null) {
			$conditions->addStatement('Date to', new sql\WhereStatement(
				'timestamp',
				'<=',
				$dateTo->get(\Zend_Date::ISO_8601),
				'Report'
			));
		}

		if ($isTemplate !== null) {
			$conditions->addStatement('Template', new sql\WhereStatement(
				'isTemplate',
				'=',
				$isTemplate,
				'Report'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`Report`.`id`
				FROM
					`Report`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `Report`";

		$reportList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Report',
			array()
		);

		$reportList->addOrderVariant('timestamp.ASC',  '`Report`.`timestamp` ASC');
		$reportList->addOrderVariant('timestamp.DESC', '`Report`.`timestamp` DESC');

		return $reportList;
	}

	/**
	 * Saves Report information
	 *
	 * If the Report object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $data
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $report) {
		$fieldDataMapping = array(
			'id'         => $report->id,
			'timestamp'  => TimeUtility::dateOrNull($report->timestamp),
			'userId'     => $report->userId,
			'customerId' => $report->customerId,
			'dateFrom'   => TimeUtility::dateOrNull($report->dateFrom),
			'dateTo'     => TimeUtility::dateOrNull($report->dateTo),
			'isTemplate' => $report->isTemplate
		);

		if ($report->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Report',
				$fieldDataMapping,
				sprintf('id = %d', $report->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Report',
				$fieldDataMapping
			);
			$report->id = Application::getDbInstance()->lastInsertId();
		}

		return $report->getIdentifier();
	}

	/**
	 * Delete a Report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'Report',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
