<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;
use org\adfinis\framework\db\sql;

/**
 * Datamapper for the Right model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class RightMapper extends Mapper {

	/**
	 * RoleMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\RightMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Right');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\RightMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Role by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Role
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = "
				SELECT
					`ucr`.`id` as `id`,
					`ucr`.`roleId` as `roleId`,
					`uc`.`userId` as `userId`,
					`uc`.`customerId` as `customerId`,
					`Role`.`name` as `roleName`,
					`User`.`name` as `userName`,
					`Customer`.`name` as `customerName`
				FROM
					`User_Role_Customer` as `ucr`

				JOIN `User_Customer` as `uc` ON (
					`uc`.`id` = `ucr`.`userCustomerId`
				)
				
				JOIN
					`Role` as `Role`
				ON
					`Role`.`id` = `ucr`.`roleId`

				JOIN
					`User` as `User`
				ON
					`User`.`id` = `uc`.`userId`

				LEFT JOIN
					`Customer` as `Customer`
				ON
					`Customer`.`id` = `uc`.`customerId`
				WHERE
					`ucr`.`id` = ?
				LIMIT 1
			";

			$right = $this->getByQuery($query, array((int)$id));
			$map->set($right);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Stores the object right
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $right) {
		$db = Application::getDbInstance();
		$query = "
			SELECT
				`id`
			FROM
				`User_Customer`
			WHERE
				`customerId` = ?
				AND
				`userId` = ?
		";

		$statement = $db->prepare($query);
		$statement->execute(array($right->customerId, $right->userId));
		$data = $statement->fetch();
		
		if(!$data) {
			$db->insert(
				'User_Customer',
				array('customerId' => $right->customerId, 'userId' => $right->userId)
			);

			$id = $db->lastInsertId();
		}
		else {
			$id = $data['id'];
		}

		$fieldDataMapping = array(
			'roleId'         => $right->roleId,
			'userCustomerId' => $id
		);

		if ($right->hasIdentifier()) {
			$db->update(
				'User_Role_Customer',
				$fieldDataMapping,
				sprintf('id = %d', $right->id)
			);
		}
		else {
			$db->insert(
				'User_Role_Customer',
				$fieldDataMapping
			);
			$right->id = $db->lastInsertId();
		}

		return $right->getIdentifier();
	}

	/**
	 * Delete a Right
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();
		
		$right = $this->getByID($identifier['id']);
		$right->allowRead();

		Application::getDbInstance()->delete(
			'User_Role_Customer',
			sprintf('`id` = %d', $identifier['id'])
		);

		$this->fixUserCustomers();

		Application::getDbInstance()->commit();
	}

	/**
	 * Tidies up the User_Customer table
	 *
	 * Deletes all entries in User_Customer which don't have any Role assigned
	 * and are not for the principal customer of the associated user.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by denis:   Probably this should be solved with a trigger
	 * @returns       void
	 */
	public function fixUserCustomers() {
		$query = "
			DELETE
				`User_Customer`
			FROM
				`User_Customer`
			JOIN `User` ON (
				`User`.`id` = `User_Customer`.`userId`
			)
			LEFT JOIN `User_Role_Customer` ON(
				`User_Role_Customer`.`userCustomerId` = `User_Customer`.`id`
			)
			WHERE
				`User_Customer`.`customerId` != `User`.`customerId`
				AND
				`User_Role_Customer`.`id` IS NULL
		";

		Application::getDbInstance()->query($query);
	}

	/**
	 * Returns a single Right object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Right object and returns it
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Role cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Right
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find right with these parameters'),
					var_export($params, true)
				)
			);
		}

		$role = new model\Right();
		$role->fillFromRowData($data);

		return $role;
	}

	/**
	 * Returns a list of rights which are assigned to a specific customer
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *
	 * @param         int $customerId
	 * @param         int $userId optional - default current user
	 * @param         string $authenticationMethod
	 *
	 * @return        array ch\monitoring\model\Right
	 * @todo          remove the ->toArray() from the return
	 */
	public function getAll($customerId = null, $userId = null, $authenticationMethod = model\User::AUTH_METHOD_LOCAL) {
		$conditionGroup = new sql\WhereGroup();
		$conditionGroup->setOperand('AND');

		if (!is_null($customerId)) {
			$conditionGroup->addStatement(
				'customerRestriction',
				new sql\WhereStatement('customerId', '=', (int)$customerId, 'uc')
			);
		}

		if (!is_null($userId)) {
			$conditionGroup->addStatement(
				'userRestriction',
				new sql\WhereStatement('userId', '=', (int)$userId, 'uc')
			);
		}

		if (!is_null($authenticationMethod)) {
			$conditionGroup->addStatement(
				'Authentication',
				new sql\WhereStatement(
					'authenticationMethod',
					'=',
					$authenticationMethod,
					'User',
					false
				)
			);
		}

		$selectQuery = '
			SELECT
				`ucr`.`id` as `id`,
				`ucr`.`roleId` as `roleId`,
				`uc`.`userId` as `userId`,
				`uc`.`customerId` as `customerId`,
				`Role`.`name` as `roleName`,
				`User`.`name` as `userName`,
				`Customer`.`name` as `customerName`
			FROM
				`User_Role_Customer` as `ucr`

			JOIN `User_Customer` as `uc` ON (
				`uc`.`id` = `ucr`.`userCustomerId`
			)
			
			JOIN
				`Role` as `Role`
			ON
				`Role`.`id` = `ucr`.`roleId`

			JOIN
				`User` as `User`
			ON
				`User`.`id` = `uc`.`userId`

			LEFT JOIN
				`Customer` as `Customer`
			ON
				`Customer`.`id` = `uc`.`customerId`
		';

		$searchQuery = sprintf('
				SELECT
					`ucr`.`id`
				FROM
					`User_Role_Customer` as `ucr`
				
				JOIN `User_Customer` as `uc` ON (
					`uc`.`id` = `ucr`.`userCustomerId`
				)
				
				JOIN `User` ON (
					`User`.`id` = `uc`.`userId`
				)

				%s
			',
			$conditionGroup
		);

		
		$roleList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Right'
		);

		return $roleList->toArray();
	}

	/**
	 * Returns a list of roles which are assigned to a specific customer or their upper customer
	 *
	 * The returned list may not be distinct! Use util\IteratorUtil::getKeyValueArray
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @param         int $userId optional - default current user
	 * @return        array ch\monitoring\model\Right
	 * @throws        \RuntimeException if depth is too deep
	 *
	 * @todo          add a parameter "mode" for selecting the rights relative to a customerId OR userId
	 */
	public function getAllUpperRights($customerId, $userId = null, $counter = 0) {
		$counter++;
		$returnValue = array();
		
		
		if($counter > 10) {
			throw new \RuntimeException(tr('Depth is higher than 10'));
		}

		$customer = CustomerMapper::getInstance()->getByID($customerId);
		$customer->allowRead();
		if(!is_null($customer->parentId)) {
			$returnValue = array_merge($this->getAllUpperRights(
					$customer->parentId,
					$userId,
					$counter
				),
				$returnValue
			);
		}
		
		return array_merge($this->getAll(
				$customerId,
				$userId,
				null
			),
			$returnValue
		);
	}
}
