<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Role model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class RoleMapper extends Mapper {

	/**
	 * RoleMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\RoleMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Role');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\RoleMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Role by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Role
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Role`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$role = $this->getByQuery($query, array((int)$id));
			$map->set($role);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Role by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\Role
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Role`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single Role object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Role object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Role cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Role
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find role with these parameters'),
					var_export($params, true)
				)
			);
		}

		$role = new model\Role();
		$role->fillFromRowData($data);

		return $role;
	}

	/**
	 * Load all Roles
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`Role`.`id`
			FROM
				`Role`
		";

		$selectQuery = "SELECT * FROM `Role`";

		$roleList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Role',
			array()
		);

		$roleList->addOrderVariant('name.ASC',  '`Role`.`name` ASC');
		$roleList->addOrderVariant('name.DESC', '`Role`.`name` DESC');

		return $roleList;
	}

	/**
	 * Returns a list of Roles
	 * 
	 * The list will only contain the roles that the current user is allowed
	 * to set on the specific customer
	 * 
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @todo          replace the subquery with a left join
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllByCustomer($customerId, $excludeGlobalRoles = true) {
		$customerIds = CustomerMapper::getInstance()->getAllParentIDs($customerId);
		
		$globalRestriction = '';
		if($excludeGlobalRoles) {
			$globalRestriction = "
				WHERE 0 = (
					SELECT
						COUNT(*)
					FROM
						`Permission`
					JOIN `Role_Permission` ON (
						`Role_Permission`.`permissionId` = `Permission`.`id`
					)

					WHERE
						`Permission`.`globalOnly` = 1
						AND
						`Role_Permission`.`roleId` = `Role_Role`.`assignedRoleId`
				)";
		}

		$searchQuery = sprintf("
				SELECT DISTINCT
					`Role_Role`.`assignedRoleId` as `id`
				
				FROM
					`Role_Role`

				JOIN `User_Customer` ON (
					`User_Customer`.`userId` = ?
					AND 
					`User_Customer`.`customerId` IN (%s)
				)

				JOIN `User_Role_Customer` ON (
					`User_Role_Customer`.`userCustomerId` = `User_Customer`.`id`
					AND
					`User_Role_Customer`.`roleId` = `Role_Role`.`roleId`
				)

				JOIN `Role` ON (
					`Role`.`id` = `Role_Role`.`roleId`
				) 

				JOIN `Role` as `AssignedRole` ON (
					`AssignedRole`.`id` = `Role_Role`.`assignedRoleId`
				)
				
				%s	
			",
			implode(',', $customerIds),
			$globalRestriction
		);

		$selectQuery = "SELECT * FROM `Role`";

		$roleList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Role',
			array(Application::getUser()->id)
		);

		$roleList->addOrderVariant('name.ASC',  '`AssignedRole`.`name` ASC');
		$roleList->addOrderVariant('name.DESC', '`AssignedRole`.`name` DESC');

		return $roleList;
	}

	/**
	 * Load all assigned Roles
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getAssignedRoleIds($roleId) {
		$query = "
			SELECT
				`rr`.`assignedRoleId`
			FROM
				`Role` as `r`

			JOIN
				`Role_Role` as `rr`
			ON (
				rr.roleId = `r`.`id`
			)

			WHERE
				`r`.`id` = ?
		";

		return Application::getDbInstance()->fetchCol($query, array($roleId));
	}


	/**
	 * Saves Role information
	 *
	 * If the Role object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $role
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $role) {
		$fieldDataMapping = array(
			'id'          => $role->id,
			'name'        => $role->name,
			'description' => $role->description,
		);

		if ($role->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Role',
				$fieldDataMapping,
				sprintf('id = %d', $role->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Role',
				$fieldDataMapping
			);
			$role->id = Application::getDbInstance()->lastInsertId();
		}

		return $role->getIdentifier();
	}

	/**
	 * Delete a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		$this->removeAllRoles($identifier['id']);

		Application::getDbInstance()->delete(
			'Role',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Assigns a role to a user
	 * @author        matthieu.riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @deprecated    it should be handled of the Rightmodel/mapper
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $roleId
	 * @param         int $userId
	 * @param         int $customerId
	 * @returns       void
	 */
	public function assign($roleId, $userId, $customerId) {
		UserMapper::getInstance()->addCustomer($userId, $customerId);

		$query = '
			INSERT INTO `User_Role_Customer`
				(`userCustomerId`, `roleId`)
				SELECT
					`User_Customer`.`id`, ?
				FROM
					`User_Customer`
				WHERE
					`User_Customer`.`userId` = ?
					AND
					`User_Customer`.`customerId` = ?
		';

		$stm = Application::getDBInstance()->prepare($query);
		$stm->execute(array($roleId, $userId, $customerId));
	}

	/**
	 * Unassigns all roles from user
	 * @deprecated    it should be handled of the Rightmodel/mapper
	 * @author        matthieu.riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId
	 * @returns       void
	 */
	public function unassign($userId) {
		$query = "
			DELETE
				`User_Role_Customer`
			FROM
				`User_Role_Customer`
			
			JOIN `User_Customer` ON (
				`User_Customer`.`id` = `User_Role_Customer`.`userCustomerId`
			)

			WHERE
				`User_Customer`.`userId` = ?
		";

		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute(array($userId));
	}


	/**
	 * Assignes a role to a role
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $roleId
	 * @param         int $assignedRoleId
	 * @todo          check if we're allowed to set that role
	 * @returns       void
	 */
	public function addRole($roleId, $assignedRoleId) {
		$data = array(
			'roleId' => (int)$roleId,
			'assignedRoleId' => (int)$assignedRoleId
		);

		Application::getDbInstance()->insert(
			'Role_Role',
			$data
		);
	}

	/**
	 * Removes the assigned role
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $roleId
	 * @param         int $customerId
	 * @returns       void
	 */
	public function removeRole($roleId, $assignedRoleId) {
		$data = array(
			'`roleId` = ?' => (int)$userId,
			'`assignedRoleId` = ?' => (int)$assignedRoleId
		);

		Application::getDbInstance()->delete(
			'Role_Role',
			$data
		);
	}

	/**
	 * Removes all the relations to any roles
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $roleId
	 * @returns       void
	 */
	public function removeAllRoles($roleId) {
		$data = array(
			'`roleId` = ?' => (int)$roleId
		);

		Application::getDbInstance()->delete(
			'Role_Role',
			$data
		);
	}
}
