<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the SLAAction model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SLAActionMapper extends Mapper {

	/**
	 * SLAActionMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\SLAActionMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\SLAAction');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\SLAActionMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a SLAAction by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\SLAAction
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`SLAAction`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$slaAction = $this->getByQuery($query, array((int)$id));
			$map->set($slaAction);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single SLAAction object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a SLAAction object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the SLAAction cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\SLAAction
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find SLA action with these parameters'),
					var_export($params, true)
				)
			);
		}

		$slaAction = new model\SLAAction();
		$slaAction->fillFromRowData($data);

		return $slaAction;
	}

	/**
	 * Load all SLAActions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`SLAAction`.`id`
			FROM
				`SLAAction`
		";

		$selectQuery = "SELECT * FROM `SLAAction`";

		$slaActionList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\SLAAction',
			array()
		);

		$slaActionList->addOrderVariant('name.ASC',  '`SLAAction`.`name` ASC');
		$slaActionList->addOrderVariant('name.DESC', '`SLAAction`.`name` DESC');

		return $slaActionList;
	}

	/**
	 * Saves SLAAction information
	 *
	 * If the SLAAction object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $slaAction
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $slaAction) {
		$fieldDataMapping = array(
			'id'          => $slaAction->id,
			'name'        => $slaAction->name,
			'description' => $slaAction->description
		);

		if ($slaAction->hasIdentifier()) {
			Application::getDbInstance()->update(
				'SLAAction',
				$fieldDataMapping,
				sprintf('id = %d', $slaAction->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'SLAAction',
				$fieldDataMapping
			);
			$slaAction->id = Application::getDbInstance()->lastInsertId();
		}

		return $slaAction->getIdentifier();
	}

	/**
	 * Delete a SLAAction
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'SLAAction',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Get the SLA actions which are assigned to an infofeed
	 *
	 * @param         int $infofeedId
	 * @returns       oaf\DbIterator
	 */
	public function getByInfofeedId($infofeedId) {

		$searchQuery = '
			SELECT
				`SLAAction`.`id`
			FROM
				`SLAAction`
			JOIN `Infofeed_NotificationType` ON (
				`Infofeed_NotificationType`.`slaActionId` = `SLAAction`.`id`
			)
			WHERE
				`Infofeed_NotificationType`.`infofeedId` = :infofeedId
		';

		$selectQuery = "SELECT * from `SLAAction`";

		$slaActionList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\SLAAction',
			array('infofeedId' => $infofeedId)
		);

		return $slaActionList;
	}

	/**
	 * Get the SLA actions which are assigned to infofeeds of a user
	 *
	 * @param         int $userID
	 * @returns       oaf\DbIterator
	 */
	public function getByInfofeedUserId($userID) {

		$searchQuery = '
			SELECT DISTINCT
				`slaa`.`id`
			FROM
				`SLAAction` AS `slaa`
			JOIN `Infofeed_NotificationType` as `int` ON (
				`int`.`slaActionId` = `slaa`.`id`
			)
			JOIN `Infofeed` AS `i` ON (
				`int`.`infofeedId` = `i`.`id`
			)
			JOIN `User` AS `u`
			WHERE
				`u`.`id` = :userID
		';

		$selectQuery = "SELECT * from `SLAAction`";

		$slaActionList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\SLAAction',
			array('userID' => $userID)
		);

		return $slaActionList;
	}
}
