<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the SLA model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SLAMapper extends Mapper {

	/**
	 * SLAMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\SLAMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\SLAMapper');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\SLAMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a SLA by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`SLA`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$sla = $this->getByQuery($query, array($id));
			$map->set($sla);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a SLA by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`SLA`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/**
	 * Returns a single SLA object identified by the given SQL query
	 *
	 * Runs the given query, initialises a SLA object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the SLA cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\SLA
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find sla with these parameters'),
					var_export($params, true)
				)
			);
		}

		$sla = new model\SLA();
		$sla->fillFromRowData($data);

		return $sla;
	}

	/**
	 * Load all SLAs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName Defautls to slas read
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$permissionName = model\Permission::SLAS_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$customerNull = new sql\WhereStatement(
			'customerId',
			'IS',
			null,
			'SLA',
			false
		);

		$customers = Application::getUser()->getCustomersWithPermission(
			$permissionName
		);

		$customerValid = new sql\WhereStatement(
			'customerId',
			$customers === null ? 'IS' : 'IN',
			$customers,
			'SLA',
			false
		);

		$conditions->addStatement('Permission', new sql\OrStatement(
			$customerNull,
			$customerValid
		));

		$searchQuery = sprintf("
				SELECT
					`SLA`.`id`
				FROM
					`SLA`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `SLA`";

		$slaList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\SLA',
			array()
		);

		$slaList->addOrderVariant('name.ASC',  '`SLA`.`name` ASC');
		$slaList->addOrderVariant('name.DESC', '`SLA`.`name` DESC');

		return $slaList;
	}

	/**
	 * Saves SLA information
	 *
	 * If the SLA object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $sla
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $sla) {
		$fieldDataMapping = array(
			'id'              => $sla->id,
			'customerId'      => $sla->customerId,
			'name'            => $sla->name,
			'description'     => $sla->description,
			'delayCycle'      => $sla->delayCycle,
			'delayRestart'    => $sla->delayRestart,
			'timeout'         => $sla->timeout,
			'minAvailability' => $sla->minAvailability,
			'isDeleted'       => $sla->isDeleted
		);

		if ($sla->hasIdentifier()) {
			Application::getDbInstance()->update(
				'SLA',
				$fieldDataMapping,
				sprintf('id = %d', $sla->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'SLA',
				$fieldDataMapping
			);
			$sla->id = Application::getDbInstance()->lastInsertId();
		}

		return $sla->getIdentifier();
	}

	/**
	 * Delete a SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			Application::getDbInstance()->delete(
				'SLA',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					tr("This SLA cannot be deleted because of dependencies")
				);
			}
			else {
				throw $e;
			}
		}
	}

	/**
	 * Assign objects which use an SLA to a new SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $oldSlaId
	 * @param         int $newSlaId
	 * @returns       void
	 */
	public function batchAssign($oldSlaId, $newSlaId) {
		// Customers
		Application::getDBInstance()->update(
			'Customer',
			array(
				'slaId' => $newSlaId
			),
			sprintf('slaId = %d', $oldSlaId)
		);

		// Hosts
		Application::getDBInstance()->update(
			'Host',
			array(
				'slaId' => $newSlaId
			),
			sprintf('slaId = %d', $oldSlaId)
		);

		// Services
		Application::getDBInstance()->update(
			'HostServiceInstance',
			array(
				'slaId' => $newSlaId
			),
			sprintf('slaId = %d', $oldSlaId)
		);
	}
}
