<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ServiceArgument model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceArgumentMapper extends Mapper {

	/**
	 * ServiceArgumentMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ServiceArgumentMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ServiceArgument');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\ServiceArgumentMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ServiceArgument by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ServiceArgument
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`ServiceArgument`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$serviceArgument = $this->getByQuery($query, array((int)$id));
			$map->set($serviceArgument);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single ServiceArgument object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a ServiceArgument object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ServiceArgument cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ServiceArgument
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service argument with these parameters'),
					var_export($params, true)
				)
			);
		}

		$serviceArgument = new model\ServiceArgument();
		$serviceArgument->fillFromRowData($data);

		return $serviceArgument;
	}

	/**
	 * Load all ServiceArguments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceId The service the argument must be assigned to (optional)
	 * @param         string $permissionName Defaults to Permission::SERVICES_READ
	 * @param         string $type
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$serviceId = null,
		$permissionName = model\Permission::SERVICES_READ,
		$type = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($serviceId !== null) {
			$conditions->addStatement('Service', new sql\WhereStatement(
				'serviceId',
				'=',
				$serviceId,
				'ServiceArgument'
			));
		}

		if ($type !== null) {
			$conditions->addStatement('Type', new sql\WhereStatement(
				'type',
				'=',
				$type,
				'ServiceArgument'
			));
		}

		if ($permissionName !== null) {
			$customerNull = new sql\WhereStatement(
				'customerId',
				'IS',
				null,
				'Service',
				false
			);

			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$customerValid = new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'Service',
				false
			);

			$conditions->addStatement('Permission', new sql\OrStatement(
				$customerNull,
				$customerValid
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ServiceArgument`.`id`
				FROM
					`ServiceArgument`
				JOIN `Service` ON (
					`Service`.`id` = `ServiceArgument`.`serviceId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `ServiceArgument`";

		$serviceArgumentList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ServiceArgument',
			array()
		);

		$serviceArgumentList->addOrderVariant('position.ASC',  '`ServiceArgument`.`position` ASC');

		return $serviceArgumentList;
	}

	/**
	 * Saves ServiceArgument information
	 *
	 * If the ServiceArgument object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $serviceiArgument
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $serviceArgument) {
		$fieldDataMapping = array(
			'id'          => $serviceArgument->id,
			'serviceId'   => $serviceArgument->serviceId,
			'position'    => $serviceArgument->position,
			'option'      => $serviceArgument->option,
			'value'       => $serviceArgument->value,
			'description' => $serviceArgument->description,
			'type'        => $serviceArgument->type
		);

		if ($serviceArgument->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ServiceArgument',
				$fieldDataMapping,
				sprintf('id = %d', $serviceArgument->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ServiceArgument',
				$fieldDataMapping
			);
			$serviceArgument->id = Application::getDbInstance()->lastInsertId();
		}

		return $serviceArgument->getIdentifier();
	}

	/**
	 * Delete a ServiceArgument
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		Application::getDbInstance()->delete(
			'ServiceArgument',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->delete(
			'HostServiceArgument',
			sprintf('`serviceArgumentId` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Delete ServiceArguments by Service id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceId
	 * @returns       void
	 */
	public function deleteByServiceId($serviceId) {
		Application::getDbInstance()->beginTransaction();

		$serviceArguments = $this->getAll($serviceId);
		foreach ($serviceArguments as $serviceArgument) {
			$this->delete($serviceArgument->getIdentifier());
		}

		Application::getDbInstance()->commit();
	}
}
