<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ServiceGroup model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceGroupMapper extends Mapper {

	/**
	 * ServiceGroupMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ServiceGroupMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ServiceGroup');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\ServiceGroupMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ServiceGroup by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ServiceGroup
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`ServiceGroup`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$serviceGroup = $this->getByQuery($query, array((int)$id));
			$map->set($serviceGroup);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a ServiceGroup by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @returns       ch\monitoring\model\ServiceGroup
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`ServiceGroup`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		$serviceGroup = $this->getByQuery($query, array($name));
		return $serviceGroup;
	}

	/** 
	 * Returns a single ServiceGroup object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a ServiceGroup object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ServiceGroup cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ServiceGroup
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service group with these parameters'),
					var_export($params, true)
				)
			);
		}

		$serviceGroup = new model\ServiceGroup();
		$serviceGroup->fillFromRowData($data);

		return $serviceGroup;
	}

	/**
	 * Load all ServiceGroups
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName Defaults to services read
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$permissionName = model\Permission::SERVICES_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$customerNull = new sql\WhereStatement(
			'customerId',
			'IS',
			null,
			'ServiceGroup',
			false
		);

		$customers = Application::getUser()->getCustomersWithPermission(
			$permissionName
		);

		$customerValid = new sql\WhereStatement(
			'customerId',
			$customers === null ? 'IS' : 'IN',
			$customers,
			'ServiceGroup',
			false
		);

		$conditions->addStatement('Permission', new sql\OrStatement(
			$customerNull,
			$customerValid
		));

		$searchQuery = sprintf("
				SELECT
					`ServiceGroup`.`id`
				FROM
					`ServiceGroup`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `ServiceGroup`";

		$serviceGroupList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ServiceGroup',
			array()
		);

		$serviceGroupList->addOrderVariant('name.ASC',  '`ServiceGroup`.`name` ASC');
		$serviceGroupList->addOrderVariant('name.DESC', '`ServiceGroup`.`name` DESC');

		return $serviceGroupList;
	}

	/**
	 * Saves ServiceGroup information
	 *
	 * If the ServiceGroup object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $serviceGroup
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $serviceGroup) {
		$fieldDataMapping = array(
			'id'          => $serviceGroup->id,
			'customerId'  => $serviceGroup->customerId,
			'name'        => $serviceGroup->name,
			'description' => $serviceGroup->description
		);

		if ($serviceGroup->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ServiceGroup',
				$fieldDataMapping,
				sprintf('id = %d', $serviceGroup->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ServiceGroup',
				$fieldDataMapping
			);
			$serviceGroup->id = Application::getDbInstance()->lastInsertId();
		}

		return $serviceGroup->getIdentifier();
	}

	/**
	 * Delete a ServiceGroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->beginTransaction();

		ServiceGroupMemberMapper::getInstance()->deleteByServiceGroupId(
			$identifier['id']
		);

		Application::getDbInstance()->delete(
			'ServiceGroup',
			sprintf('`id` = %d', $identifier['id'])
		);

		Application::getDbInstance()->commit();
	}

	/**
	 * Assigns a service group to a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceGroupId
	 * @param         int $hostId
	 * @returns       void
	 */
	public function assignToHost($serviceGroupId, $hostId) {
		$serviceGroup = $this->getByID($serviceGroupId)->allowRead();
		$host         = HostMapper::getInstance()->getByID($hostId)->allowRead();

		$members = $serviceGroup->getMembers(model\Permission::HOSTS_READ);
		if (count($members) == 0) {
			return;
		}

		foreach ($members as $member) {
			if (HostServiceMapper::getInstance()->existsByHostAndService(
					$hostId,
					$member->serviceId
			)) {
				continue;
			}

			$hostService = new model\HostService();
			$hostService->allowWrite(model\Permission::HOSTS_WRITE);

			$hostService->hostId          = $hostId;
			$hostService->serviceId       = $member->serviceId;
			$hostService->slaId           = $member->slaId;
			$hostService->IPv4            = $member->IPv4;
			$hostService->IPv6            = $member->IPv6;
			$hostService->isPassive       = $host->isSatellite || $host->satelliteId !== null;
			$hostService->isEnabled       = $member->isEnabled;
			$hostService->hasTrouble      = false;
			$hostService->forAvailability = false;

			$hostService->save();
		}
	}
}
