<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the ServiceGroupMember model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceGroupMemberMapper extends Mapper {

	/**
	 * ServiceGroupMemberMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ServiceGroupMemberMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\ServiceGroupMember');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\ServiceGroupMemberMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a ServiceGroupMember by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\ServiceGroupMember
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`ServiceGroupMember`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$serviceGroupMember = $this->getByQuery($query, array((int)$id));
			$map->set($serviceGroupMember);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single ServiceGroupMember object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a ServiceGroupMember object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the ServiceGroupMember cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\ServiceGroupMember
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service group member with these parameters'),
					var_export($params, true)
				)
			);
		}

		$serviceGroupMember = new model\ServiceGroupMember();
		$serviceGroupMember->fillFromRowData($data);

		return $serviceGroupMember;
	}

	/**
	 * Load all ServiceGroupMembers
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceGroupId Optional
	 * @param         string $permissionName Defaults to Permission::SERVICES_READ
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$serviceGroupId = null,
		$permissionName = model\Permission::SERVICES_READ
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($serviceGroupId !== null) {
			$conditions->addStatement('ServiceGroup', new sql\WhereStatement(
				'groupId',
				'=',
				$serviceGroupId,
				'ServiceGroupMember'
			));
		}

		if ($permissionName !== null) {
			$customerNull = new sql\WhereStatement(
				'customerId',
				'IS',
				null,
				'ServiceGroup',
				false
			);

			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$customerValid = new sql\WhereStatement(
				'customerId',
				$customers === null ? 'IS' : 'IN',
				$customers,
				'ServiceGroup',
				false
			);

			$conditions->addStatement('Permission', new sql\OrStatement(
				$customerNull,
				$customerValid
			));
		}

		$searchQuery = sprintf("
				SELECT
					`ServiceGroupMember`.`id`
				FROM
					`ServiceGroupMember`
				JOIN `ServiceGroup` ON (
					`ServiceGroup`.`id` = `ServiceGroupMember`.`groupId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `ServiceGroupMember`";

		$serviceGroupMemberList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\ServiceGroupMember',
			array()
		);

		$serviceGroupMemberList->addOrderVariant('id.ASC',  '`ServiceGroupMember`.`id` ASC');

		return $serviceGroupMemberList;
	}

	/**
	 * Saves ServiceGroupMember information
	 *
	 * If the ServiceGroupMember object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $serviceGroupMember
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $serviceGroupMember) {
		$fieldDataMapping = array(
			'id'        => $serviceGroupMember->id,
			'groupId'   => $serviceGroupMember->groupId,
			'serviceId' => $serviceGroupMember->serviceId
		);

		if ($serviceGroupMember->hasIdentifier()) {
			Application::getDbInstance()->update(
				'ServiceGroupMember',
				$fieldDataMapping,
				sprintf('id = %d', $serviceGroupMember->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'ServiceGroupMember',
				$fieldDataMapping
			);
			$serviceGroupMember->id = Application::getDbInstance()->lastInsertId();
		}

		return $serviceGroupMember->getIdentifier();
	}

	/**
	 * Delete a ServiceGroupMember
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'ServiceGroupMember',
			sprintf('`id` = %d', $identifier['id'])
		);
	}

	/**
	 * Delete members by ServiceGroup Id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceGroupId
	 * @returns       void
	 */
	public function deleteByServiceGroupId($serviceGroupId) {
		Application::getDbInstance()->beginTransaction();

		$serviceGroups = $this->getAll($serviceGroupId);
		foreach ($serviceGroups as $serviceGroup) {
			$this->delete($serviceGroup->getIdentifier());
		}

		Application::getDbInstance()->commit();
	}

	/**
	 * Verify whether service is already assigned to a group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $groupId
	 * @param         int $serviceId
	 * @returns       bool
	 */
	public function existsByGroupAndService($groupId, $serviceId) {
		$query = '
			SELECT
				*
			FROM
				`ServiceGroupMember`
			WHERE
				`groupId` = ?
				AND
				`serviceId` = ?
			LIMIT
				1
		';

		try {
			$this->getByQuery($query, array($groupId, $serviceId))->allowRead();
			return true;
		}
		catch (\OutOfBoundsException $e) {
			return false;
		}
	}
}
