<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\db\sql;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\exceptions\UserException;

use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Service model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceMapper extends Mapper {
	/**
	 * ServiceMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\ServiceMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Service');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\ServiceMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Service by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Service
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(__CLASS__);
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Service`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$service = $this->getByQuery($query, array((int)$id));
			$map->set($service);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a Service by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $name
	 * @returns       ch\monitoring\model\Service
	 */
	public function getByName($name) {
		$query = '
			SELECT
				*
			FROM
				`Service`
			WHERE
				`name` = ?
			LIMIT
				1
		';

		return $this->getByQuery($query, array($name));
	}

	/** 
	 * Returns a single Service object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a Service object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Service cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Service
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find service with these parameters'),
					var_export($params, true)
				)
			);
		}

		$service = new model\Service();
		$service->fillFromRowData($data);

		return $service;
	}

	/**
	 * Load all Services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId The host id the desired services must be assigned to (optional)
	 * @param         int $serviceGroupId The service group the desired services must be assigned to (optional)
	 * @param         int $satelliteId Return services which are assigned to the given Satellite or its Hosts (optional)
	 * @param         bool $nrpeOnly Return (non-)nrpe Services (optional)
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$hostId         = null,
		$serviceGroupId = null,
		$satelliteId    = null,
		$nrpe           = null,
		$commandId      = null,
		$permissionName = model\Permission::SERVICES_READ
	) {
		$join = array();
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($hostId !== null) {
			$join[] = sprintf("
					JOIN `HostService` ON (
						`HostService`.`serviceId` = `Service`.`id`
						AND
						`HostService`.`hostId` = %d
					)
				",
				$hostId
			);
		}

		if ($serviceGroupId !== null) {
			$join[] = sprintf("
					JOIN `HostServiceGroup` on (
						`HostServiceGroupMember`.`serviceId` = `Service`.`id`
						AND
						`HostServiceGroupMember`.`groupId` = %d
					)
				",
				$serviceGroupId
			);
		}

		if ($satelliteId !== null) {
			$join[] = sprintf("
					JOIN `HostService`, `Host` ON (
						`HostService`.`serviceId` = `Service`.`id`
						AND
						`Host`.`id` = `HostService`.`hostId`
						AND (
							`Host`.`id` = %1$d
							OR
							`Host`.`satelliteId` = %1$d
						)
					)
				",
				$satelliteId
			);
		}

		if ($nrpe !== null) {
			$conditions->addStatement('Nrpe', new sql\WhereStatement(
				'nrpe',
				'=',
				$nrpe == true ? 'TRUE' : 'FALSE',
				'Service'
			));
		}

		if ($commandId !== null) {
			$conditions->addStatement('Command', new sql\WhereStatement(
				'commandId',
				'=',
				$commandId,
				'Service'
			));
		}

		$customerNull = new sql\WhereStatement(
			'customerId',
			'IS',
			null,
			'Service',
			false
		);


		try {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$customerValid = new sql\WhereStatement(
				'customerId',
				'IN',
				$customers,
				'Service',
				false
			);

			$conditions->addStatement('Permission', new sql\OrStatement(
				$customerNull,
				$customerValid
			));
		}
		catch(\ForbiddenException $e) {
			$conditions->addStatement('Permission', new sql\OrStatement(
				$customerNull
			));
		}
		


		$searchQuery = sprintf("
				SELECT
					`Service`.`id`
				FROM
					`Service`
				%s
				%s
			",
			implode("\n", $join),
			$conditions
		);

		$selectQuery = "SELECT * FROM `Service`";

		$serviceList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Service',
			array()
		);

		$serviceList->addOrderVariant('name.ASC',  '`Service`.`name` ASC');
		$serviceList->addOrderVariant('name.DESC', '`Service`.`name` DESC');

		return $serviceList;
	}

	/**
	 * Saves Service information
	 *
	 * If the Service object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $service
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $service) {
		$fieldDataMapping = array(
			'id'               => $service->id,
			'customerId'       => $service->customerId,
			'commandId'        => $service->commandId,
			'slaId'       => $service->slaId,
			'name'             => $service->name,
			'description'      => $service->description,
			'maxCheckAttempts' => $service->maxCheckAttempts,
			'checkInterval'    => $service->checkInterval,
			'retryInterval'    => $service->retryInterval,
			'freshness'        => $service->freshness,
			'isNrpe'           => $service->isNrpe,
			'isDeleted'        => $service->isDeleted
		);

		if ($service->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Service',
				$fieldDataMapping,
				sprintf('id = %d', $service->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Service',
				$fieldDataMapping
			);
			$service->id = Application::getDbInstance()->lastInsertId();
		}

		return $service->getIdentifier();
	}

	/**
	 * Delete a Service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			Application::getDbInstance()->delete(
				'Service',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(sprintf(
					"%s: %s",
					tr("This service cannot be deleted because of dependencies"),
					$e->getMessage()
				));
			}
			else {
				throw $e;
			}
		}
	}
}
