<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\SortingIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the Setting model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SettingMapper extends Mapper {

	/**
	 * SettingMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\SettingMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Setting');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\SettingMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Setting by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Setting
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('key' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Setting`
				WHERE
					`key` = ?
				LIMIT
					1
			';

			$setting = $this->getByQuery($query, array($id));
			$map->set($setting);
		}

		return $map->get(array('key' => $id));
	}

	/**
	 * Returns a single Setting object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a Setting object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Setting cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Setting
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find setting with these parameters'),
					var_export($params, true)
				)
			);
		}

		$setting = new model\Setting();
		$setting->fillFromRowData($data);

		return $setting;
	}

	/**
	 * Load all Settings
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll() {
		$searchQuery = "
			SELECT
				`Setting`.`key`
			FROM
				`Setting`
		";

		$selectQuery = "SELECT * FROM `Setting`";

		$settingList = SortingIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Setting',
			array()
		);

		$settingList->addOrderVariant('key.ASC',  '`Setting`.`key` ASC');
		$settingList->addOrderVariant('key.DESC', '`Setting`.`key` DESC');

		return $settingList;
	}

	/**
	 * Saves Setting information
	 *
	 * If the Setting object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $setting
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $setting) {
		$statement = Application::getDbInstance()->prepare("
			INSERT INTO
				`Setting`
			SET
				`key`   = :key,
				`value` = :value
			ON DUPLICATE KEY UPDATE
				`value` = :value
		");

		$statement->execute(array(
			'key'   => $setting->key,
			'value' => $setting->value
		));

		return $setting->getIdentifier();
	}

	/**
	 * Delete a Setting
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'Setting',
			sprintf('`key` = %d', $identifier['id'])
		);
	}
}
