<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the Timeframe model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class TimeframeMapper extends Mapper {

	/**
	 * TimeframeMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\TimeframeMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\Timeframe');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\TimeframeMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a Timeframe by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\Timeframe
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`Timeframe`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$timeframe = $this->getByQuery($query, array($id));
			$map->set($timeframe);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Returns a single Timeframe object identified by the given SQL query
	 *
	 * Runs the given query, initialises a Timeframe object and returns it
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the Timeframe cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\Timeframe
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find Timeframe with these parameters'),
					var_export($params, true)
				)
			);
		}

		$timeframe = new model\Timeframe();
		$timeframe->fillFromRowData($data);

		return $timeframe;
	}

	/**
	 * Load all Timeframes
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $slaId
	 * @param         bool $current  Whether to get only the currently
	 *                               active timeframe(s) or not
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($slaId, $current = false) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
	
		$conditions->addStatement('SLA', new sql\WhereStatement(
			'slaId',
			'=',
			(int)$slaId,
			'Timeframe',
			false
		));

		// If only the currently active timeframe(s) shall be returned
		if ($current) {
			// A quick overview of the following query; There are two cases:
			//
			// CASE1 (first condition):
			//
			//              NOW()
			//                |
			//
			// timeFrom               timeTo
			// 08:00                  20:00
			//
			// CASE 2 (second codition):
			//
			// NOW()
			//   |
			//
			//          timeTo                   timeFrom
			//          08:00                    20:00
			$currentSql = "
				(
					(
						`Timeframe`.`timeFrom` <= `Timeframe`.`timeTo`
						AND
						TIME_FORMAT(NOW(), '%H:%i') BETWEEN
						TIME_FORMAT(`Timeframe`.`timeFrom`, '%H:%i')
						AND
						TIME_FORMAT(`Timeframe`.`timeTo`, '%H:%i')
					)
					OR (
						TIME_FORMAT(`Timeframe`.`timeFrom`, '%H:%i') > TIME_FORMAT(`Timeframe`.`timeTo`, '%H:%i')
						AND (
							TIME_FORMAT(NOW(), '%H:%i') > TIME_FORMAT(`Timeframe`.`timeFrom`, '%H:%i')
							OR
							TIME_FORMAT(NOW(), '%H:%i') < TIME_FORMAT(`Timeframe`.`timeTo`, '%H:%i')
						)
					)
				)
			";

			if (empty($conditions)) {
				$currentSql = "WHERE ".$currentSql;
			}
			else {
				$currentSql = " AND ".$currentSql;
			}
		}
		else {
			$currentSql = "";
		}

		$searchQuery = sprintf("
				SELECT
					`Timeframe`.`id`
				FROM
					`Timeframe`
				%s
				%s
			",
			$conditions,
			$currentSql
		);

		$selectQuery = "SELECT * FROM `Timeframe`";

		$timeframeList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\Timeframe',
			array()
		);

		$timeframeList->addOrderVariant('name.ASC',      '`Timeframe`.`id`       ASC');
		$timeframeList->addOrderVariant('name.DESC',     '`Timeframe`.`id`       DESC');
		$timeframeList->addOrderVariant('timeFrom.ASC',  '`Timeframe`.`timeFrom` ASC');
		$timeframeList->addOrderVariant('timeFrom.DESC', '`Timeframe`.`timeFrom` DESC');

		return $timeframeList;
	}

	/**
	 * Saves Timeframe information
	 *
	 * If the Timeframe object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $timeframe
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $timeframe) {
		$fieldDataMapping = array(
			'id'           => $timeframe->id,
			'monday'       => (bool)$timeframe->monday,
			'tuesday'      => (bool)$timeframe->tuesday,
			'wednesday'    => (bool)$timeframe->wednesday,
			'thursday'     =>(bool)$timeframe->thursday,
			'friday'       => (bool)$timeframe->friday,
			'saturday'     => (bool)$timeframe->saturday,
			'sunday'       => (bool)$timeframe->sunday,
		
			'timeFrom'     => $timeframe->timeFrom,
			'timeTo'       => $timeframe->timeTo,

			'customerId'   => $timeframe->customerId,
			'slaId'        => $timeframe->slaId
		);

		if ($timeframe->hasIdentifier()) {
			Application::getDbInstance()->update(
				'Timeframe',
				$fieldDataMapping,
				sprintf('id = %d', $timeframe->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'Timeframe',
				$fieldDataMapping
			);
			$timeframe->id = Application::getDbInstance()->lastInsertId();
		}

		return $timeframe->getIdentifier();
	}

	/**
	 * Delete a Timeframe
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			Application::getDbInstance()->delete(
				'Timeframe',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					tr("This Timeframe cannot be deleted because of dependencies")
				);
			}
			else {
				throw $e;
			}
		}
	}
}
