<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\db\SortingPolicyIterator;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Datamapper for the TimeframeRule model.
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class TimeframeRuleMapper extends Mapper {

	/**
	 * TimeframeRuleMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\TimeframeRuleMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\TimeframeRule');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\TimeframeRuleMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a TimeframeRule by id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\TimeframeRule
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`TimeframeRule`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$rule = $this->getByQuery($query, array((int)$id));
			$map->set($rule);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single TimeframeRule object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a TimeframeRule object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the TimeframeRule cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\TimeframeRule
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find TimeframeRule with these parameters'),
					var_export($params, true)
				)
			);
		}

		$rule = new model\TimeframeRule();
		$rule->fillFromRowData($data);

		return $rule;
	}

	/**
	 * Load all TimeframeRules
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $timeframeId.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll($timeframeId) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($timeframeId !== null) {
			$conditions->addStatement('Timeframe', new sql\WhereStatement(
				'timeframeId',
				'=',
				$timeframeId,
				'TimeframeRule'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`TimeframeRule`.`id`
				FROM
					`TimeframeRule`
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `TimeframeRule`";

		$list = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\TimeframeRule',
			array()
		);

		$list->addOrderVariant('delay.ASC',  '`TimeframeRule`.`delay` ASC');
		$list->addOrderVariant('delay.DESC', '`TimeframeRule`.`delay` DESC');

		return $list;
	}

	/**
	 * Saves TimeframeRule information
	 *
	 * If the TimeframeRule object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $rule
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $rule) {
		$fieldDataMapping = array(
			'id'           => $rule->id,
			'timeframeId'  => $rule->timeframeId,
			'actionId'     => $rule->actionId,
			'delay'        => $rule->delay
		);

		if ($rule->hasIdentifier()) {
			Application::getDbInstance()->update(
				'TimeframeRule',
				$fieldDataMapping,
				sprintf('id = %d', $rule->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'TimeframeRule',
				$fieldDataMapping
			);
			$rule->id = Application::getDbInstance()->lastInsertId();
		}

		return $rule->getIdentifier();
	}

	/**
	 * Delete a TimeframeRule
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'TimeframeRule',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
