<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the UserLogin model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class UserLoginMapper extends Mapper {

	/**
	 * UserLoginMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\UserLoginMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\UserLogin');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\UserLoginMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a UserLogin by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\UserLogin
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`UserLogin`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$userLogin = $this->getByQuery($query, array((int)$id));
			$map->set($userLogin);
		}

		return $map->get(array('id' => $id));
	}

	/** 
	 * Returns a single UserLogin object identified by the given SQL query
	 *   
	 * Runs the given query, initialises a UserLogin object and returns it
	 *   
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the UserLogin cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\UserLogin
	 */  
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find user with these parameters'),
					var_export($params, true)
				)
			);
		}

		$userLogin = new model\UserLogin();
		$userLogin->fillFromRowData($data);

		return $userLogin;
	}

	/**
	 * Load all UserLogins
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$userId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($userId !== null) {
			$conditions->addStatement('User', new sql\WhereStatement(
				'userId',
				'=',
				$userId,
				'UserLogin'
			));
		}

		$searchQuery = sprintf("
				SELECT
					`UserLogin`.`id`
				FROM
					`UserLogin`
				JOIN `User` ON (
					`User`.`id` = `UserLogin`.`userId`
				)
				%s
			",
			$conditions
		);

		$selectQuery = "SELECT * FROM `UserLogin`";

		$userLoginList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\UserLogin',
			array()
		);

		return $userLoginList;
	}

	/**
	 * Saves UserLogin information
	 *
	 * If the UserLogin object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $userLogin
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $userLogin) {
		$fieldDataMapping = array(
			'id'        => $userLogin->id,
			'userId'    => $userLogin->userId,
			'timestamp' => util\TimeUtility::dateOrNull($userLogin->timestamp),
			'IP'        => $userLogin->IP
		);

		if ($userLogin->hasIdentifier()) {
			Application::getDbInstance()->update(
				'UserLogin',
				$fieldDataMapping,
				sprintf('id = %d', $userLogin->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'UserLogin',
				$fieldDataMapping
			);
			$userLogin->id = Application::getDbInstance()->lastInsertId();
		}

		return $userLogin->getIdentifier();
	}

	/**
	 * Delete a UserLogin
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @returns       void
	 */
	public function delete($identifier) {
		Application::getDbInstance()->delete(
			'UserLogin',
			sprintf('`id` = %d', $identifier['id'])
		);
	}
}
