<?php

namespace ch\monitoring\datamapper;

use org\adfinis\framework\exceptions\UserException;
use org\adfinis\framework\datamapper\Mapper;
use org\adfinis\framework\datamapper\IdentityMap;
use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;
use ch\monitoring\db\SortingPolicyIterator;

/**
 * Datamapper for the User model.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class UserMapper extends Mapper {

	/**
	 * UserMapper object to implement singleton
	 *
	 * @var            ch\monitoring\datamapper\UserMapper $singleton
	 */
	private static $singleton;

	/**
	 * Class constructor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct() {
		parent::__construct('ch\monitoring\model\User');
	}

	/**
	 * Implements a singleton
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\hplus\datamapper\UserMapper
	 */
	public static function getInstance() {
		if (!self::$singleton) {
			self::$singleton = new self();
		}

		return self::$singleton;
	}

	/**
	 * Load a User by id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $id
	 * @returns       ch\monitoring\model\User
	 */
	public function getByID($id) {
		$map = IdentityMap::getInstance(get_called_class());
		if (!$map->exists(array('id' => $id))) {
			$query = '
				SELECT
					*
				FROM
					`User`
				WHERE
					`id` = ?
				LIMIT
					1
			';

			$user = $this->getByQuery($query, array((int)$id));
			$map->set($user);
		}

		return $map->get(array('id' => $id));
	}

	/**
	 * Load a User by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $name
	 * @param         array $authenticationMethods
	 * @returns       ch\monitoring\model\User
	 */
	public function getByName(
		$name, $authenticationMethods=null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');
		$conditions->addStatement(
			'Name',
			new sql\WhereStatement(
				'name',
				'=',
				$name,
				'User',
				false
			)
		);
		if(!is_null($authenticationMethods)) {
			if (!is_array($authenticationMethods)) {
				$authenticationMethods = array($authenticationMethods);
			}
			$conditions->addStatement(
				'Authentication',
				new sql\WhereStatement(
					'authenticationMethod',
					'IN',
					$authenticationMethods,
					'User',
					false
				)
			);
		}
		$user = $this->getByCondition($conditions);
		if ($user->getCount() > 0) {
			$user->rewind();
			$user = $user->current();
			return $user;
		}
		else {
			throw new \OutOfBoundsException(
				sprintf(
					'%s (%s %s): %s',
					tr('Could not find user with name'),
					tr('using'),
					var_export($authenticationMethods, true),
					var_export($name, true)
				)
			);
		}
	}

	/**
	 * Returns a single User object identified by the given SQL query
	 *
	 * Runs the given query, initialises a User object and returns it
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \OutOfBoundsException if the User cannot be found
	 * @param         string $query
	 * @param         array  $params
	 * @returns       ch\monitoring\model\User
	 */
	protected function getByQuery($query, $params) {
		$statement = Application::getDbInstance()->prepare($query);
		$statement->execute($params);

		$data = $statement->fetch();

		if (!$data) {
			throw new \OutOfBoundsException(
				sprintf(
					'%s: %s',
					tr('Could not find user with these parameters'),
					var_export($params, true)
				)
			);
		}

		$user = new model\User();
		$user->fillFromRowData($data);

		return $user;
	}

	/**
	 * Returns a DBIterator of users which can be found by a SQLCondition
	 *
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *
	 * @param         sql\WhereGroup $conditions optional
	 * @param         array $joins optional
	 * @param         array $paramaters optional
	 * @param         string $permissionName A permission to restrict users to
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	protected function getByCondition(
		sql\WhereGroup $conditions = null,
		Array $joins = array(),
		Array $parameters = array(),
		$permissionName = null
	) {
		if (is_null($conditions)) {
			$conditions = new sql\WhereGroup();
			$conditions->setOperand('AND');
		}

		if (!is_null($permissionName)) {
			$customers = Application::getUser()->getCustomersWithPermission(
				$permissionName
			);

			$joins[] = "
				JOIN `User_Customer` ON (
					`User_Customer`.`userId` = `User`.`id` 
				)
			";

			$conditions->addStatement('Permission', new sql\WhereStatement(
				'customerId',
				'IN',
				$customers,
				'User_Customer',
				false
			));
		}
		$searchQuery = sprintf("
				SELECT DISTINCT
					`User`.`id`
				FROM
					`User`
				%s
				%s
			",
			implode("\n", $joins),
			$conditions
		);
		$selectQuery = "SELECT * FROM `User`";
		$list = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\User',
			$parameters
		);
		$list->addOrderVariant('id.ASC',        '`User`.`id` ASC');
		$list->addOrderVariant('id.DESC',       '`User`.`id` DESC');
		$list->addOrderVariant('name.ASC',      '`User`.`name` ASC');
		$list->addOrderVariant('name.DESC',     '`User`.`name` DESC');
		$list->addOrderVariant('fullName.ASC',  '`User`.`fullName` ASC');
		$list->addOrderVariant('fullName.DESC', '`User`.`fullName` DESC');

		return $list;
	}

	/**
	 * Returns a DBIterator for all users which a specific user is allowed to "see"
	 * 
	 * By "see" is meant that it contains all users from all customers that 
	 * the specific user is assigned to
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         User $user
	 * @param         string $authenticationMethod Defaults to AUTH_METHOD_LOCAL
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllByUser($user, $authenticationMethod = model\User::AUTH_METHOD_LOCAL) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		$customers = $user->getCustomersWithPermission(
			model\Permission::USERS_READ
		);

		if(!is_null($authenticationMethod)) {
			$conditions->addStatement(
				'Authentication',
				new sql\WhereStatement(
					'authenticationMethod',
					'=',
					$authenticationMethod,
					'User',
					false
				)
			);
		}

		$primaryCustomers = new sql\WhereStatement(
			'customerId',
			'IN',
			$customers,
			'User',
			false
		);

		$indirectCustomers = new sql\WhereStatement(
			'customerId',
			'IN',
			$customers,
			'User_Customer',
			false
		);

		$conditions->addStatement('Permission', new sql\OrStatement(
			$primaryCustomers,
			$indirectCustomers
		));

		$join = array("
			LEFT JOIN `User_Customer` ON (
				`User_Customer`.`userId` = `User`.`id` 
			)
		");

		return $this->getByCondition($conditions, $join);
	}

	/**
	 * Load all Users
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName A permission to restrict the resutl set to
	 * @param         bool $isEnabled Whether to return enabled/disabled users only
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAll(
		$permissionName = null,
		$isEnabled = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($isEnabled !== null) {
			$conditions->addStatement('Enabled', new sql\WhereStatement(
				'isEnabled',
				'=',
				$isEnabled,
				'User'
			));
		}

		return $this->getByCondition($conditions, array(), array(), $permissionName);
	}

	/**
	 * Saves User information
	 *
	 * If the User object is new, it is INSERTed, otherwise UPDATEd.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $user
	 * @returns       array
	 */
	public function save(\org\adfinis\framework\model\Entity $user) {
		$fieldDataMapping = array(
			'id'                   => $user->id,
			'name'                 => $user->name,
			'fullName'             => $user->fullName,
			'passwordHash'         => $user->passwordHash,
			'passwordSalt'         => $user->passwordSalt,
			'phone'                => $user->phone,
			'sms'                  => $user->sms,
			'email'                => $user->email,
			'jabber'               => $user->jabber,
			'mattermost'           => $user->mattermost,
			'authenticationMethod' => $user->authenticationMethod,
			'isEnabled'            => $user->isEnabled,
			'isDeleted'            => $user->isDeleted,
			'customerId'           => $user->customerId
		);
		if ($user->hasIdentifier()) {
			Application::getDbInstance()->update(
				'User',
				$fieldDataMapping,
				sprintf('id = %d', $user->id)
			);
		}
		else {
			Application::getDbInstance()->insert(
				'User',
				$fieldDataMapping
			);
			$user->id = Application::getDbInstance()->lastInsertId();
		}
		$map = IdentityMap::getInstance(get_called_class());
		$map->set($user);

		return $user->getIdentifier();
	}

	/**
	 * Delete a User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $identifier
	 * @throws        UserException
	 * @returns       void
	 */
	public function delete($identifier) {
		try {
			Application::getDbInstance()->beginTransaction();

			$query = '
				DELETE FROM `AlertComment`
				WHERE
				`AlertComment`.`userId` = ?
			';
			$statement = Application::getDbInstance()->prepare($query);
			$statement->execute(array(
				$identifier['id']
			));
			$query = '
				DELETE FROM `AlertLog`
				WHERE
				`AlertLog`.`userId` = ?
			';
			$statement = Application::getDbInstance()->prepare($query);
			$statement->execute(array(
				$identifier['id']
			));
			$query = '
				UPDATE
				`Alert` `al`
				SET
				`al`.`acknowledgedBy` = NULL
				WHERE
				`al`.`acknowledgedBy` = ?
';
			$statement = Application::getDbInstance()->prepare($query);
			$statement->execute(array(
				$identifier['id']
			));

			Application::getDbInstance()->delete(
				'User',
				sprintf('`id` = %d', $identifier['id'])
			);

			Application::getDbInstance()->commit();
		}
		catch (\Zend_Db_Statement_Exception $e) {
			if ($e->getCode() == 23000) {
				throw new UserException(
					sprintf(
						"%s: %s",
						tr("This user cannot be deleted because of dependencies. Probably you want to disable it."),
						$e->getMessage()
					)
				);
			}
			else {
				throw $e;
			}
		}

	}

	/**
	 * Count the users
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId Optional
	 * @returns       int
	 * @todo by denis:   Doesn't work yet because of customerId
	 */
	public function count(
		$customerId = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if ($customerId !== null) {
			$conditions->addStatement('Customer', new sql\WhereStatement(
				'customerId',
				'IN',
				array_merge(
					array($customerId),
					datamapper\CustomerMapper::getInstance()->getSubCustomers(
						$customerId
					)
				),
				'User'
			));
		}

		$query = sprintf('
				SELECT
					count(*) AS `count`
				FROM
					`User`
				%s
			',
			$conditions
		);

		$stmt = Application::getDBInstance()->query($query);

		return $stmt->fetchOne();
	}

	/**
	 * Returns all users which have a certain permission on a certain customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId Null for any customer
	 * @param         bool $isEnabled Whether the user shall be enabled
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllByPermission(
		$permissionName,
		$customerId,
		$isEnabled = null
	) {
		$conditions = new sql\WhereGroup();
		$conditions->setOperand('AND');

		if(!is_null($customerId)) {
			$customerValid = new sql\WhereStatement(
				'customerId',
				is_array($customerId) ? 'IN' : '=',
				$customerId,
				'User_Customer',
				false
			);

			$conditions->addStatement('Permission', $customerValid);
		}

		if ($isEnabled !== null) {
			$conditions->addStatement('Enabled', new sql\WhereStatement(
				'isEnabled',
				'=',
				$isEnabled,
				'User'
			));
		}

		$permission = PermissionMapper::getInstance()->getByName(
			$permissionName
		);

		$conditions->addStatement('PermissionFlag', new sql\WhereStatement(
			'permissionId',
			'=',
			$permission->id,
			'Role_Permission'
		));

		$searchQuery = sprintf("
				SELECT DISTINCT
					`User`.`id`
				FROM
					`User`
				JOIN `User_Customer` ON (
					`User_Customer`.`userId` = `User`.`id`
				)
				JOIN `User_Role_Customer` ON (
					`User_Role_Customer`.`userCustomerId` = `User_Customer`.`id`
				)
				JOIN `Role_Permission` ON (
					`Role_Permission`.`roleId` = `User_Role_Customer`.`roleId`
				)
				%s
			",
			$conditions
		);
		$selectQuery = "SELECT * FROM `User`";

		$userList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\User',
			array()
		);

		$userList->addOrderVariant('name.ASC',  '`User`.`name` ASC');
		$userList->addOrderVariant('name.DESC', '`User`.`name` DESC');

		return $userList;
	}

	/**
	 * Returns all users which are assigned to a customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllByCustomer($customerId) {
		$searchQuery = "
			SELECT DISTINCT
				`userId` as `id`
			FROM
				`User_Customer`
			LEFT JOIN
				`User`
			ON
				`User`.`id` = `User_Customer`.`userId`
			WHERE
				`User_Customer`.`customerId` = ?
		";

		$selectQuery = "SELECT * FROM `User`";

		$userList = SortingPolicyIterator::create(
			$searchQuery,
			$selectQuery,
			'ch\monitoring\model\User',
			array($customerId)
		);

		$userList->addOrderVariant('name.ASC',  '`User`.`name` ASC');
		$userList->addOrderVariant('name.DESC', '`User`.`name` DESC');

		return $userList;
	}

	/**
	 * Assignes an user to a company
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $userId
	 * @param         int $customerId
	 * @returns       void
	 */
	public function addCustomer($userId, $customerId) {
		$query = '
			INSERT IGNORE INTO
				`User_Customer`
			SET
				`userId`     = ?,
				`customerId` = ?
		';

		$statement = Application::getDBInstance()->prepare($query);
		$statement->execute(array($userId, $customerId));
	}

	/**
	 * Removes the relation between customer and user
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $userId
	 * @param         int $customerId
	 * @returns       void
	 */
	public function removeCustomer($userId, $customerId) {
		$data = array(
			'`userId` = ?'     => $userId,
			'`customerId` = ?' => $customerId
		);

		Application::getDbInstance()->delete(
			'User_Customer',
			$data
		);
	}

	/**
	 * Removes all the relations to any customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $userId
	 * @returns       void
	 */
	public function removeAllCustomer($userId) {
		$data = array(
			'`userId` = ?' => $userId
		);

		Application::getDbInstance()->delete(
			'User_Customer',
			$data
		);
	}

	/**
	 * Set the default role of the given customer for the given user
	 *
	 * @param         \org\adfinis\framework\model\Entity $user
	 * @param         \org\adfinis\framework\model\Entity $customer
	 * @throws        UserException  if the default role is global
	 * @returns       void
	 */
	public function setDefaultRole($user, $customer) {
		$role = RoleMapper::getInstance()->getById(
			$customer->defaultRoleId
		);
		$role->allowRead();
		$rootCustomerId = Application::getUser()->getACL()->getRootCustomerIdentifier();
		if($rootCustomerId != $customer->id) {
			if($role->isGlobal()) {
				throw new UserException(
					sprintf(
						tr("The customer %s has a global role assigned as default role"),
						$customer->name
					)
				);
			}
		}
		$right = new model\Right();
		$right->customerId = $customer->id;
		$right->roleId = $role->id;
		$right->userId = $user->id;
		$right->allowWrite(
			model\Permission::USERS_WRITE
		);
		$right->getMapper()->save($right);
	}
}
