<?php

namespace ch\monitoring\db;

use ch\monitoring\view\datatable;
use ch\monitoring\model;
use org\adfinis\framework\db\sql\WhereGroup;
use org\adfinis\framework\db\sql\WhereStatement;

/**
 * Handles data for interacting with jQuery DataTables
 *
 * Uses the SortingPolicyIterator functionality
 *
 * Note: if you use this iterator instead of the default DBIterator, you
 * MUST NOT add an ORDER BY statement to the search query.
 *
 * This DBIterator works with DataTables please extend the target model
 * from the DataTable class in order to use this Iterator.
 *
 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2013
 * @license       LGPLv3
 **/
class DataTableIterator extends SortingPolicyIterator {

	/**
	 * Defines the default limit for the outputed entries
	 *
	 * @const         int DEFAULT_LIMIT
	 */
	const DEFAULT_LIMIT = 20;

	/**
	 * Holds the requested page, number of entires, times drawn and other settings
	 *
	 * @var           array $settings
	 */
	protected $settings;

	/**
	 * Creates a jQuery DataTable iterator
	 *
	 * Similar to the default create method but it needs a settings parameter,
	 * @see http://legacy.datatables.net/usage/server-side
	 *
	 * example:
	 * 	{
	 * 		"draw": 1,
	 * 		"start": x,
	 * 		"length": x,
	 * 		"search": ...,
	 * 		...
	 * }
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         string $searchQuery
	 * @param         string $selectQuery
	 * @param         string $targetClass
	 * @param         array  $searchQueryParameters (optional)
	 * @param         array  $selectQueryParameters (optional)
	 * @throws        \RuntimeException
	 * @return        self
	 */
	public static function create(
		$searchQuery,
		$selectQuery,
		$targetClass,
		$searchQueryParameters = array(),
		$selectQueryParameters = array()
	) {
		$object = new $targetClass();

		if ($object instanceof model\EntityWithPolicy) {
			$object->allowRead();
		}

		if (!(static::getView($targetClass) instanceof datatable\View)) {
			throw new \RuntimeException(
				'The given target class is not compatible with this iterator'
			);
		}

		$iterator = parent::create(
			$searchQuery,
			$selectQuery,
			$targetClass,
			$searchQueryParameters,
			$selectQueryParameters
		);

		return $iterator;
	}

	/**
	 * Sets the settings for the datatable iterator
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         array $settings
	 * @return        void
	 */
	public function setSettings($settings) {
		$this->settings = $settings;
	}


	/**
	 * Gets the settings for the datatable iterator
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        array
	 */
	private function getSettings() {
		if (!$this->settings) {
			throw new \RuntimeException(
				'No settings given, please assign the settings after iterator creation.'
			);
		}
		return $this->settings;
	}

	/**
	 * Returns a new instance of the target class
	 *
	 * Overriden to allow non policy objects too
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @todo by ps:   Find a better way to allow non policy objects
	 * @return        Entity
	 */
	protected function getTargetClassObject() {
		$targetClass = $this->targetClass;
		$object = new $targetClass();

		if ($object instanceof model\EntityWithPolicy) {
			$object->allowRead();
		}
		return $object;
	}

	/**
	 * Parse the desired page and other settings
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        void
	 */
	private function parseSettings() {
		$settings = $this->getSettings();
		$start    = $settings['start'];
		$length   = $settings['length'];

		$page   = 1 + floor($start / $length);

		$this->parseOrderBy($settings);

		// Paging is disabled
		if ($length < 0) {
			return;
		}
		$this->setPage($page, $length);
	}

	/**
	 * Internal function to parse the sorting parameter of the datatable
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         array $settings
	 * @return        void
	 */
	private function parseOrderBy($settings) {
		if (isset($settings['order']) && isset($settings['order'][0])) {
			$order = $settings['order'][0];
			if (count($order) <= 0) {
				return;
			}
		}
		else {
			return;
		}

		$view = static::getView($this->targetClass);
		$field = $view::getFieldByIndex($order['column']);
		$dir   = strtoupper($order['dir'] ?: $order['dir']);
		$this->setOrderBy(sprintf('%s.%s', $field, $dir));
	}

	/**
	 * Returns the iterator data as an json structure for jQuery DataTables.
	 */
	public function toDataTable() {
		$data     = [];
		$settings = $this->getSettings();

		$this->parseSettings();

		foreach ($this as $row) {
			$tableData = static::getView($row)->getDataTableData();
			$tableData['DT_RowId'] = sprintf('item%d', $row->id);
			if (isset($row->isEnabled) && !$row->isEnabled) {
				$tableData['DT_RowClass'] = 'disabled';
			}
			$data[] = $tableData;
		}

		$records  = $this->getCount();

		$list = [
			'draw'            => $settings['draw'],
			'recordsTotal'    => $records,
			'recordsFiltered' => $records,
			'data'            => $data
		];

		return $list;
	}

	/**
	 * Get the matching datatable view for given model
	 *
	 * @param         mixed $model A instantiable Entity class or a Entity object
	 * @return        datatable\View
	 */
	private static function getView($model) {
		if (is_string($model)) {
			$model = new $model;
			if ($model instanceof model\EntityWithPolicy) {
				$model->allowRead();
			}

			return $model->view('datatable');
		}
		return $model->view('datatable');
	}

	/**
	 * Get SQL statements with given DataTable settings
	 *
	 * The $fieldNameMapping is used to identify the DB field names. It is only
	 * needed for searchable flagged fields.
	 *
	 * Structure:
	 * 	[
	 * 		view::FIELD => [ 'table', 'fieldName', 'operator' ],
	 * 		(string)         (string) (string)     (string, optional)
	 * 		...
	 * 	]
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         string $model
	 * @param         array  $settings
	 * @param         array  $fieldNameMapping
	 * @return        string
	 */
	public static function getFilter($model, $settings, $fieldNameMapping) {
		$view = static::getView($model);
		$fields = $view::getDataTableFields();
		$conditions = [];

		foreach (['AND', 'OR'] as $operand) {
			$condition = new WhereGroup();
			$condition->setOperand($operand);

			$conditions[$operand] = $condition;
		}

		foreach ($fields as $field => $flags) {
			$autoSearch = (bool)($flags & $view::FIELD_SEARCHABLE);
			self::addFieldFilterToConditions($conditions, $field, $settings, $fieldNameMapping, $autoSearch);
		}

		return join(' AND ', array_filter(array_map(function($condition) { return $condition->getSQLConditions(); }, $conditions)));
	}

	/**
	 * Add to given condition object the user wanted filter
	 *
	 * @param         array<WhereGroup> $conditions
	 * @param         string $field
	 * @param         array  $settings
	 * @param         array  $fieldNameMapping
	 * @param         bool   $autoSearch
	 * @return        bool   Added or not
	 * @throws        \RuntimeException
	 */
	private static function addFieldFilterToConditions(
		$conditions,
		$field,
		$settings,
		$fieldNameMapping,
		$autoSearch
	) {
		$operand = null;
		$search = '';
		$index = datatable\View::getIndexOfField($field);
		if (isset($settings['columns']) && isset($settings['columns'][$index])) {
			$column = $settings['columns'][$index];
			if (isset($column['search']) && $column['search']['value'] != '') {
				$search  = $column['search']['value'];
				$operand = 'AND';
			}
		}
		if (isset($settings['search']['value']) && $settings['search']['value'] != '') {
			if (!$autoSearch) {
				return false;
			}
			$search  = $settings['search']['value'];
			$operand = 'OR';
		}
		else {
			return false;
		}

		if (!isset($fieldNameMapping[$field])) {
			throw new \RuntimeException(
				sprintf('Field name mapping for field "%s" is not defined', $field)
			);
		}

		// Need to pad the array because the operator is optional
		list($table, $fieldName, $operator) = array_pad($fieldNameMapping[$field], 3, null);

		switch ($operator) {
			default:
				$operator = 'LIKE';
			case 'LIKE':
				$search = sprintf('%%%s%%', $search);
				break;
			// If a search value is defined and IS is used then it IS NOT NULL
			case 'IS NOT':
				$search != !$search;
			case 'IS':
				$operator = !$search ? 'IS' : 'IS NOT';
				$search = null;
				break;
		}

		$conditions[$operand]->addStatement($field, new WhereStatement(
			$fieldName, $operator, $search, $table, false // Placeholder
		));

		return true;
	}
}
