<?php

namespace ch\monitoring\intelligence;

use org\adfinis\framework\db\sql;
use ch\monitoring\model;
use ch\monitoring\Application;

/**
 * Get a fact from the BI tables
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
abstract class Fact {

	/**
	 * Availability fact
	 *
	 * @const         int AVAILABILITY
	 */
	const AVAILABILITY = "BI_FactAvailability";

	/**
	 * The conditions for the final sql query
	 *
	 * @var           org\adfinis\framework\db\sql\WhereStatement $conditions
	 */
	private $conditions;

	/**
	 * The JOIN SQL expressions to be added to the query
	 *
	 * @var           array $join
	 */
	private $join = array();

	/**
	 * Initializes the conditions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function __construct() {
		$this->conditions = new sql\WhereGroup();
		$this->conditions->setOperand('AND');
	}

	/**
	 * Adds a condition with free operator
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $dimension
	 * @param         string $field
	 * @param         string $operator
	 * @param         mixed $value
	 * @todo by denis:   Somehow escape dimension, field and operator
	 * @returns       void
	 */
	public function addConditionWithOperator($dimension, $field, $operator, $value) {
		$dimension      = strtolower($dimension);
		$dimensionTable = sprintf("BI_Dimension%s", ucfirst($dimension));

		$this->addJoinExpression($dimension);

		$this->conditions->addStatement(
			sprintf("%s%s", $dimension, $field),
			new sql\WhereStatement(
				$field,
				$operator,
				$value,
				$dimensionTable,
				false
			)
		);
	}

	/**
	 * Adds a condition (equals operator)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $dimension
	 * @param         string $field
	 * @param         string $value
	 * @returns       void
	 */
	public function addCondition($dimension, $field, $value) {
		if ($value === null) {
			$this->addConditionWithOperator($dimension, $field, 'IS', null);
		}
		else {
			$this->addConditionWithOperator($dimension, $field, '=', $value);
		}
	}

	/**
	 * Adds a join expression if not yet present
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $dimension
	 * @returns       void
	 */
	private function addJoinExpression($dimension) {
		$dimension      = strtolower($dimension);
		$dimensionTable = sprintf("BI_Dimension%s", ucfirst($dimension));

		if (!array_key_exists($dimension, $this->join)) {
			$this->join[$dimension] = sprintf("
					JOIN `%s` ON (
						`%s`.`id` = `FactTable`.`%s`
					)
				",
				$dimensionTable,
				$dimensionTable,
				sprintf("%sId", $dimension)
			);
		}
	}

	/**
	 * Return the SUM aggregated fact fields
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $fact The fact table to be used
	 * @todo by denis:   Somehow verify permissions
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function get(
		$fact
	) {
//		$fact = Application::getDbInstance()->quote($fact);

		$query = sprintf("
				SELECT
					%s
				FROM
					`%s` as `FactTable`
				%s
				%s
				",
			$this->getAggregationExpression(),
			$fact,
			implode(",", $this->join),
			$this->conditions
		);

		return Application::getDbInstance()->fetchRow($query);
	}

	/**
	 * Returns the available aggregator fields
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	abstract public function getAggregators();

	/**
	 * Returns the aggregators as SQL expression (SUM)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getAggregationExpression() {
		$aggregators = $this->getAggregators();
		foreach ($aggregators as &$aggregator) {
			$aggregator = sprintf(
				"SUM(%s) as %s",
				$aggregator,
				$aggregator
			);
		}

		return implode(",", $aggregators);
	}
}
