<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\AlertValidator;
use ch\monitoring\policy;
use ch\monitoring\util;

/**
 * Model for an alert
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Alert extends EntityWithPolicy {
	/**
	 * Alert is new
	 *
	 * @const         string STATE_NEW
	 */
	const STATE_NEW = 'new';

	/**
	 * Alert was acknowledged
	 *
	 * @const         string STATE_OPEN
	 */
	const STATE_OPEN = 'open';

	/**
	 * Alert was resolved
	 *
	 * @const         string STATE_CLOSED
	 */
	const STATE_CLOSED = 'closed';

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer this alert belongs to.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The host this alert belongs to.
	 *
	 * @var           int $hostId
	 */
	public $hostId;

	/**
	 * The service this alert belongs to.
	 *
	 * @var           int $serviceInstanceId
	 */
	public $serviceInstanceId;

	/**
	 * The command this alert belongs to.
	 *
	 * @var           int $commandId
	 */
	public $commandId;

	/**
	 * The sla this alert belongs to.
	 *
	 * @var           int $slaId
	 */
	public $slaId;

	/**
	 * The customer the sla of this alert belongs to.
	 *
	 * @var           int $slaCustomerId
	 */
	public $slaCustomerId;

	/**
	 * When did the alert occur?
	 *
	 * @var           Zend_Date $datetimeOccured
	 */
	public $datetimeOccured;

	/**
	 * When was the alert acknowledged?
	 *
	 * @var           Zend_Date $datetimeAcknowledged
	 */
	public $datetimeAcknowledged;

	/**
	 * When was the trouble recovery?
	 *
	 * @var           Zend_Date $datetimeRecovery
	 */
	public $datetimeRecovery;

	/**
	 * The problem state.
	 *
	 * @var           string $state
	 */
	public $state;

	/**
	 * Raw output from Nagios.
	 *
	 * @var           string $output
	 */
	public $output;
	
	/**
	 * Has this alert been acknowledged?
	 *
	 * @var           bool $acknowledged
	 */
	public $acknowledged;

	/**
	 * True when the recovery was detected before anyone acknowledged.
	 *
	 * @var           bool $acknowledgedAutomatically
	 */
	public $acknowledgedAutomatically;

	/**
	 * Who acknowledged this alert?
	 *
	 * @var           int $acknowledgedBy
	 */
	public $acknowledgedBy;

	/**
	 * Additional comment for this alert.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                        = (int)$data['id'];
		$this->customerId                = (int)$data['customerId'];
		$this->hostId                    = (int)$data['hostId'];
		$this->commandId                 = (int)$data['commandId'];
		$this->serviceInstanceId         = (int)$data['serviceInstanceId'] ?: null;
		$this->slaId                     = (int)$data['slaId'] ?: null;
		$this->slaCustomerId             = (int)$data['slaCustomerId'] ?: null;
		$this->datetimeOccured           = new \Zend_Date($data['datetimeOccured'], \Zend_Date::ISO_8601);
		$this->datetimeAcknowledged      = new \Zend_Date($data['datetimeAcknowledged'], \Zend_Date::ISO_8601);
		$this->datetimeRecovery          = $data['datetimeRecovery'] ? new \Zend_Date($data['datetimeRecovery'], \Zend_Date::ISO_8601) : null;
		$this->state                     = $data['state'];
		$this->output                    = $data['output'];
		$this->acknowledgedBy            = (int)$data['acknowledgedBy'] ?: null;
		$this->comment                   = $data['comment'];
		$this->acknowledged              = (bool)$data['acknowledged'];
		$this->acknowledgedAutomatically = (bool)$data['acknowledgedAutomatically'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\AlertPolicy
	 */
	public function getPolicy() {
		return new policy\AlertPolicy($this);
	}

	/**
	 * Returns a validator for the current Alert object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\AlertValidator
	 */
	public function getValidator() {
		return new AlertValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\AlertMapper
	 */
	public static function getMapper() {
		return datamapper\AlertMapper::getInstance();
	}

	/**
	 * Returns the Customer of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the Host of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}

	/**
	 * Returns the Command of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Command
	 */
	public function getCommand() {
		return datamapper\CommandMapper::getInstance()->getByID(
			$this->commandId
		)->allowRead();
	}

	/**
	 * Returns the Service of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return $this->getHostServiceInstance()->getHostService()->getService();
	}

	/**
	 * Returns the HostService of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostService
	 */
	public function getHostService() {
		return $this->getHostServiceInstance()->getHostService();
	}

	/**
	 * Returns the HostServiceInstance of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getHostServiceInstance() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getByID(
			$this->serviceInstanceId
		)->allowRead();
	}

	/**
	 * Returns the SLA of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}

	/**
	 * Returns the SLACustomer of this Alert
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getSLACustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->slaCustomerId
		)->allowRead();
	}

	/**
	 * Acknowledge this alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       ch\symonitoring\model\Alert
	 */
	public function acknowledge($userID) {
		$result = self::getMapper()->acknowledge($this->id, $userID);

		return $result;
	}

	/**
	 * Return the alert log
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\AlertLog
	 */
	public function getAlertLog() {
		return datamapper\AlertLogMapper::getInstance()->getAll(
			$this->id,
			null
		);
	}

	/**
	 * Add a comment to this alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $comment Comment
	 * @param         array $tags The tag IDs to associate
	 * @param         int $userID The ID of the user who made the comment
	 */
	public function addComment($comment, $tags, $userId) {
		datamapper\AlertCommentMapper::getInstance()->create($comment, $userId, $this->id, $tags);
	}

	/**
	 * Returns the associated comments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $tags Limit comments to this list of tags
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getComments(array $tags = null) {
		return datamapper\AlertCommentMapper::getInstance()->getAll($this->id, $tags, null);
	}

	/**
	 * Return the amount of comments of this alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getCommentCount() {
		return datamapper\AlertCommentMapper::getInstance()->countByAlertId($this->id);
	}

	/**
	 * Return the user that acknowledged the alert
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getAcknowledgeUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->acknowledgedBy
		)->allowRead();
	}

	/**
	 * Return the duration in hours, minutes and seconds
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getDurationRecovery() {
		$duration = $this->datetimeRecovery->sub($this->datetimeOccured)->get(\Zend_Date::TIMESTAMP);
		$hoursArray = util\TimeUtility::secondsToHours($duration);

		return sprintf(
			"%02d%s %02d%s %02d%s",
			$hoursArray['hours'],
			tr('h'),
			$hoursArray['minutes'],
			tr('m'),
			$hoursArray['seconds'],
			tr('s')
		);
	}

	/**
	 * Return the duration in hours, minutes and seconds
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getDurationAcknowledge() {
		$duration = $this->datetimeAcknowledged->sub($this->datetimeOccured)->get(\Zend_Date::TIMESTAMP);
		$hoursArray = util\TimeUtility::secondsToHours($duration);

		return sprintf(
			"%02d%s %02d%s %02d%s",
			$hoursArray['hours'],
			tr('h'),
			$hoursArray['minutes'],
			tr('m'),
			$hoursArray['seconds'],
			tr('s')
		);
	}

	/**
	 * Set the alert to recovered
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $reason
	 * @param         Zend_Date $date
	 * @returns       void
	 */
	public function recover(
		$reason,
		\Zend_Date $date = null
	) {
		$this->comment          = $reason;
		$this->datetimeRecovery = $date ?: new \Zend_date();

		if (!$this->acknowledged) {
			$this->acknowledged              = true;
			$this->acknowledgedAutomatically = true;
		}

		// Circumvent the policy system by using the mapper's save() method
		$this->getMapper()->save($this);

		if ($this->serviceInstanceId === null) {
			$host = $this->getHost();
			$host->hasTrouble = false;
			$host->getMapper()->save($host);
		}
		else {
			$instance = $this->getHostServiceInstance();
			$instance->hasTrouble = false;
			$instance->getMapper()->save($instance);
		}
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'commandId' => array(
				'type' => 'relation',
				'mapper' => 'CommandMapper'
			),

			'serviceInstanceId' => array(
				'type' => 'relation',
				'mapper' => 'HostServiceInstanceMapper'
			),
			
			'hostId' => array(
				'type' => 'relation',
				'mapper' => 'HostMapper',
				'name' => 'hostname'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'slaCustomerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'acknowledgedBy' => array(
				'type' => 'relation',
				'mapper' => 'UserMapper'
			),

			'acknowledged' => array(
				'type' => 'boolean'
			),

			'acknowledgedAutomatically' => array(
				'type' => 'boolean'
			),

			'acknowledgedProcessed' => array(
				'type' => 'boolean'
			)
		);
	}

	/**
	 * Returns the state of the alert as string
	 *
	 * @returns       const  The state of the alert
	 */
	public function getAlertState() {
		if ($this->datetimeRecovery === null) {
			if ($this->acknowledged) {
				return Alert::STATE_OPEN;
			}
			return Alert::STATE_NEW;
		}
		return Alert::STATE_CLOSED;
	}
}
