<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\AlertCommentValidator;
use ch\monitoring\policy;
use ch\monitoring\util;

/**
 * Model for an alert comment
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertComment extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The alert this comment belongs to.
	 *
	 * @var           int $alertId
	 */
	public $alertId;

	/**
	 * The user who created this comment.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * Creation date and time.
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * The comment.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id        = (int)$data['id'];
		$this->alertId   = (int)$data['alertId'];
		$this->userId    = (int)$data['userId'];
		$this->timestamp = new \Zend_Date($data['timestamp']);
		$this->comment   = $data['comment'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\AlertCommentPolicy
	 */
	public function getPolicy() {
		return new policy\AlertCommentPolicy($this);
	}

	/**
	 * Returns a validator for the current AlertComment object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\AlertCommentValidator
	 */
	public function getValidator() {
		return new AlertCommentValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\AlertCommentMapper
	 */
	public static function getMapper() {
		return datamapper\AlertCommentMapper::getInstance();
	}

	/**
	 * Returns the Alert this Comment belongs to.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Alert
	 */
	public function getAlert() {
		return datamapper\AlertMapper::getInstance()->getByID(
			$this->alertId
		)->allowRead();
	}

	/**
	 * Returns the User who generated this comment.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Return the alert tags
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAlertTags() {
		return datamapper\AlertTagMapper::getInstance()->getByAlertCommentID(
			$this->id
		);
	}

	/**
	 * Return the tags as an array
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getAlertTagsAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getAlertTags(),
			'id',
			'name'
		);
	}

	/**
	 * Add a tag to the comment
	 *
	 * The old function had the possibility to pass an array of tags
	 * to add. I thought, let the controller handle the loop over
	 * these tags.
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         int $tagID The tag ID
	 */
	public function addAlertTag($tagID) {
		datamapper\AlertTagMapper::getInstance()
			->createAssociation($tagID, $this->id);
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }
}
