<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\AlertLogValidator;
use ch\monitoring\policy;

/**
 * Model for an alert log entry 
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertLog extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The notification type.
	 *
	 * @var           int $typeId
	 */
	public $typeId;

	/**
	 * The user who was notified.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The alert this notification was sent for.
	 *
	 * @var           int $alertId
	 */
	public $alertId;

	/**
	 * Date and time when the notification was sent.
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * Whether the alerting has failed
	 *
	 * @var           bool $hasFailed
	 */
	public $hasFailed = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id        = (int)$data['id'];
		$this->typeId    = (int)$data['typeId'];
		$this->userId    = (int)$data['userId'];
		$this->alertId   = (int)$data['alertId'];
		$this->timestamp = new \Zend_Date($data['timestamp']);
		$this->hasFailed = (bool)$data['hasFailed'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\AlertLogPolicy
	 */
	public function getPolicy() {
		return new policy\AlertLogPolicy($this);
	}

	/**
	 * Returns a validator for the current AlertLog object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\AlertLogValidator
	 */
	public function getValidator() {
		return new AlertLogValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\AlertLogMapper
	 */
	public static function getMapper() {
		return datamapper\AlertLogMapper::getInstance();
	}

	/**
	 * Returns the NotificationType of this AlertLog entry
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\NotificationType
	 */
	public function getNotificationType() {
		return datamapper\NotificationTypeMapper::getInstance()->getByID(
			$this->typeId
		);
	}

	/**
	 * Returns the alert this AlertLog entry was created for
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Alert
	 */
	public function getAlert() {
		return datamapper\AlertMapper::getInstance()->getByID(
			$this->alertId
		)->allowRead();
	}

	/**
	 * Returns the user who created this alert log entry
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }
}
