<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\model;

use org\adfinis\framework\lang\Language;


class AuditLog extends Entity {

	/**
	 * Identifier
	 *
	 * @var           int $id
	 */
	public $id;
	

	/**
	 * Entity name
	 *
	 * @var           string $entity
	 */
	public $entity;

	/**
	 * The id of the referencing entity
	 *
	 * @var           int $entityID
	 */
	public $entityID;

	/**
	 * The name of the referencing entity
	 *
	 * @var           string $entityName
	 */
	public $entityName;
	
	/**
	 * Operation (create, update, delete)
	 *
	 * @var           string $operation
	 */
	public $operation;
	
	/**
	 * User ID
	 *
	 * @var           int $userID
	 */
	public $userID;
	
	/**
	 * User name
	 *
	 * @var           string $userName
	 */
	public $userName;
	
	/**
	 * Timestmap
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;
	
	/**
	 * AuditLogAction id
	 *
	 * @var           int $auditLogActionID
	 */
	public $auditLogActionID;

	/**
	 * The order in which they have been created/updated/deleted
	 *
	 * @var           int $order
	 */
	public $order;

	public function getIdentifier() {
		return array('id' => $this->id);
	}

	public function fillFromRowData($data)  {
		$this->id               = intval($data['id']);
		$this->entity           = $data['entity'];
		$this->entityID         = $data['entityID'] ? intval($data['entityID']) : NULL;
		$this->entityName       = $data['entityName'];
		$this->operation        = $data['operation'];
		$this->userID           = $data['userID'] ? intval($data['userID']) : NULL;
		$this->userName         = $data['userName'];
		$this->timestamp        = new \Zend_Date($data['timestamp']);
		$this->auditLogActionID = $data['auditLogActionID'] ? intval($data['auditLogActionID']) : NULL;
		$this->order            = (int)$data['order'];
	}


	public function doesRecordMatch($data) {
		return ($this->id == (int)$data['id']);
	}

	/**
	* Returns a data mapper for handling persistence of this object
	*
	* @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	* @returns       ch\monitoring\datamapper\AuditLogMapper
	*/
	public static function getMapper() {
		return datamapper\AuditLogMapper::getInstance();
	}


	/**
	 * Return the user by the id
	 *
	 * It might be that a user was deleted, we return NULL in this
	 * case
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getSystemUser() {
		try {
			$user = datamapper\UserMapper::getInstance()->getByID(
				$this->userID
			);

			$user->enforceReadPermission(Permission::USERS_READ);
		}
		catch (\OutOfBoundsException $e) {
			return NULL;
		}

		return $user->fullName;
	}

	/**
	 * Returns a list of the changed columns
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getLogChanges() {
		$result = datamapper\AuditLogMapper::getInstance()->getLogChanges($this->id);
		$entity = $class = 'ch\\monitoring\\model\\' . $this->entity;

		$ret = array();

		if(!class_exists($entity) || !method_exists($entity, 'getEntityDefinition')) {
			foreach($result as $row) {
				$row['type'] = 'string';
				$fullColumnName = $this->entity.'.'. $row['column'];
				$fullColumnDescription = 'auditlog.description.' . $fullColumnName;
				
				$row['description'] = Language::gettext($fullColumnDescription);
				if($row['description'] == $fullColumnDescription) {
					$row['description'] = '';
				}

				$row['name'] = Language::gettext('auditlog.name.' . $fullColumnName);
				if($row['name'] == 'auditlog.name.' . $fullColumnName) {
					$row['name'] = $row['column'];
				}

				$ret[] = $row;
			}

			return $ret;
		}

		
		$mapping = $entity::getEntityDefinition();
		$keys = array_keys($mapping);

		foreach($result as $row) {
			$row['type'] = 'string';
			$fullColumnName = $this->entity.'.'. $row['column'];
			$fullColumnDescription = 'auditlog.description.' . $fullColumnName;
			
			$row['description'] = Language::gettext($fullColumnDescription);
			if($row['description'] == $fullColumnDescription) {
				$row['description'] = '';
			}

			$row['name'] = Language::gettext('auditlog.name.' . $fullColumnName);
			if($row['name'] == 'auditlog.name.' . $fullColumnName) {
				$row['name'] = $row['column'];
			}


			if(in_array($row['column'], $keys)) {
				$fieldInfo = $mapping[$row['column']];

				$row['type'] = $fieldInfo['type'];

				if($fieldInfo['type'] == 'relation') {
					$propertyName = isset($fieldInfo['name']) ? $fieldInfo['name'] : 'name';
					$class = 'ch\\monitoring\\datamapper\\' . $fieldInfo['mapper'];
					$class = $class::getInstance();

					if(empty($row['oldValue']) || is_null($row['oldValue'])) {
						$row['oldValue'] = '&lt;NULL&gt;';
					}else {
						try {
							$oldValue = $class->getByID($row['oldValue']);
							if($oldValue instanceof model\EntityWithPolicy) {
								$oldValue->allowRead();
							}

							if(isset($oldValue->$propertyName)) {
								$row['oldValue'] = $row['oldValue'] . ' (' . $oldValue->$propertyName . ')';
							}
						}catch(\Exception $e) {
							$row['oldValue'] = $row['oldValue'] . ' (deleted)';
						}
					}

					if(empty($row['newValue']) || is_null($row['newValue'])) {
						$row['newValue'] = '&lt;NULL&gt;';
					}else {
						try {
							$newValue = $class->getByID($row['newValue']);
							if($newValue instanceof model\EntityWithPolicy) {
								$newValue->allowRead();
							}
							if(isset($newValue->$propertyName)) {
								$row['newValue'] = $row['newValue'] . ' (' . $newValue->$propertyName . ')';
							}
						}catch(\Exception $e) {
							$row['newValue'] = $row['newValue'] . ' (deleted)';
						}
					}
				}
			}

			$ret[] = $row;
		}

		return $ret;
	}

	/**
	 * Returns all sub auditlogs
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getSubAuditLogs() {
		return $this->getMapper()->getSubAuditLogs($this->auditLogActionID);
	}

	/**
	 * Returns all sub auditlogs
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function hasSubAuditLogs() {
		return $this->order == 0;
	}
}
