<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\BREApplicationValidator;

/**
 * Model for a service assigned to a host
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class BREApplication extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * Customer foreignkey
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * Name of the application.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The minimal amount of available components.
	 *
	 * @var           int $minAvailable
	 */
	public $minAvailable;

	/**
	 * Description
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		$this->customerId   = (int)$data['customerId'];
		$this->name         = (string)$data['name'];
		$this->minAvailable = (int)$data['minAvailable'];
		$this->description  = (string)$data['description'];
	}

	/**
	 * This method fills the object from an object passed by the api
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromAPIData($data) {
		$this->id           = (int)$data->id;
		$this->customerId   = (int)$data->customer;
		$this->name         = (string)$data->name;
		$this->minAvailable = (int)$data->min_available;
		$this->description  = (string)$data->description;
	}

	/**
	 * This method fills the object from a request object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $request
	 * @returns       void
	 */
	public function fillFromRequestData($request) {
		$this->id           = $request->getInt('applicationId');
		$this->customerId   = $request->getInt('customerId');
		$this->name         = $request->getString('name');
		$this->minAvailable = $request->getInt('minAvailable');
		$this->description  = $request->getString('description');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Application object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\BREApplicationValidator
	 */
	public function getValidator() {
		return new BREApplicationValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\BREApplicationMapper
	 */
	public static function getMapper() {
		return datamapper\BREApplicationMapper::getInstance();
	}

}
