<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\BREComponentValidator;

/**
 * Model for a service assigned to a host
 *
 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
 **/
class BREComponent extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * Name of the component
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The minimal amount of available components.
	 *
	 * @var           int $minAvailable
	 */
	public $minAvailable;

	/**
	 * BREApplication foreignkey
	 *
	 * @var           int $applicationId
	 */
	public $applicationId;

	/**
	 * Parent component foreignkey
	 *
	 * @var           int $applicationId
	 */
	public $parentId;

	/**
	 * Description
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id             = (int)$data['id'];
		$this->name           = (string)$data['name'];
		$this->minAvailable   = (int)$data['minAvailable'];
		$this->applicationId  = (int)$data['applicationId'];
		$this->parentId       = (int)$data['parentId'];
		$this->description    = (string)$data['description'];
	}

	/**
	 * Fill the object from data received by the RPC API.
	 *
	 * @param         stdObject $data object containing the data received by
	 *                                RPC API
	 */
	public function fillFromAPIData($data) {
		$this->id            = (int)$data->id;
		$this->name          = (string)$data->name;
		$this->minAvailable  = (int)$data->min_available;
		$this->applicationId = (int)$data->application;
		$this->parentId      = (int)$data->parent;
		$this->description   = (string)$data->description;
	}

	/**
	 * Fill the object from data received by a (form-) request.
	 *
	 * @param         Request $request object containing the request
	 */
	public function fillFromRequestData($request) {
		$id = (int)   $request->getInt('componentId');
		if ($id > 0) {
			$this->id        = $id;
		}
		$this->name          = $request->getString('name');
		$this->minAvailable  = $request->getInt('minAvailable');
		$this->applicationId = $request->getInt('applicationId');
		$this->parentId      = $request->getInt('parentId');
		$this->description   = $request->getString('description');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * TODO: do we need this policy?
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Component object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\BREComponentValidator
	 */
	public function getValidator() {
		return new BREComponentValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\BREComponentMapper
	 */
	public static function getMapper() {
		return datamapper\BREComponentMapper::getInstance();
	}

}
