<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\BREServiceValidator;

/**
 * Model for a service assigned to a host
 *
 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
 **/
class BREService extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * BREComponent foreignkey
	 *
	 * @var           int $componentId
	 */
	public $componentId;

	/**
	 * Service foreignkey
	 *
	 * @var           int $serviceId
	 */
	public $serviceId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->componentId = (int)$data['componentId'];
		$this->serviceId   = (int)$data['serviceId'];
	}

	/**
	 * Fill the object from data received by the RPC API.
	 *
	 * @param         stdObject $data object containing the data received by
	 *                                RPC API
	 */
	public function fillFromAPIData($data) {
		$this->id          = (int)$data->id;
		$this->name        = (string)$data->name;
		$this->componentId = (int)$data->component;
		$this->serviceId   = (int)$data->service;
	}

	/**
	 * Fill the object from data received by a (form-) request.
	 *
	 * @param         Request $request object containing the request
	 */
	public function fillFromRequestData($request) {
		$this->name        = $request->getString('name');
		$this->componentId = $request->getInt('componentId');
		$this->serviceId   = $request->getInt('serviceId');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * TODO: do we need this policy?
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Service object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\BREServiceValidator
	 */
	public function getValidator() {
		return new BREServiceValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\BREServiceMapper
	 */
	public static function getMapper() {
		return datamapper\BREServiceMapper::getInstance();
	}

}
