<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\CommandValidator;
use ch\monitoring\policy;
use ch\monitoring\util;

/**
 * Model for a command
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Command extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The command name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the command.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * The command line.
	 *
	 * @var           string $commandline
	 */
	public $commandline;

	/**
	 * Is this command IPv6 capable?
	 *
	 * @var           bool $v6Capable
	 */
	public $v6Capable = false;

	/**
	 * Is this command deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->name        = $data['name'];
		$this->description = $data['description'];
		$this->commandline = $data['commandline'];
		$this->v6Capable   = (bool)$data['v6Capable'];
		$this->isDeleted   = (bool)$data['isDeleted'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Command object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\CommandValidator
	 */
	public function getValidator() {
		return new CommandValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\CommandMapper
	 */
	public static function getMapper() {
		return datamapper\CommandMapper::getInstance();
	}

	/**
	 * Return the serivces assigned to this command
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator<model\Service>
	 */
	public function getServices() {
		return datamapper\ServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			null,
			$this->id
		);
	}

	/**
	 * Return the serivces assigned to this command as id => name pairs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getServicesAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getServices()
		);
	}

	/**
	 * Returns all guidelines for current
	 * command
	 *
	 * @returns       org\adfinis\framework\db\DbIterator<model\Guideline>
	 */
	public function getGuidelines() {
		return datamapper\GuidelineMapper::getInstance()->getAllByCommandID($this->id);
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'v6Capable' => array(
				'type' => 'boolean'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			)
		);
	}

	/**
	 * Returns all alerts for hosts which are related to this command
	 *
	 * @param         int $openOnly Optional Return only open alerts
	 * @returns       db\adfinis\framework\db\DbIterator
	 */
	public function getHostAlerts($openOnly=null) {
		return datamapper\AlertMapper::getInstance()->getAll(
			null,
			$openOnly,
			null,
			null,
			null,
			null,
			null,
			true,
			Permission::ALERTS_READ,
			false,
			$this->id
		);
	}
}
