<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\util;
use ch\monitoring\validate\CustomerValidator;
use ch\monitoring\policy;

/**
 * Model for a customer
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Customer extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The parent customer.
	 *
	 * @var           int $customerId
	 */
	public $parentId;

	/**
	 * The default SLA of this customer.
	 *
	 * @var           int $slaId
	 */
	public $slaId;

	/**
	 * The customer name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The contact eMail address of this customer.
	 *
	 * @var           string $email
	 */
	public $email;

	/**
	 * URL to the documentation of this customer.
	 *
	 * @var           string $wiki
	 */
	public $wiki;

	/**
	 * A comment with more information on this customer.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * The password to change the customer information from outside.
	 *
	 * @var           string $passwordInformation
	 */
	public $passwordInformation;

	/**
	 * The password to schedule downtimes from outside.
	 *
	 * @var           string $passwordMaintenance
	 */
	public $passwordMaintenance;

	/**
	 * Last access for Information update.
	 *
	 * @var           Zend_Date $lastAccess
	 */
	public $lastAccess;

	/**
	 * Send alert related information to the hoster?
	 *
	 * @var           bool $warnHoster
	 */
	public $warnHoster;

	/**
	 * Is this Customer enabled?
	 *
	 * @var           bool $isEnabled
	 */
	public $isEnabled = false;

	/**
	 * Is this Customer deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;

	/**
	 * LDAP group of the customer for identifying its users
	 *
	 * @var           string $ldapGroup
	 */
	public $ldapGroup;

	/**
	 * Default role a user gets assigned when logging in
	 * using LDAP authentication belonging to this customer
	 * mapped by $this->ldapGroup.
	 *
	 * @var           int $defaultRoleId
	 */
	public $defaultRoleId = null;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @return        bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @return        array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @return        void
	 */
	public function fillFromRowData($data) {
		$this->id                  = (int)$data['id'];
		$this->parentId            = (int)$data['parentId'] ?: null;
		$this->slaId               = (int)$data['slaId'];
		$this->name                = $data['name'];
		$this->email               = $data['email'] ?: null;
		$this->wiki                = $data['wiki'] ?: null;
		$this->comment             = $data['comment'] ?: null;
		$this->passwordInformation = $data['passwordInformation'];
		$this->passwordMaintenance = $data['passwordMaintenance'];
		$this->lastAccess          = new \Zend_Date($data['lastAccess'], \Zend_Date::ISO_8601);
		$this->warnHoster          = (bool)$data['warnHoster'];
		$this->isEnabled           = (bool)$data['isEnabled'];
		$this->isDeleted           = (bool)$data['isDeleted'];
		$this->ldapGroup           = $data['ldapGroup'] ?: null;
		$this->defaultRoleId       = $data['defaultRoleId'] ?: null;
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerPolicy($this);
	}

	/**
	 * Returns a validator for the current Customer object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\CustomerValidator
	 */
	public function getValidator() {
		return new CustomerValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\CustomerMapper
	 */
	public static function getMapper() {
		return datamapper\CustomerMapper::getInstance();
	}

	/**
	 * Returns the parent Customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getParent() {
		$customer = datamapper\CustomerMapper::getInstance()->getByID(
			$this->parentId
		);
		$customer->allowRead();

		return $customer;
	}

	/**
	 * Returns the SLA of this Customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}

	/**
	 * Returns the associated customer information
	 *
	 * If no information is associated, a new CustomerModel is returned.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\CustomerInformation
	 */
	public function getCustomerInformation() {
		try {
			return datamapper\CustomerInformationMapper::getInstance()->getByID(
				$this->id
			)->allowRead();
		}
		catch (\OutOfBoundsException $e) {
			$customerInformation = new CustomerInformation();
			return $customerInformation->allowRead();
		}
	}

	/**
	 * Returns the hosts which belong to this customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isEnabled Optional
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHosts($isEnabled = null) {
		return datamapper\HostMapper::getInstance()->getAll(
			false,
			false,
			false,
			null,
			$this->id,
			null,
			$isEnabled
		);
	}

	/**
	 * Returns the hosts which belong to this customer as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isEnabled Optional
	 * @param         bool $removeDisabledHosts Optional
	 * @returns       array
	 */
	public function getHostsAsArray($isEnabled = null, $removeDisabledHosts = false) {
		$allHosts = $this->getHosts($isEnabled);
		$hosts = array();
		if($removeDisabledHosts) {
			foreach($allHosts as $host) {
				if($host->isEnabled()) {
					$hosts[] = $host;
				}
			}
		}else {
			$hosts = $allHosts;
		}

		return util\IteratorUtil::getKeyValueArray(
			$hosts,
			'id',
			'hostname'
		);
	}

	/**
	 * Verifies whether a given password equals to the customer password
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $password
	 * @returns       bool
	 */
	public function verifyPassword($password) {
		return $this->password == $password;
	}

	/**
	 * Returns the current customer team of given date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Zend_Date $date Null means default team.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getCustomerTeam($date) {
		return datamapper\CustomerTeamMapper::getInstance()->getAll(
			$date,
			$this->id
		);
	}

	/**
	 * Returns the subcustomer ids of this customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getSubCustomers() {
		return $this->getMapper()->getSubCustomers($this->id);
	}

	/**
	 * Returns the subcustomer names of this customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         bool $removeDisabledCustomers optional
	 * @returns       array
	 */
	public function getSubCustomersNames($removeDisabledCustomers = false) {
		if($removeDisabledCustomers) {
			//if this customer is disabled than none of the subcustomers can be returned (they are disabled too)
			if(!$this->isEnabled()) {
				return array();
			}
		}
		
		return $this->getMapper()->getSubCustomersNames($this->id, $removeDisabledCustomers);
	}

	/**
	 * Returns the total reaction time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       int Reaction time in seconds
	 */
	public function getReactionTime($dateFrom = null, $dateTo = null) {
		return datamapper\AlertMapper::getInstance()->getReactionTime(
			datamapper\AlertMapper::TYPE_REACTION,
			$dateFrom,
			$dateTo,
			$this->id
		);
	}

	/**
	 * Returns the total down time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       int Down time in seconds
	 */
	public function getDownTime($dateFrom = null, $dateTo = null) {
		return datamapper\AlertMapper::getInstance()->getReactionTime(
			datamapper\AlertMapper::TYPE_DOWN,
			$dateFrom,
			$dateTo,
			$this->id
		);
	}

	/**
	 * Returns the count of alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $openOnly Optional
	 * @returns       int
	 */
	public function getAlertCount($openOnly = null) {
		return datamapper\AlertMapper::getInstance()->count($this->id, null, null, $openOnly);
	}

	/**
	 * Returns the count of hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isSatellite Optional
	 * @param         bool $hasTrouble Optional
	 * @returns       int
	 */
	public function getHostCount($isSatellite = null, $hasTrouble = null) {
		return datamapper\HostMapper::getInstance()->count($this->id, $isSatellite, $hasTrouble);
	}

	/**
	 * Returns the count of assigned services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $hasTrouble Optional
	 * @returns       int
	 */
	public function getServiceCount($hasTrouble = null) {
		return datamapper\HostServiceMapper::getInstance()->count($this->id, null, $hasTrouble);
	}

	/**
	 * Sets the value (disabled, enabled, inherit) of an AlertType
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $alertTypeId
	 * @param         bool|null $value
	 * @returns       array
	 */
	public function setAlertSetting($alertTypeId, $value = null) {
		datamapper\CustomerMapper::getInstance()->setAlertSetting($this->id, $alertTypeId, $value);
	}

	/**
	 * Returns an associative array with the key as AlertSetting ID and as value the status of the setting
	 *
	 * This method already handles the cases where the value must be inherited from the parent customer! 
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         bool $inheritValues
	 * @returns       array
	 */
	public function getAlertSettings($inheritValues = true) {
		$alertSettings = $this->getMapper()->getAlertSettings($this->id);

		if($inheritValues) {
			// no parent = we cannot inherit the value
			if(is_null($this->parentId)) {
				foreach($alertSettings as $alertSettingId => &$alertSettingValue) {
					if($alertSettingValue == 0) {
						//set the value to a default value
						$alertSettingValue = 2;
					}
				}
			}else {
				//there is a parent = we can inherit the value
				$inheritedAlertSettings = $this->getParent()->getAlertSettings(true);
				foreach($alertSettings as $alertSettingId => &$alertSettingValue) {
					if($alertSettingValue == 0) {
						$alertSettingValue = $inheritedAlertSettings[$alertSettingId];
					}
				}
			}
		}

		return $alertSettings;
	}

	/**
	 * Computed field which express if a customer is enabled or disabled (depends on the parent customer)
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *
	 * @returns       bool
	 */
	public function isEnabled() {
		if($this->parentId) {
			$parentCustomer = datamapper\CustomerMapper::getInstance()->getByID($this->parentId);
			$parentCustomer->allowRead();
			
			if(!$parentCustomer->isEnabled()) {
				return false;
			}
		}

		return $this->isEnabled;
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'parentId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'warnHoster' => array(
				'type' => 'boolean'
			),
			
			'isEnabled' => array(
				'type' => 'boolean'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			)
		);
	}
}
