<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\CustomerInformationValidator;
use ch\monitoring\policy;

/**
 * Model for a customer information
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerInformation extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The name of the hoster.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The address.
	 *
	 * @var           string $address
	 */
	public $address;

	/**
	 * Technical contact person
	 *
	 * @var           string $contactTechnical
	 */
	public $contactTechnical;

	/**
	 * Commercial contact person
	 *
	 * @var           string $contactCommercial
	 */
	public $contactCommercial;

	/**
	 * Regular phone number
	 *
	 * @var           string $phoneRegular
	 */
	public $phoneRegular;

	/**
	 * Emergency phone number
	 *
	 * @var           string $phoneEmergency
	 */
	public $phoneEmergency;

	/**
	 * Comment on the hoster
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Comment on emergency situations
	 *
	 * @var           string $commentEmergency
	 */
	public $commentEmergency;

	/**
	 * Whether the hoster may be contacted
	 *
	 * @var           bool $mayContact
	 */
	public $mayContact = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                = (int)$data['id'];
		$this->name              = $data['name'] ?: null;
		$this->address           = $data['address'] ?: null;
		$this->contactTechnical  = $data['contactTechnical'] ?: null;
		$this->contactCommercial = $data['contactCommercial'] ?: null;
		$this->phoneRegular      = $data['phoneRegular'] ?: null;
		$this->phoneEmergency    = $data['phoneEmergency'] ?: null;
		$this->comment           = $data['comment'] ?: null;
		$this->commentEmergency  = $data['commentEmergency'] ?: null;
		$this->mayContact        = (bool)$data['mayContact'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerInformationPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerInformationPolicy($this);
	}

	/**
	 * Returns a validator for the current CustomerInformation object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\CustomerInformationValidator
	 */
	public function getValidator() {
		return new CustomerInformationValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\CustomerTeamMapper
	 */
	public static function getMapper() {
		return datamapper\CustomerInformationMapper::getInstance();
	}
}
