<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\CustomerTeamValidator;
use ch\monitoring\policy\CustomerTeamPolicy;

/**
 * Model for a customer team member
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerTeam extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer this member belongs to.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The user in question.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The date this entry is valid on.
	 *
	 * Null means "default".
	 *
	 * @var           Zend_Date $date
	 */
	public $date;

	/**
	 * The priority.
	 *
	 * Lower is more important.
	 *
	 * @var           int $priority
	 */
	public $priority;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id         = (int)$data['id'];
		$this->customerId = (int)$data['customerId'];
		$this->userId     = (int)$data['userId'];
		$this->date       = new \Zend_Date($data['date'], \Zend_Date::ISO_8601);
		$this->priority   = (int)$data['priority'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerTeamPolicy
	 */
	public function getPolicy() {
		return new CustomerTeamPolicy($this);
	}

	/**
	 * Returns a validator for the current CustomerTeam object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\CustomerTeamValidator
	 */
	public function getValidator() {
		return new CustomerTeamValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\CustomerTeamMapper
	 */
	public static function getMapper() {
		return datamapper\CustomerTeamMapper::getInstance();
	}

	/**
	 * Returns the Customer for this entry.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the User in question.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'userId' => array(
				'type' => 'relation',
				'mapper' => 'UserMapper'
			),

			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			)
		);
	}
}
