<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\DowntimeValidator;

/**
 * Model for a service assigned to a host
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Downtime extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The host this downtime belongs to.
	 *
	 * @var           int $customerId
	 */
	public $hostId;

	/**
	 * The serviceInstance this downtime belongs to.
	 *
	 * Null for whole host.
	 *
	 * @var           int $serviceInstanceId
	 */
	public $serviceInstanceId;

	/**
	 * Downtime start.
	 *
	 * @var           Zend_Date $datetimeFrom
	 */
	public $datetimeFrom;

	/**
	 * Downtime end.
	 *
	 * @var           Zend_Date $datetimeTo
	 */
	public $datetimeTo;

	/**
	 * RFC 2445 Recurrence Rule Format
	 *
	 * @var           string|null $rrule
	 */
	public $rrule = null;

	/**
	 * Define the duration
	 *
	 * @var           float|null $durationHours
	 */
	public $durationHours;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                = (int)$data['id'];
		$this->hostId            = (int)$data['hostId'];
		$this->serviceInstanceId = (int)$data['serviceInstanceId'] ?: null;
		$this->datetimeFrom      = new \Zend_Date($data['datetimeFrom']);
		$this->datetimeTo        = empty($data['datetimeTo']) ? null : new \Zend_Date($data['datetimeTo']);
		$this->durationHours     = (float)$data['durationHours'] ?: null;
		$this->rrule             = empty($data['rrule']) ? null : $data['rrule'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\DowntimePolicy
	 */
	public function getPolicy() {
		return new policy\DowntimePolicy($this);
	}

	/**
	 * Returns a validator for the current Downtime object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\DowntimeValidator
	 */
	public function getValidator() {
		return new DowntimeValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\DowntimeMapper
	 */
	public static function getMapper() {
		return datamapper\DowntimeMapper::getInstance();
	}

	/**
	 * Returns the Host of this Downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}

	/**
	 * Returns the HostServiceInstance of this Downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\hostServiceInstance
	 */
	public function getHostServiceInstance() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getByID(
			$this->serviceInstanceId
		)->allowRead();
	}

	/**
	 * Returns the HostService of this Downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\hostServiceInstance
	 */
	public function getHostService() {
		return $this->getHostServiceInstance()->getHostService();
	}

	/**
	 * Returns the Service of this Downtime
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return $this->getHostService()->getService();
	}
}
