<?php

namespace ch\monitoring\model;

use ch\monitoring\policy;

/**
 * An entity which verifies policy
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
abstract class EntityWithPolicy extends Entity implements policy\ModelPolicy {

	/**
	 * Stores whether the read permission has been checked already
	 *
	 * @var           bool $readPermissionChecked
	 */
	protected $readPermissionChecked = false;

	/**
	 * Stores whether the write permission has been checked already
	 *
	 * @var           bool $writePermissionChecked
	 */
	protected $writePermissionChecked = false;

	/**
	 * Stores the write permission
	 *
	 * @var           string $writePermission
	 */
	private $writePermission = null;

	/**
	 * Destructor
	 *
	 * Throws an exception if $permissionChecked is false
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        RuntimeException
	 * @returns       void
	 */
	public function __destruct() {
		if (!$this->readPermissionChecked) {
			throw new \RuntimeException(
				sprintf(
					tr('Read permission was not checked: %s'),
					var_export($this, true)
				)
			);
		}
	}

	/**
	 * Enforces permission to read this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       void
	 */
	public function enforceReadPermission($permissionName) {
		$this->getPolicy()->enforcePermission($permissionName);
		$this->readPermissionChecked = true;
	}

	/**
	 * Enforces permission to write this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       void
	 */
	public function enforceWritePermission($permissionName, $isSystemWide = false) {
		// Needs to be set before enforcing permission as otherwise misleading
		// readpermission message would be shown
		$this->readPermissionChecked  = true;

		if($isSystemWide) {
			$this->getPolicy()->enforcePermissionForWholeSystem($permissionName);
		}else {
			$this->getPolicy()->enforcePermission($permissionName);
		}
		$this->writePermissionChecked = true;
		$this->writePermission        = $permissionName;
	}

	/**
	 * Save entity but verify permission first
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function save() {
		$this->ensureWritePermission();
		return parent::save();
	}

	/**
	 * Delete entity but verify permission first
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function delete() {
		$this->ensureWritePermission();
		$this->deleteDependencies();
		parent::delete();
	}

	/**
	 * Function to delete dependencies of an entity.
	 */
	public function deleteDependencies() {}

	/**
	 * Enforces write permission check
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        RuntimeException When write permission was not checked
	 * @returns       void
	 */
	private function ensureWritePermission() {
		if (!$this->writePermissionChecked) {
			throw new \RuntimeException(
				tr('Write permission was not checked')
			);
		}

		$this->getPolicy()->enforcePermission($this->writePermission);
	}

	/**
	 * Allows reading without enforceReadPermission()
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       EntityWithPolicy
	 */
	public function allowRead() {
		$this->readPermissionChecked = true;
		return $this;
	}

	/**
	 * Allows writing without enforceWritePermission()
	 *
	 * Write implies read
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       EntityWithPolicy
	 */
	public function allowWrite($permissionName) {
		$this->readPermissionChecked  = true;
		$this->writePermissionChecked = true;
		$this->writePermission        = $permissionName;
		return $this;
	}

	/**
	it's probably smarter to leave this abstract


	 * Impelementation of getPolicy()
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       policy\ModelPolicy
	 
	public function getPolicy() {
	}*/
}
