<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\acl;
use ch\monitoring\util;
use ch\monitoring\model\Permission;
use ch\monitoring\validate\GuidelineValidator;
use org\adfinis\framework\config\ConfManager;


/**
 * Model for a Guideline
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Guideline extends EntityWithPolicy {
	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The content (main text) of the guideline
	 *
	 * @var           string $content
	 */
	public $content;


	/**
	 * The creator of the guideline
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The creation date of the guideline
	 *
	 * @var           Zend_Date $created
	 */
	public $created;


	/**
	 * The updating date of the guideline
	 *
	 * @var           Zend_Date $updated
	 */
	public $updated;

	/**
	 * Does the guideline need manual installation?
	 *
	 * @var         boolean $needsManualInstallation
	 */
	public $needsManualInstallation = false;


	/**
	 * Path for installation file for manual
	 * installation
	 *
	 * @var            string $filename
	 */
	public $filename = '';

	/**
	 * Command ID
	 *
	 * @var            int $commandId
	 */
	public $commandId;

	/**
	 * HostOSVersion ID
	 *
	 * @var            int $versionId
	 */
	public $versionId;

	/**
	 * Cache for the user object
	 *
	 * @var           ch\monitoring\model\User $user
	 */
	private $user = null;

	/**
	 * initializes the model
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Guideline
	 */
	function __construct() {
		$this->created = new \Zend_Date();
	}


	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                      = (int)$data['id'];
		$this->content                 = $data['content'];
		$this->userId                  = (int)$data['userId'];
		$this->created                 = new \Zend_Date($data['created'], \Zend_Date::ISO_8601);
		$this->updated                 = new \Zend_Date($data['updated'], \Zend_Date::ISO_8601);
		$this->needsManualInstallation = (bool)$data['needsManualInstallation'];
		$this->filename                = array_key_exists('filename', $data)
			? (string)$data['filename'] : '' ;
		$this->commandId               = (int)$data['commandId'];
		$this->versionId               = (int)$data['versionId'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\GuidelineValidator
	 */
	public function getValidator() {
		return new GuidelineValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\GuidelineMapper
	 */
	public static function getMapper() {
		return datamapper\GuidelineMapper::getInstance();
	}


	/**
	 * Returns the creator of the guideline
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       null|ch\monitoring\model\User
	 */
	public function getUser() {
		if(is_null($this->user) && $this->userId) {
			$this->user = datamapper\UserMapper::getInstance()->getByID($this->userId);
			$this->user->enforceReadPermission(Permission::USERS_READ);
		}

		return $this->user;
	}

	/**
	 * Returns HostOS Version of this guideline
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	public function getHostOSVersion() {
		return datamapper\HostOSVersionMapper::getInstance()->getByID($this->versionId)->allowRead();
	}

	/**
	 * Returns HostOS of this guideline
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	public function getHostOS() {
		$osVersion = $this->getHostOSVersion();
		return datamapper\HostOSMapper::getInstance()->getByID($osVersion->osId)->allowRead();
	}

	/**
	 * Returns Command of this guideline
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Command
	 */
	public function getCommand() {
		return datamapper\CommandMapper::getInstance()->getByID($this->commandId)->allowRead();
	}

	/**
	 * Returns the URI folder path of this guideline
	 *
	 * Follows the rule dirRootPath/hostOSname/osVersionShortName/commandName/
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       string URI of this guideline
	 */
	public function getGuidelineURI() {
		return sprintf(
			'%s/%s/%s/%s/',
			ConfManager::getVar('sm', 'command', 'path', 'uri'),
			$this->getHostOS()->name,
			$this->getHostOSVersion()->shortName,
			$this->getCommand()->name
		);
	}
}
