<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\util;
use ch\monitoring\intelligence;
use ch\monitoring\validate\HostValidator;

/**
 * Model for a host
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Host extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer this host belongs to.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The SLA assigned to this host.
	 *
	 * @var           int $slaId
	 */
	public $slaId;

	/**
	 * A host this host depends on.
	 *
	 * @var           int $dependencyId
	 */
	public $dependencyId;

	/**
	 * A satellite this host belongs to.
	 *
	 * @var           int $satelliteId
	 */
	public $satelliteId;

	/**
	 * The operating system which runs on this host.
	 *
	 * @var           int $osId
	 */
	public $osId;

	/**
	 * The operating system version.
	 *
	 * @var           int $osVersionId
	 */
	public $osVersionId;

	/**
	 * The maintenance cycle of this host.
	 *
	 * @var           int $maintenanceCycleId
	 */
	public $maintenanceCycleId;

	/**
	 * The type of this hsot
	 *
	 * @var           int $typeId
	 */
	public $typeId;

	/**
	 * Whether to inherit the SLA from customer.
	 *
	 * @var           bool $inheritSla
	 */
	public $inheritSla = false;

	/**
	 * The hostname.
	 *
	 * @var           string $hostname
	 */
	public $hostname;

	/**
	 * Short description of the host.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * The IPv4 address.
	 *
	 * @var           string $IPv4
	 */
	public $IPv4;

	/**
	 * The IPv6 address.
	 *
	 * @var           string $IPv6
	 */
	public $IPv6;

	/**
	* Whether to check the host availability.
	*
	* @var           bool $checkHostAlive
	*/
	public $checkHostAlive = true;

	/**
	 * Is this host a satellite?
	 *
	 * @var           bool $isSatellite
	 */
	public $isSatellite = false;

	/**
	 * Is this host enabled?
	 *
	 * @var           bool $isEnabled
	 */
	public $isEnabled = true;

	/**
	 * Is this host in trouble?
	 *
	 * @var           bool $inTrouble
	 */
	public $hasTrouble = false;

	/**
	 * Additional comment for this host.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * When was this host created?
	 *
	 * @var           Zend_Date $datetimeCreated
	 */
	public $datetimeCreated;

	/**
	 * Date of the last export
	 *
	 * @var           Zend_Date $exportDate
	 */
	public $exportDate = null;

	/**
	 * Current export version
	 *
	 * @var           int $version
	 */
	public $version = 0;

	/**
	 * The host password
	 *
	 * @var           string $password
	 */
	public $password;

	/**
	 * Is this host deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;

	/**
	 * URL to the documentation of this host.
	 *
	 * @var           string $wiki
	 */
	public $wiki;

	/**
	 * Cache for services in trouble
	 *
	 * @var           array $servicesInTrouble
	 */
	private static $servicesInTrouble = array();

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
	function __construct() {
		$this->datetimeCreated = new \Zend_Date();
	}

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                   = (int)$data['id'];
		$this->customerId           = (int)$data['customerId'];
		$this->slaId                = (int)$data['slaId'] ?: null;
		$this->dependencyId         = (int)$data['dependencyId'] ?: null;
		$this->satelliteId          = (int)$data['satelliteId'] ?: null;
		$this->osId                 = (int)$data['osId'];
		$this->osVersionId          = (int)$data['osVersionId'] ?: null;
		$this->typeId               = (int)$data['typeId'] ?: null;
		$this->hostname             = $data['hostname'];
		$this->description          = $data['description'];
		$this->IPv4                 = $data['IPv4'];
		$this->IPv6                 = $data['IPv6'] ?: null;
		$this->exportDate           = new \Zend_Date($data['exportDate'], \Zend_Date::ISO_8601);
		$this->version              = (int)$data['version'] ?: null;
		$this->password             = $data['password'];
		$this->datetimeCreated      = new \Zend_Date($data['datetimeCreated'], \Zend_Date::ISO_8601);
		$this->comment              = $data['comment'] ?: null;
		$this->isSatellite          = (bool)$data['isSatellite'];
		$this->isEnabled            = (bool)$data['isEnabled'];
		$this->inheritSla           = (bool)$data['inheritSla'];
		$this->hasTrouble           = (bool)$data['hasTrouble'];
		$this->checkHostAlive       = (bool)$data['checkHostAlive'];
		$this->wiki                 = $data['wiki'] ?: null;
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\HostPolicy
	 */
	public function getPolicy() {
		return new policy\HostPolicy($this);
	}

	/**
	 * Returns a validator for the current Host object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostValidator
	 */
	public function getValidator() {
		return new HostValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostMapper
	 */
	public static function getMapper() {
		return datamapper\HostMapper::getInstance();
	}

	/**
	 * Returns the Customer of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the SLA of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}

	/**
	 * Returns the SLA of this Host (considering inheritance)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getRealSLA() {
		if ($this->slaId === null) {
			return $this->getCustomer()->getSLA();
		}
		else {
			return $this->getSLA();
		}
	}

	/**
	 * Returns the Host this Host depends on
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getDependency() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->dependencyId
		)->allowRead();
	}

	/**
	 * Returns all dependency host ids
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getDependencies() {
		return array($this->getDependency());
	}

	/**
	 * Does this host have dependencies?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function hasDependencies() {
		return $this->dependencyId !== null;
	}

	/**
	 * Returns the Satellite of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getSatellite() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->satelliteId
		)->allowRead();
	}

	/**
	 * Returns the operating system (HostOS) of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOS
	 */
	public function getHostOS() {
		return datamapper\HostOSMapper::getInstance()->getByID(
			$this->osId
		)->allowRead();
	}

	/**
	 * Returns the operating system version (HostOSVersion) of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOS
	 */
	public function getHostOSVersion() {
		return datamapper\HostOSVersionMapper::getInstance()->getByID(
			$this->osVersionId
		)->allowRead();
	}

	/**
	 * Returns the HostType of this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostType
	 */
	public function getHostType() {
		return datamapper\HostTypeMapper::getInstance()->getByID(
			$this->typeId
		)->allowRead();
	}

	/**
	 * Returns all assigned HostServices
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isNrpe Optional. Defaults to null.
	 * @param         bool $isEnabled Optional.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHostServices() {
		return datamapper\HostServiceMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns all assigned HostServiceInstances
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isNrpe Optional. Defaults to null.
	 * @param         bool $isEnabled Optional.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHostServiceInstances($isNrpe = null, $isEnabled = null) {
		return datamapper\HostServiceInstanceMapper::getInstance()->getAll(
			null,
			$this->id,
			null,
			$isNrpe,
			null,
			$isEnabled,
			null
		);
	}

	/**
	 * Returns all assigned HostserviceInstances as array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $isEnabled Optional.
	 * @returns       array
	 */
	public function getHostServiceInstancesAsArray($isNrpe = null, $isEnabled = null) {
		$hostServiceInstances = $this->getHostServiceInstances($isNrpe, $isEnabled);

		$services = array();
		foreach ($hostServiceInstances as $hostServiceInstance) {
			$services[$hostServiceInstance->id] = $hostServiceInstance->getObjectName();
		}

		return $services;
	}

	/**
	 * Assigns a service group to host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceGroupId
	 * @returns       void
	 */
	public function assignServiceGroup($serviceGroupId) {
		datamapper\ServiceGroupMapper::getInstance()->assignToHost(
			$serviceGroupId,
			$this->id
		);
	}

	/**
	 * Returns the current emergency team
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getCurrentEmergencyTeam() {
		return datamapper\CustomerTeamMapper::getInstance()->getCurrentTeam(
			$this->id
		);
	}

	/**
	 * Returns assigned satellite hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getSatelliteHosts() {
		return $this->getMapper()->getAll(false, false, false, $this->id);
	}

	/**
	 * Returns assigned satellite hosts as an array of ids
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getSatelliteHostIds($permissionName=Permission::HOSTS_READ) {
		$hosts = $this->getMapper()->getAll(
			false, false, false, $this->id, null, $permissionName
		);
		return util\IteratorUtil::getKeyValueArray(
			$hosts,
			'id',
			'id'
		);
	}

	/**
	 * Returns whether a host id is reachable from this host
	 *
	 * Returns true when the host is neither behind a satellite nor a satellite.
	 * When this host is a satellite or behind a satellite, this function returns
	 * - true if the given host is also behind the satellite,
	 * - false otherwise.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function isReachable($hostId, $permissionName=Permission::HOSTS_READ) {
		// If not in a satellite context, a host is always reachable
		if (!$this->isSatellite && $this->satelliteId === null) {
			return true;
		}

		// A host is reachable from itself
		if ($this->id === $hostId) {
			return true;
		}

		if ($this->isSatellite) {
			$hostList = $this->getSatelliteHostIds($permissionName);
		}
		else
		{
			$hostList = array_merge(
				$this->getSatellite()->getSatelliteHostIds($permissionName),
				array($this->satelliteId)
			);
		}

		return in_array($hostId, $hostList);
	}

	/**
	 * Returns all BREComponents which this host is assigned to
	 *
	 * @return        ch\monitoring\model\BREComponent
	 */
	public function getBREComponents() {
		return datamapper\BREComponentMapper::getInstance()->getByHost($this);
	}

	/**
	 * Returns all maintenance tickets for this host
	 *
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getMaintenanceTicketHosts() {
		return datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
			null,
			$this->id
		);
	}

	/**
	 * Returns the last maintenance ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       model\MaintenanceTicketHost
	 */
	public function getLatestMaintenanceTicketHost() {
		$tickets = datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
			null,
			$this->id,
			array(
				MaintenanceState::STATE_NEW
			)
		);

		if (count($tickets) == 0) {
			return null;
		}

		$tickets->setOrderBy('dateDue.DESC');
		$tickets->next();
		return $tickets->current();
	}

	/**
	 * Returns all assigned services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getServices() {
		return datamapper\ServiceMapper::getInstance()->getAll(
			$this->id,
			null,
			null,
			null,
			null,
			Permission::IMPORT
		);
	}

	/**
	 * Returns all assigned services as a id => name array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getServicesAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getServices()
		);
	}

	/**
	 * Returns all assigned services which are in trouble
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getServicesInTrouble() {
		if (!array_key_exists($this->id, self::$servicesInTrouble)) {
			self::$servicesInTrouble[$this->id] = datamapper\HostServiceInstanceMapper::getInstance()->getAll(
				null,
				$this->id,
				true
			)->toArray();
		}

		return self::$servicesInTrouble[$this->id];
	}

	/**
	 * Returns the down times
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $serviceId Optional
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getDowntimes($serviceId = null, $dateFrom = null, $dateTo = null) {
		return datamapper\DowntimeMapper::getInstance()->getAll(
			null,
			$this->id,
			$serviceId,
			$dateFrom,
			$dateTo,
			null
		);
	}

	/**
	 * Returns the MaintenanceCycle of this Host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getMaintenanceGroups() {
		return datamapper\MaintenanceGroupMapper::getInstance()->getAllByHost(
			$this->id
		);
	}

	/**
	 * Returns the MaintenanceCycle of this Host
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         ch\monitoring\model\MaintenanceTicket $ticket
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	public function getMaintenanceGroupByTicket($ticket) {
		return datamapper\MaintenanceGroupMapper::getInstance()->getMaintenanceGroupByTicket(
			$ticket->id
		);
	}

	/**
	 * Returns the count of alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $openOnly Optional
	 * @returns       int
	 */
	public function getAlertCount($openOnly = null) {
		return datamapper\AlertMapper::getInstance()->count(null, $this->id, null, $openOnly);
	}

	/**
	 * Returns the count of assigned services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $hasTrouble Optional
	 * @returns       int
	 */
	public function getServiceCount($hasTrouble = null) {
		return datamapper\HostServiceInstanceMapper::getInstance()->count(null, $this->id, $hasTrouble);
	}

	/**
	 * Returns the total reaction time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       int Reaction time in seconds
	 */
	public function getReactionTime($dateFrom = null, $dateTo = null) {
		return datamapper\AlertMapper::getInstance()->getReactionTime(
			datamapper\AlertMapper::TYPE_REACTION,
			$dateFrom,
			$dateTo,
			null,
			$this->id
		);
	}

	/**
	 * Returns the total time this host exists
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int Existence time in seconds
	 */
	public function getExistenceTime() {
		$now = new \Zend_Date();
		$now->sub($this->datetimeCreated);
		return $now->get(\Zend_Date::TIMESTAMP);
	}

	/**
	 * Returns the availability of this host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @throws        \OutOfBoundsException
	 * @returns       double [0, 1]
	 */
	public function getAvailability($dateFrom, $dateTo) {
		$fact = new intelligence\FactAvailability();
		$fact->addCondition('host', 'hostId',    $this->id);
		$fact->addCondition('host', 'serviceId', null);
		$availability = $fact->get(intelligence\Fact::AVAILABILITY);

		$count = $availability['available'] + $availability['unavailable'];

		if ($count == 0) {
			throw new \OutOfBoundsException(
				tr('No information available in given date range')
			);
		}

		return $availability['available'] / $count;
	}

	/**
	 * Returns the availability of this host
	 *
	 * Null when unknown
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @returns       double [0, 1]
	 */
	public function getAvailabilityOrNull($dateFrom, $dateTo) {
		try {
			return $this->getAvailability($dateFrom, $dateTo);
		}
		catch (\OutOfBoundsException $e) {
			return null;
		}
	}

	/**
	 * Returns the pnp4nagios graph for this host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @returns       \Zend_Http_Response
	 */
	public function getGraph($dateFrom, $dateTo) {
		return util\PNP4Nagios::getGraph(
			$this->hostname,
			'_HOST_',
			$dateFrom,
			$dateTo
		);
	}

	/**
	 * Returns the services assigned to this host and relevant for availability
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHostServicesAvailability() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getAll(
			null,
			$this->id,
			null,
			null,
			true,
			null,
			null
		);
	}

	/**
	 * Returns the  template-checklists which are assigned to this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getTemplateChecklists() {
		return datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			$this->id,
			null,
			true,
			false
		);
	}

	/**
	 * Returns all questions which are assigned to this host
	 *
	 * @param         bool $recursive Recursively follow parents
	 * @param         bool $primaryOnly Only return questions assigned directly to the primary checklist
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by denis:   With recursion, $question->isExcluded() does not work the way it is done now
	 * @returns       array
	 */
	public function getQuestions($recursive = false, $primaryOnly = false) {
		if ($recursive) {
			throw new \RuntimeException(
				tr("Recursion does not yet work")
			);
		}

		if ($primaryOnly) {
			$checklists = array($this->getPrimaryChecklist());
		}
		else {
			$checklists = $this->getChecklists();
		}

		list($includeArray, $excludeArray) = $this->getQuestionsForChecklists(
			$checklists,
			$recursive
		);

		if (!$primaryOnly) {
			list($templateIncludeArray, $templateExcludeArray) = $this->getQuestionsForChecklists(
				$this->getTemplateChecklists(),
				$recursive
			);

			// Using + here to not loose the original indexing
			// (IDs of the questions)
			$includeArray = $includeArray + $templateIncludeArray;
			$excludeArray = $excludeArray + $templateExcludeArray;
		}

		foreach ($includeArray as $id => $question) {
			if (array_key_exists($id, $excludeArray) && array_key_exists($id, $includeArray)) {
				unset($includeArray[$id]);
			}
		}

		return $includeArray;
	}

	/**
	 * Returns all checklists which are assigned to this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       db\adfinis\framework\db\DbIterator
	 */
	public function getChecklists() {
		return datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			$this->id,
			null,
			false,
			false
		);
	}

	/**
	 * Returns this hosts's primary checklist
	 *
	 * If it doesn't exist yet, it is created
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       model\MaintenanceChecklist
	 */
	public function getPrimaryChecklist() {
		$checklists = datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			$this->id,
			null,
			false,
			false
		)->toArray();

		if (count($checklists) == 0) {
			$checklist = new MaintenanceChecklist();
			$checklist->allowRead();

			$checklist->name       = sprintf('Primary checklist %s', microtime());
			$checklist->comment    = sprintf("Primary checklist for host '%s'", $this->hostname);
			$checklist->isTemplate = false;
			$checklist->getMapper()->save($checklist);

			datamapper\MaintenanceChecklistMapper::getInstance()->assignHost($this->id, $checklist->id);

			return $checklist;
		}

		return $checklists[0];
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'dependencyId' => array(
				'type' => 'relation',
				'mapper' => 'HostMapper'
			),

			'satelliteId' => array(
				'type' => 'relation',
				'mapper' => 'HostMapper'
			),

			'osId' => array(
				'type' => 'relation',
				'mapper' => 'HostOSMapper'
			),

			'osVersionId' => array(
				'type' => 'relation',
				'mapper' => 'HostOSVersionMapper'
			),

			'typeId' => array(
				'type' => 'relation',
				'mapper' => 'HostTypeMapper'
			),

			'inheritSla' => array(
				'type' => 'boolean'
			),

			'checkHostAlive' => array(
				'type' => 'boolean'
			),

			'isSatellite' => array(
				'type' => 'boolean'
			),

			'isEnabled' => array(
				'type' => 'boolean'
			),

			'hasTrouble' => array(
				'type' => 'boolean'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			)
		);
	}

	/**
	 * Helper method for getting all included and excluded
	 * questions for given checklists as a tuple (yeah, right,
	 * PHP has no such thing.. we return an array instead..)
	 *
	 * @param         \org\adfinis\framework\db\SortingIterator $checklists
	 *                                      Checklists to fetch the questions
	 *                                      from
	 * @param         bool $getRecursive    Wether to get the questions
	 *                                      recursive or not
	 *
	 * @returns       array                 The questions to include and
	 *                                      exclude
	 */
	private function getQuestionsForChecklists($checklists, $getRecursive) {
		$includeArray = array();
		$excludeArray = array();

		foreach ($checklists as $checklist) {
			if ($getRecursive) {
				$questions = $checklist->getQuestionsRecursive();
			}
			else {
				$questions = $checklist->getQuestions();
			}

			foreach ($questions as $question) {
				if ($question->isExcluded($checklist->id, $this->id)) {
					$excludeArray[$question->id] = $question;
				}
				else {
					$includeArray[$question->id] = $question;
				}
			}
		}

		return array($includeArray, $excludeArray);
	}

	/**
	 * Computed field which express if a host is enabled or disabled
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 *
	 * @returns       bool
	 */
	public function isEnabled() {
		if($this->getCustomer()->isEnabled()) {
			return $this->isEnabled;
		}

		return false;
	}

	/**
	 * Returns all alerts which are related to this host service instance
	 *
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAlerts($openOnly=null) {
		return datamapper\AlertMapper::getInstance()->getAll(
			null,
			$openOnly,
			$this->id
		);
	}

	/**
	 * Returns all dependant hosts of current host
	 *
	 * @returns DbIterator
	 */
	public function getDependantHosts() {
		return $this->getMapper()->getAll(
			null,
			false,
			false,
			null,
			null,
			Permission::HOSTS_READ,
			null,
			null,
			null,
			null,
			$this->id
		);
	}
}
