<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\HostOSVersionValidator;

/**
 * Model for a Host OS
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostOSVersion extends EntityWithPolicy {
	/**
	 * Constant for version 'ALL'
	 *
	 * @const        int VERSION_ALL
	 */
	const VERSION_ALL = 0;

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The host OS
	 *
	 * @var           int $osId
	 */
	public $osId;

	/**
	 * The os name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The short name of the OS
	 *
	 * @var
	 */
	public $shortName;

	/**
	 * The architecture of this OS version
	 *
	 * @var
	 */
	public $architectureId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                   = (int)$data['id'];
		$this->osId                 = (int)$data['osId'];
		$this->name                 = $data['name'];
		$this->shortName            = $data['shortName'];
		$this->architectureId       = (int) $data['architectureId'];
	}

	/**
	 * Returns a validator for the current HostOSVersion object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostOSVersionValidator
	 */
	public function getValidator() {
		return new HostOSVersionValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostOSVersionMapper
	 */
	public static function getMapper() {
		return datamapper\HostOSVersionMapper::getInstance();
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns the corresponding HostOS
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       model\HostOS
	 */
	public function getHostOS() {
		return datamapper\HostOSMapper::getInstance()->getByID(
			$this->osId
		)->allowRead();
	}
}
