<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\HostServiceValidator;
use ch\monitoring\util;
use ch\monitoring\intelligence;

/**
 * Model for a service assigned to a host
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostService extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The host this service belongs to.
	 *
	 * @var           int $customerId
	 */
	public $hostId;

	/**
	 * The service assigned to the host.
	 *
	 * @var           int $serviceId
	 */
	public $serviceId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id              = (int)$data['id'];
		$this->hostId          = (int)$data['hostId'];
		$this->serviceId       = (int)$data['serviceId'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\HostServicePolicy
	 */
	public function getPolicy() {
		return new policy\HostServicePolicy($this);
	}

	/**
	 * Returns a validator for the current HostService object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostServiceValidator
	 */
	public function getValidator() {
		return new HostServiceValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostServiceMapper
	 */
	public static function getMapper() {
		return datamapper\HostServiceMapper::getInstance();
	}

	/**
	 * Returns the Host of this HostService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}

	/**
	 * Returns the Service of this HostService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return datamapper\ServiceMapper::getInstance()->getByID(
			$this->serviceId
		)->allowRead();
	}

	/**
	 * Returns the assigned instances
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getInstances() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the assigned instances as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getInstancesAsArray() {
		$hostServiceInstances = $this->getInstances();

		$services = array();
		foreach ($hostServiceInstances as $hostServiceInstance) {
			$services[$hostServiceInstance->id] = $hostServiceInstance->getObjectName();
		}

		return $services;
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'hostId' => array(
				'type' => 'relation',
				'mapper' => 'HostMapper',
				'name' => 'hostname'
			),

			'serviceId' => array(
				'type' => 'relation',
				'mapper' => 'ServiceMapper'
			)
		);
	}
}
