<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\HostServiceArgumentValidator;

/**
 * Model for an argument of a service assigned to a host
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostServiceArgument extends EntityWithPolicy {

	/**
	 * Primary key
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The HostService instance this argument belongs to
	 *
	 * @var           int $instanceId
	 */
	public $instanceId;

	/**
	 * The ServiceArgument this argument belongs to
	 *
	 * @var           int $serviceArgumentId
	 */
	public $serviceArgumentId;

	/**
	 * The assigned value
	 *
	 * @var           string $assignedValue
	 */
	public $assignedValue;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return $this->id == (int)$data['id'];
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array(
			'id' => $this->id
		);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                = (int)$data['id'];
		$this->instanceId        = (int)$data['instanceId'];
		$this->serviceArgumentId = (int)$data['serviceArgumentId'];
		$this->assignedValue     = $data['assignedValue'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\HostServiceArgumentPolicy
	 */
	public function getPolicy() {
		return new policy\HostServiceArgumentPolicy($this);
	}

	/**
	 * Returns a validator for the current HostServiceArgument object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostServiceArgumentValidator
	 */
	public function getValidator() {
		return new HostServiceArgumentValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostServiceArgumentMapper
	 */
	public static function getMapper() {
		return datamapper\HostServiceArgumentMapper::getInstance();
	}

	/**
	 * Returns the HostService of this HostServiceArgument
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostServiceInstance
	 */
	public function getHostServiceInstance() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getByID(
			$this->instanceId
		)->allowRead();
	}

	/**
	 * Returns the ServiceArgument of this HostServiceArgument
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ServiceArgument
	 */
	public function getServiceArgument() {
		return datamapper\ServiceArgumentMapper::getInstance()->getByID(
			$this->serviceArgumentId
		)->allowRead();
	}
}
