<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\HostServiceInstanceValidator;
use ch\monitoring\util;
use ch\monitoring\intelligence;

/**
 * Model for a service instance
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostServiceInstance extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The host this service belongs to.
	 *
	 * @var           int $customerId
	 */
	public $hostServiceId;

	/**
	 * The sla for this assignment.
	 *
	 * @var           int $slaId
	 */
	public $slaId;

	/**
	 * When was this host created?
	 *
	 * @var           Zend_Date $datetimeCreated
	 */
	public $datetimeCreated;

	/**
	 * Name
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Passive service?
	 *
	 * @var           bool $isPassive
	 */
	public $isPassive = false;

	/**
	 * Is this Service assignment enabled?
	 *
	 * @var           bool $isEnabled
	 */
	public $isEnabled = true;

	/**
	 * Is this Service on the host in trouble?
	 *
	 * @var           bool $hasTrouble
	 */
	public $hasTrouble = false;

	/**
	 * Is this service used for availability calculations?
	 *
	 * @var           bool $forAvailability
	 */
	public $forAvailability = false;

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
	function __construct() {
		$this->datetimeCreated = new \Zend_Date();
	}

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * Returns the name of the host service instance.
	 * If no name was set, the name of the service is returned instead.
	 *
	 * @return        string the name of this host service instance
	 */
	public function getName() {
		if (empty($this->name)) {
			return $this->getService()->name;
		}
		else {
			return $this->name;
		}
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id              = (int)$data['id'];
		$this->hostServiceId   = (int)$data['hostServiceId'];
		$this->slaId           = (int)$data['slaId'] ?: null;
		$this->name            = $data['name'];
		$this->isPassive       = (bool)$data['isPassive'];
		$this->isEnabled       = (bool)$data['isEnabled'];
		$this->hasTrouble      = (bool)$data['hasTrouble'];
		$this->forAvailability = (bool)$data['forAvailability'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\HostServiceInstancePolicy
	 */
	public function getPolicy() {
		return new policy\HostServiceInstancePolicy($this);
	}

	/**
	 * Returns a validator for the current HostServiceInstance object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostServiceInstanceValidator
	 */
	public function getValidator() {
		return new HostServiceInstanceValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostServiceInstanceMapper
	 */
	public static function getMapper() {
		return datamapper\HostServiceInstanceMapper::getInstance();
	}

	/**
	 * Returns the HostService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostService
	 */
	public function getHostService() {
		return datamapper\HostServiceMapper::getInstance()->getByID(
			$this->hostServiceId
		)->allowRead();
	}
	
	/**
	 * Returns the Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostService
	 */
	public function getHost() {
		return $this->getHostService()->getHost();
	}

	/**
	 * Returns the Service of this HostService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return $this->getHostService()->getService();
	}

	/**
	 * Returns the SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}

	/**
	 * Returns the SLA of this Service (considering inheritance)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getRealSLA() {
		if ($this->slaId === null) {
			if ($this->getHost()->slaId === null) {
				return $this->getHost()->getCustomer()->getSLA();
			}
			else {
				return $this->getHost()->getSLA();
			}
		}
		else {
			return $this->getSLA();
		}
	}

	/**
	 * Returns the arguments which belong to this host service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getArguments() {
		return datamapper\HostServiceArgumentMapper::getInstance()->getAll($this->id);
	}

	/**
	 * Returns the arguments which belong to this host service as a string
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getArgumentsAsString() {
		return $this->getMapper()->getArgumentsAsString($this->id);
	}

	/**
	 * Returns all alerts which are related to this host service instance
	 *
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAlerts($openOnly=null) {
		return datamapper\AlertMapper::getInstance()->getAll(
			null,
			$openOnly,
			$this->getHostService()->hostId,
			null,
			null,
			$this->id
		);
	}

	/**
	 * Returns the count of alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $openOnly Optional
	 * @returns       int
	 */
	public function getAlertCount($openOnly = null) {
		return datamapper\AlertMapper::getInstance()->count(
			null,
			$this->getHostService()->hostId,
			$this->id,
			$openOnly
		);
	}

	/**
	 * Returns the total reaction time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       int Reaction time in seconds
	 */
	public function getReactionTime($dateFrom = null, $dateTo = null) {
		return datamapper\AlertMapper::getInstance()->getReactionTime(
			datamapper\AlertMapper::TYPE_REACTION,
			$dateFrom,
			$dateTo,
			null,
			$this->getHostService()->hostId,
			$this->getHostService()->serviceId
		);
	}

	/**
	 * Returns the total down time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom Optional
	 * @param         \Zend_Date $dateTo Optional
	 * @returns       int Down time in seconds
	 */
	public function getDownTime($dateFrom = null, $dateTo = null) {
		return datamapper\AlertMapper::getInstance()->getReactionTime(
			datamapper\AlertMapper::TYPE_DOWN,
			$dateFrom,
			$dateTo,
			null,
			$this->getHostService()->hostId,
			$this->getHostService()->serviceId
		);
	}

	/**
	 * Returns all Downtimes this ServiceInstance is assigned to
	 *
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getDowntimes() {
		return datamapper\DowntimeMapper::getInstance()->getAll(
			null,
			null,
			$this->id
		);
	}


	/**
	 * Returns the pnp4nagios graph for this host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @returns       \Zend_Http_Response
	 */
	public function getGraph($dateFrom, $dateTo) {
		return util\PNP4Nagios::getGraph(
			$this->getHost()->hostname,
			$this->getObjectName(),
			$dateFrom,
			$dateTo
		);
	}

	/**
	 * Returns the availability of this service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @throws        \OutOfBoundsException
	 * @returns       double [0, 1]
	 */
	public function getAvailability($dateFrom, $dateTo) {
		$fact = new intelligence\FactAvailability();
		$fact->addCondition('host', 'hostId',            $this->getHostService()->hostId);
		$fact->addCondition('host', 'serviceInstanceId', $this->id);
		$availability = $fact->get(intelligence\Fact::AVAILABILITY);

		$count = $availability['available'] + $availability['unavailable'];

		if ($count == 0) {
			throw new \OutOfBoundsException(
				tr('No information available in given date range')
			);
		}

		return $availability['available'] / $count;
	}

	/**
	 * Returns the availability of this service
	 *
	 * Null when unknown
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @returns       double [0, 1]
	 */
	public function getAvailabilityOrNull($dateFrom, $dateTo) {
		try {
			return $this->getAvailability($dateFrom, $dateTo);
		}
		catch (\OutOfBoundsException $e) {
			return null;
		}
	}

	/**
	 * Returns the name to be used in nagios/icinga context
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getObjectName() {
		if ($this->name === null) {
			return $this->getService()->name;
		}

		return sprintf(
			"%s__%s",
			$this->getService()->name,
			$this->name
		);
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'hostServiceId' => array(
				'type' => 'relation',
				'mapper' => 'HostServiceMapper'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'isPassive' => array(
				'type' => 'boolean'
			),
			
			'forAvailability' => array(
				'type' => 'boolean'
			),

			'hasTrouble' => array(
				'type' => 'boolean'
			),
			
			'isEnabled' => array(
				'type' => 'boolean'
			)
		);
	}

	/**
	 * Returns the name to be used in nagios/icinga context fitting for nrpe checks.
	 *
	 * @author        Sven Osterwalder<sven.osterwalder@adfinis-sygroup.ch>
	 *
	 * @returns       string
	 */
	public function getObjectNameForExport() {
		$serviceInstanceName = "";

		if ($this->name === null) {
			$serviceInstanceName = $this->getService()->name;
		}
		else {
			$serviceInstanceName = $this->name;

			// Remove all special characters, except
			// 0-9, a-z and A-Z
			$serviceInstanceName = preg_replace('/[^a-zA-Z0-9\s\s+]/s', '', $serviceInstanceName);

			// Convert whitespaces to underscores
			$serviceInstanceName = preg_replace('/\s+/', '_', $serviceInstanceName);

			// Convert to lower case
			$serviceInstanceName = strtolower($serviceInstanceName);

			$serviceInstanceName = sprintf(
				"%s__%s",
				$this->getService()->name,
				$serviceInstanceName
			);
		}

		return $serviceInstanceName;
	}

	/**
	 * Returns all BREComponents which this host-service-instance is assigned to
	 *
	 * @return        ch\monitoring\model\BREComponent
	 */
	public function getBREComponents() {
		return datamapper\BREComponentMapper::getInstance()->getByService($this);
	}
}
