<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ImportCommandValidator;
use ch\monitoring\util;

/**
 * Model for a command to be imported
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportCommand extends EntityForImport {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The file this command belongs to.
	 *
	 * @var           int $fileId
	 */
	public $fileId;

	/**
	 * An optionally associated command
	 *
	 * @var           int $commandId
	 */
	public $commandId;

	/**
	 * The command name
	 *
	 * @var           string $command_name
	 */
	public $command_name;

	/**
	 * The command line
	 *
	 * @var           string $command_line
	 */
	public $command_line;

	/**
	 * Template name
	 *
	 * @var           bool $name
	 */
	public $name;

	/**
	 * Is this command to be registered?
	 *
	 * @var           bool $register
	 */
	public $register = false;

	/**
	 * The conflicting command id if this command generates a conflict
	 *
	 * @var           bool $conflictCommandId
	 */
	public $conflictCommandId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                = (int)$data['id'] ?: null;
		$this->fileId            = (int)$data['fileId'];
		$this->commandId         = (int)$data['commandId'] ?: null;
		$this->command_name      = $data['command_name'];
		$this->command_line      = $data['command_line'];
		$this->name              = $data['name'] ?: null;
		$this->register          = (bool)$data['register'];
		$this->conflictCommandId = (int)$data['conflictCommandId'] ?: null;
	}

	/**
	 * Implemented from EntityForImport
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	protected function getKeys() {
		return array('id', 'fileId', 'commandId', 'command_name',
					 'command_line', 'name', 'register', 'conflictCommandId');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Command object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\CommandValidator
	 */
	public function getValidator() {
		return new ImportCommandValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\CommandMapper
	 */
	public static function getMapper() {
		return datamapper\ImportCommandMapper::getInstance();
	}

	/**
	 * Returns the File of this ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ImportFile
	 */
	public function getFile() {
		return datamapper\ImportFileMapper::getInstance()->getByID(
			$this->fileId
		)->allowRead();
	}

	/**
	 * Returns the Command of this ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Command
	 */
	public function getCommand() {
		return datamapper\CommandMapper::getInstance()->getByID(
			$this->commandId
		)->allowRead();
	}

	/**
	 * Returns the conflicting command of this ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Command
	 */
	public function getConflictCommand() {
		return datamapper\CommandMapper::getInstance()->getByID(
			$this->conflictCommandId
		)->allowRead();
	}

	/**
	 * Return all services which use this command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getServices() {
		return datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			$this->command_name,
			true
		);
	}

	/**
	 * Return all services which use this command (names)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getServiceNames() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getServices(),
			'id',
			'service_description'
		);
	}

	/**
	 * Rename this Command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $newName
	 * @returns       void
	 */
	public function rename($newName) {
		datamapper\ImportCommandMapper::getInstance()->rename(
			$this->id,
			$newName
		);
	}

	/**
	 * Returns all the commands included by this one
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getIncludedCommands() {
		return datamapper\ImportCommandMapper::getInstance()->getAll(
			null,
			null,
			null,
			$this->id
		);
	}
}
