<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ImportHostValidator;
use ch\monitoring\util;

/**
 * Model for a host to be imported
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportHost extends EntityForImport {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The file this host belongs to.
	 *
	 * @var           int $fileId
	 */
	public $fileId;

	/**
	 * An optionally associated host
	 *
	 * @var           int $hostId
	 */
	public $hostId;

	/**
	 * An associated operating system
	 *
	 * @var           int $osId
	 */
	public $osId;

	/**
	 * An associated customer
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The host name
	 *
	 * @var           string $host_name
	 */
	public $host_name;

	/**
	 * The alias
	 *
	 * @var           string $alias
	 */
	public $alias;

	/**
	 * The address
	 *
	 * @var           string $address
	 */
	public $address;

	/**
	 * The parents
	 *
	 * @var           string $parents
	 */
	public $parents;

	/**
	 * The host groups
	 *
	 * @var           string $hostgroups
	 */
	public $hostgroups;

	/**
	 * Template name
	 *
	 * @var           bool $name
	 */
	public $name;

	/**
	 * Is this host to be registered?
	 *
	 * @var           bool $register
	 */
	public $register = false;

	/**
	 * The conflicting host id if this host generates a conflict
	 *
	 * @var           bool $conflictHostId
	 */
	public $conflictHostId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id             = (int)$data['id'] ?: null;
		$this->fileId         = (int)$data['fileId'];
		$this->hostId         = (int)$data['hostId'] ?: null;
		$this->osId           = (int)$data['osId'] ?: null;
		$this->customerId     = (int)$data['customerId'] ?: null;
		$this->host_name      = $data['host_name'] ?: null;
		$this->alias          = $data['alias'] ?: null;
		$this->address        = $data['address'] ?: null;
		$this->parents        = $data['parents'] ?: null;
		$this->hostgroups     = $data['hostgroups'] ?: null;
		$this->name           = $data['name'] ?: null;
		$this->register       = (bool)$data['register'];
		$this->conflictHostId = (int)$data['conflictHostId'] ?: null;
	}

	/**
	 * Implemented from EntityForImport
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	protected function getKeys() {
		return array('id', 'fileId', 'hostId', 'host_name', 'alias', 'address',
					 'parents', 'hostgroups', 'name', 'register',
					 'conflictHostId');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Host object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostValidator
	 */
	public function getValidator() {
		return new ImportHostValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\HostMapper
	 */
	public static function getMapper() {
		return datamapper\ImportHostMapper::getInstance();
	}

	/**
	 * Returns the File of this ImportCommand
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ImportFile
	 */
	public function getFile() {
		return datamapper\ImportFileMapper::getInstance()->getByID(
			$this->fileId
		)->allowRead();
	}

	/**
	 * Returns the Host of this ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}

	/**
	 * Returns the conflicting host of this ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getConflictHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->conflictHostId
		)->allowRead();
	}

	/**
	 * Returns the Customer of this ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the operating system of this ImportHost
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOS
	 */
	public function getHostOS() {
		return datamapper\HostOSMapper::getInstance()->getByID(
			$this->osId
		)->allowRead();
	}

	/**
	 * Return all assigned services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $emptyName Whether to return services with empty name
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getServices($emptyName = null) {
		return datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			$this->host_name,
			null,
			false,
			null,
			null,
			null,
			$emptyName
		);
	}

	/**
	 * Return all assigned services (name array)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $emptyName Whether to return services with empty name
	 * @returns       array
	 */
	public function getServiceNames($emptyName = null) {
		return util\IteratorUtil::getKeyValueArray(
			$this->getServices($emptyName),
			'id',
			'service_description'
		);
	}

	/**
	 * Rename this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $newName
	 * @returns       void
	 */
	public function rename($newName) {
		datamapper\ImportHostMapper::getInstance()->rename(
			$this->id,
			$newName
		);
	}

	/**
	 * Returns all the hosts included by this one
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getIncludedHosts() {
		return datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			null,
			null,
			null,
			$this->id
		);
	}
}
