<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ImportHostgroupValidator;
use ch\monitoring\util;

/**
 * Model for a host group to be imported
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportHostgroup extends EntityForImport {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The file this host group belongs to.
	 *
	 * @var           int $fileId
	 */
	public $fileId;

	/**
	 * An optionally associated host group
	 *
	 * @var           int $hostgroupId
	 */
	public $hostgroupId;

	/**
	 * The host group name
	 *
	 * @var           string $hostgroup_name
	 */
	public $hostgroup_name;

	/**
	 * The alias
	 *
	 * @var           string $alias
	 */
	public $alias;

	/**
	 * The members
	 *
	 * @var           string $members
	 */
	public $members;

	/**
	 * The host groups
	 *
	 * @var           string $hostgroup_members
	 */
	public $hostgroup_members;

	/**
	 * Template name
	 *
	 * @var           bool $name
	 */
	public $name;

	/**
	 * Is this host group to be registered?
	 *
	 * @var           bool $register
	 */
	public $register = false;

	/**
	 * The conflicting host group id if this host generates a conflict
	 *
	 * @var           bool $conflictHostgroupId
	 */
	public $conflictHostgroupId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                  = (int)$data['id'] ?: null;
		$this->fileId              = (int)$data['fileId'];
		$this->hostgroupId         = (int)$data['hostgroupId'] ?: null;
		$this->hostgroup_name      = $data['hostgroup_name'];
		$this->alias               = $data['alias'];
		$this->members             = $data['members'] ?: null;
		$this->hostgroup_members   = $data['hostgroup_members'] ?: null;
		$this->name                = $data['name'] ?: null;
		$this->register            = (bool)$data['register'];
		$this->conflictHostgroupId = (int)$data['conflictHostgroupId'] ?: null;
	}

	/**
	 * Implemented from EntityForImport
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	protected function getKeys() {
		return array('id', 'fileId', 'hostgroupId', 'hostgroup_name', 'alias',
					 'members', 'hostgroup_members', 'name', 'register',
					 'conflictHostgroupId');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Hostgroup object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\HostgroupValidator
	 */
	public function getValidator() {
		return new ImportHostgroupValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ImportHostgroupMapper
	 */
	public static function getMapper() {
		return datamapper\ImportHostgroupMapper::getInstance();
	}

	/**
	 * Returns the File of this ImportHostgroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ImportFile
	 */
	public function getFile() {
		return datamapper\ImportFileMapper::getInstance()->getByID(
			$this->fileId
		)->allowRead();
	}

	/**
	 * Returns the HostGroup of this ImportHostgroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getHostGroup() {
		return datamapper\HostGroupMapper::getInstance()->getByID(
			$this->hostgroupId
		)->allowRead();
	}

	/**
	 * Returns the conflicting host group of this ImportHostgroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getConflictHostgroup() {
		return datamapper\HostGroupMapper::getInstance()->getByID(
			$this->conflictHostgroupId
		)->allowRead();
	}

	/**
	 * Returns all the host groups included by this one
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getIncludedHostgroups() {
		return datamapper\ImportHostgroupMapper::getInstance()->getAll(
			null,
			null,
			null,
			$this->id
		);
	}

	/**
	 * Returns the members as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getMembersAsArray() {
		if (!empty($this->members)) {
			return explode(',', $this->members);
		}

		return array();
	}
}
