<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ImportServiceValidator;
use ch\monitoring\util;

/**
 * Model for a service to be imported
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ImportService extends EntityForImport {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The file this service belongs to.
	 *
	 * @var           int $fileId
	 */
	public $fileId;

	/**
	 * An optionally associated service
	 *
	 * @var           int $serviceId
	 */
	public $serviceId;

	/**
	 * The name of the associated host
	 *
	 * @var           string $host_name
	 */
	public $host_name;

	/**
	 * The name of the associated hostgroup
	 *
	 * @var           string $hostgroup_name
	 */
	public $hostgroup_name;

	/**
	 * The service description
	 *
	 * @var           string $service_description
	 */
	public $service_description;

	/**
	 * The check command
	 *
	 * @var           string $check_command
	 */
	public $check_command;

	/**
	 * The max check attempts
	 *
	 * @var           int $max_check_attempts
	 */
	public $max_check_attempts;

	/**
	 * The check interval
	 *
	 * @var           int $check_interval
	 */
	public $check_interval;

	/**
	 * The retry interval
	 *
	 * @var           int $retry_interval
	 */
	public $retry_interval;

	/**
	 * Template name
	 *
	 * @var           bool $name
	 */
	public $name;

	/**
	 * Is this service to be registered?
	 *
	 * @var           bool $register
	 */
	public $register = false;

	/**
	 * The conflicting service id if this service generates a conflict
	 *
	 * @var           bool $conflictServiceId
	 */
	public $conflictServiceId;

	/**
	 * Cache for included services
	 *
	 * @var           array $includedServicesCache
	 */
	private static $includedServicesCache = array();

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                  = (int)$data['id'] ?: null;
		$this->fileId              = (int)$data['fileId'];
		$this->serviceId           = (int)$data['serviceId'] ?: null;
		$this->host_name           = $data['host_name'] ?: $this->host_name;
		$this->hostgroup_name      = $data['hostgroup_name'] ?: $this->hostgroup_name;
		$this->service_description = $data['service_description'] ?: $this->service_description;
		$this->check_command       = $data['check_command'] ?: $this->check_command;
		$this->max_check_attempts  = (int)$data['max_check_attempts'] ?: $this->max_check_attempts;
		$this->check_interval      = (int)$data['check_interval'] ?: $this->check_interval;
		$this->retry_interval      = (int)$data['retry_interval'] ?: $this->retry_interval;
		$this->name                = $data['name'] ?: null;
		$this->register            = (bool)$data['register'];
		$this->conflictServiceId   = (int)$data['conflictServiceId'] ?: null;
	}

	/**
	 * Implemented from EntityForImport
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	protected function getKeys() {
		return array('id', 'fileId', 'serviceId', 'host_name', 'hostgroup_name',
					 'service_description', 'check_command',
					 'max_check_attempts', 'check_interval', 'retry_interval',
					 'name', 'register', 'conflictServiceId');
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Service object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ServiceValidator
	 */
	public function getValidator() {
		return new ImportServiceValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ServiceMapper
	 */
	public static function getMapper() {
		return datamapper\ImportServiceMapper::getInstance();
	}

	/**
	 * Returns the File of this ImportService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ImportFile
	 */
	public function getFile() {
		return datamapper\ImportFileMapper::getInstance()->getByID(
			$this->fileId
		)->allowRead();
	}

	/**
	 * Returns the Service of this ImportService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return datamapper\ServiceMapper::getInstance()->getByID(
			$this->serviceId
		)->allowRead();
	}

	/**
	 * Returns the conflicting Service of this ImportService
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getConflictService() {
		return datamapper\ServiceMapper::getInstance()->getByID(
			$this->conflictServiceId
		)->allowRead();
	}

	/**
	 * Returns the command split by !
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getCommand() {
		return explode('!', $this->check_command);
	}

	/**
	 * Returns the arguments as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getArguments() {
		return array_slice($this->getCommand(), 1);
	}

	/**
	 * Returns the command name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getCommandName() {
		$command = $this->getCommand();
		return $command[0];
	}

	/**
	 * Return all assigned hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHosts() {
		return datamapper\ImportHostMapper::getInstance()->getAll(
			null,
			$this->service_description,
			null,
			true
		);
	}

	/**
	 * Return all assigned hosts (names)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getHostNames() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getHosts(),
			'id',
			'host_name'
		);
	}

	/**
	 * Delete all services with own name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function deleteByName($param_name) {
		$this->getMapper()->deleteByName($this->service_description);
	}

	/**
	 * Returns all the services included by this one
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getIncludedServices() {
		return datamapper\ImportServiceMapper::getInstance()->getAll(
			null,
			null,
			null,
			false,
			null,
			null,
			$this->id
		);
	}
}
