<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\InfofeedValidator;
use ch\monitoring\util;

/**
 * Model for an infofeed
 *
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class Infofeed extends Entity {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The user this infofeed belongs to.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * Notify on alert.
	 *
	 * @var           bool $onAlert
	 */
	public $onAlert = false;

	/**
	 * Notify on warning.
	 *
	 * @var           bool $onAlert
	 */
	public $onWarning = false;

	/**
	 * Notify on acknowledge.
	 *
	 * @var           bool $onAcknowledge
	 */
	public $onAcknowledge = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id            = (int)$data['id'];
		$this->userId        = (int)$data['userId'];
		$this->onAlert       = (bool)$data['onAlert'];
		$this->onWarning     = (bool)$data['onWarning'];
		$this->onAcknowledge = (bool)$data['onAcknowledge'];
	}

	/**
	 * Returns a validator for the current Infofeed object
	 *
	 * @returns       ch\monitoring\validate\InfofeedValidator
	 */
	public function getValidator() {
		return new InfofeedValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @returns       ch\monitoring\datamapper\InfofeedMapper
	 */
	public static function getMapper() {
		return datamapper\InfofeedMapper::getInstance();
	}

	/**
	 * Returns the User of this Infofeed
	 *
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Returns the object and his type of this infofeed
	 *
	 * @returns       array
	 */
	public function getObjectAndType() {
		try {
			$host = $this->getHost();
			return array(
				'object' => $host,
				'type'   => 'Host'
			);
		}
		catch (\OutOfBoundsException $e) {
			try {
				$hostgroup = $this->getHostGroup();
				return array(
					'object' => $hostgroup,
					'type'   => 'Host Group'
				);
			}
			catch (\OutOfBoundsException $e) {
				try {
					$customer = $this->getCustomer();
					return array(
						'object' => $customer,
						'type'   => 'Customer'
					);
				}
				catch (\OutOfBoundsException $e) {
					return array(
						'object' => null,
						'type'   => null
					);
				}
			}
		}
	}

	/**
	 * Returns the object name of this infofeed
	 *
	 * @returns       string
	 */
	public function getObjectName() {
		$keys = array(
			'Host'       => 'hostname',
			'Host Group' => 'name',
			'Customer'   => 'name'
		);

		$objectAndType = $this->getObjectAndType();

		return $objectAndType['object']->$keys[$objectAndType['type']];
	}

	/**
	 * Returns the object type of this infofeed
	 *
	 * @returns       string
	 */
	public function getObjectType() {
		$objectAndType = $this->getObjectAndType();

		return $objectAndType['type'];
	}

	/**
	 * Returns the excluded hosts
	 *
	 * @returns       array
	 */
	public function getExcludedHosts() {
		if($this->getObjectType() == 'Customer') {
			return $this->getMapper()->excludedHostForCustomer($this->id);
		}
		
		if($this->getObjectType() == 'Host Group') {
			return $this->getMapper()->excludedHostForHostGroup($this->id);
		}
		
		return array();
	}

	/**
	 * Returns the Host of this Infofeed
	 *
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByInfofeedID(
			$this->id
		)->allowRead();
	}

	/**
	 * Returns the HostGroup of this Infofeed
	 *
	 * @returns       ch\monitoring\model\HostGroup
	 */
	public function getHostGroup() {
		return datamapper\HostGroupMapper::getInstance()->getByInfofeedID(
			$this->id
		)->allowRead();
	}

	/**
	 * Returns the Customer of this Infofeed
	 *
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByInfofeedID(
			$this->id
		)->allowRead();
	}

	/**
	 * Returns all assigned SLA actions
	 *
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getNotificationTypes() {
		return datamapper\SLAActionMapper::getInstance()->getByInfofeedId(
			$this->id
		);
	}

	/**
	 * Assign a host to this Infofeed
	 *
	 * @param         int $hostId
	 * @returns       void
	 */
	public function assignHost($hostId) {
		$this->getMapper()->assignHost($this->id, $hostId);
	}

	/**
	 * Assign a hostgroup to this Infofeed
	 *
	 * @param         int $hostGroupId
	 * @returns       void
	 */
	public function assignHostGroup($hostGroupId) {
		$this->getMapper()->assignHostGroup($this->id, $hostGroupId);
	}

	/**
	 * Assign a customer to this Infofeed
	 *
	 * @param         int $customerId
	 * @returns       void
	 */
	public function assignCustomer($customerId) {
		$this->getMapper()->assignCustomer($this->id, $customerId);
	}

	/**
	 * Assign a SLA action to this Infofeed
	 *
	 * @param         int $slaActionId
	 * @returns       void
	 */
	public function assignNotificationType($slaActionId) {
		$this->getMapper()->assignNotificationType($this->id, $slaActionId);
	}

	/**
	 * Unassign a SLA action from this Infofeed
	 *
	 * @param         int $slaActionId
	 * @returns       void
	 */
	public function unassignNotificationType($slaActionId) {
		$this->getMapper()->unassignNotificationType($this->id, $slaActionId);
	}
}
