<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceChecklistValidator;

/**
 * Model for a maintenance checklist
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklist extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The checklist name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The checklist comment.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Whether this list is a template
	 *
	 * @var           bool $isTemplate
	 */
	public $isTemplate = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id         = (int)$data['id'];
		$this->name       = $data['name'];
		$this->comment    = $data['comment'];
		$this->isTemplate = (bool)$data['isTemplate'];
	}

	/**
	 * Returns a validator for the current MaintenanceChecklist object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceChecklistValidator
	 */
	public function getValidator() {
		return new MaintenanceChecklistValidator($this);
	}

	/**
	 * Returns a policy for the current MaintenanceChecklist object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}


	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceChecklistMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceChecklistMapper::getInstance();
	}

	/**
	 * Returns the hosts which are assigned to this checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHosts() {
		return datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
			false,
			false,
			false,
			null,
			null,
			model\Permission::MAINTENANCE_READ,
			null,
			null,
			$this->id
		);
	}

	/**
	 * Returns the MaintenanceGroups which are assigned to this checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getGroups() {
		return datamapper\MaintenanceGroupMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the questions which are assigned to this checklist
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $exclude Optional. Whether to return include or exclude questions.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getQuestions($exclude = null) {
		return datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getAll(
			$this->id,
			$exclude
		);
	}

	/**
	 * Recursively returns an iteration of questions
	 *
	 * Iterations are limited to 5
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $checklists
	 * @param         int $iteration Current iteration
	 * @returns       array
	 */
	public function getQuestionsIteration($checklists, $iteration, $exclude) {
		if ($iteration > 5) {
			return array();
		}

		$questions = array();
		$nextChecklists = array();
		foreach ($checklists as $checklist) {
			$questions[]      = $checklist->getQuestions($exclude)->toArray();
			$nextChecklists[] = $checklist->getParentChecklists();
		}

		return array_merge($questions, $this->getQuestionsIteration($checklists, $iteration + 1, $exclude));
	}

	/**
	 * Returns the questions which are assigned to this checklist (recursively)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         bool $exclude Optional. Whether to return include or exclude questions.
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getQuestionsRecursive($exclude = null) {
		return $this->getQuestionsIteration(array($this), 0, $exclude);
	}

	/**
	 * Returns the questions which are assigned to this checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getParentChecklists() {
		return datamapper\MaintenanceChecklistMapper::getInstance()->getParentsByID(
			$this->id
		);
	}	
}
