<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceChecklistQuestionValidator;

/**
 * Model for a MaintenanceChecklistQuestion
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceChecklistQuestion extends Entity {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The question name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short comment of the question.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id      = (int)$data['id'];
		$this->name    = $data['name'];
		$this->comment = $data['comment'];
	}

	/**
	 * Returns a validator for the current MaintenanceChecklistQuestion object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceChecklistQuestionValidator
	 */
	public function getValidator() {
		return new MaintenanceChecklistQuestionValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceChecklistQuestionMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceChecklistQuestionMapper::getInstance();
	}

	/**
	 * Indicate if a question is available or not inside the given checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $checklistId
	 * @param         int $hostId
	 * @param         int $groupId
	 * @returns       bool
	 */
	public function isExcluded($checklistId, $hostId = null, $groupId = null) {
		if(is_null($hostId) && is_null($groupId)) {
			//this exception will be thrown for the devs not user!
			throw new \Exception('One of the parameters hostId or groupId must not be null!');
		}
		return datamapper\MaintenanceChecklistQuestionMapper::getInstance()->isExcluded($checklistId, $this->id, $hostId, $groupId);
	}

	/**
	 * Indicates if a question is used by a checklist
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function isInUse() {
		return datamapper\MaintenanceChecklistQuestionMapper::getInstance()->isInUse($this->id);
	}

	public function __toString() {
		return $this->name;
	}
}
