<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\util;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceCommentValidator;

/**
 * Model for a maintenance ticket comment
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceComment extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The maintenance ticket this comment is related to.
	 *
	 * @var           int $ticketHostId
	 */
	public $ticketHostId;

	/**
	 * The user who created this comment.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The old ticket state.
	 *
	 * @var           int $oldStateId
	 */
	public $oldStateId;

	/**
	 * The mew ticket state.
	 *
	 * @var           int $newStateId
	 */
	public $newStateId;

	/**
	 * The old OS version.
	 *
	 * @var           string $oldOsVersionId
	 */
	public $oldOsVersionId;

	/**
	 * The new OS version.
	 *
	 * @var           string $newOsVersionId
	 */
	public $newOsVersionId;

	/**
	 * Creation date and time.
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * The comment.
	 *
	 * @var           string $comment
	 */
	public $comment;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id             = (int)$data['id'];
		$this->ticketHostId   = (int)$data['ticketHostId'];
		$this->userId         = (int)$data['userId'];
		$this->oldStateId     = (int)$data['oldStateId'] ?: null;
		$this->oldOsVersionId = (int)$data['oldOsVersionId'] ?: null;
		$this->newOsVersionId = (int)$data['newOsVersionId'] ?: null;
		$this->newStateId     = (int)$data['newStateId'] ?: null;
		$this->timestamp      = new \Zend_Date($data['timestamp']);
		$this->comment        = $data['comment'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\MaintenanceCommentPolicy
	 */
	public function getPolicy() {
		return new policy\MaintenanceCommentPolicy($this);
	}

	/**
	 * Returns a validator for the current MaintenanceComment object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceCommentValidator
	 */
	public function getValidator() {
		return new MaintenanceCommentValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceCommentMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceCommentMapper::getInstance();
	}

	/**
	 * Returns the Maintenance ticket this Comment belongs to.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceTicketHost
	 */
	public function getTicketHost() {
		return datamapper\MaintenanceTicketHostMapper::getInstance()->getByID(
			$this->ticketHostId
		)->allowRead();
	}

	/**
	 * Returns the old MaintenanceState of the Maintenanceticket.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceState
	 */
	public function getOldState() {
		return datamapper\MaintenanceStateMapper::getInstance()->getByID(
			$this->oldStateId
		);
	}

	/**
	 * Returns the new MaintenanceState of the Maintenance ticket.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceState
	 */
	public function getNewState() {
		return datamapper\MaintenanceStateMapper::getInstance()->getByID(
			$this->newStateId
		);
	}

	/**
	 * Returns the old os version of the Maintenanceticket.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	public function getOldOsVersion() {
		return datamapper\HostOSVersionMapper::getInstance()->getByID(
			$this->oldOsVersionId
		)->allowRead();
	}

	/**
	 * Returns the new os version of the Maintenanceticket.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\HostOSVersion
	 */
	public function getNewOsVersion() {
		return datamapper\HostOSVersionMapper::getInstance()->getByID(
			$this->newOsVersionId
		)->allowRead();
	}

	/**
	 * Returns the User who generated this comment. Null if it is from system
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		if (!$this->userId) {
			return null;
		}

		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Returns the assigned MaintenanceTags
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getTags() {
		return datamapper\MaintenanceTagMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the assigned MaintenanceTags as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getTagsAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getTags()
		);
	}

	/**
	 * Assigns a tag to this comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $tagId
	 * @returns       void
	 */
	public function addMaintenanceTag($tagId) {
		datamapper\MaintenanceTagMapper::getInstance()->createAssociation(
			$tagId,
			$this->id
		);
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }
}
