<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceGroupValidator;
use org\adfinis\framework\exceptions\UserException;

/**
 * Model for a service group
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceGroup extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer owning this group.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The maintenance cycle of this group.
	 *
	 * @var           int $maintenanceCycleId
	 */
	public $maintenanceCycleId;

	/**
	 * The service group name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the service group.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Maintenance start date
	 *
	 * @var           string $dateStart
	 */
	public $dateStart;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                 = (int)$data['id'];
		$this->customerId         = (int)$data['customerId'];
		$this->maintenanceCycleId = (int)$data['maintenanceCycleId'];
		$this->name               = $data['name'];
		$this->description        = $data['description'];
		$this->dateStart          = $data['dateStart'] ? new \Zend_Date($data['dateStart'], \Zend_Date::ISO_8601) : null;
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\MaintenanceGroupPolicy
	 */
	public function getPolicy() {
		return new policy\MaintenanceGroupPolicy($this);
	}

	/**
	 * Returns a validator for the current MaintenanceGroup object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceGroupValidator
	 */
	public function getValidator() {
		return new MaintenanceGroupValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceGroupMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceGroupMapper::getInstance();
	}

	/**
	 * Returns the Customer of this group.
	 *
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the assigned members
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getMembers() {
		return datamapper\MaintenanceGroupMemberMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the maintenance checklists which are assigned to this MaintenanceGroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getTemplateChecklists() {
		return datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			null,
			$this->id,
			true
		);
	}

	/**
	 * Returns the next maintenance date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @throws        UserException
	 * @returns       Zend_Date Null when no start date or maintenance cycle was set
	 */
	public function getNextMaintenanceDate() {
		if ($this->maintenanceCycleId === null) {
			throw new UserException(sprintf(
				tr("No maintenance interval defined")
			));
		}

		$interval = $this->getMaintenanceCycle()->interval;

		$lastTicket = datamapper\MaintenanceTicketMapper::getInstance()->getLatestGroupMaintenanceTicket($this->id);
		if ($lastTicket) {
			$lastTicket->allowRead();
			if ($lastTicket->dateArchived === null) {
				throw new UserException(sprintf(
					tr("No dateArchived given for ticketId %s"),
					$lastTicket->id
				));
			}

			$nextMaintenanceDate = clone $lastTicket->dateArchived;
			$nextMaintenanceDate->add(
				$interval,
				\Zend_Date::MONTH
			);

			return $nextMaintenanceDate;
		}

		if ($this->dateStart === null) {
			throw new UserException(sprintf(
				tr("No date start defined and no last ticket found")
			));
		}

		$nextDate = clone $this->dateStart;
		$nextDate = $nextDate->add(
			$interval,
			\Zend_Date::MONTH
		);

		return $nextDate;
	}

	/**
	 * Returns the MaintenanceCycle of this Group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceCycle
	 */
	public function getMaintenanceCycle() {
		return datamapper\MaintenanceCycleMapper::getInstance()->getByID(
			$this->maintenanceCycleId
		);
	}

	/**
	 * Returns all questions which are assigned to this group
	 *
	 * @param         bool $recursive Recursively follow parents
	 * @param         bool $primaryOnly Only return questions assigned to the primary checklist
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by denis:   With recursion, $question->isExcluded() does not work the way it is done now
	 * @returns       array
	 */
	public function getQuestions($recursive = false, $primaryOnly = false) {
		if ($recursive) {
			throw new \RuntimeException(
				tr("Recursion does not yet work")
			);
		}

		if ($primaryOnly) {
			$checklists = array($this->getPrimaryChecklist());
		}
		else {
			$checklists = $this->getChecklists();
		}

		$includeArray = array();
		$excludeArray = array();
		foreach ($checklists as $checklist) {
			if ($recursive) {
				$questions = $checklist->getQuestionsRecursive();
			}
			else {
				$questions = $checklist->getQuestions();
			}

			foreach ($questions as $question) {
				if ($question->isExcluded($checklist->id, null, $this->id)) {
					$excludeArray[$question->id] = $question;
				}
				else {
					$includeArray[$question->id] = $question;
				}
			}
		}

		foreach ($includeArray as $id => $question) {
			if (array_key_exists($id, $excludeArray) && array_key_exists($id, $includeArray)) {
				unset($includeArray[$id]);
			}
		}

		return $includeArray;
	}

	/**
	 * Returns all checklists which are assigned to this Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       db\adfinis\framework\db\DbIterator
	 */
	public function getChecklists() {
		return datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			null,
			$this->id
		);
	}

	/**
	 * Returns this group's primary checklist
	 *
	 * If it doesn't exist yet, it is created
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       model\MaintenanceChecklist
	 */
	public function getPrimaryChecklist() {
		$checklists = datamapper\MaintenanceChecklistMapper::getInstance()->getAll(
			null,
			$this->id,
			false,
			false
		)->toArray();

		if (count($checklists) == 0) {
			$checklist = new MaintenanceChecklist();
			$checklist->allowRead();

			$checklist->name       = sprintf('Primary checklist %s', microtime());
			$checklist->comment    = sprintf("Primary checklist for group '%s'", $this->name);
			$checklist->isTemplate = false;
			$checklist->getMapper()->save($checklist);

			datamapper\MaintenanceChecklistMapper::getInstance()->assignGroup($this->id, $checklist->id);

			return $checklist;
		}

		return $checklists[0];
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'maintenanceCycleId' => array(
				'type' => 'relation',
				'mapper' => 'MaintenanceCycleMapper'
			)
		);
	}
}
