<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceGroupMemberValidator;

/**
 * Model for a service group member
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceGroupMember extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The host relation
	 *
	 * @var           int $hostId
	 */
	public $hostId;

	/**
	 * The service group this member belongs to.
	 *
	 * @var           int $groupId
	 */
	public $groupId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array(
			'id' => $this->id
		);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id      = (int)$data['id'];
		$this->hostId  = (int)$data['hostId'];
		$this->groupId = (int)$data['groupId'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\MaintenanceGroupMemberPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current MaintenanceGroupMember object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceGroupMemberValidator
	 */
	public function getValidator() {
		return new MaintenanceGroupMemberValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceGroupMemberMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceGroupMemberMapper::getInstance();
	}

	/**
	 * Returns the MaintenanceGroup of this Member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ServiceGroup
	 */
	public function getMaintenanceGroup() {
		return datamapper\MaintenanceGroupMapper::getInstance()->getByID(
			$this->groupId
		)->allowRead();
	}

	/**
	 * Returns the Host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}
}
