<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceTicketValidator;

/**
 * Model for a maintenance ticket
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceTicket extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The maintenance group this maintenance ticket belongs to.
	 *
	 * NULL means no maintenance group (manual creation or single host)
	 *
	 * @var           int $groupId
	 */
	public $groupId;

	/**
	 * The customer this maintenance ticket belongs to.
	 *
	 * @var           int $customerId
	 */
	public $customerId = 0;

	/**
	 * The assigned user.
	 *
	 * @var           int $userId
	 */
	public $userId = 0;

	/**
	 * Due date.
	 *
	 * @var           Zend_Date $dateCreated
	 */
	public $dateCreated;

	/**
	 * Due date.
	 *
	 * May be null
	 *
	 * @var           Zend_Date $dateDue
	 */
	public $dateDue;

	/**
	 * Whether this ticket is archived
	 *
	 * @var           Zend_Date $dateArchived
	 */
	public $dateArchived;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		$this->groupId      = (int)$data['groupId'] ?: null;
		$this->customerId   = (int)$data['customerId'] ?: null;
		$this->userId       = (int)$data['userId'] ?: null;
		$this->dateCreated  = new \Zend_Date($data['dateCreated'], \Zend_Date::ISO_8601);
		$this->dateDue      = $data['dateDue'] ? new \Zend_Date($data['dateDue'], \Zend_Date::ISO_8601) : null;
		$this->dateArchived = $data['dateArchived'] ? new \Zend_Date($data['dateArchived'], \Zend_Date::ISO_8601) : null;
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\MaintenanceTicketPolicy
	 * @todo by denis:   Find a way to verify ownership
	 */
	public function getPolicy() {
		return new policy\MaintenanceTicketPolicy($this);
	}

	/**
	 * Returns a validator for the current Maintenance object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceTicketValidator
	 */
	public function getValidator() {
		return new MaintenanceTicketValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceTicketMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceTicketMapper::getInstance();
	}

	/**
	 * Returns the MaintenanceGroup of this Maintenance ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceGroup
	 */
	public function getGroup() {
		return datamapper\MaintenanceGroupMapper::getInstance()->getByID(
			$this->groupId
		)->allowRead();
	}

	/**
	 * Returns the Customer of this MaintenanceTicket
	 *
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the assigned User of this ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Returns the associated host tickets
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getTicketHosts() {
		return datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the associated host tickets
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAllRelatedTicketHosts() {
		return datamapper\MaintenanceTicketHostMapper::getInstance()->getAll(
			$this->id,
			null,
			array(),
			null,
			null,
			null
		);
	}
}
