<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\MaintenanceTicketHostValidator;
use ch\monitoring\util;

/**
 * Model for a maintenance ticket
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class MaintenanceTicketHost extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The the ticket this entity belongs to.
	 *
	 * @var           int $customerId
	 */
	public $ticketId;

	/**
	 * The host this maintenance ticket belongs to.
	 *
	 * @var           int $customerId
	 */
	public $hostId;

	/**
	 * The ticket state.
	 *
	 * @var           int $stateId
	 */
	public $stateId;

	/**
	 * Last change on ticket.
	 *
	 * @var           Zend_Date $datetimeChanged
	 */
	public $datetimeChanged;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id              = (int)$data['id'];
		$this->ticketId        = (int)$data['ticketId'];
		$this->hostId          = (int)$data['hostId'];
		$this->stateId         = (int)$data['stateId'];
		$this->datetimeChanged = new \Zend_Date($data['datetimeChanged'], \Zend_Date::ISO_8601);
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\MaintenanceTicketHostPolicy
	 */
	public function getPolicy() {
		return new policy\MaintenanceTicketHostPolicy($this);
	}

	/**
	 * Returns a validator for the current MaintenanceTicketHost object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\MaintenanceTicketHostValidator
	 */
	public function getValidator() {
		return new MaintenanceTicketHostValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\MaintenanceTicketHostMapper
	 */
	public static function getMapper() {
		return datamapper\MaintenanceTicketHostMapper::getInstance();
	}

	/**
	 * Returns the Ticket of this entity
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getTicket() {
		return datamapper\MaintenanceTicketMapper::getInstance()->getByID(
			$this->ticketId
		)->allowRead();
	}

	/**
	 * Returns the Host of this entity
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Host
	 */
	public function getHost() {
		return datamapper\HostMapper::getInstance()->getByID(
			$this->hostId
		)->allowRead();
	}

	/**
	 * Returns the State of this ticket
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\MaintenanceState
	 */
	public function getMaintenanceState() {
		return datamapper\MaintenanceStateMapper::getInstance()->getByID(
			$this->stateId
		);
	}

	/**
	 * Returns the associated comments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getComments() {
		return datamapper\MaintenanceCommentMapper::getInstance()->getAll(
			$this->id
		);
	}

	/**
	 * Returns the associated answered questions
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getQuestions() {
		return datamapper\MaintenanceChecklistQuestionMapper::getInstance()->getAll(
			null,
			$this->id,
			$this->hostId
		);
	}

	/**
	 * Returns the associated answered questions
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAnswers() {
		return datamapper\MaintenanceChecklistAnswerMapper::getInstance()->getAll(
			$this->hostId,
			$this->ticketId
		);
	}

	/**
	 * Returns the associated answered questions as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getAnswersAsArray() {
		$answers = $this->getAnswers();

		$answersArray = array();
		foreach ($answers as $answer) {
			$answersArray[$answer->id] = $answer;
		}

		return $answersArray;
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->datetimeChanged = new \Zend_Date();
   }
}
