<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\validate\NagiosLogValidator;

/**
 * Model for a nagios log entry
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class NagiosLog extends Entity {

	/**
	 * Constants for processed state
	 */
	const STATE_OPEN             = '0';
	const STATE_ALERTED          = '1';
	const STATE_NO_SLA           = '2';
	const STATE_MALFORMED_ROW    = '3';
	const STATE_HOST_DISABLED    = '4';
	const STATE_ALERT_CLOSED     = '5';
	const STATE_ALERT_REPETITION = '6';
	const STATE_NO_TEAM          = '7';
	const STATE_IN_MAINTENANCE   = '8';
	const STATE_SOFT             = '9';
	const HOST_DOWN              = '10';
	const NO_PREVIOUS_DOWN       = '11';
	const DEPENDENCY_DOWN        = '12';
	const DOES_NOT_EXIST         = '13';

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * When did the notification arrive?
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * Nagios object type
	 *
	 * Host or Service
	 *
	 * @var           string $objectType
	 */
	public $objectType;

	/**
	 * Object state type
	 *
	 * SOFT or HARD
	 *
	 * @var           string $stateType
	 */
	public $stateType;

	/**
	 * Object name
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Object hostname
	 *
	 * @var           string $hostname
	 */
	public $hostname;

	/**
	 * The command which reported the problem
	 *
	 * @var           string $checkCommand
	 */
	public $checkCommand;

	/**
	 * The IP address of the host
	 *
	 * @var           string $address
	 */
	public $address;

	/**
	 * Raw output of the nagios plugin
	 *
	 * @var           string $output
	 */
	public $output;

	/**
	 * Additional performance data
	 *
	 * @var           string $perfdata
	 */
	public $perfdata;

	/**
	 * Duration of the reported state
	 *
	 * @var           string $duration
	 */
	public $duration;

	/**
	 * Problem detection date and time
	 *
	 * @var           Zend_Date $datetime
	 */
	public $datetime;

	/**
	 * Current check attempt.
	 *
	 * @var           string $attempt
	 */
	public $attempt;

	/**
	 * Whether this notification has been processed yet
	 *
	 * @var           int $processed
	 */
	public $processed;

	/**
	 * The alert which was created for this notification
	 *
	 * @var           int $alertId
	 */
	public $alertId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		$this->timestamp    = new \Zend_Date($data['timestamp']);
		$this->objectType   = $data['objectType'];
		$this->stateType    = $data['stateType'];
		$this->state        = $data['state'];
		$this->name         = $data['name'];
		$this->hostname     = $data['hostname'];
		$this->checkCommand = $data['checkCommand'];
		$this->address      = $data['address'];
		$this->output       = $data['output'];
		$this->perfdata     = $data['perfdata'];
		$this->duration     = $data['duration'];
		$this->datetime     = new \Zend_Date($data['datetime']);
		$this->attempt      = (int)$data['attempt'];
		$this->processed    = (int)$data['processed'];
		$this->alertId      = (int)$data['alertId'];
	}

	/**
	 * Returns a validator for the current NagiosLog object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\NagiosLogValidator
	 */
	public function getValidator() {
		return new NagiosLogValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\NagiosLogMapper
	 */
	public static function getMapper() {
		return datamapper\NagiosLogMapper::getInstance();
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }
}
