<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\validate\PermissionValidator;

/**
 * Model for a permission
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Permission extends Entity {

	/**
	 * All available Permissions (references the Permission table rows)
	 */
	const ALERTS_READ             = 'Alerts';
	const ALERTS_WRITE            = 'Alerts write';
	const AUDIT_LOG_READ          = 'Audit Log';
	const BREAPPLICATION_READ     = 'BRE Application';
	const BREAPPLICATION_WRITE    = 'BRE Application write';
	const CACTI_WEB               = 'Cacti web';
	const CACTI_MASTER            = 'Cacti master';
	const CACTI_ADMIN             = 'Cacti admin';
	const COMMANDS_READ           = 'Commands';
	const COMMANDS_WRITE          = 'Commands write';
	const CUSTOMERS_READ          = 'Customers';
	const CUSTOMERS_WRITE         = 'Customers write';
	const DOWNTIMES_READ          = 'Downtimes';
	const DOWNTIMES_WRITE         = 'Downtimes write';
	const EMERGENCY_READ          = 'Emergency';
	const EMERGENCY_WRITE         = 'Emergency write';
	const EMERGENCY_ALERT         = 'Emergency alert';
	const GROUPS_READ             = 'Groups read';
	const GROUPS_WRITE            = 'Groups write';
	const HOSTS_READ              = 'Hosts';
	const HOSTS_WRITE             = 'Hosts write';
	const HOSTS_MASTER            = 'Hosts master';
	const HOSTGROUPS_READ         = 'Host groups';
	const HOSTGROUPS_WRITE        = 'Host groups write';
	const MAINTENANCE_READ        = 'Maintenance';
	const MAINTENANCE_WRITE       = 'Maintenance write';
	const MAINTENANCE_ADMIN       = 'Maintenance admin';
	const MAINTENANCE_MASTER      = 'Maintenance master';
	const MAINTENANCEGROUPS_READ  = 'Maintenance groups';
	const MAINTENANCEGROUPS_WRITE = 'Maintenance groups write';
	const MONITORING_WEB          = 'Monitoring web';
	const MONITORING_STATUS       = 'Monitoring status';
	const MONITORING_MASTER       = 'Monitoring master';
	const NOTIFICATIONS_READ      = 'Notifications';
	const ROLES_READ              = 'Roles';
	const ROLES_WRITE             = 'Roles write';
	const REPORTS                 = 'Reports';
	const REPORTS_FULL            = 'Reports full';
	const SLAS_READ               = 'Slas';
	const SLAS_WRITE              = 'Slas write';
	const SERVICES_READ           = 'Services';
	const SERVICES_WRITE          = 'Services write';
	const USERS_READ              = 'Users';
	const USERS_WRITE             = 'Users write';
	const INFOFEEDS               = 'Infofeeds';
	const USER_SETTINGS           = 'User settings';
	const GLOBAL_SETTINGS         = 'Global settings';
	const IMPORT                  = 'Import';

	//permission to see the external links
	const NFSEN_WEB               = 'Nfsen web';
	const SMOKEPING_WEB           = 'Smokeping web';
	const WEATHERMAP_WEB          = 'Weathermap web';


	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The permission name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the permission.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Indicates if that permission will have a systemwide effect
	 *
	 * access it over the function isGlobal()
	 *
	 * @var           bool $isGlobal
	 */
	private $isGlobal = true;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->name        = $data['name'];
		$this->description = $data['description'];
		$this->isGlobal    = (bool)$data['globalOnly'];
	}

	/**
	 * Returns a validator for the current Permission object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\PermissionValidator
	 */
	public function getValidator() {
		return new PermissionValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\PermissionMapper
	 */
	public static function getMapper() {
		return datamapper\PermissionMapper::getInstance();
	}

	/**
	 * Indicates if that permission will have a systemwide effect
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function isGlobal() {
		return (bool)$this->isGlobal;
	}
}
