<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ReportValidator;
use ch\monitoring\Application;
use ch\monitoring\util;

/**
 * Model for a report
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Report extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * When was this report created?
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * The user who created this report
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The customer
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The start date
	 *
	 * @var           Zend_Date $dateFrom
	 */
	public $dateFrom;

	/**
	 * The end date
	 *
	 * @var           Zend_Date $dateTo
	 */
	public $dateTo;

	/**
	 * Is this a template?
	 *
	 * @var           bool $isTemplate
	 */
	public $isTemplate = false;

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id         = (int)$data['id'];
		$this->timestamp  = new \Zend_Date($data['timestamp']);
		$this->userId     = (int)$data['userId'];
		$this->customerId = (int)$data['customerId'];
		$this->dateFrom   = new \Zend_Date($data['dateFrom']);
		$this->dateTo     = new \Zend_Date($data['dateTo']);
		$this->isTemplate = (bool)$data['isTemplate'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\ReportPolicy
	 */
	public function getPolicy() {
		return new policy\ReportPolicy($this);
	}

	/**
	 * Returns a validator for the current Report object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ReportValidator
	 */
	public function getValidator() {
		return new ReportValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ReportMapper
	 */
	public static function getMapper() {
		return datamapper\ReportMapper::getInstance();
	}

	/**
	 * Returns the User of this Report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Returns the Customer of this Report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns all assigned hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHosts() {
		return datamapper\HostMapper::getInstance()->getByReportId($this->id);
	}

	/**
	 * Returns all assigned hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHostsAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			datamapper\HostMapper::getInstance()->getByReportId($this->id),
			'id',
			'hostname'
		);
	}

	/**
	 * Delete the association of a host to this report
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $commentID The ID of the comment
	 */
	public function removeHost($hostId) {
		Application::getDBInstance()->delete(
			'Report_Host',
			sprintf('`hostId` = %d', $hostId)
		);
	}

	/**
	 * Create an association to a host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $hostId
	 */
	public function addHost($hostId) {
		Application::getDBInstance()->insert(
			'Report_Host',
			array(
				'reportId' => $this->id,
				'hostId'   => $hostId
			)
		);
	}
}
