<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\util;
use ch\monitoring\validate;
use ch\monitoring\Application;

/**
 * Model for a role
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Right extends EntityWithPolicy {
	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * Foreignkey to the user
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * Foreignkey to the customer
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * Foreignkey to the role
	 *
	 * @var           int $roleId
	 */
	public $roleId;

	/**
	 * The name of the assigned user
	 *
	 * @var           string $userName
	 */
	public $userName;
	
	/**
	 * The name of the assigned customer
	 *
	 * @var           string $customerName
	 */
	public $customerName;

	/**
	 * The name of the assigned role
	 *
	 * @var           string $roleName
	 */
	public $roleName;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		$this->userId       = (int)$data['userId'];
		$this->roleId       = (int)$data['roleId'];
		$this->customerId   = is_null($data['customerId']) ? NULL : (int)$data['customerId'];

		$this->userName     = $data['userName'];
		$this->roleName     = $data['roleName'];
		$this->customerName = $data['customerName'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\RightPolicy
	 */
	public function getPolicy() {
		return new policy\RightPolicy($this);
	}

	/**
	 * Returns a validator for the current Role object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\RightValidator
	 */
	public function getValidator() {
		return new validate\RightValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\RightMapper
	 */
	public static function getMapper() {
		return datamapper\RightMapper::getInstance();
	}

	/**
	 * Checks if the role assignement to a customer can be removed( or changed)
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @returns       bool
	 */
	public function isEditable() {
		/*
		a right is editable if
			the currentUser has the right on the selected customer (or above) to edit users
			AND
			if the role that needs to be edited is inside the list of assigned Roles
			AND
			if the user is mutable
		*/
		$user = Application::getUser();
		if($user->hasPermissionForCustomer(Permission::USERS_WRITE, $this->customerId)) {
			$roles = Role::getMapper()->getAllByCustomer($this->customerId, false)->toArray();
			$subroleIds = array_map(function($role) {
				return $role->id;
			}, $roles);
			

			if(in_array($this->roleId, $subroleIds)) {
				$user = User::getMapper()->getById($this->userId);
				$user->allowRead();
				return $user->isMutable();
			}
		}

		return false;
	}
}
