<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\util;
use ch\monitoring\validate\RoleValidator;

/**
 * Model for a role
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Role extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The role name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the role.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @return        bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @return        array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @return        void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->name        = $data['name'];
		$this->description = $data['description'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a validator for the current Role object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        ch\monitoring\validate\RoleValidator
	 */
	public function getValidator() {
		return new RoleValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        ch\monitoring\datamapper\RoleMapper
	 */
	public static function getMapper() {
		return datamapper\RoleMapper::getInstance();
	}

	/**
	 * Returns the permissions associated to a Role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        org\adfinis\framework\db\DbIterator
	 */
	public function getPermissions() {
		return datamapper\PermissionMapper::getInstance()->getAll($this->id);
	}

	/**
	 * Returns the permissions associated to a Role as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        array
	 */
	public function getPermissionsAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getPermissions()
		);
	}

	/**
	 * Returns the permission ids associated to a Role as an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return        array
	 */
	public function getPermissionIdsAsArray() {
		return util\IteratorUtil::getKeyValueArray(
			$this->getPermissions(),
			'id',
			'id'
		);
	}

	/**
	 * Add a permission to this role
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $permissionId
	 * @return        void
	 */
	public function addPermission($permissionId) {
		datamapper\PermissionMapper::getInstance()->createAssociation($permissionId, $this->id);
	}

	/**
	 * Indicates if that role will have a systemwide effect
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @return        bool
	 */
	public function isGlobal() {
		foreach($this->getPermissions() as $permission) {
			if($permission->isGlobal()) {
				return true;
			}
		}

		return false;
	}
}
