<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\SLAValidator;

/**
 * Model for a SLA
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SLA extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer id for this SLA
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The SLA name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the SLA.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * The delay for cycling through emergency persons
	 *
	 * @var           int $delayCycle
	 */
	public $delayCycle;

	/**
	 * The delay for restarting a cycle.
	 *
	 * @var           int $delayRestart
	 */
	public $delayRestart;

	/**
	 * The cycle timeout.
	 *
	 * @var           int $timeout
	 */
	public $timeout;

	/**
	 * The minimum availability in percent
	 *
	 * @var           int $minAvailability
	 */
	public $minAvailability;

	/**
	 * Is this SLA deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;

	/**
	 * Cache for rules
	 *
	 * @var           array $rules
	 */
	private static $rules = array();

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id              = (int)$data['id'];
		$this->customerId      = (int)$data['customerId'] ?: null;
		$this->name            = $data['name'];
		$this->description     = $data['description'];
		$this->delayCycle      = (int)$data['delayCycle'];
		$this->delayRestart    = (int)$data['delayRestart'];
		$this->timeout         = (int)$data['timeout'];
		$this->minAvailability = (double)$data['minAvailability'];
		$this->isDeleted       = (bool)$data['isDeleted'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\SLAPolicy($this);
	}

	/**
	 * Returns a validator for the current Command object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\SLAValidator
	 */
	public function getValidator() {
		return new SLAValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\SLAMapper
	 */
	public static function getMapper() {
		return datamapper\SLAMapper::getInstance();
	}

	/**
	 * Returns the Customer of this Service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the assigned Timeframes
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getTimeframes() {
		$timeframes = datamapper\TimeframeMapper::getInstance()->getAll($this->id);
		$timeframes->setOrderBy('timeFrom.ASC');

		return $timeframes;
	}

	/**
	 * Returns the Customers with an assignment to this SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getCustomers() {
		return datamapper\CustomerMapper::getInstance()->getAll(
			Permission::SLAS_READ,
			$this->id
		);
	}
	/**
	 * Returns the Hosts with an assignment to this SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHosts() {
		return datamapper\HostMapper::getInstance()->getAll(
			false,
			false,
			false,
			null,
			null,
			Permission::SLAS_READ,
			null,
			$this->id
		);
	}

	/**
	 * Returns the HostServices with an assignment to this SLA
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getHostServiceInstances() {
		return datamapper\HostServiceInstanceMapper::getInstance()->getAll(
			null,
			null,
			null,
			null,
			null,
			null,
			Permission::SLAS_READ,
			$this->id
		);
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			)
		);
	}
}
