<?php

namespace ch\monitoring\model;

use ch\monitoring\datamapper;
use ch\monitoring\validate\SLAActionValidator;

/**
 * Model for an SLA action
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class SLAAction extends Entity {

	/**
	 * The ID of the sms type
	 *
	 * @const         int SMS
	 */
	const SMS = 1;

	/**
	 * The ID of the phone type
	 *
	 * @const         int PHONE
	 */
	const PHONE = 2;

	/**
	 * The ID of the email type
	 *
	 * @const         int EMAIL
	 */
	const EMAIL = 3;

	/**
	 * The ID of the jabber type
	 *
	 * @const         int JABBER
	 */
	const JABBER = 4;

	/**
	 * The ID of the Mattermost type
	 *
	 * @const         int MATTERMOST
	 */
	const MATTERMOST = 5;

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The action name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The action description.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->name        = $data['name'];
		$this->description = $data['description'];
	}

	/**
	 * Returns a validator for the current SLAAction object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\SLAActionValidator
	 */
	public function getValidator() {
		return new SLAActionValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\SLAActionMapper
	 */
	public static function getMapper() {
		return datamapper\SLAActionMapper::getInstance();
	}

	/**
	 * Returns all alert templates for this kind
	 * of SLA action
	 *
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getAlertTemplates() {
		return datamapper\AlertTemplateMapper::getInstance()->getBySlaAction(
			$this->id
		);
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'multiple' => array(
				'type' => 'boolean'
			)
		);
	}
}
