<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ServiceValidator;
use ch\monitoring\util;

/**
 * Model for a service
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Service extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The default sla for this service
	 *
	 * @var           int $slaId
	 */
	public $slaId;


	/**
	 * The customer this service belongs to.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The command assigned to this service.
	 *
	 * @var           int $commandId
	 */
	public $commandId;

	/**
	 * The service name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the service.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * The max check attempts nagios parameter
	 *
	 * @var           int $maxCheckAttempts
	 */
	public $maxCheckAttempts;

	/**
	 * The check interval nagios parameter
	 *
	 * @var           int $checkInterval
	 */
	public $checkInterval;

	/**
	 * The retry interval nagios parameter
	 *
	 * @var           int $retryInterval
	 */
	public $retryInterval;
	
	/**
	 * The freshness nagios parameter
	 *
	 * @var           int $freshness
	 */
	public $freshness;

	/**
	 * Is this a nrpe service?
	 *
	 * @var           bool $isNrpe
	 */
	public $isNrpe;

	/**
	 * Is this service deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id               = (int)$data['id'];
		$this->slaId            = (int)$data['slaId'];
		$this->commandId        = (int)$data['commandId'];
		$this->customerId       = (int)$data['customerId'] ?: null;
		$this->name             = $data['name'];
		$this->description      = $data['description'];
		$this->maxCheckAttempts = (int)$data['maxCheckAttempts'];
		$this->checkInterval    = (int)$data['checkInterval'];
		$this->retryInterval    = (int)$data['retryInterval'];
		$this->freshness        = (int)$data['freshness'];
		$this->isNrpe           = (bool)$data['isNrpe'];
		$this->isDeleted        = (bool)$data['isDeleted'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\ServicePolicy($this);
	}

	/**
	 * Returns a validator for the current Service object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ServiceValidator
	 */
	public function getValidator() {
		return new ServiceValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ServiceMapper
	 */
	public static function getMapper() {
		return datamapper\ServiceMapper::getInstance();
	}

	/**
	 * Returns the Customer of this Service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the Command of this Service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Command
	 */
	public function getCommand() {
		return datamapper\CommandMapper::getInstance()->getByID(
			$this->commandId
		)->allowRead();
	}

	/**
	 * Returns the assigned arguments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $type
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getArguments($type = null) {
		return datamapper\ServiceArgumentMapper::getInstance()->getAll($this->id, null, $type);
	}

	/**
	 * Returns the assigned arguments as array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $type
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getArgumentsAsArray($type = null) {
		return util\IteratorUtil::getKeyValueArray(
			$this->getArguments($type),
			'id',
			'value'
		);
	}

	/**
	 * Returns true if this service has assigned arguments
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function hasArguments() {
		return (bool)count(datamapper\ServiceArgumentMapper::getInstance()->getAll($this->id, null));
	}

	/**
	 * Returns all HostServices this Service is assigned to
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\DbIterator
	 */
	public function getHostServices() {
		return datamapper\HostServiceMapper::getInstance()->getAll(
			null,
			$this->id
		);
	}

	/**
	 * Returns all Hosts this Service is assigned to as id => name pair
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getHostsAsArray() {
		$hostServices = $this->getHostServices();

		$hostServiceArray = array();
		foreach ($hostServices as $hostService) {
			$hostServiceArray[$hostService->hostId] = $hostService->getHost()->hostname;
		}

		return $hostServiceArray;
	}

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'commandId' => array(
				'type' => 'relation',
				'mapper' => 'CommandMapper'
			),

			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'isNrpe' => array(
				'type' => 'boolean'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			),

			'serviceId' => array(
				'type' => 'relation',
				'mapper' => 'ServiceArgumentMapper'
			)
		);
	}

	/**
	 * Returns all alerts which are related to this service
	 *
	 * @param         int $openOnly Optional Return only open alerts
	 * @returns       AppendIterator
	 */
	public function getAlerts($openOnly=null) {
		$itr = new \AppendIterator();
		foreach ($this->getHostServices() as $hs) {
			foreach ($hs->getInstances() as $hsi) {
				$itr->append($hsi->getAlerts($openOnly));
			}
		}
		return $itr;
	}

	/**
	 * Deletes dependencies of the service.
	 */
	public function deleteDependencies() {
		$serviceAlerts = $this->getAlerts();
		foreach ($serviceAlerts as $alert) {
			datamapper\AlertMapper::getInstance()->delete(
				array('id' => $alert->id)
			);
		}
		$hostServices = $this->getHostServices();
		foreach ($hostServices as $hostService) {
			$hostServiceInstances = $hostService->getInstances();
			foreach ($hostServiceInstances as $hsi) {
				$breComponents = $hsi->getBREComponents();
				foreach ($breComponents as $breComponent) {
					datamapper\BREServiceMapper::getInstance()->deleteService(
						$breComponent->id,
						$hsi->id
					);
				}
				$downtimes = $hsi->getDowntimes();
				foreach ($downtimes as $downtime) {
					$downtime->enforceWritePermission(model\Permission::SERVICES_WRITE);
					datamapper\DowntimeMapper::getInstance()->delete(
						array('id' => $downtime->id)
					);
				}
				datamapper\HostServiceInstanceMapper::getInstance()->delete(
					array('id' => $hsi->id)
				);
			}
			datamapper\HostServiceMapper::getInstance()->delete(
				array('id' => $hostService->id)
			);
		}
	}
}
