<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ServiceArgumentValidator;

/**
 * Model for a service argument
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceArgument extends EntityWithPolicy {

	/**
	 * Constant for type "Fixed"
	 *
	 * @const         string TYPE_FIXED
	 */
	const TYPE_FIXED = "Fixed";

	/**
	 * Constant for type "Variable"
	 *
	 * @const         string TYPE_VARIABLE
	 */
	const TYPE_VARIABLE = "Variable";

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The service this argument is assigned to.
	 *
	 * @var           int $serviceId
	 */
	public $serviceId;

	/**
	 * Position of the argument
	 *
	 * @var           int $position
	 */
	public $position;

	/**
	 * The name of this argument (e.g. commandline flag "-c" for nrpe calls).
	 *
	 * @var           string $option
	 */
	public $option;

	/**
	 * The argument (default) value.
	 *
	 * @var           string $value
	 */
	public $value;

	/**
	 * Human readable description of this argument.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Argument type (fixed or variable)
	 *
	 * @var           string $type
	 */
	public $type;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->serviceId   = (int)$data['serviceId'];
		$this->position    = (int)$data['position'];
		$this->option      = $data['option'];
		$this->value       = $data['value'];
		$this->description = $data['description'];
		$this->type        = $data['type'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\ServiceArgumentPolicy
	 */
	public function getPolicy() {
		return new policy\ServiceArgumentPolicy($this);
	}

	/**
	 * Returns a validator for the current ServiceArgument object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ServiceArgumentValidator
	 */
	public function getValidator() {
		return new ServiceArgumentValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ServiceArgumentMapper
	 */
	public static function getMapper() {
		return datamapper\ServiceArgumentMapper::getInstance();
	}

	/**
	 * Returns the Service this ServiceArgument belongs to.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return datamapper\ServiceMapper::getInstance()->getByID(
			$this->serviceId
		)->allowRead();
	}

	/**
	 * Returns the assigned value for variable argument values
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $instanceId
	 * @returns       string
	 */
	public function getAssignedValue($instanceId) {
		if ($this->type == self::TYPE_FIXED) {
			return $this->value;
		}

		try {
			return datamapper\HostServiceArgumentMapper::getInstance()->getByForeignKeys(
				$instanceId,
				$this->id
			)->allowRead()->assignedValue;
		}
		catch (\OutOfBoundsException $e) {
			return null;
		}
	}
}
