<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ServiceGroupValidator;

/**
 * Model for a service group
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceGroup extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The customer this service group belongs to.
	 *
	 * Null means every customer.
	 *
	 * @var           int $customerId
	 */
	public $customerId;

	/**
	 * The service group name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Short description of the service group.
	 *
	 * @var           string $description
	 */
	public $description;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id          = (int)$data['id'];
		$this->customerId  = (int)$data['customerId'] ?: null;
		$this->name        = $data['name'];
		$this->description = $data['description'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\ServiceGroupPolicy
	 */
	public function getPolicy() {
		return new policy\ServiceGroupPolicy($this);
	}

	/**
	 * Returns a validator for the current ServiceGroup object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ServiceGroupValidator
	 */
	public function getValidator() {
		return new ServiceGroupValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ServiceGroupMapper
	 */
	public static function getMapper() {
		return datamapper\ServiceGroupMapper::getInstance();
	}

	/**
	 * Returns the Customer of this ServiceGroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns the assigned members
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getMembers($permissionName = Permission::SERVICES_READ) {
		return datamapper\ServiceGroupMemberMapper::getInstance()->getAll(
			$this->id,
			$permissionName
		);
	}

	/**
	 * Returns the assigned members as an array of IDs
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getMemberIds($permissionName = null) {
		$members = $this->getMembers($permissionName);

		$membersArray = array();
		foreach ($members as $member) {
			$membersArray[] = $member->serviceId;
		}
		
		return $membersArray;
	}
}
