<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\ServiceGroupMemberValidator;

/**
 * Model for a service group member
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceGroupMember extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The service group this member belongs to.
	 *
	 * @var           int $groupId
	 */
	public $groupId;

	/**
	 * The service.
	 *
	 * @var           int $serviceId
	 */
	public $serviceId;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id        = (int)$data['id'];
		$this->groupId   = (int)$data['groupId'];
		$this->serviceId = (int)$data['serviceId'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\ServiceGroupMemberPolicy
	 */
	public function getPolicy() {
		return new policy\ServiceGroupMemberPolicy($this);
	}

	/**
	 * Returns a validator for the current ServiceGroupMember object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\ServiceGroupMemberValidator
	 */
	public function getValidator() {
		return new ServiceGroupMemberValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\ServiceGroupMemberMapper
	 */
	public static function getMapper() {
		return datamapper\ServiceGroupMemberMapper::getInstance();
	}

	/**
	 * Returns the ServiceGroup of this Member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\ServiceGroup
	 */
	public function getServiceGroup() {
		return datamapper\ServiceGroupMapper::getInstance()->getByID(
			$this->groupId
		)->allowRead();
	}

	/**
	 * Returns the Service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Service
	 */
	public function getService() {
		return datamapper\ServiceMapper::getInstance()->getByID(
			$this->serviceId
		)->allowRead();
	}

	/**
	 * Returns the SLA of this Member
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}
}
