<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\TimeframeValidator;
use ch\monitoring\model\EntityWithPolicy;

/**
 * Model for a Timeframe
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Timeframe extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The sla this rule belongs to.
	 *
	 * @var           int $slaId
	 */
	public $slaId;

	/**
	 * Whether this rule counts on Mondays.
	 *
	 * @var           bool $monday
	 */
	public $monday;

	/**
	 * Whether this rule counts on Tuesdays.
	 *
	 * @var           bool $tuesday
	 */
	public $tuesday;

	/**
	 * Whether this rule counts on Wednesdays.
	 *
	 * @var           bool $wednesday
	 */
	public $wednesday;

	/**
	 * Whether this rule counts on Thursdays.
	 *
	 * @var           bool $thursday
	 */
	public $thursday;

	/**
	 * Whether this rule counts on Fridays.
	 *
	 * @var           bool $friday
	 */
	public $friday;

	/**
	 * Whether this rule counts on Saturdays.
	 *
	 * @var           bool $saturday
	 */
	public $saturday;

	/**
	 * Whether this rule counts on Sundays.
	 *
	 * @var           bool $sunday
	 */
	public $sunday;

	/**
	 * Time from.
	 *
	 * @var           Zend_Date $timeFrom
	 */
	public $timeFrom;

	/**
	 * Time to.
	 *
	 * @var           Zend_Date $timeTo
	 */
	public $timeTo;

	/**
	 * Desfines the corresponding customer for the Timeframe
	 *
	 * @var           int|null $notifyHoster
	 */
	public $customerID;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		
		$this->monday       = (bool)$data['monday'];
		$this->tuesday      = (bool)$data['tuesday'];
		$this->wednesday    = (bool)$data['wednesday'];
		$this->thursday     = (bool)$data['thursday'];
		$this->friday       = (bool)$data['friday'];
		$this->saturday     = (bool)$data['saturday'];
		$this->sunday       = (bool)$data['sunday'];
		
		$this->timeFrom     = $data['timeFrom'];
		$this->timeTo       = $data['timeTo'];

		$this->slaId        = (int)$data['slaId'];
		$this->customerId   = (int)$data['customerId'];
	}

	/**
	 * Returns a policy for the current Timeframe object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a policy for the current Timeframe object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\TimeframeValidator
	 */
	public function getValidator() {
		return new TimeframeValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\TimeframeMapper
	 */
	public static function getMapper() {
		return datamapper\TimeframeMapper::getInstance();
	}

	/**
	 * Returns the SLA of this timeframe
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLA
	 */
	public function getSLA() {
		return datamapper\SLAMapper::getInstance()->getByID(
			$this->slaId
		)->allowRead();
	}

	/**
	 * Returns the assigned Customer of this Timeframe
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}


	/**
	 * Returns the assigned Rules of this Timeframe
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       org\adfinis\framework\db\DbIterator
	 */
	public function getRules() {
		return datamapper\TimeframeRuleMapper::getInstance()->getAll(
			$this->id
		);
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'monday' => array(
				'type' => 'boolean'
			),

			'tuesday' => array(
				'type' => 'boolean'
			),

			'wednesday' => array(
				'type' => 'boolean'
			),

			'thursday' => array(
				'type' => 'boolean'
			),

			'friday' => array(
				'type' => 'boolean'
			),

			'saturday' => array(
				'type' => 'boolean'
			),

			'sunday' => array(
				'type' => 'boolean'
			),

			'slaId' => array(
				'type' => 'relation',
				'mapper' => 'SLAMapper'
			),

			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			)
		);
	}
}
