<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\TimeframeRuleValidator;
use ch\monitoring\model\EntityWithPolicy;

/**
 * Model for a TimeframeRule
 *
 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class TimeframeRule extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The Timeframe this TimeframeRule belongs to.
	 *
	 * @var           int $slaId
	 */
	public $timeframeId;

	/**
	 * To action to take.
	 *
	 * @var           int $actionId
	 */
	public $actionId;

	/**
	 * The delay before this rule applies.
	 *
	 * @var           int $delay
	 */
	public $delay;


	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id           = (int)$data['id'];
		$this->timeframeId  = (int)$data['timeframeId'];
		$this->actionId     = (int)$data['actionId'];
		$this->delay        = (int)$data['delay'];
	}

	/**
	 * Returns a validator for the current TimeframeRule object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\TimeframeRuleValidator
	 */
	public function getValidator() {
		return new TimeframeRuleValidator($this);
	}

	/**
	 * Returns a policy for the current TimeframeRule object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\CustomerBoundPolicy
	 */
	public function getPolicy() {
		return new policy\CustomerBoundPolicy($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\TimeframeRuleMapper
	 */
	public static function getMapper() {
		return datamapper\TimeframeRuleMapper::getInstance();
	}


	/**
	 * Returns the action of this rule
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\SLAAction
	 */
	public function getSLAAction() {
		return datamapper\SLAActionMapper::getInstance()->getByID(
			$this->actionId
		);
	}

	/**
	 * Returns the teimframe of this rule
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Timeframe
	 */
	public function getTimeframe() {
		return datamapper\TimeframeMapper::getInstance()->getByID(
			$this->timeframeId
		)->allowRead();
	}


	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'actionId' => array(
				'type' => 'relation',
				'mapper' => 'SLAActionMapper'
			)
		);
	}
}
