<?php

namespace ch\monitoring\model;

use ch\monitoring\Application;
use org\adfinis\framework\model;
use org\adfinis\framework\config\ConfManager;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\acl;
use ch\monitoring\validate\UserValidator;
use ch\monitoring\util;
use ch\monitoring\model\Permission;

/**
 * Model for a user
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class User extends EntityWithPolicy {
	/**
	 * Local authentication method
	 *
	 * @const         string AUTH_METHOD_LOCAL
	 */
	const AUTH_METHOD_LOCAL = 'LOCAL';

	/**
	 * LDAP authentication with sync
	 *
	 * @const         string AUTH_METHOD_LDAP_SYNC
	 */
	const AUTH_METHOD_LDAP_SYNC = 'LDAP-SYNC';

	/**
	 * LDAP authentication without sync
	 *
	 * @const         string AUTH_METHOD_LDAP
	 */
	const AUTH_METHOD_LDAP = 'LDAP';

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The user name.
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * The user full name.
	 *
	 * @var           string $fullName
	 */
	public $fullName;

	/**
	 * The user password for login.
	 *
	 * @var           string $password
	 */
	public $passwordHash;

	/**
	 * The salt for the password hash
	 *
	 * @var           string $passwordSalt
	 */
	public $passwordSalt;
	
	/**
	 * Cell phone number for notification.
	 *
	 * @var           string $phone
	 */
	public $phone;

	/**
	 * eMail address for notification.
	 *
	 * @var           string $email
	 */
	public $email;


	/**
	 * Phone number for SMS for notification.
	 *
	 * @var           string $sms
	 */
	public $sms;

	/**
	 * Jabber for notification.
	 *
	 * @var           string $jabber
	 */
	public $jabber;

	/**
	 * Mattermost for notification.
	 *
	 * @var           string $mattermost
	 */
	public $mattermost;

	/**
	 * Authentication method
	 *
	 * @var           string $authenticationMethod
	 */
	public $authenticationMethod;

	/**
	 * Is this user enabled?
	 *
	 * @var           bool $isEnabled
	 */
	public $isEnabled;

	/**
	 * Is this user deleted?
	 *
	 * @var           bool $isDeleted
	 */
	public $isDeleted = false;


	/**
	 * Default Customer for this user
	 *
	 * @var           bool $customerId
	 */
	public $customerId;

	/**
	 * Stores an implementation of acl\ACL
	 *
	 * @var           acl\ACL $acl
	 */
	private $ACL = null;

	/**
	 * Stores the general menu structure
	 *
	 * @var           string $menuStructure
	 */
	private $menuStructure = null;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id                   = (int)$data['id'];
		$this->name                 = $data['name'];
		$this->fullName             = $data['fullName'];
		$this->passwordHash         = $data['passwordHash'];
		$this->passwordSalt         = $data['passwordSalt'];
		$this->phone                = $data['phone'];
		$this->sms                  = $data['sms'];
		$this->email                = $data['email'];
		$this->jabber               = $data['jabber'];
		$this->mattermost           = $data['mattermost'];
		$this->authenticationMethod = $data['authenticationMethod'];
		$this->isEnabled            = (bool)$data['isEnabled'];
		$this->isDeleted            = (bool)$data['isDeleted'];
		$this->customerId           = (int)$data['customerId'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\UserPolicy
	 */
	public function getPolicy() {
		return new policy\UserPolicy($this);
	}

	/**
	 * Returns a validator for the current User object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\UserValidator
	 */
	public function getValidator() {
		return new UserValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\UserMapper
	 */
	public static function getMapper() {
		return datamapper\UserMapper::getInstance();
	}

	/**
	 * Sets the password as hash with a password salt
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $password
	 * @returns       void
	 */
	public function setPassword($password) {
		$this->passwordSalt = uniqid(null, true);

		$this->passwordHash = sha1(sprintf(
			'%s:%s',
			sha1($password),
			$this->passwordSalt
		));
	}

	/**
	 * Authenticates the user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $password
	 * @returns       bool
	 */
	public function authenticate($password) {
		$allowLocal = ConfManager::getVar('af', 'auth', 'local') == 'yes';
		$allowLdap  = ConfManager::getVar('af', 'auth', 'ldap')  == 'yes';

		if ($allowLdap && $this->hasLDAPAuthentication()) {
			$ldap = new util\Ldap();
			$ldapUser = $ldap->authenticate($this->name, $password);
			if ($this->authenticationMethod === self::AUTH_METHOD_LDAP) {
				$this->setPassword($password);
				$ldap->synchronize($ldapUser, $this);
			}
			return true;
		}
		else if ($allowLocal && $this->hasLocalAuthentication()) {
			$passwordHash = sha1(sprintf(
				'%s:%s',
				sha1($password),
				$this->passwordSalt
			));
			return $passwordHash === $this->passwordHash;
		}
		throw new \RuntimeException(
			tr('User authentication method not allowed')
		);
	}

	/**
	 * Returns an array of customer ids and Role objects.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getCustomersWithRoles() {
		return datamapper\CustomerMapper::getInstance()->getCustomersWithRoles(
			$this->id
		);
	}

	/**
	 * Returns whether a user got a certain permission for any customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function hasPermission($permissionName) {
		return $this->getACL()->hasPermission($permissionName);
	}

	/**
	 * Enforce a permission
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       void
	 */
	public function enforcePermission($permissionName, $all=false) {
		if ($all) {
			$customers = $this->getCustomersWithRoles();

			foreach (array_keys($customers) as $customerId) {
				Application::getUser()->getACL()->enforcePermissionForCustomer(
					$permissionName,
					$customerId
				);
			}
		}
		else {
			$this->getACL()->hasPermission($permissionName);
		}

		$this->readPermissionChecked = true;
	}

	/**
	 * Returns whether a user got a certain permission for a certain customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId
	 * @returns       bool
	 */
	public function hasPermissionForCustomer($permissionName, $customerId) {
		return $this->getACL()->hasPermissionForCustomer(
			$permissionName,
			$customerId
		);
	}

	/**
	 * Returns whether a user got a certain permission for a certain customer
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function hasPermissionForWholeSystem($permissionName) {
		return $this->getACL()->hasPermissionForWholeSystem($permissionName);
	}

	/**
	 * Enforces a permission for the whole system (global customer)
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       void
	 */
	public function enforcePermissionForWholeSystem($permissionName) {
		return $this->getACL()->enforcePermissionForWholeSystem($permissionName);
	}

	/**
	 * Enforce a permission for customer
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @param         int $customerId
	 * @returns       void
	 */
	public function enforcePermissionForCustomer($permissionName, $customerId) {
		$this->getACL()->enforcePermissionForCustomer(
			$permissionName,
			$customerId
		);
	}

	/**
	 * Returns all the customers this user got a certain permission on
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @returns       array
	 */
	public function getCustomersWithPermission($permissionName) {
		return $this->getACL()->getCustomersWithPermission($permissionName);
	}

	/**
	 * Returns a SessionCachedACL
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       acl\SessionCachedACL
	 */
	public function getACL() {
		if ($this->ACL == null) {
			$this->ACL = acl\ACLFactory::factory($this->id);
		}

		return $this->ACL;
	}

	/**
	 * Returns the matching Ldap user, if there is none null is returned
	 *
	 * @return        User
	 */
	public function getLdapUser() {
		if (!$this->hasLDAPAuthentication()) {
			throw new \RuntimeException(
				tr('Cannot get the matching Ldap user if the authentication method is not Ldap.')
			);
		}
		try {
			$ldap = new util\Ldap();
		}
		catch (\RuntimeException $e) {
			return null;
		}
		return $ldap->getUserByName(
			$this->name,
			array("cn", "uid", "mail", "mobile", "jabber")
		);
	}

	/**
	 * This function indicates if the data of a user can be changed
	 *
	 * It does NOT check for permission! Beside the fact, that the model can not be edited
	 * there are a few extra restrictions which prevents the system for changing the data
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string|null $field the property that should be checked
	 * @returns       bool
	 */
	public function isMutable($field = null) {
		if ($this->authenticationMethod === self::AUTH_METHOD_LDAP) {
			$user = $this->getLdapUser();
			if (isset($user) && $user['count'] > 0) {
				$user = $user[0];
				$optionalLdapColumns = array(
					'phone'  => 'mobile',
					'sms'    => 'mobile',
					'email'  => 'mail',
					'jabber' => 'jabber',
					'mattermost' => 'mattermost'
				);
				if(isset($optionalLdapColumns[$field])) {
					$ldapColumName = $optionalLdapColumns[$field];
					if(!isset($user[$ldapColumName][0])) {
						return true;
					}
				}
			}
			return false;
		}
		return true;
	}

	/**
	 * Get or create an active user
	 *
	 * If there is no current user, create a new one with read permission
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        self
	 */
	public static function getOrCreateActive() {
		$session = \org\adfinis\framework\session\SessionManager::getInstance();

		if (isset($session->user) && $session->user instanceof self) {
			return $session->user;
		}
		else {
			$user = new self();
			$user->allowRead();
			return $user;
		}
	}

	/**
	 * Returns the primary Customer of this User
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\Customer
	 */
	public function getCustomer() {
		return datamapper\CustomerMapper::getInstance()->getByID(
			$this->customerId
		)->allowRead();
	}

	/**
	 * Returns AlertTemplateUser for a specific notificationtype
	 *
	 * @param         ch\monitoring\model\SLAAction $slaAction
	 *
	 * @returns       ch\monitoring\model\AlertTemplateUser
	 */
	public function getAlertTemplateSettings($slaAction) {
		return datamapper\AlertTemplateUserMapper::getInstance()->getByNotificationType(
			$this->id,
			$slaAction->id
		);
	}

	/**
	 * Returns AlertNotificationExcludeUser for 
	 * specific SLA action and Customer
	 *
	 * @param         ch\monitoring\model\Customer  $customerId
	 * @param         ch\monitoring\model\SLAAction $slaAction
	 *
	 * @returns       ch\monitoring\model\AlertNotificationExcludeUser
	 */
	public function getAlertSettingsForSlaAction($customer, $slaAction) {
		return datamapper\AlertNotificationExcludeMapper::getInstance()->getByNotificationType(
			$this->id,
			$customer->id,
			$slaAction->id
		);
	}

	/**
	 * Fills the menu structure
	 *
	 * @todo by denis:   Move to a more obvious place
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function fillMenu() {
		$this->menuStructure = array();
		
		$this->addLink("Manage", "Hosts",             "hosts",          Permission::HOSTS_READ);
		$this->addLink("Manage", "Host Groups",       "hostGroups",     Permission::HOSTS_READ);
		$this->addLink("Manage", "Applications",      "applications",   Permission::HOSTS_READ);

		$this->addSeparator("Manage", "hostseparator");

		$this->addLink("Manage", "Commands",          "commands",       Permission::SERVICES_READ);
		$this->addLink("Manage", "Services",          "services",       Permission::SERVICES_READ);
		$this->addLink("Manage", "Service groups",    "groups",         Permission::SERVICES_READ);
		
		$this->addSeparator("Manage", "usercategory");

		$this->addLink("Manage", "Users",             "users",          Permission::USERS_READ);
		$this->addLink("Manage", "Customers",         "customers",      Permission::CUSTOMERS_READ);
		$this->addLink("Manage", "Roles",             "roles",          Permission::ROLES_READ);
		
		$this->addSeparator("Manage", "operatingsystemcategory");

		$this->addLink("Manage", "Operating Systems", "hostOS",         Permission::HOSTS_MASTER);
		$this->addLink("Manage", "Architectures",     "architectures",  Permission::HOSTS_MASTER);
		$this->addLink("Manage", "Host Types",        "hostTypes",      Permission::HOSTS_MASTER);
		$this->addLink("Manage", "Global Settings",   "setting",        Permission::GLOBAL_SETTINGS);
		$this->addLink("Manage", "Alert Templates",   "alerttemplate",  Permission::SLAS_READ);
		$this->addLink("Manage", "Alert Schedulers",  "slas",           Permission::SLAS_READ);
		


		$this->addLink("Monitoring", "New alerts",           "alerts",        Permission::ALERTS_READ);
		$this->addLink("Monitoring", "Open alerts",          "alertsOpen",    Permission::ALERTS_READ);
		$this->addLink("Monitoring", "Closed alerts",        "alertsHistory", Permission::ALERTS_READ);
		$this->addLink("Monitoring", "Notifications",        "notifications", Permission::NOTIFICATIONS_READ);

		$this->addSeparator("Monitoring", "downtimescategory");

		$this->addLink("Monitoring", "Downtimes",            "downtimes",     Permission::DOWNTIMES_READ);
		$this->addLink("Monitoring", "On-call teams",        "calendar",      Permission::EMERGENCY_READ);

		$this->addSeparator("Monitoring", "exportcategory");

		$this->addLink("Monitoring", "Export config",        "hostsExport",   Permission::HOSTS_WRITE);
		$this->addLink("Monitoring", "Import config",        "import",        Permission::IMPORT);
		

		$this->addLink("Maintenance", "Tickets",            "maintenanceOverview",   Permission::MAINTENANCE_READ);
		$this->addLink("Maintenance", "History",            "maintenanceHistory",    Permission::MAINTENANCE_READ);
		$this->addLink("Maintenance", "Checklists",         "maintenanceChecklists", Permission::MAINTENANCE_ADMIN);
		$this->addLink("Maintenance", "Hosts",              "maintenanceHosts",      Permission::MAINTENANCE_ADMIN);
		$this->addLink("Maintenance", "Maintenance groups", "maintenanceGroups",     Permission::MAINTENANCEGROUPS_READ);

		$this->addLink("Analysis", "Reports",         "reports",        Permission::REPORTS);
		$this->addLink("Analysis", "Hosts hierarchy", "hostsHierarchy", Permission::HOSTS_READ);
		$this->addLink("Analysis", "Audit log",       "auditLog",       Permission::AUDIT_LOG_READ);
	}

	/**
	 * Helper function to add a link to the menu structure
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $menu The name of the menu
	 * @param         string $submenu The name of the submenu
	 * @param         string $page The page to be called
	 * @param         string optional $permission The required permission for the link
	 * @returns       void
	 */
	function addLink($menu, $submenu, $page, $permission = null) {
		$this->menuStructure[$menu][$submenu] = array(
			'page' => $page,
			'permission' => $permission
		);
	}

	/**
	 * Helper function to add a separator (between to menupoints) to the menu structure
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $menu The name of the menu
	 * @param         string $separatorname The name of the separator
	 * @returns       void
	 */
	public function addSeparator($menu, $separatorname) {
		$this->menuStructure[$menu][$separatorname] = array(
			'isSeparator' => true
		);
	}

	
	/**
	 * Helper function to remove links, where the user has no access on.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         array  submenu like this menuStructure
	 * @returns       array  submenu without forbidden links
	 */
	protected function clearForbiddenLinks($submenu) {
		foreach ($submenu as &$sub) {
			foreach ($sub as $linkId => $link) {
				if (
					isset($link['permission'])
					&& !$this->hasPermission($link['permission'])
				) {
					unset($sub[$linkId]);
				}
			}
		}
		return $submenu;
	}

	/**
	 * Helper function to remove spare separators
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         array  submenu like this menuStructure
	 * @returns       array  submenu without spare separators
	 */
	protected function removeSpareSeparators($submenu) {
		$hasPrevious = false;
		foreach ($submenu as &$sub) {
			foreach ($sub as $link => $content) {
				//check if separator
				if(!isset($content['permission'])) {
					//if separator is first element in list remove
					if (!$hasPrevious) {
						unset($sub[$link]);
					}
					else {
						//if hasPrevious let separator stay but set hasPrevious to false
						$hasPrevious = false;
					}
				}
				//previous element was not separator -> hasPrevious true
				else {
					$hasPrevious = true;
				}
			}
			//delete separator, if it is last element in list
			$sub_end = end($sub);
			if (isset($sub_end['isSeparator']) && $sub_end['isSeparator']) {
				array_pop($sub);
			}
		}

		return $submenu;
	}

	/**
	 * Returns the menu structure for the current user
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $submenu Limit returned value to submenu
	 * @throws        \OutOfBoundsException
	 * @returns       array
	 */
	public function getMenu($submenu = null) {
		if ($this->menuStructure === null) {
			$this->fillMenu();
		}

		if ($submenu !== null) {
			if (!array_key_exists($submenu, $this->menuStructure)) {
				throw new \OutOfBoundsException(
					tr("Submenu does not exist")
				);
			}
			$menu = array($submenu => $this->menuStructure[$submenu]);
		}
		else {
			$menu = $this->menuStructure;
		}
		$menu = $this->clearForbiddenLinks($menu);
		$menu = $this->removeSpareSeparators($menu);
		return $menu;
	}

	/**
	 * Returns the submenu name of a certain page
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $page The page name to be looked up
	 * @throws        \OutOfBoundsException
	 * @returns       string
	 */
	public function getMenuByPage($page) {
		foreach ($this->getMenu() as $menu => $submenu) {
			foreach ($submenu as $link) {
				if (isset($link['page']) && $link['page'] == $page) {
					return $this->getMenu($menu);
				}
			}
		}

		throw new \OutOfBoundsException(
			tr("Page does not exist")
		);
	}

	/**
	 * Returns wether a User has LOCAL authentification or not
	 *
	 * @returns       bool Wether the current user has LOCAL authentification or
	 *                     not
	 */
	public function hasLocalAuthentication() {
		return (
			$this->authenticationMethod === self::AUTH_METHOD_LOCAL
		);
	}

	/**
	 * Returns wether a User has LDAP authentification or not
	 *
	 * @returns       bool Wether the current user has LDAP authentification or
	 *                     not
	 */
	public function hasLDAPAuthentication() {
		return (
			$this->authenticationMethod === self::AUTH_METHOD_LDAP or
			$this->authenticationMethod === self::AUTH_METHOD_LDAP_SYNC
		);
	}

	/**
	 * Returns the translated authentication method of the current user
	 *
	 * @returns       string           the translated authentification method
	 */
    public function getAuthenticationMethod() {
        $authMethods = array(
			self::AUTH_METHOD_LOCAL     => tr('Local'),
			self::AUTH_METHOD_LDAP      => tr('LDAP synced automatically'),
			self::AUTH_METHOD_LDAP_SYNC => tr('LDAP synced manually'),
		);
        return $authMethods[$this->authenticationMethod];
    }

	/**
	 * Returns a list that can be used to map the columns to the corresponding datamapper
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	static public function getEntityDefinition() {
		return array(
			'customerId' => array(
				'type' => 'relation',
				'mapper' => 'CustomerMapper'
			),

			'emergency' => array(
				'type' => 'boolean'
			),

			'isEnabled' => array(
				'type' => 'boolean'
			),

			'isDeleted' => array(
				'type' => 'boolean'
			)
		);
	}
}
