<?php

namespace ch\monitoring\model;

use org\adfinis\framework\model;
use ch\monitoring\datamapper;
use ch\monitoring\policy;
use ch\monitoring\validate\UserLoginValidator;

/**
 * Model for a user login
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class UserLogin extends EntityWithPolicy {

	/**
	 * Primary key.
	 *
	 * @var           int $id
	 */
	public $id;

	/**
	 * The user which logged in.
	 *
	 * @var           int $userId
	 */
	public $userId;

	/**
	 * The timestamp when it happened
	 *
	 * @var           Zend_Date $timestamp
	 */
	public $timestamp;

	/**
	 * The IP address of the user.
	 *
	 * @var           string $IP
	 */
	public $IP;

	/**
	 * Does the database record belong to this object?
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	public function doesRecordMatch($data) {
		return ($this->id === (int) $data['id']);
	}

	/**
	 * Returns the identifier of the current object
	 *
	 * The identifier is an associative array that contains all the information
	 * for identifying the current object.
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       array
	 */
	public function getIdentifier() {
		return array('id' => $this->id);
	}

	/**
	 * This method fills the object from an associative array, usually a row
	 * from the database.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @returns       void
	 */
	public function fillFromRowData($data) {
		$this->id        = (int)$data['id'];
		$this->userId    = (int)$data['userId'];
		$this->timestamp = new \Zend_Date($data['timestamp']);
		$this->IP        = $data['IP'];
	}

	/**
	 * Returns a policy for the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\policy\UserLoginPolicy
	 */
	public function getPolicy() {
		return new policy\UserLoginPolicy($this);
	}

	/**
	 * Returns a validator for the current UserLogin object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\validate\UserLoginValidator
	 */
	public function getValidator() {
		return new UserLoginValidator($this);
	}

	/**
	 * Returns a data mapper for handling persistence of this object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\datamapper\UserLoginMapper
	 */
	public static function getMapper() {
		return datamapper\UserLoginMapper::getInstance();
	}

	/**
	 * Returns the User who logged in
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       ch\monitoring\model\User
	 */
	public function getUser() {
		return datamapper\UserMapper::getInstance()->getByID(
			$this->userId
		)->allowRead();
	}

	/**
	 * Initialize fields
	 *
	 * @author        Jean-Louis Fuchs <jean-louis.fuchs@adfinis-sygroup.ch>
	 */
   function __construct() {
       $this->timestamp = new \Zend_Date();
   }
}
