<?php

namespace ch\monitoring\policy;

use ch\monitoring\Application;
use org\adfinis\framework\model\Entity;
use org\adfinis\framework\exceptions\ForbiddenException;

/**
 * Base policy to enforce or verify permissions on an object
 * This class implements the basic for verifying the object in relation with a set of customers
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2013
 * @license       LGPLv3
 **/

class CustomerBoundPolicy {
	/**
	 * Stores the model which needs to be checked
	 *
	 * @var           Entity $model
	 */
	protected $model;

	/**
	 * Save the model to be verified
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Entity $model
	 * @returns       void
	 */
	final public function __construct(Entity $model) {
		$this->model = $model;
	}

	/**
	 * Returns true if the user has access to this object
	 *
	 * @param         string $permissionName
	 * @returns       bool
	 */
	public function hasPermission($permissionName, $checkAll=false) {
		$identifiers   = $this->getCustomerIdentifiers();
		$user          = Application::getUser();

		if ($checkAll) {
			foreach($identifiers as $id) {
				if ($id === null) {
					continue;
				}

				if(!$user->hasPermissionForCustomer($permissionName, $id)) {
					return false;
				}
			}
			return true;
		}
		else {
			foreach($identifiers as $id) {
				if ($id === null) {
					continue;
				}

				if($user->hasPermissionForCustomer($permissionName, $id)) {
					return true;
				}
			}
			return false;
		}

	}

	/**
	 * Throws an exception if the user has no access
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $permissionName
	 * @throws        ForbiddenException
	 * @returns       void
	 */
	final public function enforcePermission($permissionName) {
		$hasPermission = $this->hasPermission($permissionName);
		if (!$hasPermission) {
			throw new ForbiddenException(
				tr("You are not authorized to modify this object")
			);
		}
	}

	/**
	 * This function returns an array of the customerId's that the associated object belongs to
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       array
	 */
	protected function getCustomerIdentifiers() {
		//if an object does not belong to a specific customer, it belongs to the root customer
		return array(Application::getUser()->getACL()->getRootCustomerIdentifier());
	}
}
