<?php

namespace ch\monitoring\util;

use ch\monitoring\model;
use ch\monitoring\datamapper;
use ch\monitoring\Application;

/**
 * Manipulate cacti database
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Cacti {

	/**
	 * The user information
	 *
	 * @var           array $user
	 */
	private $user;

	/**
	 * The password for the user
	 *
	 * @var           string $password
	 */
	private $password;

	/**
	 * Cacti internal value for representating the ldap access for an user
	 *
	 * @var           int $REALM_LDAP
	 */
	const REALM_LDAP  = 1;

	/**
	 * Cacti internal value for representating the db access for an user
	 *
	 * @var           int $REALM_LDAP
	 */
	const REALM_LOCAL  = 0;

	/**
	 * Object constuctor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\User $user
	 */
	public function __construct($user, $password) {
		$this->user = $user;
		$this->password = $password;
	}

	/**
	 * Synchronizes a user for cacti
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $userId SyMonitoring user id
	 * @throws        \UnexpectedValueException
	 * @returns       void
	 */
	public function synchronize() {
		if ($this->user === null) {
			throw new \UnexpectedValueException('A user must be provided');
		}

		if (!$this->user->hasPermission(model\Permission::CACTI_WEB)) {
			Application::getDbInstance('cacti')->delete(
				'user_auth',
				array(
					'`username` = ?' => $this->user->name,
					'`realm` = ?' => self::REALM_LDAP
				)
			);

			return;
		}

		if ($this->user->authenticationMethod == model\User::AUTH_METHOD_LOCAL) {
			$realm = self::REALM_LOCAL;
			$password = md5($this->password);
		}
		else if ($this->user->hasLDAPAuthentication()) {
			$realm    = self::REALM_LDAP;
			$password = '!';
		}
		else {
			throw new \UnexpectedValueException('Unknown user authentification method');
		}

		// Check whether this user already exists (username is not defined as unique in cacti db scheme)
		$statement = Application::getDbInstance('cacti')->prepare("
			SELECT
				`id`
			FROM
				`user_auth`
			WHERE
				`username` = :username
				AND
				`realm` = :realm
		");

		$statement->execute(
			array(
				'username' => $this->user->name,
				'realm'    => $realm
			)
		);

		$cactiUser = $statement->fetch();

		$priviledged = $this->user->hasPermission(
			model\Permission::CACTI_MASTER
		);

		$isAdmin = $this->user->hasPermission(
			model\Permission::CACTI_ADMIN
		);

		// User does not exist, create
		if (empty($cactiUser)) {
			// Insert new user with realm 'Ldap' and Tree Policy 'Allow' (1) or 'Deny' (2)
			$statement = Application::getDbInstance('cacti')->prepare("
				INSERT INTO
					`user_auth`
				SET
					`username` = :username,
					`full_name` = :fullname,
					`password` = :password,
					`realm` = :realm,
					`policy_trees` = :policyTrees
			");

			$statement->execute(array(
				'username'    => $this->user->name,
				'fullname'    => $this->user->fullName,
				'policyTrees' => $priviledged ? 1 : 2,
				'realm'       => $realm,
				'password'    => $password
			));

			if ($statement->rowCount() == 0) {
				throw new \UnexpectedValueException('Cacti user creation failed');
			}

			$cactiUser['id'] = Application::getDbInstance('cacti')->lastInsertId();
		}
		else {
			// Insert new user with realm 'Ldap' and Tree Policy 'Allow' (1) or 'Deny' (2)
			$statement = Application::getDbInstance('cacti')->prepare("
				UPDATE
					`user_auth`
				SET
					`username` = :username,
					`full_name` = :fullname,
					`policy_trees` = :policyTrees,
					`password` = :password
				WHERE
					`id` = :id
			");

			$statement->execute(array(
				'username' => $this->user->name,
				'fullname' => $this->user->fullName,
				'policyTrees' => $priviledged ? 1 : 2,
				'id' => $cactiUser['id'],
				'password' => $password
			));
		}

		$this->setPermissions($cactiUser['id'], $priviledged, $isAdmin);
	}

	/**
	 * Restricts access of the user to the customer trees
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId Cacti user id
	 * @param         bool $priviledged
	 * @param         bool $isAdmin
	 * @throws        \UnexpectedValueException
	 * @returns       void
	 */
	private function setPermissions($userId, $priviledged, $isAdmin) {
		// Remove any permissions for this user
		$statement = Application::getDbInstance('cacti')->prepare("
			DELETE FROM
				`user_auth_perms`
			WHERE
				`user_id` = :userId
		");
		$statement->execute(array('userId' => $userId));

		// Remove old priviledges and grant "View Graphs" realm permission
		$statement = Application::getDbInstance('cacti')->prepare("
			DELETE FROM
				`user_auth_realm`
			WHERE
				`user_id` = :userId
		");
		$statement->execute(array('userId' => $userId));

		$customers = $this->user->getCustomersWithPermission(
			model\Permission::CACTI_WEB
		);

		// == instead of === is correct, the following INSERT allows full access!
		if (!$priviledged && !$isAdmin && $customers == null) {
			return;
		}

		if ($isAdmin) {
			$realmIds = array(1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17);
		}
		else {
			$realmIds = array(7);
		}

		Application::getDbInstance('cacti')->beginTransaction();

		$statement = Application::getDbInstance('cacti')->prepare("
			INSERT IGNORE INTO
				`user_auth_realm`
			SET
				`realm_id` = :realmId,
				`user_id`  = :userId
		");

		foreach ($realmIds as $realmId) {
			$statement->execute(array(
				'userId' => $userId,
				'realmId' => $realmId
			));
		}

		$statement = Application::getDbInstance('cacti')->prepare("
			UPDATE
				`user_auth`
			SET
				`show_list` = :show_list,
				`show_preview` = :show_preview,
				`graph_settings` = :graph_settings
			WHERE
				`id` = :id
		");

		$statement->execute(array(
			'id' => $userId,
			'show_list'      => $isAdmin ? 'on' : '',
			'show_preview'   => $isAdmin ? 'on' : '',
			'graph_settings' => $isAdmin ? 'on' : '' 
		));
		
		// No tree restrictions if priviledged
		if ($priviledged || $isAdmin) {
			Application::getDbInstance('cacti')->commit();
			return;
		}

		foreach ($customers as $customerId) {
			$customer = datamapper\CustomerMapper::getInstance()->getByID(
				$customerId
			)->allowRead();

			$treeId = $this->getTreeIdByName($customer->name, true);
			$preparedData = array(
				'userId' => $userId,
				'itemId' => $treeId
			);

			// Restrict view to the tree
			$statement = Application::getDbInstance('cacti')->prepare("
				INSERT INTO
					`user_auth_perms`
				SET
					`user_id` = :userId,
					`item_id` = :itemId,
					`type` = 2
			");

			$statement->execute($preparedData);
		}

		Application::getDbInstance('cacti')->commit();
	}

	/**
	 * Get tree id by tree name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $treeName
	 * @param         bool $create Whether to create a new tree if it does not exist
	 * @throws        \UnexpectedValueException
	 * @returns       int
	 */
	private function getTreeIdByName($treeName, $create = false) {
		$statement = Application::getDbInstance('cacti')->prepare("
			SELECT
				`id`
			FROM
				`graph_tree`
			WHERE
				`name` = :name
		");
		$statement->execute(array('name' => $treeName));
		$tree = $statement->fetch();

		if (empty($tree)) {
			// Create on failure?
			if (!$create) {
				throw new \UnexpectedValueException(
					tr('Customer tree does not exist')
				);
			}

			$statement = Application::getDbInstance('cacti')->prepare("
				INSERT INTO
					`graph_tree`
				SET
					`sort_type` = 1,
					`name` = :name
			");
			$statement->execute(array('name' => $treeName));

			return Application::getDbInstance('cacti')->lastInsertId();
		}

		return $tree['id'];
	}

	/**
	 * Update customer name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \UnexpectedValueException
	 * @param         int $customerId
	 * @param         string $oldName
	 * @returns       void
	 */
	public function updateCustomer($newName, $oldName) {
		if ($newName == null || $oldName == null) {
			throw new \UnexpectedValueException(
				tr('New and old tree name must be provided')
			);
		}

		$statement = Application::getDbInstance('cacti')->prepare("
			UPDATE
				`graph_tree`
			SET
				`name` = :newName
			WHERE
				`name` = :oldName
		");
		$statement->execute(array(
			'newName' => $newName,
			'oldName' => $oldName
		 ));
	}
}
