<?php

namespace ch\monitoring\util;

use ch\monitoring\Application;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use org\adfinis\framework\template\Skittle;
use org\adfinis\framework\config\ConfManager;

/**
 * Export configuration files from smarty template
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ConfigExport {

	/**
	 * Smarty/Skittle instance
	 * @var           org\adfinis\framework\template\Skittle $tpl
	 */
	public $tpl = null;

	/**
	 * Specifies if the global permissions should be checked
	 *
	 * @var           bool $shouldCheckPermission
	 */
	protected $shouldCheckPermission = true;

	/**
	 * Object constuctor
	 *
	 * @param         bool $writeCheckOnly indicates wether the real or the
	 *                                     configuration for performing
	 *                                     configuration check through Icinga
	 *                                     shall be written
	 * @throws        UnexpectedValueException
	 */
	public function __construct($writeCheckOnly=false) {
		// create smarty instance if needed
		if (!($this->tpl instanceof Skittle) && !$this->tpl = new Skittle()){
			throw new UnexpectedValueException(
				tr('Could not create a Skittle instance')
			);
		}
		$this->writeCheckOnly = $writeCheckOnly;
		if ($this->writeCheckOnly) {
			$this->objectsRoot = ConfManager::getVar(
				'af', 'objectsTempRoot', 'uri'
			);
		}
		else {
			$this->objectsRoot = ConfManager::getVar(
				'af', 'objectsRoot', 'uri'
			);
		}

		return $this;
	}

	/**
	 * Disable global permission checks
	 *
	 * @return        void
	 */
	public function disablePermissionChecks() {
		$this->shouldCheckPermission = false;
	}

	/**
	 * Enable global permission checks
	 *
	 * @return        void
	 */
	public function enablePermissionChecks() {
		$this->shouldCheckPermission = true;
	}

	/**
	 * Export the template to a file
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $tplFile
	 * @param         string $exportFile
	 * @param         string $chgrp
	 * @returns       void
	 */
	public function exportToFile($tplFile, $exportFile, $chgrp=null) {
		$fp = fopen($exportFile, 'w');

		if (!$fp) {
			Application::getLogInstance()->err(sprintf(
				"%s: Could not open file '%s' for writing",
				__METHOD__, $exportFile
			));
			throw new \RuntimeException(
				sprintf(tr('Could not open file "%s" for writing.'), $exportFile)
			);
		}

		// utf8_decode because ido2db only handles latin1
		fwrite($fp, utf8_decode($this->tpl->fetch($tplFile)));
		fclose($fp);
		Application::getLogInstance()->debug(sprintf(
			"%s: Successfully wrote file '%s'",
			__METHOD__, $exportFile
		));

		if ($chgrp) {
			chgrp($exportFile, $chgrp);
		}

		$this->tpl->clearAllAssign();
	}

	/**
	 * Build master system nagios/icinga configuration files
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function buildMaster() {
		Application::getLogInstance()->debug(sprintf(
			"Going to build master configuration",
			__METHOD__
		));
		$allHosts = datamapper\HostMapper::getInstance()->getAll(
			false,
			false,
			false,
			null,
			null,
			null,
			true
		);

		//only take those hosts that are not disabled by the customer
		$hosts = array();
		foreach($allHosts as $host) {
			if($host->isEnabled()) {
				$hosts[] = $host;
			}
		}

		// Hosts
		$this->tpl->assign('hosts', $hosts);
		$this->tpl->assign('forSatellite', false);
		$this->exportToFile('export/hosts.tpl', sprintf(
			"%s/hosts.cfg",
			$this->objectsRoot
		));
		Application::getLogInstance()->debug(sprintf(
			"%s: Exported hosts",
			__METHOD__
		));

		// Services
		$this->tpl->assign('hosts', $hosts);
		$this->tpl->assign('forSatellite', false);
		$this->exportToFile('export/services.tpl', sprintf(
			"%s/services.cfg",
			$this->objectsRoot
		));
		Application::getLogInstance()->debug(sprintf(
			"%s: Exported services (%s/services.cfg)",
			__METHOD__, $this->objectsRoot
		));

		// Commands
		$commands = datamapper\CommandMapper::getInstance()->getAll();
		$this->tpl->assign('commands', $commands);
		$this->exportToFile('export/commands.tpl', sprintf(
			"%s/commands.cfg",
			$this->objectsRoot
		));
		Application::getLogInstance()->debug(sprintf(
			"%s: Exported commands",
			__METHOD__
		));

		// Host groups
		$this->tpl->assign(
			'customers',
			datamapper\CustomerMapper::getInstance()->getAll(null)
		);
		$permissionName = $this->shouldCheckPermission ? model\Permission::HOSTS_READ : null;
		$this->tpl->assign(
			'groups',
			datamapper\HostGroupMapper::getInstance()->getAll($permissionName)
		);
		$this->exportToFile('export/hostGroups.tpl', sprintf(
			"%s/hostGroups.cfg",
			$this->objectsRoot
		));
		Application::getLogInstance()->debug(sprintf(
			"%s: Exported host groups",
			__METHOD__
		));
	}

	/**
	 * Build satellite nagios/icinga configuration files
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\Host $satellite
	 * @returns       void
	 */
	public function buildSatellite($satellite) {
		Application::getLogInstance()->debug(sprintf(
			"%s: Going to build satellite: %s",
			__METHOD__, $satellite->id
		));
		$permissionName = $this->shouldCheckPermission ? model\Permission::HOSTS_READ : null;
		$allHosts = datamapper\HostMapper::getInstance()->getAll(
			false,
			false,
			false,
			$satellite->id,
			null,
			$permissionName,
			true
		)->toArray();


		//only take those hosts that are not disabled by the customer
		$hosts = array();
		foreach($allHosts as $host) {
			if($host->isEnabled()) {
				$hosts[] = $host;
			}
		}

		if($satellite->isEnabled()) {
			$hosts[] = $satellite;
		}

		
		$satelliteDirectory = sprintf(
			"%s/htdocs/satellites/%d",
			ConfManager::getVar('af', 'webRoot', 'uri'),
			$satellite->id
		);

		if (!file_exists($satelliteDirectory)) {
			mkdir($satelliteDirectory, 0770);
		}

		$lockFile = sprintf(
			"%s/.lock",
			$satelliteDirectory
		);

		$fp = fopen($lockFile, 'w');
		if (flock($fp, LOCK_EX)) {
			// Hosts
			$this->tpl->assign('hosts', $hosts);
			$this->tpl->assign('forSatellite', true);
			$this->tpl->assign('permissionName', $permissionName);
			$this->exportToFile('export/hosts.tpl', sprintf(
				"%s/hosts.cfg",
				$satelliteDirectory
			));

			// Services
			$this->tpl->assign('hosts', $hosts);
			$this->tpl->assign('forSatellite', true);
			$this->exportToFile('export/services.tpl', sprintf(
				"%s/services.cfg",
				$satelliteDirectory
			));

			// Commands
			$commands = datamapper\CommandMapper::getInstance()->getAll($satellite->id);
			$this->tpl->assign('commands', $commands);
			$this->exportToFile('export/commands.tpl', sprintf(
				"%s/commands.cfg",
				$satelliteDirectory
			));

			// Cronjob
			$this->tpl->assign('domain', ConfManager::getVar('af', 'domain', 'default'));
			$this->tpl->assign('satellite', $satellite);
			$this->exportToFile('export/satelliteCronjob.tpl', sprintf(
				"%s/updateSatelliteConfig.sh",
				$satelliteDirectory
			));

			// Setup script
			$this->tpl->assign('IP', ConfManager::getVar('af', 'address', 'default'));
			$this->tpl->assign('wgetLink', sprintf(
				"https://monitoring:%s@%s/satellites/%d",
				$satellite->password,
				ConfManager::getVar('af', 'domain', 'default'),
				$satellite->id
			));
			$this->exportToFile('export/satelliteSetup.tpl', sprintf(
				"%s/setup.sh",
				$satelliteDirectory
			));

			// Nrpe script
			$this->tpl->assign('hosts', $hosts);
			$this->tpl->assign('domain', ConfManager::getVar('af', 'domain', 'default'));
			$this->exportToFile('export/nrpeSatelliteCronjob.tpl', sprintf(
				"%s/nrpeSatelliteCronjob.sh",
				$satelliteDirectory
			));

			// .htaccess
			$this->tpl->assign('path', $satelliteDirectory);
			$this->exportToFile('export/htaccess.tpl', sprintf(
				"%s/.htaccess",
				$satelliteDirectory
			));

			// .htpasswd
			$this->tpl->assign('users', array(
				array (
					'name'		=> 'monitoring',
					'password'	=> crypt($satellite->password, base64_encode($satellite->password))
				)
			));

			$this->exportToFile('export/htpasswd.tpl', sprintf(
				"%s/.htpasswd",
				$satelliteDirectory
			));

			$satellite->version++;
			if ($this->shouldCheckPermission) {
				$satellite->save();
			}
			else {
				$satellite->getMapper()->save($satellite);
			}

			// Version file
			system(sprintf(
				'echo %d > %s/version',
				$satellite->version,
				$satelliteDirectory
			));

			$tarballName = sprintf(
				'objects-v%d.tar.gz',
				$satellite->version
			);

			// Copy generic files
			system(sprintf(
				'cp %s/htdocs/satellites/.generic/* %s',
				ConfManager::getVar('af', 'webRoot', 'uri'),
				$satelliteDirectory
			));

			// Create tar ball
			system(sprintf(
				'cd %s && tar -czf %s *.cfg',
				$satelliteDirectory,
				$tarballName
			), $result);

			// Delete config files
			system(sprintf(
				'rm %s/*.cfg',
				$satelliteDirectory
			));

			$tarballSymlink = sprintf(
				'%s/objects-latest.tar.gz',
				$satelliteDirectory,
				$satellite->id
			);

			if (file_exists($tarballSymlink)) {
				unlink($tarballSymlink);
			}

			symlink(
				$tarballName,
				$tarballSymlink
			);

			flock($fp, LOCK_UN);
		}
		fclose($fp);
	}

	/**
	 * Build NRPE configuration files
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $hosts
	 * @returns       void
	 */
	public function buildNrpe($host) {
		$webserverGroup = ConfManager::getVar('af', 'webserver', 'group');
		$lockFile = sprintf(
			"%s/htdocs/nrpe/.lock",
			ConfManager::getVar('af', 'webRoot', 'uri')
		);

		$fp = fopen($lockFile, 'w');
		chgrp($lockFile, $webserverGroup);
		chmod($lockFile, 0664);
		if (flock($fp, LOCK_EX)) {
			$hostServiceInstances = $host->getHostServiceInstances(true);

			$nrpeDirectory = sprintf(
				"%s/htdocs/nrpe/%d",
				ConfManager::getVar('af', 'webRoot', 'uri'),
				$host->id
			);

			if (!file_exists($nrpeDirectory)) {
				mkdir($nrpeDirectory, 0770);
				chgrp($nrpeDirectory, $webserverGroup);
			}

			// This host does not have any services with nrpe assigned
			if (count($hostServiceInstances) == 0) {
				$filename = sprintf(
					"%s/nrpeHost.cfg",
					$nrpeDirectory
				);

				if (file_exists($filename)) {
					unlink($filename);
				}

				@touch($filename);
				chgrp($filename, $webserverGroup);

				$host->version++;
				if ($this->shouldCheckPermission) {
					$host->save();
				}
				else {
					$host->getMapper()->save($host);
				}

				// Write version file
				system(sprintf(
					'echo %d > %s/version',
					$host->version,
					$nrpeDirectory
				));

				return;
			}

			// Write nrpe configuration for current host
			$this->tpl->assign('hostServiceInstances', $hostServiceInstances);
			$this->exportToFile('export/nrpe.tpl', sprintf(
				"%s/nrpeHost.cfg",
				$nrpeDirectory
			), $webserverGroup);

			// .htaccess
			$this->tpl->assign('path', $nrpeDirectory);
			$this->exportToFile('export/htaccess.tpl', sprintf(
				"%s/.htaccess",
				$nrpeDirectory
			), $webserverGroup);

			// .htpasswd
			$this->tpl->assign('users', array(
				array (
					'name'		=> 'monitoring',
					'password'	=> crypt($host->password, base64_encode($host->password))
				)
			));
			$this->exportToFile('export/htpasswd.tpl', sprintf(
				"%s/.htpasswd",
				$nrpeDirectory
			), $webserverGroup);

			// version file
			$domain = $host->satelliteId == null ? ConfManager::getVar('af', 'domain', 'default') : $host->getSatellite()->IPv4;
			$this->tpl->assign('domain', $domain);
			$this->tpl->assign('host',   $host);
			$this->exportToFile('export/nrpeInstall.tpl', sprintf(
				"%s/nrpeInstall.sh",
				$nrpeDirectory
			), $webserverGroup);
			$this->tpl->assign('domain', $domain);
			$this->tpl->assign('host',   $host);
			$this->exportToFile('export/nrpeVariables.tpl', sprintf(
				"%s/nrpeVariables.sh",
				$nrpeDirectory
			), $webserverGroup);

			$this->exportToFile('export/nrpeCronjob.tpl', sprintf(
				"%s/updateNrpeConfig.sh",
				$nrpeDirectory
			), $webserverGroup);

			$host->version++;
			$host->exportDate = strftime('%Y-%m-%d');
			if ($this->shouldCheckPermission) {
				$host->save();
			}
			else {
				$host->getMapper()->save($host);
			}

			// Write version file
			system(sprintf(
				'echo %d > %s/version',
				$host->version,
				$nrpeDirectory
			));
			flock($fp, LOCK_UN);
		} // if
		else {
			throw new \RuntimeException(sprintf(
				"Could not lock lock-file %s",
				$lockFile
			));
		}
		fclose($fp);
	} // if
}
