<?php

namespace ch\monitoring\util;

use org\adfinis\framework\exceptions\UserException;

/**
 * Handles directory actions, such as creating, moving and removing
 * directories in a filesystem
 *
 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class DirectoryManager {

	private $dirRootPath;
	private $callback;

	/**
	 * Creates an instance of this class.
	 *
	 * Directory Root Path has to be set to where all directories
	 * should be created in.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string dirPath directory path
	 * @param         array function that is called in error case
	 * @return        void
	 */
	public function __construct($dirRootPath, $callback = null) {
		$this->dirRootPath = $dirRootPath .'/';
		$this->callback = $callback;
	}

	/**
	 * Creates a directory at specified directory path
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string dirPath directory path
	 * @return        void
	 */
	public function makeDir($dirPath) {
		if (!file_exists($dirPath)) {
			if (!@mkdir($dirPath, 0755, true)) {
				$this->handleError(
					tr(
						"Failed to create directory for " . $dirPath .
						"Please install the directory manually.\n"
					)
				);
			}
		}
		else if (!is_dir($dirPath)) {
			$this->handleError(
				tr(sprintf(
					"The Path '%s' already exists but is no directory.",
					$dirPath
				))
			);
		}
	}

	/**
	 * Moves a directory from old path to new path.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string oldDirPath current path to directory
	 * @param         string newDirPath new path to directory
	 * @return        void
	 */
	public function moveDir($oldDirPath, $newDirPath) {
		if (!rename($oldDirPath, $newDirPath)) {
			$this->handleError(
				tr("Failed to move directory or it does not exist.
					Please rename the directory manually at path \n"
					. $oldDirPath . " to \n" . $newDirPath)
				);
		}
	}

	/**
	 * Removes a directory and eventual subdirectories and files
	 *
	 * Recursive has to be set to true, if directory should be deleted
	 * with all its content, such as files.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string dirPath path to directory
	 * @return        void
	 */
	public function removeDir($dirPath, $recursive = true) {
		// There is nothing to delete, abort
		if (!file_exists($dirPath)) {
			return;
		}

		//scan dir, if not empty and not recursive -> return
		if (is_dir($dirPath)) {
			foreach(scandir($dirPath) as $item) {
				if (!$recursive) return;
				if ($item == '.' || $item == '..') {
					continue;
				}
				//delete file, if it is a file, else remove directory recursively
				if (is_file($dirPath . $item)) {
					unlink($dirPath . $item);
				}
				else {
					$this->removeDir($dirPath . $item);
				}
			}
		}
		if (!rmdir($dirPath)) {
			$this->handleError(
				tr(
					"Failed to delete directory because it does not exist or given path" .
					"is no directory. Please remove the directory manually at path:\n" .
					$dirPath
				)
			);
		}
	}

	/**
	 * Creates a directory path string with the dir root path as base.
	 *
	 * Takes all arguments and creates an inorder directory path
	 * with each string as one directory.
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string any number of strings as folders
	 * @return        string directory path
	 */
	public function createDirectoryPath() {
		$dirPath = $this->dirRootPath;
		foreach (\func_get_args() as $argument) {
			$dirPath = $dirPath . $argument . '/';
		}
		return $dirPath;
	}

	/**
	 * Calls callback function in case of an error
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @param         string message
	 */
	private function handleError($message) {
		if($this->callback) {
			call_user_func($this->callback, $message);
		}
	}
}
