<?php

namespace ch\monitoring\util;

use ch\monitoring\Application;

/**
 * This class uses PEAR Image_Graph to display stats on alerts.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Graph {

	/**
	 * Show alerts on a daily basis
	 *
	 * @const         string TYPE_DAY
	 */
	const TYPE_DAY = "day";

	/**
	 * Show alerts on a monthy basis
	 *
	 * @const         string TYPE_MONTH
	 */
	const TYPE_MONTH = "month";

	/**
	 * Prevent object creation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	private function __construct() {
	}

	/**
	 * Prevent object cloning
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	private function __clone() {
	}

	/**
	 * Draw alerts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $type Monthly or daily
	 * @param         int $customerId The customer the graph shall be shown for.
	 * @param         int $width The width of the resulting image.
	 * @param         int $height The height of the resulting image.
	 * @returns       void
	 */
	public function drawAlerts($type, $customerId, $width, $height) {
		switch ($type) {
			case self::TYPE_DAY:
				self::drawAlertsPerDay($customerId, $width, $height);
				break;
			case self::TYPE_MONTH:
				self::drawAlertsPerMonth($customerId, $width, $height);
				break;
			default:
				throw new \UnexpectedValueException(
					sprintf("Unknown type '%s'", $type)
				);
		}
	}

	/**
	 * Collects data for alerts per day
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @param         int $width
	 * @param         int $height
	 * @returns       void
	 */
	private static function drawAlertsPerDay($customerId, $width, $height) {
		$result = Application::getDbInstance()->prepare("
			SELECT
				COUNT(`Alert`.`id`) AS alerts,
				DATE_FORMAT(`Alert`.`datetimeOccured`, '%d.%m.') AS day
			FROM
				`Alert`
			WHERE
				`Alert`.`customerId` = :customerId
				AND
				`Alert`.`datetimeOccured` BETWEEN SUBDATE(NOW(), 15) AND NOW()
			GROUP BY
				DAY(`Alert`.`datetimeOccured`)
		");
		$result->execute(array('customerId' => $customerId));
		$points =  $result->fetchAll(\PDO::FETCH_ASSOC);

		$now = time();
		$date = strtotime("-15 days", $now);
		while ($date <= $now) {
			$data[date('d.m.', $date)] = 0;
			$date = strtotime("+1 day", $date);
		}

		foreach ($points as $point) {
			$data[$point['day']] = $point['alerts'];
		}

		self::draw($data, $width, $height);
	}

	/**
	 * Collect data for alerts per month
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $customerId
	 * @param         int $width
	 * @param         int $height
	 * @returns       void
	 */
	private static function drawAlertsPerMonth($customerId, $width, $height) {
		$result = Application::getDbInstance()->prepare("
			SELECT
				COUNT(`Alert`.`id`) AS alerts,
				MONTH(`Alert`.`datetimeOccured`) AS month
			FROM
				`Alert`
			WHERE
				`Alert`.`customerId` = :customerId
				AND
				YEAR(`Alert`.`datetimeOccured`) = YEAR(NOW())
			GROUP BY
				MONTH(`Alert`.`datetimeOccured`)"
		);
		$result->execute(array('customerId' => $customerId));
		$points =  $result->fetchAll(\PDO::FETCH_ASSOC);

		for ($i = 1; $i <= 12; $i++) {
			$data[$i] = 0;
		}

		foreach ($points as $point) {
			$data[$point['month']] = $point['alerts'];
		}

		self::draw($data, $width, $height);
	}

	/**
	 * Draw graph
	 *
	 * This function uses a data array which is generated by the drawAlertsPer* functions.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $data
	 * @param         int $width
	 * @param         int $height
	 * @returns       void
	 */
	private static function draw($data, $width, $height)
	{
		$Graph =& \Image_Graph::factory('graph', array($width, $height)); 
		$Font =& $Graph->addNew('font', 'Verdana');
		$Font->setSize(8);
		$Graph->setFont($Font);

		$Graph->add(
			\Image_Graph::vertical(
				\Image_Graph::factory('title', array('', 1)),
				\Image_Graph::vertical(
					$Plotarea = \Image_Graph::factory('plotarea'),
					$Legend = \Image_Graph::factory('plotarea'),
					100
				),
				5
			)
		);

		$Dataset =& \Image_Graph::factory('dataset');
		foreach ($data as $key => $value) {
			$Dataset->addPoint($key, $value);
		}

		$Plot =& $Plotarea->addNew('Image_Graph_Plot_Bar', array(&$Dataset));
		$Plot->setLineColor('gray');
		$Plot->setFillColor('blue@0.2');

		$Graph->done();
	}
}
