<?php

namespace ch\monitoring\util;

use ch\monitoring\Application;
use ch\monitoring\model;
use ch\monitoring\datamapper;
use org\adfinis\framework\config\ConfManager;

/**
 * Manipulate icinga web database
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class IcingaWeb {

	/**
	 * Target 'IcingaHostgroup'
	 *
	 * @const         int TARGET_ICINGA_HOSTGROUP
	 */
	const TARGET_ICINGA_HOSTGROUP = 1;

	/**
	 * Target 'icinga.user'
	 *
	 * @const         int TARGET_ICINGA_USER
	 */
	const TARGET_ICINGA_USER = 8;

	/**
	 * Target 'appkit.admin.groups'
	 *
	 * @const         int TARGET_APPKIT_ADMIN_GROUPS
	 */
	const TARGET_APPKIT_ADMIN_GROUPS = 9;

	/**
	 * Target 'appkit.admin.users'
	 *
	 * @const         int TARGET_APPKIT_ADMIN_USERS
	 */
	const TARGET_APPKIT_ADMIN_USERS = 10;

	/**
	 * Target 'appkit.admin'
	 *
	 * @const         int TARGET_APPKIT_ADMIN
	 */
	const TARGET_APPKIT_ADMIN = 11;

	/**
	 * Target 'icinga.cronk.category.admin'
	 *
	 * @const         int TARGET_ICINGA_CRONK_CATEGORY_ADMIN
	 */
	const TARGET_ICINGA_CRONK_CATEGORY_ADMIN = 15;

	/**
	 * Target 'icinga.cronk.log'
	 *
	 * @const         int TARGET_ICINGA_CRONK_LOG
	 */
	const TARGET_ICINGA_CRONK_LOG = 16;

	/**
	 * Target 'icinga.control.view'
	 *
	 * @const         int TARGET_ICINGA_CONTROL_VIEW
	 */
	const TARGET_ICINGA_CONTROL_VIEW = 17;

	/**
	 * Target 'icinga.control.admin'
	 *
	 * @const         int TARGET_ICINGA_CONTROL_ADMIN
	 */
	const TARGET_ICINGA_CONTROL_ADMIN = 18;

	/**
	 * The user information
	 *
	 * @var           array $user
	 */
	private $user;

	/**
	 * The user password
	 *
	 * @var           string $password
	 */
	private $password;

	/**
	 * Object constuctor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $user
	 * @param         string $password
	 */
	public function __construct($user, $password) {
		$this->user     = $user;
		$this->password = $password;
	}

	/**
	 * Synchronizes a user with icinga web
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \UnexpectedValueException
	 */
	public function synchronize() {
		// abort sync due to it being broken with icingaweb2
		// TODO: refactor this to completely remove the code
		// most likely icingaweb2 is more than happy with just grabbing users
		// from apache using REMOTE_USER. Given that apache won't let them i
		// without a proper login this is probably the way to go forward.
		// If need be we can implement a way to remove old users from the
		// icingaweb2 database if it's every needed. As far as I can tell
		// the new icingaweb2 doesn't strictly need the database for anything
		// with business relevance.
		return;
		if ($this->user === null) {
			throw new \UnexpectedValueException('No user or no password set');
		}

		$allowLdap  = ConfManager::getVar('af', 'auth', 'ldap')  == 'yes';
		$authenticationMethod = $this->user->authenticationMethod;

		if ($allowLdap && $this->user->hasLDAPAuthentication()) {
			$authSource = ConfManager::getVar('af', 'icingaWeb', 'authSource');
		}
		else if ($authenticationMethod === model\User::AUTH_METHOD_LOCAL) {
			$authSource = 'internal';
		}
		else {
			throw new \UnexpectedValueException(
				'Current user has an unknown authentication method set'
			);
		}

		if (!$this->user->hasPermission(model\Permission::MONITORING_WEB)) {
			Application::getDbInstance('icingaWeb')->delete(
				'nsm_user',
				array(
					'`user_name` = ?'    => $this->user->name,
					'`user_authsrc` = ?' => $authSource
				)
			);

			$errorMessage = sprintf(
				"User '%s' has no rights to access MONITORING_WEB, right was removed from user",
				$this->user->name
			);
			Application::getLogInstance()->warn($errorMessage);
			error_log($errorMessage);

			return;
		}

		if ($this->password !== null) {
			$algorithm = ConfManager::getVar('af', 'icingaWeb', 'algorithm');

			$salt = hash(
				$algorithm,
				uniqid(sprintf(
					"%s_%d",
					$this->user->name,
					mt_rand()
				))
			);

			$password  = hash_hmac(
				$algorithm,
				$this->password,
				$salt
			);
		}
		else {
			$salt     = '';
			$password = '';
		}

		if ($allowLdap && $this->user->hasLDAPAuthentication()) {
			// Get user dn from ldap
			$attributes = array("dn");
			$ldap = new Ldap();
			$ldapUser = $ldap->getUserByName($this->user->name, $attributes);
			$authId = $ldapUser[0]['dn'];
		}
		else {
			$authSource = "internal";
			$authId = null;
		}

		$statement = Application::getDbInstance('icingaWeb')->prepare("
			INSERT INTO
				`nsm_user`
			SET
				`user_name`      = :username,
				`user_lastname`  = :fullname,
				`user_firstname` = '',
				`user_authsrc`   = :authSource,
				`user_authid`    = :authId,
				`user_password`  = :password,
				`user_salt`      = :salt,
				`user_email`     = :email,
				`user_disabled`  = '0',
				`user_created`   = NOW()
			ON DUPLICATE KEY UPDATE
				`user_name`      = :username,
				`user_lastname`  = :fullname,
				`user_authsrc`   = :authSource,
				`user_authid`    = :authId,
				`user_password`  = :password,
				`user_salt`      = :salt,
				`user_email`     = :email,
				`user_modified`  = NOW()
		");
		$statement->execute(array(
			'username'   => $this->user->name,
			'fullname'   => $this->user->fullName,
			'authSource' => $authSource,
			'authId'     => $authId,
			'password'   => $password,
			'salt'       => $salt,
			'email'      => $this->user->email ?: 'placeholder@placeholder.nx'
		));

		$isPriviledged = $this->user->hasPermission(
			model\Permission::MONITORING_MASTER
		);

		$icingaWebUser = $this->getUser(
			$this->user->name,
			$authSource
		);

		$customers = null;
		if (!$isPriviledged) {
			$customers = datamapper\CustomerMapper::getInstance()->getAll(
				model\Permission::MONITORING_WEB
			);

			if (count($customers) === 0) {
				$userId = $icingaWebUser['user_id'];
				$principalId = $this->getPrincipalId($userId);
				$this->removePermissions($userId, $principalId);
				return;
			}
		}

		$this->setPermissions(
			$icingaWebUser['user_id'],
			$customers,
			$isPriviledged
		);

		$message = sprintf(
			"Successfully synced user '%s' (%s, %s) with IcingaWeb",
			$this->user->name,
			$authenticationMethod,
			$authSource
		);
		Application::getLogInstance()->debug($message);
	}

	/**
	 * Returns a user from the icinga-web database
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        
	 * @param         string $username
	 * @param         string $authSource
	 * @returns       array
	 */
	public function getUser($username, $authSource) {
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			SELECT
				`user_id`
			FROM
				`nsm_user`
			WHERE
				`user_name` = :username
				AND
				`user_authsrc` = :authSource
		");

		$statement->execute(array(
			'username'   => $username,
			'authSource' => $authSource
		));

		return $statement->fetch();
	}

	/**
	 * Retuns the principal for the given user.
	 *
	 * If the user has already a principal, that principal is returned,
	 * otherwise a new principal is getting created.
	 *
	 * @param         int $userId      Icinga web user ID
	 * @returns       int $principalId The ID of the principal
	 */
	public function getPrincipalId($userId) {
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			SELECT
				`principal_id`
			FROM
				`nsm_principal`
			WHERE
				`nsm_principal`.`principal_user_id` = :userId
		");
		$statement->execute(array(
			'userId'   => $userId
		));
		$principal = $statement->fetch();
		if (empty($principal)) {
			Application::getDbInstance('icingaWeb')->beginTransaction();
			$statement = Application::getDbInstance('icingaWeb')->prepare("
				INSERT INTO
					`nsm_principal`
				SET
					`principal_user_id` = :userId,
					`principal_type` = 'user',
					`principal_disabled` = 0
			");
			$statement->execute(array(
				'userId' => $userId
			));
			Application::getDbInstance('icingaWeb')->commit();
			$statement = Application::getDbInstance('icingaWeb')->prepare("
				SELECT
				`principal_id`
				FROM
				`nsm_principal`
				WHERE
				`nsm_principal`.`principal_user_id` = :userId
				");
			$statement->execute(array(
				'userId'   => $userId
			));
			$principal = $statement->fetch();
		}
		$principalId = $principal['principal_id'];
		return $principalId;
	}

	/**
	 * Restricts access of a user to the customer hostgroup
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $userId Icinga web user id
	 * @param         DBIterator $customers
	 * @param         bool $isPriviledged
	 * @throws        \UnexpectedValueException
	 * @returns       void
	 */
	private function setPermissions($userId, $customers, $isPriviledged) {
		$principalId = $this->getPrincipalId($userId);
		$this->removePermissions($userId, $principalId);

		if (!$isPriviledged && empty($customers)) {
			Application::getLogInstance()->warn('Customer list is empty');
			throw new \UnexpectedValueException(
				tr("Customer list is empty")
			);
		}
        $this->insertRole($userId, $isPriviledged);
		if (!$isPriviledged) {
			foreach ($customers as $customer) {
				$principalTargetId = $this->createPrincipalTargetId($principalId);
				Application::getLogInstance()->debug(sprintf(
					"User `%d` is missing the role `MONITORING_MASTER`, " .
					"setting `hostgroup` for customer `%s` with " .
					"principal target `%d`",
					$userId, $customer->name, $principalTargetId
				));
				$this->addTargetValue(
					$principalTargetId,
					'hostgroup',
					$customer->name
				);
			}
		}
		Application::getDbInstance('icingaWeb')->commit();
	}

	/**
	 * Remove permissions for a user
	 *
	 * @param    int   $userId      the identifier of the user
	 * @param    int   $principalId the identifier of the users principal
	 * @returns  void
	 */
	private function removePermissions($userId, $principalId) {
		$this->deleteRole($userId);
		$this->deletePrincipalTargets($principalId);
		$this->deleteTargetValues($userId);
	}

	/**
	 * Adds the role based upon privilieges for given user.
	 *
	 * @param   int  $userId the identifier of the user
	 * @param   bool $isPriviledged whether the user is admin or user
	 * @returns void
	 */
    private function insertRole($userId, $isPriviledged) {
		Application::getDbInstance('icingaWeb')->beginTransaction();
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			INSERT INTO
				`nsm_user_role`
			SET
				`usro_user_id` = :userId,
				`usro_role_id` = :roleId
		");
		$roleId = $this->getRoleIdByName(
			$isPriviledged
				? ConfManager::getVar('af', 'icingaWeb', 'priviledgedGroup')
				: ConfManager::getVar('af', 'icingaWeb', 'restrictedGroup'),
			true,
			false
		);
		$statement->execute(array(
			'userId' => $userId,
			'roleId' => $roleId
		));
		Application::getLogInstance()->debug(
			sprintf("Added role %d to user %d", $roleId, $userId)
		);
	}

	/**
	 * Removes all roles from given user.
	 *
	 * @param   int  $userId the identifier of the user
	 * @returns void
	 */
	private function deleteRole($userId) {
		Application::getDbInstance('icingaWeb')->beginTransaction();
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			DELETE FROM
				`nsm_user_role`
			WHERE
				`usro_user_id` = :userId
		");
		$statement->execute(array('userId' => $userId));
		Application::getDbInstance('icingaWeb')->commit();
	}

	/**
	 * Deletes all targets for given principal.
	 *
	 * @param   int $principalId the identifier of the principal
	 * @returns void
	 */
	private function deletePrincipalTargets($principalId) {
		Application::getDbInstance('icingaWeb')->beginTransaction();
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			DELETE FROM
				`nsm_principal_target`
			WHERE
				`pt_principal_id` = :principalId
		");
		$statement->execute(array('principalId' => $principalId));
		Application::getDbInstance('icingaWeb')->commit();
	}

	/**
	 * Deletes the target values for the given user.
	 *
	 * @param   int  $userId the identifier of the user
	 * @returns void
	 */
	private function deleteTargetValues($userId) {
		Application::getDbInstance('icingaWeb')->beginTransaction();
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			DELETE FROM
				`nsm_target_value`
			USING
				`nsm_target_value`
			JOIN `nsm_principal_target` ON (
				`nsm_principal_target`.`pt_id` = `nsm_target_value`.`tv_pt_id`
				AND
				`nsm_principal_target`.`pt_target_id` = :targetId
			)
			JOIN `nsm_principal` ON (
				`nsm_principal`.`principal_id` = `nsm_principal_target`.`pt_principal_id`
			)
			WHERE
				`nsm_principal`.`principal_user_id` = :userId
		");
		$statement->execute(array(
			'targetId' => self::TARGET_ICINGA_HOSTGROUP,
			'userId'   => $userId
		));
		Application::getDbInstance('icingaWeb')->commit();
	}

	/**
	 * Get role id by name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $roleName
	 * @param         bool $isPriviledged
	 * @param         bool $create Whether to create a new role if it does not exist
	 * @throws        \UnexpectedValueException
	 * @returns       int
	 */
	private function getRoleIdByName($roleName, $isPriviledged, $create = false) {
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			SELECT
				`role_id`
			FROM
				`nsm_role`
			WHERE
				`role_name` LIKE :name
		");
		$statement->execute(array('name' => $roleName));
		$role = $statement->fetch();

		if (empty($role)) {
			throw new \UnexpectedValueException('Customer role does not exist');
		}
		return $role['role_id'];
	}

	/**
	 * Adds a target value related to a principal
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $principalId
	 * @param         string $key
	 * @param         string $value
	 * @returns       void
	 */
	private function addTargetValue($principalTargetId, $key, $value) {
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			INSERT INTO
				`nsm_target_value`
			SET
				`tv_pt_id` = :principalTargetId,
				`tv_key`   = :key,
				`tv_val`   = :value
		");

		$statement->execute(array(
			'principalTargetId' => $principalTargetId,
			'key'               => $key,
			'value'             => $value
		));
	}

	/**
	 * Creates a principal target for the given principal.
	 *
	 * @param         int $principalId the identifier of the principal
	 * @returns       int              the identifier of the target
	 */
	private function createPrincipalTargetId($principalId) {
		$statement = Application::getDbInstance('icingaWeb')->prepare("
			INSERT INTO
				`nsm_principal_target`
			SET
				`pt_principal_id` = :principalId,
				`pt_target_id` = :targetId
		");
		$statement->execute(array(
			'targetId' => self::TARGET_ICINGA_HOSTGROUP,
			'principalId' => $principalId
		));
		return Application::getDbInstance('icingaWeb')->lastInsertId();
	}

	/**
	 * Update customer name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \UnexpectedValueException
	 * @param         int $customerId
	 * @param         string $oldName
	 * @returns       void
	 */
	public function updateCustomer($newName, $oldName) {
		if ($newName === null || $oldName === null) {
			throw new \UnexpectedValueException('New and old hostgroup name must be provided');
		}

		$statement = Application::getDbInstance('icingaWeb')->prepare("
			UPDATE
				`nsm_target_value`
			SET
				`tv_val` = :newName
			WHERE
				`tv_key` = 'hostgroup'
				AND
				`tv_val` = :oldName
		 ");
		$statement->execute(array(
			'newName' => $newName,
			'oldName' => $oldName
		 ));
	}
}
