<?php

namespace ch\monitoring\util;

use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\Application;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;

/**
 * Do ldap stuff
 *
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Ldap {
	/**
	 * Allow all LDAP users
	 * @const           string AUTH_METHOD_ALL
	 */
	const AUTH_METHOD_ALL = 'all';

	/**
	 * Allow only LDAP users within defined group
	 * @const           string AUTH_METHOD_GROUP
	 */
	const AUTH_METHOD_GROUP = 'group';

	/**
	 * ldap identifier instance
	 * @var          int  $ldap
	 */
	public $ldap = null;

	/**
	 * Attributes to sync when syncing data from LDAP
	 * @var           array $syncedAttributes
	 */
	public static $syncedAttributes = array(
		'uid' => array(
			'label'   => 'Username',
			'id'      => 'uid',
			'mapping' => 'name',
		),
		'cn' => array(
			'label'   => 'Full name',
			'id'      => 'cn',
			'mapping' => 'fullName',
		),
		'mail' => array(
			'label'   => 'E-Mail',
			'id'      => 'mail',
			'mapping' => 'email',
		),
		'mobile' => array(
			'label'   => 'SMS',
			'id'      => 'mobile',
			'mapping' => 'sms',
		),
		'telephoneNumber' => array(
			'label'   => 'Phone',
			'id'      => 'mobile',
			'mapping' => 'phone',
		),
		'jabber' => array(
			'label'   => 'Jabber',
			'id'      => 'jabber',
			'mapping' => 'jabber',
		),
	);

	/**
	 * Object constuctor
	 *
	 * @throws        \RuntimeException
	 */
	public function __construct(){
		$this->ldap = ldap_connect(
			ConfManager::getVar('af', 'ldap', 'default', 'server')
		);
		if (!$this->ldap) {
			throw new \RuntimeException(
				tr('Could not create an ldap instance')
			);
		}
		Application::getLogInstance()->debug(sprintf(
			'Successfully connected to LDAP-server: %s',
			ConfManager::getVar('af', 'ldap', 'default', 'server')
		));

		ldap_set_option($this->ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
		Application::getLogInstance()->debug(sprintf(
			'Set LDAP-option %s to %d',
			LDAP_OPT_PROTOCOL_VERSION, 3
		));

		$username = ConfManager::getVar('af', 'ldap', 'default', 'accessDN');
		$password = ConfManager::getVar('af', 'ldap', 'default', 'password');

		// Anonymous bind if no username given
		if (empty($username)) {
			if (!@ldap_bind($this->ldap)) {
				throw new \RuntimeException(
					tr('LDAP authentication failed, could not bind anonymously')
				);
			}
		}
		else if (!@ldap_bind($this->ldap, $username, $password)) {
			throw new \RuntimeException(sprintf(
				tr('LDAP authentication failed, could not bind with user %s'),
				$username
			));
		}
	}

	/**
	 * Authenticate a username and password pair
	 *
	 * @param         string $username
	 * @param         string $password
	 * @throws        \UnexpectedValueException
	 * @returns       array
	 */
	public function authenticate($username, $password) {
		try {
			$user = $this->getUserByName($username);
		}
		catch (\UnexpectedValueException $e) {
			throw $e;
		}
		$user = $this->getUserDataByObject($user);
		if (!@ldap_bind($this->ldap, $user['username'], $password)) {
			throw new \UnexpectedValueException(
				tr('Password wrong')
			);
		}

		$authMethod = ConfManager::getVar('af', 'ldap', 'default', 'authMethod');
		if ($authMethod === self::AUTH_METHOD_ALL) {
			// Do nothing
		}
		elseif ($authMethod === self::AUTH_METHOD_GROUP) {
			$authGroupTemplate = ConfManager::getVar('af', 'ldap', 'default', 'authGroup');
			$groupDNs = ConfManager::getVar('af', 'ldap', 'default', 'groupDN');
			if (!is_array($groupDNs)) {
				$groupDNs = array($groupDNs);
			}
			$baseDN = ConfManager::getVar('af', 'ldap', 'default', 'baseDN');
			$isInGroups = false;
			foreach ($groupDNs as $groupDN) {
				$authGroup = sprintf("cn=%s,%s,%s", $authGroupTemplate, $groupDN, $baseDN);
				Application::getLogInstance()->debug(sprintf(
					'Checking if user is in auth-group %s', $authGroup
				));
				if ($this->isGroupInGroups($authGroup, $user['groups'])) {
					$isInGroups = true;
				}
			}
			if (!$isInGroups) {
				throw new \UnexpectedValueException(
					sprintf(
						"%s: %s",
						tr("User is not within auth group(s)"),
						$authGroupTemplate
					)
				);
			}
		}
		else {
			throw new \UnexpectedValueException(
				sprintf(
					"%s: %s %s",
					tr('Unknown authentication method'),
					$authMethod,
					sprintf(
						tr('use either %s or %s'),
						self::AUTH_METHOD_ALL,
						self::AUTH_METHOD_GROUP
					)
				)
			);
		}
		return $user;
	}

	/**
	 * Returns a user from ldap
	 *
	 * @param         string $username
	 * @param         array $attributes
	 * @throws        \UnexpectedValueException
	 * @returns       array
	 */
	public function getUserByName($username, $attributes = null) {
		$filter = sprintf(
			"(uid=%s)",
			$username
		);
		if (is_null($attributes)) {
			$attributes = array_keys(self::$syncedAttributes);
		}
		$userDNs = ConfManager::getVar('af', 'ldap', 'default', 'userDN');
		if (!is_array($userDNs)) {
			$userDNs = array($userDNs);
		}
		return $this->ldapSearch(
			$filter,
			$attributes,
			$userDNs
		);
	}

	/**
	 * Returns a user from given ldap user object
	 *
	 * @param         array $ldapUser
	 * @throws        \UnexpectedValueException
	 * @return        array
	 */
	public function getUserDataByObject($ldapUser) {
		Application::getLogInstance()->debug(sprintf(
			'LDAP user  %s', var_export($ldapUser, true)
		));
		if ($ldapUser['count'] == 0) {
			throw new \UnexpectedValueException(
				tr('User does not exist')
			);
		}
		$user = $ldapUser[0];
		if (!isset($user["mail"][0])) {
			$user["mail"] = array('');
		}
		// Unfortunately ldap_search doesn't return fields that are not set in ldap
		if (!isset($user["mobile"][0])) {
			$user["mobile"] = array('');
		}
		if (!isset($user["jabber"][0])) {
			$user["jabber"] = array('');
		}
		// Get all the groups this user belongs to (excluding primary group)
		$groups = $this->getUserGroups(
			$user["uid"][0],
			array("cn")
		);
		if (count($groups) === 0) {
			throw new \UnexpectedValueException(
				tr('This user is not in any ldap groups')
			);
		}
		// We iterate through the array later so we don't want the count.
		unset($groups['count']);
		return array(
			'username' => $user["dn"],
			'name'     => $user["uid"][0],
			'fullName' => $user["cn"][0],
			'email'    => $user["mail"][0],
			'jabber'   => $user["jabber"][0],
			'phone'    => str_replace('+', '00', $user["mobile"][0]),
			'sms'      => str_replace('+', '00', $user["mobile"][0]),
			'groups'   => $groups
		);
	}


	/**
	 * Get the groups a user is a member of
	 *
	 * @param         int $username
	 * @param         array $attributes
	 * @returns       array
	 */
	public function getUserGroups($username, $attributes) {
		$filter = sprintf(
			"(&(objectClass=posixGroup)(memberUid=%s))",
			$username
		);
		$groupDNs = ConfManager::getVar('af', 'ldap', 'default', 'groupDN');
		if (!is_array($groupDNs)) {
			$groupDNs = array($groupDNs);
		}

		return $this->ldapSearch(
			$filter,
			$attributes,
			$groupDNs
		);
	}

	/**
	 * Returns all groups a given customer is allowed to see
	 *
	 * @param         string $customerName Name of the customer
	 * @param         array  $attributes   Foo
	 * @returns       array                Allowed groups
	 */
	public function getCustomerGroups($customerName, $attributes) {
		$subDNs = ConfManager::getVar('af', 'ldap', 'default', 'groupDN');
		if (!is_array($subDNs)) {
			$subDNs = array($subDNs);
		}
		if ($customerName) {
			$filter = sprintf(
				"(&(objectClass=posixGroup)(%s))",
				$customerName
			);
		}
		else {
			$filter = "(&(objectClass=posixGroup))";
		}
		return $this->ldapSearch(
			$filter, $attributes, $subDNs
		);
	}

	/**
	 * Returns search results of a ldap query
	 *
	 * @param         string $filter
	 * @param         array $attributes
	 * @param         array $subDNs To be prepended to the base dn
	 * @returns       array
	 */
	private function ldapSearch($filter, $attributes, $subDNs) {
		$objects = array();
		foreach ($subDNs as $subDN) {
			$dn = sprintf(
				"%s,%s",
				$subDN, ConfManager::getVar('af', 'ldap', 'default', 'baseDN')
			);
			Application::getLogInstance()->debug(sprintf(
				'Searching for %s at DN %s', $filter, $dn
			));
			$result = @ldap_search($this->ldap, $dn, $filter, $attributes);
			$entries = @ldap_get_entries($this->ldap, $result);
			if (!empty($entries)) {
				if (!empty($objects)) {
					$count = $objects['count'] + $entries['count'];
				}
				else {
					$count = $entries['count'];
				}
				$objects = array_merge($objects, $entries);
				$objects['count'] = $count;
			}
		}
		if (empty($objects)) {
			Application::getLogInstance()->warn(sprintf(
				'Search for %s at DN %s returned no objects', $filter, $dn
			));
			throw new \UnexpectedValueException(tr('No entries found'));
		}
		Application::getLogInstance()->debug(sprintf(
			'Search for %s at DN %s returned %s objects', $filter, $dn, count($objects)
		));
		return $objects;
	}

	/**
	 * Synchronizes a user from ldap
	 *
	 * @param         array $ldapUser  Array holding user data from LDAP
	 * @param         model\User $user Optional. An user object to fill data into
	 * @param         array $attributesToSync Optional. Which attributes shall
	 *                                        be synced.
	 *                                        Has to be an associative array in
	 *                                        the form of:
	 *                                        'fieldname' => array(
	 *                                            'mapping' => 'fieldname'
	 *                                        )
	 * @throws        \RuntimeException  when no LDAP user was given
	 * @returns       model\User
	 */
	public function synchronize($ldapUser, $user = null, $attributesToSync = null) {
		$saveDefaultRole = false;
		if ($ldapUser == null) {
			throw new \RuntimeException(
				tr('Preceding LDAP authentication required')
			);
		}

		if (!$user) {
			try {
				$user = datamapper\UserMapper::getInstance()->getByName(
					$ldapUser['userName'],
					array(
						model\User::AUTH_METHOD_LDAP,
						model\User::AUTH_METHOD_LDAP_SYNC
					)
				)->allowRead();

				if (!$user->hasLDAPAuthentication()) {
					throw new \RuntimeException(
						tr('This user already exists locally. Please contact an administrator.')
					);
				}
			}
			catch (\OutOfBoundsException $e) {
				$user = new model\User();
				$user->allowRead();
			}
		}

		if (!$user->hasIdentifier()) {
			$mapper = datamapper\CustomerMapper::getInstance();
			$customers = array();
			foreach ($ldapUser['groups'] as $ldapGroup) {
				$ldap_customers = $mapper->getByLDAPGroupName($ldapGroup['dn']);
				foreach ($ldap_customers as $customer) {
					$customers[$customer->id] = $customer;
				}
			}
			if (count($customers) == 0) {
				throw new \RuntimeException(tr(
					'User could not be synchronized with ldap because no ldap groups '
					. 'correspond to a customer. Please contact an administrator.'
				));
			}
			else if (count($customers) == 1) {
				$user->customerId = current($customers)->id;
				$saveDefaultRole = true;
			}
			else {
				// TODO: Ask user to select
				$user->customerId = current($customers)->id;
				$saveDefaultRole = true;
			}
		}
		if (is_null($attributesToSync)) {
			$attributesToSync = self::$syncedAttributes;
			$user->authenticationMethod = model\User::AUTH_METHOD_LDAP;
		}
		foreach ($attributesToSync as $attribute) {
			$ldapValue = $ldapUser[$attribute['mapping']];
			if (!empty($ldapValue)) {
				Application::getLogInstance()->debug(sprintf(
					'Changing attribute `%s` from `%s` to `%s`',
					$attribute['mapping'], $user->{$attribute['mapping']}, $ldapValue
				));
				$user->{$attribute['mapping']} = $ldapValue;
			}
		}
		$user->isEnabled = true;
		$identity = $user->getMapper()->save($user);
		if ($saveDefaultRole) {
			foreach ($customers as $customer) {
				datamapper\UserMapper::getInstance()->setDefaultRole(
					$user, $customer
				);
			}
		}
		return $user;
	}

	/**
	 * Returns the (LDAP) attributes of a user which differ from the local ones
	 *
	 * @returns       array          (LDAP) attributes which differ from local
	 *                               ones
	 */
	public function getChangedAttributes($localUser) {
		$changedAttributes = array();
		$ldapAttributes = self::$syncedAttributes;
		$ldapValues = $this->getUserByName(
			$localUser->name,
			array_keys($ldapAttributes)
		);
		foreach ($ldapAttributes as $attribute) {
			if (array_key_exists($attribute['id'], $ldapValues[0])) {
				$ldapValue = $ldapValues[0][$attribute['id']][0];
				$localValue = $localUser->{$attribute['mapping']};
				if ($localValue !== $ldapValue) {
					$attribute['value'] = $ldapValue;
					array_push($changedAttributes, $attribute);
				}
			}
		}
		return $changedAttributes;
	}

	/**
	 * Checks if the given group is within the given array of LDAP groups 
	 *
	 * @param         array $needle   Group to be found
	 * @param         array $groups  LDAP groups to search within
	 * @returns       bool           true if the groups array contains the given
	 *                               group, otherwise false
	 */
	private function isGroupInGroups($needle, $groups) {
		foreach ($groups as $group) {
			if ($group['dn'] === $needle) {
				return true;
			}
		}
		return false;
	}
}
