<?php

namespace ch\monitoring\util;

use ch\monitoring\Application;
use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\exceptions\UserException;

/**
 * Send commands to nagios or icinga (monitoring)
 *
 * Uses the named pipe, normally called nagios.cmd or icinga.cmd
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class Monitoring {
	private static $socket;
	private static $history;

	/**
	 * Prevent object creation
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	private function __construct() {
	}

	/**
	 * Prevent object cloning
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	private function __clone() {
	}

	/**
	 * Send a command to nagios or icinga (external commands file)
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         array $command
	 * @returns       void
	 */
	public static function sendCommand($command) {
		$file = ConfManager::getVar('af', 'monitoring', 'cmd');

		if (file_exists($file)) {
			$fh = fopen($file, 'r+');
			if (!$fh) {
				$msg = sprintf("There was an error opening the Icinga cmd file '%s'.", $file);
				Application::getLogInstance()->err($msg);
				error_log($msg);

				throw new UserException(tr($msg));
			}

			if (!fwrite($fh, sprintf(
				"[%d] %s\n",
				time(),
				implode(";", $command)
			))) {
				$msg = sprintf("There was an error writing to the Icinga cmd file '%s'.", $file);
				Application::getLogInstance()->err($msg);
				error_log($msg);
				throw new UserException(tr($msg));
			}

			if (!fclose($fh)) {
				$msg = sprintf("There was an error closing the Icinga cmd file '%s'.", $file);
				Application::getLogInstance()->err($msg);
				error_log($msg);
				throw new UserException(tr($msg));
			}
		}
		else {
				$msg = sprintf("The Icinga cmd file '%s' does not exist.", $file);
				Application::getLogInstance()->err($msg);
				error_log($msg);
				throw new UserException(tr($msg));
		}
	}

	/**
	 * Start monitoring
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public static function start() {
		exec(sprintf(
			"sudo %s",
			ConfManager::getVar('af', 'monitoring', 'start')
		), $dummy, $result);

		return $result == 0;
	}

	/**
	 * Stop monitoring
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public static function stop() {
		exec(sprintf(
			"sudo %s",
			ConfManager::getVar('af', 'monitoring', 'stop')
		), $dummy, $result);

		return $result == 0;
	}

	/**
	 * Monitoring status
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public static function status() {
		exec(sprintf(
			"sudo %s",
			ConfManager::getVar('af', 'monitoring', 'status')
		), $dummy, $result);

		return $result == 0;
	}

	/**
	 * Reload monitoring configuration
	 *
	 * @returns       array          Result and output
	 */
	public static function reload() {
		$command = sprintf(
			"sudo %s",
			ConfManager::getVar('af', 'monitoring', 'reload')
		);
		exec($command, $stdout, $result);
		Application::getLogInstance()->debug(sprintf(
			'%s: Executing command to reload configuration: %s (result: %s)',
			__METHOD__, $command, $result
		));

		return array($result === 0, $stdout);
	}

	/**
	 * Verify monitoring configuration
	 *
	 * @returns       array          Result and output
	 */
	public static function verify($verifyCheckOnly=false) {
		if ($verifyCheckOnly) {
			$configPath = ConfManager::getVar('af', 'monitoring', 'tempConfig');
		}
		else {
			$configPath = ConfManager::getVar('af', 'monitoring', 'config');
		}
		$command = sprintf(
			"sudo %s -v %s",
			ConfManager::getVar('af', 'monitoring', 'binary'),
			$configPath
		);
		exec($command, $stdout, $result);
		Application::getLogInstance()->debug(sprintf(
			'%s: Executing command to verify configuration: %s (result: %s)',
			__METHOD__, $command, $result
		));

		return array($result === 0, $stdout);
	}
}
