<?php

namespace ch\monitoring\util;

use org\adfinis\framework\config\ConfManager;
use ch\monitoring\model;
use ch\monitoring\datamapper;

/**
 * Parse Nagios or Icinga configuration files
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
class NagiosParser {
	// These variables hold the objects found
	private $hosts = array();
	private $hostgroups = array();
	private $services = array();
	private $commands = array();

	private $currentObject = null;

	/**
	 * Cache for included hosts
	 *
	 * @var           array $includedHostsCache
	 */
	private static $includedHostsCache = array();

	/**
	 * Cache for included services
	 *
	 * @var           array $includedServicesCache
	 */
	private static $includedServicesCache = array();

	/**
	 * Cache for included commands
	 *
	 * @var           array $includedCommandsCache
	 */
	private static $includedCommandsCache = array();

	/**
	 * Cache for included host groups
	 *
	 * @var           array $includedHostgroupsCache
	 */
	private static $includedHostgroupsCache = array();

	/**
	 * Lookup table for host groups
	 *
	 * @var           array $hostgroupTable
	 */
	private static $hostgroupTable = array();

	/**
	 * Constructor
	 *
	 * @param         string $file Path to the configuration file to be parsed
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 */
	public function __construct($file) {
		$this->parseFile($file);
	}

	/**
	 * Parse a given file
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $file
	 * @throws        RuntimeException
	 * @returns       void
	 */
	private function parseFile($file) {
		$fileHandler = fopen($file, 'r');

		if ($fileHandler === FALSE) {
			throw new \RuntimeException(sprintf(
				tr('Could not open configuration file %s'),
				$file
			));
		}

		$lineNumber = 0;
		while ($line = fgets($fileHandler)) {
			++$lineNumber;

			// Empty lines or comment lines must be ignored
			if (preg_match('/^\s*([#;](.*)?)?$/', $line)) {
				continue;
			}

			try {
				// Match the beginning of a configuration block
				if (preg_match('/^\s*define\s+(\S+)\s*\{\s*$/', $line, $matches)) {
					$this->beginBlock($matches[1]);
					continue;
				}

				// Match a configuration line inside a block
				if (preg_match('/^\s*(\S+)\s+([^#;\s][^;#]*)([#;](.*)?)?$/', $line, $matches)) {
					$this->addDirective(trim($matches[1]), trim($matches[2]));
					continue;
				}

				// End of a configuration block
				if (preg_match('/^\s*}\s*(|;.*)$/', $line)) {
					$this->endBlock();
					continue;
				}
			}
			catch (\RuntimeException $e) {
				throw new \RuntimeException(sprintf(
					'%s %d: %s',
					tr('Line'),
					$lineNumber,
					$e->getMessage()
				));
			}

			throw new \RuntimeException(sprintf(
				'%s %d: %s',
				tr('Line'),
				$lineNumber,
				tr('Malformed input')
			));
		}
	}

	/**
	 * Begin a block
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @param         string $objectName
	 * @returns       void
	 */
	protected function beginBlock($objectName) {
		if ($this->currentObject !== null) {
			throw new \RuntimeException(
				tr('Begin of a block but another block is already open')
			);
		}

		$this->currentObject = new NagiosObject($objectName);

	}

	/**
	 * Add a directive to the current object
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @param         string $key
	 * @returns       void
	 */
	protected function addDirective($key, $value) {
		if ($this->currentObject === null) {
			throw new \RuntimeException(
				tr('Add directive outside of a block')
			);
		}

		$this->currentObject->addDirective($key, $value);
	}

	/**
	 * End of block, add current object to corresponding array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	protected function endBlock() {
		if ($this->currentObject === null) {
			throw new \RuntimeException(
				tr('End of block outside of a block')
			);
		}

		$objectsArray = null;
		switch ($this->currentObject->getType()) {
			case 'host':
				$objectsArray = &$this->hosts;
				break;
			case 'service':
				$objectsArray = &$this->services;
				break;
			case 'command':
				$objectsArray = &$this->commands;
				break;
			case 'hostgroup':
				$objectsArray = &$this->hostgroups;
				break;
			case 'hostextinfo':
				$objectsArray = &$this->hostextinfos;
				break;
			case 'servicegroup':
			case 'contact':
			case 'contactgroup':
			case 'timeperiod':
			case 'servicedependency':
			case 'serviceescalation':
			case 'hostdependency':
			case 'hostescalation':
			case 'serviceextinfo':
				// Not implemented
				break;
		}

		if ($objectsArray !== null) {
			$objectsArray[] = $this->currentObject;
		}

		$this->currentObject = null;
	}

	/**
	 * Creates an object from raw data
	 *
	 * Basically, this just writes the raw data from the configuration file to the database
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         Entity $object The model to be filled
	 * @param         array $rawData The raw data
	 * @param         int $fileId The file identifier
	 * @returns       Entity
	 */
	protected function createObject($object, $rawObject, $fileId) {
		$rawData = $rawObject->getDirectives();

		$rawData['id']     = null;
		$rawData['fileId'] = $fileId;

		if (!array_key_exists('register', $rawData)) {
			$rawData['register'] = true;
		}
		else if ($rawData['register'] === null) {
			$rawData['register'] = false;
		}

		$object->allowWrite(model\Permission::IMPORT);
		$object->fillFromImport($rawData);
		$object->save();

		foreach ($rawObject->getIncludes() as $include) {
			$object->addInclude($include);
		}

		return $object;
	}

	/**
	 * Save import information to database
	 *
	 * Includes creating SyMonitoring objects from raw nagios objects
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function save() {
		$file = new model\ImportFile();
		$file->allowWrite(model\Permission::IMPORT);

		$file->name       = $_FILES['file']['name'];
		$file->hosts      = $this->getHostsCount();
		$file->services   = $this->getServicesCount();
		$file->commands   = $this->getCommandsCount();
		$file->hostgroups = $this->getHostgroupsCount();
		$file->save();

		foreach ($this->hosts as $rawHost) {
			$this->createObject(
				new model\ImportHost(),
				$rawHost,
				$file->id
			);
		}

		foreach ($this->services as $rawService) {
			$this->createObject(
				new model\ImportService(),
				$rawService,
				$file->id
			);
		}

		foreach ($this->commands as $rawCommand) {
			$this->createObject(
				new model\ImportCommand(),
				$rawCommand,
				$file->id
			);
		}

		foreach ($this->hostgroups as $rawHostgroup) {
			$this->createObject(
				new model\ImportHostgroup(),
				$rawHostgroup,
				$file->id
			);
		}

		return $file;
	}

	/**
	 * Resolve dependencies
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by denis:   Probably a more generic way for replacing values must be found
	 * @returns       void
	 */
	public static function resolve() {
		$hosts      = datamapper\ImportHostMapper::getInstance()->getAll();
		$services   = datamapper\ImportServiceMapper::getInstance()->getAll();
		$commands   = datamapper\ImportCommandMapper::getInstance()->getAll();
		$hostgroups = datamapper\ImportHostgroupMapper::getInstance()->getAll();

		foreach ($hosts as $host) {
			self::resolveHost($host);

			if (!empty($host->hostgroups)) {
				foreach (explode(',', $host->hostgroups) as $hostgroup) {
					self::$hostgroupTable[$hostgroup][] = $host->host_name;
				}
			}
		}

		foreach ($services as $service) {
			self::resolveService($service);
		}

		foreach ($commands as $command) {
			self::resolveCommand($command);
		}

		foreach ($hostgroups as $hostgroup) {
			self::resolveHostgroup($hostgroup);
		}
	}

	/**
	 * Resolve dependencies for a single host
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\ImportHost $host
	 * @returns       void
	 */
	private static function resolveHost(model\ImportHost $host) {
		$host->allowWrite(model\Permission::IMPORT);

		$new = false;
		if (!array_key_exists((int)$host->id, self::$includedHostsCache)) {
			$new = true;
			self::$includedHostsCache[(int)$host->id] = $host->getIncludedHosts();
		}

		$includedHosts = self::$includedHostsCache[(int)$host->id];

		/*
		 * The include order is important. If a value is defined in more than
		 * one included template, the value from the first include wins. Also,
		 * locally defined values weigh more than values from included
		 * templates.
		 *
		 * See http://docs.icinga.org/latest/en/objectinheritance.html for
		 * further information.
		 */
		$includedHosts->setOrderBy('includeId.ASC');
		foreach ($includedHosts as $includedHost) {
			// If a service already is in the cache it has already been resolved.
			if ($new) {
				// Recursion. See recursion.
				self::resolveHost($includedHost);
			}

			// Earlier values win
			$host->host_name  = $host->host_name ?: $includedHost->host_name;
			$host->alias      = $host->alias ?: $includedHost->alias;
			$host->address    = $host->address ?: $includedHost->address;
			$host->parents    = $host->parents ?: $includedHost->parents;

			// If string values start with a +, the values are concatenated
			$hostgroups = preg_replace('/^\+/', '', $host->hostgroups, 1, $replaced);
			if ($replaced && !empty($hostgroups) && !empty($includedHost->hostgroups)) {
				$host->hostgroups = sprintf(
					'%s,%s',
					$includedHost->hostgroups,
					$hostgroups
				);
			}
			else {
				$host->hostgroups = $hostgroups ?: $includedHost->hostgroups;
			}
		}

		$host->save();
	}

	/**
	 * Resolve dependencies for a single service
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\ImportService $service
	 * @todo by denis:   Handle ! at check_command?
	 * @returns       void
	 */
	private static function resolveService(model\ImportService $service) {
		$service->allowWrite(model\Permission::IMPORT);

		$new = false;
		if (!array_key_exists((int)$service->id, self::$includedServicesCache)) {
			$new = true;
			self::$includedServicesCache[(int)$service->id] = $service->getIncludedServices();
		}

		$includedServices = self::$includedServicesCache[(int)$service->id];

		/*
		 * The include order is important. If a value is defined in more than
		 * one included template, the value from the first include wins. Also,
		 * locally defined values weigh more than values from included
		 * templates.
		 *
		 * See http://docs.icinga.org/latest/en/objectinheritance.html for
		 * further information.
		 *
		 * Not yet implemented:
		 * - check_command definitions with preceding exclamation mark (!) take
		 *   precedence over local values, but only for one recusion step.
		 */
		$includedServices->setOrderBy('includeId.ASC');
		foreach ($includedServices as $includedService) {
			// If a service already is in the cache it has already been resolved.
			if ($new) {
				// Recursion. See recursion.
				self::resolveService($includedService);
			}

			// Earlier values win
			$service->host_name           = $service->host_name ?: $includedService->host_name;
			$service->hostgroup_name      = $service->hostgroup_name ?: $includedService->hostgroup_name;
			$service->service_description = $service->service_description ?: $includedService->service_description;
			$service->check_command       = $service->check_command ?: $includedService->check_command;
			$service->max_check_attempts  = $service->max_check_attempts ?: $includedService->max_check_attempts;
			$service->check_interval      = $service->check_interval ?: $includedService->check_interval;
			$service->retry_interval      = $service->retry_interval ?: $includedService->retry_interval;
		}

		$service->save();
	}

	/**
	 * Resolve dependencies for a single command
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\ImportCommand $command
	 * @returns       void
	 */
	private static function resolveCommand(model\ImportCommand $command) {
		$command->allowWrite(model\Permission::IMPORT);

		$new = false;
		if (!array_key_exists((int)$command->id, self::$includedCommandsCache)) {
			$new = true;
			self::$includedCommandsCache[(int)$command->id] = $command->getIncludedCommands();
		}

		$includedCommands = self::$includedCommandsCache[(int)$command->id];

		/*
		 * The include order is important. If a value is defined in more than
		 * one included template, the value from the first include wins. Also,
		 * locally defined values weigh more than values from included
		 * templates.
		 *
		 * See http://docs.icinga.org/latest/en/objectinheritance.html for
		 * further information.
		 */
		$includedCommands->setOrderBy('includeId.ASC');
		foreach ($includedCommands as $includedCommand) {
			// If a command already is in the cache it has already been resolved.
			if ($new) {
				// Recursion. See recursion.
				self::resolveCommand($includedCommand);
			}

			// Earlier values win
			$command->command_name = $command->command_name ?: $includedCommand->command_name;
			$command->command_line = $command->command_line ?: $includedCommand->command_line;
		}

		$command->save();
	}

	/**
	 * Resolve dependencies for a single host group
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         model\ImportHostgroup $hostgroup
	 * @returns       void
	 */
	private static function resolveHostgroup(model\ImportHostgroup $hostgroup) {
		$hostgroup->allowWrite(model\Permission::IMPORT);

		$new = false;
		if (!array_key_exists((int)$hostgroup->id, self::$includedHostgroupsCache)) {
			$new = true;
			self::$includedHostgroupsCache[(int)$hostgroup->id] = $hostgroup->getIncludedHostgroups();
		}

		$includedHostgroups = self::$includedHostgroupsCache[(int)$hostgroup->id];

		/*
		 * The include order is important. If a value is defined in more than
		 * one included template, the value from the first include wins. Also,
		 * locally defined values weigh more than values from included
		 * templates.
		 *
		 * See http://docs.icinga.org/latest/en/objectinheritance.html for
		 * further information.
		 */
		$includedHostgroups->setOrderBy('includeId.ASC');
		foreach ($includedHostgroups as $includedHostgroup) {
			// If a host group already is in the cache it has already been resolved.
			if ($new) {
				// Recursion. See recursion.
				self::resolveHostgroup($includedHostgroup);
			}

			// Earlier values win
			$hostgroup->hostgroup_name    = $hostgroup->hostgroup_name ?: $includedHostgroup->hostgroup_name;
			$hostgroup->alias             = $hostgroup->alias ?: $includedHostgroup->alias;
			$hostgroup->members           = $hostgroup->members ?: $includedHostgroup->members;
			$hostgroup->hostgroup_members = $hostgroup->hostgroup_members ?: $includedHostgroup->hostgroup_members;
		}

		// Add all the host group memberships defined in Host objects to the corresponding host group
		if (array_key_exists($hostgroup->hostgroup_name, self::$hostgroupTable)) {
			$addHostgroups = implode(',', self::$hostgroupTable[$hostgroup->hostgroup_name]);

			if (!empty($hostgroup->members) && !empty($addHostgroups)) {
				$hostgroup->members = sprintf(
					'%s,%s',
					$hostgroup->hostgroup_members,
					$addhostgroups
				);
			}
			else {
				$hostgroup->members = $hostgroup->members ?: $addHostgroups;
			}
		}

		$hostgroup->save();
	}

	/**
	 * Getter for the hosts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getHosts() {
		return $this->hosts;
	}

	/**
	 * Return amount of hosts found
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getHostsCount() {
		return count($this->hosts);
	}

	/**
	 * Getter for the services
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getServices() {
		return $this->services;
	}

	/**
	 * Return amount of services found
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getServicesCount() {
		return count($this->services);
	}

	/**
	 * Getter for the commands
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getCommands() {
		return $this->commands;
	}

	/**
	 * Return amount of commands found
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getCommandsCount() {
		return count($this->commands);
	}

	/**
	 * Getter for the host groups
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       array
	 */
	public function getHostgroups() {
		return $this->hostgroups;
	}

	/**
	 * Return amount of hostgroups found
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       int
	 */
	public function getHostgroupsCount() {
		return count($this->hostgroups);
	}
}
