<?php

namespace ch\monitoring\util;

use ch\monitoring\datamapper;
use org\adfinis\framework\config\ConfManager;

/**
 * Read from and write to RRD files
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class RRD {

	/**
	 * The AVERAGE consolidation function
	 *
	 * @const         string CF_AVERAGE
	 */
	const CF_AVERAGE = "AVERAGE";

	/**
	 * The MIN consolidation function
	 *
	 * @const         string CF_MIN
	 */
	const CF_MIN = "MIN";

	/**
	 * The MAX consolidation function
	 *
	 * @const         string CF_MAX
	 */
	const CF_MAX = "MAX";

	/**
	 * The LAST consolidation function
	 *
	 * @const         string CF_LAST
	 */
	const CF_LAST = "LAST";

	/**
	 * The rrd database file to be edited
	 *
	 * @var           string $file
	 */
	private $file;

	/**
	 * The database information
	 *
	 * @var           array $information
	 */
	private $information;

	/**
	 * The first entry in the rrd
	 *
	 * @var           int $timestampFirst
	 */
	private $timestampFirst;

	/**
	 * The last entry in the rrd
	 *
	 * @var           int $timestampLast
	 */
	private $timestampLast;

	/**
	 * Object constuctor
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 */
	public function __construct($file){
		if (!file_exists($file)) {
			throw new \RuntimeException(
				tr("File not found")
			);
		}

		$information = rrd_info($file);
		$timestampFirst = rrd_first($file);
		$timestampLast = rrd_last($file);

		if ($information === FALSE || $timestampFirst === FALSE || $timestampLast === FALSE) {
			throw new \RuntimeException(
				sprintf(
					tr("Could not read RRD information: %s"),
					rrd_error()
				)
			);
		}

		$this->file           = $file;
		$this->information    = $information;
		$this->timestampFirst = $timestampFirst;
		$this->timestampLast  = $timestampLast;
	}

	/**
	 * Fetches data from RRD
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $cf The consolidation function to be used
	 * @param         \Zend_Date $dateFrom
	 * @param         \Zend_Date $dateTo
	 * @throws        \UnexpectedValueException
	 * @returns       array
	 */
	public function fetch($cf, $dateFrom, $dateTo) {
		$dateTo->add(1, \Zend_Date::DAY);

		if ($dateFrom->get(\Zend_Date::TIMESTAMP) <= $this->timestampFirst) {
			throw new \OutOfBoundsException(
				tr("Start time is earlier than first record")
			);
		}

		if ($dateTo->get(\Zend_Date::TIMESTAMP) >= $this->timestampLast) {
			throw new \OutOfBoundsException(
				tr("End time is later than last record")
			);
		}

		$timestampFrom = max($dateFrom->get(\Zend_Date::TIMESTAMP), $this->timestampFirst + 1);
		$timestampTo   = min($dateTo->get(\Zend_Date::TIMESTAMP), $this->timestampLast - 1);
		$timestampTo   = max($timestampFrom, $timestampTo);

		$options = array(
			$cf,
			'--start', $timestampFrom,
			'--end'  , $timestampTo
		);

		$result = rrd_fetch($this->file, $options);

		if ($result === FALSE || !is_array($result)) {
			throw new \UnexpectedValueException(
				tr("No results")
			);
		}

		return $result;
	}
}
