<?php

namespace ch\monitoring\validate;

use ch\monitoring\datamapper;
use ch\monitoring\model;

/**
 * Validator for Alert objects
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class AlertValidator extends ModelValidator {

	/**
	 * Validates the customer id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function customerIdValidator() {
		$this->referenceExists('customerId', datamapper\CustomerMapper::getInstance());
	}

	/**
	 * Validates the host id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function hostIdValidator() {
		$this->referenceExists('hostId', datamapper\HostMapper::getInstance());
	}

	/**
	 * Validates the command id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function commandIdValidator() {
		$this->referenceExists('commandId', datamapper\CommandMapper::getInstance());
	}

	/**
	 * Validates the service instance id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function serviceInstanceIdValidator() {
		if ($this->model->serviceInstanceId !== null) {
			$this->referenceExists('serviceInstanceId', datamapper\HostServiceInstanceMapper::getInstance());
		}
	}

	/**
	 * Validates the sla id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function slaIdValidator() {
		if ($this->model->slaId !== null) {
			$this->referenceExists('slaId', datamapper\SLAMapper::getInstance());
		}
	}

	/**
	 * Validates the sla customer id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function slaCustomerIdValidator() {
		if ($this->model->slaCustomerId !== null) {
			$this->referenceExists('slaCustomerId', datamapper\CustomerMapper::getInstance());
		}
	}

	/**
	 * Validates the datetimeOccured
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function datetimeOccuredValidator() {
		$this->isNotNull('datetimeOccured');
	}

	/**
	 * Validates the datetimeAcknowledged
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function datetimeAcknowledgedValidator() {
		// Nothing
	}

	/**
	 * Validates the datetimeRecovery
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function datetimeRecoveryValidator() {
		// Nothing
	}

	/**
	 * Validates the state
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function stateValidator() {
		$list = array(
			'UP',
			'DOWN',
			'UNREACHABLE',
			'OK',
			'WARNING',
			'UNKNOWN',
			'CRITICAL'
		);
		$this->isInList('state', $list);
	}

	/**
	 * Validates the output
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function outputValidator() {
		$this->isNotNull('output');
	}

	/**
	 * Validates the acknowledgedBy field
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function acknowledgedByValidator() {
		if ($this->model->acknowledgedBy !== null) {
			$this->referenceExists('acknowledgedBy', datamapper\UserMapper::getInstance());
		}
	}

	/**
	 * Validates the acknowledged flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function acknowledgedValidator() {
		$this->isNotNull('acknowledged');
	}

	/**
	 * Validates the acknowledgedAutomatically flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function acknowledgedAutomaticallyValidator() {
		$this->isNotNull('acknowledgedAutomatically');
	}

	/**
	 * Validates the comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function commentValidator() {
		// Nothing
	}

	/**
	 * Validates the complete Alert object
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function validateEntity() {
		$this->validateField('customerId');
		$this->validateField('hostId');
		$this->validateField('commandId');
		$this->validateField('serviceInstanceId');
		$this->validateField('slaId');
		$this->validateField('slaCustomerId');
		$this->validateField('datetimeOccured');
		$this->validateField('state');
		$this->validateField('output');
		$this->validateField('acknowledgedBy');
		$this->validateField('acknowledged');
		$this->validateField('acknowledgedAutomatically');
	}
}
