<?php

namespace ch\monitoring\validate;

use ch\monitoring\datamapper;

/**
 * Validator for Customer objects
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class CustomerValidator extends ModelValidator {

	/**
	 * Validates the parent id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function parentIdValidator() {
		if ($this->model->parentId !== null) {
			$this->referenceExists(
				'parentId',
				datamapper\CustomerMapper::getInstance()
			);

			if ($this->model->parentId === $this->model->id) {
				$this->setIsInvalid(
					'parentId',
					tr('Customer cannot be a subcustomer of itself')
				);
			}
		}
	}

	/**
	 * Validates the sla id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function slaIdValidator() {
		$this->referenceExists(
			'slaId',
			datamapper\SLAMapper::getInstance()
		);
	}

	/**
	 * Validates the default role ID
	 *
	 * @returns       bool  true if the default role ID is valid, false
	 *                      otherwise
	 */
	protected function defaultRoleIdValidator() {
		$this->referenceExists(
			'defaultRoleId',
			datamapper\RoleMapper::getInstance()
		);
		$this->isNotGlobal(
			'defaultRoleId',
			datamapper\RoleMapper::getInstance()
		);
	}

	/**
	 * Validates the name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function nameValidator() {
		$this->isNotEmpty('name');
		$this->isNagiosString('name');
		$this->nameIsFree('name');
	}

	/**
	 * Validates the email address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function emailValidator() {
		if ($this->model->email !== null) {
			$this->isEmail('email');
		}
	}

	/**
	 * Validates the warnHoster flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function warnHosterValidator() {
		$this->isNotNull('warnHoster');
	}

	/**
	 * Validates the wiki URL
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function wikiValidator() {
		if ($this->model->wiki !== null) {
			$this->isURL('wiki');
		}
	}

	/**
	 * Validates the comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function commentValidator() {
		// Nothing
	}

	/**
	 * Validates the information password
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function passwordInformationValidator() {
		// Nothing
	}

	/**
	 * Validates the maintenance password
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function passwordMaintenanceValidator() {
		// Nothing
	}

	/**
	 * Validates the last access flat
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function lastAccessValidator() {
		// Nothing
	}

	/**
	 * Validates the isEnabled flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isEnabledValidator() {
		$this->isNotNull('isEnabled');
	}

	/**
	 * Validates the isDeleted flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isDeletedValidator() {
		// Nothing
	}

	/**
	 * Validates the complete Customer object
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function validateEntity() {
		$this->validateField('parentId');
		$this->validateField('slaId');
		$this->validateField('defaultRoleId');
		$this->validateField('name');
		$this->validateField('email');
		$this->validateField('warnHoster');
		$this->validateField('wiki');
		$this->validateField('isEnabled');
	}
}
