<?php

namespace ch\monitoring\validate;

use ch\monitoring\datamapper;
use ch\monitoring\Application;
use ch\monitoring\model;

/**
 * Validator for Host objects
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class HostValidator extends ModelValidator {

	/**
	 * Validates the customer id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function customerIdValidator() {
		$this->referenceExists('customerId', datamapper\CustomerMapper::getInstance());
	}

	/**
	 * Validates the sla id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function slaIdValidator() {
		if ($this->model->slaId !== null) {
			$this->referenceExists('slaId', datamapper\SLAMapper::getInstance());
		}
	}

	/**
	 * Validates the dependency id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function dependencyIdValidator() {
		if ($this->model->dependencyId !== null) {
			$this->referenceExists(
				'dependencyId',
				datamapper\HostMapper::getInstance()
			);

			if ($this->model->dependencyId === $this->model->id) {
				$this->setIsInvalid(
					'dependencyId',
					tr('Host cannot depend on itself')
				);
			}

			if ($this->model->satelliteId !== null) {
				$this->isInList(
					'dependencyId',
					array_merge(
						$this->model->getSatellite()->getSatelliteHostIds(),
						array($this->model->satelliteId)
					),
					tr('The host does not depend on a satellite')
				);
			}
			else if ($this->model->isSatellite) {
				$this->isInList(
					'dependencyId',
					$this->model->getSatelliteHostIds(),
					tr('The satellite can only depend on a host that uses this host as satellite')
				);
			}
		}
	}

	/**
	 * Validates the satellite id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function satelliteIdValidator() {
		if ($this->model->satelliteId !== null) {
			$this->referenceExists('satelliteId', datamapper\HostMapper::getInstance());

			if ($this->model->isSatellite !== false) {
				$this->setIsInvalid(
					'satelliteId',
					tr('validate_error_satelliteAndBehindSatellite')
				);
			}

			if ($this->model->satelliteId === $this->model->id) {
				$this->setIsInvalid(
					'satelliteId',
					tr('validate_error_selfSatellite')
				);
			}
		}
	}

	/**
	 * Validates the os id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function osIdValidator() {
        if ($this->model->osId === null) {
            return $this->setIsInvalid(
                'osId',
                tr('No operating system was selected.')
            );
        }
		$this->referenceExists('osId', datamapper\HostOSMapper::getInstance());
	}

	/**
	 * Validates the os version
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function osVersionIdValidator() {
		if ($this->model->osVersionId !== null) {
			$this->referenceExists('osVersionId', datamapper\HostOSVersionMapper::getInstance());
		}
	}

	/**
	 * Validates the type id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function typeIdValidator() {
		$this->referenceExists('typeId', datamapper\HostTypeMapper::getInstance());
	}

	/**
	 * Validates the maintenance inherit sla flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function inheritSlaValidator() {
		// Nothing
	}

	/**
	 * Validates the check host alive flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function checkHostAliveValidator() {
		$this->isNotNull('checkHostAlive');
	}

	/**
	 * Validates the hostname
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function hostnameValidator() {
		$this->isNotEmpty('hostname');
		$this->nameIsFree('hostname');
	}

	/**
	 * Validates the description
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function descriptionValidator() {
		$this->isNotEmpty('description');
	}

	/**
	 * Validates the IPv4 address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function IPv4Validator() {
		$this->isIPv4('IPv4');
	}

	/**
	 * Validates the IPv6 address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function IPv6Validator() {
		if ($this->model->IPv6 !== null) {
			$this->isIPv6('IPv6');
		}
	}

	/**
	 * Validates the isSatellite flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isSatelliteValidator() {
		// Nothing
	}

	/**
	 * Validates the isEnabled flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isEnabledValidator() {
		$this->isNotNull('isEnabled');
	}

	/**
	 * Validates the inTrouble flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function inTroubleValidator() {
		// Nothing
	}

	/**
	 * Validates the comment
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function commentValidator() {
		// Nothing
	}

	/**
	 * Validates the creation date and time
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function datetimeCreatedValidator() {
		$this->isNotNull('datetimeCreated');
	}

	/**
	 * Validates the last export date
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function dateExportValidator() {
		// Nothing
	}

	/**
	 * Validates the export version
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function versionValidator() {
		$this->isGreaterThanOrEqualZero('version');
	}

	/**
	 * Validates the isDeleted flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isDeletedValidator() {
		// Nothing
	}

	/**
	 * Validates the wiki property
	 *
	 * @author        Tobias Feller <tobias.feller@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function wikiValidator() {
		if(isset($this->model->wiki)) {
			$this->isURL('wiki');
		}
	}

	/**
	 * Validates the complete Host object
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function validateEntity() {
		$this->validateField('customerId');
		$this->validateField('slaId');
		$this->validateField('dependencyId');
		$this->validateField('satelliteId');
		$this->validateField('osId');
		$this->validateField('osVersionId');
		$this->validateField('inheritSla');
		$this->validateField('checkHostAlive');
		$this->validateField('hostname');
		$this->validateField('description');
		$this->validateField('IPv4');
		$this->validateField('IPv6');
		$this->validateField('isEnabled');
		$this->validateField('comment');
		$this->validateField('datetimeCreated');
		$this->validateField('version');
		$this->validateField('wiki');
	}
}
