<?php

namespace ch\monitoring\validate;

use ch\monitoring\model\Permission;
use ch\monitoring\model\EntityWithPolicy;
use ch\monitoring\Application;

/**
 * Abstract class for a model validator.
 *
 * This defines the behaviour of a validator that can check a given Entity object.
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
abstract class ModelValidator extends \org\adfinis\framework\validate\ModelValidator {

	/**
	 * Validates the given field of the entity.
	 *
	 * This function calls the function $fieldValidator().
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	public function validateField($field) {
		$method = array(
			$this,
			sprintf(
				"%sValidator",
				$field
			)
		);

		if (is_callable($method)) {
			call_user_func($method);
		}
		else {
			throw new \InvalidArgumentException(sprintf(
				tr('Field %s does not exist'),
				$field
			));
		}
	}

	/**
	 * Verifies whether a variable is not null
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isNotNull($field) {
		if ($this->model->$field === null) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Field \'%s\' is mandatory'),
					$field
				)
			);
		}
	}

	/**
	 * Verifies whether a variable is greater than or equal to zero
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @returns       void
	 */
	protected function isGreaterThanOrEqualZero($field) {
		if ($this->model->$field < 0) {
			$this->setIsInvalid(
				$field,
				tr('Value must be greater than or equal to zero')
			);
		}
	}

	/**
	 * Verifies whether a variable is greater than zero
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @returns       void
	 */
	protected function isGreaterThanZero($field) {
		if ($this->model->$field < 1) {
			$this->setIsInvalid(
				$field,
				tr('Value must be greater than zero')
			);
		}
	}

	/**
	 * Verifies whether a variable is greater than a number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @param         int $value
	 * @returns       void
	 */
	protected function isGreaterThan($field, $value) {
		if ($this->model->$field <= $value) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Value must be greater than %d'),
					$value
				)
			);
		}
	}

	/**
	 * Verifies whether a variable is greater than or equal a number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @param         int $value
	 * @returns       void
	 */
	protected function isGreaterThanOrEqual($field, $value) {
		if ($this->model->$field < $value) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Value must be greater than or equal %d'),
					$value
				)
			);
		}
	}

	/**
	 * Verifies whether a variable is less than a number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @param         int $value
	 * @returns       void
	 */
	protected function isLessThan($field, $value) {
		if ($this->model->$field >= $value) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Value must be less than %d'),
					$value
				)
			);
		}
	}

	/**
	 * Verifies whether a variable is less than or equal a number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         int $field
	 * @param         int $value
	 * @returns       void
	 */
	protected function isLessThanOrEqual($field, $value) {
		if ($this->model->$field > $value) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Value must be less than or equal %d'),
					$value
				)
			);
		}
	}

	/**
	 * Verifies whether a field is not empty
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isNotEmpty($field) {
		if (empty($this->model->$field)) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('The field "%s" must not be empty'),
					$field
				)
			);
		}
	}

	/**
	 * Verifies whether a field is a valid string for nagios
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isNagiosString($field) {
		$disallowed = "`~!\$%\^&\*\|'\"<>\?,:()=";
		$regexp = sprintf(
			"/^[^%s]+$/",
			$disallowed
		);

		if (!preg_match($regexp, $this->model->$field)) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr('Not a valid string "%s" in field "%s". Disallowed characters: %s'),
					$this->model->$field,
					$field,
					$disallowed
				)
			);
		}
	}

	/**
	 * Verifies whether the given value is a valid eMail address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isEmail($field) {
		$validator = new \Zend_Validate_EmailAddress();
		if (!$validator->isValid($this->model->$field)) {
			$this->setIsInvalid(
				$field,
				tr('Invalid e-mail address')
			);
		}
	}

	/**
	 * Verifies whether the given value is a valid phone number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isPhoneNumber($field) {
		if (!preg_match("/^[0-9]+$/", $this->model->$field)) {
			$this->setIsInvalid(
				$field,
				tr('Invalid phone number (only numbers 0-9 are allowed)')
			);
		}
	}

	/**
	 * Verifies whether the given value is a valid web URL 
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isURL($field) {
		$filterResult = filter_var(
			$this->model->$field,
			\FILTER_VALIDATE_URL
		);
		if ($filterResult === false) {
			$this->setIsInvalid(
				$field,
				tr('Invalid URL')
			);
		}
	}

	/**
	 * Verifies whether a given value is in an array
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @param         string $field
	 * @param         array $list
	 * @returns       void
	 */
	protected function isInList($field, $list, $msg = null) {
		if(is_null($msg)) {
			$msg = tr('Invalid Value');
		}

		if (!in_array($this->model->$field, $list, true)) {
			$this->setIsInvalid(
				$field,
				$msg
			);
		}
	}

	/**
	 * Verifies whether a given reference value is valid
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @param         org\adfinis\framework\datamapper\Mapper $mapper
	 * @returns       void
	 */
	protected function referenceExists($field, $mapper) {
		try {
			$model = $mapper->getByID($this->model->$field);

			if ($model instanceof EntityWithPolicy) {
				$model->allowRead();
			}
		}
		catch (\OutOfBoundsException $e) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr("Invalid reference: %s.%s(%s)"),
					get_class($this->model),
					$field,
					$this->model->$field
				)
			);
		}
	}

	/**
	 * Verifies whether the name is free
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @param         bool $auth Optional. Whether to add auth method
	 * @returns       void
	 */
	protected function nameIsFree($field, $auth = false) {
		try {
			if ($auth) {
				$entity = $this->model->getMapper()->getByName(
					$this->model->$field,
					$this->model->authenticationMethod
				);
			}
			else {
				$entity = $this->model->getMapper()->getByName(
					$this->model->$field
				);
			}

			if ($entity instanceof EntityWithPolicy) {
				$entity->allowRead();
			}

			// Probably we found ourself :)
			if ($entity->doesRecordMatch($this->model->getIdentifier())) {
				return;
			}

			$this->setIsInvalid(
				$field,
				tr('The name is already in use')
			);
		}
		catch (\OutOfBoundsException $e) {
			// Name is free so don't do anything
			return;
		}
	}

	/**
	 * Verifies whether a value is an IPv4 address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isIPv4($field) {
		$regex = '/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/';

		if (!preg_match($regex, $this->model->$field)) {
			$this->setIsInvalid(
				$field,
				tr('Not an IPv4 compliant address')
			);
		}
	}

	/**
	 * Verifies whether a value is an IPv6 address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $field
	 * @returns       void
	 */
	protected function isIPv6($field) {
		$regex = '/^(((?=(?>.*?(::))(?!.+\3)))\3?|([\dA-F]{1,4}(\3|:(?!$)|$)|\2))(?4){5}((?4){2}|(25[0-5]|(2[0-4]|1\d|[1-9])?\d)(\.(?7)){3})\z/i';

		if(!preg_match($regex, $this->model->$field)) {
			$this->setIsInvalid(
				$field,
				tr('Not an IPv6 compliant address given')
			);
		}
	}

	/**
	 * Verifies whether a string contains a value
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @param         string $value
	 * @returns       void
	 */
	protected function contains($field, $value) {
		if (strpos($this->model->$field, $value) === FALSE) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr("Must contain the value %s"),
					$value
				)
			);
		}
	}

	/**
	 * Verifies whether a string does not contain a value
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $field
	 * @param         string $value
	 * @returns       void
	 */
	protected function containsNot($field, $value) {
		if (strpos($this->model->$field, $value) !== FALSE) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr("Must not contain the value %s"),
					$value
				)
			);
		}
	}

	/**
	 * Returns true if there are errors
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       bool
	 */
	public function hasErrors() {
		return (bool)sizeof($this->modelValidity);
	}
	
    /**
	 * Checks if given string is an alphanumeric string or has '_' or '-'
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        void
	 */
	protected function isExtendedAlphanumeric($field) {
		if (!preg_match('/^[a-zA-Z0-9_\-]*$/', $this->model->$field)) {
			$this->setIsInvalid(
				$field,
				sprintf(
					tr("The field '%s' may only contain alphanumeric values, '-' or '_'!"),
					$field
				)
			);
		}
	}

    /**
	 * Verifies that the given object is not global
	 */
	protected function isNotGlobal($field, $mapper) {
		$model = $mapper->getByID($this->model->$field);
		if ($model->isGlobal()) {
			$this->setIsInvalid(
				$field,
				tr("May not be global")
			);
		}
	}
}
