<?php

namespace ch\monitoring\validate;

use ch\monitoring\datamapper;
use ch\monitoring\model;

/**
 * Validator for Service objects
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class ServiceValidator extends ModelValidator {
	/**
	 * Validates the sla id
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function slaIdValidator() {
		if(!is_null($this->model->slaId)) {
			$this->referenceExists('slaId', datamapper\SLAMapper::getInstance());
		}
	}


	/**
	 * Validates the command id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function commandIdValidator() {
		$this->referenceExists('commandId', datamapper\CommandMapper::getInstance());
	}

	/**
	 * Validates the customer id
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function customerIdValidator() {
		if ($this->model->customerId !== null) {
			$this->referenceExists('customerId', datamapper\CustomerMapper::getInstance());
		}
	}

	/**
	 * Validates the name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function nameValidator() {
		$this->isNotEmpty('name');
		$this->isNagiosString('name');
		$this->nameIsFree('name');
		
		if (strpos($this->model->name, '__') !== false) {
			$this->setIsInvalid(
				'name',
				tr('Service name may not include double underscore (__).')
			);
		}
	}

	/**
	 * Validates the description
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function descriptionValidator() {
		$this->isNotEmpty('description');
	}

	/**
	 * Validates the maxCheckAttempts
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function maxCheckAttemptsValidator() {
		$this->isGreaterThanZero('maxCheckAttempts');
		$this->isLessThanOrEqual('maxCheckAttempts', 10080);
	}

	/**
	 * Validates the checkInterval
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function checkIntervalValidator() {
		$this->isGreaterThanZero('checkInterval');
		$this->isLessThanOrEqual('checkInterval', 10080);
	}

	/**
	 * Validates the retryInterval
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function retryIntervalValidator() {
		$this->isGreaterThanZero('retryInterval');
		$this->isLessThanOrEqual('retryInterval', 10080);
	}

	/**
	 * Validates the freshness
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function freshnessValidator() {
		$this->isGreaterThanOrEqualZero('freshness');
		$this->isLessThanOrEqual('freshness', 604800);
	}

	/**
	 * Validates the isNrpe flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isNrpeValidator() {
		$this->isNotNull('isNrpe');
	}

	/**
	 * Validates the isDeleted flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isDeletedValidator() {
		// Nothing
	}

	/**
	 * Validates the complete Service object
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function validateEntity() {
		$this->validateField('slaId');
		$this->validateField('commandId');
		$this->validateField('customerId');
		$this->validateField('name');
		$this->validateField('description');
		$this->validateField('maxCheckAttempts');
		$this->validateField('checkInterval');
		$this->validateField('retryInterval');
		$this->validateField('freshness');
		$this->validateField('isNrpe');
	}
}
