<?php

namespace ch\monitoring\validate;

use ch\monitoring\datamapper;
use ch\monitoring\model;

/**
 * Validator for User objects
 *
 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2012
 * @license       LGPLv3
 **/
class UserValidator extends ModelValidator {

	/**
	 * Validates the name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function nameValidator() {
		$this->isNotEmpty('name');
		$this->nameIsFree('name', true);
	}

	/**
	 * Validates the full name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function fullNameValidator() {
		$this->isNotEmpty('fullName');
	}

	/**
	 * Validates the password hash
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function passwordHashValidator() {
		// Nothing
	}
	
	/**
	 * Validates the password salt
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function passwordSaltValidator() {
		// Nothing
	}

	/**
	 * Validates the phone number
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function phoneValidator() {
		if ($this->model->phone !== null) {
			$this->isPhoneNumber('phone');
		}
	}

	/**
	 * Validates the phone number for the sms
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function smsValidator() {
		if ($this->model->sms !== null) {
			$this->isPhoneNumber('sms');
		}
	}

	/**
	 * Validates the jabber address / JID
	 *
	 * It has the same pattern as an email -> http://www.jabber.org/faq.html#jid
	 *
	 * @author        Matthieu Riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function jabberValidator() {
		if ($this->model->jabber !== null) {
			$this->isEmail('jabber');
		}
	}

	/**
	 * Validates the mattermost name
	 *
	 * Just check if it's not empty
	 *
	 * @author        Andrea Bettich <andrea.bettich@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function mattermostValidator() {
		$this->isNotEmpty('mattermost');
	}

	/**
	 * Validates the email address
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function emailValidator() {
		if ($this->model->email !== null) {
			$this->isEmail('email');
		}
	}

	/**
	 * Validates the authentication method
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function authenticationMethodValidator() {
		$this->isInList(
			'authenticationMethod',
			array(
				model\User::AUTH_METHOD_LDAP,
				model\User::AUTH_METHOD_LDAP_SYNC,
				model\User::AUTH_METHOD_LOCAL,
			)
		);
	}

	/**
	 * Validates the isEnabled flag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isEnabledValidator() {
		$this->isNotNull('isEnabled');
	}

	/**
	 * Validates the isDeletedflag
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function isDeletedValidator() {
		// Nothing
	}

	/**
	 * Validates the complete User object
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @author        David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function validateEntity() {
		$this->validateField('name');
		$this->validateField('fullName');
		$this->validateField('phone');
		$this->validateField('sms');
		$this->validateField('email');
		$this->validateField('jabber');
		$this->validateField('mattermost');
		$this->validateField('isEnabled');
	}
}
