<?php

namespace ch\monitoring\view\datatable;

use org\adfinis\framework\template\Skittle;

/**
 * Abstract definition of an DataTable View class
 *
 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
 * @copyright     Adfinis SyGroup AG 2014
 * @license       LGPLv3
 **/
abstract class View extends \ch\monitoring\view\View {

	/**
	 * Flag which defines that a field is sortable
	 *
	 * @const         int FIELD_SEARCHABLE
	 */
	const FIELD_SEARCHABLE = 1;

	/**
	 * Flag which defines that a field is searchable
	 *
	 * Not integrated yet
	 *
	 * @const         int FIELD_SORTABLE
	 */
	const FIELD_SORTABLE = 2;

	/**
	 * Flag which defines that a field is hidden
	 *
	 * Not integrated yet
	 *
	 * @const         int FIELD_HIDDEN
	 */
	const FIELD_HIDDEN = 4;

	/**
	 * Holds the mapping for the datatable fields and their templates and flags
	 *
	 * The flags are optional
	 *
	 * @example:
	 * 	array [
	 * 		model::FIELD => [ "template.tpl", self::FIELD_FLAGS | ... ],
	 * 		(string)          (string)        (int, optional)
	 * 		...
	 * 	]
	 *
	 * @var           array $dataTableMapping
	 */
	protected static $dataTableMapping;

	/**
	 * Create a new datatable view handler class
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @param         \org\adfinis\framework\model\Entity $model
	 * @param         \org\adfinis\framework\template\Skittle $tpl
	 */
	public function __construct($model = null, $tpl = null) {
		parent::__construct($model, $tpl);
	}

	/**
	 * Return a numeric index representing the index of given field
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @var           string $field
	 * @return        int
	 */
	public static function getIndexOfField($field) {
		return array_search($field, array_keys(static::getDataTableMapping()));
	}

	/**
	 * Return a numeric index representing the index of given field
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @var           string $field
	 * @return        int
	 */
	public static function getFieldByIndex($index) {
		$fields = array_keys(static::getDataTableMapping());
		return $fields[$index];
	}

	/**
	 * Get all relevant data in a compatible format for DataTables
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        array
	 */
	public function getDataTableData() {
		$renderTemplates = function ($template) {
			return $this->render($template);
		};

		return array_values(array_map($renderTemplates, $this->getFieldTemplates()));
	}

	/**
	 * Get the DataTable mapping
	 *
	 * This method must be overwritten* in the subclass, otherwise the
	 * DataTableIterator cannot parse the data from the model
	 *
	 * *PHP does not support to overwrite static methods, so its just rewritten
	 *
	 * @example:
	 * 	array [
	 * 		model::FIELD => [ "template.tpl", self::FIELD_FLAGS | ... ],
	 * 		(string)          (string)        (int, bitoperator compatible)
	 * 		...
	 * 	]
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @throws        \RuntimeException
	 * @return        void
	 */
	public static function getDataTableMapping() {
		if (!static::$dataTableMapping) {
			throw new \RuntimeException(
				'No DataTableFields are defined, please add the definition to this model'
			);
		}

		return static::$dataTableMapping;
	}

	/**
	 * Get the DataTable fields and flags
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        void
	 */
	public static function getDataTableFields() {
		$mapping = static::getDataTableMapping();
		$fields = array_keys($mapping);

		// Checks if fields are set and return them, return null if not
		$getFlags = function($field) {
			if (isset($field[1])) {
				return $field[1];
			}
			return null;
		};

		$flags = array_map($getFlags, $mapping);

		return array_combine($fields, $flags);
	}

	/**
	 * Get the templates for the data table fields for this model
	 *
	 * @author        Philipp Schärer <philipp.schaerer@adfinis-sygroup.ch>
	 * @return        void
	 */
	public static function getFieldTemplates() {
		$dataTableFields = static::getDataTableMapping();
		$getTemplates = function ($fieldSettings) {
			list($template) = $fieldSettings;
			return $template;
		};

		return array_map($getTemplates, $dataTableFields);
	}
}