# coding: utf-8

"""Tests the downtimesAPI's functionality"""

# System imports
try:
    import unittest2 as unittest
except ImportError:  # pragma: no coverih
    import unittest

# Project imports
try: # Python2
    from urllib import urlencode, urlopen
except ImportError: # Python3
    from urllib.request import urlopen
    from urllib.parse import urlencode

from ssl import _create_unverified_context as cuc
from json import loads as loadjson
from re import match
from functools import wraps
from copy import deepcopy
from xml.etree.ElementTree import fromstring

# nice decorator by David Vogt
def repeat_with(lst):
    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            for x in lst:
                fn(*args, style=x, **kwargs)
        return wrapper
    return decorator

class DowntimesAPITest(unittest.TestCase):
    _base_url = "https://symonitoring.vm/"
    _def_parsed = {
        'notify': {
            'hasNotifyMessages': False,
            'notifyMessages': []
        },
        'error': {
            'hasErrorMessages': False,
            'errorMessages': []
        }
    }
    _messages = {
        'add_action':     "Downtime was added",
        'stop_action':    "All active downtimes of this host/service are stopped",
        'wrong_action':   "Action '%%' is not defined",
        'wrong_host_id':  "Could not find host with the given parameters",
        'wrong_login':    "Access denied",
        'wrong_duration': "Please provide a valid duration",
    }
    # default post parameterss
    _def_post = {
        'action':   'add',
        'hostId':   3,
        'password': 'pihX5mVNPzeOItg9KevV1CWzv3z53LI9',
        'duration': 1
    }
    # default get parameters
    _def_get = {
        'page': 'downtimesAPI',
        'style': 'text'
    }
    # the possible serialization styles
    _styles = ['text', 'json', 'xml']

    def parse_response(self, response, style='text'):
        switcher = {
            'text': self.parse_text_response,
            'json': self.parse_json_response,
            'xml' : self.parse_xml_response
        }
        func = switcher.get(style, self.parse_text_response)
        return func(response)

    def parse_text_response(self, response):
        messages = response.decode('utf-8').split("\n")
        notifications = [n.replace("NOTIFICATION: ",'') for n in messages if "NOTIFICATION" in n]
        errors = [e.replace("ERROR: ",'') for e in messages if "ERROR" in e]
        params = dict([p.replace("PARAMETER: ", '').split(' ') for p in messages if "PARAMETER" in p])
        parsed = {
            'notify': {
                'hasNotifyMessages': bool(notifications),
                'notifyMessages': notifications
            },
            'error': {
                'hasErrorMessages': bool(errors),
                'errorMessages': errors
            },
            'parameters': {
                'action': params['action'],
                'avoidAlerting': bool(params['avoidAlerting'])
            }
        }
        return parsed

    def parse_xml_response(self, response):
        resp = fromstring(response.decode('utf-8'))
        notifications = [ntfc.text for ntfc in resp.findall('.//notification')]
        errors = [err.text for err in resp.findall('.//error')]
        parsed = {
            'notify': {
                'hasNotifyMessages': bool(notifications),
                'notifyMessages': notifications
            },
            'error': {
                'hasErrorMessages': bool(errors),
                'errorMessages': errors
            },
            'parameters': {
                'action': resp.find(".//parameter[@type='action']").text,
                'avoidAlerting': bool(resp.find(".//parameter[@type='avoidAlerting']").text)
            }
        }
        return parsed

    def parse_json_response(self, response):
        return loadjson(response.decode('utf-8'))

    def call(self, get_params = {}, post_params = {}):
        # make copies of the default parameters
        get, post = deepcopy(self._def_get), deepcopy(self._def_post)
        get.update(get_params)
        post.update(post_params)

        try: # Python2
            return urlopen(
                "%s?%s" % (self._base_url, urlencode(get)),
                urlencode(post),
                context=cuc()
            )
        except TypeError: # Python3
            return urlopen(
                "%s?%s" % (self._base_url, urlencode(get)),
                bytes(urlencode(post), 'utf-8'),
                context=cuc()
            )

    def call_and_parse(self, get_params = {}, post_params = {}):
        response = self.call(get_params, post_params)
        return self.parse_response(response.read(), get_params['style'])

    @repeat_with(_styles)
    def test_add_action(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'action': 'add'})
        # generate the expected response
        expected = deepcopy(self._def_parsed)
        expected['notify']['hasNotifyMessages'] = True
        expected['notify']['notifyMessages'] = [self._messages['add_action']]
        expected['parameters'] = {'action': 'add', 'avoidAlerting': False }
        # compare
        self.assertEqual(
            expected, parsed,
            msg="test_add_action for style %s failed" % style
        )

    @repeat_with(_styles)
    def test_stop_action(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'action': 'stop'})
        # generate the expected response
        expected = deepcopy(self._def_parsed)
        expected['notify']['hasNotifyMessages'] = True
        expected['notify']['notifyMessages'] = [self._messages['stop_action']]
        expected['parameters'] = {'action': 'stop', 'avoidAlerting': False }
        # compare
        self.assertEqual(
            expected, parsed,
            msg="test_stop_action for style %s failed" % style
        )

    @repeat_with(_styles)
    def test_wrong_action(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'action': 'not_a_valid_action'})
        # generathe the expected response
        expected = deepcopy(self._def_parsed)
        expected['error']['hasErrorMessages'] = True
        expected['error']['errorMessages'] = [self._messages['wrong_action'].replace('%%', 'not_a_valid_action')]
        expected['parameters'] = {'action': 'not_a_valid_action', 'avoidAlerting': False}
        # compare
        self.assertEqual(
            expected, parsed,
            msg="test_wrong_action for style %s failed" % style
        )

    @repeat_with(_styles)
    def test_wrong_host_id(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'hostId': -2})
        # generate the expected response
        expected = deepcopy(self._def_parsed)
        expected['error']['hasErrorMessages'] = True
        expected['error']['errorMessages'] = [
            self._messages['wrong_host_id'],
            self._messages['wrong_login']
        ]
        expected['parameters'] = {'action': 'add', 'avoidAlerting': False}
        # compare
        self.assertEqual(
            expected, parsed,
            msg="test_wrong_host_id for style %s failed" % style
        )

    @repeat_with(_styles)
    def test_wrong_login(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'password': 'not_a_valid_password'})
        # generate the expected response
        expected = deepcopy(self._def_parsed)
        expected['error']['hasErrorMessages'] = True
        expected['error']['errorMessages'] = [self._messages['wrong_login']]
        expected['parameters'] = {'action': 'add', 'avoidAlerting': False }
        self.assertEqual(
            expected, parsed,
            msg="test_wrong_login for style %s failed" % style
        )

    @repeat_with(_styles)
    def test_wrong_duration(self, style):
        # the parsed response
        parsed = self.call_and_parse({'style': style}, {'duration': -1})
        # generate the expected response
        expected = deepcopy(self._def_parsed)
        expected['error']['hasErrorMessages'] = True
        expected['error']['errorMessages'] = [self._messages['wrong_duration']]
        expected['parameters'] = {'action': 'add', 'avoidAlerting': False}
        self.assertEqual(
            expected, parsed,
            msg="test_wrong_duration for style %s failed" % style
        )

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(DowntimesAPITest)
    unittest.TextTestRunner(verbosity=2).run(suite)

