define([
    'jquery',
    'vis',
    'handlebars',
], function($, vis, Handlebars) { 'use_strict';

  var network
  var nodeObjects   = {}
  var selectedNodes = []

  function decorate(label, state) {
    if (state === 0) {
      return label + ' \u2714' // ✔
    }
    else if (state === 1) {
      return label + ' \u2707' // radioactive?
    }
    else if (state === 3) {
      return label + ' \u2731' // asterisk
    }
    else {
      return label + ' \u2718' // ✘
    }
  }

  function handleApplication(data) {

    var app         = data.parameters.application.data
    var nodeList    = []
    var edgeList    = []
    var nodeCounter = 0

    function addNodeObject(parentId, type, item) {
      nodeObjects[parentId] = item
    }

    function setNodesAndEdges(parentId, items) {
      if (items.length === 0) {
        return true
      }
      items.forEach(function(item) {

        var color = getColor(item.state, item.type)
        var componentNodeId = nodeCounter++
        nodeList.push({
          id:     componentNodeId,
          label:  decorate(item.name, item.state),
          shape:  'box',
          font:   color.foreground,
          color:  color.background,
        })
        edgeList.push({
          from:   parentId,
          to:     componentNodeId,
          color:  { inherit: 'to' },
        })
        nodeObjects[componentNodeId] = item

        item.hosts.forEach(function(item) {
          var color      = getColor(item.state, item.type)
          var hostNodeId = nodeCounter++
          nodeList.push({
            id:     hostNodeId,
            label:  'H: ' + decorate(item.name, item.state),
            shape:  'dot',
            size:   10,
            font:   color.foreground,
            color:  color.background,
          })
          edgeList.push({
            from:   componentNodeId,
            to:     hostNodeId,
            color:  { inherit: 'to' },
          })
          nodeObjects[hostNodeId] = item
        })

        item.services.forEach(function(item) {
          var color         = getColor(item.state, item.type)
          var serviceNodeId = nodeCounter++
          nodeList.push({
            id:     serviceNodeId,
            label:  'S: ' + decorate(
              item.host_name + '.' + item.name,
              item.state
            ),
            shape:  'dot',
            size:   10,
            font:   color.foreground,
            color:  color.background,
          })
          edgeList.push({
            from:       componentNodeId,
            to:         serviceNodeId,
            color:      { inherit: 'to' }
          })
          nodeObjects[serviceNodeId] = item
        })

        item.applications.forEach(function(item) {
          var color             = getColor(item.state, item.type)
          var applicationNodeId = nodeCounter++
          nodeList.push({
            id:     applicationNodeId,
            label:  decorate(
                item.customer_name + '.' + item.name,
                item.state
            ),
            shape:  'star',
            size:   10,
            font:   color.foreground,
            color:  color.background
          })
          edgeList.push({
            from:   componentNodeId,
            to:     applicationNodeId,
            color:  { inherit: 'to', }
          })
          nodeObjects[applicationNodeId] = item
        })

        setNodesAndEdges(componentNodeId, item.components)
      })
    }

    var color = getColor(app.state, 'root')
    nodeList.push({
      id:     'root',
      label:  decorate(app.name, app.state),
      font:   color.foreground,
      color:  color.background,
    })

    nodeObjects['root'] = app
    setNodesAndEdges('root', app.components)

    var nodes   = new vis.DataSet(nodeList)
    var edges   = new vis.DataSet(edgeList)
    var graph   = {
      nodes: nodes,
      edges: edges,
    }
    var options = {
      layout: {
        hierarchical: {
          enabled:          true,
          levelSeparation:  250,
          direction:        'LR',
          sortMethod:       'directed',
        },
      },
      interaction: {
        hover: true,
      },
      nodes: {
        borderWidthSelected: 2,
      },
      edges: {
        hoverWidth: 0,
      },
    }
    var container = document.getElementById('application_container')

    network = new vis.Network(container, graph, options)
    network.fit()
    registerEvents()
    network.selectNodes(['root'])
    $('#application_container').trigger('selectRoot')
  }

  function getColor(state, type) {
    var okColor = {
      background: {
        border: '#D6E9C6',
        background: '#DFF0D8',
        highlight: {
          border: '#3C763D',
          background: '#DFF0D8',
        },
        hover: {
          border: '#3C763D',
          background: '#DFF0D8',
        },
      },
      foreground: {
        color: '#3C763D',
      },
    }
    var warningColor = {
      background: {
        border: '#FAEBCC',
        background: '#FCF8E3',
        highlight: {
          border: '#8A6D3B',
          background: '#FCF8E3',
        },
        hover: {
          border: '#8A6D3B',
          background: '#FCF8E3',
        },
      },
      foreground: {
        color: '#8A6D3B',
      },
    }
    var errorColor = {
      background: {
        border: '#EBCCD1',
        background: '#F2DEDE',
        highlight: {
          border: '#A94442',
          background: '#F2DEDE',
        },
        hover: {
          border: '#A94442',
          background: '#F2DEDE',
        },
      },
      foreground: {
        color: '#A94442',
      },
    }
    var unknownColor = {
      background: {
        border: '#93a1a1',
        background: '#b5c3c3',
        highlight: {
          border: '#586e75',
          background: '#b5c3c3',
        },
        hover: {
          border: '#586e75',
          background: '#b5c3c3',
        },
      },
      foreground: {
        color: '#586e75',
      },
    }

    if (state === 0) {
      return okColor
    }
    else if (
      type == 'service' && state === 1 ||
      type == 'application' && state === 3
    ) {
      return warningColor
    }
    else if (
      type == 'service' && state === 3 ||
      type == 'host' && state === 3
    ) {
      return unknownColor
    }
    else {
      return errorColor
    }
  }

  function getStateClasses(state) {
    switch (state) {
      case 0:
        return 'fa fa-check text-success'
      case 1:
        return 'fa fa-warning text-warning'
      case 2:
        return 'fa fa-times text-danger'
      default:
        return 'fa fa-question text-warning'
      }
  }

  function registerHelpers() {
    Handlebars.registerHelper('stateClasses', function() {
      var state = parseInt(this.state)
      var classes = getStateClasses(state)
      return new Handlebars.SafeString(classes)
    })
    Handlebars.registerHelper('capitalize', function(str) {
      return str.charAt(0).toUpperCase() + str.slice(1)
    })
  }

  function getDetails(type) {
      if (type == 'component' || type == 'root') {
        return $('#component_details').html()
      }
      return $('#item_details').html()
  }

  function getButtons(type) {
      if (type == 'component') {
        return $('#component_buttons').html()
      }
      else if (type == 'host' || type == 'service' || type == 'application') {
        return $('#item_buttons').html()
      }
      else if (type == 'root') {
        return $('#root_buttons').html()
      }
  }

  function registerEvents() {

    function selectNode(params) {
      // Can only happen if more than one node is selected
      if (params.nodes.length !== 1) {
        initInformation()
        return
      }

      var node = params.nodes[0]

      if (!(node in nodeObjects)) {
        return
      }

      var nodeObject = nodeObjects[node]

      var source     = getDetails(nodeObject.type)
      var template   = Handlebars.compile(source)
      var html       = template(nodeObject)
      $('#component_information').html(html)

      source    = getButtons(nodeObject.type)
      template  = Handlebars.compile(source)
      html      = template(nodeObject)
      $('#buttons_container').html(html)

      selectedNodes = params.nodes
    }

    $('#application_container').on('selectRoot', function(e) {
      selectNode({
        'nodes': ['root']
      })
    });

    /*
     It would be fancy to preselect the component which is given by the response parameters
    $('#application_container').on('selectComponent', function(e, params) {
      selectNode({
        'nodes': [params.component]
      })
    });
    */

    network.on('selectNode', selectNode)
    network.on('deselectNode', function(params) {
      initInformation()
    })
    network.on('hoverNode', function(params) {
      $('#application_container').css('cursor','pointer')
    })
    network.on('blurNode', function(params) {
      $('#application_container').css('cursor','auto')
    })
  }

  function process(e, callback) {
    var applicationId = $(this).val()

    if (applicationId < 1) {

      return
    }

    $.getJSON(
      '/',
      {
        'page'   : 'applications',
        'action' : 'getApplication',
        'id'     : applicationId,
      },
      handleApplication
    )
  }

  function initInformation() {
    $('#component_information').html('')
    $('#buttons_container').html('')
  }

  initInformation()
  registerHelpers()

  $('#applicationId').on('change', function() {
    initInformation()
  })

  $('#reload-application').on('click', function() {
    $('#applicationId').trigger('change')
  })

  return process
})
