define([
    'jquery',
    'handlebars',
], function($, Handlebars) { 'use_strict';

  // ------->>>>>  Handlebars helpers
  // let's convert our seconds int into a human readable string
  // total_seconds should be an integer
  Handlebars.registerHelper('humantime', function(total_seconds) {
    results = []
    // days
    if (total_seconds >= 24*60*60) {
      results.push(Math.floor(total_seconds/(24*60*60)) + "d")
      total_seconds = total_seconds % (24*60*60)
    }
    // hours
    if (total_seconds >= 60*60) {
      results.push(Math.floor(total_seconds/(60*60)) + "h")
      total_seconds = total_seconds % 3600
    }
    // minutes
    if (total_seconds >= 60) {
      results.push(Math.floor(total_seconds/60) + "m")
      total_seconds = total_seconds % 60
    }
    // seconds
    if (total_seconds > 0) {
      results.push(total_seconds + "s")
    }
    // total_seconds was 0, then results is going to be empty,
    // if it's empty send '0s', else sende the computed results
    return results.length > 0 ? results.join(' ') : '0s'
  })

  // convert the datetime object into a human readable string
  Handlebars.registerHelper('datetimestr', function(datetime) {
    // check if the object sent is a datetime object and send it's formatted string
    // if datetime is no datetime object, just return an empty string
    if (datetime === undefined || !('year' in datetime)) {
      return ''
    }

    // Subtract 1 of the month because the month is 0 indexed when parsed from an object
    datetime.month -= 1
    return moment(datetime).format('YYYY-MM-DD HH:mm:ss')
  })

  // round to a certain amount of decimal places
  // to round to 3 decimal places just use the helper like this
  // {{ round value 3 }}
  Handlebars.registerHelper('round', function(value, places, options) {
    // handlebars passes an options object, check if enough arguments
    // were passed, if not, places has to be set to it's standard value
    // (ergo 2 decimal places)
    if (arguments.length < 3) {
      options = places
      places = 2
    }
    var factor = Math.pow(10,places)
    return Math.round(value*factor)/factor
  })

  // make the first letter a capital letter
  Handlebars.registerHelper('capitalize', function(str) {
    return str.charAt(0).toUpperCase() + str.slice(1)
  })
  // <<<<<-------  Handlebars helpers

  // used to store the ids of hosts and apps for api requests
  // these arrays are used by eventlisteners inside populateDialog
  // we're going to use these arrays when making getJSON calls
  // a good reason to put them outside the populateDialog
  // function scope
  var selectedHostIds = []
  var selectedAppIds  = []

  // used to store the time range for which to request reports
  // these values are updated by an eventlistener of daterangepicker
  // we're going to use these values when making getJSON calls
  // initialize the variables with the value preset by php
  var startDate = $('#dateRange').attr('value').split(' - ')[0]
  var endDate   = $('#dateRange').attr('value').split(' - ')[1]

  // this function handles functionality of the selection fields
  function populateDialog() {
    // arrays to store the selected hosts and applications:
    // when a host or an application is selected in a selection field,
    // we add it to this array in order to be able to remove empty optgroups
    var selectedHosts = []
    var selectedApps  = []
    // store the selectionfields somewhere (since we need their options for later)
    var hostsClone = $('#hostId').clone()
    var appsClone = $('#applicationId').clone()
    // start with empty selection fields. we want the user to select a customer first
    // in order to let chosenjs update the fields after removal, we need to trigger the
    // "chosen:updated" event on the selection fields
    $('#formCriteria select.grouped optgroup').remove()
    $('#formCriteria select.grouped').trigger("chosen:updated")
    // when an option group is selected, we have to update the host and application selection field
    $("#optgroups").on('change', function(e) {
      // get the label of the selected option
      var label = $(e.target)[0].selectedOptions[0].label
      // update the selection fieds according to the selected option group
      if ($('#hostId optgroup[label="' + label + '"]').length == 0) {
        $('#hostId').append(hostsClone.find('optgroup[label="' + label + '"]').clone())
      }
      if ($('#applicationId optgroup[label="' + label + '"]').length == 0) {
        $('#applicationId').append(appsClone.find('optgroup[label="' + label + '"]').clone())
      }
      // remove optgroups which have no selected items
      // TODO: if you don't like the scope where this function is defined, place it somewhere else
      // i've put it here for semantical reasons
      function remove_empty_optgroups(optgroup, selected) {
          if ($.inArray(optgroup.attr('label'), selected.concat([label])) == -1) {
            optgroup.remove()
          }
      }
      $('#hostId').find('optgroup').each(function() {
        remove_empty_optgroups($(this), selectedHosts)
      })
      $('#applicationId').find('optgroup').each(function() {
        remove_empty_optgroups($(this), selectedApps)
      })
      // once an option group was selected and the selection fields were updated,
      // trigger "chosen:updated" event on the selection fields to display these changes
      $("#formCriteria select.grouped").trigger("chosen:updated")
    })

    // when an item is selected/deselected, we have to add/remove it to the array of selected items
    function add_or_remove(e, params, selected, clones, ids) {
      // if an item is selected there's a 'selected' attribute/property/whatever in params
      // add the label of the option group to the list of selected groups and the id of the
      // selected option to the list of selected ids
      if ('selected' in params) {
        selected.push(
          clones.find('optgroup option[value="' + params.selected + '"]').parent().attr('label')
        )
        ids.push(params.selected)
      }
      // if an item is deselected there's a 'deselected' attribute/property/whatever in params
      // remove the groups with no selected options and the deselected option
      if ('deselected' in params) {
        var index = selected.indexOf(
          clones.find('optgroup option[value="' + params.deselected + '"]').parent().attr('label')
        )
        if (index > -1) {
          selected.splice(index, 1)
        }
        index = ids.indexOf(params.deselected)
        if (index > -1) {
          ids.splice(index, 1)
        }
        $("#optgroups").trigger('change')
      }
    }
    // apply add_or_remove on the hosts and applications selection fields
    $("#hostId").on('change', function(e, params) {
      add_or_remove(e, params, selectedHosts, hostsClone, selectedHostIds)
    })
    $("#applicationId").on('change', function(e, params) {
      add_or_remove(e, params, selectedApps, appsClone, selectedAppIds)
    })
  }

  // lists hosts, applications, alerts or downtimes
  function list_if_any(objs, objType, headerTemplate, bodyTemplate, callable=null) {
    if (objs && objs.length) {
      // add the table to display objects
      var html = headerTemplate({
        type: objType,
        isAlert: objType == 'alert',
        isDowntime: objType == 'downtime',
        isState: objType == 'state'
      })

      $("#" + objType + "Reports").removeClass('hide').append(html)
      $.each(objs, function(index, item) {
        var html = bodyTemplate($.extend(
            item,
            {
                isAlert: objType == 'alert',
                isDowntime: objType == 'downtime',
                isState: objType == 'state',
                isService: false
            }
        ))
        $("#" + objType + "Reports tbody").append(html)

        if (objType == 'host') {
          $.each(item.services, function(sid, service) {
            var html = bodyTemplate($.extend(
                service,
                {
                    isAlert: false,
                    isDowntime: false,
                    isState: false,
                    isService: true,
                    hostId: item.id
                }
            ))
            $("#hostReports tbody").append(html)
          })
        }
      })

      // add the action for the button's click event if there's one defined
      if (callable != null) {
        $("#" + objType + "Reports button.show-issues").click(callable)
      }

      if (objType == 'host') {
        $("#hostReports .show-more").click(toggleServices)
      }
    }
  }

  function toggleServices() {
    var services = $("." + $(this).context.dataset.showId)

    if (services.hasClass('hide')) {
      services.removeClass('hide')
    } else {
      services.addClass('hide')
    }
  }

  // generates incidents report from data which is sent by the api
  function handleIncidents(data) {
    var alerts           = data.parameters.incidents.data.alerts
    var downtimes        = data.parameters.incidents.data.downtimes
    var states           = data.parameters.incidents.data.states
    var headerTemplate   = Handlebars.compile($("#incident_header").html())
    var bodyTemplate     = Handlebars.compile($("#incident_details").html())
    list_if_any(alerts, "alert", headerTemplate, bodyTemplate)
    //list_if_any(downtimes, "downtime", headerTemplate, bodyTemplate)
    list_if_any(states, "state", headerTemplate, bodyTemplate)
  }

  // generates reports from data wich is sent by the api
  function handleReport(data) {
    // define the callable for the buttons
    // in this case, we want to display the incidents
    callable = function() {
      var id = $(this).attr('id').split(':')[1]
      var type = $(this).attr('id').split(':')[0]
      $("#alertReports").addClass('hide').html('')
      $("#downtimeReports").addClass('hide').html('')
      $("#stateReports").addClass('hide').html('')
      // request the incidents for the clicked host
      $.getJSON(
        '/',
        {
          'page'   : 'reports',
          'action' : 'getIncidents',
          'id'     : id,
          'type'   : type,
          'start'  : startDate,  // defined in the 'global' scope
          'end'    : endDate,    // defined in the 'global' scope
        },
        handleIncidents
      )
    }
    // define the callback used on the buttons of the lists
    var hosts             = data.parameters.facts.data.hosts
    var applications      = data.parameters.facts.data.applications
    var headerTemplate    = Handlebars.compile($("#fact_header").html())
    var bodyTemplate      = Handlebars.compile($("#fact_details").html())
    var headerAppTemplate = Handlebars.compile($("#fact_app_header").html())
    var bodyAppTemplate   = Handlebars.compile($("#fact_app_details").html())

    list_if_any(hosts, 'host', headerTemplate, bodyTemplate, callable)
    list_if_any(applications, 'application', headerAppTemplate, bodyAppTemplate, callable)
  }

  function printReport(data) {
    window.frames["print_report"].document.body.innerHTML = data.html
    window.frames["print_report"].window.focus()
    window.frames["print_report"].window.print()
    window.frames["print_report"].document.body.innerHTML = ''
  }

  // takes the form's data and sends a request to the frontend
  function sendForm() {
    // reset data fields
    $("#hostReports").html('')
    $("#applicationReports").html('')
    $("#alertReports").html('')
    $("#downtimeReports").html('')
    $("#stateReports").html('')
    // send the request for the selected hosts and applications
    $.getJSON(
      '/',
      {
        'page'           : 'reports',
        'action'         : 'getReport',
        'hostIds'        : selectedHostIds,  // defined in "global" scope
        'applicationIds' : selectedAppIds,   // defined in "global" scope
        'start'          : startDate,        // defined in "global" scope
        'end'            : endDate,          // defined in "global" scope
      },
      handleReport
    )
    // we don't want to send the form, we need to return false
    return false
  }

  function sendPrintReport() {
    $.getJSON(
      '/',
      {
        'page'           : 'reports',
        'action'         : 'getPrintView',
        'hostIds'        : selectedHostIds,  // defined in "global" scope
        'applicationIds' : selectedAppIds,   // defined in "global" scope
        'start'          : startDate,        // defined in "global" scope
        'end'            : endDate,          // defined in "global" scope
      },
      printReport
    )

    return false
  }

  function updateDates(event, picker) {
    startDate = picker.startDate.format('YYYY-MM-DD')
    endDate   = picker.endDate.format('YYYY-MM-DD')
    event.target.value = `${startDate} - ${endDate}`
  }

  // register events for the form
  function registerEvents() {
    // what to do when the form is tried to be sent
    $("#formCriteria #submit-btn").click(sendForm)
    $("#formCriteria #print-report-btn").click(sendPrintReport)

    // update startDate and endDate when a date was picked
    $("#formCriteria .daterangePicker").on('apply.daterangepicker', updateDates)
    $("#formCriteria .daterangePicker").on('hide.daterangepicker', updateDates)
  }

  registerEvents()
  populateDialog()

})
