<?php

namespace SyRPCServer;

use SyRPC;

use ch\monitoring\datamapper;
use ch\monitoring\model;
use ch\monitoring\util;
use ch\monitoring\Application;
use org\adfinis\framework\config\ConfManager;

/**
 * Handles icinga configuration writing, called by RPC
 */
class Api {
	/**
	 * Specifies the name of the RPC action to write icinga configs
	 *
	 * @const string REQUEST_TYPE_WRITE_CONFIG
	 */
	const REQUEST_TYPE_WRITE_CONFIG = 'writeIcingaConfig';
	/**
	 * Specifies the name of the RPC action to get the current timeout
	 *
	 * @const string REQUEST_TYPE_GET_TIMEOUT
	 */
	const REQUEST_TYPE_GET_TIMEOUT  = 'getTimeout';

	/**
	 * Used to identify which kind of icinga config write should be made
	 */
	const KIND_MASTER_CONFIG    = 'master';
	const KIND_SATELLITE_CONFIG = 'satellite';
	const KIND_NRPE_CONFIG      = 'NRPE';

	/**
	 * Holds the persistent database setting name of the last reload date
	 *
	 * @const string RELOAD_CONFIG_SETTING
	 */
	const RELOAD_CONFIG_SETTING = 'lastIcingaConfigReload';

	/**
	 * Holds the persistent database setting name of the last requested reload
	 * date
	 *
	 * @const string RELOAD_REQUEST_CONFIG_SETTING
	 */
	const RELOAD_REQUEST_CONFIG_SETTING = 'lastIcingaConfigReloadRequest';

	/**
	 * Specifies if and which configs should be written
	 */
	private static $writeMasterConfig    = false;
	private static $writeSatelliteConfig = array();
	private static $writeNRPEConfig      = array();

	/**
	 * Holds the cached date of the last reload
	 */
	private static $lastReloadDate = null;

	/**
	 * Holds the cached date of the last requested reload
	 */
	private static $lastReloadRequestDate = null;

	/**
	 * Gets a list of valid config write kinds
	 *
	 * @return string[]
	 */
	private static function getValidKinds() {
		return [
			static::KIND_MASTER_CONFIG,
			static::KIND_SATELLITE_CONFIG,
			static::KIND_NRPE_CONFIG
		];
	}

	/**
	 * Get the last reload date (caches the result from the database)
	 *
	 * @return string
	 */
	public static function getLastReloadDate() {
		if (!isset(static::$lastReloadDate)) {
			static::$lastReloadDate = util\Setting::get(static::RELOAD_CONFIG_SETTING, false);
		}
		return static::$lastReloadDate;
	}

	/**
	 * Sets the reload date to the current date
	 *
	 * @return void
	 */
	public static function updateLastReloadDate() {
		$now = date('c');
		util\Setting::set(static::RELOAD_CONFIG_SETTING, $now);
		static::$lastReloadDate = $now;
	}

	/**
	 * Gets the date for the last requested relaod
	 *
	 * @return string
	 */
	public static function getLastReloadRequestDate() {
		return static::$lastReloadRequestDate;
	}

	/**
	 * Sets the date for the last requested relaod to now
	 *
	 * @return void
	 */
	public static function updateLastReloadRequestDate() {
		$now = date('c');
		util\Setting::set(static::RELOAD_REQUEST_CONFIG_SETTING, $now);
		static::$lastReloadRequestDate = $now;
	}

	/**
	 * Purges the date for the last requested reload
	 *
	 * @return void
	 */
	public static function purgeLastReloadRequestDate() {
		util\Setting::set(static::RELOAD_REQUEST_CONFIG_SETTING, null);
		static::$lastReloadRequestDate = null;
	}

	/**
	 * Registers a new config write request of given kind
	 *
	 * @param  string $kind
	 * @param  object $data
	 * @return void
	 */
	private static function addWriteRequest($kind, $data) {
		switch ($kind) {
			case static::KIND_MASTER_CONFIG:
				static::$writeMasterConfig = true;
				return; // No more checks are needed for master config
			case static::KIND_SATELLITE_CONFIG:
				$list = &static::$writeSatelliteConfig;
				break;
			case static::KIND_NRPE_CONFIG:
				$list = &static::$writeNRPEConfig;
				break;
		}
		if ($list === true || !isset($data->IDs)) {
			return $list = true;
		}
		if (is_array($list) && count($list) > 0) {
			$list = array_unique(array_merge($list, $data->IDs));
		}
		else {
			$list = $data->IDs;
		}
	}

	/**
	 * Calculate the time delta since last reload
	 *
	 * @return int
	 */
	private static function timeToNextReload() {
		$lastReloadDate = static::getLastReloadDate();
		if ($lastReloadDate) {
			$lastReloadDate    = strtotime($lastReloadDate);
			$difference        = time() - $lastReloadDate;
				$configuredTimeout = ConfManager::getVar('af', 'icinga', 'config', 'writeTimeout');
			if ($difference < $configuredTimeout) {
				return $configuredTimeout - $difference;
			}
		}
		return 0;
	}

	/**
	 * Calculate the time delta since last reload request
	 *
	 * @return int
	 */
	private static function timeDeltaSinceLastReloadRequest() {
		$lastReloadRequestDate = static::getLastReloadRequestDate();
		if ($lastReloadRequestDate) {
			$lastReloadRequestDate = strtotime($lastReloadRequestDate);
			$difference            = time() - $lastReloadRequestDate;
			$configuredTimeout     = ConfManager::getVar('af', 'icinga', 'config', 'writeTimeout');
			if ($difference < $configuredTimeout) {
				return $configuredTimeout - $difference;
			}
		}
		return 0;
	}

	/**
	 * Writes Icinga config files according to current database state
	 *
	 * The current config state is taken from database and verified. If the
	 * verification fails, an error is returned.
	 * 
	 * If the verification is ok, the new configuration gets written
	 * immediately. Then a request for reloading is registered. The reload
	 * gets then performed after the configured time.
	 *
	 * To write all registered write requests, but without registering new write requests,
	 * call this function without the data parameter
	 *
	 * @param  object $data
	 * @return void
	 */
	public static function writeIcingaConfig($data = null) {
		if ($data === null) {
			$data = new \stdClass();
		}
		if (isset($data->kind)) {
			$kind = $data->kind;
			if (!in_array($kind, static::getValidKinds())) {
				throw new \InvalidArgumentException(tr('Wrong config write kind was specified'));
			}
			static::addWriteRequest($kind, $data);
		}
		static::writeMasterConfig(true);
		list($verifyResult, $verifyStdout) = util\Monitoring::verify(true);
		if (!$verifyResult) {
			$data->hasInvalidConfiguration = true;
			$data->verifyResult            = $verifyResult;
			$data->verifyStdout            = $verifyStdout;
		}
		else {
			static::writeMasterConfig();
			static::writeSatelliteConfig($data);
			static::writeNRPEConfig($data);
			static::updateLastReloadRequestDate();
			$timeToNextReload = static::timeToNextReload();
			if ($timeToNextReload === 0) {
				SyRPC\Common::$lg->addDebug(sprintf(
					"%s: Reloading Icinga",
					__METHOD__
				));
				$reload = util\Monitoring::reload();
				static::updateLastReloadDate();
				static::purgeLastReloadRequestDate();
			}
			$data->timeout = $timeToNextReload;
			$data->hasInvalidConfiguration = false;
		}
	}

  /**
   * Enables all config writes.
   */
  public static function enableAllWrites() {
    static::$writeMasterConfig = true;
    static::$writeSatelliteConfig = true;
    static::$writeNRPEConfig = true;
  }

	/**
	 * Returns the time delta since the last request for reloading Icinga
	 *
	 * @param  object $data
	 * @return int $timeout  the time delta since the last reload request
	 */
	public static function getTimeout($data = null) {
		if ($data === null) {
			$data = new \stdClass();
		}
		if (static::$lastReloadRequestDate) {
			$data->timeout = static::timeToNextReload();
		}
		else {
			$data->timeout = 0;
		}
		return $data;
	}

	/**
	 * Used to write the icinga master configuration
	 *
	 * @return bool
	 */
	private static function writeMasterConfig($writeCheckOnly=false) {
		if (!static::$writeMasterConfig) {
			SyRPC\Common::$lg->addDebug(sprintf(
				"%s: Not writing master config, something went wrong",
				__METHOD__
			));
			return false;
		}
		$monitoringExport = new util\ConfigExport($writeCheckOnly);
		$monitoringExport->disablePermissionChecks();
		$monitoringExport->buildMaster();

		if (!$writeCheckOnly) {
			static::$writeMasterConfig = false;
		}

		return true;
	}

	/**
	 * Used to write icinga satellite configurations
	 *
	 * @param  object $data
	 * @return bool
	 */
	private static function writeSatelliteConfig($data) {
		if (!static::$writeSatelliteConfig) {
			return false;
		}

		$monitoringExport = new util\ConfigExport;
		$monitoringExport->disablePermissionChecks();

		if (is_array(static::$writeSatelliteConfig)) {
			foreach (static::$writeSatelliteConfig as $currentID) {
				if (empty($currentID)) {
					continue;
				}

				$satellite = datamapper\HostMapper::getInstance()->getByID(
					$currentID
				);
				$satellite->allowRead();

				$monitoringExport->buildSatellite($satellite, null);
			}
		}
		else {
			$satellites = datamapper\HostMapper::getInstance()->getAll(
				false,
				false,
				true,
				null,
				isset($data->customerID) ? $data->customerID : null,
				null,
				true
			);

			foreach ($satellites as $satellite) {
				$satellite->allowRead();
				$monitoringExport->buildSatellite($satellite);
			}
		}

		static::$writeSatelliteConfig = [];
		return true;
	}

	/**
	 * Used to write nrpe configuration
	 *
	 * @param  object $data
	 * @return bool
	 */
	private static function writeNRPeConfig($data) {
		if (!static::$writeNRPEConfig) {
			return false;
		}

		$nrpeExport = new util\ConfigExport();
		$nrpeExport->disablePermissionChecks();

		if (is_array(static::$writeNRPEConfig)) {
			foreach (static::$writeNRPEConfig as $currentID) {
				try {
					$host = datamapper\HostMapper::getInstance()->getByID($currentID);

					$host->allowRead();
					$nrpeExport->buildNrpe($host);
				}
				catch (\OutOfBoundsException $e) {
					throw new UserException(
						tr('Host not found')
					);
				}
			}
		}
		else {
			$hosts = datamapper\HostMapper::getInstance()->getAll(
				false,
				true,
				false,
				null,
				isset($data->customerID) ? $data->customerID : null,
				null,
				true
			);

			foreach ($hosts as $host) {
				$nrpeExport->buildNrpe($host);
			}
		}

		static::$writeNRPEConfig = [];
		return true;
	}
}
