<?php

namespace SyRPCServer;

use SyRPC;
use Monolog\Logger;
use org\adfinis\framework\config\ConfManager;

// Base RPC handler class
abstract class Base {
	/**
	 * Holds the settings for this RPC instance
	 *
	 * @var mixed[] $settings
	 */
	public $settings;

	/**
	 * Holds the parameters and results passed between the client,
	 * server and the API
	 *
	 * @var mixed[] $data
	 */
	public $data = array();

	/**
	 * Specifies how long the get request should wait for an answer
	 *
	 * @var int $timeout
	 */
	public $timeout = 5;

	/**
	 * Initializes the RPC instance
	 */
	public function __construct() {
		$this->initSettings();
	}

	/**
	 * Initializes the settings for this RPC instance
	 *
	 * @return mixed[] $settings
	 */
	protected function initSettings() {
		$globalSettings = ConfManager::getVar('sm', 'RPC');
		$settings = array();
		$settings['app_name']        = $globalSettings['wqAppName'];
		$settings['amq_virtualhost'] = $globalSettings['amq']['virtualhost'];
		$settings['amq_host']        = $globalSettings['amq']['host'];
		$settings['amq_user']        = $globalSettings['amq']['user'];
		$settings['amq_password']    = $globalSettings['amq']['password'];
		$settings['amq_num_queues']  = $globalSettings['wqQueues'];
		return $this->settings = $settings;
	}

	/**
	 * Runs a request listening and handles found request
	 *
	 * @param  Server $instance
	 * @param  int    $timeout
	 * @return void
	 */
	protected abstract function runner($instance, $timeout);

	/**
	 * This is called after a timeout occured.
	 *
	 * @return void
	 */
	protected function cleanupAfterTimeout() {
	}

	/**
	 * Used to run the RPC instance
	 *
	 * @param  SyRPC\RPCBase $instance
	 * @param  bool       $runnerMode Specifies if only one request should be handled
	 *                                or if it should listen forever for new requests
	 * @return void
	 */
	public function run($instance, $runnerMode = false) {
		if ($runnerMode) {
			while (true) {
				try {
					$this->runner($instance, $this->timeout);
				}
				catch (\PhpAmqpLib\Exception\AMQPTimeoutException $e) {
					$this->cleanupAfterTimeout();
					continue;
				}
			}
		}
		else {
			$this->runner($instance, $this->timeout);
		}
	}
}
