<?php

namespace SyRPCServer;

use SyRPC;
use Monolog\Logger;
use org\adfinis\framework\datamapper;
use org\adfinis\framework\db\DbManager;
use org\adfinis\framework\config\ConfManager;

/**
 * Handles icinga configuration RPC write requests
 */
class Server extends Base {

	/**
	 * Create a new RPC server to write icinga configuration
	 */
	public function __construct() {
		$this->resetTimeout();
		parent::__construct();
	}

	/**
	 * Resets the timeout to the standard configured value
	 *
	 * @return void
	 */
	public function resetTimeout() {
		$this->timeout = ConfManager::getVar('af', 'icinga', 'config', 'writeTimeout');
	}

	/**
	 * Runs a request listening and handles found request
	 *
	 * Calls methods from the API class
	 *
	 * @param  Server $instance
	 * @param  int    $timeout
	 * @return void
	 */
	protected function runner($instance, $timeout) {
		SyRPC\Common::$lg->addDebug("Server trying to get request");
		list($type, $resultID, $data) = $instance->getRequest($timeout);
		DbManager::disconnect();
		$context = datamapper\IdentityMap::getContext();
		try {
			if ($resultID) {
				switch ($type) {
					case Api::REQUEST_TYPE_WRITE_CONFIG:
						if (empty($data)) {
							$data = new \stdClass();
						}
						Api::writeIcingaConfig($data);
						// Check if there is a icinga writing timeout
						if ($data->timeout) {
							// Set the request listening timeout to given value
							$this->timeout = $data->timeout;
						}
						break;
					case Api::REQUEST_TYPE_GET_TIMEOUT:
						$data = Api::getTimeout();
						$this->timeout = $data->timeout;
						break;
					default:
						$message = sprintf('The given API call "%s" is invalid', $type);
						SyRPC\Common::$lg->addError($message);
						throw new \InvalidArgumentException($message);
				}
				$result = array(
					'timeout'                 => $this->timeout,
					'hasInvalidConfiguration' => $data->hasInvalidConfiguration,
					'verifyResult'            => $data->verifyResult,
					'verifyStdout'            => $data->verifyStdout,
				);
				SyRPC\Common::$lg->addDebug(sprintf(
					"Server got result %s: %s", $resultID, var_export($result, true)
				));
				$instance->putResult($resultID, $result);
				SyRPC\Common::$lg->addDebug("Server put result for request {$resultID}");
			}
			else {
				SyRPC\Common::$lg->addDebug("Server got no result within timeout {$timeout}s");
			}
		} catch (\Exception $e) {
			$msg = $e->getMessage() . "\n" . $e->getTraceAsString();
			SyRPC\Common::$lg->addError(sprintf("Server got an exception: %s", $msg));
			$instance->putResult($resultID, array('exception' => $msg));
		}
	}

	/**
	 * If there is an timeout, try to write pending configuration changes
	 * and reset the timeout timer
	 *
	 * @return void
	 */
	protected function cleanupAfterTimeout() {
		SyRPC\Common::$lg->addDebug("Running cleanup after timeout");
		Api::writeIcingaConfig();
		$this->resetTimeout();
	}

	/**
	 * Used to run the server
	 *
	 * @param  bool $runnerMode Specifies if only one request should be handled
	 *                          or if it should listen forever for new requests
	 * @return void
	 */
	public function run($runnerMode = false) {
		global $argv;

		\SyRPC\Common::$lg = \SyRPC\Common::setupLogger(
			'symonitoring-php-rpc-server',
			(int) ConfManager::getVar('sm', 'logging', 'level')
		);
		SyRPC\Common::$lg->addDebug(
			sprintf("Starting server in runner mode: %s", ($runnerMode === true))
		);
		$settings = $this->settings;
		$server = SyRPC\Server::getInstance($settings);
		if($argv[1] !== '--write-config') {
			parent::run($server, $runnerMode);
		}
		else {
			Api::enableAllWrites();
			Api::writeIcingaConfig();
		}
	}
}

$server = new Server();
$server->run(true);
