<?php

namespace #NAMESPACE#\presentation;

use \org\adfinis\framework\presentation\Page;
use \org\adfinis\framework\config\ConfManager;

/**
 * Base page for HTML output.
 *
 * This class contains common code for all page classes in this project.
 *
 * Also this class implements some special features, so you may call addNotifyMessage()
 * even before a redirect - the message will then be displayed upon the next page view.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class BasePage extends Page {

	/**
	 * Error message storage.
	 *
	 * @var           array<string> $errorMessages
	 */
	private $errorMessages = array();

	/**
	 * Notification message storage.
	 *
	 * @var           array<string> $notifyMessages
	 */
	private $notifyMessages = array();

	/**
	 * Constructor
	 *
	 * Passes all parameters to parent's constructor, then goes on to
	 * assign default variables to the template object.
	 *
	 * This also loads stored messages from the session
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        UnexpectedValueException
	 * @param         bool   $allowSecure      (optional, defaults to false)
	 * @param         bool   $needSecure       (optional, defaults to false)
	 * @param         string $domainIdentifier (optional, defaults to 'default')
	 */
	public function __construct($allowSecure = false, $needSecure = false, $domainIdentifier = 'default') {
		parent::__construct($allowSecure, $needSecure, $domainIdentifier);

		$this->loadMessagesFromSession();
		$this->assignDefaultTemplateVariables();
		$this->initLanguage();

	}

	/**
	 * Initializes translation subsystem
	 *
	 * Sets the default language.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   Note: This is only an example. Implement language selection according to your project!
	 * @returns       void
	 */
	private function initLanguage() {
		\org\adfinis\framework\lang\Language::init('de_CH');
	}
	

	/**
	 * Assigns default template variables to the smarty object ($this->tpl)
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	protected function assignDefaultTemplateVariables() {

		// jQuery
		$this->addScriptFile('jslib/jQuery/jquery.js', self::SCRIPT_POSITION_TOP);

		// Translation helper
		$this->addScriptFile('framework-utils/js-translate.php', self::SCRIPT_POSITION_TOP);

		// Our stuff
		$this->addLinkWithSourcePath  ('themes/default/css/layout.css');
	}

	/**
	 * Overload oaf\presentation\Page::outputPage().
	 *
	 * This method also adds the following variables to the template:
	 * - headerTitle, as set by the setHeaderTitle() method
	 * - metaFields, as set by the addMetaData() method
	 * - linkFields, as set by the addLinkWithSourcePath() method
	 * - topScripts & bottomScripts, as set by the addScriptFile()
	 *              and addScriptCode() methods
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $template
	 * @returns       void
	 */
	public function outputPage($template = 'gui/layout.tpl') {

		$this->tpl->assign('headerTitle',   $this->headerTitle);
		$this->tpl->assign('metaFields',    $this->getMetaTags());
		$this->tpl->assign('linkFields',    $this->getLinkTags());
		$this->tpl->assign('topScripts',    $this->getScriptTags(Page::SCRIPT_POSITION_TOP));
		$this->tpl->assign('bottomScripts', $this->getScriptTags(Page::SCRIPT_POSITION_BOTTOM));

		$this->tpl->assign('hasErrorMessages', (bool)sizeof($this->errorMessages));
		$this->tpl->assign('errorMessages',    $this->errorMessages);

		$this->tpl->assign('hasNotifyMessages', (bool)sizeof($this->notifyMessages));
		$this->tpl->assign('notifyMessages',    $this->notifyMessages);

		$this->tpl->display($template);
	}

	/**
	 * Sets the header title of the current page.
	 *
	 * Adds the project name in front of the title, too.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws
	 * @param         string $title
	 * @returns       void
	 */
	public function setHeaderTitle($title) {
		parent::setHeaderTitle(
			sprintf(
				'%s | %s',
				ConfManager::getVar('af', 'projectName'),
				$title
			)
		);
	}

	/**
	 * Adds the given message to the error messages.
	 *
	 * The given message is added to a list, which is then passed to the template when outputPage() is called.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $message
	 * @returns       void
	 */
	public function addErrorMessage($message) {
		$this->errorMessages[] = $message;
	}

	/**
	 * Adds the given message to the notification messages.
	 *
	 * The given message is added to a list, which is then passed to the template when outputPage() is called.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $message
	 * @returns       void
	 */
	public function addNotifyMessage($message) {
		$this->notifyMessages[] = $message;
	}

	/**
	 * Redirects the user to another URL.
	 *
	 * In addition to the URL, any notification or error message is temporarily stored in the session.

	 * The second parameter defines, if the page is permanently moved to a new
	 * location. In this case a permanently (301) header will be sent instead
	 * of a 302 (moved temporarly) header.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $URL
	 * @param         bool   $permanent (optional, default=false)
	 * @returns       void
	 */
	public function redirectToURL($url, $permanent = false) {

		// save messages in session
		$this->saveMessagesToSession();

		parent::redirectToURL($url, $permanent);
	}

	/**
	 * Saves messages to the session
	 *
	 * This is used before a redirect, so messages can be displayed on the next page.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	private function saveMessagesToSession() {
		$this->session->basepageStoredMessages = serialize(array(
			'errorMessages'  => $this->errorMessages,
			'notifyMessages' => $this->notifyMessages
		));
	}

	/**
	 * Loads messages from session
	 *
	 * This is called in the constructor and loads messages from the session
	 * that have been put there before a redirect.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	private function loadMessagesFromSession() {
		if(isset($this->session->basepageStoredMessages)) {
			$data = unserialize($this->session->basepageStoredMessages);
			$this->errorMessages  = $data['errorMessages'];
			$this->notifyMessages = $data['notifyMessages'];

			unset($this->session->basepageStoredMessages);
		}
	}

	/**
	 * Adds instance edit scripts to the template.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         int $instanceID
	 * @returns       void
	 */
	public function addInstanceEditScripts($instanceID) {

		$this->addScriptFile('/javascript/instancestatus.js');
		$this->addScriptFile('/javascript/edit.instance.js');

		// This protects the forms, so they can't be submitted if the
		// instance is not stopped. This is required because LibVirt
		// Wouldn't accept changes if the instance is still running.
		$this->addScriptCode(
			sprintf(
				'jQuery(function(){jQuery(".content form").instanceEditProtection(%d);});',
				$instanceID
			)
		);
	}
}
