<?php

namespace org\adfinis\framework\config;

/**
 * Registry class helps to store information that is used throughout the project.
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
 */
class Registry {

	/**
	 * Access key for the request object.
	 *
	 * @const string REQUEST
	 */
	const REQUEST = 'request';

	/**
	 * Stores the registered information.
	 * 
	 * @var array $data
	 */
	private static $data = array();

	/**
	 * Stores a request object.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       org\adfinis\framework\request\Request $request
	 * @return      void
	 */
	public static function setRequest($request) {
		self::set(self::REQUEST, $request);
	}

	/**
	 * Retrieves the stored request object.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      org\adfinis\framework\request\Request $request
	 */
	public static function getRequest() {
		self::get(self::REQUEST);
	}

	/**
	 * Set an object in the registry.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      InvalidArgumentException if the given object key is not known.
	 * @param       string $key
	 * @param       mixed  $value
	 * @return      void
	 */
	protected static function set($key, $value){
		self::validateKey($key);
		self::$data[$key] = $value;
	}

	/**
	 * Get a stored object from the registry.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      OutOfRangeException if the requested object is not registered.
	 * @throws      InvalidArgumentException if the given object key is not known.
	 * @param       string $key
	 * @return      mixed
	 */
	protected static function get($key){
		self::validateKey($key);

		if(!isset(self::$data[$key])) {
			throw new \OutOfRangeException(
					sprintf(tr('Object with key "%s" was not stored'), $key)
			);
		}
		return self::$data[$key];
	}

	/**
	 * Little helper to validate key access.
	 *
	 * Throws an InvalidArgumentException if the key is not allowed.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      InvalidArgumentException
	 * @param       string $key
	 * @return      void
	 */
	private static function validateKey($key) {
		// Fall-through validation of input
		switch($key) {
			case REQUEST:
				break;
			default:
				throw new \InvalidArgumentException(
					sprintf(tr('Registry key "%s" unknown'), $key)
				);
		}
	}
}
