<?php

namespace org\adfinis\framework\controller;

use org\adfinis\framework\presentation\Page;

/**
 * This class provides advanced request handling based on an asynchronous approach.
 *
 * You can build your page in the usual way, by implementing handleGET() and handlePOST().
 * Instead of handleAJAX(), however, you can use some nifty magic:
 * Once the page is rendered in the browser, communication with this controller is
 * done using so called "messages". A message consists of an identifier and an optional payload.
 *
 * To allow receiving messages from the client, you can register a method using registerServerMessage():
 * @code
 *    function handleFormData($data) {
 *        ...
 *    }
 *    function __construct(...) {
 *        ...
 *        $this->registerServerMessage('formChanged', 'handleFormData');
 *    }
 * @endcode
 *
 * This controller automatically injects some javascript into the rendered page, so you can
 * then activate the above message handler by using the following javascript client-side:
 * @code
 *    $.serverMessage('formChanged', {'foo' : 'bar', ...});
 * @endcode
 * 
 * Instead of handling the result from the serverMessage directly, the same concept 
 * of the server side also applies to the client side.
 * When handling a request (be it GET, POST, anything...), you can call $this->clientMessage()
 * and the given message will be sent to the client:
 *
 * @code
 *    function handleFormData($data) {
 *        $this->clientMessage('formOK');
 *    }
 * @endcode
 *
 * Of course, you need to register the message handler in Javascript before using it:
 *
 * @code
 *    $(function() {
 *        $.registerClientMessage(identifier, function(payload) {
 *            ...
 *        });
 *    });
 * @endcode
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @todo by dv:   Javascript functionality
 * @todo by dv:   Message handler registry
 * @todo by dv:   Facility for sending messages to client
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
abstract class AsyncController extends BaseController implements ControllerInterface {

	/**
	 * Stores the message handlers
	 *
	 * @var           array $serverMessageHandlers
	 */
	private $serverMessageHandlers;

	/**
	 * Stores client messages until the request is done
	 *
	 * @var           array $clientMessages
	 */
	private $clientMessages = array();

	/**
	 * Handles the request when it's an XMLHTTP request (ie. AJAX)
	 *
	 * Don't overload this if you want AsyncController functionality!
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function handleAJAX() {
		// registered message handler.
		$identifier = $this->getRequest()->getString('serverMessage');
		$payload    = $this->getRequest()->getRaw   ('payload');

		if(isset($this->serverMessageHandlers[$identifier])) {
			foreach($this->serverMessageHandlers[$identifier] as $callback) {
				if(is_string($callback)) {
					$this->$callback($payload);
				}
				elseif (is_array($callback)) {
					call_user_func($callback, $payload);
				}
				elseif(is_callable($callback)) {
					$callback($payload);
				}
			}
		}

		header('Content-Type: application/json');
		echo(json_encode($this->clientMessages));
		exit();
	}

	/**
	 * Call this method to send messages to the user agent
	 *
	 * The client message contains of an identifier and a payload.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $identifier
	 * @param         mixed  $payload (optional)
	 * @returns       void
	 */
	public function clientMessage($identifier, $payload = null) {
		$message = array(
			'identifier' => $identifier,
			'payload'    => $payload
		);
		$this->clientMessages[] = $message;
	}

	/**
	 * Registers a method for handling the given server message.
	 *
	 * The given callback may be either a string, in which case it's assumed to
	 * be a method name on the current object. It may also be a closure object,
	 * or a "traditional" callback (array consisting of an object and a method name).
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $identifier
	 * @param         mixed  $callback
	 * @returns       void
	 */
	public function registerServerMessage($identifier, $callback) {
		// TODO: only allow callbacks that we can handle

		if(isset($this->serverMessageHandlers[$identifier])) {
			$this->serverMessageHandlers[$identifier][] = $callback;
		}
		else {
			$this->serverMessageHandlers[$identifier] = array($callback);
		}
	}

	/**
	 * Handles the request when it's of type POST
	 *
	 * Don't overload this, we're doing async stuff here
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function handlePOST() {
		// TODO: throw exception, kill user, take over the world
	}

	/**
	 * Outputs page.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $tpl
	 * @returns       void
	 */
	public function outputPage($tpl='') {
		$this->tpl->display($tpl);
	}
	
}
