<?php

namespace org\adfinis\framework\controller;

use org\adfinis\framework\presentation\Page;
use org\adfinis\framework\request;
use org\adfinis\framework\exceptions;

/**
 * Abstract class that you can use as a base to implement object-oriented controller classes.
 *
 * Instead of implementing the (very basic) handle() method yourself, you can implement
 * your stuff in the handleGET(), handlePOST() and handleAJAX() methods instead.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
abstract class BaseController extends Page implements ControllerInterface {

	/**
	 * Request object cache
	 *
	 * @var           org\adfinis\framework\request\RequestInterface $request
	 */
	protected $request;

	/**
	 * URL regexp match
	 *
	 * @var           string $urlMatch
	 */
	 protected $urlMatch;

	/**
	 * Page name
	 *
	 * @var           string $page
	 */
	 protected $page;

	/**
	 * Constructor - initializes the controller
	 *
	 * Note that the $urlMatch array is not handled. If you need 
	 * something out of it, write your own constructor.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         org\adfinis\framework\request\RequestInterface $request
	 * @param         array                                          $urlMatch
	 */
	public function __construct($request, $urlMatch) {
		parent::__construct($this->sslAllowed(), $this->sslRequired(), $this->getDomainIdentifier());
		$this->request = $request;
		$this->urlMatch = $urlMatch;
		$this->page = $this->request->getString('page', request\REQUEST_GET);
	}

	/**
	 * Returns true if the current controller allows SSL to be enabled (but it's not
	 * really required).
	 *
	 * If you want to enforce SSL, override sslRequired() and return true from there.
	 *
	 * Note however that this is called during the construction phase, so
	 * the controller object is NOT FULLY INITIALIZED AT THIS POINT!
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       boolean
	 */
	protected function sslAllowed() {
		return true;
	}

	/**
	 * Returns true if the current controller requires SSL to be enabled.
	 *
	 * Note however that this is called during the construction phase, so
	 * the controller object is NOT FULLY INITIALIZED AT THIS POINT!
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       boolean
	 */
	protected function sslRequired() {
		return true;
	}

	/**
	 * Returns the domain identifier.
	 *
	 * If you need your controller to return something different, you can
	 * override this method in your controller.
	 *
	 * Note however that this is called during the construction phase, so
	 * the controller object is NOT FULLY INITIALIZED AT THIS POINT!
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	protected function getDomainIdentifier() {
		return 'default';
	}

	/**
	 * Returns the request object.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       org\adfinis\framework\request\RequestInterface
	 */
	public function getRequest() {
		return $this->request;
	}

	/**
	 * Returns the page name
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getPage() {
		return $this->page;
	}

	/**
	 * Handles the request.
	 *
	 * Depending on request type, calls handleGET(), handlePOST() or handleAJAX().
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function handle() {
		if($this->request->isAJAX()) {
			return $this->handleAJAX();
		}
		elseif($this->request->isPOST()) {
			return $this->handlePOST();
		}
		return $this->handleGET();
	}

	/**
	 * Handles the request when it's of type GET
	 *
	 * This should be implemented by the "lowest" level class.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   create response classes, return these.
	 * @returns       void
	 */
	public abstract function handleGET();

	/**
	 * Handles the request when it's of type POST
	 *
	 * This should be implemented by the "lowest" level class.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   create response classes, return these.
	 * @returns       void
	 */
	public abstract function handlePOST();
	
	/**
	 * Handles the request when it's an XMLHTTP request (ie. AJAX)
	 *
	 * This should be implemented by the "lowest" level class.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   create response classes, return these.
	 * @returns       void
	 */
	public abstract function handleAJAX();
}
