<?php

namespace org\adfinis\framework\controller;

use org\adfinis\framework\request;
use org\adfinis\framework\utils\URLHelper;

/**
 * Dispatches a given URL to a given controller object.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class Dispatcher {

	/**
	 * Stores the url => controller mapping
	 *
	 * @var           array $mapping
	 */
	private $mapping;

	/**
	 * Takes an array that maps URLs to controller class names.
	 *
	 * The mapping must have the following form: The keys must be valid
	 * regular expressions, the values are class names of a Controller.
	 * The given controller MUST implement the ControllerInterface.
	 *
	 * Optionally, it is allowed to give another Dispatcher object instead
	 * of a controller. This way, nested dispatching can be implemented.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        InvalidArgumentException if a regexp cannot be parsed, or 
	 *                                         if the target is not a valid 
	 *                                         ControllerInterface class.
	 * @param         array $mapping
	 * @returns       void
	 */
	public function setMapping($mapping) {
		$this->mapping = $mapping;
	}
	
	/**
	 * Returns the controller for the current URL. The controller is determined
	 * using the mapping set by the setMapping() method.
	 *
	 * If you pass in a request object as a parameter, this request object will
	 * be passed along to the controller's constructor. If you omit that parameter,
	 * a new request object is generated from the current GET/POST/COOKIE etc. values
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        InvalidArgumentException if a regexp cannot be parsed, or 
	 *                                         if the target is not a valid 
	 *                                         ControllerInterface class.
	 * @param         RequestInterface $request (optional)
	 * @returns       ControllerInterface
	 */
	public function getController($request = null) {

		// We only want the path part of the URL, not the whole thing...
		$urlWithPath = URLHelper::getURLWithRequestString();
		$baseURL     = URLHelper::getURL();
		$url         = substr($urlWithPath, strlen($baseURL));

		if($request === null) {
			$request = request\Request::fromCurrentGPC();
		}
		foreach ($this->mapping as $regex => $class) {
			$match = array();
			if(preg_match($regex, $url, $match)) {
				if(!class_exists($class)) {
					throw new \InvalidArgumentException(
						"Could not find controller for given URL, check dispatcher config ($class)"
					);
				}
				$controller = new $class($request, $match);
				return $controller;
			}
		}

		throw new NotFoundException('Page not found');
	}
}
