<?php

namespace org\adfinis\framework\datamapper;

use \org\adfinis\framework as oaf;

/**
 * IdentityMap
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class IdentityMap {

	private static $classInstances = array();
	private static $contextCount   = 0;

	/**
	 * Currently we're using an array for the map
	 *
	 * @var           array $map
	 */
	private $map = array();

	public static function getInstance($class) {
		if (!isset(self::$classInstances[$class])) {
			self::$classInstances[$class] = new IdentityMap();
		}

		return self::$classInstances[$class];
	}

	public static function clearAll() {
		self::$classInstances = array();
	}

	/**
	 * Add an object to the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         oaf\model\Entity $obj
	 * @returns       this
	 */
	public function set(oaf\model\Entity $obj) {
		$this->verifyContextCount();
		$this->map[self::identifierToString($obj->getIdentifier())] = $obj;

		return $this;
	}

	/**
	 * Get an object from the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array $identifier
	 * @returns       oaf\model\Entity
	 */
	public function get(array $identifier) {
		$this->verifyContextCount();
		return $this->map[self::identifierToString($identifier)];
	}

	/**
	 * Check if the given identifier is already set
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array $identifier
	 * @returns       bool
	 */
	public function exists(array $identifier) {
		$this->verifyContextCount();
		return isset($this->map[self::identifierToString($identifier)]);
	}

	/**
	 * Remove the given identifier from the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array $identifier
	 * @returns       this
	 */
	public function remove(array $identifier) {
		unset($this->map[self::identifierToString($identifier)]);

		return $this;
	}

	/**
	 * Empties the IdentityMap
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       this
	 */
	public function clear() {
		$this->map = array();

		return $this;
	}

	public function getContext() {
		self::$contextCount++;
		return new IdentityMapContext();
	}

	public function exitContext() {
		self::$contextCount--;
	}

	/**
	 * Constructor of IdentityMap
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 */
	private function __construct() { }

	/**
	 * Convert an entity identifier to a string, which is needed as array index
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array $identifier
	 * @returns       string
	 */
	private function identifierToString(array $identifier) {
		$parts = array();
		foreach($identifier as $key => $value) {
			$parts[] = sprintf('%s:%s', $key, $value);
		}

		sort($parts);
		return implode(',', $parts);
	}

	/**
	 * This gets called all over the place.
	 *
	 * It's still here for that reason. All it used to do was spam the logs
	 * full with "IdentityMap used without context" that where probably had
	 * the developer as target.
	 *
	 * @return void
	 */
	private function verifyContextCount() {
		;
	}
}
