<?php

namespace org\adfinis\framework\datamapper;

use \org\adfinis\framework as oaf;

/**
 * SimpleMapper
 *
 * Generic mapper for Entity objects
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class SimpleMapper extends Mapper {

	/**
	 * Table to use with this mapper
	 *
	 * @var           string $tableName
	 */
	private $tableName;

	/**
	 * Primary key field to use with this mapper
	 *
	 * @var           string $primaryKeyName
	 */
	private $primaryKeyName;

	/**
	 * Holds an identity map for the created objects of this mapper
	 *
	 * @var           IdentityMap $identityMap
	 */
	private $identityMap;

	/**
	 * Constructor of SimpleMapper
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $targetClass
	 * @param         string $tableName
	 * @param         string $primaryKeyName
	 */
	public function __construct($targetClass, $tableName, $primaryKeyName){
		parent::__construct($targetClass);

		$this->tableName      = $tableName;
		$this->primaryKeyName = $primaryKeyName;
	}

	/**
	 * Empties the IdentityMap
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       this
	 */
	public function clearIdentityMap() {
		$this->getIdentityMap()->clear();

		return $this;
	}

	/**
	 * Fetch an object by field
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $field
	 * @param         string $value
	 * @returns       object
	 */
	protected function getObjectByField($field, $value) {
		$preparedStatement = oaf\db\DbManager::prepare(
			sprintf('SELECT * FROM `%s` WHERE `%s` = ?', $this->tableName, $field));
		$preparedStatement->execute(array($value));

		$object = new $this->targetClass();
		$object->fillFromRowData($preparedStatement->fetch());

		$this->getIdentityMap()->set($object);

		return $object;
	}

	/**
	 * Fetch an object by ID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $ID
	 * @returns       object
	 */
	public function getObjectByID($ID) {
		$identifier = array($this->primaryKeyName => $ID);

		if ($this->getIdentityMap()->exists($identifier)) {
			return $this->identityMap->get($identifier);
		}

		return $this->getObjectByField($this->primaryKeyName, $ID);
	}

	/**
	 * Fetch objects by field
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $field
	 * @param         string $value
	 * @param         string $comparison [optional]
	 * @returns       oaf\db\DBIterator
	 */
	protected function findObjectsByField($field, $value, $comparison = '=') {
		$searchQuery = sprintf('SELECT `%s` FROM `%s` WHERE `%s` %s ?',
			$this->primaryKeyName, $this->tableName, $field, $comparison);

		$selectQuery = sprintf('SELECT * FROM `%s`', $this->tableName);

		$queryParameters = array($value);

		return oaf\db\DBIterator::create(
			$searchQuery,
			$selectQuery,
			$this->targetClass,
			$queryParameters
		);
	}

	/**
	 * Updates the this entity in the database, if no entityID is given,
	 * the entity will be added as a new row instead
	 * please make sure all values are sane
	 * returns the entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        Zend_Db_Exception
	 * @param         $this->targetClass $obj
	 * @returns       array
	 */
	public function save(oaf\model\Entity $obj) {
		// If we don't have an identifier we're dealing with a new entity
		if(!$obj->hasIdentifier()) return $this->add($obj);

		// Prepare update query "SET" part
		$values   = array();
		$setParts = array();
		foreach($obj as $key => $value){
			$values[$key] = $value;

			// Skip primary key as we need it in the WHERE part
			if($key === $this->primaryKeyName) continue;

			$setParts[] = sprintf('`%1$s` = :%1$s', $key);
		}

		$query = sprintf('UPDATE `%1$s` SET %2$s WHERE `%3$s` = :%3$s',
			$this->tableName,
			implode(', ', $setParts),
			$this->primaryKeyName);

		oaf\db\DbManager::prepare($query)->execute($values);

		if(!$this->identityMap->exists($obj->getIdentifier())) {
			$this->identityMap->set($obj);
		}

		return $obj->getIdentifier();
	}

	/**
	 * Adds the current entity into the database
	 * returns the new entitys identifier
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        Zend_Db_Exception
	 * @param         $this->targetClass $obj
	 * @returns       array
	 */
	private function add($obj){
		$fieldParts      = array();
		$fieldValueParts = array();
		foreach($obj as $key => $value){
			$fieldParts[]      = sprintf('`%s`', $key);
			$fieldValueParts[] = sprintf(':%s',  $key);
		}

		$query = sprintf('INSERT INTO `%s` (%s) VALUES (%s)',
			$this->tableName,
			implode(', ', $fieldParts),
			implode(', ', $fieldValueParts));
		oaf\db\DbManager::prepare($query)->execute((array)$obj);

		$ID = oaf\db\DbManager::lastInsertId();
		$obj->{$this->primaryKeyName} = $ID;

		$this->getIdentityMap()->set($obj);

		return $obj->getIdentifier();
	}

	/**
	 * Prevent cloning SimpleMapper
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 */
	private function __clone() { }

	/**
	 * Get the IdentityMap instance of this Mapper
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       IdentityMap
	 */
	private function getIdentityMap() {
		return $this->identityMap ?: $this->identityMap = new IdentityMap();
	}

	/**
	 * Holds the instance of SimpleMapper
	 *
	 * @var           array<SimpleMapper> $instance
	 */
	private static $instance;

	/**
	 * Get instance of SimpleMapper
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $tableName
	 * @return        SimpleMapper
	 */
	public static function getInstance($tableName) {
		return self::$instance[$tableName] ?:
			self::$instance[$tableName] = new self($tableName);
	}
}
