<?php

namespace org\adfinis\framework\db;

use org\adfinis\framework\model\Entity;

/**
 * Iterates over the result set of a given query.
 *
 * Provides pagination and looping though DB results in an object-oriented way.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class DBIterator implements \Iterator, \Countable {

	/**
	 * The placeholder identifier
	 *
	 * @const         string IDS_PLACEHOLDER
	 */
	const IDS_PLACEHOLDER = "/*--IDS--*/";

	/**
	 * Cache for the count - use getCount() instead of $this->count.
	 *
	 * @var           int|null $count
	 */
	private $count = null;

	/**
	 * Name of DB instance that should be used (default value "default")
	 *
	 * @var           string $dbInstanceName
	 */
	private $dbInstanceName = 'default';

	/**
	 * Internal storage to remember offset in fromQuery()
	 *
	 * @var           int|null $queryOffset
	 */
	public $queryOffset;

	/**
	 * Internal storage to remember limit in fromQuery()
	 *
	 * @var           int|null $queryLimit
	 */
	public $queryLimit;

	/**
	 * Stores the query used for searching the required entities.
	 *
	 * @var           string $searchQuery
	 */
	protected $searchQuery;

	/**
	 * Stores the query used to select the data required by the entity.
	 *
	 * @var           string $selectQuery
	 */
	protected $selectQuery;

	/**
	 * Name of the class of the returned entities
	 *
	 * @var           string $targetClass
	 */
	protected $targetClass;

	/**
	 * Query parameters for the search statement
	 *
	 * @var           array $searchQueryParameters
	 */
	protected $searchQueryParameters;

	/**
	 * Query parameters for the select statement
	 *
	 * @var           array $selectQueryParameters
	 */
	protected $selectQueryParameters;

	/**
	 * Stores the current object when looping
	 *
	 * @var           org\adfinis\framework\model\Entity $currentObject
	 */
	protected $currentObject;

	/**
	 * Stores the current row for reading the DB results.
	 *
	 * @var           array $currentRow
	 */
	protected $currentRow;

	/**
	 * Stores the PDO statement for fetching data.
	 *
	 * @var           PDOStatement $preparedStatement
	 */
	protected $preparedStatement;

	/**
	 * If enabled then a placeholder is used to filter the select query
	 *
	 * e.g (without the "\"):
	 *	"SELECT * FROM `NagiosLog` WHERE \/*--IDS--*\/"
	 * will be replaced to:
	 *	"SELECT * FROM `NagiosLog` WHERE `id` IN (1,2,...)
	 *
	 * @see           static::IDS_PLACEHOLDER
	 * @var           PDOStatement $preparedStatement
	 */
	protected $usePlaceholder = false;

	/**
	 * Returns an iterator object for the given query.
	 *
	 * Takes two SQL queries and a target class name as parameters.
	 *
	 * The search query should return a DISTINCT list of IDs to be listed.
	 * The select query selects all the fields required by the entity class.
	 * The result of this query will be passed to Entity::fillFromRowData().
	 * The $targetClass parameter describes which class to create when iterating
	 * over the results.
	 * Optionally, you may pass in query parameters that will be used in a prepared statement.
	 *
	 * Here's an example of how to use this:
	 * @code
	 *     $select = 'SELECT * FROM `users` WHERE `id` in (%s)';
	 *     $search = 'SELECT `id` FROM `users` WHERE `name` LIKE ?';
	 *     $users = db\DBIterator::create($search, $select, 'User', array('foo'));
	 * @endcode
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $searchQuery
	 * @param         string $selectQuery
	 * @param         string $targetClass
	 * @param         array  $searchQueryParameters (optional)
	 * @param         array  $selectQueryParameters (optional)
	 * @returns       DBIterator
	 */
	public static function create($searchQuery, $selectQuery, $targetClass, $searchQueryParameters = array(), $selectQueryParameters = array()) {
		$iterator = new static();
		$iterator->searchQuery     = $searchQuery;
		$iterator->selectQuery     = $selectQuery;
		$iterator->targetClass     = $targetClass;
		$iterator->searchQueryParameters = $searchQueryParameters;
		$iterator->selectQueryParameters = $selectQueryParameters;

		return $iterator;
	}

	/**
	 * Enable or disable the use of IDs placeholder
	 *
	 * @param         bool $use
	 * @return        void
	 */
	public function usePlaceholder($use) {
		$this->usePlaceholder = $use;
	}

	/**
	 * Returns the current object when looping
	 *
	 * Implements \Iterator requirement
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       org\adfinis\framework\model\Entity
	 */
	public function current() {
		while($this->currentRow = $this->getStatement()->fetch()){
			if(!$this->currentObject->hasIdentifier() || $this->currentObject->doesRecordMatch($this->currentRow)) {
				$this->currentObject->fillFromRowData($this->currentRow);
			}
			else {
				break;
			}
		}
		return $this->currentObject;
	}

	/**
	 * Rewind is called before the first loop iteration
	 *
	 * Warning: As PDOStatements cannot be rewound, the statement
	 * is re-created and the iterator reset. Inefficient, but it works.
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function rewind(){

		if(!is_null($this->preparedStatement)) {
			$this->preparedStatement->closeCursor();
			$this->preparedStatement = null;
		}
		// re-init statement, set row...
		$this->currentRow = $this->getStatement()->fetch();
		// ... so we can initialize the first object
		$this->next();
	}

	/**
	 * Initialize the next object
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function next(){
		$this->currentObject = $this->getTargetClassObject();

		if($this->currentRow){
			$this->currentObject->fillFromRowData($this->currentRow);
		}
	}

	/**
	 * Returns the current object's identifier
	 *
	 * In case the identifier is an array, return it
	 * as JSON string. The reason is smarty: smarty will
	 * always call this method in a foreach loop, but arrays
	 * used as array keys are illegal and therefore a warning
	 * is thrown.
	 * @author        Damian Senn <ds@adfinis.com>
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @returns       array
	 */
	public function key(){
		$ID = $this->currentObject->getIdentifier();

		if (gettype($ID) == 'array') {
			return json_encode($ID);
		} else {
			return $ID;
		}
	}


	/**
	 * Checks if the current object is valid
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       bool
	 */
	public function valid(){
		return $this->currentObject instanceof Entity && $this->currentObject->hasIdentifier();
	}

	/**
	 * Returns the search query
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @returns       string
	 */
	protected function getSearchQuery() {
		return $this->searchQuery;
	}

	/**
	 * Returns the ORDER BY statement
	 *
	 * This returns an empty string by default, but you can override
	 * this in a subclass to make ORDER BY configurable.
	 *
	 * If you don't use this customizability, you may simply add the ORDER BY statement
	 * to the search query.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	protected function getOrderBy() {
		return '';
	}

	/**
	 * Returns a PDO statement for the given query.
	 *
	 * Creates a statement for the given query and returns it.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @todo by dv:   Check if compound key queries really work in every case. Seems to work, but not proven yet
	 * @returns       PDOStatement
	 */
	private function getStatement() {
		if(is_null($this->preparedStatement)) {
			$order  = $this->getOrderBy();
			$offset = ($this->queryOffset) ? sprintf('OFFSET %d', $this->queryOffset) : '';
			$limit  = ($this->queryLimit ) ? sprintf('LIMIT  %d', $this->queryLimit ) : '';

			$idQuery = sprintf('%s %s %s %s', $this->getSearchQuery(), $order, $limit, $offset);

			$statementIDs = DbManager::getInstance()->prepare($idQuery, $this->dbInstanceName, array());
			$statementIDs->execute($this->searchQueryParameters);

			// Hack: use an empty entity object to fetch the primary key field names
			$entity   = $this->getTargetClassObject();
			$idfields = array_keys($entity->getIdentifier());

			if (sizeof($idfields) != 1) {
				throw new \Exception(
					tr('Multi-field primary keys currently not supported')
				);
			}

			$idfield = $idfields[0];

			$ids = array_map(
				function($row) use($idfield) { return $row[$idfield]; },
				$statementIDs->fetchAll()
			);

			$idsAsString = join(',', $ids);

			if (sizeof($ids)) {
				if ($this->usePlaceholder) {
					$query = str_replace(
						static::IDS_PLACEHOLDER,
						sprintf('`%s` IN (%s) ORDER BY FIELD(`%s`, %s)', $idfield, $idsAsString, $idfield, $idsAsString),
						$this->selectQuery,
						$replacements
					);
					if ($replacements <= 0) {
						throw new \RuntimeException(
							tr("The id placeholder functionality was enabled, but the needed tag was not found")
						);
					}
				}
				else {
					$query = sprintf(
						'SELECT `contents`.* FROM (%s) AS `contents` WHERE `%s` IN (%s) ORDER BY FIELD(`%s`, %s)',
						$this->selectQuery,
						$idfield,
						$idsAsString,
						$idfield,
						$idsAsString
					);
				}
			}
			else {
				// No ids found, but we need an empty result set. Yes, MySQL does
				// support DUAL as well ^^ Note that this is the first time I actually
				// needed this...
				$query = 'SELECT 1 FROM DUAL WHERE false';
			}

			$params = $this->selectQueryParameters;

			$this->preparedStatement = DbManager::getInstance()->prepare($query, $this->dbInstanceName, array());
			$this->preparedStatement->execute($params);
		}

		return $this->preparedStatement;
	}

	/**
	 * Returns a new instance of the target class
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       Entity
	 */
	protected function getTargetClassObject() {
		$targetClass = $this->targetClass;
		return new $targetClass();
	}

	/**
	 * Sets offset and limit for all following queries. This can be used to implement pagination.
	 *
	 * If you set offset to NULL, they are both reset.
	 *
	 * @author        David Vogt <dv@adfinis.com>
	 * @param         int|null $offset
	 * @param         int|null $limit
	 * @returns       void
	 */
	public function setOffsetLimit($offset = null, $limit = null) {
		$this->queryLimit  = $limit;
		$this->queryOffset = $offset;
	}

	/**
	 * Sets offset and limit according to page info, not directly. Note that page starts with 1.
	 *
	 * This overwrites the information set by setOffsetLimit().
	 *
	 * @author        David Vogt <dv@adfinis.com>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $page
	 * @param         int $itemsPerPage [optional, defaults to 20]
	 * @returns       void
	 */
	public function setPage($page, $itemsPerPage = 20) {
		if (is_null($page) or $page == 0) {
			$page = 1;
		}

		$this->queryLimit  = $itemsPerPage;
		$this->queryOffset = ($page-1) * $itemsPerPage;

	}

	/**
	 * Sets DB instance name that sould be used by the DbManager
	 *
	 * @author        Adrian Wittwer <aw@adfinis.com>
	 * @param         string $name
	 * @returns       void
	 */
	public function setDBInstance($name) {
		$this->dbInstanceName = $name;
	}

	/**
	 * Returns the total number of objects in this iterator.
	 *
	 * This counts all the entries WITHOUT respect to offset / limit. Use
	 * count() if you want to know how many entries will be returned.
	 * Note that this neccessarily executes the search query.
	 *
	 * @author        David Vogt <dv@adfinis.com>
	 * @returns       int
	 */
	public function getCount() {
		if(!is_null($this->count)) {
			return $this->count;
		}
		$countQuery = sprintf(
			'SELECT COUNT(*) AS c FROM (%s) AS q',
			$this->getSearchQuery()
		);
		$countStatement = DbManager::getInstance()->prepare($countQuery, $this->dbInstanceName, array());
		$countStatement->execute($this->searchQueryParameters);
		$countRow = $countStatement->fetch();
		$countStatement->closeCursor();

		$this->count = $countRow['c'];

		return $this->count;
	}

	/**
	 * Returns the number of elements actually iterated (respects offset / limit)
	 *
	 * This is used to implement the Countable interface.
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       int
	 */
	public function count(){
		$count = $this->getCount();

		if($this->queryLimit !== null && $this->queryOffset !== null) {
			return min($this->queryLimit, $count - $this->queryOffset);
		}

		if($this->queryLimit !== null){
			return $this->queryLimit;
		}

		if($this->queryOffset !== null){
			return $count - $this->queryOffset;
		}

		return $count;
	}

	/**
	 * Returns the iterator data as an array.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	public function toArray() {
		$out = array();

		foreach($this as $elem) {
			$out[] = $elem;
		}

		return $out;
	}
}
