<?php

namespace org\adfinis\framework\db;

use org\adfinis\framework\config\ConfManager;

/**
 * Class dealing with database connection and running queries
 *
 * The database backend is using Zend DB and most
 * functions will return Zend DB instances
 *
 * IMplemented the save points from
 * http://www.kennynet.co.uk/2008/12/02/php-pdo-nested-transactions/
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @author      Damian Senn <ds@adfinis.ch>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 * @see         http://framework.zend.com/manual/en/zend.db.html
 */
class DbManager {
	/**
	 * Holds global database connection instances
	 *
	 * @var     array $_dbZendInstances
	 */
	private static $_dbZendInstances = array();

	/**
	 * Initialize database connection to given instance
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @throws      Zend_Db_Exception
	 * @param       string $instanceName
	 * @return      bool
	 */
	public static function initConnection($instanceName = 'default') {
		if(
			isset(self::$_dbZendInstances[$instanceName]) &&
			self::$_dbZendInstances[$instanceName] instanceof \Zend_Db_Adapter_Abstract
		){
			return true;
		}

	    $pdotype = ConfManager::getVar('af', 'DB', $instanceName, 'PDOType');
        
		$params = array(
			'host'     => ConfManager::getVar('af', 'DB', $instanceName, 'host'),
			'username' => ConfManager::getVar('af', 'DB', $instanceName, 'user'),
			'password' => ConfManager::getVar('af', 'DB', $instanceName, 'pass'),
			'dbname'   => ConfManager::getVar('af', 'DB', $instanceName, 'DBName')
		);

		$wrappedInstance = new DbWrapper($pdotype, $params);

		// Set global instance
		self::$_dbZendInstances[$instanceName] = $wrappedInstance;

		// Init charset conversion
		self::getInstance($instanceName)->query(
			sprintf('SET NAMES \'%s\'', ConfManager::getVar('af', 'DB', $instanceName, 'charset'))
		); // it's important that it is single quotes

		// When initiating default instance, set default adapter for database table
		if($instanceName == 'default') {
			\Zend_Db_Table::setDefaultAdapter(self::getInstance($instanceName)->getAdapter());
		}

		return true;
	}

	/**
	 * Returns reference to given instance
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @author      Nicolas Christener <nc@adfinis.ch>
     * @author      David Vogt <david.vogt@adfinis-sygroup.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @throws      Zend_Db_Exception
	 * @param       string $instanceName
	 * @return      Zend_Db_Adapter_Abstract
	 */
	public static function getInstance($instanceName = 'default') {
		// Return reference to pdo-object if instance is loaded
		if(!isset(self::$_dbZendInstances[$instanceName])){
		    self::initConnection($instanceName);
		}

		// Try loading instance if not available
		return self::$_dbZendInstances[$instanceName];
	}

	/**
	 * Builds a quoted where statement
	 *
	 * The fields parameter contains an associated array where the key is the
	 * fieldname and it's data needs contain an array with following keys. If
	 * it's not an array however, the operator defaults to = and the value is
	 * inserted as it is.
	 *
	 * value => the value of the field
	 * operator => the operator, defaults to =
	 *
	 * <code>
	 * $fields = array(
	 * 	'name' => 'Matthias'
	 * 	'userID' =>
	 * 		array('value' => 2, 'operator' => '>')
	 * 	);
	 *
	 * $where = $SQL->buildWhere($fields)
	 * // $where = "`name` = 'Matthias' AND `user_id` > 2"
	 * </code>
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @throws      InvalidArgumentException
	 * @throws      Zend_Db_Exception
	 * @param       array $fields array containing field and value definitions
	 * @param       string $operater the operator used to join multiple where statements
	 * @param       string $instanceName
	 * @return      string built where statement
	 */
	public static function buildWhere($fields, $operator = 'AND', $instanceName = 'default') {
		if(!is_array($fields)) {
			throw new InvalidArgumentException(tr('wrong parameter type for fields, should be an array'));
		}

		if(strtolower($operator) != 'or') {
			$operator = ' AND ';
		} else {
			$operator = ' OR ';
		}

		$where = array();

		foreach($fields as $key => $data) {
			if(!is_array($data)) {
				$data = array('value' => $data, 'operator' => '=');
			} else {
				if(!isset($data['value'])) {
					throw new InvalidArgumentException(tr('Wrong array format for fields, see doc for examples'));
				}

				if(!isset($data['operator'])) {
					$data['operator'] = '=';
				}
			}

			// Syntax is different for null values
			if(is_null($data['value'])) {
				$where[] = sprintf(
					'%s IS NULL',
					self::getInstance($instanceName)->quoteIdentifier($key)
				);
			} else {
				$where[] = self::getInstance($instanceName)->quoteInto(
					sprintf(
						'%s %s ?',
						self::getInstance($instanceName)->quoteIdentifier($key),
						$data['operator']
					),
					$data['value']
				);
			}
		}

		return join($operator, $where);
	}

	/**
	 * Disconnects the database.
	 *
	 * Optionally, pass the instanceName to disconnect.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $instanceName
	 * @return        void
	 */
	public static function disconnect($instanceName = 'default') {
		$inst = self::getInstance($instanceName);
		if(is_null($inst)) {
			return;
		}
		$inst->closeConnection();
		unset(self::$_dbZendInstances[$instanceName]);
	}
}
