<?php

namespace org\adfinis\framework\db;

/**
 * A wrapper for the database adapter
 *
 * We needed this to implement a special version
 * of the transaction stuff, to enable nested transaction.
 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
 * @copyright     adfinis GmbH 2012
 * @license       BSD 3-Clause
 * @see           http://framework.zend.com/manual/en/zend.db.html
 * @see           http://www.kennynet.co.uk/2008/12/02/php-pdo-nested-transactions/
 */
class DbWrapper {

    /**
     * The argument for the zend db adapter factory
     * 
     * @var           TODO 
     */
    private $pdotype;

    /**
     * The parameters for the zend db adapter factory
     *
     * @var           TODO
     */
    private $params;

	/**
	 * The Database adapter
	 *
	 * @var           Zend_Db_Adapter $adapter
	 */
	private $adapter;

	/**
	 * Transaction level
	 *
	 * @var           int $transactionLevel
	 */
	private $transactionLevel;

	/**
	 * Database drivers that support save points
	 *
	 * @var   array $savepointTransactions
	 */
	protected static $savepointTransactions = array("pgsql", "mysql");

	/**
     * Constructor
     *
     * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         Zend_Db_Adapter_Abstract $adapter The Adapter from the factory
     * @param         string $instanceName The name of the adapter instance in the DbManager.class
	 */
	public function __construct($pdotype, $params) {
        $this->pdotype = $pdotype;
        $this->params = $params;
        $this->adapter = $this->connect();
	}

	/**
     * Constructor
     *
     * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         Zend_Db_Adapter_Abstract $adapter The Adapter from the factory
     * @param         string $instanceName The name of the adapter instance in the DbManager.class
     */
    private function connect() {
		return \Zend_Db::factory(
            $this->pdotype,
			$this->params
		);
    }

	/**
	 * Wrap the methods. Pass most to the adapter
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
     * @author        Pablo Vergés <pablo.verges@adfinis-sygroup.ch>
	 * @param         string $method The called method
	 * @param         array $pdotypes The arguments
	 */
	public function __call($method, $args) {
        return $this->callWithRetry($method, $args);
	}

	private function callWithRetry($method, $args, $isRetry = false) {
		try {
			return call_user_func_array(
				array(
					$this->adapter,
					$method
				),
				$args
			);
		}
		catch (\Zend_Db_Statement_Exception $e) {

			if ($isRetry) {
				throw $e;
			}

			// if the server disconnected during runtime, reconnect and retry
			if (strpos($e->getMessage(), "SQLSTATE[HY000]") !== false) {
				$this->adapter = $this->connect();
			}

			return $this->callWithRetry($method, $args, true);
		}
	}

	/**
	 * Start a transaction
	 *
	 * If supported, use save points to nest the 
	 * transactions
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	public function beginTransaction() {
		if (!$this->nestable() || $this->transactionLevel == 0) {
			$this->adapter->beginTransaction();
		}
		else {
			$this->adapter->exec("SAVEPOINT LEVEL{$this->transactionLevel}");
		}

		$this->transactionLevel++;
	}

	/**
	 * Commit the current transaction
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	public function commit() {
		$this->transactionLevel--;

		if (!$this->nestable() || $this->transactionLevel == 0) {
			$this->adapter->commit();
		}
		else {
			$this->adapter->exec("RELEASE SAVEPOINT LEVEL{$this->transactionLevel}");
		}
	}

	/**
	 * Roll back the current transaction
	 */
	public function rollback() {
		$this->transactionLevel--;

		if (!$this->nestable() || $this->transactionLevel == 0) {
			$this->adapter->rollBack();
		}
		else {
			$this->adapter->exec("ROLLBACK TO SAVEPOINT LEVEL{$this->transactionLevel}");
		}
	}

	/**
	 * Return if the current data base driver supports save points
	 *
	 * @returns   bool
	 */
	private function nestable() {
		return in_array($this->adapter->getConnection()->getAttribute(\PDO::ATTR_DRIVER_NAME),
			self::$savepointTransactions);
	}

	/**
	 * Return the adapter
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       Zend_Db_Adapter_Abstract
	 */
	public function getAdapter() {
		return $this->adapter;
	}
}
