<?php

namespace org\adfinis\framework\db;

/**
 * Adds sorting ability to the DBIterator
 *
 * Note: if you use this iterator instead of the default DBIterator, you
 * MUST NOT add an ORDER BY statement to the search query.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class SortingIterator extends DBIterator {

	/**
	 * Stores the possible ORDER BY statements.
	 *
	 * They are identified by a given key and contain a single placeholder for ASC/DESC.
	 *
	 * @var           array $possibleOrderBy
	 */
	private $possibleOrderBy = array();

	/**
	 * Stores the currently desired order identifier
	 *
	 * @var           string $orderIdentifier
	 */
	private $orderIdentifier = null;

	/**
	 * Adds the given statement as a possible ORDER BY statement.
	 *
	 * The first call to this method implicitly defines the default ORDER BY.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         array $identifier
	 * @param         string $order
	 * @returns       void
	 */
	public function addOrderVariant($identifier, $order) {
		$this->possibleOrderBy[$identifier] = $order;
	}

	/**
	 * Returns a custom ORDER BY statement, or an empty string if no ORDER BY is configured
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	protected function getOrderBy() {
		if(is_null($this->orderIdentifier)) {
			if(sizeof($this->possibleOrderBy)) {
				$possible = array_keys($this->possibleOrderBy);
				$this->orderIdentifier = $possible[0];
			}
			else {
				return '';
			}
		}
		return sprintf(
			'ORDER BY %s',
			$this->possibleOrderBy[$this->orderIdentifier]
		);
	}
	
	/**
	 * Selects which order by statement is to be used.
	 *
	 * The given identifier must be registered by addOrderVariant() before
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        InvalidArgumentException if the identifier is not known.
	 * @param         string $identifier
	 * @param         boolean $ignoreError (optional, defaults to false). If set to true, no exceptions are thrown.
	 * @returns       void
	 */
	public function setOrderBy($identifier, $ignoreError = false) {
		if($identifier === null) {
			$this->orderIdentifier = null;
			return;
		}
		if(isset($this->possibleOrderBy[$identifier])) {
			$this->orderIdentifier = $identifier;
		}
		elseif($ignoreError) {
			$this->orderIdentifier = null;
		}
		else {
			$this->orderIdentifier = null;
			throw new \InvalidArgumentException("Sort identifier not known!");
		}

	}
}
