<?php

namespace org\adfinis\framework\db\sql;

/**
 * Class to deal with SQL where statements
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class WhereGroup implements \ArrayAccess {
	/**
	 * Holds the SQLWhereStatements (may be strings or SqlWhereStatement objects)
	 *
	 * @var array $SQLWhereStatements
	 */
	private $SQLWhereStatements = array();

	/**
	 * Holds the operator to group the SQLWhereStatements
	 *
	 * @var string $operator
	 */
	private $operator;

	/**
	 * Adds a new statement to the SQLWhereStatements array
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int                        $identifier
	 * @param       string|WhereStatement|OrStatement $SQLWhereStatement
	 * @return      void
	 */
	public function addStatement($identifier, $SQLWhereStatement){
		$this->SQLWhereStatements[$identifier] = $SQLWhereStatement;
	}

	/**
	 * Sets the operand to group the SQLWhereStatements
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $operator
	 * @return      void
	 */
	public function setOperand($operator){
		$this->operator = sprintf(' %s ', $operator);
	}

	/**
	 * Magic method __toString is called in string contexts
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @author      Matthieu riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @return      string
	 */
	public function __toString() {
		$conditions = $this->getSQLConditions();
		return empty($conditions) == false ? sprintf('WHERE %s', $conditions) : '';
	}

	/**
	 * Returns the raw SQL conditions without the a beginning WHERE
	 *
	 * @author      Matthieu riolo <matthieu.riolo@adfinis-sygroup.ch>
	 * @return      string
	 */
	public function getSQLConditions() {
		return count($this->SQLWhereStatements) > 0 ? sprintf('(%s)', implode($this->operator, $this->SQLWhereStatements)) : '';
	}


	/**
	 * Assigns a value to the specified offset.
	 *
	 * @author     Damian Senn <ds@adfinis.com>
	 * @param      string|int                             $offset
	 * @param      string|SqlWhereStatement|SqlWhereGroup $SQLWhereStatement
	 * @return     void
	 */
	public function offsetSet($offset, $SQLWhereStatement){
		$this->addStatement($offset, $SQLWhereStatement);
	}

	/**
	 * Whether or not an offset exists.
	 * This method is executed when using isset() or empty() on objects implementing ArrayAccess.
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int $offset
	 * @return      bool
	 */
	public function offsetExists($offset){
		return isset($this->SQLWhereStatements[$offset]);
	}

	/**
	 * Unsets an offset.
	 * Note: This method will not be called when type-casting to (unset)
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int $offset
	 * @return      void
	 */
	public function offsetUnset($offset){
		unset($this->SQLWhereStatements[$offset]);
	}

	/**
	 * Returns the value at specified offset.
	 *
	 * This method is executed when checking if offset is empty().
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int $offset
	 * @return      string|SqlWhereStatement|SqlWhereGroup
	 */
	public function offsetGet($offset){
		return (isset($this->SQLWhereStatement[$offset])
			? $this->SQLWhereStatement[$offset]
			: null);
	}
}
