<?php

namespace org\adfinis\framework\db\sql;

use org\adfinis\framework\db\DbManager;

/**
 * Class to deal with SQL where statements
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class WhereStatement {
	/**
	 * Contains the field name
	 * @var           string $field;
	 */
	private $field;

	/**
	 * Contains the table name
	 * @var           string $table;
	 */
	private $table;

	/**
	 * Contains the operator
	 * @var           string $operator;
	 */
	private $operator;

	/**
	 * Contains the value
	 * @var           string $value;
	 */
	private $value = null;

	/**
	 * Defines if placeholders should be used if no value is passed
	 *
	 * @var           bool $placeholder
	 */
	private $placeholder;

	/**
	 * Define a function to put around the field
	 *
	 * @var           string $function
	 */
	private $function;

	/**
	 * Constructor of SqlWhereStatement
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $table
	 * @param       string $field
	 * @param       string $operator
	 * @param       string $value
	 * @param       bool   $placeholder
	 * @return      void
	 */
	public function __construct($field, $operator, $value = null, $table = null, $placeholder = true, $function = null){
		$this->table       = $table;
		$this->field       = $field;
		$this->operator    = $operator;
		$this->placeholder = $placeholder;
		$this->function    = $function;

		// only set value if a value is passed to the constructor
		if($value !== null){
			$this->setValue($value);
		}
	}

	/**
	 * Sets the value escaped by the DbManager for the statement
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       mixed $value
	 * @return      void
	 */
	public function setValue($value){
		if (is_array($value)) {
			foreach ($value as &$element) {
				$element = DbManager::getInstance()->quote($element);
			}

			$this->value = sprintf(
				"(%s)",
				implode(',', $value)
			);
		}
		else {
			$this->value = DbManager::getInstance()->quote($value);
		}
	}

	/**
	 * Get the escaped value of the statement
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getValue(){
		return $this->value;
	}

	/**
	 * Magic method __toString is called in string contexts
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function __toString(){
		// prepend table alias / table name if set
		if($this->table !== null){
			$field = sprintf('`%s`.`%s`', $this->table, $this->field);
		} else {
			$field = sprintf('`%s`', $this->field);
		}

		// enclose the field with the function
		if ($this->function !== null) {
			$field = sprintf('%s(%s)', $this->function, $field);
		}

		// use placeholder if no value given and placeholders are enabled
		if($this->placeholder && $this->value === null){
			$string = sprintf('%s %s :%s',
				$field, $this->operator, $this->field);
		} else {
			if (is_array($this->placeholder)) {
				if (strtoupper($this->operator) == 'IN') {
					// the operator is IN, so we list the data in brackets
					$string = sprintf('%s IN (%s)',
						$field,
						join(", ", $this->placeholder)
					);
				} else {
					$parts = array();
					// make an OR
					foreach ($this->placeholder AS $pl) {
						$parts[] = sprintf('%s %s %s',
							$field,
							$this->operator,
							$pl
						);
					}

					$string = sprintf('(%s)',
						join(' OR ', $parts)
					);
				}
			}
			else if ($this->value === null) {
				$this->value = 'NULL';
			}
			else if ($this->value === true) {
				$this->value = 'TRUE';
			}
			else if ($this->value === false) {
				$this->value = 'FALSE';
			}
			$string = sprintf('%s %s %s',
				$field, $this->operator, $this->value);
		}

		return $string;
	}
}
