<?php

namespace org\adfinis\framework\deploy;

// ConfigHelper needs to work even without autoloader. So 
// we require_once() all files here if autoloader is not present.
if(!class_exists('org\adfinis\framework\init\ClassLoader')) {
	if(!defined('INC_PATH')) {
		die("Class loader not initialized. ConfigHelper needs INC_PATH to be defined to work.");
	}
	require_once(sprintf('%s/smarty/libs/Smarty.class.php', INC_PATH));
}

/**
 * Helper class for parsing the config templates and writing the site config files.
 *
 * @author        Matthias Blaser <mb@adfinis.ch>
 * @author        Nicolas Christener <nc@adfinis.ch>
 * @author        Stefan Heinemann <sh@adfinis.ch>
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class ConfigHelper {

	/**
	 * Configuration file used for config template generation
	 *
	 * @var         string
	 */
	private $configFile;

	/**
	 * Directory where the git checkout is stored to
	 *
	 * @var         string
	 */
	private $checkoutDir;

	/**
	 * List of rsync targets
	 *
	 * @var         array
	 */
	private $targets = array();

	/**
	 * Initialize deploy object
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $checkoutDir The repository location
	 * @param       string $configFile config file for config templates
	 */
	public function __construct($checkoutDir, $configFile){
		$this->setCheckoutDir($checkoutDir);
		$this->setConfigFile($configFile);
	}

	/**
	 * Load deployment configuration file and generate configuration
	 *
	 * @author        Matthias Blaser <mb@adfinis.ch>
	 * @param         string $templateDir Directory of config templates
	 * @param         string $targeetDir Directory where the parsed configs go
	 */
	public function generateSiteConfig($templateDir, $targetDir){
		// generate config files out of templates
		$smarty = new \Smarty();

		$smarty->template_dir = $templateDir;
		$smarty->compile_dir = '/tmp';

		#$smarty->left_delimiter  = '[[';
		#$smarty->right_delimiter = ']]';

		$config = parse_ini_file($this->configFile, true);
		foreach($config as $key => $value) {
			$smarty->assign($key, $value);
		}

		#printf("Config values are taken from: %s\n", $this->configFile);

		// I liiiieek PHP :D
		$files = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($templateDir));

		// generate all config files from template
		foreach($files as $templateFile){
			$fileName = $templateFile->getFilename();
			if(substr($fileName, 0, 1) == '.' || substr($fileName, -1, 1) == '~') {
				continue;
			}

			$filePath = $files->getSubPath()
				? $files->getSubPath() . '/'
				: ''
			;

			$relativeFilePath = $filePath . $fileName;
			printf("--- Processing config template %s\n", $relativeFilePath);

			$configFileOut = sprintf(
				'%s/%s',
				$targetDir,
				$relativeFilePath
			);

			// Make sure the subdir exists, if the config template is in a subdir.
			if($filePath) {
				$targetFilePath = $targetDir . '/'. $filePath;
				`mkdir -p $targetFilePath`;
			}

			file_put_contents(
				$configFileOut,
				$smarty->fetch($relativeFilePath)
			);
		}
		echo "\n";
	}

	/**
	 * Add target to target list
	 *
	 * @author        Matthias Blaser <mb@adfinis.ch>
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $sourceDir local source directory (relative starting from checkout directory)
	 * @param         string $targetSpec remote destination directory, e.g. username@server:/directory/
	 * @param         array  $excludes An array of patterns that are to be excluded
	 * @param         string $additonalOpts additional rsync options (placed between rsync and source target)
	 */
	public function addTarget($sourceDir, $targetSpec, array $excludes, $additionalOpts = ''){
		$this->targets[] = array(
			'sourceDir' => $sourceDir,
			'targetSpec' => $targetSpec,
			'additionalOpts' => sprintf('%s %s', $this->generateExcludeString($excludes), $additionalOpts)
			);
	}

	/**
	 * Generate an exclude string for rsync
	 *
	 * @author      Stefan Heinemann <sh@adfinis.Com>
	 * @param       array $excludes
	 * @return      string
	 */
	public function generateExcludeString(array $excludes) {
		$string = "";
		foreach ($excludes as $e) {
			$string = sprintf('%s --exclude=%s', $string, $e);
		}

		return $string;
	}
	/**
	 * Return directory of the source files (git export directory)
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @return      string
	 */
	public function getCheckoutDir(){
		return $this->checkoutDir;
	}

	/**
	 * Return configuration file used for config template generation
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @return      string
	 */
	public function getConfigFile(){
		return $this->configFile;
	}

	/**
	 * Sets directory of the source files (git export directory)
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $checkoutDir
	 */
	public function setCheckoutDir($checkoutDir){
		$this->checkoutDir = $checkoutDir;
	}
	
	/**
	 * Set configuration file used for config template generation
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $configFile
	 */
	public function setConfigFile($configFile){
		$this->configFile = $configFile;
	}

}

