<?php

namespace org\adfinis\framework\deploy;

// ConfigHelper needs to work even without autoloader. So 
// we require_once() all files here if autoloader is not present.
if(!class_exists('org\adfinis\framework\init\ClassLoader')) {
	if(!defined('INC_PATH')) {
		die("Class loader not initialized. ConfigHelper needs INC_PATH to be defined to work.");
	}
	require_once(sprintf('%s/smarty/libs/Smarty.class.php', INC_PATH));
}

/**
 * Helper class for parsing the config templates and writing the site config files.
 *
 * @author        Matthias Blaser <mb@adfinis.ch>
 * @author        Nicolas Christener <nc@adfinis.ch>
 * @author        Stefan Heinemann <sh@adfinis.ch>
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class DeployHelper {

	/**
	 * Configuration file used for config template generation
	 *
	 * @var         string
	 */
	private $config;

	/**
	 * Directory where the git checkout is stored to
	 *
	 * @var         string
	 */
	private $checkoutDir;

	/**
	 * Pretend mode. If set to true, the deployment doesn't actually happen.
	 *
	 * @var           bool $pretend
	 */
	public $pretend = false;

	/**
	 * Initialize deploy object
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $deployConfig Deploy config file
	 * @param         string $checkoutDir  Path to checkout directory
	 */
	public function __construct($deployConfig, $checkoutDir){
		$this->config      = $deployConfig;
		$this->checkoutDir = $checkoutDir;
	}

	/**
	 * Executes the deployment.
	 *
	 * Reads the deploy config dir and deploys to the respective destinations.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function run() {
		$iniData = parse_ini_file($this->config, true);

		foreach ($iniData as $section => $conf) {
			$matches = array();
			if(preg_match('#^path\s+(.*)$#', $section, $matches)) {
				$this->deployPath($matches[1], $conf);
			}
			else if(preg_match('#^local$#', $section, $matches)) {
				$this->deployPath(null, $conf);
			}
			else if(preg_match('#^server\s+([^:]*):(.*)#', $section, $matches)) {
				$this->deployServer($matches[1], $matches[2], $conf);
			}
		}
	}

	/**
	 * Deploys to a local path, according to the given config
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $path
	 * @param         array  $conf
	 * @returns       void
	 */
	private function deployPath($path, $conf) {
		if ($path) {
			printf("=== Deploying to local path %s\n", $path);
		}
		else {
			printf("=== Deploying without rsync\n");
		}
		
		$this->prepare($conf);

		if ($path) {
			// Normalize paths, to make rsync bahave in a consistent way
			$conf['source']      = preg_replace('#/*$#', '', $conf['source']);
			$conf['source']     .= '/';
			
			printf("\n--- starting rsync\n");
			
			$rsyncParams = array();
			if(isset($conf['exclude'])) {
				$patterns = preg_split('/\s+/', $conf['exclude']);
				$rsyncParams[] = $this->generateExcludeString($patterns);
			}
			if(isset($conf['options'])) {
				$rsyncParams[] = $conf['options'];
			}
			
			$rsync = sprintf(
				'rsync -avz --delete-after %s %s %s',
				join(' ', $rsyncParams),
				$this->checkoutDir .'/'. $conf['source'],
				$path
			);
			printf(">>> %s\n", $rsync);

			$this->pretend || shell_exec($rsync);
		}

		if(isset($conf['action'])) {
			printf("\n--- additional deployment actions\n");
			foreach($conf['action'] as $action) {
				$cmd = $action;
				printf(">>> %s\n", $cmd);
				$return = 0;
				$this->pretend || system($cmd, $return);
				$this->pretend || ($return && printf("ERROR: statuscode=%d\n", $return));
			}
		}

		$this->cleanUp($conf);
	}

	/**
	 * Deploys to a server, according to the given config
	 *
	 * The server spec is expected to be in the form user@server.domain, 
	 * so it may be used to rsync and ssh.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $server
	 * @param         string $destination
	 * @param         array  $conf
	 * @returns       void
	 */
	private function deployServer($server, $destination, $conf) {
		printf("=== Deploying to server %s (%s)\n", $server, $destination);

		// Normalize paths, to make rsync bahave in a consistent way
		$destination         = preg_replace('#/*$#', '', $destination);
		$conf['source']      = preg_replace('#/*$#', '', $conf['source']);
		$conf['source']     .= '/';

		$rsyncParams = array();
		$SSHParams = array();

		if(isset($conf['exclude'])) {
			$patterns = preg_split('/\s+/', $conf['exclude']);
			$rsyncParams[] = $this->generateExcludeString($patterns);
		}
		if(isset($conf['options'])) {
			$rsyncParams[] = $conf['options'];
		}
		if(isset($conf['port'])) {
			$rsyncParams[] = '-e \'ssh -p '. $conf['port'] .'\'';
			$SSHParams[] = '-p '. $conf['port'];
		}

		$this->prepare($conf);

		printf("\n--- starting rsync\n");
		$rsync = sprintf(
			'rsync -avz --delete-after %s %s %s',
			join(' ', $rsyncParams),
			$this->checkoutDir .'/'. $conf['source'],
			$server .':'. $destination
		);
		printf(">>> %s\n", $rsync);
		$this->pretend || shell_exec($rsync);

		if(isset($conf['action'])) {
			printf("\n--- additional deployment actions\n");
			foreach($conf['action'] as $action) {
				$cmd = sprintf(
					'ssh %s %s %s',
					join(' ', $SSHParams),
					$server,
					escapeshellarg($action)
				);
				printf(">>> %s\n", $cmd);
				$return = 0;
				$this->pretend || system($cmd, $return);
				$this->pretend || ($return && printf("ERROR: statuscode=%d\n", $return));
			}
		}

		$this->cleanUp($conf);
	}

	/**
	 * Prepare the deployment
	 *
	 * Check if there are prepare statements in the deploy
	 * config and run them on the local code.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         array $conf The configuration
	 */
	protected function prepare($conf) {
		printf("\n--- preparing actions before deployment\n");
		if (isset($conf['prepare'])) {
			foreach ($conf['prepare'] as $prepare) {
				printf(">>> %s\n", $prepare);
				$return = 0;
				$this->pretend || system($prepare, $return);
				$this->pretend || ($return && printf("ERROR: statuscode=%d\n", $return));
			}
		} else {
			printf(">>> none\n");
		}
	}

	/**
	 * Cleanup the deployment
	 *
	 * Check if there are cleanup statements in the deploy config
	 * and run them on the local code.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         array $conf The configuration
	 */
	protected function cleanUp($conf) {
		printf("\n--- cleanup actions after the deployment\n");
		if (isset($conf['cleanup'])) {
			foreach ($conf['cleanup'] as $cleanup) {
				printf(">>> %s\n", $cleanup);
				$return = 0;
				$this->pretend || system($cleanup, $return);
				$this->pretend || ($return && printf("ERROR: statuscode=%d\n", $return));
			}
		} else {
			printf(">>> none\n");
		}
	}
	
	/**
	 * Generate an exclude string for rsync
	 *
	 * @author      Stefan Heinemann <sh@adfinis.Com>
	 * @param       array $excludes
	 * @return      string
	 */
	public function generateExcludeString(array $excludes) {
		$string = "";
		foreach ($excludes as $e) {
			$string = sprintf('%s --exclude=%s', $string, $e);
		}

		return $string;
	}
}

