<?php

namespace org\adfinis\framework\deploy;

use \org\adfinis\framework\config\ConfManager;
use \org\adfinis\framework\db\DbManager as DBInternalManager;


/**
 * Classwrapper for porting old DbManager API to the new one
 *
 * @author          Matthieu Riolo
 *
 */
class DbManager extends DBInternalManager {

	public static function fetchOne($q) {
		return self::getInstance()->fetchOne($q);
	}

	public static function query($q) {
		return self::getInstance()->query($q);
	}

	public static function fetchRow($q) {
		return self::getInstance()->fetchRow($q);
	}
}




/**
 * Execute update scripts
 *
 * This class executes update scripts, which would usually be
 * called while deploying the project. It collects all *.sql and
 * *.php files in a certain folder (usually data/updates) and
 * executes them if they are newer than the current version of
 * the project. The version of the project is saved into the
 * database, usually the table _version.
 * 3 kinds of updates can be made:
 * - Update to newest version (updateAll())
 * - Update up to a certain version (updateRange())
 * - Update a single version (updateSingle())
 * @author        Stefan Heinemann <sh@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class UpdateHelper {
	/**
	 * The location of the update files
	 *
	 * @var           string $updateLocation
	 */
	protected $updateLocation = "../updates/";

	/**
	 * The table name that contains the version
	 *
	 * @var           string $tableName
	 */
	protected $tableName;

	/**
	 * The current version of the project
	 *
	 * @var           int $version
	 */
	protected $version = -1;

	/**
	 * The date the last update was performed
	 *
	 * @var           string $updated
	 */
	protected $updated = null;

	/**
	 * Do some initial checks
	 */
	public function __construct($updateLocation) {
		$this->updateLocation = $updateLocation;
		$this->checkUpdateLocation();

		$this->tableName =
			ConfManager::getVar('af', 'versionTable');

		if (!$this->checkVersionTable()) {
			$this->createVersionTable();
		} else {
			$this->readCurrentVersion();
		}

	}

	/**
	 * Check whether the table _version exists in the DB
	 *
	 * The _version table is used to track the version of
	 * the project. This function returns whethe this
	 * table exists or not, to be able to create it
	 * on the first usage.
	 * @author       Stefan Heinemann <sh@adfinis.com>
	 * @return       bool
	 */
	protected function checkVersionTable() {
		$query = sprintf('
			SHOW TABLES LIKE "%s"
			',
			$this->tableName
		);

		$res = DbManager::fetchOne($query);
		return (bool)$res;
	}

	/**
	 * Create the _version table
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 */
	protected function createVersionTable() {
		$query = sprintf('
			CREATE TABLE
				`%s`
				(
					`version` INT NOT NULL UNIQUE,
					`updated` TIMESTAMP NOT NULL
				)
			ENGINE = INNODB
			',
			$this->tableName
		);

		DbManager::query($query);
	}

	/**
	 * Check if the update location directory exists
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @throws        UpdateLocationUnavailableException
	 */
	protected function checkUpdateLocation() {
		if (!file_exists($this->updateLocation)) {
			throw new UpdateLocationUnavailableException(
				$this->updateLocation
			);
		}
	}

	/**
	 * Apply all available updates
	 *
	 * Apply all available updates that are newer
	 * than the current version.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 */
	public function updateAll() {
		$this->msg(
			sprintf(
				"Updating everything from version %s (%s)",
				$this->version,
				$this->updated
			)
		);

		$files = $this->readUpdateFiles();
		if ($files === null) {
			$this->msg("No updates found");
			exit(0);
		}

		foreach ($files as $file) {
			$this->runFile($file);
		}
	}

	/**
	 * Update to a certain version
	 *
	 * Only perform the updates up to a certain version.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         int $version
	 */
	public function updateRange($version) {
		$this->msg(
			sprintf(
				"Updating from version %s (%s) to %s",
				$this->version,
				$this->updated,
				$version
			)
		);

		$files = $this->readUpdateFiles();
		if ($files === null) {
			$this->msg("No updates found");
			exit(0);
		}

		foreach ($files as $file) {
			if ($file['version'] <= $version) {
				$this->runFile($file);
			}
		}
	}

	/**
	 * Update just one single version
	 *
	 * Check if the previous updates have been performed
	 * already.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @throws        VersionsMissingException
	 */
	public function updateSingle($version) {
		if ($version > $this->version + 1) {
			throw new VersionsMissingException($this->version, $version);
		}
		$this->msg(sprintf("Updating system to version %s", $version));

		if ($version <= $this->version) {
			$msg = sprintf("The update %s has already been performed", $version);
			$this->msg($msg);
			return;
		}

		$file = $this->checkFile($version);
		if (!$file) {
			$msg = sprintf("The update %s has already been performed", $version);
			$this->msg($msg);
			return;
		}

		$this->runFile($file);
	}

	/**
	 * Run an update file
	 */
	protected function runFile($file) {
		$updateFunc = null;
		if ($file['type'] == 'sql') {
			$updateFunc = "updateSQL";
		} else {
			$updateFunc = "updatePHP";
		}

		$this->msg(sprintf("Applying update %s", $file['filename']));

		$this->$updateFunc($file['filename']);
		$this->msg(
			sprintf("Successfully applied update %s", $file['filename'])
		);
		$this->updateVersion($file['version']);
	}

	/**
	 * Read the current version from the DB
	 *
	 * Read the current version and the update time from
	 * the DB and save it to the object.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 */
	protected function readCurrentVersion() {
		$query = sprintf('
			SELECT
				IF(MAX(`version`) IS NULL, -1, MAX(`version`)) AS `version`,
				MAX(`updated`)                                AS `updated`
			FROM
				`%s`
			',
			$this->tableName
		);

		$res = DbManager::fetchRow($query);
		$this->version = intval($res['version']);
		$this->updated = $res['updated'];
	}


	/**
	 * Read the update directory
	 *
	 * Read all the .php and .sql files in the directory
	 * and return those that have a bigger version number than
	 * the given one.
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @returns       array
	 */
	protected function readUpdateFiles() {
		$dir = scandir($this->updateLocation);
		$files = array();
		foreach ($dir as $f) {
			$fileData = $this->checkFile($f, $this->version);
			if ($fileData) {
				$files[] = $fileData;
			}
		}

		if (count($files) == 0) {
			return null;
		}

		sort($files);
		return $files;
	}

	/**
	 * Check if a file is an update
	 *
	 * Check if a file is an update. This includes files
	 * that end with .sql or .php and have a number as their
	 * filename. If it is an update, return the file's values
	 * like filename, version and type.
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $file
	 * @returns       array/bool
	 */
	protected function checkFile($file) {
		if (preg_match('/^(\d+).*?\.(php|sql)$/i', $file, $matches)) {
			$fileVersion = intval($matches[1]);
			if ($fileVersion > $this->version) {
				return array(
					'filename' => $file,
					'version'  => $fileVersion,
					'type'     => $matches[2]
				);
			}
		}

		return false;
	}

	/**
	 * Print out a message
	 *
	 * Might be overwritten to print to another location
	 * or a log or something.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $str The message
	 */
	public function msg($str) {
		print sprintf("%s\n", $str);
	}

	/**
	 * Update the version in the database
	 *
	 * After an update script is applied save the version number
	 * to the database
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         int $version
	 */
	protected function updateVersion($version) {
		$query = sprintf('
			INSERT INTO
				`%s`
			(`version`, `updated`)
			VALUES
			(%s, NOW())
			',
			$this->tableName,
			$version
		);

		DbManager::query($query);
	}

	/**
	 * Apply an SQL file
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $file
	 * @throws        Exception
	 */
	protected function updateSQL($file) {
		$status = \org\adfinis\framework\utils\Shell::runCmd(sprintf(
			'mysql -u %s -h%s -p%s %s --default-character-set=%s < %s',
			escapeshellarg(ConfManager::getVar('af', 'DB', 'default', 'user')),
			escapeshellarg(ConfManager::getVar('af', 'DB', 'default', 'host')),
			escapeshellarg(ConfManager::getVar('af', 'DB', 'default', 'pass')),
			escapeshellarg(ConfManager::getVar('af', 'DB', 'default', 'DBName')),
			escapeshellarg(ConfManager::getVar('af', 'DB', 'default', 'charset')),
			escapeshellarg($this->getFilePath($file))
		));
		if($status['return']) {
			throw new \Exception($status['stderr']);
		}

		return;
	}

	/**
	 * Apply a PHP file
	 *
	 * The return value contains whether the update script
	 * was successful or not, which is very important since
	 * the update process stops when there's an error.
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $file
	 * @throws
	 */
	protected function updatePHP($file) {
		$cliPath = sprintf("%s/bin/cliExec.sh", _CONF_FW);

		$path = $this->getFilePath($file);
		$output = system(sprintf("%s %s", $cliPath, $path), $retval);

		if ($retval != 0) {
			throw new PHPUpdateException($path, $output, $retval);
		}
	}

	/**
	 * Return the file path
	 *
	 * @author       Stefan Heinemann <sh@adfinis.com>
	 * @param        string $file
	 * @return       string
	 */
	protected function getFilePath($file) {
		return sprintf("%s/%s", $this->updateLocation, $file);
	}
}
