<?php

namespace org\adfinis\framework\entity;

use \org\adfinis\framework\db\DbManager;

/**
 * Class to deal with entity specific data
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2010 - 2011
 * @license       BSD 3-Clause
 */
abstract class Entity implements \IteratorAggregate {

	/**
	 * Holds the entity information.
	 * @var           array $container
	 */
	protected $container = array();

	/**
	 * Table of the entity.
	 * @var           string $table
	 * @todo by dv:   What is this table good for?
	 */
	protected $table;

	/**
	 * Primary key field of the entity.
	 * @var           string $entityIDField
	 */
	protected $entityIDField;

	/**
	 * Holds instances of the entities to implement the Identity-Map pattern.
	 *
	 * The array is built like this:
	 *
	 * array(
	 * 	'class1' = array(
	 * 		1 => class1instance,
	 * 		2 => class1instance
	 * 	),
	 * 	'class2' => array(
	 * 		1 => class2instance
	 * 	)
	 * )
	 *
	 * @var           array<array<Entity>> $identityMap
	 */
	private static $identityMap;

	/**
	 * Constructor of the entity class
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $table
	 * @param         string $entityIDField
	 * @param         int    $entityID      (optional)
	 */
	public function __construct($table, $entityIDField, $entityID = null){
		$this->table         = $table;
		$this->entityIDField = $entityIDField;

		if($entityID !== null){
			$this->getDataByEntityID($entityID);
		}

		// Initialize ID field to prevent access on undefined index
		if(!sizeof($this->container)){
			$this->container[$entityIDField] = null;
		}
	}

	/**
	 * Populates this object with the given entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        DatabaseException
	 * @param         int $entityID
	 * @returns       this
	 */
	public function getDataByEntityID($entityID){
		return $this->getDataByField($this->entityIDField, $entityID);
	}

	/**
	 * Populates this object by a specific database field
	 * WARNING: field _must_ be a unique field and _must not_ be user input
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        DatabaseException
	 * @param         string $field
	 * @param         string $value
	 * @returns       this
	 */
	protected function getDataByField($field, $value){
		$preparedStatement = DbManager::prepare(
			sprintf('SELECT * FROM `%s` WHERE `%s` = ?', $this->table, $field)
		);
		$preparedStatement->execute(array($value));
		foreach($preparedStatement->fetch() ?: array() as $field => $value){
			$this->setValue($field, $value);
		}

		self::$identityMap[get_called_class()][$this->getID()] = $this;

		return $this;
	}

	/**
	 * Checks if a given field value is already in use
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $field
	 * @param         string $value
	 * @returns       bool
	 */
	protected function checkUniqueField($field, $value){
		if($this->container[$field] == $value) return false;

		$preparedStatement = DbManager::prepare(
			sprintf('SELECT * FROM `%s` WHERE `%s` = ?', $this->table, $field));
		$preparedStatement->execute(array($value));

		return (bool)$preparedStatement->fetch();
	}

	/**
	 * Updates the this entity in the database, if no entityID is given,
	 * the entity will be added as a new row instead
	 * please make sure all values are sane
	 * returns the entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        LogicException
	 * @throws        DatabaseException
	 * @returns       int
	 */
	public function save(){
		// if we don't have an entityID we're dealing with a new entity
		if($this->container[$this->entityIDField] == null) return $this->add();

		// prepare update query "SET" part
		$setParts = array();
		foreach($this->container as $key => $value){
			// skip entityID as it is in the WHERE part
			if($key === $this->entityIDField) continue;

			$setParts[] = sprintf('`%1$s` = :%1$s', $key);
		}

		$query = sprintf('UPDATE `%1$s` SET %2$s WHERE `%3$s` = :%3$s',
			$this->table,
			implode(', ', $setParts),
			$this->entityIDField);
		DbManager::prepare($query)->execute($this->container);

		return $this->container[$this->entityIDField];
	}

	/**
	 * Adds the current entity into the database
	 * returns the new entitys entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        DatabaseException
	 * @returns       int
	 */
	private function add(){
		unset($this->container[$this->entityIDField]);

		$fieldParts      = array();
		$fieldValueParts = array();
		foreach($this->container as $key => $value){
			$fieldParts[]      = sprintf('`%s`', $key);
			$fieldValueParts[] = sprintf(':%s',  $key);
		}

		$query = sprintf('INSERT INTO `%s` (%s) VALUES (%s)',
			$this->table,
			implode(', ', $fieldParts),
			implode(', ', $fieldValueParts));
		DbManager::prepare($query)->execute($this->container);

		$entityID = DbManager::lastInsertId();
		$this->container[$this->entityIDField] = $entityID;

		self::$identityMap[get_called_class()][$entityID] = $this;

		return $entityID;
	}

	/**
	 * Returns the ID of this entity
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       int
	 */
	public function getID(){
		return isset($this->container[$this->entityIDField])
			? (int)$this->container[$this->entityIDField]
			: 0;
	}

	/**
	 * Sets the ID of this entity
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $ID
	 * @returns       this
	 */
	public function setID($ID){
		$ID = (int)$ID;

		// Remove old reference if ID changed
		if($ID !== $this->getID() && static::inIdentityMap($this->getID())) {
			static::removeFromIdentityMap($this->getID());
		}

		$this->container[$this->entityIDField] = $ID;

		$this->toIdentityMap();

		return $this;
	}

	/**
	 * Internal setter for properties
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $offset
	 * @param         mixed  $value
	 * @returns       this
	 */
	protected function setValue($offset, $value){
		if($offset === $this->entityIDField) return $this->setID($value);

		$this->container[$offset] = $value;

		return $this;
	}

	/**
	 * Property setter
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @param          mixed $offset
	 * @param          mixed $value
	 * @returns        void
	 */
	public function __set($offset, $value){
		$this->setValue($offset, $value);
	}

	/**
	 * This method implements the IteratorAggregate interface
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @returns        array
	 */
	public function getIterator(){
		return new \ArrayIterator($this->container);
	}

	/**
	 * Magic isset method for properties
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @param          mixed $offset
	 * @returns        bool
	 */
	public function __isset($offset){
		return array_key_exists($offset, $this->container);
	}

	/**
	 * Property unsetter
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         mixed $offset
	 * @returns       void
	 */
	public function __unset($offset){
		unset($this->container[$offset]);
	}

	/**
	 * Property getter
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         mixed $offset
	 * @returns       mixed
	 */
	public function &__get($offset){
		if(isset($this->container[$offset])){
			$value = &$this->container[$offset];
		}
		else {
			$value = null;
		}

		return $value;
	}

	/**
	 * Check if the given ID is in the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $entityID
	 * @returns       bool
	 */
	public static function inIdentityMap($entityID){
		return isset(self::$identityMap[get_called_class()][(int)$entityID]);
	}

	/**
	 * Stores the object in the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        \RuntimeException
	 * @returns       void
	 */
	public function toIdentityMap(){
		$class    = get_called_class();
		$entityID = $this->getID();

		if(!$entityID){
			throw new \RuntimeException(sprintf(
				tr('%s can not be stored in %s::$identityMap with an invalid ID %d'),
				$class, get_class(), $entityID
			));
		}

		if(isset(self::$identityMap[$class][$entityID])){
			$e = new \RuntimeException(sprintf(
				tr('%s with ID %d is already stored in %s::$identityMap'),
				$class, $entityID, get_class()
			));

			trigger_error($e);

			throw $e;
		}

		self::$identityMap[$class][$entityID] = $this;
	}

	/**
	 * Removes the picture from the identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int|array $entityID
	 * @returns       void
	 */
	public static function removeFromIdentityMap($entityID){
		$class = get_called_class();

		if(is_array($entityID)) {
			foreach($entityID as $ID) {
				unset(self::$identityMap[$class][$ID]);
			}
		}
		else {
			unset(self::$identityMap[$class][$entityID]);
		}
	}

	/**
	 * Create Entity by entityID
	 *
	 * This method implements the Identity-Map pattern
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $entityID
	 * @returns       Entity
	 */
	public static function getByID($entityID){
		$class = get_called_class();

		if(isset(self::$identityMap[$class][$entityID])){
			return self::$identityMap[$class][$entityID];
		}

		$entity = new static();
		$entity->getDataByEntityID($entityID);

		self::$identityMap[$class][$entityID] = $entity;

		return $entity;
	}
}
