<?php

namespace org\adfinis\framework\entity;

/**
 * Class to deal with entity preferences
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2010
 * @license       BSD 3-Clause
 */
class EntityPreference implements EntityPreferenceInterface {

	/**
	 * Holds the preferences.
	 * @var           array $container
	 */
	private $container = array();

	/**
	 * Entity type.
	 * @var           string $entity
	 */
	private $entity;

	/**
	 * Entity ID.
	 * @var           int $entityID
	 */
	private $entityID;

	/**
	 * Holds the preference IDs.
	 * @var           array $preferences
	 */
	private $preferences = array();

	/**
	 * Constructor of the preference class
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $entity
	 */
	public function __construct($entity){
		$this->entity = $entity;

		$query = '
			SELECT
				*
			FROM
				`preference`
			WHERE
				`entity` = ?
		';
		$preparedStatement = \dbManager::prepareQuery($query);
		$preparedStatement->execute(array($this->entity));

		foreach($preparedStatement as $preference){
			// save the preference IDs and datatypes, they're needed when we save them
			$this->preferences[$preference['name']] = array(
				'preferenceID' => $preference['preferenceID'],
				'type' => $preference['type'],
				'defaultValue' => $preference['defaultValue']
			);

			/* set type of the value field
			 * the mysql driver only returns strings
			 */
			if(isset($preference['defaultValue'])){
				settype($preference['defaultValue'], $preference['type']);
			} else {
				$preference['defaultValue'] = null;
			}

			if(intval($preference['isArray'])){
				// set an empty array if value is null
				$this->container[$preference['name']] = ($preference['defaultValue'] === null
					? new ArrayObject()
					: new ArrayObject(array($preference['defaultValue']))
				);
			} else {
				$this->container[$preference['name']] = $preference['defaultValue'];
			}
		}

		return $this;
	}

	/**
	 * Populates this object by entity id (user, company etc.)
	 * if entityID is given it will be set in the object too
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        DatabaseException
	 * @param         int $entityID
	 * @returns       this
	 */
	public function getDataByEntityID($entityID = null){
		$this->entityID = ($entityID === null
			? $this->entityID
			: $entityID);

		$query = sprintf('
			SELECT
				*
			FROM
				`rel%1$sPreference`
			JOIN
				`preference` USING(`preferenceID`)
			WHERE
				`%1$sID` = ?
			',
			ucfirst($this->entity)
		);
		$preparedStatement = \dbManager::prepareQuery($query);
		$preparedStatement->execute(array($this->entityID));

		foreach($preparedStatement as $preference){
			/* set type of the value field
			 * the mysql driver only returns strings
			 */
			settype($preference['value'], $preference['type']);

			if(intval($preference['isArray']) == 1){
				$this->container[$preference['name']] = new ArrayObject(
					array($preference['arrayKey'] => $preference['value'])
				);
			} else {
				$this->container[$preference['name']] = $preference['value'];
			}
		}

		return $this;
	}

	/**
	 * Sets the entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         int $entityID
	 * @returns       this
	 */
	public function setEntityID($entityID){
		$this->entityID = (int)$entityID;

		return $this;
	}

	/**
	 * Gets the entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       int|null
	 */
	public function getEntityID(){
		return $this->entityID;
	}

	/**
	 * Deletes all user preferences in the database
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        LogicException
	 * @throws        DatabaseException
	 * @returns       void
	 */
	private function delete(){
		if(!isset($this->entityID)){
			throw new LogicException(tr('Cannot delete preferences without set entityID'));
		}

		$query = sprintf('DELETE FROM `rel%1$sPreference` WHERE `%1$sID` = ?',
			ucfirst($this->entity));
		\dbManager::prepareQuery($query)->execute(array($this->entityID));
	}

	/**
	 * Updates the user preferences in the database,
	 * please make sure all datatypes are correct
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @todo by ds:   only save modified preferences
	 * @throws        LogicException
	 * @throws        DatabaseException
	 * @returns       void
	 */
	public function save(){
		if(!sizeof($this->container)) return;

		if(!isset($this->entityID)){
			throw new LogicException(tr('Cannot save preferences without set entityID'));
		}

		/* Clear database first because we don't know which rows got updated
		 * and which were added
		 */
		$this->delete();

		$query = sprintf('
			INSERT INTO
				`rel%1$sPreference` (
					`preferenceID`,
					`%1$sID`,
					`arrayKey`,
					`value`
				) VALUES (
					:preferenceID,
					:entityID,
					:arrayKey,
					:value
				)
			',
			ucfirst($this->entity)
		);
		$preparedStatement = \dbManager::prepareQuery($query);

		foreach($this->container as $name => $value){
			// check if we got an array and save each value
			if($value instanceof ArrayObject){
				foreach($value as $key => $singleValue){
					settype($singleValue, $this->preferences[$name]['type']);
					$preparedStatement->execute(array(
						'preferenceID' => $this->preferences[$name]['preferenceID'],
						'entityID'     => $this->entityID,
						'arrayKey'     => $key,
						'value'        => $singleValue
					));
				}
			} else {
				settype($value, $this->preferences[$name]['type']);
				$preparedStatement->execute(array(
					'preferenceID' => $this->preferences[$name]['preferenceID'],
					'entityID'     => $this->entityID,
					'arrayKey'     => 0, // array index is always 0 on non arrays
					'value'        => $value
				));
			}
		}
	}

	/**
	 * This method is called if this object is in an iterator context
	 * (i.e. foreach)
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       ArrayIterator
	 */
	public function getIterator(){
		return new ArrayIterator($this->container);
	}

	/**
	 * This method is called from context if an index is set
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @param          mixed $offset
	 * @param          mixed $value
	 * @returns        void
	 */
	public function offsetSet($offset, $value){
		if(!sizeof($this->container)) $this->getDataByEntityID();

		if(array_key_exists($offset, $this->container)){
			settype($value, $this->preferences[$offset]['type']);
			$this->container[$offset] = $value;
		} else {
			throw new InvalidArgumentException(tr('Cannot set undefined index from context'));
		}
	}

	/**
	 * This method is called if isset/array_key_exists looks up an index
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @param          mixed $offset
	 * @returns        bool
	 */
	public function offsetExists($offset){
		if(!sizeof($this->container)) $this->getDataByEntityID();

		return array_key_exists($offset, $this->container);
	}

	/**
	 * This method is called if unset tries to unset an index
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         mixed $offset
	 * @returns       void
	 */
	public function offsetUnset($offset){
		$this->offsetSet($offset, $this->preferences[$offset]['defaultValue']);
	}

	/**
	 * This method is called from context if an index is accessed
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         mixed $offset
	 * @returns       mixed
	 */
	public function offsetGet($offset){
		if(!sizeof($this->container)) $this->getDataByEntityID();

		return (isset($this->container[$offset])
			? $this->container[$offset]
			: null);
	}
}
