<?php

namespace org\adfinis\framework\entity;

/**
 * Class to deal with entity specific data
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2010
 * @license       BSD 3-Clause
 */
abstract class EntityWithPreferences extends Entity {

	/**
	 * Holds the entity preferences.
	 * @var           Preference $preference
	 */
	public $preference = null;

	/**
	 * Constructor of the entity class
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $table
	 * @param         string $entityIDField
	 * @param         int    $entityID      (optional)
	 * @param         string $entityType    Table name of the entity.
	 */
	public function __construct($table, $entityIDField, $entityID = null, $entityType){
		parent::__construct($table, $entityIDField, $entityID);

		if($this->preference === null){
			$this->preference = new EntityPreference($entityType);
		}

		if($this->container[$this->entityIDField]){
			$this->preference->setEntityID($this->container[$this->entityIDField]);
		}
	}

	/**
	 * Populates this object by a specific database field
	 * WARNING: field _must_ be a unique field and _must not_ be user input
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        DatabaseException
	 * @param         string $field
	 * @param         string $value
	 * @returns       this
	 */
	protected function getDataByField($field, $value){
		parent::getDataByField($field, $value);

		$this->preference->setEntityID($this->container[$this->entityIDField]);

		return $this;
	}

	/**
	 * Updates the this entity in the database, if no entityID is given,
	 * the entity will be added as a new row instead
	 * please make sure all values are sane
	 * returns the entityID
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        LogicException
	 * @throws        DatabaseException
	 * @returns       int
	 */
	public function save(){
		$entityID = parent::save();
		$this->preference->setEntityID($entityID)->save();

		return $entityID;
	}

	/**
	 * Internal setter for properties
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $offset
	 * @param         mixed  $value
	 * @returns       this
	 */
	protected function setValue($offset, $value){
		if($offset === $this->entityIDField){
			if($this->preference instanceof EntityPreferenceInterface){
				$this->preference->setEntityID($value);
			}
		}

		return parent::setValue($offset, $value);
	}

	/**
	 * This method implements the IteratorAggregate interface
	 *
	 * @author         Damian Senn <ds@adfinis.com>
	 * @returns        ArrayIterator
	 */
	public function getIterator(){
		return new \ArrayIterator($this->container);
	}
}
