<?php

namespace org\adfinis\framework\event;

/**
 * Base class for event emitters
 *
 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
 */
class Emitter {

	/**
	 * Holds the callbacks for each event
	 *
	 * @var           array<callable> $callbacks
	 */
	private $callbacks = array();

	/**
	 * Attach an event handler
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @throws        \InvalidArgumentException
	 * @param         string   $event
	 * @param         callable $callback
	 * @returns       Emitter
	 */
	public function on($event, $callback) {
		if (!is_callable($callback)) {
			throw new \InvalidArgumentException('Argument is not callable!');
		}

		$this->callbacks[$event][] = $callback;

		return $this;
	}

	/**
	 * Dettach all or a specific event handler
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @param         string   $event
	 * @param         callable $callback [optional]
	 * @returns       Emitter
	 */
	public function off($event, $callback = null) {
		if ($event === null) {
			$this->callbacks = array();

			return;
		}

		if (!$callback) {
			unset($this->callbacks[$event]);
		}
		else {
			$index = in_array($callback, $this->callbacks[$event], true);

			if ($index !== false) {
				unset($this->callbacks[$event][$index]);
			}
		}

		return $this;
	}

	/**
	 * Execute an event handler only once
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @throws        \InvalidArgumentException
	 * @param         string   $event
	 * @param         callable $callback
	 * @returns       Emitter
	 */
	public function once($event, $callback) {
		if (!is_callable($callback)) {
			throw new \InvalidArgumentException('Argument is not callable!');
		}

		$wrapper = function($self) use ($callback, $wrapper) {
			call_user_func_array($callback, func_get_args());

			$self->off($event, $wrapper);
		};

		return $this->on($event, $wrapper);
	}

	/**
	 * Trigger an event
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @param         string $event
	 * @returns       Emitter
	 */
	public function emit($event) {
		if (!isset($this->callbacks[$event])) return;

		$args = array_slice(func_get_args(), 1);

		array_unshift($args, $this);

		foreach ($this->callbacks[$event] as $callback) {
			call_user_func_array($callback, $args);
		}

		return $this;
	}
}
