<?php

namespace org\adfinis\framework\exceptions;

use \org\adfinis\framework\config\ConfManager;

/**
 * Exception Handler class
 *
 * @author        Damian Senn <ds@adfinis.com>
 */
class Handler {

	/**
	 * Holds the exception to handle
	 *
	 * @var            \Exception $e
	 */
	private $e;

	/**
	 * Constructor of the exception handler
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         \Exception $e
	 */
	public function __construct(\Exception $e) {
		$this->e = $e;
	}

	/**
	 * Get the thrown exception
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       \Exception
	 */
	public function getException() {
		return $this->e;
	}

	/**
	 * Get the exception message
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getMessage() {
		return sprintf('Uncaught %s', $this->e);
	}

	/**
	 * Get the exception message formatted for HTML
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getHTML() {
		return sprintf('<pre class="php-exception php-exception-%s">%s</pre>',
			strtolower(get_class($this->e)),
			$this->getMessage()
		);
	}

	/**
	 * Get the exception as JSON object
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getJSON() {
		$e = $this->e;

		return json_encode(array(
			'error'     => true,
			'message'   => $this->getMessage(),
			'exception' => array(
				'name'    => get_class($e),
				'message' => $e->getMessage(),
				'code'    => $e->getCode(),
				'file'    => $e->getFile(),
				'line'    => $e->getLine(),
				'trace'   => $e->getTrace()
			)
		));
	}

	/**
	 * Get the exception as XML
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getXML() {
		$e = $this->e;

		$xml = new \XMLWriter();
		$xml->openMemory();
		$xml->startDocument();

		$xml->startElement('error');

		$xml->writeElement('message', $this->getMessage());

		$xml->startElement('exception');

		$xml->writeElement('name',    get_class($e));
		$xml->writeElement('message', $e->getMessage());
		$xml->writeElement('code',    $e->getCode());
		$xml->writeElement('file',    $e->getFile());
		$xml->writeElement('line',    $e->getLine());
		$xml->writeElement('trace',   $e->getTraceAsString());

		$xml->endElement(); // exception

		$xml->endElement(); // error

		return $xml->outputMemory();
	}

	/**
	 * Show the message to the user
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function show() {
		if ($this->isCLI()) {
			$this->showCLI();
		}
		else if ($this->isXHR()) {
			if (strpos($_SERVER['HTTP_ACCEPT'], 'application/json') !== false) {
				$this->showJSON();
			}
			else if (strpos($_SERVER['HTTP_ACCEPT'], 'application/xml') !== false) {
				$this->showXML();
			}
			else {
				$this->showHTML();
			}
		}
		else {
			$this->showHTML();
		}
	}

	/**
	 * Method outputs the message on the command line interface
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function showCLI() {
		echo $this->getMessage();
	}

	/**
	 * Method outputs the message as JSON on a XMLHttpRequest
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function showJSON() {
		header('Content-type: application/json');
		echo $this->getJSON();
	}

	/**
	 * Method outputs the message as XML on a XMLHttpRequest
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function showXML() {
		header('Content-type: application/xml');
		echo $this->getXML();
	}

	/**
	 * Method outputs the message as HTML
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function showHTML() {
		echo $this->getHTML();
	}

	/**
	 * Was the script called from a XMLHttpRequest?
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       bool
	 */
	public function isXHR() {
		return isset($_SERVER['HTTP_X_REQUESTED_WITH']);
	}

	/**
	 * Was the script called from cli
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       bool
	 */
	public function isCLI() {
		return \PHP_SAPI === 'cli';
	}

	/**
	 * Send an email to the developers
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function sendEmail() {
		try {
			$mailto = ConfManager::getVar('af', 'handler', 'exception', 'mailto');

			if (!$mailto) return;

			$mail = new \Zend_Mail('UTF-8');

			if (!$this->isCLI()) {
				$mail->addHeader('X-Originating-IP', $_SERVER['REMOTE_ADDR']);
			}

			$mail->setSubject('Uncaught exception');
			$mail->setFrom(
				sprintf('php@%s', $_SERVER['SERVER_NAME']),
				$_SERVER['SERVER_NAME']
			);
			$mail->addTo($mailto);

			$text  = self::getMessage() . "\n";
			$text .= sprintf("\n\$_SERVER = %s",  print_r($_SERVER,  true));
			$text .= sprintf("\n\$_POST = %s",    print_r($_POST,    true));
			$text .= sprintf("\n\$_GET = %s",     print_r($_GET,     true));
			$text .= sprintf("\n\$_SESSION = %s", print_r($_SESSION, true));

			$mail->setBodyText($text, 'UTF-8', \Zend_Mime::ENCODING_QUOTEDPRINTABLE);

			$mail->send(new \Zend_Mail_Transport_Smtp(ConfManager::getVar('af', 'smtp')));
		}
		catch (\Exception $e) {
			trigger_error($e->getMessage(), \E_USER_WARNING);
		}
	}

	/**
	 * Setup the exception handler
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public static function init() {
		$class = get_called_class();

		set_exception_handler(function($e) use ($class) {
			try {
				$handler = new $class($e);
				$handler->show();

				// Mails are only sent if the af.handler.exception.mailto config is set
				$handler->sendEmail();
			}
			catch(\Exception $e) {
				die("Oh my... something went horribly wrong here!\n"
				   ."============================================\n$e");
			}
		});
	}
}
