<?php

namespace org\adfinis\framework\export;

/**
 * A simple exporter for excel files
 *
 * This class provides a simple interface to export data into excel
 * files. Basically only a path can be provided, then appendData()
 * called to append data to the excel file and eventually save() to 
 * save the file. 
 * @author        Stefan Heinemann <sh@adfinis.com>
 */
class ExportExcelSimple {
	/**
	 * The path to the file to write to
	 *
	 * @var           string $path
	 */
	protected $path;
	
	/**
	 * The PHPExcel object
	 *
	 * @var           obj $excel
	 */
	protected $excel;

	/**
	 * Row position
	 *
	 * @var           int $row
	 */
	protected $row = 1;

	/**
	 * Whether it's been saved already
	 *
	 * @var           bool $saved
	 */
	protected $saved = false;

	/**
	 * Date format
	 *
	 * The date format to set to the cell when provided with a
	 * Zend_Date 
	 * @var           string $dateFormat
	 */
	protected $dateFormat = "DD.MM.YYYY HH:MM:SS";
	

	/**
	 * Create an ExportExcelSimple instance
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 */
	public function __construct($path) {
		$this->path  = $path;

		$this->excel = new \PHPExcel();

		$this->excel->setActiveSheetIndex(0);
		$this->sheet = $this->excel->getActiveSheet();
	}

	/**
	 * Append the data
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         array $data
	 */
	public function appendData($data) {
		if ($this->saved) {
			throw new Exception("File has already been saved");
		}

		$col = 0;

		foreach ($data as $column) {
			if ($column instanceof \Zend_Date) {
				$this->insertDate($column, $this->row, $col++);
			} else {
				$this->insertText($column, $this->row, $col++);
			}
		}

		$this->row++;
	}

	/**
	 * Insert text into a cell
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $text
	 * @param         int $row
	 * @param         int $col
	 */
	protected function insertText($text, $row, $col) {
		$this->sheet->setCellValueByColumnAndRow(
			$col,
			$row,
			(string)$text
		);
	}

	/**
	 * Insert a date
	 *
	 * A date needs to be converted to a special format
	 * before inserted
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         Zend_Date $date
	 * @param         int $row
	 * @param         int $col
	 */
	protected function insertDate($date, $row, $col) {
		$time = $date->get(\Zend_Date::TIMESTAMP); 
		$rawDiff = $date->get(\Zend_Date::GMT_DIFF);
		$sign    = (substr($rawDiff, 0, 1) == '+') ? 1 : -1;
		$hours   = intval(substr($rawDiff, 1, 2));
		$minutes = intval(substr($rawDiff, 3, 2));
		$diff    = ($hours * 60 * 60 + $minutes * 60) * $sign;

		$time += $diff;

		$this->sheet->setCellValueByColumnAndRow(
			$col,
			$row,
			\PHPExcel_Shared_Date::PHPToExcel($time)
		);

		$style = $this->sheet->getStyleByColumnAndRow($col, $row);
		$style->getNumberFormat()->setFormatCode(
			$this->dateFormat
		);
	}

	/**
	 * Save the excel file
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 */
	public function save() {
		$writer = new \PHPExcel_Writer_Excel2007($this->excel);
		$writer->save($this->path);
	}

	/**
	 * Set the format of the date
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $format
	 */
	public function setDateFormat($format) {
		$this->dateFormat = $format;
	}
}




