<?php

namespace org\adfinis\framework\form;

use org\adfinis\framework\form\fields\FormField;
use org\adfinis\framework\utils\URLHelper;
use org\adfinis\framework\template\Skittle;
use org\adfinis\framework\request\InputVars;

/**
 * Class to create and handle HTML forms
 *
 * @author      Adrian Wittwer <aw@adfinis.ch>
 * @author      Damian Senn <ds@adfinis.ch>
 * @author      Toni Rudolf <tr@adfinis.ch>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 */
class FormHandler {
	const VALIDATOR_JQUERY = 1;

	/**
	 * Holds FormField objects
	 *
	 * @var array $formFields
	 */
	private $formFields = array();

	/**
	 * Holds the form buttons
	 *
	 * @var array $buttons
	 */
	private $buttons = array();

	/**
	 * Holds the html form attributes
	 *
	 * @var array $formAttributes
	 */
	private $formAttributes = array();

	/**
	 * Holds the errormessages which were thrown by the validateValue method
	 *
	 * @var array $errors
	 */
	private $errors = array();

	/**
	 * Holds the unique identifier field and its value
	 *
	 * @var array $uniqueIdentifier
	 */
	private $uniqueIdentifier = array();

	/**
	 * Groups FormField objects
	 *
	 * @var array $formFieldGroups
	 */
	private $formFieldGroups = array();

	/**
	 * Holds the message after the submit event
	 *
	 * @var string $formMessage
	 */
	private $formMessage = '';

	/**
	 * Skittle/Smarty container. Access using getSmarty()
	 * @var Skittle
	 */
	private $tpl;


	/**
	 * Constructor
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 */
	public function __construct(){
		// set the HTML form attributes
		$this->formAttributes['action'] = URLHelper::getRequestURI();
		$this->formAttributes['method'] = 'post';
		$this->formAttributes['id'] = 'form1';

		$this->tpl = new Skittle();
	}

	/**
	 * adds a form field object to the FormHandler
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       FormField $field
	 * @return      this
	 */
	public function addFormField(FormField $field = null){
		$this->formFields[$field->getName()] = $field;
		return $this;
	}

	/**
	 * adds an array of form fields to the formFields array
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $groupName
	 * @param       array  $fields
	 * @return      this
	 */
	public function addFormFieldGroup($groupName, array $fields){
		$this->formFieldGroups[$groupName] = $fields;

		foreach($fields as $field){
			if($field instanceof FormField){
				$this->formFields[$field->getName()] = $field;
			} else if(is_array($field)){
				foreach($field as $singleField){
					$this->formFields[$singleField->getName()] = $singleField;
				}
			}
		}

		return $this;
	}

	/**
	 * Adds a button
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       string $type
	 * @param       string $name
	 * @param       string $innerHTML
	 * @param       array  $attributes (optional)
	 */
	public function addButton($type, $name, $innerHTML, $attributes = array()){
		$attributes['type'] = $type;
		$attributesString = self::getAttributesString($attributes);
		$this->buttons[$name] = sprintf('<button %s>%s</button>',
			$attributesString, $innerHTML);
	}

	/**
	 * Insert form as new row into the given table
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @throws      Zend_Db_Exception
	 * @param       string $table
	 * @return      int|bool
	 */
	public function insert($table){
		if(count($this->errors) == 0){
			$query = sprintf('
				INSERT INTO
					`%s` (
						%s
					) VALUES (
						%s
					)
				',
				$table,
				implode(',', array_map(
					function($key){
						return sprintf('`%s`',$key);
					},
					array_keys($this->formFields)
				)),
				implode(',', array_map(
					function($key){
						return sprintf(':%s',$key);
					},
					array_keys($this->formFields)
				))
			);
			DbManager::prepare($query)->execute($this->getValues());
			return DbManager::lastInsertId();
		} else {
			return false;
		}
	}

	/**
	 * Updates a table row
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @throws      Zend_Db_Exception
	 * @param       string     $table
	 * @param       string     $field
	 * @param       string|int $value
	 * @return      bool
	 */
	public function update($table, $field, $value){
		if(count($this->errors) == 0){
			$query = sprintf('
				UPDATE
					`%s`
				SET
					%s
				WHERE
					`%s` = %s
				',
				$table,
				implode(',', array_map(
					function($key){
						return sprintf('`%1$s` = :%1$s',$key);
					},
					array_keys($this->getValues())
				)),
				$field,
				DbManager::getInstance()->quote($value)
			);
			DbManager::prepare($query)->execute($this->getValues());
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Sets the value to every FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       array $data
	 * @param       bool  $validate (optional)
	 * @return      bool
	 */
	private function fillForm(array $data, $validate = true){
		$errors = array();
		foreach($this->formFields as $formField){
			$formField->setValue($data[$formField->getName()], $validate);
			$error = $formField->getErrorMessage();
			if($error){
				$errors[$formField->getName()] = $error;
			}
		}
		$this->errors = $errors;

		return (count($errors) > 0);
	}

	/**
	 * Fills the form fields with the given HTTP method
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @see         InputVars
	 * @param       int  $method InputVars::METHOD_* (optional)
	 * @param       bool $validate (optional)
	 * @return      bool
	 */
	public function fillFormGPC($method = InputVars::METHOD_POST, $validate = true){
		if(InputVars::isVarSet($method)){
			$data = array();
			foreach($this->formFields as $formField){
				$data[$formField->getName()] = InputVars::getVar(
					$method,
					$formField->getName(),
					InputVars::TYPE_STR
				);
			}
			$this->fillForm($data, $validate);
			return true;
		}
		return false;
	}

	/**
	 * Fills the form with data from a database table
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @throws      Zend_Db_Exception
	 * @param       string $table
	 * @param       string $where
	 * @param       bool   $validate (optional)
	 * @return      bool
	 */
	public function fillFormDB($table, $where, $validate = false){
		$DBFields = array();
		foreach($this->formFields as $formField){
			$DBFields[] = sprintf('`%s`', $formField->getName());
		}

		$query = sprintf('SELECT %s FROM `%s` WHERE %s',
			implode(',', $DBFields), $table, $where);

		$this->fillForm(DbManager::fetchRow($query), $validate);
		return true;
	}

	/**
	 * Returns the FormField key => value pairs
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      array
	 */
	public function getValues(){
		$data = array();
		foreach($this->formFields as $formField){
			$data[$formField->getName()] = $formField->getValue();
		}
		return $data;
	}

	/**
	 * Sets a form attribute.
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       string $attribute
	 * @param       string $value
	 * @return      void
	 */
	public function setFormAttribute($attribute, $value){
		$this->formAttributes[$attribute] = $value;
	}

	/**
	 * Generates a string of attributes for a HTML element
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       array $attributes
	 * @return      string
	 */
	private static function getAttributesString(array $attributes){
		$attributesStringParts = array();

		foreach($attributes as $attribute => $attributeValue){
			$attributesStringParts[] = sprintf('%s="%s"', $attribute, $attributeValue);
		}

		return implode(' ', $attributesStringParts);
	}

	/**
	 * Sets the form message and its CSS class
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       string $class
	 * @return      void
	 */
	public function setFormMessage($message, $class){
		$this->formMessage = sprintf('<p class="%s">%s</p>', $class, $message);
	}

	/**
	 * Returns the internal smarty/skittle instance
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      Smarty
	 */
	public function getSmarty(){
		return $this->tpl;
	}

	/**
	 * Sets the Smarty instance used in the FormHandler
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       Smarty $smarty
	 * @return      this
	 */
	public function setSmarty(\Smarty $smarty){
		$this->tpl = $smarty;

		return $this;
	}


	/**
	 * Returns the form message as HTML P tag and a specified CSS class
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnFormMessage(){
		return $this->formMessage;
	}

	/**
	 * Returns the whole form as HTML Source
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $template (optional)
	 * @return      string
	 */
	public function returnFormSource($template = 'adfinis/form/form.tpl'){
		$this->tpl->assign('formMessage', $this->returnFormMessage());
		$this->tpl->assign('fields', $this->formFields);
		$this->tpl->assign('fieldGroups', $this->formFieldGroups);
		$this->tpl->assign('buttonSource', implode(' ', $this->buttons));
		$this->tpl->assign('formAttributesString',
			self::getAttributesString($this->formAttributes));

		return $this->tpl->fetch($template);
	}

	/**
	 * This method is used by the AJAX-Validator, it should be called after
	 * the fields are added to the FormHandler. This method returns a JSON-
	 * string and exits with code 0
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      void
	 */
	public function liveValidateAndExit(){
		if(InputVars::isVarSet(InputVars::METHOD_GET, 'ajaxValidate')){
			$fieldName = InputVars::getVar(InputVars::METHOD_GET, 'name');
			$values    = InputVars::getVar(InputVars::METHOD_GET, 'values');
			$formField = $this->formFields[
				preg_replace('/\[.*$/','',$fieldName, -1, $replaced)
			];

			if($replaced < 1){
				$values = $values[0];
			}

			if(!$formField->validateValue($values)){
				$return = array(
					'error'   => true,
					'message' => $formField->getErrorMessage()
				);
			} else {
				$return = array('error' => false, 'message' => '');
			}

			header('Content-type: application/x-json');
			echo json_encode($return);
			exit(0); // die, we're done! :)
		}
	}

	/**
	 * Used for ajax validation of unique fields
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string     $identifierField
	 * @param       string|int $identifierValue
	 * @return      void
	 */
	public function setUniqueIdentifier($identifierField, $identifierValue){
		$this->uniqueIdentifier = array(
			'field' => $identifierField,
			'value' => $identifierValue
		);
	}

	/**
	 * Adds an AJAX-Validator to the FormHandler
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       BasePage $basePage
	 * @param       int      $validator   (optional)
	 * @param       string   $script      (optional)
	 * @param       bool     $checkOnLoad (optional)
	 * @return      void
	 */
	public function addLiveValidator(BasePage $basePage, $validator = self::VALIDATOR_JQUERY, $script = 'index.php', $checkOnLoad = true){
		if(!empty($this->uniqueIdentifier)){
			$script .= sprintf('?%s=%s',
			$this->uniqueIdentifier['field'], $this->uniqueIdentifier['value']);
		}

		switch($validator){
			default:
			case self::VALIDATOR_JQUERY:
				$validatorSRC = sprintf('%s/javascript/validate/jquery.validate.js',
					ConfManager::getVar('af', 'webRoot', 'uri')
				);
				$validatorCallbackSRC = sprintf('%s/javascript/validate/FormHandlerValidator.class.js',
					ConfManager::getVar('af', 'webRoot', 'uri')
				);
				$basePage->addScriptFile($validatorSRC);
				$basePage->addScriptFile($validatorCallbackSRC);

				$basePage->addScriptCode(sprintf('
					$(function(){
						$("form input[type=text], form textarea").not(".noValidate").validate({
							url:
								"%2$s",
							callback:
								FormHandlerValidator.getInputCallback(),
							checkOnLoad:
								%1$s
						});

						$("form .passwordgroup").not(".noValidate").each(function(){
							var $this = $(this),
							    $inputFields = $this.find("input");

							$($inputFields[0]).validate({
								url:
									"%2$s",
								linkFields:
									[$($inputFields[1])],
								callback:
									FormHandlerValidator.getPasswordCallback($inputFields),
								checkOnLoad:
									%1$s
							});
						});
					});
				', ($checkOnLoad ? 'true' : 'false'), $script));
				break;
		}
	}

	/**
	* check if errors occured
	*
	* @author      Adrian Wittwer <aw@adfinis.com>
	* @return      bool
	*/
	public function errorOccurred() {
		return !empty($this->errors);
	}
}
