<?php

namespace org\adfinis\framework\form;

/**
 * Class returning some data check functions
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class FormValidator {
	/**
	 * Checks if a given URI is valid
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       bool   $withScheme
	 * @param       string $scheme (optional)
	 * @return      Closure
	 */
	public static function checkURI($message, $withScheme = true, $scheme = 'http://'){
		return function($URI) use ($withScheme, $scheme, $message){
			if(!$withScheme){
				$URI = sprintf('%s%s',$scheme,$URI);
			}

			return array(
				'error' => !Zend_Uri::check($URI),
				'message' => sprintf($message, $value, $scheme)
			);
		};
	}

	/**
	 * Check if a given hostname is valid
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       bool   $DNS
	 * @param       bool   $IP
	 * @param       bool   $checkEmpty
	 * @return      Closure
	 */
	public static function hostname($message, $DNS = true, $IP = false, $checkEmpty = true){
		return function($hostname) use ($DNS, $IP, $message, $checkEmpty) {
			$zendParam = null;
			if($DNS && $IP){
				$zendParam = Zend_Validate_Hostname::ALLOW_DNS | Zend_Validate_Hostname::ALLOW_IP;
			} elseif($DNS && !$IP){
				$zendParam = Zend_Validate_Hostname::ALLOW_DNS;
			} elseif(!$DNS && $IP){
				$zendParam = Zend_Validate_Hostname::ALLOW_IP;
			}
			$valid = new Zend_Validate_Hostname($zendParam);

			return array(
				'error' =>  (
					!$valid->isValid($hostname) &&
					!(!$checkEmpty && mb_strlen($hostname) === 0)),
				'message' => sprintf($message, $hostname, $DNS, $IP)
			);
		};
	}

	/**
	 * Checks if a unique table field is already set
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       string $table
	 * @param       string $field
	 * @param       mixed  $own (optional)
	 * @return      Closure
	 */
	public static function unique($message, $table, $field, $own = null){
		return function($value) use ($table, $field, $own, $message){
			if($own !== $value){
				$query = sprintf('SELECT `%1$s` FROM `%2$s` WHERE `%1$s` = ?',
					$field, $table);
				$preparedStatement = DbManager::prepare($query);
				$preparedStatement->execute(array($value));

				$error = (bool)$preparedStatement->fetch();
			} else {
				$error = false;
			}

			return array(
				'error' => $error,
				'message' => sprintf($message, $value, $table, $field)
			);
		};
	}

	/**
	 * Checks if a given string length is within a specified character range
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       int    $max        (optional)
	 * @param       int    $min        (optional)
	 * @param       bool   $checkEmpty (optional)
	 * @return      Closure
	 */
	public static function length($message, $max = null, $min = 0, $checkEmpty = true){
		return function($value) use ($min, $max, $message, $checkEmpty){
			$length = mb_strlen($value);

			return array(
				'error' => (
					($length < $min && !(!$checkEmpty && $length === 0)) ||
					($length > $max && $max !== null)),
				'message' => sprintf($message, $max, $min)
			);
		};
	}

	/**
	 * Checks if a given email address is valid
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string                 $message
	 * @param       bool                   $checkMX           (optional)
	 * @param       Zend_Validate_Hostname $hostnameValidator (optional)
	 * @return      Closure
	 */
	public static function email($message, $checkMX = false, Zend_Validate_Hostname $hostnameValidator = null){
		return function($value) use ($checkMX, $hostnameValidator, $message){
			$validator = new Zend_Validate_EmailAddress(
				Zend_Validate_Hostname::ALLOW_DNS,
				$checkMX,
				$hostnameValidator
			);

			return array(
				'error' => !$validator->isValid($value),
				'message' => sprintf($message, $value)
			);
		};
	}

	/**
	 * Checks if a given value is within a specified range.
	 *
	 * Note that the boundaries are INCLUSIVE: The code
	 *
	 *     FormValidator::range('', 0, 5)
	 *
	 * allows the numbers 0, 1, 2, 3, 4, and 5
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @param       int $from
	 * @param       int $to
	 * @return      Closure
	 */
	public static function range($message, $from, $to){

		return function($value) use ($message, $from, $to){
			 #use ($message, $form, $to)
			return array(
				'error' => ($value > $to) || ($value < $from),
				'message' => sprintf($message, $from, $to, $value)
			);
		};
	}


	/**
	 * Checks if a given value is available in the specified array
	 *
	 * If you want to check for a value NOT to be among a list of given elements,
	 * you can set the optional parameter $invert to true, thus reversing the meaning.
	 *
	 * Examples:
	 *     FormValidator::enum('xx', array('a', 'b', 'c'))       # ensures value to be either a, b or c
	 *
	 *     FormValidator::enum('xx', array('a', 'b', 'c'), true) # ensures value to NONE OF a, b or c
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $message
	 * @param       array  $values
	 * @param       boolean $invert Optional; if set to true, inverts the meaning ("valid if not-in-given-list")
	 * @return      Closure
	 */
	public static function enum($message, array $values, $invert = false){

		return function($value) use($message, $values, $invert) {
			$error = $invert
				?  in_array($value, $values)
				: !in_array($value, $values)
			;

			return array(
				'error'   => $error,
				'message' => sprintf($message, $value, implode(', ', $values))
			);
		};
	}

	/**
	 * Checks the given value against a regular expression.
	 *
	 * If the optional parameter $invert is set to true, the check is
	 * inverted, ie. the regex matches the "forbidden" values.
	 *
	 * Examples:
	 * @code
	 *     FormValidator::regexp('xx', '/\s/');  # accepts everything with a space
	 *
	 *     FormValidator::regexp('xx', '/\s/', true);  # accepts everything WITHOUT a space
	 * @endcode
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $message
	 * @param       string $regexp
	 * @param       bool   $invert     (optional)
	 * @return      Closure
	 */
	public static function regexp($message, $regexp, $invert = false){
		return function($value) use ($message, $regexp, $invert ){
			$matches = array();
			$error = $invert
				? !!preg_match($regexp, $value, $matches)
				:  !preg_match($regexp, $value, $matches)
			;

			$message = ($error)
				? sprintf($message, $regexp, $value, implode(', ', $matches))
				: ''
			;

			return array(
				'error'   => $error,
				'message' => $message
			);
		};
	}

	/**
	 * Wraps multiple closures in a single closure.
	 *
	 * The returned closure groups functions of this class, and returns
	 * the result of the function which returns the first error.
	 *
	 * @code
	 * FormValidator::functionGroup(
	 *     FormValidator::regexp(...),
	 *     FormValidator::unique(...)
 	 * );
	 * @endcode
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      Closure
	 */
	public static function functionGroup(/*function[, functions...]*/){
		$args = func_get_args();
		return function($value) use ($args){
			$error = false;
			$message = '';
			foreach($args as $arg){
				$function = $arg($value);

				if($function['error'] == true){
					return $function;
				}
			}
			return array('error' => false, 'message' => '');
		};
	}

	/**
	 * Method to group checks of this class. Returns the result of the function
	 * which returns the first error if all functions returned an error.
	 *
	 * @code
	 * FormValidator::weakFunctionGroup(
	 *     FormValidator::regexp(...),
	 *     FormValidator::unique(...)
 	 * );
	 * @endcode
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      Closure
	 */
	public static function weakFunctionGroup(/*function[, functions...]*/){

		$args = func_get_args();

		return function ($value) use($args){
			$message = null;

			$goodCount = 0;
			$badCount  = 0;

			foreach($args as $arg){
				$returnValue = $arg($value);

				if ($returnValue['error']) $badCount  += 1;
				else                       $goodCount += 1;


				// store message if this is the first error
				if($returnValue['error'] && is_null($message)){
					$message = $returnValue['message'];
				}
			}
			$error = ($badCount > 0) && ($goodCount == 0);
			return array('error' => $error, 'message' => $message);
		};
	}
}
