<?php

namespace org\adfinis\framework\form\fields;

/**
 * FormCheckboxGroup class which handles a set of FormCheckboxFields
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class FormCheckboxGroup extends FormField {
	private $formInputFields = array();
	private $name;
	private $values;

	/**
	 * Constructor of FormCheckboxGroup
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string  $name
	 * @param       string  $title
	 * @param       array   $values           (optional)
	 * @param       array   $selectedValues   (optional)
	 * @param       Closure $validateCallback (optional)
	 * @return      void
	 */
	public function __construct($name, $title, array $values = array(), array $selectedValues = array(), Closure $validateCallback = null){
		foreach($values as $key => $value){
			$checkboxFieldName = sprintf('%s[%s]',$name, $key);
			$this->formInputFields[$key] = new FormCheckboxField(
				$checkboxFieldName, $value, 1, $selectedValues[$key]);
		}

		$this->name = $name;
		$this->attributes['title'] = $title;
		$this->setValidateCallback($validateCallback);
	}

	/**
	 * Returns the type of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getType(){
		return 'checkboxgroup';
	}

	/**
	 * Returns the current value
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      array
	 */
	public function getValue(){
		$values = array();
		foreach($this->values as $key => $value){
			if($value){
				$values[] = $key;
			}
		}
		return $values;
	}

	/**
	 * Returns the name of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getName(){
		return $this->name;
	}

	/**
	 * Sets the value of the field and call the validateValue method
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @todo by dv: The $validate parameter is not used currently and is 
	 *              only put here for FormField compatibility.
	 * @param       string $value
	 * @param       bool   $validate (optional)
	 * @return      void
	 */
	public function setValue($values, $validate = true){
		$this->values = $values;
		foreach($this->formInputFields as $key => $formField){
			if($values[$key]){
				$formField->setValue(true);
			} else {
				$formField->setValue(false);
			}
		}
	}

	/**
	 * Calls the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $value
	 * @return      bool
	 */
	public function validateValue($value){
		// we need to get the callback method before we can call it
		$callback = $this->validateCallback;
		// get callback message
		$message = $callback($value);
		if(!$message['error']){
			$this->attributes['class'] = 'valid';
			return true;
		} else {
			$this->attributes['class'] = 'invalid';
			$this->setErrorMessage($message['message']);
			return false;
		}
	}

	/**
	 * Returns the label of this form field
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnLabel(){
		return sprintf('<label for="%s">%s</label>',
			$this->name, $this->attributes['title']);
	}

	/**
	 * Returns the HTML checkbox group element
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnField(){
		$fields = sprintf('<span id="%s">', $this->name);
		foreach($this->formInputFields as $formField){
			$fields .= sprintf('%s%s',$formField->returnField(), $formField->returnLabel());
		}
		$fields .= '</span>';
		return $fields;
	}
}
