<?php

namespace org\adfinis\framework\form\fields;

/**
 * FormDateTimeField class which handles a set of FormSelectFields
 * handling a BaseDateTime object.
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class FormDateTimeField extends FormField {
	private $formInputFields = array();
	private $shownFields = array();
	private $name;
	private $time;

	/**
	 * Constructor of FormDateTimeField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string                    $name
	 * @param       string                    $title
	 * @param       string|BaseDateTime|array $time             (optional)
	 * @param       array                     $shownFields      (optional)
	 * @param       Closure                   $validateCallback (optional)
	 * @return      void
	 */
	public function __construct($name, $title, $time = null, array $shownFields = array(), Closure $validateCallback = null){
		$defaultFields = array(
			'day' => 'Day',
			'month' => 'Month',
			'year' => 'Year',
			'hour' => false,
			'minute' => false,
			'second' => false
		);

		$this->shownFields = array_merge($defaultFields, $shownFields);
		$this->name = $name;
		$this->attributes['title'] = $title;
		$this->setValidateCallback($validateCallback);
		$this->setValue($time);
	}

	/**
	 * Returns the type of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getType(){
		return 'datetime';
	}

	/**
	 * Returns the current value
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      BaseDateTime
	 */
	public function getValue(){
		return $this->time;
	}

	/**
	 * Returns the name of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getName(){
		return $this->name;
	}

	/**
	 * Selects the date time with the given value
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int|BaseDateTime|array $value
	 * @param       bool                          $validate (unused)
	 * @return      void
	 */
	public function setValue($value, $validate = true){
		if($value instanceof BaseDateTime){
			$this->time = $value;
		} elseif($value == null){
			$this->time = new BaseDateTime();
		} elseif(is_array($value)){
			$this->time = new BaseDateTime();
			$this->time->setTime(
				(isset($value['hour'])  ? $value['hour']   : 0),
				(isset($value['minute'])? $value['minute'] : 0),
				(isset($value['second'])? $value['second'] : 0)
			);
			$this->time->setDate(
				(isset($value['year'])  ? $value['year']  : 0),
				(isset($value['month']) ? $value['month'] : 0),
				(isset($value['day'])   ? $value['day']   : 0)
			);
		} else {
			$this->time = new BaseDateTime($value);
		}

		/** @todo by ds: move this out of this method... */
		if($this->shownFields['day']){
			$this->formInputFields['day'] = new FormSelectField(
				sprintf('%s[day]', $this->name),
				'',
				array($this->shownFields['day'] => 0)+array_combine(range(1, 31), range(1, 31)),
				$this->time->format('d')
			);
		}
		if($this->shownFields['month']){
			$this->formInputFields['month'] = new FormSelectField(
				sprintf('%s[month]', $this->name),
				'',
				array($this->shownFields['month'] => 0)+array_combine(range(1, 12), range(1, 12)),
				$this->time->format('m')
			);
		}
		if($this->shownFields['year']){
			$this->formInputFields['year'] = new FormSelectField(
				sprintf('%s[year]', $this->name),
				'',
				array($this->shownFields['year'] => 0)+array_combine(range(1900, 2042), range(1900, 2042)),
				$this->time->format('Y')
			);
		}
		if($this->shownFields['hour']){
			$this->formInputFields['hour'] = new FormSelectField(
				sprintf('%s[hour]', $this->name),
				'',
				array($this->shownFields['hour'] => 0)+array_combine(range(1, 24), range(1, 24)),
				$this->time->format('h')
			);
		}
		if($this->shownFields['minute']){
			$this->formInputFields['minute'] = new FormSelectField(
				sprintf('%s[minute]', $this->name),
				'',
				array($this->shownFields['minute'] => 0)+range(0, 59),
				$this->time->format('i')
			);
		}
		if($this->shownFields['second']){
			$this->formInputFields['second'] = new FormSelectField(
				sprintf('%s[second]', $this->name),
				'',
				array($this->shownFields['second'] => 0)+range(0, 59),
				$this->time->format('s')
			);
		}
	}

	/**
	 * Calls the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $value
	 * @return      bool
	 */
	public function validateValue($value){
		// we need to get the callback method before we can call it
		$callback = $this->validateCallback;
		// get callback message
		$message = $callback($value);
		if(!$message['error']){
			$this->attributes['class'] = 'valid';
			return true;
		} else {
			$this->attributes['class'] = 'invalid';
			$this->setErrorMessage($message['message']);
			return false;
		}
	}

	/**
	 * Returns the label of this form field
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnLabel(){
		return sprintf('<label for="%s">%s</label>',
			$this->name, $this->attributes['title']);
	}

	/**
	 * Returns the HTML select field element
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnField(){
		$fields = sprintf('<span id="%s">', $this->name);
		foreach($this->formInputFields as $formField){
			$fields .= $formField->returnField();
		}
		$fields .= '</span>';
		return $fields;
	}
}
