<?php

namespace org\adfinis\framework\form\fields;

/**
 * Abstract FormField class
 *
 * @author      Toni Rudolf <tr@adfinis.ch>
 * @author      Adrian Wittwer <aw@adfinis.ch>
 * @author      Damian Senn <ds@adfinis.ch>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
abstract class FormField {
	/**
	 * @var array $attributes
	 */
	protected $attributes;

	/**
	 * @var Closure $validateCallback
	 */
	protected $validateCallback;

	/**
	 * @var string $errorMessage
	 */
	private $errorMessage;

	/**
	 * @var bool $autoFill
	 */
	private $autoFill;

	/**
	 * Generates a string of attributes for a HTML element
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       string  $name
	 * @param       string  $title
	 * @param       bool    $autoFill         (optional)
	 * @param       array   $attributes       (optional)
	 * @param       Closure $validateCallback (optional)
	 * @return      void
	 */
	public function __construct($name, $title, $autoFill = true, array $attributes = array(), Closure $validateCallback = null){
		$this->attributes = $attributes;
		$this->setTitle($title);
		$this->attributes['name'] = $name;

		// set callback method
		$this->setValidateCallback($validateCallback);

		$this->setAutoFill($autoFill);
	}

	/**
	 * Sets if the FormField should be filled by GPC or DB
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       bool $autoFill
	 * @return      void
	 */
	public function setAutoFill($autoFill){
		$this->autoFill = (bool)$autoFill;
	}

	/**
	 * Gets the autoFill state of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      bool
	 */
	public function getAutoFill(){
		return $this->autoFill;
	}

	/**
	 * Sets the function which is used to validate the FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       Closure $validateCallback (optional)
	 * @return      void
	 */
	public function setValidateCallback(Closure $validateCallback = null){
		if($validateCallback !== null && is_callable($validateCallback)){
			$this->validateCallback = $validateCallback;
		} else {
			$this->validateCallback = function(){
				return true;
			};
		}
	}

	/**
	 * Generates a string of attributes for a HTML element
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @param       array $attributes
	 * @return      string
	 */
	protected static function getAttributesString(array $attributes){
		$attributesStringParts = array();

		foreach($attributes as $attribute => $attributeValue){
			$attributesStringParts[] = sprintf('%s="%s"', $attribute, htmlentities($attributeValue, ENT_COMPAT, 'UTF-8'));
		}

		return implode(' ', $attributesStringParts);
	}

	/**
	 * Returns the label of a field
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @return      string
	 */
	public function returnLabel(){
		$title = $this->getTitle();

		if($title){
			return sprintf('<label for="%s">%s</label>',
				$this->attributes['id'], $title);
		}

		return '';
	}

	/**
	 * Returns the name of the field
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.ch>
	 * @return      string
	 */
	public function getName(){
		return $this->attributes['name'];
	}

	/**
	 * Returns the title
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @return      string
	 */
	public function getTitle(){
		return $this->attributes['title'];
	}

	/**
	 * Sets the title of the field
	 *
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       array $title
	 * @return      void
	 */
	public function setTitle($title){
		$this->attributes['title'] = $title;
	}

	/**
	 * Sets a single HTML attribute
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $key
	 * @param       string $value
	 * @return      void
	 */
	public function setAttribute($key, $value){
		$this->attributes[$key] = $value;
	}

	/**
	 * Sets the value of the field and call the validateValue method
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $value
	 * @param       bool   $validate (optional)
	 * @return      void
	 */
	public function setValue($value, $validate = true){
		$this->attributes['value'] = $value;
		if($validate){
			$this->validateValue($value);
		}
	}

	/**
	 * Sets the errorMessage returned by the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $message
	 * @return      void
	 */
	protected function setErrorMessage($message){
		$this->errorMessage = $message;
	}

	/**
	 * Returns the errorMessage which was given by the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string|false
	 */
	public function getErrorMessage(){
		if(isset($this->errorMessage)){
			return $this->errorMessage;
		}
		return false;
	}

	/**
	 * Returns the errorMessage element
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnError(){
		if($this->getErrorMessage()){
			return sprintf('<span class="errorMessage">%s</span>', $this->getErrorMessage());
		} else {
			return '';
		}
	}

	/**
	 * Abstract method which should return the type of the FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	abstract public function getType();

	/**
	 * Abstract method which should return the HTML string of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	abstract public function returnField();

	/**
	 * Abstract method which should return the value of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      mixed
	 */
	abstract public function getValue();

	/**
	 * Abstract method which should be executed in setValue and return a boolean
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      bool
	 */
	abstract protected function validateValue($value);
}
