<?php

namespace org\adfinis\framework\form\fields;

/**
 * FormPasswordGroup class which handles a set of 2 password fields
 *
 * @author      Damian Senn <ds@adfinis.ch>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class FormPasswordGroup extends FormField {
	private $formInputFields = array();
	private $name;
	private $value;

	/**
	 * Constructor of FormPasswordGroup
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @param       string $titles
	 * @param       array  $attributes (optional)
	 * @param       Closure $validateCallback (optional)
	 * @return      void
	 */
	public function __construct($name, array $titles, array $attributes = array(), Closure $validateCallback = null){
		$titles = array_merge(
			array(
				'password1'     => '',
				'password2'     => '',
				'passwordGroup' => ''
			),
			$titles
		);

		$this->formInputFields[$name.'1'] = new FormInputField(sprintf('%s[]', $name), $titles['password1'], '', 'password', false, $attributes);
		$this->formInputFields[$name.'1']->setAttribute('id', $name.'1');
		$this->formInputFields[$name.'2'] = new FormInputField(sprintf('%s[]', $name), $titles['password2'], '', 'password', false, $attributes);
		$this->formInputFields[$name.'2']->setAttribute('id', $name.'2');

		$this->name = $name;
		$this->attributes['title'] = $titles['passwordGroup'];
		$this->setValidateCallback($validateCallback);
	}

	/**
	 * Returns the FormPasswordGroup label
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      void
	 */
	public function returnLabel(){
		if($this->attributes['title']){
			return sprintf('<label for="%s">%s</label>',
				$this->name, $this->attributes['title']);
		}

		return '';
	}

	/**
	 * Selects the FormRadioField with the given value
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string|int $value
	 * @param       bool $validate (optional)
	 * @return      void
	 */
	public function setValue($value, $validate = true){
		if(!$validate || $this->validateValue($value)){
			$this->value = $value[0];
		}
	}

	/**
	 * Returns the value of the password field
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string|int
	 */
	public function getValue(){
		return $this->value;
	}

	/**
	 * Calls the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       array $value
	 * @return      bool
	 */
	public function validateValue($value){
		$message = array('error' => false);
		if($value[0] !== $value[1]){
			$this->setErrorMessage(tr('Passwords did not match!'));
			$class = 'invalid';
			$message['error'] = true;
			$message['message'] = tr('Passwords did not match');
		}

		if($message['error'] === false){
			// we need to get the callback method before we can call it
			$callback = $this->validateCallback;
			// get callback message
			$message = $callback($value[0]);
		}

		if(!$message['error']){
			$this->attributes['class'] = 'valid';
		} else {
			$this->attributes['class'] = 'invalid';
			$this->setErrorMessage($message['message']);
		}

		foreach($this->formInputFields as $formField){
			$formField->setAttribute('class', $this->attributes['class']);
		}

		return !$message['error'];
	}

	/**
	 * Returns the type of the FormPasswordGroup
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getType(){
		return 'passwordgroup';
	}

	/**
	 * Returns the name of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getName(){
		return $this->name;
	}

	/**
	 * Returns the FormPasswordGroup HTML element
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnField(){
		$field = sprintf('<span id="%s">', $this->getName());
		foreach($this->formInputFields as $formField){
			$field .= sprintf('<span class="password">%s %s</span>',
				$formField->returnLabel(), $formField->returnField());
		}
		$field .= '</span>';
		return $field;
	}
}
