<?php

namespace org\adfinis\framework\form\fields;

/**
 * FormTextInputGroup class which handles a set of FormInputFields type 'text'
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class FormTextInputGroup extends FormField {
	private $formInputFields = array();
	private $name;
	private $titles;
	private $values;
	private $validateCallbacks;

	/**
	 * Constructor of FormTextInputGroup
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string  $name
	 * @param       array   $titles
	 * @param       array   $values           (optional)
	 * @param       array   $attributes       (optional)
	 * @param       array   $validateCallbacks (optional)
	 * @return      void
	 */
	public function __construct($name, array $titles, array $values = array(), array $attributes = array(), array $validateCallbacks = array()){
		$this->attributes = $attributes;
		$this->attributes['title'] = $titles['inputGroup'];
		unset($titles['inputGroup']);
		$this->titles = $titles;

		foreach($values as $i => $values){
			foreach($titles as $key => $title){
				$fieldName = sprintf('%s[%d][%s]', $name, $i, $key);
				$this->formInputFields[$i][$key] = new FormInputField(
					$fieldName, $title, $values[$key], 'text', true, array(), (isset($validateCallbacks[$key])
						? $validateCallbacks[$key]
						: null
					)
				);
			}
		}

		$this->name = $name;
		$this->validateCallbacks = $validateCallbacks;
	}

	/**
	 * Returns the type of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getType(){
		return 'textinputgroup';
	}

	/**
	 * Returns the current value
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      array
	 */
	public function getValue(){
		return $this->values;
	}

	/**
	 * Returns the name of this FormField
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function getName(){
		return $this->name;
	}

	public function setValue($value, $validate = true){
		$this->values = $value;
		foreach($this->values as $i => $values){
			foreach($values as $key => $value){
				if(!isset($this->formInputFields[$i][$key])){
					$fieldName = sprintf('%s[%d][%s]', $this->name, $i, $key);
					$this->formInputFields[$i][$key] = new FormInputField(
						$fieldName, '', '', 'text', true, array(), (isset($this->validateCallbacks[$key])
							? $this->validateCallbacks[$key]
							: null
						)
					);
				}
				$this->formInputFields[$i][$key]->setValue($value, $validate);
			}
		}
	}

	/**
	 * Calls the validateCallback function
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $value
	 * @return      bool
	 */
	public function validateValue($value){
		// we need to get the callback method before we can call it
		$callback = $this->validateCallback;
		// get callback message
		$message = $callback($value);
		if(!$message['error']){
			$this->attributes['class'] = 'valid';
			return true;
		} else {
			$this->attributes['class'] = 'invalid';
			$this->setErrorMessage($message['message']);
			return false;
		}
	}

	/**
	 * Returns the label of this form field
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnLabel(){
		return sprintf('<label for="%s">%s</label>',
			$this->name, $this->attributes['title']);
	}

	/**
	 * Returns the HTML checkbox group element
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function returnField(){
		$fields = sprintf('<table id="%1$s" summary="%1$s" %2$s>',
			$this->name, self::getAttributesString($this->attributes));

		$firstLoop = true;
		foreach($this->formInputFields as $key => $formFields){
			if($firstLoop){
				$fields .= '<tr>';
				foreach($this->titles as $title){
					$fields .= sprintf('<th>%s</th>', $title);
				}
				$fields .= '</tr>';
				$firstLoop = false;
			}

			$fields .= '<tr>';
			foreach($formFields as $formField){
				$fields .= sprintf('<td>%s</td>', $formField->returnField());
			}
			$fields .= '</tr>';
		}
		$fields .= '</table>';
		return $fields;
	}
}
