<?php

namespace org\adfinis\framework\html;

/**
 * Represents Meta Information for an HTML Tag
 *
 * @author        Jan Krause <jk@gameswelt.de>
 * @copyright     adfinis GmbH 2011
 * @license       LGPL 3.0
 */
class Meta {
	/**
	 * Hold all possibly properties in right order
	 *
	 * @var           array $data
	 */
	protected $data = array(
		'name'      => null,
		'httpEquiv' => null,
		'property'  => null,
		'scheme'    => null,
		'lang'      => null,
		'dir'       => null,
		'content'   => null
	);

	/**
	 * Field mapping to camelCase
	 *
	 * @var           array $fieldMap
	 */
	protected static $fieldMap = array(
		'name'      => 'name',
		'httpEquiv' => 'http-equiv',
		'property'  => 'property',
		'scheme'    => 'scheme',
		'lang'      => 'lang',
		'dir'       => 'dir',
		'content'   => 'content'
	);

	/**
	 * Generates an <meta /> tag
	 *
	 * We will prefer a 'name' based tag, but if 'property' set and 'name'
	 * is null, it generates an 'property' based tag
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @returns       string
	 */
	public function __toString() {
		$options = array();
		
		if(strlen($this->content) == 0) {
			return '';
		}

		foreach($this->data as $key => $value) {
			if(strlen($value) == 0) {
				continue;
			}

			$options[] = sprintf('%s="%s"', static::$fieldMap[$key], $value);
		}

		return sprintf('<meta %s />', join(' ', $options));
	}

	/**
	 * Magic set method for all possibilities
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @throws        \InvalidArgumentException
	 * @param         string $name
	 * @param         string $value
	 * @returns       void
	 */
	public function __set($name, $value) {
		if(!$this->__isset($name)) {
			throw new \InvalidArgumentException(sprintf(
				'property %s is not supported', $name));
		}
		
		$this->data[$name] = htmlspecialchars($value, ENT_COMPAT, 'UTF-8');
	}

	/**
	 * Magic method, that ever returns an string, even if property does not exists
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @todo by jk:   throw an exception, if we make an wrong call?
	 * @param         string $name
	 * @returns       string
	 */
	public function __get($name) {
		return ($this->__isset($name)) ? $this->data[$name] : null;
	}

	/**
	 * Magic method, for checking for an valid property
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @param         string  $name
	 * @returns       boolean
	 */
	public function __isset($name) {
		return array_key_exists($name, $this->data);
	}

	/**
	 * Magic method, for unset an property
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @todo by jk:   throw an exception, if we make an wrong call?
	 * @param         string  $name
	 * @returns       void
	 */
	public function __unset($name) {
		if($this->__isset($name)) {
			$this->data[$name] = null;
		}
	}
}
