<?php

namespace org\adfinis\framework\init;

/**
 * Default loader class
 *
 * Contains autoload code and helpers for finding classes.
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 **/
class ClassLoader
{

	/**
	 * Initializes autoloader
	 *
	 * This basically registers DefaultLoader::autoload() with the spl.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      void
	 */
	public static function initAutoload() {
		spl_autoload_register(array('self', 'autoload'));
		include_once(sprintf('%s/vendor/autoload.php', _CONF_LIB));
	}

	/**
	 * Global class autoloader
	 *
	 * Use absolute path for including to avoid unnecessary stat calls
	 * NOTE: You should not need to call this explicitly.
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      David Vogt <dv@adfinis.ch>
	 * @param       string $classname
	 * @return      bool true if class successfully included, false if not
	 */
	public static function autoload($classname){
		if (self::load3rdParty  ($classname)) return true;
		if (self::loadNamespaced($classname)) return true;
		if (self::loadNormal    ($classname)) return true;
		return false;
	}

	/**
	 * Checks for third-party library.. if found & loaded, returns true; false otherwise.
	 *
	 * @author        David Vogt <dv@adfinis.com>
	 * @todo by dv:   Load classes in a more generic way.
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function load3rdParty($classname) {
		if (self::loadZendClasses       ($classname)) return true;
		if (self::loadPearClasses       ($classname)) return true;
		if (self::loadFPDFClasses       ($classname)) return true;
		if (self::loadPQLiteClasses     ($classname)) return true;
		if (self::loadPHPExcelClass     ($classname)) return true;
		if (self::loadSyRPCServerClasses($classname)) return true;
		if (self::loadSyRPCClasses      ($classname)) return true;
		if (self::loadMonologClasses    ($classname)) return true;
		if (self::loadPSRClasses        ($classname)) return true;
		if (self::loadAMQPClasses       ($classname)) return true;

		return false;
	}

	/**
	 * Load Pear classes
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadPearClasses($classname) {
		$file = sprintf(
			'%s/pear/inc/%s.php',
			_CONF_LIB,
			str_replace('_', '/', $classname)
		);

		if (file_exists($file)) {
			return include_once($file);
		}

		return false;
	}

	/**
	 * Load Zend classes
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @author        David Vogt <dv@adfinis.com>
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadZendClasses($classname) {
		if(strstr($classname, 'Zend')){
			$file = sprintf(
				'%s/ZendFramework/library/%s.php',
				_CONF_LIB,
				str_replace('_', '/', $classname)
			);
			return include_once($file);
		}

		return false;
	}

	/**
	 * Load FPDF classes
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @author        David Vogt <dv@adfinis.com>
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadFPDFClasses($classname) {
		if(strstr($classname, 'FPDF')){
			$file = sprintf(
				'%s/fpdf/fpdf.php',
				_CONF_LIB
			);
			return include_once($file);
		}

		return false;
	}

	/**
	 * Load PQLite classes
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @author        David Vogt <dv@adfinis.com>
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadPQLiteClasses($classname) {
		if(strstr($classname, 'PQLite')){
			$file = sprintf(
				'%s/pqlite/PQLite.php',
				_CONF_LIB
			);
			return include_once($file);
		}

		return false;
	}

	/**
	 * Load PHPExcel classes
	 *
	 * @author        Stefan Heinemann <sh@adfinis.com>
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadPHPExcelClass($classname) {
		if (strstr($classname, 'PHPExcel')) {
			$file = sprintf(
				'%s/PHPExcel/%s.php',
				_CONF_LIB,
				str_replace('_', '/', $classname)
			);
			return include_once($file);
		}

		return false;
	}

	/**
	 * Load SyRPC classes
	 *
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadSyRPCClasses($classname) {
		if(strstr($classname, 'SyRPC')){
			list($namespace, $classname) = explode("\\", $classname);
			$file = sprintf(
				'%s/syrpc-php/adfinis-sygroup/syrpc-php/src/%s.class.php',
				_CONF_LIB,
				$classname
			);
			return include_once($file);
		}
		return false;
	}

	/**
	 * Load Monolog classes
	 *
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadMonologClasses($classname) {
		if (strstr($classname, 'Monolog')){
			$file = sprintf(
				'%s/syrpc-php/monolog/monolog/src/%s.php',
				_CONF_LIB,
				str_replace('\\', '/', $classname)
			);
			return include_once($file);
		}
		return false;
	}

	/**
	 * Load PSR classes
	 *
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadPSRClasses($classname) {
		if(strstr($classname, 'Psr')){
			$file = sprintf(
				'%s/syrpc-php/psr/log/%s.php',
				_CONF_LIB,
				str_replace('\\', '/', $classname)
			);
			return include_once($file);
		}
		return false;
	}

	/**
	 * Load AMQP classes
	 *
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadAMQPClasses($classname) {
		if(strstr($classname, 'PhpAmqpLib')){
			$file = sprintf(
				'%s/syrpc-php/videlalvaro/php-amqplib/%s.php',
				_CONF_LIB,
				str_replace('\\', '/', $classname)
			);
			return include_once($file);
		}
		return false;
	}

	/**
	 * Load SyRPCServer classes
	 *
	 * @param         string $classname
	 * @return        boolean
	 */
	private static function loadSyRPCServerClasses($classname) {
		if(strstr($classname, 'SyRPCServer')){
			list($namespace, $classname) = explode("\\", $classname);
			$file = sprintf(
				'%s/SyRPCServer/%s.class.php',
				_CONF_LIB,
				$classname
			);
			return include_once($file);
		}

		return false;
	}

	/**
	 * Tries to find normal, non-namespaced classes. If found & loaded, returns true; false otherwise.
	 *
	 * load default libraries using package/classname or
	 * package/subpackage/classname pattern
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $classname
	 * @return      boolean
	 */
	private static function loadNormal($classname) {
		$package = array();
		preg_match('/^([A-Z]+[a-z]+)([A-Z][a-z0-9]+)/', $classname, $package);


		if (!empty($package)) {
			$file = sprintf(
				'%s/%s/%s.class.php',
				_CONF_APP_INC,
				strtolower($package[1]),
				$classname
			);

			if($file != ''){
				// let's see if file exists
				if(file_exists($file)){
					return include_once($file);
				} else {
					/**
					 * package/classname pattern didn't match a file, use
					 * package/subpackage/classname pattern instead
					 */
					$file = sprintf(
						'%s/%s/%s/%s.class.php',
						_CONF_APP_INC,
						strtolower($package[1]),
						strtolower($package[2]),
						$classname
					);
					if(file_exists($file)){
						return include_once($file);
					}
				}
			}
		}

		return false;
	}


	/**
	 * Tries to find & load namespaced classes. If found & loaded, returns true; false otherwise
	 *
	 * The namespaced classes are the ones with backslashes in them. They're
	 * looked for in inc/lib/(path_of_namespace). Note: these classes don't have the full
	 * package name in the filename (or class name) anymore, this information
	 * is provided by the namespace now.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @todo by dv: This needs to have some more clever logic, involving include_path
	 * @param       string $classname
	 * @return      boolean
	 */
	private static function loadNamespaced($classname) {

		if (strstr($classname, '\\') !== false) {
			$isApp = true;
			if (substr($classname, 0, 21) == 'org\\adfinis\\framework') {
				$isApp = false;
			}

			//special check if it is a test
			if(substr($classname, 0, 6) == 'tests\\') {
				$basePath  = _CONF_APP;
			}
			else {
				$basePath  = ($isApp) ? _CONF_APP_INC : _CONF_LIB_FW;
			}

			$classFile = str_replace('\\', '/', $classname);

			#echo "<br/>---classname='$classname'";
			#echo "<br/>---isapp='$isApp'";
			#echo "<br/>---classfile='$classFile'";
			#echo "<br/>---basepath='$basePath'";

			$file = sprintf('%s/%s.class.php',
				$basePath,
				$classFile
			);

			if (file_exists($file)) {
				return include_once($file);
			}
		}
		return false;
	}

}

