<?php

namespace org\adfinis\framework\init;

/**
 * Adfinis Framework bootloader
 *
 * This class contains helper methods for initializing various components
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 **/
class FWInit {

	/**
	 * Loads the configuration file and initializes the Config Manager
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      InvalidConfigFileException
	 * @param       string $configFile
	 * @return      void
	 */
	public static function initConfig($configFile) {
		global $_conf;

		// Load config file
		if(!is_readable($configFile)) {
			throw new InvalidConfigFileException(
				sprintf(
					tr('Config file "%s" could not be read'),
					$configFile
				)
			);
		}
		require($configFile);

		// Load Conf Manager. Since we don't have the autoloader yet,
		// We'll have to do an explicit require().
		$confManagerInclude = sprintf(
			'%s/org/adfinis/framework/config/ConfManager.class.php',
			_CONF_LIB_FW
		);
		require_once($confManagerInclude);
	}

	/**
	 * Initializes include paths as given in the config file.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      void
	 */
	public static function initIncludePaths() {
		/**
		 * Register additional include paths
		 */
		ini_set(
			'include_path',
			sprintf(
				'%s:%s',
				ini_get('includePath'),
				implode(':', \org\adfinis\framework\config\ConfManager::getVar('af', 'include', 'path'))
			)
		);
	}

	/**
	 * Initializes the session helpers
	 *
	 * This loads the adfinis session class, Zend Session and Page 
	 * history tools, then initializes the session manager.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      void
	 */
	public static function initSession() {
		/**
		 * Directly load some important core classes
		 */
		require_once(_CONF_LIB_FW  . '/org/adfinis/framework/session/SessionManager.class.php');
		require_once(_CONF_LIB_FW  . '/org/adfinis/framework/presentation/PageHistory.class.php');

		/**
		 * Start session and initialize zend session namespace
		 */
		\org\adfinis\framework\session\SessionManager::init();
	}

	/**
	 * Initializes the time zone from config data.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      void
	 */
	public static function initTimezone() {
		$timezone = \org\adfinis\framework\config\ConfManager::getVar('af', 'timezone');
		date_default_timezone_set($timezone);
	}

	/**
	 * Initializes the class autoloader
	 *
	 * Loads the adfinis autoloader and registers it with SPL.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      void
	 */
	public static function initAutoload() {
		// We have a few constants from the config to help with finding stuff..
		$loaderInclude = sprintf(
			'%s/org/adfinis/framework/init/ClassLoader.class.php',
			_CONF_LIB_FW
		);
		require_once($loaderInclude);

		\org\adfinis\framework\init\ClassLoader::initAutoload();
	}

	/**
	 * Initializes the language / translation stuff.
	 *
	 * Note: This sets the default language to en_US. Depending on the configuration, you need to set it to the user's language.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	private static function initLanguage() {
		\org\adfinis\framework\lang\Language::init('en_US');
	}

	/**
	 * Initializes all adfinis framework components.
	 *
	 * This loads the config file (as given), sets include paths, registers the autoloader, initializes the session and sets the timezone.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      InvalidConfigFileException
	 * @param       string $configFile
	 * @return      void
	 */
	public static function initAll($configFile) {
		self::initConfig($configFile);
		self::initIncludePaths();
		self::initAutoload();
		self::initSession();
		self::initTimezone();
		self::initLanguage();
	}
}

