<?php


namespace org\adfinis\framework\lang;

require 'TranslationFunctions.class.php';

use org\adfinis\framework\config\ConfManager;

/**
 * Translation / Language helper for the adfinis framework
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class Language {

	/**
	 * Stores the current "global" language object.
	 *
	 * @var           Language $currentLanguage
	 */
	public static $currentLanguage;

	/**
	 * Translation adapter
	 *
	 * @var           Zend_Translate $translator
	 */
	private $translator;

	/**
	 * Constructs a new language helper.
	 *
	 * Note that the given locale must be recognized by the zend framework.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $locale
	 */
	public function __construct($locale) {

		$files = array();
		foreach (ConfManager::getVar('af', 'translate', 'path') as $path) {
			$matcher = sprintf('%s/%s/*.mo', $path, $locale);
			$pathFiles = glob($matcher);
			$files = array_merge($files, $pathFiles);
		}

		// Well this is stoopid, but Zend_Translate requires a content part, so we cannot
		// init this as cleanly as it could be. fu?
		$this->translator = new \Zend_Translate(array(
			'adapter' => 'gettext',
			'locale'  => $locale,
			'content' => array_pop($files)
		));

		foreach($files as $file) {
			$this->translator->addTranslation(array(
				'content' => $file,
				'locale'  => $locale
			));
		}
	}

	/**
	 * Returns a list of all locales supported.
	 *
	 * Scans the locale directories for translation files, then returns a list of supported locales.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	public static function getSupportedLocales() {
		$supported = array();
		foreach (ConfManager::getVar('af', 'translate', 'path') as $path) {
			$locales = glob(sprintf('%s/*', $path));

			foreach($locales as $locale) {
				$locale = basename($locale);
				$supported[$locale] = true;
			}
		}
		return array_keys($supported);
	}


	/**
	 * Returns the Zend_Translate adapter
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       Zend_Translate
	 */
	public function getTranslator() {
		return $this->translator;
	}

	/**
	 * Initializes the language subsystem while trying to detect the user's
	 * language from the HTTP headers sent by the browser.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public static function initFromBrowser() {
		$locale = \Locale::acceptFromHttp($_SERVER['HTTP_ACCEPT_LANGUAGE']);
		self::init($locale);
	}

	/**
	 * Initializes the language for the given locale.
	 *
	 * Note that this also sets the global language object, and thus affects every "normal" translation.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws
	 * @param         string $locale
	 * @returns       void
	 */
	public static function init($locale) {
		self::$currentLanguage = new self($locale);
	}

	/**
	 * Translates a single string
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $text
	 * @returns       string
	 */
	static function gettext($text) {
		if(is_null(Language::$currentLanguage)) {
			throw new Exception(tr('Unable to translate, no "current" language set yet'));
		}
		$translator = Language::$currentLanguage->getTranslator();

		return $translator->_($text);
	}

	/**
	 * Shorthand to translate using the current language - plural version
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $text1  Singular form of the phrase
	 * @param         string $textn  Plural form of the phrase
	 * @param         int    $number Number to decide between singular/plural
	 * @returns       string
	 */
	static function ngettext($text1, $textn, $number) {
		if(is_null(Language::$currentLanguage)) {
			throw new Exception(tr('Unable to translate, no "current" language set yet'));
		}
		$translator = Language::$currentLanguage->getTranslator();

		if ($number != 1) {
			return $translator->_($textn);
		}
		else {
			return $translator->_($text1);
		}
		//return $translator->plural($text1, $textn, $number);
	}
}
