<?php

namespace org\adfinis\framework\model;

use org\adfinis\framework\exceptions;

/**
 * Entity base class.
 *
 * This provides an abstract base class for all your business logic models.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
abstract class Entity {

	/**
	 * Returns the identifier of the current object.
	 *
	 * The identifier is an associative array that contains all the information for identifying the current object.
	 *
	 * In the normal cause, this is probably just a simple one-entry array like this: 
	 * @code
	 *     array('id' => 5)
	 * @endcode
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	abstract public function getIdentifier();

	/**
	 * Returns true if the object's identifier fields are set to a non-null value.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       bool
	 */
	public function hasIdentifier() {
		foreach($this->getIdentifier() as $field => $value) {
			if(is_null($value)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Returns a data mapper for handling persistence of this object.
	 *
	 * The data mapper can be used to persistently load/save objects, search, and so on
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       org\adfinis\framework\datamapper\Mapper
	 */
	public static function getMapper() {
		throw new \RuntimeException(
			tr("The getMapper() function must be overridden")
		);
	}

	/**
	 * This method fills the object from an associative array, usually a row from the database.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         array $data
	 * @returns       void
	 */
	abstract public function fillFromRowData($data);

	/**
	 * Saves the object, using the associated data mapper.
	 *
	 * After the object is saved, it is returned.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        RuntimeException if the validator is invalid.
	 * @throws        InvalidInputException if the model is not validated properly.
	 * @returns       Entity
	 */
	public function save() {
		$validator = $this->getValidator();
		$valid     = false;
		if(is_null($validator)) {
			// No validator means we don't care.
			$valid = true;
		}
		else if($validator instanceof \org\adfinis\framework\validate\ModelValidator) {
			if($validator->isValid()) {
				$valid = true;
			}
		}
		else {
			throw new \RuntimeException(tr(
				'Cannot save - invalid validator!'
			));
		}

		if(!$valid) {
			$messages = $validator->getMessages();
			foreach ($messages as $field => &$message) {
				$message = sprintf(
					"%s: %s",
					tr($field),
					$message
				);
			}
			throw new exceptions\InvalidInputException(join("\n", $messages));
		}
		return $this->getMapper()->save($this);
	}

	/**
	 * Deletes the object, using the associated data mapper.
	 *
	 * @author        Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @returns       void
	 */
	public function delete() {
		$this->getMapper()->delete($this->getIdentifier());
	}

	/**
	 * Returns true if the given database record belongs to this object.
	 *
	 * This is used in the DBIterator to decide if there's more data to be
	 * added to this object, or if a a new entity needs to be built for the
	 * given record.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         array $data
	 * @returns       bool
	 */
	abstract public function doesRecordMatch($data);

	/**
	 * Returns a validator for the current object.
	 *
	 * The given validator MUST be a subclass of org\adfinis\framework\validate\ModelValidator.
	 *
	 * If you want validation, you should overload this method in your model class and return
	 * a validator object.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       \org\adfinis\framework\validate\ModelValidator
	 */
	public function getValidator() {
		return null;
	}
}
