<?php

namespace org\adfinis\framework\presentation;

use org\adfinis\framework\utils\URLHelper;
use org\adfinis\framework\session\SessionManager;
use org\adfinis\framework\template\Skittle;

/**
 * Class dealing with generic page handling (smarty, meta tags, redirecting, ...)
 *
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
abstract class Page {
	const SCRIPT_POSITION_TOP    = 'top';
	const SCRIPT_POSITION_BOTTOM = 'bottom';

	/**
	 * Holds the data for the meta tags.
	 *
	 * @var           array<org\adfinis\framework\html\Meta> $metaFields
	 */
	protected $metaFields = array();

	/**
	 * holds the data for the script tags
	 * @var           array $scriptFields
	 */
	protected $scriptFields = array();

	/**
	 * Holds the data for the css link tags
	 * @var           array $cssLinkFields
	 */
	protected $cssLinkFields = array();

	/**
	 * Holds the title of the page
	 * @var           string $headerTitle
	 */
	protected $headerTitle = '';

	/**
	 * Smarty/Skittle instance
	 * @var           org\adfinis\framework\template\Skittle $tpl
	 */
	public $tpl = null;

	/**
	 * Session Manager instance
	 * @var           org\adfinis\framework\session\SessionManager $session
	 */
	public $session = null;

	/**
	 * Object constuctor
	 *
	 * $allowSecure defines if this page can be requested over HTTPS
	 *
	 * $needSecure defines if the page needs to be requested over HTTPS
	 *
	 * $domainIdentifier defines, which URL should be used in this instance.
	 * If you skip this parameter, we'll use the domain which is defined as
	 * 'default' in globalConfig.php. Define more URLs there if needed.
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @throws        UnexpectedValueException
	 * @param         bool   $allowSecure      (optional)
	 * @param         bool   $needSecure       (optional)
	 * @param         string $domainIdentifier (optional)
	 */
	public function __construct(
		$allowSecure = false,
		$needSecure = false,
		$domainIdentifier = 'default'
	){
		/*
		 * check if page allows/needs secure connection and do a
		 * location-redirect accordingly
		 */
		$this->checkSecure($allowSecure, $needSecure);

		$this->session = SessionManager::getInstance();

		/*
		 * check for users country information: if no information then gets
		 * the information and writes it in session
		 */
		if(!isset($this->session->countryInfos)){
			/*
			 * detects country infos from users which are not logged in and
			 * stores them into the session
			 */
			//$countryInfos = GeoIp::getCountryByIp($_SERVER['REMOTE_ADDR']);
			//SessionManager::getInstance()->countryInfos = $countryInfos;
		}

		// create smarty instance if needed
		if(!($this->tpl instanceof Skittle) && !$this->tpl = new Skittle()){
			throw new UnexpectedValueException(
				tr('Could not create a Skittle instance')
			);
		}

		$this->tpl->assign('page', $this);

		/*
		 * ssl pages should not load unsecure content, we need a smarty
		 * var for the templates
		 */
		$this->tpl->assign('isSecure', $this->isSecure());

		$this->tpl->registerPlugin(
			'block',
			'sortable',
			array('org\\adfinis\\framework\\presentation\\TableTools', 'sortable')
		);

		return $this;
	}

	/**
	 * Checks if page/script was requested over HTTPS
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @return        bool
	 */
	public function isSecure(){
		return URLHelper::currentURLIsSecure();
	}

	/**
	 * Checks if secure-mode is allowed AND/OR needed for this page and
	 * redirects accordingly.
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         bool $allow Can the page be requested over HTTPS?
	 * @param         bool $need Does the page needs to be secured over HTTPS?
	 * @return        void
	 */
	public function checkSecure($allow = false, $need = false){
		/**
		 * If we don't allow HTTPS for this page but it's requested with
		 * HTTPS -> reload unsecure.
		 */
		if($allow === false && $this->isSecure() === true){
			$this->reloadCurrentPage(false);
			return;
		}

		/**
		 * If page needs AND allows HTTPS but is request without
		 * HTTPS -> reload secure
		 */
		if ($need === true && $allow === true && $this->isSecure() === false){
			$this->reloadCurrentPage(true);
			return;
		}
	}

	/**
	 * Redirects the user to the given URL
	 *
	 * The second parameter defines, if the page is permanently moved to a new
	 * location. In this case a permanently (301) header will be sent instead
	 * of a 302 (moved temporarly) header.
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         string $URL
	 * @param         bool $permanent
	 * @return        void
	 */
	public function redirectToURL($URL, $permanent = false){
		URLHelper::redirectToURL($URL, $permanent);
	}

	/**
	 * Reloads the current page
	 *
	 * Can be used to reload the current page using secure/HTTPS instead of
	 * unsecure/HTTP.
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         bool $secured
	 * @return        void
	 */
	public function reloadCurrentPage($secured = false){
		URLHelper::reloadCurrentURL($secured);
	}

	/**
	 * Sets the title text
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         string $headerTitle
	 * @return        void
	 */
	public function setHeaderTitle($headerTitle){
		$this->headerTitle = strip_tags($headerTitle);
	}

	/**
	 * Gets the title text
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @returns       string
	 */
	public function getHeaderTitle() {
		return $this->headerTitle;
	}

	/**
	 * Adds meta data descriptions to the Page
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @deprecated    use addMetaTag now
	 * @param         string $name
	 * @param         string $content
	 * @param         string $httpEquiv
	 * @param         string $scheme
	 * @param         string $lang
	 * @param         string $dir
	 * @param         bool $replace
	 * @return        void
	 */
	public function addMetaData(
		$name,
		$content,
		$httpEquiv = null,
		$scheme    = null,
		$lang      = null,
		$dir       = 'LTR',
		$replace   = false
	) {
		$meta = new html\Meta();
		$meta->name      = $name;
		$meta->content   = $content;
		$meta->httpEquiv = $httpEquiv;
		$meta->scheme    = $scheme;
		$meta->lang      = $lang;
		$meta->dir       = $dir;

		$this->addMetaTag($meta, $replace);
	}

	/**
	 * Adds or edits an Meta field
	 *
	 * As key we use an unique string based on name, http-equiv and property
	 *
	 * @author        Jan Krause <jk@gameswelt.de>
	 * @param         html\Meta $meta
	 * @param         boolean   $replace [optional]
	 * @returns       void
	 */
	public function addMetaTag(html\Meta $meta, $replace = true) {
		$key = sprintf(
			'%s%s%s', $meta->name,$meta->httpEquiv, $meta->property);

		// Check if we should append meta content
		if(
			!$replace &&
			isset($this->metaFields[$key]) &&
			strlen($this->metaFields[$key]->content) !== 0
		) {
			$meta->content = sprintf(
				'%s, %s',
				$meta->content,
				$this->metaFields[$key]->content
			);
		}

		$this->metaFields[$key] = $meta;
	}

	/**
	 * Adds script ressources to the Page using the source path
	 *
	 * @deprecated    Please use addSriptFile() instead.
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        InvalidArgumentException
	 * @param         string $type
	 * @param         string $src
	 * @param         string $position
	 * @return        void
	 */
	private function addScriptWithSourcePath($type = 'text/javascript', $src, $position = self::SCRIPT_POSITION_BOTTOM){
		// create md5 hash to identify the script if we want to remove it
		$hash = md5($src);
		if(isset($this->scriptFields[$hash]) === false){
			$this->scriptFields[$hash] = array(
				'type' => $type,
				'src' => $src,
				'position' => $position
			);
		}
	}

	/**
	 * Adds script ressources to the Page using full script code
	 *
	 * @deprecated    Please use addSriptCode() instead.
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        \InvalidArgumentException
	 * @param         string $type
	 * @param         string $code
	 * @param         string $position
	 * @return        void
	 */
	private function addScriptWithSourceCode($type = 'text/javascript', $code, $position = self::SCRIPT_POSITION_BOTTOM){
		// create md5 hash to identify the script if we want to remove it
		$hash = md5($code);
		if(isset($this->scriptFields[$hash]) === false){
			$this->scriptFields[$hash] = array(
				'type' => $type,
				'code' => $code,
				'position' => $position
			);
		}
	}

	/**
	 * Adds script ressources to the Page using the source path
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        InvalidArgumentException
	 * @param         string $src
	 * @param         string $position
	 * @param         string $type
	 * @return        void
	 */
	public function addScriptFile($src, $position = self::SCRIPT_POSITION_BOTTOM, $type = 'text/javascript'){
		// create md5 hash to identify the script if we want to remove it
		$hash = md5($src);
		if(isset($this->scriptFields[$hash]) === false){
			$this->scriptFields[$hash] = array(
				'type' => $type,
				'src' => $src,
				'position' => $position
			);
		}
	}

	/**
	 * Adds script ressources to the Page using full script code
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        \InvalidArgumentException
	 * @param         string $code
	 * @param         string $position
	 * @param         string $type
	 * @return        void
	 */
	public function addScriptCode($code, $position = self::SCRIPT_POSITION_BOTTOM, $type = 'text/javascript'){
		// create md5 hash to identify the script if we want to remove it
		$hash = md5($code);
		if(isset($this->scriptFields[$hash]) === false){
			$this->scriptFields[$hash] = array(
				'type' => $type,
				'code' => $code,
				'position' => $position
			);
		}
	}


	/**
	 * Adds css ressources to the basePage using the source path
	 *
	 * @author        Adrian Wittwer <aw@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $type
	 * @param         string $rel
	 * @param         string $src
	 * @return        void
	 */
	public function addLinkWithSourcePath(
		$src,
		$type = 'text/css',
		$rel = 'stylesheet'
	){
		$hash = md5($src);
		if(isset($this->cssLinkFields[$hash]) === false){
			$this->cssLinkFields[$hash] = array(
				'type' => $type,
				'rel' => $rel,
				'src' => $src
			);
		}
	}

	/**
	 * Returns all registred meta - tags
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @return        array
	 */
	public function getMetaTags(){
		return $this->metaFields;
	}

	/**
	 * Returns all registred scripts - tags
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $position
	 * @return        array
	 */
	public function getScriptTags($position){
		$scriptFields = array();

		foreach($this->scriptFields as $script){
			if($script['position'] == $position){
				// First get the scripts with a source path
				if(isset($script['src'])){
					$scriptFields[] = sprintf(
						'<script type="%s" src="%s"></script>',
						$script['type'],
						$script['src']
					);
				}

				// Now get the scripts without a source path but with source code
				if(isset($script['code'])){
					$scriptFields[] = sprintf(
						'<script type="%s">%s</script>',
						$script['type'],
						$script['code']
					);
				}
			}
		}
		return $scriptFields;
	}

	/**
	 * Returns all registred link tags
	 *
	 * @author        Adrian Wittwer <aw@adfinis.ch>
	 * @return        array
	 */
	public function getLinkTags(){
		$linkFields = array();

		foreach($this->cssLinkFields as $link){
			// First get the links with a source path
			if(isset($link['src'])){
				$linkFields[] = sprintf(
					'<link rel="%s" type="%s" href="%s" />',
					$link['rel'],
					$link['type'],
					$link['src']
				);
			}
		}

		return $linkFields;
	}

	/**
	 * Output the whole page
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         string $tplFile
	 * @return        string
	 */
	abstract public function outputPage($tplFile = 'index/layout.tpl');
}
