<?php

namespace org\adfinis\framework\presentation;

use org\adfinis\framework\session\SessionManager;
use org\adfinis\framework\utils\URLHelper;

/**
 * Provide a stack for visited pages
 *
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 */
class PageHistory {

	/**
	 * Add an element to the stack
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $URL
	 * @param       string $file
	 * @return      void
	 */
	public static function addToHistory($URL, $file = null){
		if(!self::isURLForHistory($URL, $file)){
			return;
		}

		$session = SessionManager::getInstance();

		/**
		 * Check if there already is a stack for visited pages. Otherwise
		 * create one.
		 */
		if(!is_array($session->pageHistory)){
			$session->pageHistory = array();
		}

		// Don't add same urls next to each other
		if(
			isset($session->pageHistory[0]) &&
			$URL == $session->pageHistory[0]
		){
			return;
		}

		// Check if stack is full (in this case we remove the oldest entry
		if(count($session->pageHistory) < 3){
			array_unshift($session->pageHistory, $URL);
		}else{
			array_pop($session->pageHistory);
			array_unshift($session->pageHistory, $URL);
		}
	}

	/**
	 * Wrapper for basePageHistory::addToHistory($URL)
	 *
	 * Adds the current page to the stack
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @return      void
	 */
	public static function addCurrentPageToHistory(
		$checkSecure = true,
		$domainIdentifier = 'default'
	){
		/**
		 * Lighty:
		 * REQUEST_URI = what results *before* rewriting  i.e. '/articles/previews?main=home'
		 * REDIRECT_URI = what results *after* rewriting i.e. '/index.php?main=home'
		 *
		 * Apache:
		 * REQUEST_URI = what results *before* rewriting  i.e. '/articles/previews?main=home'
		 * REDIRECT_URL = what results *before* rewriting i.e. '/articles/previews'
		 */
		$URL = $_SERVER['REQUEST_URI'];
		if($checkSecure){
			if(URLHelper::currentURLIsSecure()){
				$URL = URLHelper::getURLSecure() . $URL;
			} else {
				$URL = URLHelper::getURLInsecure() . $URL;
			}
		}

		self::addToHistory($URL, $_SERVER['PHP_SELF']);
	}

	/**
	 * Get an element from the stack
	 *
	 * 0 = the last visited page
	 * 1 = the second last visited page
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       int $index
	 * @return      bool
	 */
	public static function getFromHistory($index){
		if(!is_array(SessionManager::getInstance()->pageHistory)){
			return false;
		}

		if(isset(SessionManager::getInstance()->pageHistory[$index])){
			return SessionManager::getInstance()->pageHistory[$index];
		}

		return false;
	}

	/**
	 * Some pages should not go onto the stack
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $URL
	 * @param       string $file (comes from PHP_SELF)
	 * @return      bool
	 */
	private static function isURLForHistory($URL, $file){
		/**
		 * Try to recognize 404 requests. In case of such a request
		 * $_SERVER['REQUEST_URI'] points to a non existing file, therefore do
		 * not add it to the history.
		 *
		 * @todo by nc: it would by nice to have a generic solution
		 */
		if($_SERVER['SCRIPT_NAME'] == ConfManager::getVar('af', 'handler', '404')){
			return false;
		}

		// We only accept .php, .html, .htm files
		$fileParts = explode('.', $file);
		if(is_array($fileParts)){
			$lastPart = array_pop($fileParts);
			if(
				$lastPart !== 'php' &&
				$lastPart !== 'html' &&
				$lastPart !== 'htm'
			){
				return false;
			} else {
				// $file is ok, check for blacklist and use it if ok
			}
		} else {
			return false;
		}

		// @todo by nc: can we solve this clever without hardcoding?
		$blacklist = array();
		$blacklist[] = '/foo/bar.html';

		foreach($blacklist as $part){
			if(strpos($URL, $part) !== false || strpos($file, $part) !== false){
				return false;
			}
		}

		return true;
	}
}
