<?php

namespace org\adfinis\framework\presentation;

use org\adfinis\framework\request\RequestInterface;
use org\adfinis\framework\uri\URI;
use org\adfinis\framework\db\DBIterator;

/**
 * Class to handle paging and alphabet
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @author        David Vogt <dv@adfinis.ch>
 * @todo by dv:   Remove dependency on DBIterator. Use generic slicable interface instead.
 * @copyright     adfinis GmbH 2010
 * @license       BSD 3-Clause
 */
class Paging {
	/**
	 * URI of the current page.
	 *
	 * @var org\adfinis\framework\uri\URI $URI
	 */
	public $URI;

	/**
	 * Request from which to get the page parameter.
	 *
	 * @var           org\adfinis\framework\request\RequestInterface $request
	 */
	private $request;

	/**
	 * Name of the page parameter.
	 *
	 * @var           string $pageParameterName
	 */
	public $pageParameterName = 'page';

	/**
	 * Number of surrounding pages to show.
	 * With the default 3, there will be 10 Links. Example:
	 *
	 *     5 6 7 *8* 9 10 11
	 *
	 * @var           int $surroundingPageLinks
	 */
	public $surroundingPageLinks = 3;

	/**
	 * Defines the number of entries to show per page.
	 *
	 * @var           int $entriesPerPage
	 */
	public $entriesPerPage = 20;

	/**
	 * Iterator to show entries from.
	 *
	 * @var         org\adfinis\framework\db\DBIterator $iterator
	 */
	private $iterator;

	/**
	 * Paging constructor
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         org\adfinis\framework\db\DBIterator   $iterator
	 * @param         org\adfinis\framework\request\RequestInterface $request
	 * @param         string                                $pageParameterName (optional)
	 * @param         org\adfinis\framework\uri\URI         $URL  (optional)
	 */
	public function __construct(DBIterator $iterator, RequestInterface $request, $pageParameterName = 'page', URI $URL = null){

		$this->iterator          = $iterator;
		$this->request           = $request;
		$this->pageParameterName = $pageParameterName;

		if($URL === null) {
			$this->URI       = new URI();
			$this->URI->path = $_SERVER['SCRIPT_NAME'];
			$this->URI->get  = $_GET;
		}
		else {
			$this->URI = $URL;
		}
	}

	/**
	 * Returns the page number of the current request.
	 *
	 * The page range is 1..n. If Request doesn't provide the page number, defaults to 1.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        org\adfinis\exceptions\InvalidInputException If the page parameter is not an integer
	 * @returns       integer
	 */
	private function getCurrentPage() {
		$page = 1;
		if($this->request->keyExists($this->pageParameterName, \org\adfinis\framework\request\REQUEST_GET)) {
			$page = $this->request->getInt($this->pageParameterName, \org\adfinis\framework\request\REQUEST_GET);
		}
		if($page < 1) {
			$page = 1;
		}
		if($page > $this->getLastPage()) {
			$page = $this->getLastPage();
		}
		return $page;
	}

	/**
	 * Returns the last page available for paging.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       int
	 */
	public function getLastPage() {
		$count         = $this->iterator->getCount();
		$lastPageFloat = (float)$count / (float)$this->entriesPerPage;
		$lastPage      = (int)  $lastPageFloat;

		if($lastPageFloat != $lastPage) {
			$lastPage++;
		}
		
		return $lastPage;
	}
	

	/**
	 * Returns paging information ready to be used with a template.
	 *
	 * Returns an associative array with the following keys:
	 *
	 * - entries: The iterator given in the constructor, but with limit/offset defined.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	public function getPagingData() {

		$count = $this->iterator->getCount();

		$lastPage = $this->getLastPage();

		$currentPage = $this->getCurrentPage();

		$surroundingLeft  = array();
		$surroundingRight = array();

		for($page = max(1, $currentPage - $this->surroundingPageLinks); $page < $currentPage; $page++) {
			$surroundingLeft[$page] = $this->getPageLink($page);
		}

		for($page = $currentPage + 1; $page <= min($lastPage, $currentPage + $this->surroundingPageLinks); $page++) {
			$surroundingRight[$page] = $this->getPageLink($page);
		}

		$this->iterator->setPage($currentPage, $this->entriesPerPage);

		return array(
			'entries'          => $this->iterator,
			'count'            => $count,
			'entriesPerPage'   => $this->entriesPerPage,
			'lastPage'         => $lastPage,
			'currentPage'      => $currentPage,
			'firstPageURL'     => $this->getPageLink(1),
			'lastPageURL'      => $this->getPageLink($lastPage),
			'previousPageURL'  => $this->getPageLink($currentPage - 1),
			'nextPageURL'      => $this->getPageLink($currentPage + 1),
			'surroundingLeft'  => $surroundingLeft,
			'surroundingRight' => $surroundingRight
		);
	}

	/**
	 * Returns a link to the given page.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         int $pageNumber
	 * @returns       org\adfinis\framework\uri\URI
	 */
	public function getPageLink($pageNumber) {
		$uri = clone($this->URI);
		$uri->get[$this->pageParameterName] = $pageNumber;
		return $uri;
	}
}
