<?php

namespace org\adfinis\framework\presentation;

use org\adfinis\framework\utils\URLHelper;
use org\adfinis\framework\uri\URI;

/**
 * Smarty utilities for table handling, like adding sort links to table headings.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class TableTools {

	/**
	 * Returns an array with two components: the first is the sort field, the second the direction.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	private static function getSortData() {
		$url = self::getURIObject();

		if(!isset($url->get['sort'])) {
			return array(null, null);
		}

		list($sort, $direction) = preg_split('/\./', $url->get['sort']);

		return array($sort, $direction);
	}

	/**
	 * Make the enclosed text (preferably a table heading) a link that can sort the given table.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $text
	 * @returns       string
	 */
	public static function sortable($params, $content, $smarty, &$repeat) {
		if(isset($content)) {
			$url = self::getURIObject();

			list($sort, $direction) = self::getSortData();

			$alreadySorted =                   $sort      === $params['field'];
			$needsReversal = $alreadySorted && $direction === 'ASC';
			$arrow         = '';

			if($alreadySorted) {
				$arrow = $needsReversal
					? '&uarr;'
					: '&darr;'
				;
			}

			$field     = $params['field'];
			$direction = $needsReversal ? 'DESC' : 'ASC';

			$url->get['sort']      = sprintf('%s.%s', $field, $direction);
			$url->show('get', true);
			return sprintf(
				'<a class="sort" href="%s">%s</a> %s',
				$url->toString(),
				$content,
				$arrow
			);
		}
	}

	/**
	 * Returns the current URL as an URI object.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       URI
	 */
	private static function getURIObject() {
		return URI::parse(URLHelper::getURLWithRequestString());
	}

	/**
	 * Returns the current sort column as determined from the current URL.
	 *
	 * If the URL doesn't contain a sort parameter, NULL is returned.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	public static function getSortFromURL() {
		$uri = self::getURIObject();

		if(isset($uri->get['sort'])) {
			return $uri->get['sort'];
		}
		return null;
	}

	/**
	 * Returns the current sort direction as determined from the current URL.
	 *
	 * If the URL doesn't contain a sort parameter, "ASC" is returned.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	public static function getSortDirectionFromURL() {
		$uri = self::getURIObject();

		if(isset($uri->get['direction']) && $uri->get['direction'] == 'DESC') {
			return 'DESC';
		}
		return 'ASC';
	}
}
