<?php

namespace org\adfinis\framework\request;

use org\adfinis\framework\exceptions\InvalidInputException;

/**
 * Abstract class that handles the common cases in all possible request classes.
 *
 * It also implements the RequestInterface for you and handles
 * all the data conversion / validation stuff.
 *
 * This class also provides support for accessing nested form data:
 * to access $_POST['foo']['bar'], use this:
 * @code
 * $fooBar = $request->getString('foo.bar');
 * @endcode
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
*/
abstract class AbstractRequest implements RequestInterface {

	/**
	 * POST data container
	 * @var array $postData
	 */
	protected $postData;

	/**
	 * GET data container
	 * @var array $getData
	 */
	protected $getData;

	/**
	 * FILE data container
	 * @var array $fileData
	 */
	protected $fileData;

	/**
	 * COOKIE data container
	 * @var array $cookieData
	 */
	protected $cookieData;

	/**
	 * Set to true if the request represents an AJAX request.
	 * @var bool $isAJAX
	 */
	protected $isAJAX = false;

	/**
	 * Set to true if the request represents a POST request (as opposed to GET).
	 * @var bool $isPOST
	 */
	protected $isPOST = false;

	/**
	 * Returns the container for the given request mode
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      InvalidArgumentException if request mode is not known
	 * @param       int $requestMode
	 * @return      array-reference
	 */
	protected function getContainer($requestMode) {
		switch($requestMode) {
			case REQUEST_FILE:
				return $this->fileData;
				break;
			case REQUEST_POST:
				return $this->postData;
				break;
			case REQUEST_COOKIE:
				return $this->cookieData;
				break;
			case REQUEST_GET:
				return $this->getData;
				break;
			default:
				throw new InvalidArgumentException(tr('Unknown request mode'));
		}
	}

	/**
	 * Retrieves a value from a (possibly nested) array.
	 *
	 * The key may be given as a dot-separated string to identify a value in a nested array.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $key
	 * @param[in]   array &$data
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @return      mixed
	 */
	private function getNestedKey($key, &$data) {
		$keys = preg_split('#\.#', $key);
		$ptr = $data;
		foreach($keys as $currentKey) {
			if(!isset($ptr[$currentKey])) {
				throw new \OutOfRangeException(
					sprintf(
						tr('Key "%s" was not found; failed at "%s"'),
						$key, $currentKey
					)
				);
			}
			$ptr = $ptr[$currentKey];
		}
		return $ptr;
	}

	/**
	 * Returns the content of the given variable as an integer.
	 *
	 * The validity of the type conversion is checked by looking
	 * at the string representation of the converted integer.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      int
	 */
	public function getInt($name, $source = REQUEST_POST) {
		// Is there a more efficient way to check correct conversion?
		$valueAsFloat = $this->getFloat($name, $source);
		$valueAsInt   = (int)$valueAsFloat;

		if($valueAsFloat == $valueAsInt) {
			return $valueAsInt;
		}
		throw new InvalidInputException();
	}

	/**
	 * Returns the content of the given variable as a boolean value.
	 *
	 * Boolean values are represented as integers. 0 is false, any other integer
	 * is true. Strings are considered to be invalid.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\framework\exceptions\InvalidInputException If the variable could not be converted
	 * @return      boolean
	 */
	public function getBoolean($name, $source = REQUEST_POST) {
		$value = $this->getRaw($name, $source);

		if(!is_numeric($value)) {
			throw new InvalidInputException();
		}
		return (bool)$value;
	}

	/**
	 * Returns the content of the given variable as floating-point number.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\framework\exceptions\InvalidInputException If the variable could not be converted
	 * @return      boolean
	 */
	public function getFloat($name, $source = REQUEST_POST) {
		$value = $this->getRaw($name, $source);

		if(!is_numeric($value)) {
			throw new InvalidInputException();
		}
		return (float)$value;
	}

	/**
	 * Interprets the given variable as JSON data and returns an array.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\framework\exceptions\InvalidInputException If the variable could not be converted
	 * @return      array
	 */
	public function getJSON($name, $source = REQUEST_POST) {
		$value = $this->getRaw($name, $source);

		$data = json_decode($value);
		if (is_null($data)) {
			// JSON error
			throw new InvalidInputException();
		}
		return $data;
	}

	/**
	 * Returns the requested variable unmodified (ie. as a string or array).
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\framework\exceptions\InvalidInputException If the variable could not be converted
	 * @return      mixed
	 */
	public function getRaw($name, $source = REQUEST_POST) {
		$data = $this->getContainer($source);
		return $this->getNestedKey($name, $data);
	}

	/**
	 * Returns the requested variable as a string.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      string
	 */
	public function getString($name, $source = REQUEST_POST) {
		$data = $this->getRaw($name, $source);
		return (string)$data;
	}

	/**
	 * Returns true if the current request is an AJAX request.
	 *
	 * This should return true if the request came from an XMLHTTPRequest.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isAJAX() {
		return $this->isAJAX;
	}

	/**
	 * Returns true if the current request is a POST request.
	 *
	 * This should return true if the request came from a form using method=POST.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isPOST() {
		return $this->isPOST;
	}

	/**
	 * Returns true if the requested key exists in the given data source, false otherwise.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException  If the data source does not exist
	 * @return      bool
	 */
	public function keyExists($name, $source = REQUEST_POST) {
		$data = $this->getContainer($source);
		try {
			$this->getNestedKey($name, $data);
		}
		catch(\Exception $e) {
			return false;
		}
		return true;
	}
}

