<?php

namespace org\adfinis\framework\request;

/**
 * A forgiving variant of the request object.
 *
 * This means that it catches all conversion/notfound exceptions and
 * returns NULL in those cases.
 *
 * For simplified usage, this class is implemented as a decorator pattern:
 * Use it as follows:
 * @code
 * $request = org\adfinis\framework\request\Request::fromCurrentGPC();
 * # ...
 * $request = new org\adfinis\framework\request\Request($request);
 * @endcode
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
*/
class ForgivingRequest implements RequestInterface {
	private $innerRequest;

	/**
	 * Creates a new Request object initialized from the $_POST, $_GET, $_COOKIE and $_FILES superglobals.
	 *
	 * You should consider using Request::fromCurrentGPC()
	 * instead of calling this constructor directly.
	 *
	 * @author        David Vogt <dv@adfinis.com>
	 * @param         RequestInterface $innerRequest
	 */
	public function __construct($innerRequest) {
		$this->innerRequest = $innerRequest;
	}

	/**
	 * Returns the content of the given variable as an integer.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      int
	 */
	public function getInt($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getInt($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Returns the content of the given variable as a boolean value.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      boolean
	 */
	public function getBoolean($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getBoolean($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Returns the content of the given variable as floating-point number.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      boolean
	 */
	public function getFloat($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getFloat($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Interprets the given variable as JSON data and returns an array.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      array
	 */
	public function getJSON($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getJSON($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Returns the requested variable unmodified (ie. as a string or array).
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      mixed
	 */
	public function getRaw($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getRaw($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Returns the requested variable as a string.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @return      string
	 */
	public function getString($name, $source = REQUEST_POST) {
		try {
			return $this->innerRequest->getString($name, $source);
		}
		catch(\Exception $e) {
			return null;
		}
	}

	/**
	 * Returns true if the current request is an AJAX request.
	 *
	 * This should return true if the request came from an XMLHTTPRequest.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isAJAX() {
		return $this->innerRequest->isAJAX();
	}

	/**
	 * Returns true if the current request is a POST request.
	 *
	 * This should return true if the request came from a form using method=POST.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isPOST() {
		return $this->innerRequest->isPOST();
	}

	/**
	 * Returns true if the requested key exists in the given data source, false otherwise.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException  If the data source does not exist
	 * @return      bool
	 */
	public function keyExists($name, $source = REQUEST_POST) {
		return $this->innerRequest->keyExists($name, $source);
	}

	/**
	 * Returns a new ForgivingRequest from the current GPC values.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       ForgivingRequest
	 */
	public static function fromCurrentGPC() {
		$rq = Request::fromCurrentGPC();
		return new self($rq);
	}
	
}

