<?php

namespace org\adfinis\framework\request;

/**
 * Class dealing with GPC vars
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
*/
class InputVars {
	const METHOD_POST   = 0;
	const METHOD_GET    = 1;
	const METHOD_COOKIE = 2;

	const TYPE_INT     = 0;
	const TYPE_STR     = 1;
	const TYPE_FLOAT   = 2;
	const TYPE_JSON    = 3;
	const TYPE_BOOL    = 4;
	const TYPE_NOCLEAN = 5;

	/**
	 * Gets an input variable like post, get or cookie
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       int    $method (optional)
	 * @param       string $name   (optional)
	 * @param       int    $type   (optional)
	 * @return      mixed
	 */
	public static function getVar($method = self::METHOD_GET, $name = null, $type = self::TYPE_STR){
		$vars = self::getMethodVar($method);

		$return = null;
		if($name === null){
			// no name set, we fetch the whole variable
			if(is_array($vars)){
				$return = $vars;
				self::setArrayType($return, $type);
			}
		} else {
			// name set, fetch the specific variable
			if(isset($vars[$name]) && is_array($vars[$name])){
				$return = $vars[$name];
				self::setArrayType($return, $type);
			} else {
				$return = (isset($vars[$name])
					? self::setType($vars[$name], $type)
					: self::setType(null, $type));
			}
		}

		return $return;
	}

	/**
	 * Recursive method required to clean recursive arrays
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       array  &$vars
	 * @param       string  $type
	 */
	public static function setArrayType(&$vars, $type){
		if(is_array($vars)){
			foreach($vars as $key => $var){
				if(is_array($var)){
					self::setArrayType($var, $type);
					$vars[$key] = $var;
				} else {
					$vars[$key] = self::setType($var, $type);
				}
			}
		}
	}

	/**
	 * Alias for self::getVar(BaseInputVars::METHOD_GET).
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @param       int    $type
	 * @return      mixed
	 */
	public static function getGetVar($name, $type = self::TYPE_STR){
		return self::getVar(self::METHOD_GET, $name, $type);
	}

	/**
	 * Alias for self::getVar(BaseInputVars::METHOD_POST).
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @param       int    $type
	 * @return      mixed
	 */
	public static function getPostVar($name, $type = self::TYPE_STR){
		return self::getVar(self::METHOD_POST, $name, $type);
	}

	/**
	 * Alias for self::getVar(BaseInputVars::METHOD_COOKIE).
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @param       int    $type
	 * @return      mixed
	 */
	public static function getCookieVar($name, $type = self::TYPE_STR){
		return self::getVar(self::METHOD_COOKIE, $name, $type);
	}

	/**
	 * Checks if one or more input vars are set
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       int          $method
	 * @param       string|array $name
	 * @return      bool
	 */
	public static function isVarSet($method, $name = null){
		$vars = self::getMethodVar($method);
		if(is_array($name)){
			foreach($name as $varName){
				if(!isset($vars[$varName])){
					return false;
				}
			}
			return true;
		} else if($name !== null){
			return isset($vars[$name]);
		} else {
			// $_POST / $_GET / $_COOKIE is always an array, check if it's not empty
			return !empty($vars);
		}
	}

	/**
	 * Returns the chosen input var
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       int $method
	 * @return      array
	 */
	private static function getMethodVar($method){
		switch($method){
			case self::METHOD_POST:
				$vars = $_POST;
				break;
			default:
			case self::METHOD_GET:
				$vars = $_GET;
				break;
			case self::METHOD_COOKIE:
				$vars = $_COOKIE;
				break;
		}
		return $vars;
	}

	/**
	 * Sets the type of a variable
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       mixed $var
	 * @param       int   $type
	 * @return      mixed
	 */
	public static function setType($var, $type){
		switch($type){
			default:
			case self::TYPE_NOCLEAN:
				return $var;
				break;
			case self::TYPE_INT:
				return intval($var);
				break;
			case self::TYPE_FLOAT:
				return floatval($var);
				break;
			case self::TYPE_STR:
				return strval($var);
				break;
			case self::TYPE_BOOL:
				/**
				 * @see http://www.php.net/manual/en/language.types.boolean.php#language.types.boolean.casting
				 */
				return (bool)$var;
				break;
			case self::TYPE_JSON:
				return json_decode($var, true);
				break;
		}
	}

	/**
	 * Get the GET vars of a string
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @todo by nc: What if a anchor is defined in $URL? ie foo.com?bar=1#lala
	 * @param       string $URL
	 * @return      array
	 */
	public static function getGetVarsFromString($URL) {
		// first split the parameter from the rest of the url
		$getParameters = explode('?', $URL);
		// now get the key/value pairs
		$getParameters = explode('&', $getParameters[1]);

		// build array
		$getData = array();
		foreach ($getParameters as $getParameter) {
			list($key, $value) = explode('=', $getParameter);
			$getData[$key] = $value;
		}

		return $getData;
	}
}
