<?php

namespace org\adfinis\framework\request;

/**
 * Used to identify a POST variable
 * 
 * @const REQUEST_POST
 */
const REQUEST_POST = 1;

/**
 * Used to identify a GET variable
 * 
 * @const REQUEST_GET
 */
const REQUEST_GET = 2;

/**
 * Used to identify a COOKIE variable
 * 
 * @const REQUEST_COOKIE
 */
const REQUEST_COOKIE = 3;

/**
 * Used to identify a FILE variable
 * 
 * @const REQUEST_FILE
 */
const REQUEST_FILE = 4;

/**
 * Interface describing various possible request classes.
 *
 * The interface is designed to work like we had static typing: There is a
 * "getter" method for every supported data type, which should guarantee
 * correct behaviour when we get garbled data.
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
*/
interface RequestInterface {

	/**
	 * Returns the content of the given variable as an integer.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      int
	 */
	public function getInt($name, $source = REQUEST_POST);

	/**
	 * Returns the content of the given variable as a boolean value.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      boolean
	 */
	public function getBoolean($name, $source = REQUEST_POST);

	/**
	 * Returns the content of the given variable as floating-point number.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      boolean
	 */
	public function getFloat($name, $source = REQUEST_POST);

	/**
	 * Interprets the given variable as JSON data and returns an array.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      array
	 */
	public function getJSON($name, $source = REQUEST_POST);

	/**
	 * Returns the requested variable unmodified (ie. as a string or array).
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      mixed
	 */
	public function getRaw($name, $source = REQUEST_POST);

	/**
	 * Returns the requested variable as a string.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_FILE,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException                     If the data source does not exist
	 * @throws      OutOfRangeException                          If the variable does not exist.
	 * @throws      org\adfinis\exceptions\InvalidInputException If the variable could not be converted
	 * @return      string
	 */
	public function getString($name, $source = REQUEST_POST);

	/**
	 * Returns true if the current request is an AJAX request.
	 *
	 * This should return true if the request came from an XMLHTTPRequest.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isAJAX();

	/**
	 * Returns true if the current request is a POST request.
	 *
	 * This should return true if the request came from a form using method=POST.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      bool
	 */
	public function isPOST();

	/**
	 * Returns true if the requested key exists in the given data source, false otherwise.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $name   Variable name to get
	 * @param       int    $source Data source. Can be one of
	 *                         oaf\request\REQUEST_GET,
	 *                         oaf\request\REQUEST_POST,
	 *                         oaf\request\REQUEST_COOKIE.
	 *                     This is optional, and defaults to REQUEST_POST.
	 * @throws      InvalidArgumentException  If the data source does not exist
	 * @return      bool
	 */
	public function keyExists($name, $source = REQUEST_POST);
}


