<?php

namespace org\adfinis\framework\session;

/**
 * memcache session handling class and wrapping functions
 *
 * Used to register memcache servers for php session storage. There's a class
 * handling all stuff and some functions wrapping the class methods as
 * session_set_save_handler cannot be used with static class methods at all.
 * The memcache connection handling depends on memcacheManager class.
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @copyright   adfinis GmbH 2005-2010
 * @license     BSD 3-Clause
 */
class SessionHandlerMemcache {
	/**
	 * Session life time
	 *
	 * @var         int $life_time
	 */
	public static $life_time;

	/**
	 * Registers session storage handler
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @return      bool
	 */
	public static function initHandler(){
		return session_set_save_handler(
			'__session_handler_memcache_open',
			'__session_handler_memcache_close',
			'__session_handler_memcache_read',
			'__session_handler_memcache_write',
			'__session_handler_memcache_destroy',
			'__session_handler_memcache_gc');
	}

	/**
	 * Opens session storage
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $save_path
	 * @param       string $sess_name
	 * @return      bool true
	 */
	public static function open($save_path, $sess_name){
		// get session-lifetime
		self::$life_time = ini_get('session.gc_maxlifetime');
		return true;
	}

	/**
	 * Close session storage
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @return      bool true
	 */
	public static function close(){
		// don't do anything at all
		return true;
	}

	/**
	 * Read session data from storage
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $sess_id
	 * @return      mixed
	 */
	public static function read($sess_id){
		$id = sprintf('session_%s', $sess_id);
		return memcacheManager::getConnection()->get($id);
	}

	/**
	 * Write session data to storage
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $sess_id
	 * @param       string $sess_data
	 * @return      bool
	 */
	public static function write($sess_id, $sess_data){
		$id = sprintf('session_%s', $sess_id);
		return memcacheManager::getConnection()->set(
			$id,
			$sess_data,
			0,
			self::$life_time * 60
		);
	}

	/**
	 * Remove session data from storage
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $sess_id
	 * @return      bool
	 */
	public static function destroy($sess_id){
		$id = sprintf('session_%s', $sess_id);
		return memcacheManager::getConnection()->delete($id);
	}

	/**
	 * Session storage garbage collector
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       int $sess_maxlifetime
	 * @return      void
	 */
	public static function gc($sess_maxlifetime){
		// don't do anything, garbage collection is internally handled
		// by memcache
	}
}

/**
 * session handler object wrapping function for open
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @param       string $save_path
 * @param       string $sess_name
 * @return      bool
 */
function __session_handler_memcache_open($save_path, $sess_name){
	return sessionHandlerMemcache::open($save_path, $sess_name);
}

/**
 * session handler object wrapping function for close
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @return      bool
 */
function __session_handler_memcache_close(){
	return sessionHandlerMemcache::close();
}

/**
 * session handler object wrapping function for read
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @param       string $sess_id
 * @return      mixed
 */
function __session_handler_memcache_read($sess_id){
	return sessionHandlerMemcache::read($sess_id);
}

/**
 * session handler object wrapping function for write
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @param       string $sess_id
 * @param       string $sess_data
 * @return      bool
 */
function __session_handler_memcache_write($sess_id, $sess_data){
	return sessionHandlerMemcache::write($sess_id, $sess_data);
}

/**
 * session handler object wrapping function for destroy
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @param       string $sess_id
 * @return      bool
 */
function __session_handler_memcache_destroy($sess_id){
	return sessionHandlerMemcache::destroy($sess_id);
}

/**
 * session handler object wrapping function for gc
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @param       string $sess_maxlifetime
 * @return      void
 */
function __session_handler_memcache_gc($sess_maxlifetime){
	return sessionHandlerMemcache::gc($sess_maxlifetime);
}
