<?php

namespace org\adfinis\framework\template;

require_once 'Smarty.class.php';
use org\adfinis\framework\utils\URLHelper;
use org\adfinis\framework\utils\MailHelper;
use org\adfinis\framework\config\ConfManager;

/**
 * Smarty wrapper class
 *
 * Initializes and prepares smarty for use within the framework.
 * This provides various helpers and predefined values to be used within the templates.
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @todo by dv: register deactivated filters/functions again (rewrite plugins?)
 * @todo by dv: Document preset values / functions / plugins etc.
 * @copyright   adfinis GmbH 2009-2011
 * @license     BSD 3-Clause
 * @see         http://smarty.php.net/manual/en/smarty.for.programmers.php
 */
class Skittle extends \Smarty {

	/**
	 * Initializes Smarty and registers some basic application variables
	 *
	 * $domainIdentifier defines, which URL should be used in this instance.
	 * If you skip this parameter, we'll use the domain which is defined as
	 * 'default' in globalConfig.php. Define more URLs there if needed.
	 *
	 * @author      Mathias Blaser <mb@adfinis.ch>
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @todo by dv: When updating smarty, consider rewriting the smarty constructor
	 */
	public function __construct($domainIdentifier = 'default'){
		// Initialize parent constructor
		parent::__construct();

		// Set default template directories
		$this->setTemplateDir(
			ConfManager::getVar('af', 'smarty', 'templates', 'path'));
		$this->setCompileDir(
			ConfManager::getVar('af', 'smarty', 'compiled',  'path'));
		$this->setCacheDir(
			ConfManager::getVar('af', 'smarty', 'cached',    'path'));

		$this->setCaching(false);
		$this->setDebugging(false);

		// Check if there's a new version of template to compile (e.g. newer timestamp)
		//$this->compile_check = false;

		$this->assignDefaultVariables();

		$this->registerPlugin(
			'block',
			'xmltransform',
			array('org\\adfinis\\framework\\template\\XMLTransform', 'doTransform')
		);

		// @todo by dv: register those filters/functions again
		// Register some additional smarty functions
		$this->registerPlugin(
			'modifier',
			'tr',
			array($this, 'modifierI18n')
		);

		$this->registerPlugin(
			'function',
			'trn',
			array($this, 'functionI18n')
		);


		// Do not print notices in templates...
		$this->error_reporting = E_ALL & ~E_NOTICE;
	}

	/**
	 * Assigns a few default variables. Called from the constructor.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @todo by nc: where/why do we use $server['request_url']? Is it still needed?
	 * @return      void
	 */
	protected function assignDefaultVariables() {
		// Some super globals
		$server = array_change_key_case($_SERVER);

		if(isset($_SERVER['REDIRECT_URL'])){
			$server['request_url'] = sprintf(
				'http://%s%s',
				$_SERVER['HTTP_HOST'],
				$_SERVER['REDIRECT_URL']
			);
			$server['request_url_relative'] = $_SERVER['REDIRECT_URL'];
		}
		elseif(isset($_SERVER['HTTP_HOST'])) {
			$server['request_url'] = sprintf(
				'http://%s%s',
				$_SERVER['HTTP_HOST'],
				$_SERVER['REQUEST_URI']
			);
			$server['request_url_relative'] = $_SERVER['REQUEST_URI'];
		}

		// Some often used URLs
		$this->assign('URLNoSSL', URLHelper::getURLInsecure());
		$this->assign('URLSSL',   URLHelper::getURLSecure());

		// Some global smarty variables
		$this->assign('server', $server);
		$this->assign('webRoot', sprintf(
			'%s',
			ConfManager::getVar('af', 'webRoot', 'uri'))
		);
		$this->assign('webRootPictures', sprintf(
			'%s',
			ConfManager::getVar('af', 'webRoot', 'pictures', 'uri'))
		);
		$this->assign('webRootTemplatePictures', sprintf(
			'%s',
			ConfManager::getVar('af', 'webRoot', 'templatePictures', 'uri'))
		);
	}

	/**
	 * Clears all assigned variables and assignes the default variables again
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function clearAllAssign(){
		parent::clearAllAssign();
		$this->assignDefaultVariables();
	}

	/**
	 * Fetches a cached version of the template,
	 * if no cache is available, the callback is executed.
	 *
	 * The first parameter of the callback is the Skittle instance.
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string   $template
	 * @param         string   $cacheID  [optional]
	 * @param         int      $lifetime [optional]
	 * @param         Closure  $callback [optional]
	 * @returns       string
	 */
	public function fetchCached($template, $cacheID = null, $lifetime = 600, \Closure $callback = null) {
		$caching     = $this->getCaching();
		$oldLifetime = $this->getCacheLifetime();

		$this->setCaching(true);
		$this->setCacheLifetime($lifetime);

		if ($callback !== null && !$this->isCached($template, $cacheID)) {
			try {
				$callback($this);
			}
			catch (\Exception $e) {
				$this->setCaching($caching);
				$this->setCacheLifetime($oldLifetime);

				throw $e;
			}
		}

		$output = $this->fetch($template, $cacheID);

		$this->setCaching($caching);
		$this->setCacheLifetime($oldLifetime);

		return $output;
	}

	/**
	 * Modifier function returning translation of a string
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         string $input
	 * @return        string
	 */
	public function modifierI18n($input){
		return tr($input);
	}

	/**
	 * Modifier function supporting plural/singular translation
	 *
	 * If only the text argument is given, return the normal
	 * translation. If the arguments textn and number are given as well,
	 * the singular/plural translation function will be called
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 * @param         array $params
	 * @param         Smarty_Internal_Template $template
	 * @return        string
	 */
	public function functionI18n($params, $template){
		if (!isset($params['text'])) {
			trigger_error("i18n: missing 'text' parameter");
		}

		$text1 = $params['text'];

		if (isset($params['textn']) && isset($params['number'])) {
			$textn  = $params['textn'];
			$number = $params['number'];
			return trn($text1, $textn, $number);
		}

		return tr($text1);
	}
}
