<?php

namespace org\adfinis\framework\template;

/**
 * Smarty plugin to do XML manipulation on blocks
 *
 * Provides the "xmltransform" block in smarty that can do XML level manipulation on xhtml.
 *
 * You can write a static method in your own class, taking a PQLite object as an IN/OUT parameter.
 * This method can be specified in the block tag. Here's an example:
 * 
 * @code
 * <!-- Note you can pass additional params to xmltransform, which will be passed to the method. -->
 * {xmltransform with="Menu::setCurrentActive"}
 *    <li class="entry1"><a href="entry1.php"></li>
 *    <li class="entry2"><a href="entry2.php"></li>
 *    <li class="entry3"><a href="entry3.php"></li>
 * {/xmltransform}
 * @endcode
 *
 * The transform function could be something like this:
 *
 * @code
 *    class Menu {
 *        public static function setCurrentActive($xml, $params) {
 *            global $currentMenu;
 *            $elem = $xml->find(sprintf('li.%s', self::$toplevel));
 *
 *            $class = $elem->getAttr('class');
 *            $class .= " active";
 *
 *            $elem->setAttr('class', $class);
 *        }
 *    }
 * @endcode
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class XMLTransform {

	/**
	 * Transforms the given HTML block to an XML node, calls the specified function, then returns the modified data.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        \InvalidArgumentException If the wrapped XML is not well-formed
	 * @param         array   $params
	 * @param         string  $content
	 * @param         \Smarty $smarty
	 * @param[in,out] bool    &$repeat
	 * @param         string  $template (optional, defaults to NULL)
	 * @returns       string
	 */
	public static function doTransform($params, $content, $smarty, &$repeat, $template=null) {
		// Only run on closing tag
		if($repeat) {
			return;
		}

		$xml = new \PQLite($content);

		$funcStr = $params['with'];
		unset($params['with']);

		// static call needs to be array(class, method)
		if(preg_match('/::/', $funcStr)) {
			$func = preg_split('/::/', $funcStr);
		}
		else {
			$func = $funcStr;
		}

		if(is_callable($func)) {
			call_user_func($func, $xml, $params);
		}
		else {
			throw new \InvalidArgumentException(sprintf('DOM transform: "with" "%s" does not reference a callable', $funcStr));
		}
		return $xml->getHTML();
	}
}

