<?php

namespace org\adfinis\framework\template\smorfy;

/**
 * Class to handle DOCX documents
 *
 * @author        Damian Senn <ds@adfinis.com>
 */
class DOCXDocument extends Document {

	/**
	 * Get the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getContent() {
		return $this->zip->getFromName('word/document.xml');
	}

	/**
	 * Set the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $content
	 * @returns       ODTDocument
	 */
	public function setContent($content) {
		$this->zip->deleteName('word/document.xml');
		$this->zip->addFromString('word/document.xml', $content);

		return $this;
	}

	/**
	 * Join the DOCX Documents with an optional separator (default is a page break)
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array<DOMDocument> $documents
	 * @param         DOMNode            $separator
	 * @returns       DOMDocument
	 */
	public static function join($documents, $separator = null) {
		$docx      = array_shift($documents);
		$docxpath  = new \DOMXPath($docx);
		$lastChild = $docxpath->query('/w:document/w:body/w:sectPr')->item(0);
		$parent    = $lastChild->parentNode;

		if (!$separator) $separator = self::createPageBreak($docx);

		foreach ($documents as $document) {
			$xpath = new \DOMXPath($document);

			$body = $xpath->query('/w:document/w:body')->item(0);
			$body->removeChild($body->lastChild);

			$parent->insertBefore($separator->cloneNode(true), $lastChild);

			foreach ($body->childNodes as $node) {
				$parent->insertBefore($docx->importNode($node, true), $lastChild);
			}
		}

		return $docx;
	}

	/**
	 * Creates a <w:pageBreakBefore/> element
	 *
	 * <w:p>
	 *   <w:pPr>
	 *     <w:pStyle w:val="style0"/>
	 *     <w:pageBreakBefore/>
	 *     <w:tabs>
	 *       <w:tab w:leader="none" w:pos="709" w:val="left"/>
	 *       <w:tab w:leader="none" w:pos="5710" w:val="left"/>
	 *     </w:tabs>
	 *     <w:jc w:val="left"/>
	 *   </w:pPr>
	 *   <w:r>
	 *     <w:rPr/>
	 *   </w:r>
	 * </w:p>
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @todo by ds:   createElementNS and setAttributeNS do not seem to work here? Oo
	 * @returns       DOMNode
	 */
	private static function createPageBreak($doc) {
		$p   = $doc->createElement('w:p');
		$pPr = $doc->createElement('w:pPr');
		$p->appendChild($pPr);

		$pStyle = $doc->createElement('w:pStyle');
		$pStyle->setAttribute('w:val', 'style0');
		$pPr->appendChild($pStyle);

		$pageBreakBefore = $doc->createElement('w:pageBreakBefore');
		$pPr->appendChild($pageBreakBefore);

		$r = $doc->createElement('w:r');
		$p->appendChild($r);

		$rPr = $doc->createElement('w:rPr');
		$r->appendChild($rPr);

		return $p;
	}

	/**
	 * Get a Smorfy instance to process this document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       SmorfyODT
	 */
	public static function getSmorfy() {
		return new SmorfyDOCX();
	}
}
