<?php

namespace org\adfinis\framework\template\smorfy;

/**
 * Class to represent a text processing document like ODT or DOCX
 *
 * @author        Damian Senn <ds@adfinis.com>
 */
abstract class Document {

	/**
	 * Indicator, if the file is closed
	 *
	 * This is neede if the close function is called
	 * manually, so the destructor won't close it again
	 */
	private $closed = false;

	/**
	 * Open a document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $filename
	 * @returns       Document
	 */
	public static function open($filename) {
		preg_match('/\.(\w+)$/', $filename, $matches);

		switch ($matches[1]) {
			case 'odt':
				return new ODTDocument($filename);
			case 'docx':
				return new DOCXDocument($filename);
			default:
				throw new \InvalidArgumentException('Unknown file type!');
		}
	}

	/**
	 * Get a Smorfy instance to process this type of Document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       Smorfy
	 */
	public static function getSmorfy() {
		throw new \Exception(sprintf('%s::getSmorfy() is not implemented yet!',
			get_called_class()));
	}

	/**
	 * Holds the document
	 *
	 * @var           \ZipArchive $zip
	 */
	protected $zip;

	/**
	 * Constructor of Document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $filename
	 */
	public function __construct($filename) {
		$this->zip = new \ZipArchive();
		$this->zip->open($filename);
	}

	/**
	 * Get the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	abstract public function getContent();

	/**
	 * Set the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string
	 * @returns       Document
	 */
	abstract public function setContent($content);

	/**
	 * Join several documents together
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array<DOMDocument> $documents
	 * @param         DOMNode            $separator [optional]
	 * @returns       DOMDocument
	 */
	public static function join($documents, $separator = null) {
		throw new \Exception(sprintf('%s::join() is not implemented yet!',
			get_called_class()));
	}

	/**
	 * Close the file 
	 *
	 * @author        Stefan Heinemann <stefan.heinemann@adfinis-sygroup.ch>
	 */
	public function close() {
		if (!$this->close) {
			$this->zip->close();
		}
	}

	/**
	 * Destructor of Document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 */
	public function __destruct() {
		$this->close();
	}
}
