<?php

namespace org\adfinis\framework\template\smorfy;

/**
 * Class to handle ODT documents
 *
 * @author        Damian Senn <ds@adfinis.com>
 */
class ODTDocument extends Document {

	/**
	 * Get the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       string
	 */
	public function getContent() {
		return $this->zip->getFromName('content.xml');
	}

	/**
	 * Get the document content
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $content
	 * @returns       ODTDocument
	 */
	public function setContent($content) {
		$this->zip->deleteName('content.xml');
		$this->zip->addFromString('content.xml', $content);

		return $this;
	}

	/**
	 * Join the ODT Documents with an optional separator (default is a page break)
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         array<DOMDocument> $documents
	 * @param         DOMNode            $separator
	 * @returns       DOMDocument
	 */
	public static function join($documents, $separator = null) {
		$odt     = array_shift($documents);
		$odtpath = new \DOMXPath($odt);
		$text    = $odtpath->query('/office:document-content/office:body/office:text')->item(0);

		if (!$separator) {
			$style = self::createPageBreakStyle($odt);
			$odtpath->query('//office:automatic-styles')->item(0)->appendChild($style);

			$separator = self::createPageBreak($odt);
		}

		foreach ($documents as $document) {
			$xpath = new \DOMXPath($document);

			$body = $xpath->query('/office:document-content/office:body/office:text')->item(0);
			$body->removeChild($body->firstChild);

			$text->appendChild($separator->cloneNode(true));

			foreach ($body->childNodes as $node) {
				$text->appendChild($odt->importNode($node, true));
			}
		}

		return $odt;
	}

	/**
	 * Returns a style for a pageBreak in an ODT content file
	 *
	 * <style:style style:name="pageBreak" style:family="paragraph" style:parent-style-name="Standard">
	 *   <style:paragraph-properties fo:break-before="page"/>
	 *   <style:text-properties officeooo:rsid="0013112d" officeooo:paragraph-rsid="0014d1b2"/>
	 * </style:style>
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @todo by ds:   setAttributeNS do not seem to work here? Oo
	 * @param         DOMDocument $doc
	 * @returns       DOMNode
	 */
	private static function createPageBreakStyle($doc) {
		$style = $doc->createElement('style:style');
		$style->setAttribute('style:name',   'pageBreak');
		$style->setAttribute('style:family', 'paragraph');
		$style->setAttribute('style:parent-style-name', 'Standard');

		$paragraphProperties = $doc->createElement('style:paragraph-properties');
		$paragraphProperties->setAttribute('fo:break-before', 'page');
		$style->appendChild($paragraphProperties);

		$textProperties = $doc->createElement('style:text-properties');
		$textProperties->setAttribute('officeooo:rsid', '0013112d'); // halp
		$textProperties->setAttribute('officeooo:paragraph-rsid', '0014d1b2'); // halp
		$style->appendChild($textProperties);

		return $style;
	}

	/**
	 * Returns a page break for an ODT content file
	 *
	 * <text:p text:style-name="pageBreak"/>
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @todo by ds:   createElementNS and setAttributeNS do not seem to work here? Oo
	 * @param         DOMDocument $doc
	 * @returns       DOMNode
	 */
	private static function createPageBreak($doc) {
		$p = $doc->createElement('text:p');
		$p->setAttribute('text:style-name', 'pageBreak');

		return $p;
	}

	/**
	 * Get a Smorfy instance to process this document
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       SmorfyODT
	 */
	public static function getSmorfy() {
		return new SmorfyODT();
	}
}
