<?php

namespace org\adfinis\framework\template\smorfy;

/**
 * Smorfy for processing OOo / M$ Word documents
 *
 * @author        Damian Senn <ds@adfinis.com>
 */
abstract class Smorfy {

	/**
	 * Get the document body
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       DOMNode
	 */
	abstract public function getDocumentBody($xpath);

	/**
	 * Replace simple placeholders
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         DOMXPath $xpath
	 * @param         DOMNode  $body
	 * @returns       void
	 */
	abstract protected function replaceTextPlaceholders($xpath, $body);

	/**
	 * Replace placeholders in tables
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         DOMXPath $xpath
	 * @param         DOMNode  $body
	 * @returns       void
	 */
	abstract protected function replaceTablePlaceholders($xpath, $body);

	/**
	 * Replace placeholders in list items
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         DOMXPath $xpath
	 * @param         DOMNode  $body
	 * @returns       void
	 */
	abstract protected function replaceListPlaceholders($xpath, $body);

	/**
	 * Holds the assigned variables
	 *
	 * @var           array $variables
	 */
	protected $variables = array();

	/**
	 * Assign variables to smorfy
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string|array $var
	 * @param         mixed        $val
	 * @returns       Smorfy
	 */
	public function assign($var, $val = null) {
		if (is_array($var)) {
			foreach ($var as $var => $val) {
				$this->assign($var, $val);
			}
		}

		$this->variables[$var] = $val;

		return $this;
	}

	/**
	 * Processes the given xml and returns a DOMDocument
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string  $xml
	 * @param         DOMNode $body
	 * @returns       DOMDocument
	 */
	public function process($xml, $body = null) {
		if (is_string($xml) && is_file($xml)) {
			$xml = file_get_contents($xml);
		}

		if ($xml instanceof Document) {
			$xml = $xml->getContent();
		}

		if ($xml instanceof \DOMDocument) {
			$doc = $xml;
		}
		else {
			$doc = new \DOMDocument();
			$doc->loadXML($xml);
		}

		$xpath = new \DOMXPath($doc);

		if (!$body) {
			$body = $this->getDocumentBody($xpath);
		}

		$this->replaceTextPlaceholders($xpath, $body);
		$this->replaceTablePlaceholders($xpath, $body);
		$this->replaceListPlaceholders($xpath, $body);

		return $doc;
	}

	/**
	 * Returns the XML string of the given file processed with the assigned vars
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string  $xml
	 * @param         bool    $format
	 * @param         DOMNode $body
	 * @returns       string
	 */
	public function fetch($xml, $format = false, $body = null) {
		$doc = $this->process($xml, $body);

		$doc->formatOutput = $format;

		return $doc->saveXML();
	}

	/**
	 * Saves the xml to the given path
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @throws        \RuntimeException
	 * @param         string $xml
	 * @param         string $path
	 * @param         bool   $format
	 * @returns       int
	 */
	public function save($xml, $path, $format = false) {
		$size = file_put_contents($path, $this->fetch($xml, $format));

		if ($size === false) {
			throw new \RuntimeException(sprintf('Could not save XML to "%s"!', $path));
		}

		return $size;
	}
}
