<?php

namespace org\adfinis\framework\uri;

use org\adfinis\framework\utils\URLHelper;
use org\adfinis\framework\config\ConfManager;

/**
 * Object to create and manage an URI
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @copyright   adfinis GmbH 2010
 * @license     BSD 3-Clause
 */
class URI {

	/**
	 * Holds the GET parameters of this URL
	 *
	 * @var array
	 */
	public $get;

	/**
	 * Array containing each URL part but the GET string
	 *
	 * @var array
	 */
	private $URL;

	/**
	 * Domain identifier string
	 *
	 * @var string
	 */
	private $domainIdentifier;

	/**
	 * Array to define which parts should be returned
	 *
	 * @var array
	 */
	private $show;

	/**
	 * Flag to check if the URL already has been processed
	 *
	 * @var string|false
	 */
	private $cache;

	/**
	 * URL constructor
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $domainIdentifier (optional)
	 * @param       bool   $secure           (optional)
	 */
	public function __construct($domainIdentifier = 'current', $secure = false){
		$this->domainIdentifier = $domainIdentifier;

		// set scheme through setURL method to initialize the URL property
		$configKey = $secure ? 'URLSSL' : 'URLNoSSL';

		$baseURL = ConfManager::getVar('af', $configKey, $domainIdentifier);
		$urlParts   = parse_url($baseURL);

		$this->setURL($urlParts['scheme'], $urlParts['host']);

		$this->show = array(
			'scheme' => true,
			'host'   => true,
			'port'   => true,
			'path'   => true,
			'get'    => true,
			'hash'   => true);
	}

	/**
	 * Method to set all URL specific parts at once.
	 *
	 * Warning: any omitted parameter will be overwritten
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $scheme (optional)
	 * @param       string $host   (optional)
	 * @param       int    $port   (optional)
	 * @param       array  $path   (optional)
	 * @param       hash   $hash   (optional)
	 * @return      this
	 */
	public function setURL($scheme = 'http',
	                       $host = null,
	                       $port = '',
	                       $path = '',
	                       array $get = array(),
	                       $hash = ''){
		if($host === null) $host = URLHelper::getFQDN($this->domainIdentifier);

		$this->URL = array(
			'scheme' => $scheme,
			'host'   => $host,
			'port'   => $port,
			'path'   => $path,
			'hash'   => $hash);

		$this->get   = $get;
		$this->cache = false;

		return $this;
	}

	/**
	 * Method to set which parts of the URL should be shown.
	 *
	 * This method can have several different parameter pairs.
	 *
	 * 1. String
	 *    Returns a boolean if the given part is shown or not
	 *
	 * 2. String & Bool:
	 *    Sets visibility of the given part
	 *
	 * 3. Array:
	 *    Sets visibility of the given parts, the array should look
	 *    something like this:
	 *      array('scheme' => true, 'port' => false)
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       array|string $part
	 * @param       bool         $show (optional)
	 * @return      bool|this
	 */
	public function show($part, $show = null){
		if(is_array($part)){
			foreach($part as $key => $value){
				$this->show($key, $value);
			}

			return $this;
		}

		if($show === null) return $this->show[$part];

		$this->show[$part] = (bool)$show;
		$this->cache       = false;

		return $this;
	}

	/**
	 * Returns the URL as string according to the show property
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function toString(){
		if($this->cache !== false
				&& $this->cache['get'] === $this->get){
			return $this->cache['url'];
		}

		$parts = $this->URL;

		$parts['get']    = $this->buildGETString();
		$parts['scheme'] = $parts['scheme'] ? $parts['scheme'] . '://'        : '';
		$parts['hash']   = $parts['hash']   ? '#' . urlencode($parts['hash']) : '';
		$parts['port']   = $parts['port']   ? ':' . (int)$parts['port']       : '';

		// We can't urlencode the whole path as slashes would be escaped too
		$parts['path'] = implode('/', array_map(
			function($val){
				return urlencode($val);
			},
			explode('/', $parts['path'])));

		$url = '';
		foreach($this->show as $part => $show){
			if($show) $url .= $parts[$part];
		}

		$this->cache['url'] = $url;
		$this->cache['get'] = $this->get;

		return $url;
	}

	/**
	 * Shorthand function to set if an URL should be relative or absolute
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       bool $relative (optional)
	 * @return      bool|this
	 */
	public function relative($relative = null){
		if($relative === null){
			$show = $this->show;
			return !($show['scheme'] || $show['host'] || $show['port']);
		}

		$this->cache = false;

		if($relative){
			return $this->show(array(
				'scheme' => false,
				'host'   => false,
				'port'   => false));
		}

		return $this->show(array(
			'scheme' => true,
			'host'   => true,
			'port'   => true));
	}

	/**
	 * Build a string with GET parameters
	 *
	 * @author      Nicolas Christener <nc@adfinis.com>
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	protected function buildGETString(){
		if(count($this->get) > 0){
			return '?' . http_build_query($this->get);
		}

		return '';
	}

	/**
	 * Parses the given URL and returns a new URL object
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $URL
	 * @return      URL
	 */
	public static function parse($URL){
		$urlObject = new self();

		$parts = parse_url($URL);
		$get   = array();

		if(isset($parts['query'])) parse_str($parts['query'], $get);

		$urlObject->setURL(
			isset($parts['scheme'])   ? $parts['scheme']              : 'http',
			isset($parts['host'])     ? $parts['host']                : null,
			isset($parts['port'])     ? (int)$parts['port']           : '',
			isset($parts['path'])     ? urldecode($parts['path'])     : '',
			$get,
			isset($parts['fragment']) ? urldecode($parts['fragment']) : '');

		return $urlObject;
	}

	/**
	 * Magic method to set values of the encapsulated properties
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @param       mixed  $value
	 * @return      void
	 */
	public function __set($name, $value){
		$this->cache = false;

		if(array_key_exists($name, $this->URL)){
			$this->URL[$name] = $value;

			return;
		}
	}

	/**
	 * Magic method to get values of the encapsulated properties
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $name
	 * @return      void
	 */
	public function __get($name){
		if(array_key_exists($name, $this->URL)){
			return $this->URL[$name];
		}
	}

	/**
	 * Magic method which is called in string contexts
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public function __toString(){
		return $this->toString();
	}
}
