<?php

namespace org\adfinis\framework\utils;

/**
 * Class to get some information about a browser
 *
 * Examples:
 *   $browser = Browser::getInstance($userAgent);
 *
 * - Bot check:           $browser->bot
 * - Video support:       $browser->support['video']
 * - Rendering engine:    $browser->webkit || $browser->mozilla
 * - iPad / iPhone check: $browser->ipad   || $browser->iphone
 * - user agent:          $browser->ua
 * - operating system:    $browser->os === 'linux'
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class Browser {

	/**
	 * Name of the rendering engine
	 *
	 * @var           string $name
	 */
	public $name;

	/**
	 * Browser version
	 *
	 * @var           string $version
	 */
	public $version;

	/**
	 * Instances
	 *
	 * @var           array<Browser>
	 */
	private static $instances = array();

	/**
	 * Get instance
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $ua [optional]
	 * @returns       Browser
	 */
	public static function getInstance($ua = null) {
		if (!$ua) {
			$ua = isset($_SERVER['HTTP_USER_AGENT'])
				? $_SERVER['HTTP_USER_AGENT']
				: '';
		}

		$ua = strtolower($ua);

		return isset(self::$instances[$ua])
			? self::$instances[$ua]
			: self::$instances[$ua] = new self($ua)
		;
	}

	/**
	 * Prevent from cloning
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 */
	private function __clone() { }

	/**
	 * Constructor of Browser
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 */
	private function __construct($ua) {
		$this->ua = $ua;
		$matches  = array();
		$support  = array();

		$this->webkit  = false;
		$this->opera   = false;
		$this->msie    = false;
		$this->mozilla = false;
		$this->ipad    = strpos($ua, 'ipad')    !== false;
		$this->iphone  = strpos($ua, 'iphone')  !== false;
		$this->android = strpos($ua, 'android') !== false;
		$this->os      = $this->getOS();

		if ($this->isBot()) {
			$this->support = array(
				'video' => false,
				'es5'   => false
			);

			return;
		}

		preg_match('/(webkit)[ \/]([\w.]+)/', $ua, $matches) ?:
		preg_match('/(opera)(?:.*version)?[ \/]([\w.]+)/', $ua, $matches) ?:
		preg_match('/(msie) ([\w.]+)/', $ua, $matches) ?:
		// FF not IE
		strpos($ua, 'compatible') === false &&
		preg_match('/(mozilla)(?:.*? rv:([\w.]+))?/', $ua, $matches);

		if (count($matches) > 1) {
			$this->name    = $matches[1];
			$this->version = $matches[2];
		}

		if ($this->name) $this->{$this->name} = true;

		$support['video'] = $this->videoSupport();
		$support['es5']   = $this->ES5Support(); // ES5 syntax support, not functionality

		$this->support = $support;
	}

	/**
	 * Method to set the video support
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       array
	 */
	private function videoSupport() {
		switch($this->name) {
			// Assume video on all webkit / mozilla browsers, other than safari on windows.
			// If they are not up to date, they deserve some errors on the page! ^^
			case 'webkit':
				if ($this->ipad || $this->iphone || $this->android) {
					return true;
				}

				// Safari on windows does not support any media element at all!
				// What about the "On Apple products everything just works!" well...
				if (strpos($this->ua, 'chrome') === false && // Google... why is there a string safari in your useragent?!
						strpos($this->ua, 'safari') !== false &&
						$this->os === 'windows') {
					return false;
				}
			case 'mozilla':
				return true;
			case 'msie':
				return (int)$this->version >= 9;
			case 'opera':
				return (float)$this->version >= 9.52;
			default:
				return false;
		}
	}

	/**
	 * Method to set the ECMAScript5 support
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       array
	 */
	private function ES5Support() {
		switch($this->name) {
			case 'webkit':
			case 'mozilla':
				return true;
			case 'msie':
				return (int)$this->version >= 9;
			case 'opera':
				return (float)$this->version >= 9.5;
			default:
				return false;
		}
	}

	/**
	 * Checks if user agent is a spider/crawler/bot
	 *
	 * Note that the list is shortened for various reasons, see
	 *
	 * http://www.robotstxt.org/db.html
	 * http://www.robotstxt.org/dbexport.html
	 * http://www.robotstxt.org/db/all.txt
	 *
	 * for a more complete list if you need to add additional bots
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @param         string $userAgent
	 * @returns       bool
	 */
	private function isBot(){
		$blacklist = array();

		// Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)
		$blacklist[] = 'googlebot';

		// msnbot-media/1.0 (+http://search.msn.com/msnbot.htm)
		$blacklist[] = 'msnbot-media';

		// psbot/0.1 (+http://www.picsearch.com/bot.html)
		$blacklist[] = 'psbot';

		// msnbot/1.1 (+http://search.msn.com/msnbot.htm)
		$blacklist[] = 'msnbot';

		// WikioFeedBot 1.0
		$blacklist[] = 'wikiofeedbot';

		// Mozilla/5.0 (compatible; Yahoo! Slurp; http://help.yahoo.com/help/us/ysearch/slurp)
		$blacklist[] = 'slurp';

		// Mozilla/5.0 (compatible; Ask Jeeves/Teoma; +http://about.ask.com/en/docs/about/webmasters.shtml)
		$blacklist[] = 'jeeves';

		// Mozilla/5.0 (compatible; Ask Jeeves/Teoma; +http://about.ask.com/en/docs/about/webmasters.shtml)
		$blacklist[] = 'teoma';

		foreach($blacklist as $part){
			if(strpos($this->ua, $part) !== false){
				$this->name = $part;

				return $this->bot = true;
			}
		}

		return $this->bot = false;
	}

	/**
	 * Get operating system in the user agent
	 *
	 * Just a simple function to find the most important
	 * operating systems. Returns windows, android, linux, ios, mac or null
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @author        Damian Senn <ds@adfinis.com>
	 * @return        string
	 */
	public function getOS(){
		if(strpos($this->ua, 'windows') !== false){
			return 'windows';
		}

		if($this->android){
			return 'android';
		}

		if(strpos($this->ua, 'linux') !== false){
			return 'linux';
		}

		if(strpos($this->ua, 'iphone os') !== false){
			return 'ios';
		}

		if(strpos($this->ua, 'mac') !== false){
			return 'mac';
		}

		return null;
	}
}
