<?php

namespace org\adfinis\framework\utils;

use org\adfinis\framework\config\ConfManager;

/**
 * A helper class for generating and validating CAPTCHAs.
 *
 *
 * A very simple showcase on how this works can be seen in the
 * following example:
 * @code
 * $captcha = new CaptchaHelper();
 * if($_POST) {
 *     printf('Validating....<br/>');
 *     $valid = $captcha->validate($_POST['id'], $_POST['captcha']);
 *     printf('Validation: %s', ($valid ? 'YES' : 'NO'));
 * }
 * else {
 *     printf('<form method="post">');
 *     printf('<img src="%s" /><br/>', $captcha->getPictureURL());
 *     printf('<input name="captcha" type="text" />');
 *     printf('<input name="id"      type="hidden" value="%s"/>', $captcha->getID());
 *     printf('<input type="submit">');
 *     printf('</form>');
 * }
 * @endcode
 *
 * @author      David Vogt <dv@adfinis.com>
 * @todo by dv: Write a Form field class to simplify usage
 * @todo by dv: We should also provide a CAPTCHA mechanism for sight-impaired people.
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
 */
class CaptchaHelper {

	/**
	 * Preset value for expiration time. Defaults to 10 minutes.
	 *
	 * This is the default value.
	 *
	 * @const int EXPIRATION_SHORT
	 */
	const EXPIRATION_SHORT = 600;

	/**
	 * Preset value for expiration time. Defaults to 5 days.
	 *
	 * @const int EXPIRATION_LONG
	 */
	const EXPIRATION_LONG = 864000;

	/**
	 * Internal reference to the wrapped Zend_Captcha_Image object.
	 *
	 * @var Zend_Captcha_Image $zendCaptcha
	 */
	private $zendCaptcha;

	/**
	 * This is set to true when the internal 
	 * Zend_Captcha_Image object was generate()d.
	 *
	 * @var bool $isGenerated
	 */
	private $isGenerated;

	/**
	 * Creates a new CAPTCHA helper instance.
	 *
	 * This initializes the CAPTCHA with a size of 300x80 pixels. 
	 * You may change this using setSize().
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 */
	public function __construct() {
		// Initialize basic setup
		$this->zendCaptcha = new \Zend_Captcha_Image();
		$this->zendCaptcha->setExpiration(self::EXPIRATION_SHORT);
		$this->zendCaptcha->setWidth(300);
		$this->zendCaptcha->setHeight(80);

		// Find font to use
		$fontFiles = glob(ConfManager::getVar('af', 'captcha', 'fonts', 'path') . "/*");
		$fontToUse = $fontFiles[array_rand($fontFiles)];
		$this->zendCaptcha->setFont($fontToUse);

		// Paths / URLs
		$this->zendCaptcha->setImgDir(ConfManager::getVar('af', 'picturesTmp', 'path'));
		$this->zendCaptcha->setImgUrl(ConfManager::getVar('af', 'picturesTmp', 'uri' ));

		$this->isGenerated = false;
	}

	/**
	 * Sets the size of the CAPTCHA.
	 *
	 * Takes two integers, $width and $height as parameters.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       int $width
	 * @param       int $height
	 * @return      void
	 */
	public function setSize($width, $height) {
		
		$this->zendCaptcha->setWidth($width);
		$this->zendCaptcha->setHeight($height);
	}

	/**
	 * Sets expiration time.
	 *
	 * You can use a custom amount of time (in seconds) or use 
	 * the constants EXPIRATION_SHORT or EXPIRATION_LONG.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       int $expiration
	 * @return      void
	 */
	public function setExpiration($expiration) {
		$this->zendCaptcha->setExpiration($expiration);
	}
	
	/**
	 * Ensures the CAPTCHA is generated.
	 *
	 * This is used internally before fetching data from the
	 * Zend_Captcha_Image object to make sure the data is present.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       int $expiration
	 * @return      void
	 */
	private function ensureIsGenerated() {
		if(!$this->isGenerated) {
			$this->zendCaptcha->generate();
			$this->isGenerated = true;
		}
	}

	/**
	 * Returns the URL to the generated picture.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @todo by dv: Use UrlObject maybe?
	 * @return      string
	 */
	public function getPictureURL() {
		$this->ensureIsGenerated();
		$url = $this->zendCaptcha->getImgUrl()
			. $this->zendCaptcha->getId()
			. $this->zendCaptcha->getSuffix();

		return $url;
	}

	/**
	 * Returns the identifier of the generated picture.
	 *
	 * The result is a hex string identifying the CAPTCHA
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      string
	 */
	public function getID() {
		$this->ensureIsGenerated();
		return $this->zendCaptcha->getId();
	}

	/**
	 * Returns the picture data stream of the generated CAPTCHA.
	 *
	 * The file format is image/png. You may not want to use
	 * this directly; consider using renderPicture() instead.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      string
	 */
	public function getPictureData() {
		$this->ensureIsGenerated();
		$path = $this->zendCaptcha->getImgDir()
			. $this->zendCaptcha->getId()
			. $this->zendCaptcha->getSuffix();
		return readfile($path);
	}

	/**
	 * Sends the generated CAPTCHA to the browser.
	 *
	 * The picture is sent to the browser, including the 
	 * correct content-type header and past expiry.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @return      string
	 */
	public function renderPicture() {
		header('Content-Type: image/png');

		// Past expiration header, so browsers don't cache it
		header('Expiration: Sun, 06 Nov 1994 08:49:37 GMT');
		header('Cache-Control: no-cache');

		echo $this->getPictureData();
	}

	/**
	 * Returns true if the user entered the correct value.
	 *
	 * Compares the given value with the stored value for the CAPTCHA with the given id to determine it's validity.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $id
	 * @param       string $value
	 * @return      bool
	 */
	public static function validate($id, $value) {
		// Because Zend_Captcha_Word is a bitch when it is used before 
		// validating the old value, we generate a new captcha object
		// for validating.
		$validatingCaptcha = new \Zend_Captcha_Image();
		
		// AARRRRGHHH why are they requesting such a moronic data structure?
		$valueForCaptcha = array(
				'id'    => $id,
				'input' => $value
		);
		$valid = $validatingCaptcha->isValid($valueForCaptcha);
		return $valid;
	}
}

