<?php

namespace org\adfinis\framework\utils;

/**
 * Helper Class dealing with dates
 *
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @todo by dv: Check whether this could be obsoleted by using Zend_Date.
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class DateHelper {
	/**
	 * Short month format ie "January", "February", ...
	 * @const         string MONTH_FORMAT_FULL
	 */
	const MONTH_FORMAT_FULL  = 'full';

	/**
	 * Short month format ie "Jan", "Feb", ...
	 * @const         string MONTH_FORMAT_SHORT
	 */
	const MONTH_FORMAT_SHORT = 'short';

	/**
	 * Long weekday format: "Monday", "Tuesday", ...
	 * @const         string
	 */
	const WEEKDAY_FORMAT_FULL = 'full';

	/**
	 * 3-character weekday format: "Mon", "Tue", ...
	 * @const         string
	 */
	const WEEKDAY_FORMAT_3    = '3';

	/**
	 * 2-character weekday format: "Mo", "Tu", ...
	 * @const         string
	 */
	const WEEKDAY_FORMAT_2    = '2';

	/**
	 * Returns the weekday
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @todo by dv: Use Zend_Date to translate date strings.
	 * @param       int $timeStamp
	 * @param       string $format
	 * @param       string $languageName2 (DE|FR)
	 * @return      string
	 */
	public static function getNameOfWeekday($timeStamp, $format, $languageName2){
		$weekDayState = date("w", $timeStamp);

		switch ($languageName2){
			case 'DE':
				switch($weekDayState){
					case 0:
						$data = array('full' => 'Sonntag', '3' => 'Son', '2' => 'So');
						break;
					case 1:
						$data = array('full' => 'Montag', '3' => 'Mon', '2' => 'Mo');
						break;
					case 2:
						$data = array('full' => 'Dienstag', '3' => 'Die', '2' => 'Di');
						break;
					case 3:
						$data = array('full' => 'Mittwoch', '3' => 'Mit', '2' => 'Mi');
						break;
					case 4:
						$data = array('full' => 'Donnerstag', '3' => 'Don', '2' => 'Do');
						break;
					case 5:
						$data = array('full' => 'Freitag', '3' => 'Fri', '2' => 'Fr');
						break;
					case 6:
						$data = array('full' => 'Samstag', '3' => 'Sam', '2' => 'Sa');
						break;
				}
				break;
			case 'EN':
				switch($weekDayState){
					case 0:
						$data = array('full' => 'Sunday', '3' => 'Sun', '2' => 'Su');
						break;
					case 1:
						$data = array('full' => 'Montay', '3' => 'Mon', '2' => 'Mo');
						break;
					case 2:
						$data = array('full' => 'Tuesday', '3' => 'Tue', '2' => 'Tu');
						break;
					case 3:
						$data = array('full' => 'Wednesday', '3' => 'Wed', '2' => 'We');
						break;
					case 4:
						$data = array('full' => 'Thursday', '3' => 'Thu', '2' => 'Th');
						break;
					case 5:
						$data = array('full' => 'Friday', '3' => 'Fri', '2' => 'Fr');
						break;
					case 6:
						$data = array('full' => 'Saturday', '3' => 'Sat', '2' => 'Sa');
						break;
				}
				break;
		}

		return $data[$format];
	}

	/**
	 * Returns the begin and end timestamp of a day
	 *
	 * Pass any English textual datetime description
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $dateString
	 * @return      array('start' => '...', 'end' => '...')
	 * @see         http://ch2.php.net/manual/en/function.strtotime.php
	 */
	public static function getStartEndTSOfDay($dateString = '-1 day'){
		// get the date of yesterday in a 'friendly' version
		$date = date("Y-m-d", strtotime($dateString)) . "\n";
		$parts = explode('-', $date);

		// get the unix time stamp of the begin of this day
		$startOfDay = mktime(0, 0, 0, intval($parts[1]), intval($parts[2]), intval($parts[0]));

		// get the unix time stamp of the end of this day
		$endOfDay = mktime(23, 59, 59, intval($parts[1]), intval($parts[2]), intval($parts[0]));

		return array('start' => $startOfDay, 'end' => $endOfDay);
	}

	/**
	 * Returns start and end timestamp of a specific month
	 *
	 * @author      Nicolas Christener <nc@adfinis.com>
	 * @param       int $month 1-12
	 * @param       int $year 4 digits ie. 2009
	 * @return      array
	 */
	public static function getStartEndTSOfMonth($month, $year){
		/**
		 * Calculate start and end of the given month.
		 * The last day of any given month can be expressed as the "0" day of
		 * the next month
		 */
		$start = mktime(0, 0, 0, $month, 1, $year);
		$end = mktime(23, 59, 59, $month+1, 0, $year);

		return array('start' => $start, 'end' => $end);
	}

	/**
	 * Returns name of a month by number of month (utf-8)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       int $month
	 * @param       string $format
	 * @param       string $languageName2 (DE|FR)
	 * @return      string
	 */
	public static function getNameOfMonth($month, $format, $languageName2){
		$month = intval($month);

		switch ($languageName2){
			case 'DE':
				switch($month) {
					case 1:
						$data = array('full' => 'Januar', 'short' => 'Jan');
						break;
					case 2:
						$data = array('full' => 'Februar', 'short' => 'Feb');
						break;
					case 3:
						$data = array('full' => 'März', 'short' => 'Mar');
						break;
					case 4:
						$data = array('full' => 'April', 'short' => 'Apr');
						break;
					case 5:
						$data = array('full' => 'Mai', 'short' => 'Mai');
						break;
					case 6:
						$data = array('full' => 'Juni', 'short' => 'Jun');
						break;
					case 7:
						$data = array('full' => 'Juli', 'short' => 'Jul');
						break;
					case 8:
						$data = array('full' => 'August', 'short' => 'Aug');
						break;
					case 9:
						$data = array('full' => 'September', 'short' => 'Sept');
						break;
					case 10:
						$data = array('full' => 'Oktober', 'short' => 'Okt');
						break;
					case 11:
						$data = array('full' => 'November', 'short' => 'Nov');
						break;
					case 12:
						$data = array('full' => 'Dezember', 'short' => 'Dez');
						break;
				}
				break;
			case 'EN':
				switch($month) {
					case 1:
						$data = array('full' => 'January', 'short' => 'Jan');
						break;
					case 2:
						$data = array('full' => 'February', 'short' => 'Feb');
						break;
					case 3:
						$data = array('full' => 'March', 'short' => 'Mar');
						break;
					case 4:
						$data = array('full' => 'April', 'short' => 'Apr');
						break;
					case 5:
						$data = array('full' => 'May', 'short' => 'May');
						break;
					case 6:
						$data = array('full' => 'June', 'short' => 'Jun');
						break;
					case 7:
						$data = array('full' => 'July', 'short' => 'Jul');
						break;
					case 8:
						$data = array('full' => 'August', 'short' => 'Aug');
						break;
					case 9:
						$data = array('full' => 'September', 'short' => 'Sept');
						break;
					case 10:
						$data = array('full' => 'October', 'short' => 'Oct');
						break;
					case 11:
						$data = array('full' => 'November', 'short' => 'Nov');
						break;
					case 12:
						$data = array('full' => 'December', 'short' => 'Dec');
						break;
				}
				break;
		}

		return $data[$format];
	}

	/**
	 * Returns all months (utf-8)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @todo by dv: Use Zend_Date to translate date strings.
	 * @param       string $format
	 * @param       string $languageName2 (DE|FR)
	 * @return      array
	 */
	public static function getMonths($format, $languageName2){
		switch ($languageName2){
			case 'DE':
				if($format == self::MONTH_FORMAT_FULL){
					return array(
						1 => 'Januar',
						2 => 'Februar',
						3 => 'März',
						4 => 'April',
						5 => 'Mai',
						6 => 'Juni',
						7 => 'Juli',
						8 => 'August',
						9 => 'September',
						10 => 'Oktober',
						11 => 'November',
						12 => 'Dezember'
					);
				} else {
					return array(
						1 => 'Jan',
						2 => 'Feb',
						3 => 'Mar',
						4 => 'Apr',
						5 => 'Mai',
						6 => 'Jun',
						7 => 'Jul',
						8 => 'Aug',
						9 => 'Sept',
						10 => 'Okt',
						11 => 'Nov',
						12 => 'Dez'
					);
				}
				break;
			case 'EN':
				if($format == self::MONTH_FORMAT_FULL){
					return array(
						1 => 'January',
						2 => 'February',
						3 => 'March',
						4 => 'April',
						5 => 'May',
						6 => 'June',
						7 => 'July',
						8 => 'August',
						9 => 'September',
						10 => 'October',
						11 => 'November',
						12 => 'December'
					);
				} else {
					return array(
						1 => 'Jan',
						2 => 'Feb',
						3 => 'Mar',
						4 => 'Apr',
						5 => 'May',
						6 => 'Jun',
						7 => 'Jul',
						8 => 'Aug',
						9 => 'Sept',
						10 => 'Oct',
						11 => 'Nov',
						12 => 'Dec'
					);
				}
				break;
		}
	}

	/**
	 * Lowers a timestamp to the given minutes and allows to format it
	 * like date() ($minutesToRound = 5: 19:37 -> 19:35)
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       int $minutesToRound = 1, 0 and 60 lower to an hour
	 * @param       string $format = 'Y-m-d H:i:s', see date()
	 * @param       int $timestamp = null
	 * @return      string
	 */
	public static function lowerDateToMinutes($minutesToRound = 1, $format = 'Y-m-d H:i:s', $timestamp = null){
		$date = ($timestamp) ? getdate($timestamp) : getdate();
		$time = mktime(
			$date['hours'],
			floor($date['minutes']/$minutesToRound)*$minutesToRound,
			0,
			$date['mon'],
			$date['mday'],
			$date['year']
		);

		return date($format, $time);
	}
}
