<?php

namespace org\adfinis\framework\utils;

/**
 * Download utilities
 *
 * Provides methods for sending out files as downloads. Takes
 * care of IE weirdness
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class DownloadHelper {

	/**
	 * Create a download from the given file.
	 *
	 * This tries to detect the mimetype using Download::getMimeType().
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $localFileName Local file that should be sent
	 * @param       string $mimeType Mime type to send as (optional; if not
	 *              given, tries to detect it and falls back to octet-stream on error).
	 * @param       string $saveAsName Use this name on client's side
	 *              (optional, defaults to basename of $localFileName).
	 * @return      void
	 */
	public static function makeDownload($localFileName, $mimeType = null, $saveAsName = null) {
		if (is_null($saveAsName)) $saveAsName = basename($localFileName);
		if (is_null($mimeType))   $mimeType   = self::getMimeType($saveAsName);

		// ok now we have mime type and filename. let's send it out.
		self::sendHeaders($mimeType, $saveAsName);
		self::sendFile($localFileName);
	}

	/**
	 * Create a download from the given string
	 *
	 * This tries to detect the mimetype using Download::getMimeType().
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $saveAsName Use this name on client's side.
	 * @param       string $dataString string that contains the data to be "downloaded"
	 * @param       string $mimeType Mime type to send as (optional; if not given, tries
	 *              to detect it from $saveAsName and falls back to octet-stream on error).
	 * @return      void
	 */
	public static function makeStringDownload($saveAsName, $dataString, $mimeType = null) {
		if (is_null($mimeType))   $mimeType   = self::getMimeType($saveAsName);

		self::sendHeaders($mimeType, $saveAsName);
		header(sprintf('Content-Length: %d', strlen($dataString)));
		echo $dataString;
	}

	/**
	 * Sends the headers neccessary for a "save as" download
	 *
	 * Takes care of all the workarounds so it works in all browsers, including IE
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $mimeType
	 * @param       string $fileName
	 * @return      void
	 */
	private static function sendHeaders($mimeType, $fileName) {

		// Make sure Apache and PHP do not try to compress it additionally.. especially
		// since we're sending out a ZIP file
		apache_setenv('no-gzip', 1);
		ini_set('zlib.output_compression', 'off');
		header('200 HTTP/1.0 OK');

		// This! IE is being a bitch without this line.
		header('Pragma: public');

		// Make sure it's not getting cached.
		header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
		header(sprintf('Last-Modified: %s GMT', gmdate('D, d M Y H:i:s')));

		// another hint that only IE really needs
		header('Content-Transfer-Encoding: binary');

		// Rest is fairly straightforward
		header(sprintf('Content-Type: %s', $mimeType));
		header(sprintf('Content-Disposition: attachment; filename="%s"', $fileName));
	}

	/**
	 * Sends the given file as a download to the browser
	 *
	 * Note that you need to call sendHeaders() first, or use
	 * makeDownload(), which takes care of everything.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $localFileName Local file that should be sent
	 * @return      void
	 */
	private static function sendFile($localFileName) {
		header(sprintf('Content-Length: %d', filesize($localFileName)));
		readfile($localFileName);
	}

	/**
	 * Tries to guess the mime type of the given filename, based on extension
	 *
	 * Tests the extension of the file against a list of known types. If not known, falls back to 'application/octet-stream'
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @todo by dv: Extend this list to more known file types
	 * @param       string $fileName
	 * @return      string
	 */
	public static function getMimeType($fileName) {
		$ext = substr($fileName, strrchr($fileName, '.'));

		switch($ext) {
			case '.zip':   return 'application/zip';
			case '.pdf':   return 'application/pdf';
			case '.txt':   return 'text/plain';
			case '.html':  return 'text/html';
			case '.xml':   return 'text/xml';
			case '.xhtml': return 'text/xhtml';
			default:       return 'application/octet-stream';
		}
	}
}

