<?php

namespace org\adfinis\framework\utils;

use org\adfinis\framework\utils\Shell;

/**
 * Downloader - helps you download files, get progress info etc.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class Downloader {

	/**
	 * Filename of the target file.
	 *
	 * @var           string $targetFile
	 */
	private $targetFile;

	/**
	 * URL to be fetched.
	 *
	 * @var           string $url
	 */
	private $url;

	/**
	 * Command to run to download. Expected to take wget parameters and produce similar output
	 *
	 * @var           string $downloadCommand
	 */
	private $downloadCommand = 'wget';

	/**
	 * Constructor. Initializes a download, then runs in the background.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        Exception if the target file could not be opened
	 * @param         string $url
	 * @param         string $targetFile
	 * @param         string $targetHost Hostname if the file should be downloaded to a remote server. You may use the "user@host" syntax.
	 * @param         string $targetPort SSH port to be used together with $targetHost. Defaults to 22.
	 */
	public function __construct($url, $targetFile, $targetHost = null, $targetPort = 22) {

		if($targetHost) {
			$this->downloadCommand = sprintf(
				'ssh -p%d %s %s',
				$targetPort,
				$targetHost,
				'wget'
			);
		}

		// TODO: file writability check on remote host
		if(is_null($targetHost)) {
			$fileHandle = fopen($targetFile, 'wb');
			if(!$fileHandle) {
				throw new \Exception(tr('Unable to open target file'));
			}
			fclose($fileHandle);
		}
		$this->targetFile = $targetFile;
		$this->url        = $url;
	}

	/**
	 * Starts the transfer. If you pass a callback as parameter, it will be called to update progress.
	 *
	 * The progress callback is called with a percentage of how much's downloaded yet (value: 0..100)
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         callback $callback
	 * @returns       void
	 */
	public function start($callback = null) {
		$previousProgress = 0;
		Shell::execWithHandlers(
			sprintf(
				'%s %s -O %s',
				$this->downloadCommand,
				escapeshellarg($this->url),
				escapeshellarg($this->targetFile)
			),
			'',
			null,
			function($data) use(&$previousProgress, &$callback) {
				$match = array();
				if(preg_match('/(\d+)%/', $data, $match)) {
					$percent = (int)$match[1];
					if ($percent != $previousProgress) {
						if(!$callback) {
							return;
						}
						else if($callback instanceof \Closure) {
							$callback($percent);
						}
						else if(is_array($callback) || is_string($callback)) {
							call_user_func($callback, $percent);
						}
					}
					$previousProgress = $percent;
				}
			}
		);
	}
}
