<?php

namespace org\adfinis\framework\utils;

/**
 * Downloader - helps you download files, get progress info etc.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class DownloaderBare {

	/**
	 * File handle of the target file
	 *
	 * @var           resource $fileHandle
	 */
	private $fileHandle;

	/**
	 * Size estimate, gotten from the Content-Length header. May be inaccurate or wrong.
	 *
	 * @var           int $sizeEstimate
	 */
	private $sizeEstimate;

	/**
	 * URL to be fetched.
	 *
	 * @var           string $url
	 */
	private $url;

	/**
	 * Constructor. Initializes a download, then runs in the background.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        Exception if the target file could not be opened
	 * @param         string $url
	 * @param         string $targetFile
	 */
	public function __construct($url, $targetFile) {

		$this->fileHandle = fopen($targetFile, 'wb');
		if(!$this->fileHandle) {
			throw new \Exception(tr('Unable to open target file'));
		}

		try {
			$this->sizeEstimate = $this->getURLDownloadSize($url);
		}
		catch(\Exception $e) {
			$this->sizeEstimate = 0;
		}
	}

	/**
	 * Starts the transfer. If you pass a callback as parameter, it will be called to update progress.
	 *
	 * The progress callback is called with two values: the estimated download size, and the current download
	 * size, both in bytes. Note that the estimated value may be inaccurate or plain wrong.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         callback $callback
	 * @returns       void
	 */
	public function start($callback = null) {
		$fh = fopen($this->url, 'rb', $this->context);

		$bytesRead = 0;
		while(!feof($fh)) {
			fwrite($this->fileHandle, fread($fh, 8000));
			$bytesRead += 8000;

			#$callback($this->sizeEstimate, $bytesRead);
			$callback($this->sizeEstimate, ftell($fh));
		}
	}

	/**
	 * Returns the size reported by the server, for the given URL, in bytes.
	 *
	 * Note this information may not be accurate, or may even be plain wrong.
	 *
	 * Also note, the return value is explicitly NOT converted to an integer, as
	 * the remote file might be bigger than 2^31, which may mess up the number if
	 * you are on a 32bit machine.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @throws        InvalidArgumentException
	 * @param         string $url
	 * @returns       int
	 */
	public static function getURLDownloadSize($url) {
		$parts = parse_url($url);

		if(isset($parts['port'])) {
			$port = $parts['port'];
		}
		else {
			$port = 80;
		}
		if($parts['scheme'] != 'http') {
			throw new \InvalidArgumentException(tr('Scheme not supported'));
		}

		$sock = fsockopen($parts['host'], $port, $errno, $errstr, 3);
		if(!$sock) {
			throw new \Exception(
				sprintf(
					tr('Unable to connect to host: %s'),
					$errstr
				)
			);
		}
		stream_set_timeout($sock, 5);

		fwrite($sock, sprintf("GET %s HTTP/1.1\r\n", $parts['path']));
		fwrite($sock, sprintf("Host: %s\r\n",         $parts['host']));
		fwrite($sock, sprintf("Connection: close\r\n"               ));
		fwrite($sock, sprintf("\r\n"                                ));

		$data = fread($sock, 1024*20);
		fclose($sock);

		$matchresult = array();
		if (preg_match('/Content-Length:\s+(\d+)/', $data, $matchresult)) {
			return $matchresult[1];
		}
		return 0;
	}
}
