<?php

namespace org\adfinis\framework\utils;

/**
 * Helper functions for handling files & directories
 *
 * @author      David Vogt <dv@adfinis.com>
 * @todo by dv: Use specific exception classes.
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 **/
class FileHelper
{

	/**
	 * Sets user and group of file and directories recursively
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @throws      Exception
	 * @param       string $user
	 * @param       string $group
	 * @param       string $path
	 * @return      void
	 */
	public static function chownRecursive($user, $group, $path) {
		// Check if path exists
		if (!file_exists($path)) {
			throw new \Exception(sprintf(tr('Given path "%s" does not exist!'), $path));
		}

		if (is_dir($path)) {
			// Change user and group recursively
			foreach (scandir($path) as $item) {
				if ($item == '.' || $item == '..') {
					continue;
				}

				self::chownRecursive($user, $group, sprintf('%s/%s', $path, $item));
			}

			if (!chown($path, $user) || !chgrp($path, $group)) {
				throw new \Exception(sprintf(tr('chown failed by "%s"'), $path));
			}
		} elseif (is_file($path)) {
			// Change user and group
			if (!chown($path, $user) || !chgrp($path, $group)) {
				throw new \Exception(sprintf(tr('chown failed by "%s"'), $path));
			}
		}
	}

	/**
	 * Sets access rights of a file or a directory recursively
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @throws      \Exception
	 * @param       int $fileMod
	 * @param       int $dirMod
	 * @param       string $path
	 * @return      void
	 */
	public static function chmodRecursive($fileMod, $dirMod, $path) {
		// Check if path exists
		if (!file_exists($path)) {
			throw new \Exception(sprintf(tr('Given path "%s" does not exist!'), $path));
		}

		if (is_dir($path)) {
			// Change access rights recursively
			foreach (scandir($path) as $item) {
				if ($item == '.' || $item == '..') {
					continue;
				};

				self::chmodRecursive($fileMod, $dirMod, sprintf('%s/%s', $path, $item));
			}

			if (!chmod($path, $dirMod)) {
				throw new \Exception(sprintf(tr('chmod failed by "%s"'), $path));
			}
		} elseif (is_file($path)) {
			// Change access rights
			if (!chmod($path, $fileMod)) {
				throw new \Exception(sprintf(tr('chmod failed by "%s"'), $path));
			}
		}
	}


	/**
	 * Copies a directory recursively
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @throws      \Exception
	 * @param       string $source
	 * @param       string $destination
	 * @return      void
	 */
	public static function copyDirectory($source, $destination) {
		// Check if source is a directory
		if (!is_dir($source)) {
			throw new \Exception(sprintf(tr('%s is no directory!')));
		}

		// Copy directroy recursively
		if (exec(sprintf('cp -r %s %s', $source, $destination), $output, $return)) {
			throw new \Exception(sprintf(tr('Could not copy directory: %s'), $source));
		}
	}


	/**
	 * Recursively deletes the given directory
	 *
	 * Taken from http://php.net/manual/de/function.rmdir.php
	 * (comment by asn at asn24 dot dk)
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $dir
	 * @return      boolean
	 */
	public static function deleteDirectory($dir) {
		#print "----- deleting $dir...\n";
		if (!is_link($dir) && !file_exists($dir)) {
			return true;
		}

		// WARNING: is_dir($dir) returns true if $dir is a
		// symlink pointing to a directory!
		if (!is_dir($dir) || is_link($dir)) {
			return unlink($dir);
		}

		$dirs = scandir($dir);
		foreach ($dirs as $item) {
			if ($item == '.' || $item == '..') {
				continue;
			}

			if (!self::deleteDirectory($dir . "/" . $item)) {
				// try to give us rights to delete.. avoid some ugly corner cases
				chmod($dir . "/" . $item, 0777);

				if (!self::deleteDirectory($dir . "/" . $item)) {
					return false;
				}
			}
		}
		return rmdir($dir);
	}
}
