<?php

namespace org\adfinis\framework\utils;

use org\adfinis\framework\config\ConfManager;

/**
 * Mail helper - provides various methods for sending and handling email stuff.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class MailHelper {
	/**
	 * encodes an email address for later use in a smarty template
	 *
	 * email addresses are made unrecognizable to avoid spaming
	 * we created the smarty function 'encode_mail' which calls this method
	 * Use something like {encode_mail address="foo@bar.moo"} in a smarty
	 * template to use it.
	 *
	 * @author        Nicolas Christener <nc@adfinis.ch>
	 * @param         string $email the email-address
	 * @param         string $emailTitle the title of the e-mail
	 * @param         $emailParameter parameters for the encode
	 * @return        string base64 encoded string
	 */
	public static function encodeMail($email, $emailTitle = null, $emailParameter = null){
		$len = strlen($email);
		$encoded = '';
		for($i = 0; $i < $len; $i++){
			$encoded .= chr(ord($email[$i]) ^ 156);
		}

		$encodedMailScript = sprintf("<script>print_mail_link('%s','%s','%s');</script>",
			base64_encode($encoded),
			$emailTitle,
			$emailParameter
		);

		return $encodedMailScript;
	}

	/**
	 * Sends an email
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 *
	 * @todo by dv: Parameter for mail server selection (see globalConf.php)
	 *
	 * @throws      InvalidInputException
	 * @throws      InvalidArgumentException
	 *
	 * @param       array  $to            list of associative arrays with receiver information
	 *                                    array(
	 *                                        array('firstName'=>'David', 'lastName'=>'Vogt', 'address'=>'dv@adfinis.ch'),
	 *                                        ...
	 *                                    )
	 * @param       string $subject       utf8 encoded string
	 * @param       string $messagePlain  utf8 encoded string
	 * @param       string $messageHtml   utf8 encoded string
	 * @param       bool   $utf8          send message as UTF8? If set to false, utf8_decode() is called on $message.
	 * @param       int    $wordwrap      wrap lines after how many chars? Use 0 to disable wordwrap.
	 * @param       array  $attachment    array with all attachment file data that should be added to the mail. 
	 *                                    The inner array must contain 'filePath', 'mimeType' and 'fileName'
	 * @param       array  $bcc           list of associative arrays with receiver information
	 *                                    analogous to $to
	 *
	 * @return      bool
	 */
	public static function sendMail(
		array $to,
		$subject,
		$messagePlain = '',
		$messageHtml = '',
		$utf8 = false,
		$wordwrap = 75,
		array $attachment = array(),
		array $bcc = array(),
		$mailFromMail = null,
		$mailFromName = null
	) {
		if(intval($wordwrap) > 0){
			$messagePlain = wordwrap($messagePlain, 75, "\n", false);
		}

		if($utf8 === false){
			$subject = utf8_decode($subject);
			$messagePlain = utf8_decode($messagePlain);
			$messageHtml = utf8_decode($messageHtml);
		}

		// Default constructor uses 'iso-8859-1'
		if($utf8 === true){
			$mail = new \Zend_Mail('utf-8');
		}else{
			$mail = new \Zend_Mail();
		}

		// send over smtp
		$zendMailTransport = new \Zend_Mail_Transport_Smtp(ConfManager::getVar('af', 'SMTP', 'default', 'hostname'));
		$mail->setDefaultTransport($zendMailTransport);

		/**
		 * add received header, some scripts call this function from the
		 * CLI where $_SERVER values are not set
		 */
		if(!CLIChecker::isCli()){
			$received = sprintf('from %s ([%s])',
				gethostbyaddr($_SERVER['REMOTE_ADDR']),
				$_SERVER['REMOTE_ADDR']
			);
			$mail->addHeader('Received', $received);

			$project = ConfManager::getVar('af', 'projectName');
			$received = sprintf(' by %s (%s) with HTTP',
				$_SERVER['SERVER_NAME'],
				$project
			);
			$mail->addHeader('Received', $received, true);

			// get first mail address to use as primary address
			$primaryReceiver = reset($to);
			if($primaryReceiver['address']){
				$primaryAddress = $primaryReceiver['address'];
			}else{
				throw new InvalidInputException(tr('No e-mail address given'));
			}

			$received = sprintf(' for <%s> %s; ',
				utf8_decode($primaryAddress),
				date('r')
			);
			$mail->addHeader('Received', $received, true);
		}

		// some standard headers
		$mail->setMessageId();

		/**
		 * the following is a check for an unwanted behavior. A correct
		 * message-id looks like this and contains <> signs:
		 *   Message-Id: <1245831280.3831.2.camel@oipunk.loozer.local>
		 * We patched the Zend class to fix this and add a check here so we
		 * will not forget to add the fix after the first Zend Framework
		 * update.
		 *
		 * @todo by nc: seems fixed in 1.9.0 -> update Zend FW
		 */
		$headers = $mail->getHeaders();
		if(
			strpos($headers['Message-Id'][0], '<') === false ||
			strpos($headers['Message-Id'][0], '>') === false
		){
			throw new Exception(tr('
				Zend Framework class did not add the <> signs to the message-id.
				This is needed according to RFC 2822. We once patched this, but
				it looks like this patch was reverted, maybe due to an
				update of the Zend Framework? You should fix this and add
				the <> signs again.
				'));
		}

		if($messagePlain != '' && $messageHtml != '') {
			$mail->setBodyText(strip_tags($messagePlain));
			$mail->setBodyHtml($messageHtml);
		} elseif($messagePlain != '') {
			$mail->setBodyText(strip_tags($messagePlain));
		} elseif($messageHtml != '') {
			$mail->setBodyHtml($messageHtml);
		}

		if (!empty($mailFromMail) && !empty($mailFromName)) {
			$mail->setFrom(
				$mailFromMail,
				$mailFromName
			);
		} else {
			$mail->setFrom(
				ConfManager::getVar('af', 'mail', 'fromSystem', 'fromMail'),
				ConfManager::getVar('af', 'mail', 'fromSystem', 'fromName')
			);
		}

		// Set primary recipient(s)
		foreach ($to as $receiver){
			// check if firstname and lastname exist
			$fullName = array();
			if(trim($receiver['firstName']) !== ''){
				$fullName[] = $receiver['firstName'];
			}

			if(trim($receiver['lastName']) !== ''){
				$fullName[] = $receiver['lastName'];
			}
			$mail->addTo(
				utf8_decode($receiver['address']),
				utf8_decode(implode(' ', $fullName))
			);
		}

		// Set BCC recipient(s)
		foreach ($bcc as $bccRecipient) {
			$mail->addBcc(
				utf8_decode($bccRecipient['address'])
			);
		}

		// add attachment/s
		if (!empty($attachment)) {
			foreach ($attachment as $attachmentFile) {
				if (
					!isset($attachmentFile['filePath']) ||
					!isset($attachmentFile['mimeType']) ||
					!isset($attachmentFile['fileName'])
				) {
					throw new InvalidArgumentException(tr('Attachment array must have a "filePath" and "mimeType"!'));
				}

				if (
					!is_file($attachmentFile['filePath']) ||
					!is_readable($attachmentFile['filePath'])
				) {
					throw new InvalidInputException(
						sprintf(
							tr("Attachment isn't a file or isn't readable! -> %s"),
							$attachmentFile['filePath']
						)
					);
				}
				$fileSrc = file_get_contents($attachmentFile['filePath']);

				$mail->createAttachment(
					$fileSrc,
					$attachmentFile['mimeType'],
					Zend_Mime::DISPOSITION_ATTACHMENT,
					Zend_Mime::ENCODING_BASE64,
					$attachmentFile['fileName']
				);
			}
		}

		$mail->setSubject($subject);
		$mail->send();

		return true;
	}
}

