<?php

namespace org\adfinis\framework\utils;

use org\adfinis\framework\config\ConfManager;

/**
 * Helper class for picture manipulation.
 * 
 * This provides methods for converting, validating, resizing (crop and zoom)
 * and rotating pictures.
 *
 * @author      Matthias Blaser <mb@adfinis.ch>
 * @author      Nicolas Christener <nc@adfinis.com>
 * @todo by dv: use specific exceptions
 * @todo by dv: Don't hide errors/warnings from function calls (remove the @ in function calls)
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class PictureHelper {

	/**
	 * Checks if an picture can be read by php
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @throws      Exception
	 * @param       string $src filesystem path of image to be checked
	 * @param       bool $returnId return image identifier to be used for gd functions
	 * @return      true
	 */
	public static function validateFormat($src, $returnId = false){
		if(!file_exists($src)){
			throw new Exception(tr('Image file was not found'));
		}

		// try to get current dimensions
		$imageDim = @getimagesize($src);
		if(!$imageDim){
			throw new Exception(sprintf(tr('Unknown image format, could not figure out size for %s'), $src));
		}

		// try to read the file with the appropriate function
		$func = self::getGDFunction($imageDim['mime']);
		$srcId = @$func($src);
		if(!$srcId){
			throw new Exception(tr('Image could not be read'));
		}

		// all checkes done destroy image
		imagedestroy($srcId);
		return true;
	}

	/**
	 * Resizes an picture to a defined heigth/width
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @throws      Exception
	 * @param       string $src source picture
	 * @param       string $out output picture
	 * @param       int $height height of output picture
	 * @param       int $width width of output picture
	 * @param       string $outputFormat desired picture format of output picture
	 * @param       int $quality compression quality
	 * @param       bool $crop true if the output picture should be cropped, false if the aspect ratio should be maintained
	 * @param       bool $zoom if true, the output picture will be zoomed when desired size is bigger, else it will be left untouched
	 * @return      bool
	 */
	public static function resizePicture($src, $out, $height, $width, $outputFormat = 'jpg', $quality = 85, $crop = false, $zoom = false){
		if(!file_exists($src)){
			throw new Exception(sprintf(tr('Image file %s was not found'), $src));
		}

		// source dimensions
		$dimSrc = getimagesize($src);
		if(!$dimSrc){
			throw new Exception(sprintf(tr('Unknown image format, could not figure out size for %s'), $src));
		}

		$widthSrc = $dimSrc[0];
		$heightSrc = $dimSrc[1];

		// (try to) read image
		$func = self::getGDFunction($dimSrc['mime']);
		$idSrc = @$func($src);
		if(!$idSrc){
			throw new Exception(sprintf(tr('Image %s could not be read'), $src));
		}

		// check for zoom
		if($widthSrc < $width && $heightSrc < $height && !$zoom){
			// desired thumbnail is bigger than source, just copy
			return @copy($src, $out);
		}

		// aspect ratio
		$ratioSrc = $widthSrc / $heightSrc;
		$ratioOut = $width / $height;

		if($ratioSrc == $ratioOut){
			// no need to crop even if requested, ratio is just ok
			$crop = false;
			$heightOut = $height;
			$widthOut = $width;
		} else {
			if($crop){
				// check where should be cropt
				if($ratioSrc > $ratioOut){
					// use destination height, but greater width to keep ratio temporary
					$heightOut = $height;
					$widthOut = ceil($widthSrc * ($height / $heightSrc));
				}

				if($ratioSrc < $ratioOut){
					// use destination width, but greater height to keep ratio temporary
					$heightOut = ceil($heightSrc * ($width / $widthSrc));
					$widthOut = $width;
				}
			} else {
				// no crop, so defined width/height are max values, don't go greater than them
				if($ratioSrc > $ratioOut){
					// use destination width, but smaller height to keep ratio
					$heightOut = ceil($heightSrc * ($width / $widthSrc));
					$widthOut = $width;
				}

				if($ratioSrc < $ratioOut){
					// use destination height, but smaller width to keep ratio
					$heightOut = $height;
					$widthOut = ceil($widthSrc * ($height / $heightSrc));
				}
			}
		}
		$idOut = ImageCreateTrueColor($widthOut, $heightOut);
		if(!$idOut){
			throw new Exception(tr('Image could not be read, type unknown'));
		}
		$red = imagecolorallocate($idOut, 255, 255, 255);
		imagefill($idOut, 0, 0, $red);

		if($outputFormat == 'gif' || $outputFormat == 'png'){
			imagecolortransparent($idOut, $red);
		}

		$ret = imagecopyresampled($idOut, $idSrc, 0, 0, 0, 0, $widthOut, $heightOut, $widthSrc, $heightSrc);

		// crop?
		if($crop){
			$idTmp = imagecreatetruecolor($width, $height);

			if($ratioSrc > $ratioOut){
				// crop right and left
				$src_x = ceil(($widthOut - $width) / 2);
				$ret = imagecopy($idTmp, $idOut, 0, 0, $src_x, 0, $width, $height);
			} else {
				// crop top and bottom
				$src_y = ceil(($heightOut - $height) / 2);
				$ret = imagecopy($idTmp, $idOut, 0, 0, 0, $src_y, $width, $height);
			}

			$idOut = $idTmp;
		}

		// store new image
		switch($outputFormat){
			case 'gif':
				$ret = imagegif($idOut, $out);
				break;
			case 'png':
				$ret = imagepng($idOut, $out);
				break;
			default:
				 $ret = imagejpeg($idOut, $out, $quality);
				break;
		}

		 // maybe gif or png aren't supported
		if(!$ret){
			 $ret = imagejpeg($idOut, $out, $quality);
		}

		if(!$ret){
			throw new Exception(tr('Image could not be created, unknown error'));
		}

		return true;
	}


	/**
	 * rotates a picture, needs testing and QA
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @throws      Exception
	 * @param       string $src
	 * @param       string $out
	 * @param       int $degrees
	 * @param       string $fillColor (RGB #ff00ff)
	 * @return      bool
	 */
	static public function rotatePicture($src, $out, $degrees, $fillColor){
		// check up degrees
		if(!is_numeric($degrees)){
			return false;
		}

		if(!file_exists($src)){
			throw new Exception(sprintf(tr('Image file %s was not found'), $src));
		}

		// source dimensions
		$dimSrc = getimagesize($src);
		if(!$dimSrc){
			throw new Exception(sprintf(tr('Unknown image format, could not figure out size for %s'), $src));
		}

		// check and calculate the fill color (i.e. when rotating by 45°)
		$fillColor[0] == '#' ? $fillColor = substr($fillColor, 1, 6) : $fillColor = $fillColor;
		if (!eregi('^[0-9a-f]{6}$', $fillColor)) throw new Exception(tr('unknown color string'));
		$fillColor = hexdec('0x'.$fillColor);

		// Load
		$func = self::getGDFunction($dimSrc['mime']);
		$idSrc = @$func($src);

		// Rotate
		$rotate = imagerotate($idSrc, $degrees, $fillColor);

		// Output
		imagejpeg($rotate, $out);

		return true;
	}

	/**
	 * Decides which gd loading function should be used to read a picture with a specified mime type
	 *
	 * @author      Matthias Blaser <mb@adfinis.ch>
	 * @param       string $mime mime-type to be checked
	 * @return      string function which can be used to read the image
	 */
	static public function getGDFunction($mime){
		// check image format
		switch($mime){
			case 'image/jpeg':
			case 'image/pjpeg':
			case 'image/jpg':
				$func = 'ImageCreateFromJPEG';
				break;

			case 'image/png':
			case 'image/x-png':
				$func = 'ImageCreateFromPNG';
				break;

			case 'image/gif':
				$func = 'ImageCreateFromGIF';
				break;

			default:
				$func = 'ImageCreateFromGD2';
		}

		return $func;
	}
}
