<?php

namespace org\adfinis\framework\utils;

/**
 * Contains helper methods for calling shell commands.
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2010-2010
 * @license     BSD 3-Clause
 **/
class Shell {

	/**
	 * Helper function for doing simple shell calls.
	 *
	 * This helps with debugging by actually returning output etc, even if
	 * it was written to STDERR. It returns an associative array with the
	 * following keys: 'stdout', 'stderr', 'return' (returncode)
	 *
	 * The function was copied from a PHP documentation comment:
	 * http://www.php.net/manual/de/function.system.php
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $cmd
	 * @param       string $input (optional)
	 * @return      array
	 */
	public static function runCmd($cmd, $input='') {
		$proc = proc_open($cmd,
			array(
				0=>array('pipe', 'r'),
				1=>array('pipe', 'w'),
				2=>array('pipe', 'w')
			),
			$pipes
		);
		fwrite($pipes[0], $input);
		fclose($pipes[0]);

		$stdout = stream_get_contents($pipes[1]); fclose($pipes[1]);
		$stderr = stream_get_contents($pipes[2]); fclose($pipes[2]);

		$rtn    = proc_close($proc);

		return array(
			'stdout'=>$stdout,
			'stderr'=>$stderr,
			'return'=>$rtn
		);
	 }

	/**
	 * Executes the given command using SUDO.
	 *
	 * Takes a command line (assuming it is correctly escaped!),
	 * possibly with input, then runs it with sudo.

	 * @see Shell::runCmd() for how the data is returned.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $cmd
	 * @param       string $input
	 * @return      array
	 */
	public static function execSudo($cmd, $input = '') {
		$cmd = sprintf('/usr/bin/sudo %s', $cmd);

		return self::runCmd($cmd, $input);
	}

	/**
	 * Executes the given command, and calls the given callbacks when data arrives.
	 *
	 * Returns the exit status of the command.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $cmd
	 * @param         string $input (optional)
	 * @param         callback $handleSTDOUT (optional)
	 * @param         callback $handleSTDERR (optional)
	 * @returns       int
	 */
	public static function execWithHandlers($cmd, $input = '', $handleSTDOUT = null, $handleSTDERR = null) {

		$call = function($callback, $data) {
			if(!$callback) {
				return;
			}
			if(is_callable($callback)) {
				$callback($data);
			}
			else {
				call_user_func($callback, $data);
			}
		};

		$proc = proc_open($cmd,
			array(
				0=>array('pipe', 'r'),
				1=>array('pipe', 'w'),
				2=>array('pipe', 'w')
			),
			$pipes
		);
		fwrite($pipes[0], $input);
		fclose($pipes[0]);

		$streams = array(
			'stdout' => $pipes[1],
			'stderr' => $pipes[2]
		);

		while(sizeof($streams)) {
			$watch = $streams;
			// WTF, PHP, WTF??
			$null = null;
			if(stream_select($watch, $null, $null, $null)) {
				foreach($watch as $handle) {
					if (isset($streams['stdout']) && $handle === $streams['stdout']) {
						$call($handleSTDOUT, fread($handle, 4096));
						if(feof($handle)) {
							fclose($handle);
							unset($streams['stdout']);
						}
					}
					else if (isset($streams['stderr']) && $handle === $streams['stderr']) {
						$call($handleSTDERR, fread($handle, 4096));
						if(feof($handle)) {
							fclose($handle);
							unset($streams['stderr']);
						}
					}
				}
			}
		}

		$rtn    = proc_close($proc);

		return $rtn;
	}
	
}

