<?php

namespace org\adfinis\framework\utils;

/**
 * Helper functions for handling system users.
 *
 * NOTE: This relies on a few perl scripts, so be careful!
 *
 * @author      David Vogt <dv@adfinis.com>
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 **/
class SystemUserHelper {

	/**
	 * Returns an associative array containing various user information.
	 *
	 * The keys are: username, userid, groupid, exists, shell, home.
	 *
	 * The field "exists" contains 1 if the user exists or 0 otherwise.
	 *
	 * NOTE: This calls an external Perl utility, so the PHP configuration
	 * must allow this. All data is passed using file descriptors, so you
	 * don't have to worry about bad input.
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $username
	 * @return      array
	 */
	public static function userInfo($username) {
		$perl   = '/usr/bin/perl';
		$script = sprintf('%s/data/bin/userInfo.pl', _CONF_DOC_ROOT);

		$execResult = org\adfinis\utils\Shell::runCmd(
			sprintf('%s %s', $perl, $script),
			sprintf("%s\n",  $username) // this goes to stdin! (just sayin..)
		);

		$userinfo = trim($execResult['stdout']);

		// now that we have the userinfo string, de-compose it into an array.
		$info = array();
		foreach (str_split('|', $userinfo) as $pair) {
			list($key, $value) = str_split('=', $pair);
			$info[$key] = is_numeric($value) ? intval($value) : $value;
		}
		return $info;
	}

	/**
	 * Returns true if the user name exists in the system.
	 *
	 * NOTE: This calls SystemUsers::userInfo() in turn. Be sure to
	 * read the NOTE there, as an external utility and Perl is involved!
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $userName
	 * @return      boolean
	 */
	public static function userExists($userName) {
		$info = self::userInfo($userName);
		return $info['exists'];
	}

	/**
	 * Returns the id of the given system user.
	 *
	 * NOTE: This calls SystemUsers::userInfo() in turn. Be sure to
	 * read the NOTE there, as an external utility and Perl is involved!
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $userName
	 * @return      int
	 */
	public static function systemUserID($userName) {
		$info = self::userInfo($userName);
		return $info['userid'];
	}

	/**
	 * Returns the current home path of the given system user
	 *
	 * NOTE: This calls SystemUsers::userInfo() in turn. Be sure to
	 * read the NOTE there, as an external utility and Perl is involved!
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @throws      \Exception
	 * @param       string $userName
	 * @return      string
	 */
	public static function getUserHomedir($userName) {
		$info = self::userInfo($userName);
		return $info['home'];
	}

	/**
	 * REturns true if the given username belongs to a system user
	 *
	 * This means the user's id in the system is below 1000, which most unix
	 * systems define as the offset for "normal" users.
	 *
	 * NOTE: This calls SystemUsers::userInfo() in turn. Be sure to
	 * read the NOTE there, as an external utility and Perl is involved!
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @param       string $userName
	 * @return      boolean
	 */
	public static function isSystemUser($userName) {
		$info = self::userInfo($userName);
		return $info['userid'] < 1000;
	}

}
