<?php

namespace org\adfinis\framework\utils;

use org\adfinis\framework\config\ConfManager;
use \org\adfinis\framework\utils\ArrayTools;

/**
 * Class dealing with URL specific matters
 *
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 */
class URLHelper {

	/**
	 * Returns the current domainIdentifier
	 * NOTE: The value of the domainIdentifier must be the domain itself (in the config file)
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $domainIdentifier
	 * @return      string
	 */
	public static function getDomainIdentifier($domainIdentifier = 'current'){
		$identifier = ($domainIdentifier === 'current')
			? self::getCurrentDomainIdentifier()
			: $domainIdentifier
		;
		return $identifier;
	}

	/**
	 * Returns the identifier for the current domain.
	 *
	 * Tries to match the domains from the config to the one currently in use.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       string
	 */
	public static function getCurrentDomainIdentifier() {
		$configuredDomains = ConfManager::getVar('af', 'domain');

		$parts = explode('.', self::getHost());

		$parts = array_reverse($parts); // com, example, www

		$domains = array();

		$currentDomain = null;
		foreach($parts as $part) {
			$currentDomain = $currentDomain
				? sprintf('%s.%s', $part, $currentDomain)
				: $part
			;
			$domains[] = $currentDomain;
		}

		foreach($configuredDomains as $identifier => $domain) {
			if(in_array($domain, $domains)) {
				return $identifier;
			}
		}

		# No domain identifier could be found - assume default
		return 'default';
	}


	/**
	 * Returns the name of the current used host (in lowercase).
	 *
	 * Falls back to af:domain:default from the config if HTTP Host header wasn't sent.
	 * This should only happen with very, very old HTTP/1.0 clients or on IP-only hosts.
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @return      string
	 */
	public static function getHost(){
		if(!isset($_SERVER['HTTP_HOST'])){
			return ConfManager::getVar('af', 'domain', 'default');
		}

		return strtolower($_SERVER['HTTP_HOST']);
	}

	/**
	 * Returns the current used domain
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Denis Simonet <denis.simonet@adfinis-sygroup.ch>
	 * @return      string
	 */
	public static function getCurrentDomain(){
		$parts = explode('.', self::getHost());
		$domains = ArrayTools::flatten(ConfManager::getVar('af', 'domain'));

		do {
			if (in_array(($domain = implode('.', $parts)), $domains)) {
				return $domain;
			}
		} while (array_shift($parts));

		throw new \Exception(tr('Could not get current domain!'));
	}

	/**
	 * Returns the TLD based on the current used TLD (in lowercase)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @return      string
	 */
	public static function getTLD(){
		$parts = explode('.', self::getHost());

		return strtolower(array_pop($parts));
	}

	/**
	 * Returns a domain from the list defined in globalConfig.php
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $identifier (optional, defaults to 'default')
	 * @return      string
	 */
	public static function getDomain($identifier = 'current') {
		$identifier = self::getDomainIdentifier($identifier);

		return ConfManager::getVar('af', 'domain', $identifier);
	}

	/**
	 * Returns a FQDN from the list defined in the global config file
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $identifier (optional)
	 * @return      string
	 */
	public static function getFQDN($identifier = 'current'){
		$identifier = self::getDomainIdentifier($identifier);

		return ConfManager::getVar('af', 'domain', $identifier);
	}

	/**
	 * Returns the secure URL (https:// version) (in lowercase)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $identifier (optional)
	 * @return      string
	 */
	public static function getURLSecure($identifier = 'current'){
		$identifier = self::getDomainIdentifier($identifier);

		return ConfManager::getVar('af', 'URLSSL', $identifier);
	}

	/**
	 * Returns the insecure URL (http:// version) (in lowercase)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $identifier (optional)
	 * @return      string
	 */
	public static function getURLInsecure($identifier = 'current'){
		$identifier = self::getDomainIdentifier($identifier);

		return ConfManager::getVar('af', 'URLNoSSL', $identifier);
	}

	/**
	 * Returns the URL based on the actual security level (in lowercase)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $identifier (optional)
	 * @return      string
	 */
	public static function getURL($identifier = 'current'){
		$identifier = self::getDomainIdentifier($identifier);

		if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on'){
			return self::getURLSecure($identifier);
		}

		return self::getURLInsecure($identifier);
	}

	/**
	 * Returns the current URL with the used request string
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $identifier (optional)
	 * @return      string
	 */
	public static function getURLWithRequestString($identifier = 'current'){
		$identifier = self::getDomainIdentifier($identifier);

		return self::getURL($identifier).$_SERVER['REQUEST_URI'];
	}

	/**
	 * Check if page/script was requested over HTTPS
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      bool
	 */
	public static function currentURLIsSecure(){
		return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on');
	}

	/**
	 * Redirects the user to the given URL
	 *
	 * Set $permanent to true if the URL moved permanently. A 301 header
	 * (moved permanently) should be sent in this case instead of a
	 * 302 header (moved temporarly)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       string $URL
	 * @param       bool $permanent
	 */
	public static function redirectToURL($URL, $permanent = false){
		/* Remove leading slashes (if you redirect to /blubber, then adding
		 * the URL will result in "http://www.foo.bar//blubber")
		 */
		$URL = preg_replace('#^/*#','', $URL);

		// If URL does not contain "http" we add
		if(!preg_match('/^http/', $URL)){
			if(self::currentURLIsSecure()){
				$URL = sprintf(
					"%s/%s",
					self::getURLSecure(),
					$URL
				);
			}
			else {
				$URL = sprintf(
					"%s/%s",
					self::getURLInsecure(),
					$URL
				);
			}
		}

		/**
		 * If buffer is empty, ob_clean() sends a warning, and header()
		 * won't work. Therefore suppress warnings
		 */
		@ob_clean();
		header(sprintf('Location: %s', $URL));

		if($permanent){
			// send 301 http status code instead of default 302
			header('HTTP/1.1 301 Moved permanently');
		}

		exit();
	}

	/**
	 * Reloads the current page
	 *
	 * Can be used to reload the current page using secure/HTTPS instead of
	 * unsecure/HTTP
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @param       bool $secured
	 * @return      void
	 */
	public function reloadCurrentURL($secured = false){
		// Check if it's an internal redirect (mod_rewrite)
		if(isset($_SERVER['REDIRECT_URL'])){
			$URL = $_SERVER['REDIRECT_URL'];
		}
		else {
			$URL = $_SERVER['REQUEST_URI'];
		}

		if($secured){
			$URL = self::getURLSecure() . $URL;
		}
		else {
			$URL = self::getURLInsecure() . $URL;
		}

		self::redirectToURL($URL);
	}

	/**
	 * Returns the request URI
	 *
	 * @author      Damian Senn <ds@adfinis.com>
	 * @return      string
	 */
	public static function getRequestURI(){
		return $_SERVER['REQUEST_URI'];
	}
}
