<?php

namespace org\adfinis\framework\utils\i18n;

/**
 * Class dealing with internationalization (i18n)
 *
 * @author      Nicolas Christener <nc@adfinis.ch>
 * @todo by nc: Sync LANG_* constants with `language` table
 * @copyright   adfinis GmbH 2009-2010
 * @license     BSD 3-Clause
 */
class Translator {
	const LANG_DE_ID = 1;
	const LANG_EN_ID = 2;
	const LANG_FR_ID = 3;

	/**
	 * Returns translated string of a keyword
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @throws      Zend_Db_Exception
	 * @param       string $keyword
	 * @param       int $languageID
	 * @return      string
	 */
	public static function getString($keyword, $languageID){
		$query = sprintf("
			SELECT
				`value`
			FROM
				`i18nKey` as `iK`
			JOIN
				`i18nText` as `iT` ON
					(
						`iK`.`i18nKeyID` = `iT`.`i18nKeyID`
						AND
						`iT`.`languageID` = %d
					)
			WHERE
				`iK`.`keyword` = ?
			",
			$languageID
		);

		$preparedStatement = DbManager::prepare($query);
		$preparedStatement->execute(array($keyword));
		$result = $preparedStatement->fetch();

		/**
		 * If no translation is available, try with the fallback language, which
		 * is defined as 'default' language in globalConfig.php
		 */
		if(empty($result['value'])){
			$defaultLanguageID = ConfManager::getVar(
				'af', 'language', 'default', 'languageID'
			);
			if($languageID != $defaultLanguageID) {
				return self::getString($keyword, $defaultLanguageID);
			} else {
				return '';
			}
		}

		return $result['value'];
	}

	/**
	 * Saves translated string of a keyword (insert or update)
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Toni Rudolf <tr@adfinis.ch>
	 * @throws      InvalidArgumentException
	 * @throws      Zend_Db_Exception
	 * @param       string $keyword
	 * @param       int $languageID
	 * @param       string $string
	 * @return      void
	 */
	public static function insertString($keyword, $languageID, $string){
		if (!self::isValidKeyword($keyword)) {
			throw new InvalidArgumentException(tr('Keyword does not match the keyword specifications!'));
		}

		$query = "
			SELECT
				`i18nKeyID`
			FROM
				`i18nKey`
			WHERE
				`keyword` = ?
		";

		$preparedStatement = DbManager::prepare($query);
		$preparedStatement->execute(array($keyword));
		$result = $preparedStatement->fetch();

		// If this keyword does not exist, add it
		if($result === false){
			$query = "
				INSERT INTO `i18nKey`
					(`keyword`)
				VALUES
					(?)
			";
			$preparedStatement = DbManager::prepare($query);
			$preparedStatement->execute(array($keyword));
			$i18nKeyID = DbManager::lastInsertId();
		} else {
			$i18nKeyID = $result['i18nKeyID'];
		}

		$query = sprintf("
			REPLACE INTO
				`i18nText`
			SET
				`value` = ?,
				`languageID` = %d,
				`i18nKeyID` = %d
			",
			$languageID,
			$i18nKeyID
		);

		$preparedStatement = DbManager::prepare($query);
		$preparedStatement->execute(array($string));
	}

	/**
	 * Returns the languageID based on a languageCode2 (for example 'de')
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @throws      Zend_Db_Exception
	 * @param       string $languageCode2
	 * @return      int|bool
	 */
	public static function getLanguageID($languageCode2){
		// value check
		if(!self::isValidLanguageCode2($languageCode2)){
			throw new InvalidArgumentException(tr('Unsupported languageCode2 passed'));
		}

		$query = sprintf("
			SELECT
				`languageID`
			FROM
				`language`
			WHERE
				`code2` = '%s'
			",
			$languageCode2
		);
		$result = DbManager::fetchRow($query);

		if(is_array($result)){
			return intval($result['languageID']);
		}

		return false;
	}

	/**
	 * Returns the languageCode2 (for example 'de') based on a languageID
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @throws      Zend_Db_Exception
	 * @param       int $languageID
	 * @return      string|bool
	 */
	public static function getLanguageCode2($languageID){
		$query = sprintf("
			SELECT
				`code2`
			FROM
				`language`
			WHERE
				`languageID` = %d
			",
			$languageID
		);
		$result = DbManager::fetchRow($query);

		if(is_array($result)){
			return $result['code2'];
		}

		return false;
	}

	/**
	 * checks if the languageID exists
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @throws      Zend_Db_Exception
	 * @param       int $languageID
	 * @return      bool
	 */
	public static function isValidLanguageID($languageID) {
		$query = "
			SELECT
				COUNT(*) AS `isValid`
			FROM
				`language`
			WHERE
				`languageID` = ?
		";
		$preparedStatment = DbManager::prepare($query);
		$preparedStatment->execute(array($languageID));
		$isValidLanguageID = $preparedStatment->fetch();

		if ($isValidLanguageID['isValid'] === false) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * checks if the languageCode2 exists
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @throws      Zend_Db_Exception
	 * @param       string $languageCode2
	 * @return      bool
	 */
	public static function isValidLanguageCode2($languageCode2) {
		$query = "
			SELECT
				COUNT(*) AS `isValid`
			FROM
				`language`
			WHERE
				`code2` = ?
		";
		$preparedStatment = DbManager::prepare($query);
		$preparedStatment->execute(array($languageCode2));
		$isValidLanguageCode2 = $preparedStatment->fetch();

		if ($isValidLanguageCode2['isValid'] === false) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * checks if the keyword is a valid one
	 *
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @author      Damian Senn <ds@adfinis.com>
	 * @param       string $keyword
	 * @return      bool
	 */
	private static function isValidKeyword($keyword) {
		$keyword = explode('.', $keyword);

		$partOne = array(
			'generic',
			ConfManager::getVar('af', 'projectName')
		);

		return in_array($keyword[0], $partOne);
	}

	/**
	 * Returns the languageID of the session of the current user
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @return      int
	 */
	public static function getLanguageIDCurrentSession(){
		$languageID = intval(SessionManager::getInstance()->languageID);

		// if result is unexpected, return default language
		if(!self::isValidLanguageID($languageID)){
			$languageID = self::LANG_DE_ID;
		}

		return $languageID;
	}

	/**
	 * Sets the languageID in the session of the current user
	 *
	 * @author      Nicolas Christener <nc@adfinis.ch>
	 * @author      Adrian Wittwer <aw@adfinis.com>
	 * @return      void
	 */
	public static function setLanguageIDCurrentSession($languageID){
		if(!self::isValidLanguageID($languageID)){
			throw new InvalidArgumentException('Unsupported languagID');
		}
		SessionManager::getInstance()->languageID = $languageID;
	}
}
