<?php

namespace org\adfinis\framework\validate;

/**
 * Abstract class for a model validator.
 *
 * This defines the behaviour of a validator that can check a given Entity object.
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
abstract class ModelValidator {

	/**
	 * Validation state for the whole model object.
	 *
	 * @var           array $modelValidity
	 */
	protected $modelValidity;

	/**
	 * Stores the model entity that we're going to validate.
	 *
	 * @var           Entity $model
	 */
	protected $model;

	/**
	 * Constructor. Creates a new validator from the given model entity.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         Entity $model
	 */
	public function __construct($model) {
		$this->model = $model;
	}

	/**
	 * Validates the given field of the entity.
	 *
	 * This validation should only validate the field, without cross-checking
	 * other fields. This way, we can use this method to do AJAX validation.
	 *
	 * If the field is not valid, this method should store
	 * the errors using the following call:
	 *
	 * @code
	 *     $this->setIsInvalid($fieldName, $message);
	 * @endcode
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $field
	 * @returns       void
	 */
	abstract public function validateField($field);

	/**
	 * Validates the complete entity.
	 *
	 * If the model is not valid, this should store the errors using
	 * the following calls:
	 *
	 * @code
	 *     $this->setIsInvalid($fieldName, $message);
	 * @endcode
	 *
	 * Note: You can (and should) use the validateField() method so you don't
	 * have to write your validations twice.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	abstract public function validateEntity();

	/**
	 * Returns true if the model is completely valid.
	 *
	 * This calls the getValidity() method and returns true if the object is valid.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       boolean
	 */
	public function isValid() {
		$this->validateEntity();
		$hasErrors = (bool)sizeof($this->modelValidity);

		return (!$hasErrors);
	}

	/**
	 * Returns the validation messages as an array.
	 *
	 * The array's keys are the field names of the invalid fields,
	 * the values are the messages to be displayed.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       array
	 */
	public function getMessages() {
		return $this->modelValidity;
	}

	/**
	 * Tells the validator about an invalid field.
	 *
	 * Note: you can call this multiple times per field. The
	 * messages will be concatenated using a space character.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $fieldName
	 * @returns       void
	 */
	protected function setIsInvalid($fieldName, $message) {
		if(isset($this->modelValidity[$fieldName])) {
			$this->modelValidity[$fieldName] .= " " . $message;
		}
		else {
			$this->modelValidity[$fieldName] = $message;
		}
	}

	/**
	 * Returns the stored validation messages for the given field.
	 *
	 * If no messages are stored, returns false.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $fieldName
	 * @returns       array|bool
	 */
	public function getMessagesForField($fieldName) {
		if (isset($this->modelValidity[$fieldName])) {
			return $this->modelValidity[$fieldName];
		}
		return false;
	}

	/**
	 * Resets the stored validity information.
	 *
	 * Call this from your model when data changes.
	 * Call this if you want to re-validate your model.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function reset() {
		$this->modelValidity = array();
	}

	/**
	 * Validate enum
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @returns       void
	 */
	protected function validateEnum($field, $values, $message = 'Invalid %s "%s", should be one of: %s') {
		if (!in_array($this->model->$field, $values)) {
			$this->setIsInvalid($field, sprintf($message,
				$field, $this->model->$field, join(', ', $values)));
		}
	}
}
