<?php

namespace org\adfinis\framework\validate;

use org\adfinis\framework\db\DbManager;
use org\adfinis\framework\exceptions\InvalidInputException;

/**
 * Provides various data validation helpers
 *
 * @author      Damian Senn <ds@adfinis.ch>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 */
class Validator {
	/**
	 * Checks if a given URI is valid
	 *
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       string $URI
	 * @param       bool $withScheme
	 * @param       string $defaultScheme
	 * @return      bool
	 */
	public static function URI($URI, $withScheme = true, $defaultScheme = 'http://'){
		if(!$withScheme){
			$URI = sprintf('%s%s', $defaultScheme, $URI);
		}

		try{
			$zendURI = \Zend_Uri::factory($URI);

			if($zendURI->valid()){
				return true;
			} else {
				return false;
			}
		}
		catch(\Exception $e){
			return false;
		}

		return false;
	}

	/**
	 * Check if a given uri is valid
	 *
	 * @author      Damian Senn <ds@adfinis.ch>
	 * @param       string $hostname
	 * @param       bool $DNS If set to true (default), allows domain names
	 * @param       bool $IP  If set to true (default=false), allows IP addresses as well
	 * @return      bool
	 */
	public static function hostname($hostname, $DNS = true, $IP = false){
		if($DNS && $IP){
			$validator = new \Zend_Validate_Hostname(
				\Zend_Validate_Hostname::ALLOW_DNS | \Zend_Validate_Hostname::ALLOW_IP
			);
		} elseif($DNS && !$IP){
			$validator = new \Zend_Validate_Hostname(
				\Zend_Validate_Hostname::ALLOW_DNS
			);
		} elseif(!$DNS && $IP){
			$validator = new \Zend_Validate_Hostname(
				\Zend_Validate_Hostname::ALLOW_IP
			);
		}

		if($validator->isValid($hostname)){
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Checks if the given zip code is valid
	 *
	 * @author        Damian Senn <ds@adfinis.ch>
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   Implement checking with Zend_Locale's data
	 * @param         string $ZIP
	 * @param         string $countryCode (optional, defaults to 'ch')
	 * @return        bool
	 */
	public static function ZIP($ZIP, $countryCode = 'ch'){
		switch($countryCode){
			case 'ch':
				if(preg_match('/^\d{4}$/', $ZIP)){
					$query = "
						SELECT
							`zip`
						FROM
							`city`
					";

					$ZIPCodes = array();
					foreach(DbManager::query($query) as $city){
						$ZIPCodes[] = $city['zip'];
					}

					if(in_array($ZIP, $ZIPCodes)){
						return true;
					} else {
						return false;
					}
				} else {
					return false;
				}
				break;
			default:
				if(!preg_match('/^\D*$/', $ZIP)){
					return true;
				} else {
					return false;
				}
			break;
		}

		return false;
	}

	/**
	 * Checks if a given number is a phone number
	 *
	 * @author        Damian Senn <ds@adfinis.ch>
	 * @author        David Vogt <dv@adfinis.ch>
	 * @param         string $phoneNumber
	 * @param         string $countryCode (optional, defaults to 'ch')
	 * @return        bool
	 */
	public static function phoneNumber($phoneNumber, $countryCode){
		$phoneNumber = preg_replace('/[\/\s\(\)\+\.-]/', '', $phoneNumber);

		// Check for invalid chars (we stripped out the valid ones already)
		if(!preg_match('/^\d+$/', $phoneNumber)){
			return false;
		}


		// Check number length
		$phoneNumberLength = strlen($phoneNumber);

		// Allowed lengths
		$lengths = array();

		// SWITZERLAND:
		// 3 and 4: service numbers.
		// 10: without country code, with leading 0:   031 381 70 47
		// 11: with country code, stripped +:        41 31 381 70 47
		// 13: with country code. 00 instead of +: 0041 31 381 70 47
		$lengths['ch'] = array(3, 4, 10, 11, 13);

		if(!isset($lengths[$countryCode])) {
			throw new InvalidInputException(sprintf(tr('Unknown country code "%s"'), $countryCode));
		}

		if(!in_array($phoneNumberLength, $lengths[$countryCode])) {
			return false;
		}

		return true;
	}
}
