<?php

namespace org\adfinis\framework\xml;

use org\adfinis\framework as af;

/**
 * Object oriented wrapper for xml_* functions
 *
 * This is an EventEmitter the following events are emitted in the parse()
 * method (and setup in eventSetup()):
 * - "character data"
 * - "default"
 * - "start element"
 * - "end element"
 * - "start namespace declaration"
 * - "end namespace declaration"
 * - "notation declaration"
 * - "processing instruction"
 * - "unparsed entity declaration"
 *
 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
 */
class Parser extends af\event\Emitter {

	/**
	 * Holds a resource created by xml_parser_create() or xml_parser_create_ns()
	 *
	 * @var           resource $parser
	 */
	private $parser;

	/**
	 * Current nesting level, starting at -1 (0 would be a root element)
	 *
	 * @var           int $level
	 */
	public $level = -1;

	/**
	 * Element name stack
	 *
	 * @var           array<string> $elementStack
	 */
	public $elementStack = array();

	/**
	 * Constructor of XMLParser
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @param         resource $parser
	 */
	public function __construct($parser) {
		$this->parser = $parser;

		$this->eventSetup();
	}

	/**
	 * Sets up the event handlers
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @see           http://php.net/xml_set_character_data_handler
	 * @see           http://php.net/xml_set_default_handler
	 * @see           http://php.net/xml_set_element_handler
	 * @see           http://php.net/xml_set_start_namespace_decl_handler
	 * @see           http://php.net/xml_set_end_namespace_decl_handler
	 * @see           http://php.net/xml_set_external_entity_ref_handler
	 * @see           http://php.net/xml_set_notation_decl_handler
	 * @see           http://php.net/xml_set_processing_instruction_handler
	 * @see           http://php.net/xml_set_unparsed_entity_decl_handler
	 * @returns       void
	 */
	private function eventSetup() {
		$self = $this;

		xml_set_character_data_handler($this->parser, function($parser, $data) use ($self) {
			$self->emit('character data', $data);
		});

		xml_set_default_handler($this->parser, function($parser, $data) use ($self) {
			$self->emit('default', $data);
		});

		xml_set_element_handler($this->parser,
			function($parser, $name, $attr) use ($self) {
				$name = $self->caseFolding ? strtolower($name) : $name;

				$self->level++;
				$self->elementStack[] = $name;
				$self->emit('start element', $name, $attr);
			},
			function($parser, $name) use ($self) {
				$name = $self->caseFolding ? strtolower($name) : $name;

				$self->emit('end element', $name);
				$self->level--;
				array_pop($self->elementStack);
			}
		);

		xml_set_start_namespace_decl_handler($this->parser,
			function($parser, $prefix, $uri) use ($self) {
				$self->emit('start namespace declaration', $prefix, $uri);
			}
		);

		xml_set_end_namespace_decl_handler($this->parser,
			function($parser, $prefix) use ($self) {
				$self->emit('end namespace declaration', $prefix);
			}
		);

		xml_set_external_entity_ref_handler($this->parser,
			function($parser, $openEntityNames, $base, $systemId, $publicId) use ($self) {
				$self->emit('external entity reference', $openEntityNames, $base, $systemId, $publicId);
			}
		);

		xml_set_notation_decl_handler($this->parser,
			function($parser, $notationName, $base, $systemId, $publicId) use ($self) {
				$self->emit('notation declaration', $notationName, $base, $systemId, $publicId);
			}
		);

		xml_set_processing_instruction_handler($this->parser,
			function($parser, $target, $data) use ($self) {
				$self->emit('processing instruction', $target, $data);
			}
		);

		xml_set_unparsed_entity_decl_handler($this->parser,
			function($parser, $entityName, $base, $systemId, $publicId, $notationName) use ($self) {
				$self->emit('unparsed entity declaration', $entityName, $base, $systemId, $publicId, $notationName);
			}
		);
	}

	/**
	 * Destructor of XMLParser
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 */
	private function __destruct() {
		if (is_resource($this->parser)) xml_parser_free($this->parser);
	}

	/**
	 * Implement getter for various properties
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @returns       mixed
	 */
	public function __get($prop) {
		if ($prop === 'caseFolding')
			return xml_parser_get_option($this->parser, \XML_OPTION_CASE_FOLDING);

		if ($prop === 'skipTagstart')
			return xml_parser_get_option($this->parser, \XML_OPTION_SKIP_TAGSTART);

		if ($prop === 'skipWhite')
			return xml_parser_get_option($this->parser, \XML_OPTION_SKIP_WHITE);

		if ($prop === 'targetEncoding')
			return xml_parser_get_option($this->parser, \XML_OPTION_TARGET_ENCODING);

		if ($prop === 'currentByteIndex')
			return xml_get_current_byte_index($this->parser);

		if ($prop === 'currentColumnNumber')
			return xml_get_current_column_number($this->parser);

		if ($prop === 'currentLineNumber')
			return xml_get_current_line_number($this->parser);

		if ($prop === 'errorCode')
			return xml_get_error_code($this->parser);

		if ($prop === 'errorString')
			return xml_error_string($this->errorCode);

		if ($prop === 'currentElement')
			return $this->elementStack[sizeof($this->elementStack) - 1];
	}

	/**
	 * Set parser options
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @param         string $prop
	 * @param         mixed  $val
	 * @returns       void
	 */
	public function __set($prop, $val) {
		if ($prop === 'caseFolding')
			return xml_parser_set_option($this->parser, \XML_OPTION_CASE_FOLDING, $val);

		if ($prop === 'skipTagstart')
			return xml_parser_set_option($this->parser, \XML_OPTION_SKIP_TAGSTART, $val);

		if ($prop === 'skipWhite')
			return xml_parser_set_option($this->parser, \XML_OPTION_SKIP_WHITE, $val);

		if ($prop === 'targetEncoding')
			return xml_parser_set_option($this->parser, \XML_OPTION_TARGET_ENCODING, $val);
	}

	/**
	 * Parse the given xml string
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @throws        ParserException
	 * @param         string $data
	 * @param         bool   $final [optional]
	 * @returns       void
	 */
	public function parse($data, $final = false) {
		if (!(bool)xml_parse($this->parser, $data, $final)) {
			throw new ParserException($this);
		}
	}

	/**
	 * Parse the given file
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @throws        ParserException
	 * @param         string $filename
	 * @param         int    $buffsize
	 * @returns       void
	 */
	public function parseFile($filename, $buffsize = 8192) {
		try {
			$file = fopen($filename, 'rb');
			$eof  = feof($file);
			$read = 0;

			while (!$eof) {
				$data = fread($file, $buffsize);

				$this->parse($data, $eof = feof($file));
			}

			fclose($file);
		}
		catch (\Exception $e) {
			fclose($file); // yeah kthx no finally for php...

			throw $e;
		}
	}

	/**
	 * Creates a XMLParser instance
	 *
	 * @author        Damian Senn <damian.senn@adfinis-sygroup.ch>
	 * @param         string $encoding [optional]
	 * @param         bool   $ns       [optional]
	 * @returns       Parser
	 */
	public static function create($encoding = null, $ns = false) {
		$parser = $ns ? xml_parser_create_ns($encoding) : xml_parser_create($encoding);

		return new static($parser);
	}
}
