<?php

namespace org\adfinis\framework\test\datamapper;

use org\adfinis\framework as oaf;

/**
 * Test cases for SimpleMapper
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class SimpleMapperTest extends \PHPUnit_Framework_TestCase {

	/**
	 * Creates required table in database
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function setUp() {
		oaf\db\DbManager::query('
			DROP TABLE IF EXISTS oaf_simplemapper_test;
		');

		oaf\db\DbManager::query('
			CREATE TABLE oaf_simplemapper_test (
				id  INT NOT NULL PRIMARY KEY auto_increment,
				foo VARCHAR(30),
				bar TEXT,
				num INTEGER(10)
			);
		');

		oaf\db\DbManager::insert('oaf_simplemapper_test',
			array('foo' => 'foo', 'bar' => 'bar', 'num' => 42));

		oaf\db\DbManager::insert('oaf_simplemapper_test',
			array('foo' => 'baa', 'bar' => 'foo', 'num' => 69));

		oaf\db\DbManager::insert('oaf_simplemapper_test',
			array('foo' => 'foo', 'bar' => 'bar', 'num' => 42));
	}

	/**
	 * Removes the test table
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function tearDown() {
		oaf\db\DbManager::query('DROP TABLE oaf_simplemapper_test');
		SimpleMapperTestModel::getMapper()->clearIdentityMap();
	}

	/**
	 * Tests if all row data is correctly set
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function testData() {
		$className = '\org\adfinis\framework\test\datamapper\SimpleMapperTestModel';

		$mapper = SimpleMapperTestModel::getMapper();
		$model1 = $mapper->getObjectByID(1);
		$model2 = $mapper->getObjectByID(2);
		$model3 = $mapper->getObjectByID(3);

		$this->assertInstanceOf($className, $model1);
		$this->assertInstanceOf($className, $model2);

		$this->assertEquals('bar',   $model1->bar);
		$this->assertEquals('foo',   $model1->foo);
		$this->assertEquals(42,      $model1->num);
		$this->assertNotSame('42',   $model1->num);

		$this->assertEquals('foo',   $model2->bar);
		$this->assertEquals('baa',   $model2->foo);
		$this->assertEquals(69,      $model2->num);
		$this->assertNotSame('69',   $model2->num);
	}

	/**
	 * Tests if the simple mapper uses an identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function testIdentityMap() {
		$mapper = SimpleMapperTestModel::getMapper();
		$model1 = $mapper->getObjectByID(1);
		$model3 = $mapper->getObjectByID(3);

		$this->assertNotSame($model1, $model3);
		$this->assertSame($model3, $mapper->getObjectByID(3));
	}

	/**
	 * Test if some properties are correctly saved
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function testSave() {
		$mapper = SimpleMapperTestModel::getMapper();
		$model  = $mapper->getObjectByID(1);

		$model->foo = 'blubber';

		$model->save();

		$data = oaf\db\DbManager::query(
			'SELECT foo FROM oaf_simplemapper_test WHERE id = 1')->fetchColumn();

		$this->assertEquals('blubber', $data);
	}

	/**
	 * Test duplicate entry, if we save an object, it should be in the
	 * identity map
	 *
	 * @author        Damian Senn <ds@adfinis.com>
	 * @returns       void
	 */
	public function testDuplicateEntry() {
		$mapper = SimpleMapperTestModel::getMapper();
		$model1 = new SimpleMapperTestModel();

		$model1->id  = 1;
		$model1->foo = 'blubber';

		$model1->save();

		$model2 = $mapper->getObjectByID(1);

		$this->assertSame($model1, $model2);
	}
}

/**
 * Helper class to test the simple mapper
 *
 * @author        Damian Senn <ds@adfinis.com>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 */
class SimpleMapperTestModel extends oaf\model\Entity {
	private static $mapper;

	public function getIdentifier() {
		return array('id' => $this->id);
	}

	public static function getMapper() {
		return self::$mapper ?:
			self::$mapper = new oaf\datamapper\SimpleMapper(
				'\org\adfinis\framework\test\datamapper\SimpleMapperTestModel',
				'oaf_simplemapper_test',
				'id'
			);
	}

	public function fillFromRowData($data) {
		$this->id  = (int)$data['id'];
		$this->foo = $data['foo'];
		$this->bar = $data['bar'];
		$this->num = (int)$data['num'];
	}

	public function doesRecordMatch($data) {
		return $this->id === (int)$data['id'];
	}
}
