<?php

namespace org\adfinis\framework\test\db;

use org\adfinis\framework\db\DbManager;
use org\adfinis\framework\db\DBIterator;
use org\adfinis\framework\model\Entity;

/**
 * Test cases for the DB iterator
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class DBIteratorTest extends \PHPUnit_Framework_TestCase {

	/**
	 * Creates required table in database
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function setUp() {
		DbManager::query('
			DROP TABLE IF EXISTS oaf_iterator_test;
		');
		DbManager::query('
			CREATE TABLE oaf_iterator_test (
				id       INT NOT NULL PRIMARY KEY auto_increment,
				foo      VARCHAR(30),
				bar      TEXT
			);
		');

		DbManager::insert('oaf_iterator_test', array('foo' => 'hello, world', 'bar' => 'world, hello!'));
		DbManager::insert('oaf_iterator_test', array('foo' => 'trololo',      'bar' => 'Bla Bla'      ));
		DbManager::insert('oaf_iterator_test', array('foo' => 'trololo',      'bar' => 'Second troll' ));
	}

	/**
	 * Removes the test table
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function tearDown() {
		DbManager::query('DROP TABLE oaf_iterator_test');
	}


	/**
	 * Tests the selecting capability (search) of DBIterator
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testFind() {
		$select = 'SELECT *    FROM `oaf_iterator_test`';
		$search = 'SELECT `id` FROM `oaf_iterator_test` WHERE `foo` = ?';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel',
			array('trololo')
		);

		$result = array();
		foreach($iterator as $obj) {
			$result[] = $obj;
		}

		list($obj1, $obj2) = $result;

		$this->assertEquals($obj1->bar, 'Bla Bla');
		$this->assertEquals($obj2->bar, 'Second troll');

		// Assert that we actually found the two wanted entries
		$this->assertEquals(sizeof($result), 2);
	}

	/**
	 * Tests the countable interface of the DBIterator
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testCountable() {
		$select = 'SELECT *    FROM `oaf_iterator_test`';
		$search = 'SELECT `id` FROM `oaf_iterator_test`';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel'
		);

		$page1 = clone($iterator);
		$page1->setPage(1,2);
		$this->assertEquals(2, $page1->count());

		$page2 = clone($iterator);
		$page2->setPage(2,2);
		$this->assertEquals(1, $page2->count());

	}

	/**
	 * Tests the count capability of DBIterator, especially together with paging
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testTotalCounting() {
		$select = 'SELECT *    FROM `oaf_iterator_test`                 ORDER BY `bar` DESC';
		$search = 'SELECT `id` FROM `oaf_iterator_test` WHERE `foo` = ? ORDER BY `bar` DESC';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel',
			array('trololo')
		);

		$iterator->setPage(1,1);

		$result = array();
		foreach($iterator as $obj) {
			$result[] = $obj;
		}

		list($obj1) = $result;

		$this->assertEquals($obj1->bar, 'Second troll');
		$this->assertEquals(sizeof($result), 1);

		$count = $iterator->getCount();

		// Count should be 2 despite having only 1 entry per page.
		$this->assertEquals($count, 2);
	}



	/**
	 * Tests the pagination capability of DBIterator
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testPaging() {
		$select = 'SELECT *    FROM `oaf_iterator_test`                 ORDER BY `bar` DESC';
		$search = 'SELECT `id` FROM `oaf_iterator_test` WHERE `foo` = ? ORDER BY `bar` DESC';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel',
			array('trololo')
		);

		$iterator->setPage(1,1);

		$result = array();
		foreach($iterator as $obj) {
			$result[] = $obj;
		}

		list($obj1) = $result;

		$this->assertEquals($obj1->bar, 'Second troll');

		$this->assertEquals(sizeof($result), 1);
	}

	/**
	 * Tests the ordering capability of DBIterator
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testOrdering() {
		$select = 'SELECT *    FROM `oaf_iterator_test`                 ORDER BY `bar` DESC';
		$search = 'SELECT `id` FROM `oaf_iterator_test` WHERE `foo` = ? ORDER BY `bar` DESC';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel',
			array('trololo')
		);

		$result = array();
		foreach($iterator as $obj) {
			$result[] = $obj;
		}

		list($obj1, $obj2) = $result;

		$this->assertEquals($obj1->bar, 'Second troll');
		$this->assertEquals($obj2->bar, 'Bla Bla');
	}

	/**
	 * Tests the rewind capability of DBIterator
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testRewind() {
		$select = 'SELECT *    FROM `oaf_iterator_test`';
		$search = 'SELECT `id` FROM `oaf_iterator_test` ORDER BY `id`';

		$iterator = DBIterator::create(
			$search,
			$select,
			'org\adfinis\framework\test\db\DBIteratorTestModel'
		);

		$result1 = array();
		foreach($iterator as $obj) {
			$result1[] = $obj;
		}

		$result2 = array();
		foreach($iterator as $obj) {
			$result2[] = $obj;
		}

		$this->assertEquals(sizeof($result1), sizeof($result2));
		$this->assertEquals(3, sizeof($result1));

		for($i = 0; $i < sizeof($result2); $i++) {
			$this->assertEquals($result1[$i], $result2[$i]);
		}

	}
}

/**
 * Helper class for checking the functionality of oaf\db\DBIterator
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class DBIteratorTestModel extends Entity {

	public $id;
	public $foo;
	public $bar;

	function getIdentifier() {
		return array('id' => $this->id);
	}

	static function getMapper() {
		return new TestModelMapper();
	}


	function fillFromRowData($data) {
		$this->id = $data['id'];
		$this->foo = $data['foo'];
		$this->bar = $data['bar'];
	}

	function doesRecordMatch($data) {
		return $this->id == $data['id'];
	}
}

class TestModelMapper {
	public static $saved;

	function save($object) {
		self::$saved = $object;
	}

}
