<?php
// +----------------------------------------------------------------------+
// | adfinis framework                                                    |
// +----------------------------------------------------------------------+

/**
 * Contains file utility class
 *
 * @author      David Vogt <dv@adfinis.ch>
 * @package     org.adfinis.tests.form
 * @copyright   adfinis GmbH 2007-2010
 * @license     LGPL 3.0
 */

namespace org\adfinis\framework\test\form;

use org\adfinis\framework\form\FormValidator;

/**
 * Test cases for the FormValidator class
 *
 * @author      David Vogt <dv@adfinis.com>
 * @package     org.adfinis.tests.form
 * @todo by dv: Implement tests for checkURI(), hostname(), unique(), length(), email()
 **/
class FormValidatorTest extends \PHPUnit_Framework_TestCase {

	private function assertIsClosure($obj) {
		$type = gettype($obj);
		$this->assertTrue(is_callable($obj) && $type=='object', 'Given variable is not a closure/callable');
	}

	/**
	 * Tests correct function of the range validator
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @test        FormValidator::range()
	 * @return      void
	 */
	public function testRange() {
		$rangeTest = FormValidator::range('xxx--%s---%s---%s---', 5, 10);

		// see if the validator returned the right object
		$this->assertIsClosure($rangeTest);

		// string out of range (above)
		$result = $rangeTest('11');
		$this->assertTrue($result['error']);

		// number out of range (above)
		$result = $rangeTest(11);
		$this->assertTrue($result['error']);

		// string out of range (below)
		$result = $rangeTest('4');
		$this->assertTrue($result['error']);

		// number out of range (below)
		$result = $rangeTest(4);
		$this->assertTrue($result['error']);

		// number within range (top)
		$result = $rangeTest(10);
		$this->assertFalse($result['error']);

		// number within range (bottom)
		$result = $rangeTest(5);
		$this->assertFalse($result['error']);

		// string within range (top)
		$result = $rangeTest('10');
		$this->assertFalse($result['error']);

		// string within range (bottom)
		$result = $rangeTest('5');
		$this->assertFalse($result['error']);


		// error message format
		$result = $rangeTest(123);
		$expectedMessage = 'xxx--5---10---123---';
		$this->assertSame($result['message'], $expectedMessage, 'Error message not properly formatted');

	}

	/**
	 * Test of regex validator
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @test        FormValidator::regexp()
	 * @return      void
	 */
	public function testRegexp() {
		$checkNormal   = FormValidator::regexp('xxx-%s--%s--%s--', '/foo\sbar/');
		$checkInverted = FormValidator::regexp('xxx-%s--%s--%s--', '/foo\sbar/', true);

		$goodValues = array(
			'foo bar',
			'bla bla foo bar baz',
			'bla bla foo bar',
			"bla foo\tbar",
			"bla foo\nbar"
		);

		$badValues = array(
			'Foo bar',
			'foo Bar',
			'bla bla foobar baz'
		);

		foreach ($goodValues as $val) {
			$resNorm = $checkNormal($val);
			$resInv  = $checkInverted($val);
			$this->assertFalse($resNorm['error'], sprintf('Test string "%s" was not accepted by regex',          $val));
			$this->assertTrue ($resInv ['error'], sprintf('Test string "%s" was not rejected by inverted regex', $val));
		}

		foreach ($badValues as $val) {
			$resNorm = $checkNormal($val);
			$resInv  = $checkInverted($val);
			$this->assertTrue ($resNorm['error'], sprintf('Test string "%s" was not rejected by regex',          $val));
			$this->assertFalse($resInv ['error'], sprintf('Test string "%s" was not accepted by inverted regex', $val));
		}

		$failResult = $checkNormal('aaaaaa');
		$this->assertSame('xxx-/foo\sbar/--aaaaaa----', $failResult['message']);
	}


	/**
	 * Test of enum validator
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @test        FormValidator::enum()
	 * @return      void
	 */
	public function testEnum() {
		$checkNormal   = FormValidator::enum('xx-%s-%s-', array('a', 'b', 'c'));
		$checkInverted = FormValidator::enum('xx-%s-%s-', array('a', 'b', 'c'), true);

		$goodValues = array('a', 'b', 'c');
		$badValues  = array('d', 'e', 'f');

		foreach ($goodValues as $val) {
			$resNorm = $checkNormal($val);
			$resInv  = $checkInverted($val);
			$this->assertFalse($resNorm['error'], sprintf('Test value "%s" was not accepted by enum',          $val));
			$this->assertTrue ($resInv ['error'], sprintf('Test value "%s" was not rejected by inverted enum', $val));
		}

		foreach ($badValues as $val) {
			$resNorm = $checkNormal($val);
			$resInv  = $checkInverted($val);
			$this->assertTrue ($resNorm['error'], sprintf('Test value "%s" was not rejected by enum',          $val));
			$this->assertFalse($resInv ['error'], sprintf('Test value "%s" was not accepted by inverted enum', $val));
		}

		$failResult = $checkNormal('aaaaaa');
		$this->assertSame('xx-aaaaaa-a, b, c-', $failResult['message']);
	}


	/**
	 * Test for weakFunctionGroup meta-validator
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @test        FormValidator::weakFunctionGroup()
	 * @return      void
	 */
	public function testWeakFunctionGroup() {
		$funcGood = function($val) {
			return array('error' => false, 'message' => '');
		};
		$funcBad1 = function($val) {
			return array('error' => true, 'message' => 'bad1');
		};
		$funcBad2 = function($val) {
			return array('error' => true, 'message' => 'bad2');
		};


		// should NOT fail - all sub-checks return error => 0
		$check = FormValidator::weakFunctionGroup($funcGood, $funcGood, $funcGood);
		$result = $check('test');
		$this->assertFalse($result['error'], 'All sub-validators were good, result was bad');

		// should NOT fail - only one sub-checks returns error => 1
		$check = FormValidator::weakFunctionGroup($funcGood, $funcBad1, $funcGood);
		$result = $check('test');
		$this->assertFalse($result['error'], 'All but one sub-validators were good, result was bad');

		// SHOULD fail - all sub-checks return error => 1
		$check = FormValidator::weakFunctionGroup($funcBad2, $funcBad1, $funcBad1);
		$result = $check('test');
		$this->assertTrue($result['error'],  'All sub-validators were bad, result was good');

		// we expect the message from the FIRST failed check
		$this->assertSame('bad2', $result['message']);

	}

	/**
	 * Test for functionGroup meta-validator
	 *
	 * @author      David Vogt <dv@adfinis.com>
	 * @test        FormValidator::functionGroup()
	 * @return      void
	 */
	public function testFunctionGroup() {
		$funcGood = function($val) {
			return array('error' => false, 'message' => '');
		};
		$funcBad1 = function($val) {
			return array('error' => true, 'message' => 'bad1');
		};
		$funcBad2 = function($val) {
			return array('error' => true, 'message' => 'bad2');
		};


		// should NOT fail - all sub-checks return error => 0
		$check = FormValidator::functionGroup($funcGood, $funcGood, $funcGood);
		$result = $check('test');
		$this->assertFalse($result['error'], 'All sub-validators were good, result was bad');

		// SHOULD fail - one of three sub-checks returns error => 1
		$check = FormValidator::functionGroup($funcGood, $funcBad1, $funcGood);
		$result = $check('test');
		$this->assertTrue($result['error'], 'All but one sub-validators were good, result was good');

		// SHOULD fail - all sub-checks return error => 1
		$check = FormValidator::weakFunctionGroup($funcBad2, $funcBad1, $funcBad1);
		$result = $check('test');
		$this->assertTrue($result['error'],  'All sub-validators were bad, result was good');

		// we expect the message from the FIRST failed check
		$this->assertSame('bad2', $result['message']);

	}



}
