<?php

namespace org\adfinis\framework\tests\misc;

use org\adfinis\framework\exceptions;
use org\adfinis\framework\presentation;
use org\adfinis\framework\template;
use org\adfinis\framework\utils;
use org\adfinis\framework\form;
use org\adfinis\framework\config;
use org\adfinis\framework\request;

/**
 * Checks for the loadability of framework classes
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class ClassLoadingTest extends \PHPUnit_Framework_TestCase {

	/**
	 * Tests the loadability of generic FW exception classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testExceptionClasses() {

		$e = new exceptions\UserException();
		$this->assertTrue(is_object($e));
		$this->assertTrue($e instanceof \Exception);

		$e = new exceptions\InvalidInputException();
		$this->assertTrue(is_object($e));
		$this->assertTrue($e instanceof \Exception);
	}

	/**
	 * Test loadability of presentation classes
	 *
	 * This test if Page, PageHistory and Paging are loadable
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testPresentationClasses() {

		$h = new presentation\PageHistory();
		$this->assertTrue(is_object($h), "Could not load presentation\PageHistory");

		// Use test class, as org\adfinis\framework\presentation\Page is abstract
		$h = new TestPage();
		$this->assertTrue(is_object($h), "Could not load presentation\Page");
		$this->assertTrue($h instanceof presentation\Page, "Could not load presentation\Page");

		$ok = class_exists('org\adfinis\framework\presentation\Paging');
		$this->assertTrue($ok, "Could not load presentation\Paging");
	}

	/**
	 * Test loading of skittle (custom smarty wrapper)
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testSkittle() {
		$s = new template\Skittle();
		$this->assertTrue(is_object($s));
		$this->assertTrue($s instanceof \Smarty);
	}

	/**
	 * Test loadability of various utility classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testUtilityClasses() {
		$b = utils\Browser::getInstance('');
		$this->assertTrue(is_object($b));

		$c = new utils\CLIChecker;
		$this->assertTrue(is_object($c));

		$c = new utils\WebServiceURL();
		$this->assertTrue(is_object($c));

		$s = new utils\Shell();
		$this->assertTrue(is_object($s));

		$s = new utils\PasswordGenerator();
		$this->assertTrue(is_object($s));

		$d = new utils\DownloadHelper();
		$this->assertTrue(is_object($d));

		$d = new utils\URLHelper();
		$this->assertTrue(is_object($d));

		$d = new utils\i18n\Translator();
		$this->assertTrue(is_object($d));

		# ./org/adfinis/framework/utils/CaptchaHelper.class.php
		# -- This class doesn't work with the (still broken) config just
		#    yet. So we only make sure it can be require()d automatically.
		$this->assertTrue(class_exists('org\adfinis\framework\utils\CaptchaHelper'));

		# ./org/adfinis/framework/utils/PictureHelper.class.php
		$d = new utils\PictureHelper();
		$this->assertTrue(is_object($d));


		# ./org/adfinis/framework/utils/Files.class.php
		$d = new utils\FileHelper();
		$this->assertTrue(is_object($d));

		$this->assertTrue(class_exists('org\adfinis\framework\utils\fileformats\CSVIterator'));

		# ./org/adfinis/framework/utils/DebugHelper.class.php
		$d = new utils\DebugHelper();
		$this->assertTrue(is_object($d));

		# ./org/adfinis/framework/utils/DateHelper.class.php
		$d = new utils\DateHelper();
		$this->assertTrue(is_object($d));

		# ./org/adfinis/framework/utils/SystemUserHelper.class.php
		$d = new utils\SystemUserHelper();
		$this->assertTrue(is_object($d));

	}

	/**
	 * Testing form handler class presence
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testFormhandlerClasses() {
		# ./org/adfinis/framework/form/fields/FormTextField.class.php
		$d = new form\fields\FormTextField('', '');
		$this->assertTrue(is_object($d));

		# ./org/adfinis/framework/form/fields/FormPasswordGroup.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormPasswordGroup'));

		# ./org/adfinis/framework/form/FormRadioGroup.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormRadioField'));

		# ./org/adfinis/framework/form/fields/FormDateTimeField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormDateTimeField'));

		# ./org/adfinis/framework/form/fields/FormSelectField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormDateTimeField'));

		# ./org/adfinis/framework/form/FormHandler.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\FormHandler'));

		# ./org/adfinis/framework/form/FormValidator.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\FormValidator'));

		# ./org/adfinis/framework/form/FormCheckboxGroup.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormCheckboxGroup'));

		# ./org/adfinis/framework/form/fields/FormRadioField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormRadioField'));

		# ./org/adfinis/framework/form/fields/FormTextInputGroup.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormTextInputGroup'));

		# ./org/adfinis/framework/form/fields/FormField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormField'));

		# ./org/adfinis/framework/form/fields/FormCheckboxField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormCheckboxField'));

		# ./org/adfinis/framework/form/fields/FormInputField.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\form\fields\FormInputField'));

		# ./org/adfinis/framework/model/validate/BaseDataCheck.class.php
		$validator = new \org\adfinis\framework\validate\Validator;
		$this->assertTrue(is_object($validator));

	}

	/**
	 * Testing conf manager presence
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testConfManager() {

		# ./org/adfinis/framework/config/ConfManager.class.php
		$d = new config\ConfManager();
		$this->assertTrue(is_object($d));
	}

	/**
	 * Testing request class presence
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testRequestClasses() {

		$fakeGET    = array();
		$fakePOST   = array();
		$fakeFILES  = array();
		$fakeCOOKIE = array();

		$this->assertTrue(class_exists('org\adfinis\framework\request\AbstractRequest'));

		$br = new request\Request($fakePOST, $fakeGET, $fakeCOOKIE, $fakeFILES);
		$this->assertTrue(is_object($br));
		$this->assertTrue($br instanceof request\RequestInterface);

		$fr = new request\ForgivingRequest($br);
		$this->assertTrue(is_object($fr));
		$this->assertTrue($fr instanceof request\RequestInterface);

		$iv = new request\InputVars;
		$this->assertTrue(is_object($iv));
	}

	/**
	 * Testing session utility classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testSessionClasses() {
		# ./org/adfinis/framework/session/SessionHandlerMemcache.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\session\SessionHandlerMemcache'));

		# ./org/adfinis/framework/session/SessionManager.class.php
		$this->assertTrue(class_exists('org\adfinis\framework\session\SessionManager'));
	}

	/**
	 * Testing session utility classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testInitClasses() {
		# ./org/adfinis/framework/init/InvalidConfigFileException.class.php
		$icfeOK = class_exists('org\adfinis\framework\init\InvalidConfigFileException') || die('eek' . __LINE__);
		$this->assertTrue($icfeOK);

		# ./org/adfinis/framework/init/ClassLoader.class.php
		$clOK = class_exists('org\adfinis\framework\init\ClassLoader') || die('eek' . __LINE__);
		$this->assertTrue($clOK);

		# ./org/adfinis/framework/init/FWInit.class.php
		$init = new \org\adfinis\framework\init\FWInit;
		$this->assertTrue(is_object($init));
	}

	/**
	 * Testing geo utility classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testGeoClasses() {
		# ./org/adfinis/framework/geo/GeoIp.class.php
		$bl = new \org\adfinis\framework\geo\GeoIp;
		$this->assertTrue(is_object($bl));

		# ./org/adfinis/framework/model/CityManager.class.php
		$bl = new \org\adfinis\framework\geo\CityManager;
		$this->assertTrue(is_object($bl));
	}

	/**
	 * Testing database classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testDBClasses() {
		# ./org/adfinis/framework/db/sql/WhereGroup.class.php
		$bl = new \org\adfinis\framework\db\sql\WhereGroup;
		$this->assertTrue(is_object($bl));

		# ./org/adfinis/framework/db/sql/WhereStatement.class.php
		$bl = new \org\adfinis\framework\db\sql\WhereStatement('a', 'b', 'c');
		$this->assertTrue(is_object($bl));

		# ./org/adfinis/framework/db/DbManager.class.php
		$bl = new \org\adfinis\framework\db\DbManager;
		$this->assertTrue(is_object($bl));

		# ./org/adfinis/framework/db/DbManager.debug.class.php
		$bl = new \org\adfinis\framework\db\DbManagerDebug;
		$this->assertTrue(is_object($bl));
	}

	/**
	 * Testing string utility classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testStringClasses() {
		# ./org/adfinis/framework/utils/string/StringTransliterator.class.php
		$x = new \org\adfinis\framework\utils\string\StringTransliterator;
		$this->assertTrue(is_object($x));

		# ./org/adfinis/framework/utils/string/StringTokenizer.class.php
		$x = new \org\adfinis\framework\utils\string\StringTokenizer;
		$this->assertTrue(is_object($x));

		# ./org/adfinis/framework/utils/string/StringFunctions.class.php
		$x = new \org\adfinis\framework\utils\string\StringFunctions;
		$this->assertTrue(is_object($x));

		# ./org/adfinis/framework/utils/string/StringFormat.class.php
		$x = new \org\adfinis\framework\utils\string\StringFormat;
		$this->assertTrue(is_object($x));

		# ./org/adfinis/framework/utils/string/StringCleaner.class.php
		$x = new \org\adfinis\framework\utils\string\StringCleaner;
		$this->assertTrue(is_object($x));

		# ./org/adfinis/framework/utils/string/StringWiki.class.php
		$x = new \org\adfinis\framework\utils\string\StringWiki;
		$this->assertTrue(is_object($x));
	}

	/**
	 * Testing model / entity classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testEntityClasses() {

		# org/adfinis/framework/entity/Entity.class.php
		$ok = class_exists('org\adfinis\framework\entity\Entity');
		$this->assertTrue($ok);

		# org/adfinis/framework/entity/EntityPreference.class.php
		$ok = class_exists('org\adfinis\framework\entity\EntityPreference');
		$this->assertTrue($ok);

		# org/adfinis/framework/entity/EntityPreferenceInterface.class.php
		$ok = interface_exists('org\adfinis\framework\entity\EntityPreferenceInterface');
		$this->assertTrue($ok);

		# org/adfinis/framework/entity/EntityWithPreferences.class.php
		$ok = class_exists('org\adfinis\framework\entity\EntityWithPreferences');
		$this->assertTrue($ok);
	}

	/**
	 * Testing login helper classes
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testLoginClasses() {

		# org/adfinis/framework/login/Login.class.php
		$ok = class_exists('org\adfinis\framework\login\Login') || die('eek');
		$this->assertTrue($ok);

		# org/adfinis/framework/login/LoginInterface.class.php
		$ok = interface_exists('org\adfinis\framework\login\LoginInterface') || die('eek');
		$this->assertTrue($ok);
	}
}

/**
 * Test class so we can check proper creation of the abstract class "org\adfinis\framework\presentation\Page"
 *
 * @author      David Vogt <dv@adfinis.com>
 **/
class TestPage extends presentation\Page {

	public function outputPage($tplFile = 'index/layout.tpl') {
		return "<h1>ohai</h1>";
	}
	
}
