<?php

namespace org\adfinis\framework\test\presentation;

use org\adfinis\framework\config\ConfManager;
use org\adfinis\framework\db\DBIterator;
use org\adfinis\framework\db\DbManager;
use org\adfinis\framework\model\Entity;
use org\adfinis\framework\presentation\Paging;
use org\adfinis\framework\request\ForgivingRequest;
use org\adfinis\framework\request\Request;

/**
 * Test cases for the paging helper
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class PagingTest extends \PHPUnit_Framework_TestCase {
	/**
	 * Request object used for testing.
	 *
	 * @var           org\adfinis\framework\request\RequestInterface $request
	 */
	private $request;
	
	/**
	 * Creates required table in database
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function setUp() {
		DbManager::query('DROP TABLE IF EXISTS oaf_paging_test');
		DbManager::query('
			CREATE TABLE oaf_paging_test (
				id       INT NOT NULL PRIMARY KEY auto_increment,
				foo      VARCHAR(30),
				bar      TEXT
			);
		');

		DbManager::insert('oaf_paging_test', array('foo' => 'first',    'bar' => 'world, hello!'));
		DbManager::insert('oaf_paging_test', array('foo' => 'second',   'bar' => 'Bla Bla'      ));
		DbManager::insert('oaf_paging_test', array('foo' => 'third',    'bar' => 'Second troll' ));
		DbManager::insert('oaf_paging_test', array('foo' => 'fourth',   'bar' => 'world, hello!'));
		DbManager::insert('oaf_paging_test', array('foo' => 'fifth',    'bar' => 'Bla Bla'      ));
		DbManager::insert('oaf_paging_test', array('foo' => 'sixth',    'bar' => 'Second troll' ));
		DbManager::insert('oaf_paging_test', array('foo' => 'seventh',  'bar' => 'world, hello!'));
		DbManager::insert('oaf_paging_test', array('foo' => 'eighth',   'bar' => 'Bla Bla'      ));
		DbManager::insert('oaf_paging_test', array('foo' => 'ninth',    'bar' => 'Second troll' ));
		DbManager::insert('oaf_paging_test', array('foo' => 'tenth',    'bar' => 'world, hello!'));
		DbManager::insert('oaf_paging_test', array('foo' => 'eleventh', 'bar' => 'Bla Bla'      ));
		DbManager::insert('oaf_paging_test', array('foo' => 'twelfth',  'bar' => 'Second troll' ));

		$post   = array();
		$get    = array('page' => 2);
		$cookie = array();
		$files  = array();

		$this->request = new Request($post, $get, $cookie, $files);
	}

	/**
	 * Removes the test table
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function tearDown() {
		DbManager::query('DROP TABLE oaf_paging_test');
	}

	/**
	 * Tests Paging with count() % entriesPerPage != 0 (having a remainder)
	 *
	 * @code
	 *  [ 1][ 2][ 3][ 4][ 5][ 6][ 7][ 8][ 9][10][11][12]
	 *  \________._________/\_________.________/\___.__/
	 *         Page 1             Page 2          Page 3
	 * @endcode
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testPaginationWithRemainder() {
		$url = new \org\adfinis\framework\uri\URI('default', false);

		$ids   = 'SELECT `id` FROM `oaf_paging_test` ORDER BY `foo`';
		$data  = 'SELECT *    FROM `oaf_paging_test` ORDER BY `foo`';
		$iter  = DBIterator::create($ids, $data, 'PagingTestModel');
		$pager = new Paging($iter, $this->request, 'page', $url);

		$pager->entriesPerPage       = 5;

		$pagingData = $pager->getPagingData();

		$baseURL = ConfManager::getVar('af', 'URLNoSSL', 'default');

		// Not actually testing the pagination here, just be sure we're prepared correctly.
		$this->assertEquals($baseURL, "".$url);

		// This is dependent on the data INSERT'ed by setUp().
		$this->assertEquals($pagingData['count'], 12);

		// Make sure the getPagingData() method returned sane values in output.
		$this->assertEquals($pagingData['count'],          12);
		$this->assertEquals($pagingData['entriesPerPage'],  5);
		$this->assertEquals($pagingData['lastPage'],        3);
		$this->assertEquals($pagingData['currentPage'],     2);

		$this->assertEquals("".$pagingData['firstPageURL'],    $baseURL . '?page=1');
		$this->assertEquals("".$pagingData['lastPageURL'],     $baseURL . '?page=3');

		$this->assertEquals("".$pagingData['previousPageURL'], $baseURL . '?page=1');
		$this->assertEquals("".$pagingData['nextPageURL'],     $baseURL . '?page=3');

		$left1  = $pagingData['surroundingLeft'][1];
		$right3 = $pagingData['surroundingRight'][3];

		$this->assertEquals("".$left1,  $baseURL . '?page=1');
		$this->assertEquals("".$right3, $baseURL . '?page=3');

		$this->assertEquals(sizeof($pagingData['surroundingLeft']),  1);
		$this->assertEquals(sizeof($pagingData['surroundingRight']), 1);
	}

	/**
	 * Tests Paging with count() % entriesPerPage == 0 (no remainder)
	 *
	 * @code
	 *  [ 1][ 2][ 3][ 4][ 5][ 6][ 7][ 8][ 9][10][11][12]
	 *  \____._____/\____._____/\____._____/\____._____/
	 *     Page 1      Page 2      Page 3      Page 4
	 * @endcode
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testPaginationNoRemainder() {
		$url = new \org\adfinis\framework\uri\URI('default', false);

		$ids   = 'SELECT `id` FROM `oaf_paging_test` ORDER BY `foo`';
		$data  = 'SELECT *    FROM `oaf_paging_test` ORDER BY `foo`';
		$iter  = DBIterator::create($ids, $data, 'PagingTestModel');
		$pager = new Paging($iter, $this->request, 'page', $url);

		$pager->entriesPerPage       = 3;

		$pagingData = $pager->getPagingData();

		$baseURL = ConfManager::getVar('af', 'URLNoSSL', 'default');

		// Not actually testing the pagination here, just be sure we're prepared correctly.
		$this->assertEquals($baseURL, "".$url);

		// This is dependent on the data INSERT'ed by setUp().
		$this->assertEquals($pagingData['count'], 12);

		// Make sure the getPagingData() method returned sane values in output.
		$this->assertEquals($pagingData['count'],          12);
		$this->assertEquals($pagingData['entriesPerPage'],  3);
		$this->assertEquals($pagingData['lastPage'],        4);
		$this->assertEquals($pagingData['currentPage'],     2);

		$this->assertEquals("".$pagingData['firstPageURL'],    $baseURL . '?page=1');
		$this->assertEquals("".$pagingData['lastPageURL'],     $baseURL . '?page=4');

		$this->assertEquals("".$pagingData['previousPageURL'], $baseURL . '?page=1');
		$this->assertEquals("".$pagingData['nextPageURL'],     $baseURL . '?page=3');

		$left1  = $pagingData['surroundingLeft'][1];
		$right3 = $pagingData['surroundingRight'][3];
		$right4 = $pagingData['surroundingRight'][4];

		$this->assertEquals("".$left1,  $baseURL . '?page=1');
		$this->assertEquals("".$right3, $baseURL . '?page=3');
		$this->assertEquals("".$right4, $baseURL . '?page=4');

		$this->assertEquals(sizeof($pagingData['surroundingRight']), 2);
		$this->assertEquals(sizeof($pagingData['surroundingLeft']),  1);
	}
	


}

/**
 * Helper class for checking the functionality of oaf\db\DBIterator
 *
 * @author        David Vogt <dv@adfinis.ch>
 * @copyright     adfinis GmbH 2011
 * @license       BSD 3-Clause
 **/
class PagingTestModel extends Entity {

	public $id;
	public $foo;
	public $bar;

	function getIdentifier() {
		return array('id' => $this->id);
	}

	static function getMapper() {
		return new PagingTestModelMapper();
	}


	function fillFromRowData($data) {
		$this->id = $data['id'];
		$this->foo = $data['foo'];
		$this->bar = $data['bar'];
	}

	function doesRecordMatch($data) {
		return $this->id == $data['id'];
	}
}

class PagingTestModelMapper {
	public static $saved;

	function save($object) {
		self::$saved = $object;
	}

}
