<?php

namespace org\adfinis\framework\tests\validate;

use org\adfinis\framework\exceptions\InvalidInputException;

use org\adfinis\framework\validate\Validator;
use org\adfinis\framework\db\DbManager;

/**
 * Provides various data validation helpers
 *
 * @author      Damian Senn <ds@adfinis.ch>
 * @copyright   adfinis GmbH 2007-2010
 * @license     BSD 3-Clause
 */
class ValidatorTest extends \PHPUnit_Framework_TestCase {

	/**
	 * Prepare test data in DB. This is required for testZIP().
	 *
	 * Fills the "city" table in the configured test DB
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public static function setUpBeforeClass() {
		// This is ugly, but might happen if the previous test run failed.
		// In this case, just drop the table and re-create it.
		try {
			DbManager::query('DROP TABLE `city`');
		}
		catch(\Exception $e) {}

		DbManager::query('
			CREATE TABLE `city` (
				`id`         INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
				`i18nNameID` INT,
				`cantonID`   INT,
				`onrp`       VARCHAR(50),
				`zip`        INT
			)
		');

		// data creator reqires caller to be in a specific directory.
		$currentPWD = getcwd();
		chdir(_CONF_FW . '/bin');

		require _CONF_FW . '/bin/dataCreator/004_cities.php';

		// cd back to not fuck up anything
		chdir($currentPWD);
	}

	/**
	 * Remove test data again.
	 *
	 * Removes the "city" table from the DB.
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public static function tearDownAfterClass() {
		DbManager::query('DROP TABLE `city`');
	}

	/**
	 * Tests org\adfinis\framework\validate\Validator::URI()
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testURI(){
		$badURLs = array(
			'hasf://foo.bar.com', // Bad protocol

			'http:/foo.bar.com',  // Bad separator 1
			'http//foo.bar.com',  // Bad separator 2
			'http:foo.bar.com',   // Bad separator 3
			'http/foo.bar.com',   // Bad separator 4
		);

		$goodURLs = array(
			'http://foo.bar.com',  // 3-part URL
			'http://bar.com',      // 2-part URL
			'http://foo.bar.com/', // 3-part URL with trailing slash
			'http://bar.com/',     // 2-part URL with trailing slash
			'http://foo.bar.com/', // 3-part URL with trailing slash
			'http://bar.com/',     // 2-part URL with trailing slash

			'http://blubber.com/?foo=bar',  // With query string
			'http://blubber.com/some/where.html?foo=bar', // Sub-path with query string
		);

		foreach ($badURLs as $url) {
			$this->assertFalse(Validator::URI($url), sprintf('URL %s should be invalid', $url));
		}

		foreach ($goodURLs as $url) {
			$this->assertTrue(Validator::URI($url), sprintf('URL %s should be valid', $url));
		}

		// Test $withScheme optional param
		$this->assertTrue (Validator::URI('www.foo.com', false));
		$this->assertFalse(Validator::URI('www.foo.com', true));

		// Test $defaultScheme
		$this->assertFalse(Validator::URI('www.foo.com', false, 'http'));
		$this->assertFalse(Validator::URI('www.foo.com', false, 'http:'));
		$this->assertFalse(Validator::URI('www.foo.com', false, 'http/'));
		$this->assertFalse(Validator::URI('www.foo.com', false, 'http:/'));
		$this->assertTrue (Validator::URI('www.foo.com', false, 'http://'));
	}

	/**
	 * Tests org\adfinis\framework\validate\Validator::hostname()
	 *
	 * Note: this test requires DNS to work. Otherwise, 
	 * it will produce false failures!
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testHostname(){

		// Test Domain names                                                 dns   ip
		$this->assertTrue (Validator::hostname('example.org',                true, false));
		$this->assertTrue (Validator::hostname('wewewe.t12v8dlt12343ff.com', true, false));
		$this->assertFalse(Validator::hostname('192.168.1.1',                true, false));

		// Test IP addresses                                                 dns    ip
		$this->assertFalse(Validator::hostname('example.org',                false, true));
		$this->assertFalse(Validator::hostname('wewewe.t12v8dlt12343ff.com', false, true));
		$this->assertTrue (Validator::hostname('192.168.1.1',                false, true));

		// Test both address types                                           dns   ip
		$this->assertTrue (Validator::hostname('example.org',                true, true));
		$this->assertTrue (Validator::hostname('wewewe.t12v8dlt12343ff.com', true, true));
		$this->assertTrue (Validator::hostname('192.168.1.1',                true, true));

		// Test invalid IP addresses
		$this->assertFalse(Validator::hostname('192.1.1',                   false, true));
		$this->assertFalse(Validator::hostname('192.168.1.256',             false, true));

	}

	/**
	 * Tests org\adfinis\framework\validate\Validator::ZIP()
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @returns       void
	 */
	public function testZIP(){
		$this->assertTrue (Validator::ZIP(1337, 'ch'));
		$this->assertFalse(Validator::ZIP(9001, 'ch')); // 9001 does not exist.. at least in cities.csv
	}

	/**
	 * Tests org\adfinis\framework\validate\Validator::phoneNumber()
	 *
	 * @author        David Vogt <dv@adfinis.ch>
	 * @todo by dv:   Test phones from other countries, too.
	 * @returns       void
	 */
	public function testPhoneNumber(){
		$validPhones = array(
			'+41 31 / 381 70 47',
			'0313817047',
			'41313817047',
			'0041313817047',
			'117',
			'112'
		);

		$invalidPhones = array(
			'hello',                 // non-numeric chars
			'00413138170471234',     // too long
			'12'                     // too short
		);

		foreach ($validPhones as $phone) {
			$this->assertTrue(Validator::phoneNumber($phone, 'ch'), sprintf('Phone %s should be valid', $phone));
		}

		foreach ($invalidPhones as $phone) {
			$this->assertFalse(Validator::phoneNumber($phone, 'ch'), sprintf('Phone %s should be invalid', $phone));
		}

		// Test invalid country code exception
		$hadException = false;
		$exception    = null;
		try {
			Validator::phoneNumber('031 381 70 47', 'de');
		}
		catch(\Exception $e) {
			$hadException = true;
			$exception    = $e;
		}
		$this->assertTrue($hadException);
		$this->assertTrue($exception instanceof InvalidInputException);
	}
}
