/* Implementing all the JavaScript 1.6 and 1.8 Array methods
 * 	https://developer.mozilla.org/en/New_in_JavaScript_1.6
 * 	https://developer.mozilla.org/en/New_in_JavaScript_1.8
 */

/**
 * Returns the index of the given item's first occurrence.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.indexOf(searchElement[, fromIndex]);
 *
 * @param       element searchElement element to locate in the array
 * @param       int     fromIndex     index to start the search (default 0)
 * @return      int
 */
if(!Array.prototype.indexOf){
	Array.prototype.indexOf = function(elt /*, from*/){
		var len = this.length >>> 0;
		var from = Number(arguments[1]) || 0;
		from = (from < 0)
		     ? Math.ceil(from)
		     : Math.floor(from);
		if(from < 0)
			from += len;

		for(;from < len;from++){
			if(from in this && this[from] === elt)
				return from;
		}
		return -1;
	};
}

/**
 * Returns the index of the given item's last occurrence.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.lastIndexOf(searchElement[, fromIndex]);
 *
 * @param       element searchElement element to locate in the array
 * @param       int     fromIndex     index to start the search backwards (default 0)
 * @return      int
 */
if(!Array.prototype.lastIndexOf){
	Array.prototype.lastIndexOf = function(elt /*, from*/){
		var len = this.length;

		var from = Number(arguments[1]);
		if(isNaN(from)){
			from = len - 1;
		} else {
			from = (from < 0)
			     ? Math.ceil(from)
			     : Math.floor(from);
			if(from < 0)
				from += len;
			else if(from >= len)
				from = len - 1;
		}

		for(;from > -1;from--){
			if(from in this && this[from] === elt)
				return from;
		}
		return -1;
	};
}

/**
 * Runs a function on items in the array while that function is returning true.
 * It returns true if the function returns true for every item it could visit.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.every(callback(value, index, array)[, thisObject]);
 *
 * @param       function callback   function called for every index
 * @param       object   thisObject defines the "this" reference inside the "fun" function
 * @return      bool
 */
if(!Array.prototype.every){
	Array.prototype.every = function(fun /*, thisp*/){
		var len = this.length >>> 0;
		if(typeof fun != "function")
			throw new TypeError();

		var thisp = arguments[1];
		for(var i = 0; i < len; i++){
			if(i in this && !fun.call(thisp, this[i], i, this))
				return false;
		}

		return true;
	};
}

/**
 * runs a function on every item in the array and returns an array of all items
 * for which the function returns true.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.filter(callback(value, index, array)[, thisObject]);
 *
 * @param       function callback   function called to test each element of the array
 * @param       object   thisObject defines the "this" reference inside the "fun" function
 * @return      array
 */
if(!Array.prototype.filter){
	Array.prototype.filter = function(fun /*, thisp*/){
		var len = this.length >>> 0;
		if (typeof fun != "function")
			throw new TypeError();

		var res = new Array();
		var thisp = arguments[1];
		for(var i = 0; i < len; i++){
			if(i in this){
				var val = this[i]; // in case fun mutates this
				if(fun.call(thisp, val, i, this))
					res.push(val);
			}
		}

		return res;
	};
}


/**
 * Runs a function on every item in the array.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.forEach(callback(value, index, array)[, thisObject]);
 *
 * @param       function callback   function called for each index
 * @param       object   thisObject defines the "this" reference inside the "fun" function
 */
if(!Array.prototype.forEach){
	Array.prototype.forEach = function(fun /*, thisp*/){
		var len = this.length >>> 0;
		if(typeof fun != "function")
			throw new TypeError();

		var thisp = arguments[1];
		for(var i = 0;i < len;i++){
			if(i in this)
				fun.call(thisp, this[i], i, this);
		}
	};
}

/**
 * Runs a function on every item in the array and returns the results in an array.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.map(callback(value, index, array)[, thisObject]);
 *
 * @param       function callback   function that produces an element of the new array from an element of the current one
 * @param       object   thisObject defines the "this" reference inside the "fun" function
 */
if(!Array.prototype.map){
	Array.prototype.map = function(fun /*, thisp*/){
		var len = this.length >>> 0;
		if (typeof fun != "function")
			throw new TypeError();

		var res = new Array(len);
		var thisp = arguments[1];
		for(var i = 0; i < len; i++){
			if(i in this)
				res[i] = fun.call(thisp, this[i], i, this);
		}

		return res;
	};
}

/**
 * Runs a function on items in the array while that function returns false.
 * It returns true if the function returns true for any item it could visit.
 *
 * Implemented in: JavaScript 1.6 (Gecko 1.8b2 and later)
 * Syntax: array.some(callback(value, index, array)[, thisObject]);
 *
 * @param       function callback   function to test each element
 * @param       object   thisObject defines the "this" reference inside the "fun" function
 */
if(!Array.prototype.some){
	Array.prototype.some = function(fun /*, thisp*/){
		var i = 0,
		    len = this.length >>> 0;

		if(typeof fun != "function")
			throw new TypeError();

		var thisp = arguments[1];
		for(; i < len; i++){
			if(i in this && fun.call(thisp, this[i], i, this))
				return true;
		}

		return false;
	};
}

/**
 * Apply a function against an accumulator and each value of the array
 * (from left-to-right) as to reduce it to a single value.
 *
 * Implemented in: JavaScript 1.8 (Gecko 1.9a5 and later)
 * Syntax: array.reduce(callback(value, index, array)[, initialValue]);
 *
 * @param       function callback     function to execute on each value in the array
 * @param       Object   initialValue object to use as the first argument to the first call of the callback
 */
if(!Array.prototype.reduce){
	Array.prototype.reduce = function(fun /*, initial*/){
		var len = this.length >>> 0;
		if(typeof fun != "function")
			throw new TypeError();

		// no value to return if no initial value and an empty array
		if(len == 0 && arguments.length == 1)
			throw new TypeError();

		var i = 0;
		if(arguments.length >= 2){
			var rv = arguments[1];
		} else {
			do {
				if(i in this){
					rv = this[i++];
					break;
				}

				// if array contains no values, no initial value to return
				if(++i >= len)
					throw new TypeError();
			}while(true);
		}

		for(;i < len;i++){
			if(i in this)
				rv = fun.call(null, rv, this[i], i, this);
		}

		return rv;
	};
}

/**
 * Apply a function against an accumulator and each value of the array
 * (from right-to-left) as to reduce it to a single value.
 *
 * Implemented in: JavaScript 1.8 (Gecko 1.9a5 and later)
 * Syntax: array.reduce(callback(value, index, array)[, initialValue]);
 *
 * @param       function callback     function to execute on each value in the array
 * @param       Object   initialValue object to use as the first argument to the first call of the callback
 */
if(!Array.prototype.reduceRight){
	Array.prototype.reduceRight = function(fun /*, initial*/){
		var len = this.length >>> 0;
		if(typeof fun != "function")
			throw new TypeError();

		// no value to return if no initial value, empty array
		if(len == 0 && arguments.length == 1)
			throw new TypeError();

		var i = len - 1;
		if(arguments.length >= 2){
			var rv = arguments[1];
		} else {
			do {
				if(i in this){
					rv = this[i--];
					break;
				}

				// if array contains no values, no initial value to return
				if(--i < 0)
					throw new TypeError();
			}while(true);
		}

		for(; i >= 0; i--){
			if(i in this)
				rv = fun.call(null, rv, this[i], i, this);
		}

		return rv;
	};
}
