/**
 * validate - jQuery plugin
 *
 * Plugin to validate input/textarea fields
 *
 * @author      Damian Senn <ds@adfinis.com>
 * @license     LGPLv3
 * @version     1.3.1
 *
 * ChangeLog:
 * 	2009.11.19 - 1.3.1
 * 		Fixed bug where the event wasn't bound to the linked fields.
 *
 * 	2009.11.17 - 1.3
 * 		Added options 'linkFields' which contains jQuery elements
 * 		which values are sent too, in addition those elements trigger
 * 		the element which validate is bound to too.
 * 		Values are now sent as array and is named 'values' instead of 'value'
 * 		Added additional getParameter ajaxValidate:true to distinguish between
 * 		normal and AJAX requests.
 * 		Added configurable events.
 *
 * 	2009.09.28 - 1.2
 * 		Added option 'cacheLength' which limits the amount of
 * 		cached results stored each element.
 * 		Action 'flushCache' will delete the cache for each element.
 *
 * 	2009.09.27 - 1.1.5
 * 		Added ability to reconfigure elements through
 * 		$.validate('setOptions',{});
 *
 * 	2009.09.26 - 1.1.1
 * 		Restructured cache functions
 *
 * 	2009.09.26 - 1.1
 * 		implemented simple cache
 *
 * 	2009.09.23 - 1.0
 * 		initial release
 */
;(function($){
	/**
	 * @var object contains all default options
	 */
	var defaults = {
		url:'', // URL to the file which validates the fields
		callback: // Function which will be executed after the get call
			function(
				e, // Event data
				data, // Response data
				textStatus // Response status
			){
				// Callback function code
			},
		type:'json', // Type of data to be returned to callback function: 'xml', 'html', 'script', 'json', 'jsonp', or 'text'
		name:'name', // Default element attribute which defines the value of the get parameter 'name'
		delay:300, // Delay in ms after which the get request will be fired
		cache:true, // Enable / disable cache
		cacheLength:16, // Defines how much results will be cached for each element
		linkFields:[], // Defines a group of fields which will be sent as "one", the event of those fields will trigger the field event on which the validate plugin is bound
		checkOnLoad:true, // Sets if the validate should be run on initialization
		events:['change','keyup','blur'] // Defines on which events the validation function should be fired
	};

	/**
	 * Method to append plugin to jQuery elements
	 *
	 * @author       Damian Senn <ds@adfinis.com>
	 * @param        string action (optional)
	 * @param        object options
	 * @return       void
	 */
	$.fn.validate = function(action, options){
		if(typeof options == 'undefined'){
			options = action;
		}

		switch(action){
			case 'setOptions':
				this.each(function(){
					setOptions($(this));
				});
				break;
			case 'flushCache':
				this.each(function(){
					$(this).data('validate:cache', []);
				});
				break;
			// No action defined, initialize plugin
			default:
				this.each(function(){
					var $el = $(this);
					var cache = []; // Cache array which holds recent responses as objects
					var t; // The timeout which executes validate() after the in options.delay defined delay

					setOptions($el);

					// Initialize element cache
					$el.data('validate:cache', []);

					/**
					 * Function which will be bound to the elements
					 * and calls ajaxValidate or cachedValidate
					 *
					 * @author       Damian Senn <ds@adfinis.com>
					 * @return       void
					 */
					var validate = function(){
						var values = [$el.val() || $el.text()];
						var elOptions = $el.data('validate:options');
						var elCache = $el.data('validate:cache');

						for(field in options.linkFields){
							values.push(
								options.linkFields[field].val() || options.linkFields[field].text()
							);
						}

						var cacheKey = createCacheKey(values);

						// Cancel AJAX request if it is still in progress
						if(typeof t != 'undefined'){
							clearTimeout(t);
						}

						// Check if the value is available in the cache
						var foundCache;
						if(elOptions.cache){
							$.each(elCache, function(){
								if(typeof this[cacheKey] == 'object'){
									foundCache = this[cacheKey];
									// Return false breaks out of $.each();
									return false;
								}
							});
						}

						// We only need to get the response trough ajax if we didn't cache it yet
						if(typeof foundCache == 'undefined'){
							t = setTimeout(function(){
								ajaxValidate(values, elOptions, elCache);
							},elOptions.delay);
						} else {
							cachedValidate(foundCache);
						}
					};

					/**
					 * Function which will make the AJAX request and save it in the cache
					 *
					 * @author       Damian Senn <ds@adfinis.com>
					 * @param        array values
					 * @param        object elOptions
					 * @return       void
					 */
					var ajaxValidate = function(values, elOptions, elCache){
						$.get(
							elOptions.url, // The URL of the page to load
							{ // Key/value pairs that will be sent to the server
								name:$el.attr(elOptions.name),
								'values[]':values,
								ajaxValidate:true
							},
							function(data, textStatus){ // A function to be executed whenever the data is loaded successfully
								// AJAX request succesful, we're ready for a new one
								clearTimeout(t);

								// Save result in cache
								if(elOptions.cache){
									// Prepare cached object
									var toCache = {}
									toCache[createCacheKey(values)] = {
										data:data,
										textStatus:textStatus
									};
									// Append result to cache
									elCache.push(toCache);

									// Remove the first index if we're over the limit till we reach the limit
									while(elCache.length > elOptions.cacheLength){
										elCache.shift();
									}

									// Save element cache to the element
									$el.data('validate:cache', elCache);
								}

								/* Pass the response as parameters to the callback function.
								 * textStatus will be one of the following values:
								 * 	'timeout','error','notmodified','success','parsererror'
								 */
								$el.trigger('callback',[data,textStatus]);
							},
							elOptions.type // Type of data to be returned to callback function: 'xml', 'html', 'script', 'json', 'jsonp', or 'text'.
						);
					};

					/**
					 * Function which will be executed if we got a cached result
					 *
					 * @author       Damian Senn <ds@adfinis.com>
					 * @param        string value
					 * @param        object elOptions
					 * @return       void
					 */
					var cachedValidate = function(cache){
						/* Trigger event with the cached result
						 *
						 * Pass the cached response as parameters to the callback function.
						 * textStatus will be one of the following values:
						 * 	'timeout','error','notmodified','success','parsererror'
						 */
						$el.trigger('callback',[
							cache['data'],
							cache['textStatus']
						]);
					};

					/* Rebind the functions using an own event namespace
					 * because the reference to the bound function
					 * is lost if we want to reconfigure an element.
					 */
					$el.unbind('.validate');
					// Append validate namespace to events
					events = $.map(options.events, function(n, i){
						return (n + '.validate');
					}).join(' ');
					$el.bind(events, validate);

					// Bind linked event to linked fields
					for(field in options.linkFields){
						options.linkFields[field].unbind('.validate');
						options.linkFields[field].bind(events, function(){
							// Run validate on other fields too
							validate();
						});
					}

					// Run validate function on initial call
					if(options.checkOnLoad)
						validate();
				});
				break;
		}

		/**
		 * Function which creates a key used for the cache hashmap
		 *
		 * @author       Damian Senn <ds@adfinis.com>
		 * @param        array values
		 * @return       string
		 */
		function createCacheKey(values){
			return values.join(';'); // Separate by ; and trim values
		}

		/**
		 * Function to extend the default element options
		 *
		 * @author       Damian Senn <ds@adfinis.com>
		 * @param        element $el
		 * @return       void
		 */
		function setOptions($el){
			// Rebind element events
			$.each(options, function(event, fn){
				if(typeof(fn) == 'function'){
					$el.unbind(event);
					$el.bind(event, fn);
				}
			});

			// Save the current options to the current element
			options = $.extend(
				{}, // The object which will be extended
				defaults,
				$el.data('validate:options'),
				options
			);
			$el.data('validate:options', options);
		};

		return this;
	};
})(jQuery);
